<?php

namespace Importers;

/**
 * Class that imports base database.
 *
 * @copyright YetiForce Sp. z o.o.
 * @license   YetiForce Public License 3.0 (licenses/LicenseEN.txt or yetiforce.com)
 * @author    Mariusz Krzaczkowski <m.krzaczkowski@yetiforce.com>
 */
class Base3 extends \App\Db\Importers\Base
{
	public $dbType = 'base';

	public function scheme()
	{
		$this->tables = [
			'vtiger_glacct' => [
				'columns' => [
					'glacctid' => $this->primaryKey(10),
					'glacct' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['glacct_glacct_idx', 'glacct', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_glacct_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_group2grouprel' => [
				'columns' => [
					'groupid' => $this->integer(10)->notNull(),
					'containsgroupid' => $this->integer(10)->unsigned()->notNull(),
				],
				'index' => [
					['containsgroupid', 'containsgroupid'],
					['groupid', 'groupid'],
				],
				'primaryKeys' => [
					['group2grouprel_pk', ['groupid', 'containsgroupid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_group2modules' => [
				'columns' => [
					'groupid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
				],
				'index' => [
					['groupid', 'groupid'],
					['tabid', 'tabid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_group2role' => [
				'columns' => [
					'groupid' => $this->integer(10)->notNull(),
					'roleid' => $this->stringType()->notNull(),
				],
				'index' => [
					['fk_2_vtiger_group2role', 'roleid'],
				],
				'primaryKeys' => [
					['group2role_pk', ['groupid', 'roleid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_group2rs' => [
				'columns' => [
					'groupid' => $this->integer(10)->unsigned()->notNull(),
					'roleandsubid' => $this->stringType()->notNull(),
				],
				'index' => [
					['fk_2_vtiger_group2rs', 'roleandsubid'],
				],
				'primaryKeys' => [
					['group2rs_pk', ['groupid', 'roleandsubid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_groups' => [
				'columns' => [
					'groupid' => $this->integer(10)->notNull(),
					'groupname' => $this->stringType(100),
					'description' => $this->text(),
					'color' => $this->stringType(25)->defaultValue('#E6FAD8'),
					'modules' => $this->stringType(),
				],
				'index' => [
					['groups_groupname_idx', 'groupname', true],
				],
				'primaryKeys' => [
					['groups_pk', 'groupid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_holidaysentitlement' => [
				'columns' => [
					'holidaysentitlementid' => $this->integer(10)->notNull()->defaultValue(0),
					'holidaysentitlement_no' => $this->stringType(),
					'holidaysentitlement_year' => $this->stringType(50),
					'days' => $this->integer(3)->defaultValue(0),
					'ossemployeesid' => $this->integer(10),
				],
				'index' => [
					['ossemployeesid', 'ossemployeesid'],
				],
				'primaryKeys' => [
					['holidaysentitlement_pk', 'holidaysentitlementid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_holidaysentitlement_year' => [
				'columns' => [
					'holidaysentitlement_yearid' => $this->primaryKey(10),
					'holidaysentitlement_year' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_holidaysentitlement_year_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_holidaysentitlementcf' => [
				'columns' => [
					'holidaysentitlementid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['holidaysentitlementcf_pk', 'holidaysentitlementid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_hour_format' => [
				'columns' => [
					'hour_formatid' => $this->primaryKey(10),
					'hour_format' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_hour_format_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ideas' => [
				'columns' => [
					'ideasid' => $this->integer(10)->notNull()->defaultValue(0),
					'ideas_no' => $this->stringType(),
					'subject' => $this->stringType(),
					'ideasstatus' => $this->stringType()->defaultValue(''),
					'extent_description' => $this->text(),
				],
				'primaryKeys' => [
					['ideas_pk', 'ideasid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ideascf' => [
				'columns' => [
					'ideasid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['ideascf_pk', 'ideasid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ideasstatus' => [
				'columns' => [
					'ideasstatusid' => $this->primaryKey(10),
					'ideasstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ideasstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_igdn_status' => [
				'columns' => [
					'igdn_statusid' => $this->primaryKey(10),
					'igdn_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_igdnc_status' => [
				'columns' => [
					'igdnc_statusid' => $this->primaryKey(10),
					'igdnc_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_igin_status' => [
				'columns' => [
					'igin_statusid' => $this->primaryKey(10),
					'igin_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_igrn_status' => [
				'columns' => [
					'igrn_statusid' => $this->primaryKey(10),
					'igrn_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_igrnc_status' => [
				'columns' => [
					'igrnc_statusid' => $this->primaryKey(10),
					'igrnc_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_iidn_status' => [
				'columns' => [
					'iidn_statusid' => $this->primaryKey(10),
					'iidn_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_import_locks' => [
				'columns' => [
					'vtiger_import_lock_id' => $this->primaryKey(10)->unsigned(),
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'importid' => $this->integer(10)->notNull(),
					'locked_since' => $this->dateTime(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_import_maps' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'name' => $this->stringType(36)->notNull(),
					'module' => $this->stringType(36)->notNull(),
					'content' => $this->binary(),
					'has_header' => $this->integer(1)->notNull()->defaultValue(1),
					'deleted' => $this->integer(1)->notNull()->defaultValue(0),
					'date_entered' => $this->timestamp()->null(),
					'date_modified' => $this->timestamp()->null(),
					'assigned_user_id' => $this->stringType(36),
					'is_published' => $this->stringType(3)->notNull()->defaultValue('no'),
				],
				'index' => [
					['import_maps_assigned_user_id_module_name_deleted_idx', ['assigned_user_id', 'module', 'name', 'deleted']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_import_queue' => [
				'columns' => [
					'importid' => $this->primaryKey(10),
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->unsigned()->notNull(),
					'field_mapping' => $this->text(),
					'default_values' => $this->text(),
					'merge_type' => $this->integer(10),
					'merge_fields' => $this->text(),
					'temp_status' => $this->smallInteger(1)->defaultValue(0),
				],
				'columns_mysql' => [
					'temp_status' => $this->tinyInteger(1)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_incidentregister_status' => [
				'columns' => [
					'incidentregister_statusid' => $this->primaryKey(),
					'incidentregister_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_incidentregister_type' => [
				'columns' => [
					'incidentregister_typeid' => $this->primaryKey(),
					'incidentregister_type' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_industry' => [
				'columns' => [
					'industryid' => $this->primaryKey(10),
					'industry' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['industry_industry_idx', 'industry', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_industry_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_internal_tickets_status' => [
				'columns' => [
					'internal_tickets_statusid' => $this->primaryKey(10),
					'internal_tickets_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_inventory_tandc' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'type' => $this->stringType(30)->notNull(),
					'tandc' => $this->text(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_inventoryproductrel' => [
				'columns' => [
					'id' => $this->integer(10),
					'productid' => $this->integer(10),
					'sequence_no' => $this->integer(4),
					'quantity' => $this->decimal('25,3'),
					'listprice' => $this->decimal('28,8'),
					'discount_percent' => $this->decimal('7,3'),
					'discount_amount' => $this->decimal('28,8'),
					'comment' => $this->stringType(500),
					'description' => $this->text(),
					'incrementondel' => $this->integer(10)->notNull()->defaultValue(0),
					'lineitem_id' => $this->primaryKey(10),
					'tax' => $this->stringType(10),
					'tax1' => $this->decimal('7,3'),
					'tax2' => $this->decimal('7,3'),
					'tax3' => $this->decimal('7,3'),
					'purchase' => $this->decimal('10,2'),
					'margin' => $this->decimal('10,2'),
					'marginp' => $this->decimal('10,2'),
				],
				'index' => [
					['inventoryproductrel_id_idx', 'id'],
					['inventoryproductrel_productid_idx', 'productid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_inventoryproductrel_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_inventorysubproductrel' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
					'sequence_no' => $this->integer(10)->notNull(),
					'productid' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ipreorder_status' => [
				'columns' => [
					'ipreorder_statusid' => $this->primaryKey(10),
					'ipreorder_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_istdn_status' => [
				'columns' => [
					'istdn_statusid' => $this->primaryKey(10),
					'istdn_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_istn_status' => [
				'columns' => [
					'istn_statusid' => $this->primaryKey(10),
					'istn_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_istn_type' => [
				'columns' => [
					'istn_typeid' => $this->primaryKey(10),
					'istn_type' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_istrn_status' => [
				'columns' => [
					'istrn_statusid' => $this->primaryKey(10),
					'istrn_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_knowledgebase_status' => [
				'columns' => [
					'knowledgebase_statusid' => $this->primaryKey(10),
					'knowledgebase_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_knowledgebase_view' => [
				'columns' => [
					'knowledgebase_viewid' => $this->primaryKey(10),
					'knowledgebase_view' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_language' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'name' => $this->stringType(50)->notNull(),
					'prefix' => $this->stringType(10)->notNull(),
					'label' => $this->stringType(30)->notNull(),
					'lastupdated' => $this->dateTime(),
					'sequence' => $this->integer(10),
					'isdefault' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'active' => $this->smallInteger(1)->notNull()->defaultValue(1),
				],
				'columns_mysql' => [
					'isdefault' => $this->tinyInteger(1)->notNull()->defaultValue(0),
					'active' => $this->tinyInteger(1)->notNull()->defaultValue(1),
				],
				'index' => [
					['prefix', 'prefix'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_language_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_layout' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
					'name' => $this->stringType(50),
					'label' => $this->stringType(30),
					'lastupdated' => $this->dateTime(),
					'isdefault' => $this->smallInteger(1),
					'active' => $this->smallInteger(1),
				],
				'columns_mysql' => [
					'isdefault' => $this->tinyInteger(1),
					'active' => $this->tinyInteger(1),
				],
				'primaryKeys' => [
					['layout_pk', 'id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lead_view' => [
				'columns' => [
					'lead_viewid' => $this->primaryKey(10),
					'lead_view' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lead_view_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadaddress' => [
				'columns' => [
					'leadaddressid' => $this->integer(10)->notNull()->defaultValue(0),
					'phone' => $this->stringType(50),
					'mobile' => $this->stringType(50),
					'fax' => $this->stringType(50),
					'addresslevel1a' => $this->stringType(),
					'addresslevel2a' => $this->stringType(),
					'addresslevel3a' => $this->stringType(),
					'addresslevel4a' => $this->stringType(),
					'addresslevel5a' => $this->stringType(),
					'addresslevel6a' => $this->stringType(),
					'addresslevel7a' => $this->stringType(),
					'addresslevel8a' => $this->stringType(),
					'buildingnumbera' => $this->stringType(100),
					'localnumbera' => $this->stringType(100),
					'poboxa' => $this->stringType(50),
					'phone_extra' => $this->stringType(100),
					'mobile_extra' => $this->stringType(100),
					'fax_extra' => $this->stringType(100),
				],
				'primaryKeys' => [
					['leadaddress_pk', 'leadaddressid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leaddetails' => [
				'columns' => [
					'leadid' => $this->integer(10)->notNull(),
					'lead_no' => $this->stringType(100)->notNull(),
					'email' => $this->stringType(100),
					'interest' => $this->stringType(50),
					'firstname' => $this->stringType(40),
					'salutation' => $this->stringType(200),
					'lastname' => $this->stringType(80),
					'company' => $this->stringType(100)->notNull(),
					'annualrevenue' => $this->decimal('25,8'),
					'industry' => $this->stringType(200),
					'campaign' => $this->stringType(30),
					'leadstatus' => $this->stringType(50),
					'leadsource' => $this->stringType(200),
					'converted' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'licencekeystatus' => $this->stringType(50),
					'space' => $this->stringType(250),
					'comments' => $this->text(),
					'priority' => $this->stringType(50),
					'demorequest' => $this->stringType(50),
					'partnercontact' => $this->stringType(50),
					'productversion' => $this->stringType(20),
					'product' => $this->stringType(50),
					'maildate' => $this->date(),
					'nextstepdate' => $this->date(),
					'fundingsituation' => $this->stringType(50),
					'purpose' => $this->stringType(50),
					'evaluationstatus' => $this->stringType(50),
					'transferdate' => $this->date(),
					'revenuetype' => $this->stringType(50),
					'noofemployees' => $this->integer(50),
					'secondaryemail' => $this->stringType(100),
					'assignleadchk' => $this->integer(1)->defaultValue(0),
					'noapprovalcalls' => $this->smallInteger(1),
					'noapprovalemails' => $this->smallInteger(1),
					'vat_id' => $this->stringType(30),
					'registration_number_1' => $this->stringType(30),
					'registration_number_2' => $this->stringType(30),
					'verification' => $this->text(),
					'subindustry' => $this->stringType()->defaultValue(''),
					'atenttion' => $this->text(),
					'leads_relation' => $this->stringType(),
					'legal_form' => $this->stringType(),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'active' => $this->smallInteger(1)->defaultValue(0),
				],
				'columns_mysql' => [
					'converted' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'active' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['leaddetails_converted_leadstatus_idx', ['converted', 'leadstatus']],
					['email_idx', 'email'],
					['lastname', 'lastname'],
					['converted', 'converted'],
				],
				'primaryKeys' => [
					['leaddetails_pk', 'leadid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leads_relation' => [
				'columns' => [
					'leads_relationid' => $this->primaryKey(10),
					'leads_relation' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leads_relation_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadscf' => [
				'columns' => [
					'leadid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['leadscf_pk', 'leadid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadsource' => [
				'columns' => [
					'leadsourceid' => $this->primaryKey(10),
					'leadsource' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadsource_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadstage' => [
				'columns' => [
					'leadstageid' => $this->primaryKey(10),
					'stage' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'index' => [
					['leadstage_stage_idx', 'stage', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadstatus' => [
				'columns' => [
					'leadstatusid' => $this->primaryKey(10),
					'leadstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
					'color' => $this->stringType(25)->defaultValue('#E6FAD8'),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_leadsubdetails' => [
				'columns' => [
					'leadsubscriptionid' => $this->integer(10)->notNull()->defaultValue(0),
					'website' => $this->stringType(),
					'callornot' => $this->integer(1)->defaultValue(0),
					'readornot' => $this->integer(1)->defaultValue(0),
					'empct' => $this->integer(10)->defaultValue(0),
				],
				'primaryKeys' => [
					['leadsubdetails_pk', 'leadsubscriptionid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_legal_form' => [
				'columns' => [
					'legal_formid' => $this->primaryKey(10),
					'legal_form' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_legal_form_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lettersin' => [
				'columns' => [
					'lettersinid' => $this->integer(10)->notNull()->defaultValue(0),
					'number' => $this->stringType(),
					'title' => $this->stringType(),
					'relatedid' => $this->integer(10),
					'person_receiving' => $this->integer(10),
					'parentid' => $this->integer(10),
					'date_adoption' => $this->date(),
					'lin_type_ship' => $this->stringType()->defaultValue(''),
					'lin_type_doc' => $this->text(),
					'lin_status' => $this->stringType()->defaultValue(''),
					'deadline_reply' => $this->date(),
					'cocument_no' => $this->stringType(100)->defaultValue(''),
					'no_internal' => $this->stringType(100)->defaultValue(''),
					'lin_dimensions' => $this->stringType()->defaultValue(''),
					'custom_sender' => $this->stringType(),
					'lin_type' => $this->stringType(),
					'cash_amount_on_delivery' => $this->decimal('25,8'),
					'date_of_receipt' => $this->date(),
					'outgoing_correspondence' => $this->integer(10),
				],
				'index' => [
					['vtiger_lettersin_outgoing_correspondence_idx', 'outgoing_correspondence'],
				],
				'primaryKeys' => [
					['lettersin_pk', 'lettersinid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lettersincf' => [
				'columns' => [
					'lettersinid' => $this->integer(10)->notNull(),
					'internal_notes' => $this->text(),
					'public_notes' => $this->text(),
				],
				'primaryKeys' => [
					['lettersincf_pk', 'lettersinid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lettersout' => [
				'columns' => [
					'lettersoutid' => $this->integer(10)->notNull()->defaultValue(0),
					'number' => $this->stringType(),
					'title' => $this->stringType(),
					'relatedid' => $this->integer(10),
					'person_receiving' => $this->integer(10),
					'parentid' => $this->integer(10),
					'date_adoption' => $this->date(),
					'lout_type_ship' => $this->stringType()->defaultValue(''),
					'lout_type_doc' => $this->text(),
					'lout_status' => $this->stringType()->defaultValue(''),
					'deadline_reply' => $this->date(),
					'cocument_no' => $this->stringType(100)->defaultValue(''),
					'no_internal' => $this->stringType(100)->defaultValue(''),
					'lout_dimensions' => $this->stringType()->defaultValue(''),
					'incoming_correspondence' => $this->integer(10),
				],
				'index' => [
					['vtiger_lettersout_incoming_correspondence_idx', 'incoming_correspondence'],
				],
				'primaryKeys' => [
					['lettersout_pk', 'lettersoutid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lettersoutcf' => [
				'columns' => [
					'lettersoutid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['lettersoutcf_pk', 'lettersoutid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_dimensions' => [
				'columns' => [
					'lin_dimensionsid' => $this->primaryKey(10),
					'lin_dimensions' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_dimensions_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_status' => [
				'columns' => [
					'lin_statusid' => $this->primaryKey(10),
					'lin_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_type' => [
				'columns' => [
					'lin_typeid' => $this->primaryKey(),
					'lin_type' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_type_doc' => [
				'columns' => [
					'lin_type_docid' => $this->primaryKey(10),
					'lin_type_doc' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_type_doc_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_type_ship' => [
				'columns' => [
					'lin_type_shipid' => $this->primaryKey(10),
					'lin_type_ship' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lin_type_ship_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_links' => [
				'columns' => [
					'linkid' => $this->primaryKey(10),
					'tabid' => $this->smallInteger(5),
					'linktype' => $this->stringType(50),
					'linklabel' => $this->stringType(50),
					'linkurl' => $this->stringType(),
					'linkicon' => $this->stringType(100),
					'sequence' => $this->integer(10),
					'handler_path' => $this->stringType(128),
					'handler_class' => $this->stringType(50),
					'handler' => $this->stringType(50),
					'params' => $this->stringType(),
				],
				'index' => [
					['link_tabidtype_idx', ['tabid', 'linktype']],
					['linklabel', 'linklabel'],
					['linkid', ['linkid', 'tabid', 'linktype', 'linklabel']],
					['linktype', 'linktype'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_locationregister_status' => [
				'columns' => [
					'locationregister_statusid' => $this->primaryKey(),
					'locationregister_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_login_method' => [
				'columns' => [
					'login_methodid' => $this->primaryKey(),
					'login_method' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_loginhistory' => [
				'columns' => [
					'login_id' => $this->primaryKey(10),
					'user_name' => $this->stringType(32),
					'user_ip' => $this->stringType(100)->notNull(),
					'logout_time' => $this->timestamp()->null(),
					'login_time' => $this->timestamp()->null(),
					'status' => $this->stringType(25),
					'browser' => $this->stringType(25),
				],
				'index' => [
					['user_name', 'user_name'],
					['user_ip', ['user_ip', 'login_time', 'status']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_dimensions' => [
				'columns' => [
					'lout_dimensionsid' => $this->primaryKey(10),
					'lout_dimensions' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_dimensions_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_status' => [
				'columns' => [
					'lout_statusid' => $this->primaryKey(10),
					'lout_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_type_doc' => [
				'columns' => [
					'lout_type_docid' => $this->primaryKey(10),
					'lout_type_doc' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_type_doc_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_type_ship' => [
				'columns' => [
					'lout_type_shipid' => $this->primaryKey(10),
					'lout_type_ship' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_lout_type_ship_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_mail_accounts' => [
				'columns' => [
					'account_id' => $this->integer(10)->notNull(),
					'user_id' => $this->integer(10)->notNull(),
					'display_name' => $this->stringType(50),
					'mail_id' => $this->stringType(50),
					'account_name' => $this->stringType(50),
					'mail_protocol' => $this->stringType(20),
					'mail_username' => $this->stringType(50)->notNull(),
					'mail_password' => $this->stringType(250)->notNull(),
					'mail_servername' => $this->stringType(50),
					'box_refresh' => $this->integer(10),
					'mails_per_page' => $this->integer(10),
					'ssltype' => $this->stringType(50),
					'sslmeth' => $this->stringType(50),
					'int_mailer' => $this->integer(1)->defaultValue(0),
					'status' => $this->stringType(10),
					'set_default' => $this->integer(2),
					'sent_folder' => $this->stringType(50),
				],
				'primaryKeys' => [
					['mail_accounts_pk', 'account_id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_manufacturer' => [
				'columns' => [
					'manufacturerid' => $this->primaryKey(10),
					'manufacturer' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['manufacturer_manufacturer_idx', 'manufacturer', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_manufacturer_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_modcomments' => [
				'columns' => [
					'modcommentsid' => $this->integer(10)->notNull(),
					'commentcontent' => $this->text(),
					'related_to' => $this->integer(10),
					'parent_comments' => $this->integer(10),
					'customer' => $this->integer(),
					'userid' => $this->integer(10),
					'reasontoedit' => $this->stringType(100),
				],
				'index' => [
					['relatedto_idx', 'related_to'],
					['modcommentsid', 'modcommentsid'],
					['parent_comments', 'parent_comments'],
					['userid', 'userid'],
					['related_to', ['related_to', 'parent_comments']],
				],
				'primaryKeys' => [
					['modcomments_pk', 'modcommentsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_modcommentscf' => [
				'columns' => [
					'modcommentsid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['modcommentscf_pk', 'modcommentsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_modentity_num' => [
				'columns' => [
					'id' => $this->primaryKey(5)->unsigned(),
					'tabid' => $this->smallInteger(5)->unsigned()->notNull(),
					'prefix' => $this->stringType(50)->notNull()->defaultValue(''),
					'postfix' => $this->stringType(50)->notNull()->defaultValue(''),
					'start_id' => $this->integer(10)->unsigned()->notNull(),
					'cur_id' => $this->integer(10)->unsigned()->notNull(),
				],
				'index' => [
					['semodule', 'cur_id'],
					['prefix', ['prefix', 'postfix', 'cur_id']],
					['tabid', 'tabid'],
					['tabid_2', ['tabid', 'cur_id']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_modtracker_basic' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'crmid' => $this->integer(10),
					'module' => $this->stringType(50),
					'whodid' => $this->integer(10),
					'changedon' => $this->dateTime(),
					'status' => $this->integer(1)->defaultValue(0),
					'last_reviewed_users' => $this->stringType()->defaultValue(''),
				],
				'index' => [
					['crmidx', 'crmid'],
					['idx', 'id'],
					['id', ['id', 'module', 'changedon']],
					['crmid', ['crmid', 'changedon']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_modtracker_detail' => [
				'columns' => [
					'id' => $this->integer(10),
					'fieldname' => $this->stringType(100),
					'prevalue' => $this->text(),
					'postvalue' => $this->text(),
				],
				'index' => [
					['idx', 'id'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_modtracker_relations' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
					'targetmodule' => $this->stringType(100)->notNull(),
					'targetid' => $this->integer(10)->notNull(),
					'changedon' => $this->dateTime(),
				],
				'primaryKeys' => [
					['modtracker_relations_pk', 'id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_modtracker_tabs' => [
				'columns' => [
					'tabid' => $this->smallInteger(5)->unsigned()->notNull(),
					'visible' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'columns_mysql' => [
					'visible' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'index' => [
					['tabid', ['tabid', 'visible']],
				],
				'primaryKeys' => [
					['modtracker_tabs_pk', 'tabid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_module_dashboard' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'blockid' => $this->integer(10)->notNull(),
					'linkid' => $this->integer(10),
					'filterid' => $this->stringType(100),
					'title' => $this->stringType(100),
					'data' => $this->text(),
					'size' => $this->stringType(50),
					'limit' => $this->smallInteger(2),
					'isdefault' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'owners' => $this->stringType(100),
					'cache' => $this->smallInteger(1)->defaultValue(0),
					'date' => $this->stringType(20),
				],
				'columns_mysql' => [
					'limit' => $this->tinyInteger(2),
					'isdefault' => $this->tinyInteger(1)->notNull()->defaultValue(0),
					'cache' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['vtiger_module_dashboard_ibfk_1', 'blockid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_module_dashboard_blocks' => [
				'columns' => [
					'id' => $this->primaryKey(100)->unsigned(),
					'authorized' => $this->stringType(10)->notNull(),
					'tabid' => $this->smallInteger(5)->unsigned()->notNull(),
					'dashboard_id' => $this->integer(10),
				],
				'index' => [
					['authorized', ['authorized', 'tabid']],
					['tabid', 'tabid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_module_dashboard_widgets' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'linkid' => $this->integer(10)->notNull(),
					'userid' => $this->integer(10),
					'templateid' => $this->integer(10)->notNull(),
					'filterid' => $this->stringType(100),
					'title' => $this->stringType(100),
					'data' => $this->text(),
					'size' => $this->text(),
					'limit' => $this->smallInteger(2),
					'position' => $this->text(),
					'isdefault' => $this->smallInteger(1)->defaultValue(0),
					'active' => $this->smallInteger(1)->defaultValue(0),
					'owners' => $this->stringType(100),
					'module' => $this->integer(10)->defaultValue(0),
					'cache' => $this->smallInteger(1)->defaultValue(0),
					'date' => $this->stringType(20),
					'dashboardid' => $this->integer(10),
				],
				'columns_mysql' => [
					'limit' => $this->tinyInteger(2),
					'isdefault' => $this->tinyInteger(1)->defaultValue(0),
					'active' => $this->tinyInteger(1)->defaultValue(0),
					'cache' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['vtiger_module_dashboard_widgets_ibfk_1', 'templateid'],
					['userid', ['userid', 'active', 'module']],
					['vtiger_module_dashboard_widgets_linkid_idx', 'linkid'],
					['vtiger_module_dashboard_widgets_dashboardid_idx', 'dashboardid'],
					['vtiger_module_dashboard_widgets_module_idx', 'module'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_mulcomp_status' => [
				'columns' => [
					'mulcomp_statusid' => $this->primaryKey(),
					'mulcomp_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_no_of_currency_decimals' => [
				'columns' => [
					'no_of_currency_decimalsid' => $this->primaryKey(10),
					'no_of_currency_decimals' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_no_of_currency_decimals_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_notebook_contents' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'notebookid' => $this->integer(10)->notNull(),
					'contents' => $this->text(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_notes' => [
				'columns' => [
					'notesid' => $this->integer(10)->notNull()->defaultValue(0),
					'note_no' => $this->stringType(100)->notNull(),
					'title' => $this->stringType(200)->notNull(),
					'filename' => $this->stringType(200),
					'notecontent' => $this->text(),
					'folderid' => $this->stringType(),
					'filetype' => $this->stringType(100),
					'filelocationtype' => $this->stringType(5),
					'filedownloadcount' => $this->integer(10),
					'filestatus' => $this->smallInteger(1),
					'filesize' => $this->integer(10)->notNull()->defaultValue(0),
					'fileversion' => $this->stringType(50),
					'ossdc_status' => $this->stringType(),
				],
				'index' => [
					['notes_title_idx', 'title'],
					['notes_notesid_idx', 'notesid'],
				],
				'primaryKeys' => [
					['notes_pk', 'notesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_notescf' => [
				'columns' => [
					'notesid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['notescf_pk', 'notesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_notification_status' => [
				'columns' => [
					'notification_statusid' => $this->primaryKey(10),
					'notification_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_notification_type' => [
				'columns' => [
					'notification_typeid' => $this->primaryKey(10),
					'notification_type' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_opportunitystage' => [
				'columns' => [
					'potstageid' => $this->primaryKey(10),
					'stage' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'probability' => $this->decimal('3,2')->defaultValue(0),
				],
				'index' => [
					['opportunitystage_stage_idx', 'stage', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_oproductstatus' => [
				'columns' => [
					'oproductstatusid' => $this->primaryKey(10),
					'oproductstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_oproductstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_org_share_action2tab' => [
				'columns' => [
					'share_action_id' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
				],
				'index' => [
					['fk_2_vtiger_org_share_action2tab', 'tabid'],
				],
				'primaryKeys' => [
					['org_share_action2tab_pk', ['share_action_id', 'tabid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_org_share_action_mapping' => [
				'columns' => [
					'share_action_id' => $this->integer(10)->notNull(),
					'share_action_name' => $this->stringType(200),
				],
				'primaryKeys' => [
					['org_share_action_mapping_pk', 'share_action_id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossdc_status' => [
				'columns' => [
					'ossdc_statusid' => $this->primaryKey(10),
					'ossdc_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossdc_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossemployees' => [
				'columns' => [
					'ossemployeesid' => $this->integer(10)->notNull()->defaultValue(0),
					'ossemployees_no' => $this->stringType(),
					'parentid' => $this->integer(10)->defaultValue(0),
					'employee_status' => $this->stringType(200),
					'name' => $this->stringType(200),
					'last_name' => $this->stringType(200),
					'pesel' => $this->stringType(20),
					'id_card' => $this->stringType(200),
					'employee_education' => $this->stringType(200),
					'birth_date' => $this->date(),
					'business_phone' => $this->stringType(20),
					'private_phone' => $this->stringType(25),
					'business_mail' => $this->stringType(100),
					'private_mail' => $this->stringType(100),
					'street' => $this->stringType(200),
					'code' => $this->stringType(200),
					'city' => $this->stringType(200),
					'state' => $this->stringType(200),
					'country' => $this->stringType(200),
					'ship_street' => $this->stringType(200),
					'ship_code' => $this->stringType(200),
					'ship_city' => $this->stringType(200),
					'ship_state' => $this->stringType(200),
					'ship_country' => $this->stringType(200),
					'dav_status' => $this->smallInteger(1)->defaultValue(1),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'secondary_phone' => $this->stringType(25),
					'position' => $this->stringType(),
					'rbh' => $this->decimal('25,8'),
					'business_phone_extra' => $this->stringType(100),
					'private_phone_extra' => $this->stringType(100),
					'secondary_phone_extra' => $this->stringType(100),
				],
				'columns_mysql' => [
					'dav_status' => $this->tinyInteger(1)->defaultValue(1),
				],
				'primaryKeys' => [
					['ossemployees_pk', 'ossemployeesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossemployeescf' => [
				'columns' => [
					'ossemployeesid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['ossemployeescf_pk', 'ossemployeesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osservicesstatus' => [
				'columns' => [
					'osservicesstatusid' => $this->primaryKey(10),
					'osservicesstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osservicesstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmails_logs' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'start_time' => $this->timestamp()->null(),
					'end_time' => $this->timestamp()->null(),
					'action' => $this->stringType(100),
					'status' => $this->smallInteger(3),
					'user' => $this->stringType(100),
					'count' => $this->integer(10),
					'stop_user' => $this->stringType(100),
					'info' => $this->stringType(100),
				],
				'columns_mysql' => [
					'status' => $this->tinyInteger(3),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailscanner_config' => [
				'columns' => [
					'conf_type' => $this->stringType(100)->notNull(),
					'parameter' => $this->stringType(100),
					'value' => $this->text(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailscanner_folders_uid' => [
				'columns' => [
					'user_id' => $this->integer(10)->unsigned(),
					'type' => $this->stringType(50),
					'folder' => $this->stringType(100),
					'uid' => $this->integer(10)->defaultValue(0),
				],
				'index' => [
					['user_id', 'user_id'],
					['folder', 'folder'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailscanner_log_cron' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'created_time' => $this->timestamp()->null(),
					'laststart' => $this->integer(10)->unsigned(),
					'status' => $this->stringType(50),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailview' => [
				'columns' => [
					'ossmailviewid' => $this->integer(10)->notNull(),
					'ossmailview_no' => $this->stringType(50),
					'from_email' => $this->text(),
					'to_email' => $this->text(),
					'subject' => $this->text(),
					'content' => $this->text(),
					'cc_email' => $this->text(),
					'bcc_email' => $this->text(),
					'id' => $this->integer(10),
					'mbox' => $this->stringType(100),
					'uid' => $this->stringType(150),
					'cid' => $this->char(40),
					'rc_user' => $this->stringType(3),
					'reply_to_email' => $this->text(),
					'ossmailview_sendtype' => $this->stringType(30),
					'attachments_exist' => $this->smallInteger(1)->defaultValue(0),
					'type' => $this->smallInteger(1),
					'from_id' => $this->stringType(50)->notNull(),
					'to_id' => $this->stringType(100)->notNull(),
					'orginal_mail' => $this->text(),
					'verify' => $this->smallInteger(1)->defaultValue(0),
					'rel_mod' => $this->stringType(128),
					'date' => $this->dateTime(),
				],
				'columns_mysql' => [
					'type' => $this->tinyInteger(1),
				],
				'index' => [
					['id', 'id'],
					['verify', 'verify'],
					['message_id', ['uid', 'rc_user']],
					['mbox', 'mbox'],
					['ossmailview_cid_idx', 'cid'],
				],
				'primaryKeys' => [
					['ossmailview_pk', 'ossmailviewid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailview_files' => [
				'columns' => [
					'ossmailviewid' => $this->integer(10)->notNull(),
					'documentsid' => $this->integer(10)->notNull(),
					'attachmentsid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['fk_1_vtiger_ossmailview_files', 'ossmailviewid'],
					['documentsid', 'documentsid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailview_relation' => [
				'columns' => [
					'ossmailviewid' => $this->integer(10)->notNull(),
					'crmid' => $this->integer(10)->notNull(),
					'date' => $this->dateTime(),
					'deleted' => $this->smallInteger(1)->defaultValue(0),
				],
				'columns_mysql' => [
					'deleted' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['ossmailviewid_2', ['ossmailviewid', 'crmid'], true],
					['ossmailviewid', 'ossmailviewid'],
					['crmid', ['crmid', 'deleted']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailview_sendtype' => [
				'columns' => [
					'ossmailview_sendtypeid' => $this->primaryKey(10),
					'ossmailview_sendtype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailview_sendtype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossmailviewcf' => [
				'columns' => [
					'ossmailviewid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['ossmailviewcf_pk', 'ossmailviewid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossoutsourcedservices' => [
				'columns' => [
					'ossoutsourcedservicesid' => $this->integer(10)->notNull()->defaultValue(0),
					'ossoutsourcedservices_no' => $this->stringType(),
					'productname' => $this->stringType(100)->defaultValue(''),
					'osservicesstatus' => $this->stringType(50),
					'pscategory' => $this->stringType(),
					'datesold' => $this->date(),
					'dateinservice' => $this->date(),
					'wherebought' => $this->stringType(100)->defaultValue(''),
					'parent_id' => $this->integer(10),
					'ssalesprocessesid' => $this->integer(10),
				],
				'index' => [
					['parent_id', 'parent_id'],
					['ssalesprocessesid', 'ssalesprocessesid'],
				],
				'primaryKeys' => [
					['ossoutsourcedservices_pk', 'ossoutsourcedservicesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ossoutsourcedservicescf' => [
				'columns' => [
					'ossoutsourcedservicesid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['ossoutsourcedservicescf_pk', 'ossoutsourcedservicesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osspasswords' => [
				'columns' => [
					'osspasswordsid' => $this->integer(10)->notNull(),
					'osspassword_no' => $this->stringType(100)->notNull(),
					'passwordname' => $this->stringType(100)->notNull(),
					'username' => $this->stringType(100)->notNull(),
					'password' => $this->stringType(200)->notNull(),
					'link_adres' => $this->stringType(),
					'linkto' => $this->integer(10),
					'linkextend' => $this->integer(10),
				],
				'columns_mysql' => [
					'password' => $this->varbinary(200)->notNull(),
				],
				'index' => [
					['linkto', 'linkto'],
					['linkextend', 'linkextend'],
				],
				'primaryKeys' => [
					['osspasswords_pk', 'osspasswordsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osspasswordscf' => [
				'columns' => [
					'osspasswordsid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['osspasswordscf_pk', 'osspasswordsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osssoldservices' => [
				'columns' => [
					'osssoldservicesid' => $this->integer(10)->notNull()->defaultValue(0),
					'osssoldservices_no' => $this->stringType(),
					'productname' => $this->stringType()->defaultValue(''),
					'ssservicesstatus' => $this->stringType(),
					'pscategory' => $this->stringType()->defaultValue(''),
					'datesold' => $this->date(),
					'dateinservice' => $this->date(),
					'invoice' => $this->stringType()->defaultValue(''),
					'parent_id' => $this->integer(10),
					'serviceid' => $this->integer(10),
					'ordertime' => $this->decimal('10,2'),
					'ssalesprocessesid' => $this->integer(10),
					'osssoldservices_renew' => $this->stringType(),
					'renewalinvoice' => $this->integer(10),
					'contactid' => $this->integer(19),
				],
				'index' => [
					['parent_id', 'parent_id'],
					['serviceid', 'serviceid'],
					['ssalesprocessesid', 'ssalesprocessesid'],
					['renewalinvoice', 'renewalinvoice'],
					['contactid_idx', 'contactid'],
				],
				'primaryKeys' => [
					['osssoldservices_pk', 'osssoldservicesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osssoldservices_renew' => [
				'columns' => [
					'osssoldservices_renewid' => $this->primaryKey(10),
					'osssoldservices_renew' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osssoldservicescf' => [
				'columns' => [
					'osssoldservicesid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['osssoldservicescf_pk', 'osssoldservicesid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osstimecontrol' => [
				'columns' => [
					'osstimecontrolid' => $this->integer(10)->notNull()->defaultValue(0),
					'name' => $this->stringType(128),
					'osstimecontrol_no' => $this->stringType(),
					'osstimecontrol_status' => $this->stringType(128),
					'date_start' => $this->date()->notNull(),
					'time_start' => $this->stringType(50),
					'due_date' => $this->date(),
					'time_end' => $this->stringType(50),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'deleted' => $this->integer(1)->defaultValue(0),
					'timecontrol_type' => $this->stringType(),
					'process' => $this->integer(10),
					'link' => $this->integer(10),
					'subprocess' => $this->integer(10),
					'linkextend' => $this->integer(10),
				],
				'index' => [
					['on_update_cascade', 'deleted'],
					['osstimecontrol_status_9', ['osstimecontrol_status', 'deleted']],
					['osstimecontrol_status_6', 'osstimecontrol_status'],
					['subprocess', 'subprocess'],
					['link', 'link'],
					['process', 'process'],
					['linkextend', 'linkextend'],
				],
				'primaryKeys' => [
					['osstimecontrol_pk', 'osstimecontrolid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osstimecontrol_status' => [
				'columns' => [
					'osstimecontrol_statusid' => $this->primaryKey(10),
					'osstimecontrol_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osstimecontrol_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_osstimecontrolcf' => [
				'columns' => [
					'osstimecontrolid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['osstimecontrolcf_pk', 'osstimecontrolid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_othereventduration' => [
				'columns' => [
					'othereventdurationid' => $this->primaryKey(10),
					'othereventduration' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_othereventduration_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_outsourcedproducts' => [
				'columns' => [
					'outsourcedproductsid' => $this->integer(10)->notNull()->defaultValue(0),
					'asset_no' => $this->stringType(32),
					'productname' => $this->stringType(),
					'datesold' => $this->date(),
					'dateinservice' => $this->date(),
					'oproductstatus' => $this->stringType(),
					'pscategory' => $this->stringType()->defaultValue(''),
					'wherebought' => $this->stringType()->defaultValue(''),
					'prodcount' => $this->integer(),
					'parent_id' => $this->integer(10),
					'ssalesprocessesid' => $this->integer(10),
				],
				'index' => [
					['parent_id', 'parent_id'],
					['ssalesprocessesid', 'ssalesprocessesid'],
				],
				'primaryKeys' => [
					['outsourcedproducts_pk', 'outsourcedproductsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_outsourcedproductscf' => [
				'columns' => [
					'outsourcedproductsid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['outsourcedproductscf_pk', 'outsourcedproductsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_password' => [
				'columns' => [
					'type' => $this->stringType(20)->notNull(),
					'val' => $this->stringType(100)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_passwords_config' => [
				'columns' => [
					'pass_length_min' => $this->integer(3)->notNull(),
					'pass_length_max' => $this->integer(3)->notNull(),
					'pass_allow_chars' => $this->stringType(200)->notNull(),
					'register_changes' => $this->smallInteger(1)->defaultValue(0),
				],
				'columns_mysql' => [
					'register_changes' => $this->tinyInteger(1)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsin' => [
				'columns' => [
					'paymentsinid' => $this->integer(10)->notNull()->defaultValue(0),
					'paymentsvalue' => $this->decimal('25,3'),
					'paymentsno' => $this->stringType(32),
					'paymentsname' => $this->stringType(128),
					'paymentstitle' => $this->text(),
					'paymentscurrency' => $this->stringType(32),
					'bank_account' => $this->stringType(128),
					'paymentsin_status' => $this->stringType(128),
					'relatedid' => $this->integer(10),
				],
				'primaryKeys' => [
					['paymentsin_pk', 'paymentsinid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsin_status' => [
				'columns' => [
					'paymentsin_statusid' => $this->primaryKey(10),
					'paymentsin_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsin_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsincf' => [
				'columns' => [
					'paymentsinid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['paymentsincf_pk', 'paymentsinid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsout' => [
				'columns' => [
					'paymentsoutid' => $this->integer(10)->notNull()->defaultValue(0),
					'paymentsvalue' => $this->decimal('25,3'),
					'paymentsno' => $this->stringType(32),
					'paymentsname' => $this->stringType(128),
					'paymentstitle' => $this->stringType(128),
					'paymentscurrency' => $this->stringType(32),
					'bank_account' => $this->stringType(128),
					'paymentsout_status' => $this->stringType(128),
					'relatedid' => $this->integer(10),
					'parentid' => $this->integer(10),
				],
				'primaryKeys' => [
					['paymentsout_pk', 'paymentsoutid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsout_status' => [
				'columns' => [
					'paymentsout_statusid' => $this->primaryKey(10),
					'paymentsout_status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsout_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_paymentsoutcf' => [
				'columns' => [
					'paymentsoutid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['paymentsoutcf_pk', 'paymentsoutid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_picklist' => [
				'columns' => [
					'picklistid' => $this->primaryKey(10),
					'name' => $this->stringType(200)->notNull(),
				],
				'index' => [
					['picklist_name_idx', 'name', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_picklist_dependency' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'sourcefield' => $this->stringType(),
					'targetfield' => $this->stringType(),
					'sourcevalue' => $this->stringType(100),
					'targetvalues' => $this->text(),
					'criteria' => $this->text(),
				],
				'primaryKeys' => [
					['picklist_dependency_pk', 'id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_picklist_dependency_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_picklistvalues_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_portal' => [
				'columns' => [
					'portalid' => $this->primaryKey(10),
					'portalname' => $this->stringType(200)->notNull(),
					'portalurl' => $this->stringType()->notNull(),
					'sequence' => $this->integer(3)->notNull(),
					'setdefault' => $this->integer(3)->notNull()->defaultValue(0),
					'createdtime' => $this->dateTime(),
				],
				'index' => [
					['portal_portalname_idx', 'portalname'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_pricebook' => [
				'columns' => [
					'pricebookid' => $this->integer(10)->notNull()->defaultValue(0),
					'pricebook_no' => $this->stringType(100)->notNull(),
					'bookname' => $this->stringType(100),
					'active' => $this->smallInteger(1),
					'currency_id' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'primaryKeys' => [
					['pricebook_pk', 'pricebookid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_pricebookcf' => [
				'columns' => [
					'pricebookid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['pricebookcf_pk', 'pricebookid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_pricebookproductrel' => [
				'columns' => [
					'pricebookid' => $this->integer(10)->notNull(),
					'productid' => $this->integer(10)->notNull(),
					'listprice' => $this->decimal('28,8'),
					'usedcurrency' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'index' => [
					['pricebookproductrel_pricebookid_idx', 'pricebookid'],
					['pricebookproductrel_productid_idx', 'productid'],
				],
				'primaryKeys' => [
					['pricebookproductrel_pk', ['pricebookid', 'productid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_priority' => [
				'columns' => [
					'priorityid' => $this->primaryKey(10),
					'priority' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'index' => [
					['priority_priority_idx', 'priority', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_productcf' => [
				'columns' => [
					'productid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['productcf_pk', 'productid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_productcurrencyrel' => [
				'columns' => [
					'productid' => $this->integer(10)->notNull(),
					'currencyid' => $this->integer(10)->notNull(),
					'converted_price' => $this->decimal('28,8'),
					'actual_price' => $this->decimal('28,8'),
				],
				'index' => [
					['productid', 'productid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_products' => [
				'columns' => [
					'productid' => $this->integer(10)->notNull(),
					'product_no' => $this->stringType(100)->notNull(),
					'productname' => $this->stringType(100),
					'productcode' => $this->stringType(40),
					'pscategory' => $this->stringType(200),
					'manufacturer' => $this->stringType(200),
					'qty_per_unit' => $this->decimal('11,2')->defaultValue(0),
					'unit_price' => $this->decimal('25,8'),
					'weight' => $this->decimal('11,3'),
					'pack_size' => $this->integer(10),
					'sales_start_date' => $this->date(),
					'sales_end_date' => $this->date(),
					'start_date' => $this->date(),
					'expiry_date' => $this->date(),
					'cost_factor' => $this->integer(10),
					'commissionrate' => $this->decimal('7,3'),
					'commissionmethod' => $this->stringType(50),
					'discontinued' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'usageunit' => $this->stringType(200),
					'reorderlevel' => $this->integer(10),
					'website' => $this->stringType(100),
					'mfr_part_no' => $this->stringType(200),
					'vendor_part_no' => $this->stringType(200),
					'serialno' => $this->stringType(200),
					'qtyinstock' => $this->decimal('25,3'),
					'productsheet' => $this->stringType(200),
					'qtyindemand' => $this->integer(10),
					'glacct' => $this->stringType(200),
					'vendor_id' => $this->integer(10),
					'imagename' => $this->text(),
					'currency_id' => $this->integer(10)->notNull()->defaultValue(1),
					'taxes' => $this->stringType(50),
					'ean' => $this->stringType(30),
					'subunit' => $this->stringType()->defaultValue(''),
					'renewable' => $this->smallInteger(1)->defaultValue(0),
					'category_multipicklist' => $this->text(),
				],
				'columns_mysql' => [
					'discontinued' => $this->tinyInteger(1)->notNull()->defaultValue(0),
					'renewable' => $this->tinyInteger(1)->defaultValue(0),
				],
				'primaryKeys' => [
					['products_pk', 'productid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_profile' => [
				'columns' => [
					'profileid' => $this->primaryKey(10),
					'profilename' => $this->stringType(50)->notNull(),
					'description' => $this->text(),
					'directly_related_to_role' => $this->integer(1)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_profile2field' => [
				'columns' => [
					'profileid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5),
					'fieldid' => $this->integer(10)->notNull(),
					'visible' => $this->integer(10),
					'readonly' => $this->integer(10),
				],
				'index' => [
					['profile2field_profileid_tabid_fieldname_idx', ['profileid', 'tabid']],
					['profile2field_tabid_profileid_idx', ['tabid', 'profileid']],
					['profile2field_visible_profileid_idx', ['visible', 'profileid']],
					['profile2field_readonly_idx', 'readonly'],
				],
				'primaryKeys' => [
					['profile2field_pk', ['profileid', 'fieldid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_profile2globalpermissions' => [
				'columns' => [
					'profileid' => $this->integer(10)->notNull(),
					'globalactionid' => $this->integer(10)->notNull(),
					'globalactionpermission' => $this->integer(10),
				],
				'index' => [
					['idx_profile2globalpermissions', ['profileid', 'globalactionid']],
				],
				'primaryKeys' => [
					['profile2globalpermissions_pk', ['profileid', 'globalactionid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_profile2standardpermissions' => [
				'columns' => [
					'profileid' => $this->smallInteger(5)->unsigned()->notNull(),
					'tabid' => $this->smallInteger(5)->unsigned()->notNull(),
					'operation' => $this->smallInteger(5)->unsigned()->notNull(),
					'permissions' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
				],
				'columns_mysql' => [
					'permissions' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
				],
				'index' => [
					['profile2standardpermissions_profileid_tabid_operation_idx', ['profileid', 'tabid', 'operation']],
					['profileid', ['profileid', 'tabid']],
				],
				'primaryKeys' => [
					['profile2standardpermissions_pk', ['profileid', 'tabid', 'operation']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_profile2tab' => [
				'columns' => [
					'profileid' => $this->integer(10),
					'tabid' => $this->smallInteger(5),
					'permissions' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['profile2tab_profileid_tabid_idx', ['profileid', 'tabid']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_profile2utility' => [
				'columns' => [
					'profileid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'activityid' => $this->integer(10)->notNull(),
					'permission' => $this->integer(1),
				],
				'index' => [
					['profile2utility_tabid_activityid_idx', ['tabid', 'activityid']],
					['profile2utility_profileid', 'profileid'],
				],
				'primaryKeys' => [
					['profile2utility_pk', ['profileid', 'tabid', 'activityid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_progress' => [
				'columns' => [
					'progressid' => $this->primaryKey(10),
					'progress' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_progress_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_project' => [
				'columns' => [
					'projectid' => $this->integer(10)->notNull(),
					'projectname' => $this->stringType(),
					'project_no' => $this->stringType(100),
					'startdate' => $this->date(),
					'targetenddate' => $this->date(),
					'actualenddate' => $this->date(),
					'targetbudget' => $this->stringType(),
					'projecturl' => $this->stringType(),
					'projectstatus' => $this->stringType(100),
					'projectpriority' => $this->stringType(100),
					'projecttype' => $this->stringType(100),
					'progress' => $this->stringType(100),
					'linktoaccountscontacts' => $this->integer(10),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'servicecontractsid' => $this->integer(10),
					'ssalesprocessesid' => $this->integer(10),
					'parentid' => $this->integer(10),
				],
				'index' => [
					['servicecontractsid', 'servicecontractsid'],
					['linktoaccountscontacts', 'linktoaccountscontacts'],
					['projectname', 'projectname'],
					['ssalesprocessesid', 'ssalesprocessesid'],
					['project_parentid_idx', 'parentid'],
					['project_no', 'project_no'],
				],
				'primaryKeys' => [
					['project_pk', 'projectid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectcf' => [
				'columns' => [
					'projectid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['projectcf_pk', 'projectid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectmilestone' => [
				'columns' => [
					'projectmilestoneid' => $this->integer(10)->notNull(),
					'projectmilestonename' => $this->stringType(),
					'projectmilestone_no' => $this->stringType(100),
					'projectmilestonedate' => $this->stringType(),
					'projectmilestone_status' => $this->stringType()->defaultValue(''),
					'projectid' => $this->integer(10),
					'projectmilestonetype' => $this->stringType(100),
					'projectmilestone_priority' => $this->stringType(),
					'projectmilestone_progress' => $this->stringType(10),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'parentid' => $this->integer(10),
				],
				'index' => [
					['projectid', 'projectid'],
					['vtiger_projectmilestone_parentid_idx', 'parentid'],
				],
				'primaryKeys' => [
					['projectmilestone_pk', 'projectmilestoneid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectmilestone_priority' => [
				'columns' => [
					'projectmilestone_priorityid' => $this->primaryKey(10),
					'projectmilestone_priority' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectmilestone_priority_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectmilestone_status' => [
				'columns' => [
					'projectmilestone_statusid' => $this->primaryKey(),
					'projectmilestone_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectmilestonecf' => [
				'columns' => [
					'projectmilestoneid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['projectmilestonecf_pk', 'projectmilestoneid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectmilestonetype' => [
				'columns' => [
					'projectmilestonetypeid' => $this->primaryKey(10),
					'projectmilestonetype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectmilestonetype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectpriority' => [
				'columns' => [
					'projectpriorityid' => $this->primaryKey(10),
					'projectpriority' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectpriority_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectstatus' => [
				'columns' => [
					'projectstatusid' => $this->primaryKey(10),
					'projectstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
					'color' => $this->stringType(25)->defaultValue('#E6FAD8'),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projectstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttask' => [
				'columns' => [
					'projecttaskid' => $this->integer(10)->notNull(),
					'projecttaskname' => $this->stringType(),
					'projecttask_no' => $this->stringType(100),
					'projecttasktype' => $this->stringType(100),
					'projecttaskpriority' => $this->stringType(100),
					'projecttaskprogress' => $this->stringType(100),
					'startdate' => $this->date(),
					'enddate' => $this->date(),
					'projectid' => $this->integer(10),
					'projecttasknumber' => $this->integer(10),
					'projecttaskstatus' => $this->stringType(100),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'parentid' => $this->integer(10),
					'projectmilestoneid' => $this->integer(10),
					'targetenddate' => $this->date(),
					'estimated_work_time' => $this->decimal('8,2'),
				],
				'index' => [
					['parentid', 'parentid'],
					['projectmilestoneid', 'projectmilestoneid'],
					['projectid', 'projectid'],
					['projecttaskname', 'projecttaskname'],
				],
				'primaryKeys' => [
					['projecttask_pk', 'projecttaskid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttaskcf' => [
				'columns' => [
					'projecttaskid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['projecttaskcf_pk', 'projecttaskid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttaskpriority' => [
				'columns' => [
					'projecttaskpriorityid' => $this->primaryKey(10),
					'projecttaskpriority' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttaskpriority_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttaskprogress' => [
				'columns' => [
					'projecttaskprogressid' => $this->primaryKey(10),
					'projecttaskprogress' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttaskprogress_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttaskstatus' => [
				'columns' => [
					'projecttaskstatusid' => $this->primaryKey(10),
					'projecttaskstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttaskstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttasktype' => [
				'columns' => [
					'projecttasktypeid' => $this->primaryKey(10),
					'projecttasktype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttasktype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttype' => [
				'columns' => [
					'projecttypeid' => $this->primaryKey(10),
					'projecttype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_projecttype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_publicholiday' => [
				'columns' => [
					'publicholidayid' => $this->primaryKey(10)->unsigned(),
					'holidaydate' => $this->date()->notNull(),
					'holidayname' => $this->stringType()->notNull(),
					'holidaytype' => $this->stringType(25),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
		];
		$this->foreignKey = [
			['fk_2_vtiger_group2grouprel', 'vtiger_group2grouprel', 'groupid', 'vtiger_groups', 'groupid', 'CASCADE', 'CASCADE'],
			['vtiger_group2modules_ibfk_1', 'vtiger_group2modules', 'groupid', 'vtiger_groups', 'groupid', 'CASCADE', 'RESTRICT'],
			['vtiger_group2modules_ibfk_2', 'vtiger_group2modules', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_group2role', 'vtiger_group2role', 'roleid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_group2rs', 'vtiger_group2rs', 'roleandsubid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_holidaysentitlement', 'vtiger_holidaysentitlement', 'holidaysentitlementid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_holidaysentitlementcf', 'vtiger_holidaysentitlementcf', 'holidaysentitlementid', 'vtiger_holidaysentitlement', 'holidaysentitlementid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ideas', 'vtiger_ideas', 'ideasid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ideascf', 'vtiger_ideascf', 'ideasid', 'vtiger_ideas', 'ideasid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_leadaddress', 'vtiger_leadaddress', 'leadaddressid', 'vtiger_leaddetails', 'leadid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_leaddetails', 'vtiger_leaddetails', 'leadid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_leadscf', 'vtiger_leadscf', 'leadid', 'vtiger_leaddetails', 'leadid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_leadsubdetails', 'vtiger_leadsubdetails', 'leadsubscriptionid', 'vtiger_leaddetails', 'leadid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_lettersin', 'vtiger_lettersin', 'lettersinid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_lettersincf', 'vtiger_lettersincf', 'lettersinid', 'vtiger_lettersin', 'lettersinid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_lettersout', 'vtiger_lettersout', 'lettersoutid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_lettersoutcf', 'vtiger_lettersoutcf', 'lettersoutid', 'vtiger_lettersout', 'lettersoutid', 'CASCADE', 'RESTRICT'],
			['vtiger_modcomments_ibfk_1', 'vtiger_modcomments', 'related_to', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_modcommentscf_ibfk_1', 'vtiger_modcommentscf', 'modcommentsid', 'vtiger_modcomments', 'modcommentsid', 'CASCADE', 'RESTRICT'],
			['vtiger_module_dashboard_widgets_ibfk_1', 'vtiger_module_dashboard_widgets', 'templateid', 'vtiger_module_dashboard', 'id', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_notes', 'vtiger_notes', 'notesid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_notescf_ibfk_1', 'vtiger_notescf', 'notesid', 'vtiger_notes', 'notesid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_org_share_action2tab', 'vtiger_org_share_action2tab', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ossemployees', 'vtiger_ossemployees', 'ossemployeesid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ossemployeescf', 'vtiger_ossemployeescf', 'ossemployeesid', 'vtiger_ossemployees', 'ossemployeesid', 'CASCADE', 'RESTRICT'],
			['vtiger_ossmailscanner_folders_uid_ibfk_1', 'vtiger_ossmailscanner_folders_uid', 'user_id', 'roundcube_users', 'user_id', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ossmailview', 'vtiger_ossmailview', 'ossmailviewid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ossmailview_files', 'vtiger_ossmailview_files', 'ossmailviewid', 'vtiger_ossmailview', 'ossmailviewid', 'CASCADE', 'RESTRICT'],
			['vtiger_ossmailview_relation_ibfk_1', 'vtiger_ossmailview_relation', 'ossmailviewid', 'vtiger_ossmailview', 'ossmailviewid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ossmailviewcf', 'vtiger_ossmailviewcf', 'ossmailviewid', 'vtiger_ossmailview', 'ossmailviewid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ossoutsourcedservices', 'vtiger_ossoutsourcedservices', 'ossoutsourcedservicesid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_ossoutsourcedservicescf', 'vtiger_ossoutsourcedservicescf', 'ossoutsourcedservicesid', 'vtiger_ossoutsourcedservices', 'ossoutsourcedservicesid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_osspasswords', 'vtiger_osspasswords', 'osspasswordsid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_osspasswordscf', 'vtiger_osspasswordscf', 'osspasswordsid', 'vtiger_osspasswords', 'osspasswordsid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_osssoldservices', 'vtiger_osssoldservices', 'osssoldservicesid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_osssoldservicescf', 'vtiger_osssoldservicescf', 'osssoldservicesid', 'vtiger_osssoldservices', 'osssoldservicesid', 'CASCADE', 'RESTRICT'],
			['vtiger_osstimecontrol', 'vtiger_osstimecontrol', 'osstimecontrolid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_osstimecontrolcf', 'vtiger_osstimecontrolcf', 'osstimecontrolid', 'vtiger_osstimecontrol', 'osstimecontrolid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_outsourcedproducts', 'vtiger_outsourcedproducts', 'outsourcedproductsid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_outsourcedproductscf', 'vtiger_outsourcedproductscf', 'outsourcedproductsid', 'vtiger_outsourcedproducts', 'outsourcedproductsid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_paymentsin', 'vtiger_paymentsin', 'paymentsinid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_paymentsincf', 'vtiger_paymentsincf', 'paymentsinid', 'vtiger_paymentsin', 'paymentsinid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_paymentsout', 'vtiger_paymentsout', 'paymentsoutid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_paymentsoutcf', 'vtiger_paymentsoutcf', 'paymentsoutid', 'vtiger_paymentsout', 'paymentsoutid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_pricebook', 'vtiger_pricebook', 'pricebookid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_pricebookcf', 'vtiger_pricebookcf', 'pricebookid', 'vtiger_pricebook', 'pricebookid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_pricebookproductrel', 'vtiger_pricebookproductrel', 'pricebookid', 'vtiger_pricebook', 'pricebookid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_productcf', 'vtiger_productcf', 'productid', 'vtiger_products', 'productid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_products', 'vtiger_products', 'productid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_profile2field_ibfk_1', 'vtiger_profile2field', 'profileid', 'vtiger_profile', 'profileid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_profile2globalpermissions', 'vtiger_profile2globalpermissions', 'profileid', 'vtiger_profile', 'profileid', 'CASCADE', 'RESTRICT'],
			['vtiger_profile2tab_ibfk_1', 'vtiger_profile2tab', 'profileid', 'vtiger_profile', 'profileid', 'CASCADE', 'RESTRICT'],
			['vtiger_profile2utility_ibfk_1', 'vtiger_profile2utility', 'profileid', 'vtiger_profile', 'profileid', 'CASCADE', 'RESTRICT'],
			['vtiger_project_ibfk_1', 'vtiger_project', 'projectid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_projectcf_ibfk_1', 'vtiger_projectcf', 'projectid', 'vtiger_project', 'projectid', 'CASCADE', 'RESTRICT'],
			['vtiger_projectmilestone_ibfk_1', 'vtiger_projectmilestone', 'projectmilestoneid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_projectmilestonecf_ibfk_1', 'vtiger_projectmilestonecf', 'projectmilestoneid', 'vtiger_projectmilestone', 'projectmilestoneid', 'CASCADE', 'RESTRICT'],
			['vtiger_projecttask_ibfk_1', 'vtiger_projecttask', 'projecttaskid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_projecttaskcf_ibfk_1', 'vtiger_projecttaskcf', 'projecttaskid', 'vtiger_projecttask', 'projecttaskid', 'CASCADE', 'RESTRICT'],
		];
	}

	public function data()
	{
		$this->data = [
			'vtiger_glacct' => [
				'columns' => ['glacctid', 'glacct', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, '300-Sales-Software', 1, 51, 0],
					[2, '301-Sales-Hardware', 1, 52, 1],
					[3, '302-Rental-Income', 1, 53, 2],
					[4, '303-Interest-Income', 1, 54, 3],
					[5, '304-Sales-Software-Support', 1, 55, 4],
					[6, '305-Sales Other', 1, 56, 5],
					[7, '306-Internet Sales', 1, 57, 6],
					[8, '307-Service-Hardware Labor', 1, 58, 7],
					[9, '308-Sales-Books', 1, 59, 8],
				]
			],
			'vtiger_glacct_seq' => [
				'columns' => ['id'],
				'values' => [
					[9],
				]
			],
			'vtiger_group2role' => [
				'columns' => ['groupid', 'roleid'],
				'values' => [
					[3, 'H2'],
					[4, 'H3'],
				]
			],
			'vtiger_group2rs' => [
				'columns' => ['groupid', 'roleandsubid'],
				'values' => [
					[3, 'H3'],
					[4, 'H3'],
				]
			],
			'vtiger_groups' => [
				'columns' => ['groupid', 'groupname', 'description', 'color', 'modules'],
				'values' => [
					[2, 'Team Selling', 'Group Related to Sales', '#E6FAD8', null],
					[3, 'Marketing Group', 'Group Related to Marketing Activities', '#E6FAD8', null],
					[4, 'Support Group', 'Group Related to providing Support to Customers', '#E6FAD8', null],
				]
			],
			'vtiger_holidaysentitlement_year' => [
				'columns' => ['holidaysentitlement_yearid', 'holidaysentitlement_year', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, '2020', 1, 576, 1],
					[2, '2019', 1, 577, 2],
					[3, '2018', 1, 578, 3],
					[4, '2017', 1, 579, 4],
					[5, '2016', 1, 580, 5],
					[6, '2015', 1, 581, 6],
					[7, '2014', 1, 582, 7],
					[8, '2013', 1, 583, 8],
					[9, '2012', 1, 584, 9],
					[10, '2011', 1, 585, 10],
					[11, '2010', 1, 586, 11],
				]
			],
			'vtiger_holidaysentitlement_year_seq' => [
				'columns' => ['id'],
				'values' => [
					[11],
				]
			],
			'vtiger_hour_format' => [
				'columns' => ['hour_formatid', 'hour_format', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, '12', 1, 295, 1],
					[2, '24', 1, 296, 2],
				]
			],
			'vtiger_hour_format_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_ideasstatus' => [
				'columns' => ['ideasstatusid', 'ideasstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Draft', 1, 564, 1],
					[2, 'Proposition', 1, 565, 2],
				]
			],
			'vtiger_ideasstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_igdn_status' => [
				'columns' => ['igdn_statusid', 'igdn_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 784],
					[2, 'PLL_IN_REALIZATION', 2, 1, 785],
					[3, 'PLL_OBJECTIONS', 3, 1, 786],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 787],
					[5, 'PLL_CANCELLED', 5, 1, 788],
					[6, 'PLL_ACCEPTED', 6, 0, 789],
				]
			],
			'vtiger_igdnc_status' => [
				'columns' => ['igdnc_statusid', 'igdnc_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 778],
					[2, 'PLL_IN_REALIZATION', 2, 1, 779],
					[3, 'PLL_OBJECTIONS', 3, 1, 780],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 781],
					[5, 'PLL_CANCELLED', 5, 1, 782],
					[6, 'PLL_ACCEPTED', 6, 1, 783],
				]
			],
			'vtiger_igin_status' => [
				'columns' => ['igin_statusid', 'igin_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 790],
					[2, 'PLL_IN_REALIZATION', 2, 1, 791],
					[3, 'PLL_OBJECTIONS', 3, 1, 792],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 793],
					[5, 'PLL_CANCELLED', 5, 1, 794],
					[6, 'PLL_ACCEPTED', 6, 0, 795],
				]
			],
			'vtiger_igrn_status' => [
				'columns' => ['igrn_statusid', 'igrn_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 802],
					[2, 'PLL_IN_REALIZATION', 2, 1, 803],
					[3, 'PLL_OBJECTIONS', 3, 1, 804],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 805],
					[5, 'PLL_CANCELLED', 5, 1, 806],
					[6, 'PLL_ACCEPTED', 6, 0, 807],
				]
			],
			'vtiger_igrnc_status' => [
				'columns' => ['igrnc_statusid', 'igrnc_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 796],
					[2, 'PLL_IN_REALIZATION', 2, 1, 797],
					[3, 'PLL_OBJECTIONS', 3, 1, 798],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 799],
					[5, 'PLL_CANCELLED', 5, 1, 800],
					[6, 'PLL_ACCEPTED', 6, 1, 801],
				]
			],
			'vtiger_iidn_status' => [
				'columns' => ['iidn_statusid', 'iidn_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 808],
					[2, 'PLL_IN_REALIZATION', 2, 1, 809],
					[3, 'PLL_OBJECTIONS', 3, 1, 810],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 811],
					[5, 'PLL_CANCELLED', 5, 1, 812],
					[6, 'PLL_ACCEPTED', 6, 0, 813],
				]
			],
			'vtiger_industry' => [
				'columns' => ['industryid', 'industry', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Administration', 1, 61, 1],
					[33, 'Construction Industry', 1, 455, 2],
					[34, 'Education', 1, 456, 3],
					[35, 'Power Industry', 1, 457, 4],
					[36, 'Finance', 1, 458, 5],
					[37, 'Trade', 1, 459, 6],
					[38, 'Hotels and Restaurants', 1, 460, 7],
					[39, 'Health Care', 1, 461, 8],
					[40, 'Industry / Manufacturing', 1, 462, 9],
					[41, 'Uniformed Services', 1, 463, 10],
					[42, 'Transport & Logistics', 1, 464, 11],
					[43, 'Technologies', 1, 465, 12],
				]
			],
			'vtiger_industry_seq' => [
				'columns' => ['id'],
				'values' => [
					[43],
				]
			],
			'vtiger_internal_tickets_status' => [
				'columns' => ['internal_tickets_statusid', 'internal_tickets_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_UNASSIGNED', 1, 738, 1],
					[2, 'PLL_AWAITING_REALIZATION', 1, 739, 2],
					[3, 'PLL_FOR_PROCESSING', 1, 740, 3],
					[4, 'PLL_IN_PROGRESSING', 1, 741, 4],
					[5, 'PLL_SUBMITTED_COMMENTS', 1, 742, 5],
					[6, 'PLL_FOR_APPROVAL', 1, 743, 6],
					[7, 'PLL_CANCELLED', 1, 744, 7],
					[8, 'PLL_ACCEPTED', 1, 745, 8],
				]
			],
			'vtiger_inventory_tandc' => [
				'columns' => ['id', 'type', 'tandc'],
				'values' => [
					[1, 'Inventory', ''],
				]
			],
			'vtiger_inventoryproductrel_seq' => [
				'columns' => ['id'],
				'values' => [
					[0],
				]
			],
			'vtiger_ipreorder_status' => [
				'columns' => ['ipreorder_statusid', 'ipreorder_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 814],
					[2, 'PLL_IN_REALIZATION', 2, 1, 815],
					[3, 'PLL_OBJECTIONS', 3, 1, 816],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 817],
					[5, 'PLL_CANCELLED', 5, 1, 818],
					[6, 'PLL_ACCEPTED', 6, 0, 819],
				]
			],
			'vtiger_istdn_status' => [
				'columns' => ['istdn_statusid', 'istdn_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 820],
					[2, 'PLL_IN_REALIZATION', 2, 1, 821],
					[3, 'PLL_OBJECTIONS', 3, 1, 822],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 823],
					[5, 'PLL_CANCELLED', 5, 1, 824],
					[6, 'PLL_ACCEPTED', 6, 0, 825],
				]
			],
			'vtiger_istn_status' => [
				'columns' => ['istn_statusid', 'istn_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 826],
					[2, 'PLL_IN_REALIZATION', 2, 1, 827],
					[3, 'PLL_OBJECTIONS', 3, 1, 828],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 829],
					[5, 'PLL_CANCELLED', 5, 1, 830],
					[6, 'PLL_ACCEPTED', 6, 0, 831],
				]
			],
			'vtiger_istn_type' => [
				'columns' => ['istn_typeid', 'istn_type', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_RECEIVED', 1, 0],
					[2, 'PLL_TRANSFER', 2, 0],
					[3, 'PLL_DISPATCHED', 3, 0],
				]
			],
			'vtiger_istrn_status' => [
				'columns' => ['istrn_statusid', 'istrn_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 832],
					[2, 'PLL_IN_REALIZATION', 2, 1, 833],
					[3, 'PLL_OBJECTIONS', 3, 1, 834],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 835],
					[5, 'PLL_CANCELLED', 5, 1, 836],
					[6, 'PLL_ACCEPTED', 6, 0, 837],
				]
			],
			'vtiger_knowledgebase_status' => [
				'columns' => ['knowledgebase_statusid', 'knowledgebase_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 838],
					[2, 'PLL_IN_REALIZATION', 2, 1, 839],
					[3, 'PLL_OBJECTIONS', 3, 1, 840],
					[4, 'PLL_FOR_APPROVAL', 4, 1, 841],
					[5, 'PLL_CANCELLED', 5, 0, 842],
					[6, 'PLL_ACCEPTED', 6, 0, 843],
					[7, 'PLL_ARCHIVES', 7, 0, 844],
				]
			],
			'vtiger_knowledgebase_view' => [
				'columns' => ['knowledgebase_viewid', 'knowledgebase_view', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_PAGE', 1, 0],
					[2, 'PLL_PRESENTATION', 2, 0],
				]
			],
			'vtiger_language' => [
				'columns' => ['id', 'name', 'prefix', 'label', 'lastupdated', 'sequence', 'isdefault', 'active'],
				'values' => [
					[1, 'English', 'en_us', 'US English', '2014-07-16 11:20:12', null, 1, 1],
					[2, 'Język Polski', 'pl_pl', 'Język Polski', '2014-07-16 11:20:40', null, 0, 1],
					[3, 'Deutsch', 'de_de', 'DE Deutsch', '2014-11-21 11:20:40', null, 0, 1],
					[4, 'Portuguese', 'pt_br', 'Brazilian Portuguese', '2017-06-05 11:20:40', null, 0, 1],
					[5, 'Russian', 'ru_ru', 'Russian', '2015-01-13 15:12:39', null, 0, 1],
					[6, 'Italian', 'it_it', 'Italian', '2017-12-23 15:12:39', null, 0, 1],
					[7, 'Spanish', 'es_es', 'ES Spanish', '2017-03-11 00:00:00', null, 0, 1],
					[8, 'Turkish', 'tr_tr', 'Turkish', '2018-02-13 00:00:00', null, 0, 1],
					[9, 'French', 'fr_fr', 'French', '2018-03-12 00:00:00', null, 0, 1],
				]
			],
			'vtiger_language_seq' => [
				'columns' => ['id'],
				'values' => [
					[7],
				]
			],
			'vtiger_lead_view' => [
				'columns' => ['lead_viewid', 'lead_view', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Today', 0, 1],
					[2, 'Last 2 Days', 1, 1],
					[3, 'Last Week', 2, 1],
				]
			],
			'vtiger_lead_view_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_leads_relation' => [
				'columns' => ['leads_relationid', 'leads_relation', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_B2C', 1, 1],
					[2, 'PLL_B2B', 2, 1],
				]
			],
			'vtiger_leads_relation_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_leadsource' => [
				'columns' => ['leadsourceid', 'leadsource', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Cold Call', 1, 99, 1],
					[4, 'Self Generated', 1, 101, 3],
					[9, 'Conference', 1, 106, 8],
					[10, 'Trade Show', 1, 107, 9],
					[11, 'Web Site', 1, 108, 10],
					[12, 'Word of mouth', 1, 109, 11],
					[13, 'Other', 1, 110, 12],
				]
			],
			'vtiger_leadsource_seq' => [
				'columns' => ['id'],
				'values' => [
					[13],
				]
			],
			'vtiger_leadstatus' => [
				'columns' => ['leadstatusid', 'leadstatus', 'presence', 'picklist_valueid', 'sortorderid', 'color'],
				'values' => [
					[13, 'PLL_TO_REALIZE', 1, 446, 2, '#E6FAD8'],
					[18, 'PLL_CONTACTS_IN_THE_FUTURE', 1, 451, 4, '#E6FAD8'],
					[19, 'PLL_LEAD_UNTAPPED', 1, 452, 6, '#E6FAD8'],
					[20, 'PLL_LEAD_ACQUIRED', 1, 453, 7, '#E6FAD8'],
					[21, 'PLL_PENDING', 0, 720, 1, '#E6FAD8'],
					[22, 'PLL_IN_REALIZATION', 1, 721, 3, '#E6FAD8'],
					[23, 'PLL_INCORRECT', 1, 722, 5, '#E6FAD8'],
				]
			],
			'vtiger_leadstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[23],
				]
			],
			'vtiger_legal_form' => [
				'columns' => ['legal_formid', 'legal_form', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_NATURAL_PERSON', 1, 1],
					[2, 'PLL_BUSINESS_ACTIVITY', 2, 1],
					[3, 'PLL_GENERAL_PARTNERSHIP', 3, 1],
					[4, 'PLL_PROFESSIONAL_PARTNERSHIP', 4, 1],
					[5, 'PLL_LIMITED_PARTNERSHIP', 5, 1],
					[6, 'PLL_JOINT_STOCK_PARTNERSHIP', 6, 1],
					[7, 'PLL_LIMITED_LIABILITY_COMPANY', 7, 1],
					[8, 'PLL_STOCK_OFFERING_COMPANY', 8, 1],
					[9, 'PLL_GOVERMENT_ENTERPRISE', 9, 1],
					[10, 'PLL_ASSOCIATION', 10, 1],
					[11, 'PLL_COOPERATIVE', 11, 1],
					[12, 'PLL_FOUNDATION', 12, 1],
					[13, 'PLL_EUROPEAN_PARTNERSHIP', 13, 1],
					[14, 'PLL_EUROPEAN_ECONOMIC_INTEREST_GROUPING', 14, 1],
					[15, 'PLL_EUROPEAN_COOPERATIVE', 15, 1],
					[16, 'PLL_EUROPEAN_PRIVATE_PARTNERSHIP', 16, 1],
					[17, 'PLL_EUROPEAN_RECIPROCAL_PARTNERSHIP', 17, 1],
					[18, 'PLL_EUROPEAN_ASSOCIATION', 18, 1],
					[19, 'PLL_UFCIITS', 19, 1],
				]
			],
			'vtiger_legal_form_seq' => [
				'columns' => ['id'],
				'values' => [
					[19],
				]
			],
			'vtiger_lin_dimensions' => [
				'columns' => ['lin_dimensionsid', 'lin_dimensions', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_A', 1, 1],
					[2, 'PLL_B', 2, 1],
				]
			],
			'vtiger_lin_dimensions_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_lin_status' => [
				'columns' => ['lin_statusid', 'lin_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_NEW', 1, 1, 845],
					[2, 'PLL_SETTLED', 2, 1, 846],
				]
			],
			'vtiger_lin_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_lin_type_doc' => [
				'columns' => ['lin_type_docid', 'lin_type_doc', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DEMAND_PAYMENT', 1, 593, 1],
					[2, 'PLL_BANK_STATEMENTS', 1, 594, 2],
					[3, 'PLL_ORDER', 1, 595, 3],
					[4, 'PLL_ENQUIRY', 1, 596, 4],
					[5, 'PLL_CONTRACT', 1, 597, 5],
					[6, 'PLL_RECLAMATION', 1, 598, 6],
					[7, 'PLL_ADVERTISEMENT', 1, 599, 7],
					[8, 'PLL_MESSAGE', 1, 600, 8],
					[9, 'PLL_RECEIPT', 1, 601, 9],
					[10, 'PLL_APPLICATION', 1, 602, 10],
					[11, 'PLL_OFFER', 1, 603, 11],
					[12, 'PLL_PERSONAL_LETTER', 1, 604, 12],
					[13, 'PLL_PROFORMA', 1, 605, 13],
					[14, 'PLL_CORRECTION', 1, 606, 14],
					[15, 'PLL_INVOICE', 1, 607, 15],
				]
			],
			'vtiger_lin_type_doc_seq' => [
				'columns' => ['id'],
				'values' => [
					[15],
				]
			],
			'vtiger_lin_type_ship' => [
				'columns' => ['lin_type_shipid', 'lin_type_ship', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_LOCAL_PICKUP', 1, 1],
					[2, 'PLL_POST_OFFICE_PL', 2, 1],
					[3, 'PLL_POST_OFFICE', 3, 1],
					[4, 'PLL_COURIER', 4, 1],
					[5, 'PLL_TELEX', 5, 1],
					[6, 'PLL_EMAIL', 6, 1],
					[7, 'PLL_POCZTEX', 7, 1],
					[8, 'PLL_TNT', 8, 1],
					[9, 'PLL_COURIER_POCZTEX', 9, 1],
					[10, 'PLL_COURIER_7', 10, 1],
					[11, 'PLL_COURIER_DPD', 11, 1],
					[12, 'PLL_COURIER_DHL', 12, 1],
					[13, 'PLL_COURIER_UPS', 13, 1],
					[14, 'PLL_COURIER_GLS', 14, 1],
				]
			],
			'vtiger_lin_type_ship_seq' => [
				'columns' => ['id'],
				'values' => [
					[14],
				]
			],
			'vtiger_links' => [
				'columns' => ['linkid', 'tabid', 'linktype', 'linklabel', 'linkurl', 'linkicon', 'sequence', 'handler_path', 'handler_class', 'handler', 'params'],
				'values' => [
					[1, 3, 'DASHBOARDWIDGET', 'DW_SUMMATION_BY_MONTHS', 'index.php?module=FInvoice&view=ShowWidget&name=SummationByMonths', null, 0, null, null, null, null],
					[2, 3, 'DASHBOARDWIDGET', 'DW_SUMMATION_BY_USER', 'index.php?module=FInvoice&view=ShowWidget&name=SummationByUser', null, null, null, null, null, null],
					[3, 3, 'DASHBOARDWIDGET', 'Notifications', 'index.php?module=Notification&view=ShowWidget&name=Notifications', null, 3, null, null, null, null],
					[11, 7, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[12, 4, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[13, 6, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[45, 6, 'DASHBOARDWIDGET', 'History', 'index.php?module=Accounts&view=ShowWidget&name=History', '', 1, null, null, null, null],
					[46, 6, 'DASHBOARDWIDGET', 'Upcoming Activities', 'index.php?module=Accounts&view=ShowWidget&name=CalendarActivities', '', 2, null, null, null, null],
					[47, 6, 'DASHBOARDWIDGET', 'Overdue Activities', 'index.php?module=Accounts&view=ShowWidget&name=OverdueActivities', '', 3, null, null, null, null],
					[48, 4, 'DASHBOARDWIDGET', 'History', 'index.php?module=Contacts&view=ShowWidget&name=History', '', 1, null, null, null, null],
					[49, 4, 'DASHBOARDWIDGET', 'Upcoming Activities', 'index.php?module=Contacts&view=ShowWidget&name=CalendarActivities', '', 2, null, null, null, null],
					[50, 4, 'DASHBOARDWIDGET', 'Overdue Activities', 'index.php?module=Contacts&view=ShowWidget&name=OverdueActivities', '', 3, null, null, null, null],
					[51, 7, 'DASHBOARDWIDGET', 'History', 'index.php?module=Leads&view=ShowWidget&name=History', '', 1, null, null, null, null],
					[52, 7, 'DASHBOARDWIDGET', 'Upcoming Activities', 'index.php?module=Leads&view=ShowWidget&name=CalendarActivities', '', 2, null, null, null, null],
					[53, 7, 'DASHBOARDWIDGET', 'Leads by Status', 'index.php?module=Leads&view=ShowWidget&name=LeadsByStatus', '', 4, null, null, null, null],
					[54, 7, 'DASHBOARDWIDGET', 'Leads by Source', 'index.php?module=Leads&view=ShowWidget&name=LeadsBySource', '', 5, null, null, null, null],
					[55, 7, 'DASHBOARDWIDGET', 'Leads by Industry', 'index.php?module=Leads&view=ShowWidget&name=LeadsByIndustry', '', 6, null, null, null, null],
					[56, 7, 'DASHBOARDWIDGET', 'Overdue Activities', 'index.php?module=Leads&view=ShowWidget&name=OverdueActivities', '', 7, null, null, null, null],
					[57, 13, 'DASHBOARDWIDGET', 'Tickets by Status', 'index.php?module=HelpDesk&view=ShowWidget&name=TicketsByStatus', '', 1, null, null, null, null],
					[58, 13, 'DASHBOARDWIDGET', 'Open Tickets', 'index.php?module=HelpDesk&view=ShowWidget&name=OpenTickets', '', 2, null, null, null, null],
					[59, 3, 'DASHBOARDWIDGET', 'History', 'index.php?module=Home&view=ShowWidget&name=History', '', 1, null, null, null, null],
					[60, 3, 'DASHBOARDWIDGET', 'Upcoming Activities', 'index.php?module=Home&view=ShowWidget&name=CalendarActivities', '', 2, null, null, null, null],
					[66, 3, 'DASHBOARDWIDGET', 'Leads by Status', 'index.php?module=Leads&view=ShowWidget&name=LeadsByStatus', '', 10, null, null, null, null],
					[67, 3, 'DASHBOARDWIDGET', 'Leads by Source', 'index.php?module=Leads&view=ShowWidget&name=LeadsBySource', '', 11, null, null, null, null],
					[68, 3, 'DASHBOARDWIDGET', 'Leads by Industry', 'index.php?module=Leads&view=ShowWidget&name=LeadsByIndustry', '', 12, null, null, null, null],
					[69, 3, 'DASHBOARDWIDGET', 'Overdue Activities', 'index.php?module=Home&view=ShowWidget&name=OverdueActivities', '', 13, null, null, null, null],
					[70, 3, 'DASHBOARDWIDGET', 'Tickets by Status', 'index.php?module=HelpDesk&view=ShowWidget&name=TicketsByStatus', '', 13, null, null, null, null],
					[71, 3, 'DASHBOARDWIDGET', 'Open Tickets', 'index.php?module=HelpDesk&view=ShowWidget&name=OpenTickets', '', 14, null, null, null, null],
					[72, 13, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[73, 15, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[102, 3, 'DASHBOARDWIDGET', 'Key Metrics', 'index.php?module=Home&view=ShowWidget&name=KeyMetrics', '', 15, null, null, null, null],
					[103, 3, 'DASHBOARDWIDGET', 'Mini List', 'index.php?module=Home&view=ShowWidget&name=MiniList', '', 16, null, null, null, null],
					[107, 3, 'DASHBOARDWIDGET', 'Notebook', 'index.php?module=Home&view=ShowWidget&name=Notebook', '', 18, null, null, null, null],
					[111, 42, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[114, 43, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[186, 51, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[188, 54, 'DASHBOARDWIDGET', 'Graf', 'index.php?module=OSSMailView&view=ShowWidget&name=Graf', '', 1, null, null, null, null],
					[195, 61, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[198, 48, 'HEADERSCRIPT', 'OSSMailJScheckmails', 'layouts/_layoutName_/modules/OSSMail/resources/checkmails.js', null, 0, null, null, null, null],
					[199, 3, 'DASHBOARDWIDGET', 'Mails List', 'index.php?module=Home&view=ShowWidget&name=MailsList', '', null, null, null, null, null],
					[201, 3, 'DASHBOARDWIDGET', 'Delagated Events/To Dos', 'index.php?module=Home&view=ShowWidget&name=AssignedUpcomingCalendarTasks', '', null, null, null, null, null],
					[202, 3, 'DASHBOARDWIDGET', 'Delegated (overdue) Events/ToDos', 'index.php?module=Home&view=ShowWidget&name=AssignedOverdueCalendarTasks', '', null, null, null, null, null],
					[203, 3, 'DASHBOARDWIDGET', 'Delegated project tasks', 'index.php?module=Home&view=ShowWidget&name=AssignedUpcomingProjectsTasks', null, null, null, null, null, null],
					[204, 3, 'DASHBOARDWIDGET', 'Delegated (overdue) project tasks', 'index.php?module=Home&view=ShowWidget&name=AssignedOverdueProjectsTasks', null, null, null, null, null, null],
					[205, 7, 'DASHBOARDWIDGET', 'Leads by Status Converted', 'index.php?module=Leads&view=ShowWidget&name=LeadsByStatusConverted', null, 8, null, null, null, null],
					[206, 3, 'DASHBOARDWIDGET', 'Leads by Status Converted', 'index.php?module=Leads&view=ShowWidget&name=LeadsByStatusConverted', null, 24, null, null, null, null],
					[208, 3, 'DASHBOARDWIDGET', 'Employees Time Control', 'index.php?module=OSSTimeControl&view=ShowWidget&name=TimeControl', '', 12, null, null, null, null],
					[209, 61, 'DASHBOARDWIDGET', 'Employees Time Control', 'index.php?module=OSSTimeControl&view=ShowWidget&name=TimeControl', '', 1, null, null, null, null],
					[211, 75, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 1, null, null, null, null],
					[212, 40, 'HEADERSCRIPT', 'ModCommentsCommonHeaderScript', 'modules/ModComments/ModCommentsCommon.js', '', 0, null, null, null, null],
					[213, 3, 'DASHBOARDWIDGET', 'LIST_OF_LAST_UPDATED_RECORD', 'index.php?module=Home&view=ShowWidget&name=ListUpdatedRecord', '', 1, null, null, null, null],
					[220, 81, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[222, 82, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[228, 84, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[229, 3, 'DASHBOARDWIDGET', 'Calendar', 'index.php?module=Home&view=ShowWidget&name=Calendar', '', 0, null, null, null, null],
					[232, 3, 'DASHBOARDWIDGET', 'LBL_CREATED_BY_ME_BUT_NOT_MINE_ACTIVITIES', 'index.php?module=Home&view=ShowWidget&name=CreatedNotMineActivities', '', 0, null, null, null, null],
					[235, 85, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[237, 86, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[239, 87, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[241, 88, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[243, 89, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[245, 90, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[247, 91, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[249, 92, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[251, 93, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[252, 18, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[254, 95, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[255, 99, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[256, 100, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[257, 101, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[258, 98, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[259, 102, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[260, 103, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[261, 97, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[262, 94, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[263, 104, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[264, 105, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[265, 106, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[266, 96, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[267, 3, 'DASHBOARDWIDGET', 'LBL_PRODUCTS_SOLD_TO_RENEW', 'index.php?module=Home&view=ShowWidget&name=ProductsSoldToRenew', '', 0, null, null, null, null],
					[268, 3, 'DASHBOARDWIDGET', 'LBL_SOLD_SERVICES_TO_RENEW', 'index.php?module=Home&view=ShowWidget&name=ServicesSoldToRenew', '', 0, null, null, null, null],
					[269, 107, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[270, 108, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[271, 109, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[272, 4, 'DASHBOARDWIDGET', 'Mini List', 'index.php?module=Home&view=ShowWidget&name=MiniList', '', 0, null, null, null, null],
					[273, 6, 'DASHBOARDWIDGET', 'Mini List', 'index.php?module=Home&view=ShowWidget&name=MiniList', '', 0, null, null, null, null],
					[274, 7, 'DASHBOARDWIDGET', 'Mini List', 'index.php?module=Home&view=ShowWidget&name=MiniList', '', 0, null, null, null, null],
					[275, 13, 'DASHBOARDWIDGET', 'Mini List', 'index.php?module=Home&view=ShowWidget&name=MiniList', '', 0, null, null, null, null],
					[276, 54, 'DASHBOARDWIDGET', 'Mini List', 'index.php?module=Home&view=ShowWidget&name=MiniList', '', 0, null, null, null, null],
					[277, 61, 'DASHBOARDWIDGET', 'Mini List', 'index.php?module=Home&view=ShowWidget&name=MiniList', '', 0, null, null, null, null],
					[278, 3, 'DASHBOARDWIDGET', 'LBL_ALL_TIME_CONTROL', 'index.php?module=OSSTimeControl&view=ShowWidget&name=AllTimeControl', null, 0, null, null, null, null],
					[279, 3, 'DASHBOARDWIDGET', 'LBL_NEW_ACCOUNTS', 'index.php?module=Accounts&view=ShowWidget&name=NewAccounts', '', 0, null, null, null, null],
					[280, 3, 'DASHBOARDWIDGET', 'LBL_NEGLECTED_ACCOUNTS', 'index.php?module=Accounts&view=ShowWidget&name=NeglectedAccounts', '', 0, null, null, null, null],
					[281, 3, 'DASHBOARDWIDGET', 'Chart', 'index.php?module=Reports&view=ShowWidget&name=Charts', '', 0, null, null, null, null],
					[282, 26, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, null, null, null, null],
					[283, 3, 'DASHBOARDWIDGET', 'ChartFilter', 'index.php?module=Home&view=ShowWidget&name=ChartFilter', '', 0, null, null, null, null],
					[284, 6, 'DASHBOARDWIDGET', 'ChartFilter', 'index.php?module=Home&view=ShowWidget&name=ChartFilter', '', 0, null, null, null, null],
					[285, 4, 'DASHBOARDWIDGET', 'ChartFilter', 'index.php?module=Home&view=ShowWidget&name=ChartFilter', '', 0, null, null, null, null],
					[286, 7, 'DASHBOARDWIDGET', 'ChartFilter', 'index.php?module=Home&view=ShowWidget&name=ChartFilter', '', 0, null, null, null, null],
					[287, 13, 'DASHBOARDWIDGET', 'ChartFilter', 'index.php?module=Home&view=ShowWidget&name=ChartFilter', '', 0, null, null, null, null],
					[288, 54, 'DASHBOARDWIDGET', 'ChartFilter', 'index.php?module=Home&view=ShowWidget&name=ChartFilter', '', 0, null, null, null, null],
					[289, 61, 'DASHBOARDWIDGET', 'ChartFilter', 'index.php?module=Home&view=ShowWidget&name=ChartFilter', '', 0, null, null, null, null],
					[290, 3, 'DASHBOARDWIDGET', 'Rss', 'index.php?module=Home&view=ShowWidget&name=Rss', '', 0, null, null, null, null],
					[291, 6, 'DASHBOARDWIDGET', 'Rss', 'index.php?module=Home&view=ShowWidget&name=Rss', '', 0, null, null, null, null],
					[292, 4, 'DASHBOARDWIDGET', 'Rss', 'index.php?module=Home&view=ShowWidget&name=Rss', '', 0, null, null, null, null],
					[293, 7, 'DASHBOARDWIDGET', 'Rss', 'index.php?module=Home&view=ShowWidget&name=Rss', '', 0, null, null, null, null],
					[294, 13, 'DASHBOARDWIDGET', 'Rss', 'index.php?module=Home&view=ShowWidget&name=Rss', '', 0, null, null, null, null],
					[295, 54, 'DASHBOARDWIDGET', 'Rss', 'index.php?module=Home&view=ShowWidget&name=Rss', '', 0, null, null, null, null],
					[296, 61, 'DASHBOARDWIDGET', 'Rss', 'index.php?module=Home&view=ShowWidget&name=Rss', '', 0, null, null, null, null],
					[297, 3, 'DASHBOARDWIDGET', 'LBL_CLOSED_TICKETS_BY_PRIORITY', 'index.php?module=HelpDesk&view=ShowWidget&name=ClosedTicketsByPriority', '', 0, null, null, null, null],
					[298, 3, 'DASHBOARDWIDGET', 'LBL_CLOSED_TICKETS_BY_USER', 'index.php?module=HelpDesk&view=ShowWidget&name=ClosedTicketsByUser', '', 0, null, null, null, null],
					[299, 3, 'DASHBOARDWIDGET', 'LBL_ACCOUNTS_BY_INDUSTRY', 'index.php?module=Accounts&view=ShowWidget&name=AccountsByIndustry', '', 0, null, null, null, null],
					[300, 6, 'DASHBOARDWIDGET', 'LBL_ACCOUNTS_BY_INDUSTRY', 'index.php?module=Accounts&view=ShowWidget&name=AccountsByIndustry', '', 0, null, null, null, null],
					[301, 3, 'DASHBOARDWIDGET', 'LBL_TOTAL_ESTIMATED_VALUE_BY_STATUS', 'index.php?module=SSalesProcesses&view=ShowWidget&name=EstimatedValueByStatus', '', 0, null, null, null, null],
					[302, 86, 'DASHBOARDWIDGET', 'LBL_TOTAL_ESTIMATED_VALUE_BY_STATUS', 'index.php?module=SSalesProcesses&view=ShowWidget&name=EstimatedValueByStatus', '', 0, null, null, null, null],
					[303, 111, 'DASHBOARDWIDGET', 'Notifications', 'index.php?module=Notification&view=ShowWidget&name=Notifications', '', 0, null, null, null, null],
					[304, 3, 'DASHBOARDWIDGET', 'LBL_NOTIFICATION_BY_SENDER', 'index.php?module=Notification&view=ShowWidget&name=NotificationsBySender', '', 0, null, null, null, null],
					[305, 111, 'DASHBOARDWIDGET', 'LBL_NOTIFICATION_BY_SENDER', 'index.php?module=Notification&view=ShowWidget&name=NotificationsBySender', '', 0, null, null, null, null],
					[306, 3, 'DASHBOARDWIDGET', 'LBL_NOTIFICATION_BY_RECIPIENT', 'index.php?module=Notification&view=ShowWidget&name=NotificationsByRecipient', '', 0, null, null, null, null],
					[307, 111, 'DASHBOARDWIDGET', 'LBL_NOTIFICATION_BY_RECIPIENT', 'index.php?module=Notification&view=ShowWidget&name=NotificationsByRecipient', '', 0, null, null, null, null],
					[308, 3, 'DASHBOARDWIDGET', 'LBL_EXPIRING_SOLD_PRODUCTS', 'index.php?module=Assets&view=ShowWidget&name=ExpiringSoldProducts', '', 0, null, null, null, null],
					[309, 113, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[310, 114, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[311, 115, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[312, 117, 'DETAILVIEWWIDGET', 'DetailViewBlockCommentWidget', 'block://ModComments:modules/ModComments/ModComments.php', '', 0, '', '', '', null],
					[313, 86, 'DASHBOARDWIDGET', 'DW_TEAMS_ESTIMATED_SALES', 'index.php?module=SSalesProcesses&view=ShowWidget&name=TeamsEstimatedSales', '', 0, null, null, null, null],
					[314, 86, 'DASHBOARDWIDGET', 'DW_ACTUAL_SALES_OF_TEAM', 'index.php?module=SSalesProcesses&view=ShowWidget&name=ActualSalesOfTeam', '', 0, null, null, null, null],
				]
			],
			'vtiger_lout_dimensions' => [
				'columns' => ['lout_dimensionsid', 'lout_dimensions', 'sortorderid', 'presence'],
				'values' => [
					[1, 'A', 1, 1],
					[2, 'B', 2, 1],
				]
			],
			'vtiger_lout_dimensions_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_lout_status' => [
				'columns' => ['lout_statusid', 'lout_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_NEW', 1, 1, 847],
					[2, 'PLL_SETTLED', 2, 1, 848],
				]
			],
			'vtiger_lout_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_lout_type_doc' => [
				'columns' => ['lout_type_docid', 'lout_type_doc', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DEMAND_PAYMENT', 1, 608, 1],
					[2, 'PLL_BANK_STATEMENTS', 1, 609, 2],
					[3, 'PLL_ORDER', 1, 610, 3],
					[4, 'PLL_ENQUIRY', 1, 611, 4],
					[5, 'PLL_CONTRACT', 1, 612, 5],
					[6, 'PLL_RECLAMATION', 1, 613, 6],
					[7, 'PLL_ADVERTISEMENT', 1, 614, 7],
					[8, 'PLL_MESSAGE', 1, 615, 8],
					[9, 'PLL_RECEIPT', 1, 616, 9],
					[10, 'PLL_APPLICATION', 1, 617, 10],
					[11, 'PLL_OFFER', 1, 618, 11],
					[12, 'PLL_PERSONAL_LETTER', 1, 619, 12],
					[13, 'PLL_PROFORMA', 1, 620, 13],
					[14, 'PLL_CORRECTION', 1, 621, 14],
					[15, 'PLL_INVOICE', 1, 622, 15],
				]
			],
			'vtiger_lout_type_doc_seq' => [
				'columns' => ['id'],
				'values' => [
					[15],
				]
			],
			'vtiger_lout_type_ship' => [
				'columns' => ['lout_type_shipid', 'lout_type_ship', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_LOCAL_PICKUP', 1, 1],
					[2, 'PLL_POST_OFFICE_PL', 2, 1],
					[3, 'PLL_POST_OFFICE', 3, 1],
					[4, 'PLL_COURIER', 4, 1],
					[5, 'PLL_TELEX', 5, 1],
					[6, 'PLL_EMAIL', 6, 1],
					[7, 'PLL_POCZTEX', 7, 1],
					[8, 'PLL_TNT', 8, 1],
					[9, 'PLL_COURIER_POCZTEX', 9, 1],
					[10, 'PLL_COURIER_7', 10, 1],
					[11, 'PLL_COURIER_DPD', 11, 1],
					[12, 'PLL_COURIER_DHL', 12, 1],
					[13, 'PLL_COURIER_UPS', 13, 1],
					[14, 'PLL_COURIER_GLS', 14, 1],
				]
			],
			'vtiger_lout_type_ship_seq' => [
				'columns' => ['id'],
				'values' => [
					[14],
				]
			],
			'vtiger_manufacturer' => [
				'columns' => ['manufacturerid', 'manufacturer', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'AltvetPet Inc.', 1, 124, 1],
					[3, 'LexPon Inc.', 1, 125, 2],
					[4, 'MetBeat Corp', 1, 126, 3],
				]
			],
			'vtiger_manufacturer_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_modentity_num' => [
				'columns' => ['id', 'tabid', 'prefix', 'postfix', 'start_id', 'cur_id'],
				'values' => [
					[1, 7, 'L', '', 1, 1],
					[2, 6, 'O', '', 1, 1],
					[3, 26, 'CAM', '', 1, 1],
					[4, 4, 'C', '', 1, 1],
					[5, 13, 'H', '', 1, 1],
					[6, 14, 'PRO', '', 1, 1],
					[7, 18, 'V', '', 1, 1],
					[8, 19, 'PB', '', 1, 1],
					[9, 15, 'FAQ', '', 1, 1],
					[10, 8, 'DOC', '', 1, 1],
					[11, 34, 'U', '', 1, 1],
					[12, 35, 'SER', '', 1, 1],
					[13, 37, 'A', '', 1, 1],
					[14, 41, 'PM', '', 1, 1],
					[15, 42, 'PT', '', 1, 1],
					[16, 43, 'PROJ', '', 1, 1],
					[17, 51, 'TC', '', 1, 1],
					[18, 54, 'M_', '', 1, 1],
					[19, 57, 'UO', '', 1, 1],
					[20, 58, 'US', '', 1, 1],
					[21, 59, 'UP', '', 1, 1],
					[22, 60, 'PASS', '', 1, 1],
					[23, 61, 'P', '', 1, 1],
					[24, 75, 'ID', '', 1, 1],
					[25, 78, 'HE', '', 1, 1],
					[26, 79, '', '', 1, 1],
					[27, 80, '', '', 1, 1],
					[28, 81, 'LI', '', 1, 1],
					[29, 82, 'LI', '', 1, 1],
					[30, 83, 'NO', '', 1, 1],
					[31, 84, 'RES', '', 1, 1],
					[32, 85, 'S-QE', '', 1, 1],
					[33, 86, 'S-SP', '', 1, 1],
					[34, 87, 'S-RC', '', 1, 1],
					[35, 88, 'S-C', '', 1, 1],
					[36, 89, 'S-Q', '', 1, 1],
					[37, 90, 'S-SO', '', 1, 1],
					[38, 91, 'S-RO', '', 1, 1],
					[39, 92, 'PR', '', 1, 1],
					[40, 93, 'CMP', '', 1, 1],
					[41, 94, 'F-B', '', 1, 1],
					[42, 98, 'I-GRN', '', 1, 1],
					[43, 99, 'F-IP', '', 1, 1],
					[44, 100, 'I-GDN', '', 1, 1],
					[45, 97, 'I-S', '', 1, 1],
					[46, 101, 'I-IDN', '', 1, 1],
					[47, 102, 'I-GIN', '', 1, 1],
					[48, 103, 'I-PO', '', 1, 1],
					[49, 95, 'F-I', '', 1, 1],
					[50, 96, 'KB', '', 1, 1],
					[51, 104, 'I-TD', '', 1, 1],
					[52, 105, 'I-SN', '', 1, 1],
					[53, 106, 'I-TR', '', 1, 1],
					[54, 107, 'F-CI', '', 1, 1],
					[55, 108, 'I-IGRNC', '', 1, 1],
					[56, 109, 'I-IGDNC', '', 1, 1],
					[57, 111, 'NT', '', 1, 1],
					[58, 112, 'N', '', 1, 25],
					[59, 113, 'FA', '', 1, 1],
					[60, 114, 'IT', '', 1, 1],
					[61, 115, 'FC', '', 1, 1],
					[62, 116, 'ML', '', 1, 1],
					[63, 117, 'S-VE', '', 1, 1],
				]
			],
			'vtiger_modtracker_tabs' => [
				'columns' => ['tabid', 'visible'],
				'values' => [
					[2, 1],
					[4, 1],
					[6, 1],
					[7, 1],
					[8, 1],
					[9, 1],
					[10, 1],
					[13, 1],
					[14, 1],
					[15, 1],
					[16, 1],
					[18, 1],
					[19, 1],
					[20, 1],
					[21, 1],
					[22, 1],
					[23, 1],
					[26, 1],
					[28, 1],
					[33, 1],
					[34, 1],
					[35, 1],
					[37, 1],
					[40, 1],
					[41, 1],
					[42, 1],
					[43, 1],
					[45, 1],
					[51, 1],
					[54, 0],
					[60, 1],
					[70, 1],
					[75, 1],
					[76, 1],
					[77, 1],
					[79, 1],
					[80, 1],
					[81, 1],
					[82, 1],
					[83, 1],
					[85, 1],
					[86, 1],
					[87, 1],
					[88, 1],
					[89, 1],
					[90, 1],
					[91, 1],
					[92, 1],
					[93, 1],
					[94, 1],
					[95, 1],
					[96, 1],
					[97, 1],
					[98, 1],
					[99, 1],
					[100, 1],
					[101, 1],
					[102, 1],
					[103, 1],
					[104, 1],
					[105, 1],
					[106, 1],
					[107, 1],
					[108, 1],
					[109, 1],
					[113, 1],
					[114, 1],
					[115, 1],
					[116, 1],
					[117, 1],
				]
			],
			'vtiger_no_of_currency_decimals' => [
				'columns' => ['no_of_currency_decimalsid', 'no_of_currency_decimals', 'sortorderid', 'presence'],
				'values' => [
					[1, '0', 0, 1],
					[2, '1', 1, 1],
					[3, '2', 2, 1],
					[4, '3', 3, 1],
					[5, '4', 4, 1],
					[6, '5', 5, 1],
				]
			],
			'vtiger_no_of_currency_decimals_seq' => [
				'columns' => ['id'],
				'values' => [
					[6],
				]
			],
			'vtiger_notification_status' => [
				'columns' => ['notification_statusid', 'notification_status', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_UNREAD', 1, 1],
					[2, 'PLL_READ', 2, 1],
				]
			],
			'vtiger_notification_type' => [
				'columns' => ['notification_typeid', 'notification_type', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_USERS', 0, 703, 1],
					[2, 'PLL_SYSTEM', 0, 704, 2],
				]
			],
			'vtiger_oproductstatus' => [
				'columns' => ['oproductstatusid', 'oproductstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Active', 1, 419, 1],
					[2, 'Inactive', 1, 420, 2],
				]
			],
			'vtiger_oproductstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_org_share_action2tab' => [
				'columns' => ['share_action_id', 'tabid'],
				'values' => [
					[0, 4],
					[0, 6],
					[0, 7],
					[0, 8],
					[0, 9],
					[0, 13],
					[0, 14],
					[0, 15],
					[0, 16],
					[0, 18],
					[0, 19],
					[0, 26],
					[0, 33],
					[0, 34],
					[0, 35],
					[0, 37],
					[0, 40],
					[0, 41],
					[0, 42],
					[0, 43],
					[0, 45],
					[0, 51],
					[0, 54],
					[0, 57],
					[0, 58],
					[0, 59],
					[0, 60],
					[0, 61],
					[0, 74],
					[0, 75],
					[0, 78],
					[0, 79],
					[0, 80],
					[0, 81],
					[0, 82],
					[0, 83],
					[0, 84],
					[0, 85],
					[0, 86],
					[0, 87],
					[0, 88],
					[0, 89],
					[0, 90],
					[0, 91],
					[0, 92],
					[0, 93],
					[0, 94],
					[0, 95],
					[0, 96],
					[0, 97],
					[0, 98],
					[0, 99],
					[0, 100],
					[0, 101],
					[0, 102],
					[0, 103],
					[0, 104],
					[0, 105],
					[0, 106],
					[0, 107],
					[0, 108],
					[0, 109],
					[0, 111],
					[0, 112],
					[0, 113],
					[0, 114],
					[0, 115],
					[0, 116],
					[0, 117],
					[1, 4],
					[1, 6],
					[1, 7],
					[1, 8],
					[1, 9],
					[1, 13],
					[1, 14],
					[1, 15],
					[1, 16],
					[1, 18],
					[1, 19],
					[1, 26],
					[1, 33],
					[1, 34],
					[1, 35],
					[1, 37],
					[1, 40],
					[1, 41],
					[1, 42],
					[1, 43],
					[1, 45],
					[1, 51],
					[1, 54],
					[1, 57],
					[1, 58],
					[1, 59],
					[1, 60],
					[1, 61],
					[1, 74],
					[1, 75],
					[1, 78],
					[1, 79],
					[1, 80],
					[1, 81],
					[1, 82],
					[1, 83],
					[1, 84],
					[1, 85],
					[1, 86],
					[1, 87],
					[1, 88],
					[1, 89],
					[1, 90],
					[1, 91],
					[1, 92],
					[1, 93],
					[1, 94],
					[1, 95],
					[1, 96],
					[1, 97],
					[1, 98],
					[1, 99],
					[1, 100],
					[1, 101],
					[1, 102],
					[1, 103],
					[1, 104],
					[1, 105],
					[1, 106],
					[1, 107],
					[1, 108],
					[1, 109],
					[1, 111],
					[1, 112],
					[1, 113],
					[1, 114],
					[1, 115],
					[1, 116],
					[1, 117],
					[2, 4],
					[2, 6],
					[2, 7],
					[2, 8],
					[2, 9],
					[2, 13],
					[2, 14],
					[2, 15],
					[2, 16],
					[2, 18],
					[2, 19],
					[2, 26],
					[2, 33],
					[2, 34],
					[2, 35],
					[2, 37],
					[2, 40],
					[2, 41],
					[2, 42],
					[2, 43],
					[2, 45],
					[2, 51],
					[2, 54],
					[2, 57],
					[2, 58],
					[2, 59],
					[2, 60],
					[2, 61],
					[2, 74],
					[2, 75],
					[2, 78],
					[2, 79],
					[2, 80],
					[2, 81],
					[2, 82],
					[2, 83],
					[2, 84],
					[2, 85],
					[2, 86],
					[2, 87],
					[2, 88],
					[2, 89],
					[2, 90],
					[2, 91],
					[2, 92],
					[2, 93],
					[2, 94],
					[2, 95],
					[2, 96],
					[2, 97],
					[2, 98],
					[2, 99],
					[2, 100],
					[2, 101],
					[2, 102],
					[2, 103],
					[2, 104],
					[2, 105],
					[2, 106],
					[2, 107],
					[2, 108],
					[2, 109],
					[2, 111],
					[2, 112],
					[2, 113],
					[2, 114],
					[2, 115],
					[2, 116],
					[2, 117],
					[3, 4],
					[3, 6],
					[3, 7],
					[3, 8],
					[3, 9],
					[3, 13],
					[3, 14],
					[3, 15],
					[3, 16],
					[3, 18],
					[3, 19],
					[3, 26],
					[3, 33],
					[3, 34],
					[3, 35],
					[3, 37],
					[3, 40],
					[3, 41],
					[3, 42],
					[3, 43],
					[3, 45],
					[3, 51],
					[3, 54],
					[3, 57],
					[3, 58],
					[3, 59],
					[3, 60],
					[3, 61],
					[3, 74],
					[3, 75],
					[3, 78],
					[3, 79],
					[3, 80],
					[3, 81],
					[3, 82],
					[3, 83],
					[3, 84],
					[3, 85],
					[3, 86],
					[3, 87],
					[3, 88],
					[3, 89],
					[3, 90],
					[3, 91],
					[3, 92],
					[3, 93],
					[3, 94],
					[3, 95],
					[3, 96],
					[3, 97],
					[3, 98],
					[3, 99],
					[3, 100],
					[3, 101],
					[3, 102],
					[3, 103],
					[3, 104],
					[3, 105],
					[3, 106],
					[3, 107],
					[3, 108],
					[3, 109],
					[3, 111],
					[3, 112],
					[3, 113],
					[3, 114],
					[3, 115],
					[3, 116],
					[3, 117],
				]
			],
			'vtiger_org_share_action_mapping' => [
				'columns' => ['share_action_id', 'share_action_name'],
				'values' => [
					[0, 'Public: Read Only'],
					[1, 'Public: Read, Create/Edit'],
					[2, 'Public: Read, Create/Edit, Delete'],
					[3, 'Private'],
					[4, 'Hide Details'],
					[5, 'Hide Details and Add Events'],
					[6, 'Show Details'],
					[7, 'Show Details and Add Events'],
				]
			],
			'vtiger_ossdc_status' => [
				'columns' => ['ossdc_statusid', 'ossdc_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'None', 1, 401, 1],
					[2, 'Checked', 1, 402, 2],
				]
			],
			'vtiger_ossdc_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_osservicesstatus' => [
				'columns' => ['osservicesstatusid', 'osservicesstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Active', 1, 403, 1],
					[2, 'Inactive', 1, 404, 2],
				]
			],
			'vtiger_osservicesstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_ossmailscanner_config' => [
				'columns' => ['conf_type', 'parameter', 'value'],
				'values' => [
					['emailsearch', 'fields', null],
					['cron', 'email', ''],
					['cron', 'time', ''],
					['emailsearch', 'changeTicketStatus', 'noAction'],
					['email_list', 'widget_limit', '10'],
					['email_list', 'target', '_blank'],
					['email_list', 'permissions', 'vtiger'],
					['exceptions', 'crating_mails', null],
					['exceptions', 'crating_tickets', null],
				]
			],
			'vtiger_ossmailview_sendtype' => [
				'columns' => ['ossmailview_sendtypeid', 'ossmailview_sendtype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Sent', 1, 388, 1],
					[2, 'Received', 1, 389, 2],
					[3, 'Internal', 1, 700, 3],
				]
			],
			'vtiger_ossmailview_sendtype_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_osssoldservices_renew' => [
				'columns' => ['osssoldservices_renewid', 'osssoldservices_renew', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_PLANNED', 0, 675, 1],
					[2, 'PLL_WAITING_FOR_RENEWAL', 0, 676, 2],
					[3, 'PLL_RENEWED_VERIFICATION', 0, 677, 3],
					[4, 'PLL_NOT_RENEWED_VERIFICATION', 0, 678, 4],
					[5, 'PLL_RENEWED', 0, 679, 6],
					[6, 'PLL_NOT_RENEWED', 0, 680, 7],
					[7, 'PLL_NOT_APPLICABLE', 0, 681, 8],
					[8, 'PLL_NOT_APPLICABLE_VERIFICATION', 0, 701, 5],
				]
			],
			'vtiger_osstimecontrol_status' => [
				'columns' => ['osstimecontrol_statusid', 'osstimecontrol_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[8, 'Accepted', 1, 0, 849],
					[13, 'To approval', 2, 1, 850],
				]
			],
			'vtiger_osstimecontrol_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[13],
				]
			],
			'vtiger_othereventduration' => [
				'columns' => ['othereventdurationid', 'othereventduration', 'sortorderid', 'presence'],
				'values' => [
					[1, '5', 1, 1],
					[2, '10', 2, 1],
					[3, '30', 3, 1],
					[4, '60', 4, 1],
					[5, '120', 5, 1],
				]
			],
			'vtiger_othereventduration_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_password' => [
				'columns' => ['type', 'val'],
				'values' => [
					['min_length', '8'],
					['max_length', '32'],
					['big_letters', 'true'],
					['small_letters', 'true'],
					['numbers', 'true'],
					['special', 'false'],
				]
			],
			'vtiger_passwords_config' => [
				'columns' => ['pass_length_min', 'pass_length_max', 'pass_allow_chars', 'register_changes'],
				'values' => [
					[10, 15, 'a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,w,x,y,z,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,W,X,Y,Z,0,1,2,3,4,5,6,7,8,9', 0],
				]
			],
			'vtiger_paymentsin_status' => [
				'columns' => ['paymentsin_statusid', 'paymentsin_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Created', 1, 587, 1],
					[2, 'Denied', 1, 588, 2],
					[3, 'Paid', 1, 589, 3],
				]
			],
			'vtiger_paymentsin_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_paymentsout_status' => [
				'columns' => ['paymentsout_statusid', 'paymentsout_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Created', 1, 590, 1],
					[2, 'Denied', 1, 591, 2],
					[3, 'Paid', 1, 592, 3],
				]
			],
			'vtiger_paymentsout_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_picklist' => [
				'columns' => ['picklistid', 'name'],
				'values' => [
					[96, 'accounts_status'],
					[1, 'accounttype'],
					[25, 'activitystatus'],
					[2, 'activitytype'],
					[94, 'announcementstatus'],
					[37, 'assetstatus'],
					[91, 'assets_renew'],
					[3, 'campaignstatus'],
					[4, 'campaigntype'],
					[106, 'cmileage_logbook_status'],
					[70, 'contactstatus'],
					[33, 'contract_priority'],
					[32, 'contract_status'],
					[34, 'contract_type'],
					[50, 'defaultactivitytype'],
					[49, 'defaulteventstatus'],
					[66, 'employee_education'],
					[65, 'employee_status'],
					[6, 'eventstatus'],
					[7, 'expectedresponse'],
					[8, 'faqcategories'],
					[9, 'faqstatus'],
					[92, 'fcorectinginvoice_formpayment'],
					[93, 'fcorectinginvoice_status'],
					[103, 'finvoicecost_formpayment'],
					[105, 'finvoicecost_paymentstatus'],
					[104, 'finvoicecost_status'],
					[88, 'finvoiceproforma_formpayment'],
					[89, 'finvoiceproforma_status'],
					[86, 'finvoice_formpayment'],
					[97, 'finvoice_paymentstatus'],
					[87, 'finvoice_status'],
					[98, 'finvoice_type'],
					[100, 'fixed_assets_status'],
					[99, 'fixed_assets_type'],
					[10, 'glacct'],
					[74, 'holidaysentitlement_year'],
					[47, 'hour_format'],
					[71, 'ideasstatus'],
					[108, 'igdnc_status'],
					[109, 'igdn_status'],
					[110, 'igin_status'],
					[111, 'igrnc_status'],
					[112, 'igrn_status'],
					[113, 'iidn_status'],
					[11, 'industry'],
					[102, 'internal_tickets_status'],
					[114, 'ipreorder_status'],
					[115, 'istdn_status'],
					[116, 'istn_status'],
					[117, 'istrn_status'],
					[118, 'knowledgebase_status'],
					[13, 'leadsource'],
					[14, 'leadstatus'],
					[119, 'lin_status'],
					[77, 'lin_type_doc'],
					[120, 'lout_status'],
					[78, 'lout_type_doc'],
					[15, 'manufacturer'],
					[95, 'notification_type'],
					[64, 'oproductstatus'],
					[59, 'ossdc_status'],
					[60, 'osservicesstatus'],
					[57, 'ossmailview_sendtype'],
					[90, 'osssoldservices_renew'],
					[121, 'osstimecontrol_status'],
					[75, 'paymentsin_status'],
					[76, 'paymentsout_status'],
					[45, 'progress'],
					[38, 'projectmilestonetype'],
					[85, 'projectmilestone_priority'],
					[44, 'projectpriority'],
					[42, 'projectstatus'],
					[40, 'projecttaskpriority'],
					[41, 'projecttaskprogress'],
					[46, 'projecttaskstatus'],
					[39, 'projecttasktype'],
					[43, 'projecttype'],
					[58, 'rel_mod'],
					[122, 'reservations_status'],
					[22, 'salutationtype'],
					[123, 'scalculations_status'],
					[35, 'service_usageunit'],
					[124, 'squoteenquiries_status'],
					[125, 'squotes_status'],
					[126, 'srecurringorders_status'],
					[127, 'srequirementscards_status'],
					[128, 'ssalesprocesses_status'],
					[129, 'ssingleorders_status'],
					[63, 'ssservicesstatus'],
					[48, 'start_hour'],
					[130, 'storage_status'],
					[107, 'svendorenquiries_status'],
					[24, 'taskpriority'],
					[27, 'ticketpriorities'],
					[28, 'ticketseverities'],
					[29, 'ticketstatus'],
					[31, 'tracking_unit'],
					[30, 'usageunit'],
					[68, 'verification'],
				]
			],
			'vtiger_picklist_dependency' => [
				'columns' => ['id', 'tabid', 'sourcefield', 'targetfield', 'sourcevalue', 'targetvalues', 'criteria'],
				'values' => [
					[1, 7, 'industry', 'subindustry', 'Administration', '["Ministry","Chancellery","Voivodeship Office","Marshal Office","District","City\\/Township\\/District","Social Welfare Centre","Water and Sewerage Company","Voivodeship Job Centre","District Job Center","Court of justice","Attorney General\'s Office","Other"]', null],
					[2, 7, 'industry', 'subindustry', 'Construction Industry', '["Other","Developers","Real Estate"]', null],
					[3, 7, 'industry', 'subindustry', 'Education', '["Other","Primary Schools","High Schools"]', null],
					[4, 7, 'industry', 'subindustry', 'Power Industry', '[""]', null],
					[5, 7, 'industry', 'subindustry', 'Finance', '["Other","Banking","Capital Market","Financial Services","Investments","Insurance"]', null],
					[6, 7, 'industry', 'subindustry', 'Trade', '["Other","Retail","Wholesale","Resale"]', null],
					[7, 7, 'industry', 'subindustry', 'Hotels and Restaurants', '[""]', null],
					[8, 7, 'industry', 'subindustry', 'Health Care', '[""]', null],
					[9, 7, 'industry', 'subindustry', 'Industry / Manufacturing', '["Other","Automotive","Plastics","Chamical","Raw material","Fuel","Wood and paper","Electromechanical","Pharmaceutical","Building Materials","Metal","Light","Food industry","Recycling"]', null],
					[10, 7, 'industry', 'subindustry', 'Uniformed Services', '["Army","Police"]', null],
					[11, 7, 'industry', 'subindustry', 'Transport & Logistics', '[""]', null],
					[12, 7, 'industry', 'subindustry', 'Technologies', '["Other","Information Technology","Telecommunication","Media"]', null],
					[13, 14, 'usageunit', 'subunit', 'pcs', '[""]', null],
					[14, 14, 'usageunit', 'subunit', 'pack', '[""]', null],
					[15, 14, 'usageunit', 'subunit', 'kg', '["50g","100g","300g","500g"]', null],
					[16, 14, 'usageunit', 'subunit', 'm', '[""]', null],
					[17, 14, 'usageunit', 'subunit', 'l', '["100ml","250ml","330ml","500ml"]', null],
				]
			],
			'vtiger_picklist_dependency_seq' => [
				'columns' => ['id'],
				'values' => [
					[17],
				]
			],
			'vtiger_picklistvalues_seq' => [
				'columns' => ['id'],
				'values' => [
					[909],
				]
			],
			'vtiger_profile' => [
				'columns' => ['profileid', 'profilename', 'description', 'directly_related_to_role'],
				'values' => [
					[1, 'Administrator', 'Admin Profile', 0],
					[2, 'Sales Profile', 'Profile Related to Sales', 0],
					[3, 'Support Profile', 'Profile Related to Support', 0],
					[4, 'Guest Profile', 'Guest Profile for Test Users', 0],
				]
			],
			'vtiger_profile2field' => [
				'columns' => ['profileid', 'tabid', 'fieldid', 'visible', 'readonly'],
				'values' => [
					[1, 6, 1, 0, 0],
					[1, 6, 2, 0, 0],
					[1, 6, 3, 0, 0],
					[1, 6, 4, 0, 0],
					[1, 6, 5, 0, 0],
					[1, 6, 7, 0, 0],
					[1, 6, 8, 0, 0],
					[1, 6, 9, 0, 0],
					[1, 6, 10, 0, 0],
					[1, 6, 11, 0, 0],
					[1, 6, 12, 0, 0],
					[1, 6, 14, 0, 0],
					[1, 6, 15, 0, 0],
					[1, 6, 16, 0, 0],
					[1, 6, 17, 0, 0],
					[1, 6, 18, 0, 0],
					[1, 6, 20, 0, 0],
					[1, 6, 21, 0, 1],
					[1, 6, 22, 0, 1],
					[1, 6, 23, 0, 0],
					[1, 6, 36, 0, 0],
					[1, 7, 39, 0, 0],
					[1, 7, 40, 0, 0],
					[1, 7, 41, 0, 0],
					[1, 7, 42, 0, 0],
					[1, 7, 43, 0, 0],
					[1, 7, 44, 0, 0],
					[1, 7, 46, 0, 0],
					[1, 7, 47, 0, 0],
					[1, 7, 48, 0, 0],
					[1, 7, 49, 0, 0],
					[1, 7, 50, 0, 0],
					[1, 7, 51, 0, 0],
					[1, 7, 53, 0, 0],
					[1, 7, 54, 0, 0],
					[1, 7, 55, 0, 0],
					[1, 7, 56, 0, 1],
					[1, 7, 57, 0, 1],
					[1, 7, 58, 0, 0],
					[1, 7, 65, 0, 0],
					[1, 4, 66, 0, 0],
					[1, 4, 67, 0, 0],
					[1, 4, 68, 0, 0],
					[1, 4, 69, 0, 0],
					[1, 4, 70, 0, 0],
					[1, 4, 71, 0, 0],
					[1, 4, 72, 0, 0],
					[1, 4, 74, 0, 0],
					[1, 4, 79, 0, 0],
					[1, 4, 80, 0, 0],
					[1, 4, 81, 0, 0],
					[1, 4, 85, 0, 0],
					[1, 4, 86, 0, 0],
					[1, 4, 87, 0, 0],
					[1, 4, 90, 0, 1],
					[1, 4, 91, 0, 1],
					[1, 4, 92, 0, 0],
					[1, 4, 93, 0, 0],
					[1, 4, 94, 0, 0],
					[1, 4, 95, 0, 0],
					[1, 4, 108, 0, 0],
					[1, 4, 109, 0, 0],
					[1, 26, 126, 0, 0],
					[1, 26, 127, 0, 0],
					[1, 26, 128, 0, 0],
					[1, 26, 129, 0, 0],
					[1, 26, 130, 0, 0],
					[1, 26, 131, 0, 0],
					[1, 26, 132, 0, 0],
					[1, 26, 133, 0, 0],
					[1, 26, 134, 0, 0],
					[1, 26, 135, 0, 0],
					[1, 26, 136, 0, 0],
					[1, 26, 137, 0, 1],
					[1, 26, 138, 0, 1],
					[1, 26, 139, 0, 0],
					[1, 26, 140, 0, 0],
					[1, 26, 141, 0, 0],
					[1, 26, 142, 0, 0],
					[1, 26, 143, 0, 0],
					[1, 26, 144, 0, 0],
					[1, 26, 145, 0, 0],
					[1, 26, 146, 0, 0],
					[1, 26, 147, 0, 0],
					[1, 26, 148, 0, 0],
					[1, 26, 149, 0, 0],
					[1, 26, 150, 0, 0],
					[1, 13, 155, 0, 0],
					[1, 13, 156, 0, 0],
					[1, 13, 157, 0, 0],
					[1, 13, 158, 0, 0],
					[1, 13, 159, 0, 0],
					[1, 13, 160, 0, 0],
					[1, 13, 161, 0, 0],
					[1, 13, 162, 0, 0],
					[1, 13, 163, 0, 0],
					[1, 13, 166, 0, 1],
					[1, 13, 167, 0, 1],
					[1, 13, 168, 0, 0],
					[1, 13, 169, 0, 0],
					[1, 13, 170, 0, 0],
					[1, 13, 171, 0, 0],
					[1, 14, 173, 0, 0],
					[1, 14, 174, 0, 0],
					[1, 14, 175, 0, 0],
					[1, 14, 176, 0, 0],
					[1, 14, 177, 0, 0],
					[1, 14, 178, 0, 0],
					[1, 14, 179, 0, 0],
					[1, 14, 180, 0, 0],
					[1, 14, 181, 0, 0],
					[1, 14, 182, 0, 0],
					[1, 14, 183, 0, 0],
					[1, 14, 184, 0, 0],
					[1, 14, 185, 0, 0],
					[1, 14, 186, 0, 0],
					[1, 14, 187, 0, 0],
					[1, 14, 188, 0, 0],
					[1, 14, 189, 0, 0],
					[1, 14, 190, 0, 1],
					[1, 14, 191, 0, 1],
					[1, 14, 192, 0, 0],
					[1, 14, 193, 0, 0],
					[1, 14, 194, 0, 0],
					[1, 14, 196, 0, 0],
					[1, 14, 197, 0, 0],
					[1, 14, 198, 0, 0],
					[1, 14, 199, 0, 0],
					[1, 14, 200, 0, 0],
					[1, 14, 201, 0, 0],
					[1, 14, 202, 0, 0],
					[1, 14, 203, 0, 0],
					[1, 8, 204, 0, 0],
					[1, 8, 205, 0, 1],
					[1, 8, 206, 0, 1],
					[1, 8, 207, 0, 0],
					[1, 8, 208, 0, 0],
					[1, 8, 209, 0, 0],
					[1, 8, 210, 0, 1],
					[1, 8, 211, 0, 1],
					[1, 8, 212, 0, 0],
					[1, 8, 213, 0, 0],
					[1, 8, 214, 0, 0],
					[1, 8, 215, 0, 1],
					[1, 8, 216, 0, 0],
					[1, 8, 217, 0, 0],
					[1, 8, 218, 0, 0],
					[1, 9, 231, 0, 0],
					[1, 9, 232, 0, 0],
					[1, 9, 233, 0, 0],
					[1, 9, 234, 0, 0],
					[1, 9, 235, 0, 0],
					[1, 9, 236, 0, 0],
					[1, 9, 237, 0, 0],
					[1, 9, 238, 0, 0],
					[1, 9, 239, 0, 0],
					[1, 9, 241, 0, 0],
					[1, 9, 242, 0, 0],
					[1, 9, 243, 0, 1],
					[1, 9, 244, 0, 1],
					[1, 9, 245, 0, 0],
					[1, 9, 246, 0, 0],
					[1, 9, 247, 0, 0],
					[1, 9, 248, 0, 0],
					[1, 9, 249, 0, 0],
					[1, 9, 250, 0, 0],
					[1, 9, 251, 0, 0],
					[1, 9, 253, 0, 0],
					[1, 9, 254, 0, 0],
					[1, 16, 255, 0, 0],
					[1, 16, 256, 0, 0],
					[1, 16, 257, 0, 0],
					[1, 16, 258, 0, 0],
					[1, 16, 259, 0, 0],
					[1, 16, 260, 0, 0],
					[1, 16, 262, 0, 0],
					[1, 16, 263, 0, 0],
					[1, 16, 264, 0, 0],
					[1, 16, 265, 0, 0],
					[1, 16, 266, 0, 0],
					[1, 16, 267, 0, 0],
					[1, 16, 268, 0, 0],
					[1, 16, 269, 0, 1],
					[1, 16, 270, 0, 1],
					[1, 16, 271, 0, 0],
					[1, 16, 272, 0, 0],
					[1, 16, 273, 0, 0],
					[1, 16, 274, 0, 0],
					[1, 16, 275, 0, 0],
					[1, 16, 276, 0, 0],
					[1, 16, 277, 0, 0],
					[1, 15, 278, 0, 0],
					[1, 15, 279, 0, 0],
					[1, 15, 280, 0, 0],
					[1, 15, 281, 0, 0],
					[1, 15, 282, 0, 0],
					[1, 15, 283, 0, 0],
					[1, 15, 284, 0, 0],
					[1, 15, 285, 0, 1],
					[1, 15, 286, 0, 1],
					[1, 15, 287, 0, 0],
					[1, 18, 288, 0, 0],
					[1, 18, 289, 0, 0],
					[1, 18, 290, 0, 0],
					[1, 18, 291, 0, 0],
					[1, 18, 292, 0, 0],
					[1, 18, 293, 0, 0],
					[1, 18, 294, 0, 0],
					[1, 18, 295, 0, 1],
					[1, 18, 296, 0, 1],
					[1, 18, 297, 0, 0],
					[1, 18, 304, 0, 0],
					[1, 19, 305, 0, 0],
					[1, 19, 306, 0, 0],
					[1, 19, 307, 0, 0],
					[1, 19, 308, 0, 1],
					[1, 19, 309, 0, 1],
					[1, 19, 310, 0, 0],
					[1, 19, 311, 0, 0],
					[1, 19, 312, 0, 0],
					[1, 29, 474, 0, 0],
					[1, 29, 475, 0, 0],
					[1, 29, 476, 0, 0],
					[1, 29, 477, 0, 0],
					[1, 29, 478, 0, 0],
					[1, 29, 479, 0, 0],
					[1, 29, 480, 0, 0],
					[1, 29, 481, 0, 0],
					[1, 29, 482, 0, 0],
					[1, 29, 483, 0, 0],
					[1, 29, 484, 0, 0],
					[1, 29, 485, 0, 0],
					[1, 29, 487, 0, 0],
					[1, 29, 492, 0, 0],
					[1, 29, 498, 0, 0],
					[1, 29, 500, 0, 0],
					[1, 29, 506, 0, 0],
					[1, 29, 507, 0, 0],
					[1, 29, 508, 0, 0],
					[1, 29, 509, 0, 0],
					[1, 29, 510, 0, 0],
					[1, 29, 511, 0, 0],
					[1, 29, 512, 0, 0],
					[1, 29, 513, 0, 0],
					[1, 29, 514, 0, 0],
					[1, 29, 515, 0, 0],
					[1, 29, 516, 0, 0],
					[1, 29, 517, 0, 0],
					[1, 33, 524, 0, 0],
					[1, 33, 525, 0, 0],
					[1, 33, 526, 0, 0],
					[1, 33, 527, 0, 0],
					[1, 33, 528, 0, 0],
					[1, 33, 529, 0, 0],
					[1, 33, 530, 0, 0],
					[1, 33, 531, 0, 0],
					[1, 33, 532, 0, 0],
					[1, 33, 533, 0, 0],
					[1, 33, 534, 0, 0],
					[1, 33, 535, 0, 0],
					[1, 33, 536, 0, 0],
					[1, 33, 537, 0, 0],
					[1, 33, 538, 0, 1],
					[1, 33, 539, 0, 1],
					[1, 29, 540, 0, 0],
					[1, 34, 541, 0, 0],
					[1, 34, 542, 0, 1],
					[1, 34, 543, 0, 1],
					[1, 34, 544, 0, 0],
					[1, 34, 545, 0, 1],
					[1, 34, 546, 0, 0],
					[1, 34, 547, 0, 0],
					[1, 34, 548, 0, 0],
					[1, 34, 549, 0, 0],
					[1, 34, 550, 0, 0],
					[1, 34, 551, 0, 0],
					[1, 34, 552, 0, 1],
					[1, 34, 553, 0, 1],
					[1, 34, 554, 0, 0],
					[1, 34, 555, 0, 0],
					[1, 34, 556, 0, 0],
					[1, 34, 557, 0, 1],
					[1, 34, 558, 0, 0],
					[1, 34, 559, 0, 0],
					[1, 35, 560, 0, 0],
					[1, 35, 561, 0, 0],
					[1, 35, 562, 0, 0],
					[1, 35, 563, 0, 0],
					[1, 35, 564, 0, 0],
					[1, 35, 565, 0, 0],
					[1, 35, 566, 0, 0],
					[1, 35, 567, 0, 0],
					[1, 35, 568, 0, 1],
					[1, 35, 569, 0, 1],
					[1, 35, 570, 0, 0],
					[1, 35, 571, 0, 0],
					[1, 35, 572, 0, 0],
					[1, 35, 573, 0, 0],
					[1, 35, 574, 0, 0],
					[1, 35, 575, 0, 0],
					[1, 35, 577, 0, 0],
					[1, 35, 578, 0, 0],
					[1, 37, 579, 0, 0],
					[1, 37, 580, 0, 0],
					[1, 37, 581, 0, 0],
					[1, 37, 582, 0, 0],
					[1, 37, 583, 0, 0],
					[1, 37, 584, 0, 0],
					[1, 37, 589, 0, 0],
					[1, 37, 590, 0, 0],
					[1, 37, 593, 0, 1],
					[1, 37, 594, 0, 1],
					[1, 37, 595, 0, 0],
					[1, 40, 597, 0, 0],
					[1, 40, 598, 0, 0],
					[1, 40, 599, 0, 1],
					[1, 40, 600, 0, 1],
					[1, 40, 601, 0, 0],
					[1, 40, 602, 0, 1],
					[1, 40, 603, 0, 0],
					[1, 41, 604, 0, 0],
					[1, 41, 605, 0, 0],
					[1, 41, 606, 0, 0],
					[1, 41, 607, 0, 0],
					[1, 41, 608, 0, 0],
					[1, 41, 609, 0, 0],
					[1, 41, 610, 0, 1],
					[1, 41, 611, 0, 1],
					[1, 41, 612, 0, 0],
					[1, 41, 613, 0, 0],
					[1, 42, 614, 0, 0],
					[1, 42, 615, 0, 0],
					[1, 42, 616, 0, 0],
					[1, 42, 617, 0, 0],
					[1, 42, 618, 0, 0],
					[1, 42, 619, 0, 0],
					[1, 42, 620, 0, 0],
					[1, 42, 621, 0, 0],
					[1, 42, 623, 0, 0],
					[1, 42, 624, 0, 0],
					[1, 42, 625, 0, 1],
					[1, 42, 626, 0, 1],
					[1, 42, 627, 0, 0],
					[1, 42, 628, 0, 0],
					[1, 43, 629, 0, 0],
					[1, 43, 630, 0, 0],
					[1, 43, 631, 0, 0],
					[1, 43, 632, 0, 0],
					[1, 43, 633, 0, 0],
					[1, 43, 634, 0, 0],
					[1, 43, 635, 0, 0],
					[1, 43, 636, 0, 0],
					[1, 43, 637, 0, 0],
					[1, 43, 638, 0, 0],
					[1, 43, 639, 0, 0],
					[1, 43, 640, 0, 0],
					[1, 43, 641, 0, 0],
					[1, 43, 642, 0, 1],
					[1, 43, 643, 0, 1],
					[1, 43, 644, 0, 0],
					[1, 43, 645, 0, 0],
					[1, 45, 646, 0, 0],
					[1, 45, 647, 0, 1],
					[1, 45, 648, 0, 1],
					[1, 45, 649, 0, 0],
					[1, 45, 650, 0, 0],
					[1, 29, 652, 0, 0],
					[1, 29, 689, 0, 0],
					[1, 42, 690, 0, 0],
					[1, 40, 691, 0, 0],
					[1, 29, 692, 0, 0],
					[1, 29, 693, 0, 0],
					[1, 29, 694, 0, 0],
					[1, 29, 699, 0, 0],
					[1, 6, 700, 0, 1],
					[1, 4, 701, 0, 1],
					[1, 29, 703, 0, 0],
					[1, 18, 708, 0, 0],
					[1, 40, 710, 0, 0],
					[1, 40, 711, 0, 0],
					[1, 29, 716, 0, 0],
					[1, 29, 719, 0, 0],
					[1, 13, 720, 0, 0],
					[1, 29, 721, 0, 0],
					[1, 29, 722, 0, 0],
					[1, 4, 725, 0, 1],
					[1, 6, 726, 0, 1],
					[1, 7, 727, 0, 1],
					[1, 9, 729, 0, 1],
					[1, 13, 730, 0, 1],
					[1, 18, 734, 0, 1],
					[1, 33, 741, 0, 1],
					[1, 34, 742, 0, 1],
					[1, 35, 743, 0, 1],
					[1, 37, 744, 0, 1],
					[1, 40, 745, 0, 1],
					[1, 41, 746, 0, 1],
					[1, 42, 747, 0, 1],
					[1, 43, 748, 0, 1],
					[1, 45, 749, 0, 1],
					[1, 51, 780, 0, 0],
					[1, 51, 781, 0, 0],
					[1, 51, 782, 0, 0],
					[1, 51, 783, 0, 0],
					[1, 51, 784, 0, 0],
					[1, 51, 785, 0, 0],
					[1, 51, 786, 0, 0],
					[1, 51, 787, 0, 0],
					[1, 51, 788, 0, 0],
					[1, 51, 791, 0, 1],
					[1, 51, 792, 0, 1],
					[1, 13, 814, 0, 0],
					[1, 42, 816, 0, 0],
					[1, 37, 818, 0, 0],
					[1, 34, 820, 0, 0],
					[1, 43, 826, 0, 0],
					[1, 54, 842, 0, 0],
					[1, 54, 843, 0, 0],
					[1, 54, 844, 0, 0],
					[1, 54, 845, 0, 0],
					[1, 54, 846, 0, 0],
					[1, 54, 847, 0, 0],
					[1, 54, 848, 0, 0],
					[1, 54, 849, 0, 0],
					[1, 54, 850, 0, 0],
					[1, 54, 851, 0, 0],
					[1, 54, 852, 0, 0],
					[1, 54, 853, 0, 0],
					[1, 54, 854, 0, 0],
					[1, 54, 855, 0, 0],
					[1, 54, 856, 0, 0],
					[1, 54, 857, 0, 0],
					[1, 54, 858, 0, 1],
					[1, 54, 859, 0, 1],
					[1, 54, 860, 0, 0],
					[1, 54, 861, 0, 1],
					[1, 54, 862, 0, 1],
					[1, 54, 863, 0, 0],
					[1, 54, 864, 0, 0],
					[1, 8, 865, 0, 0],
					[1, 57, 866, 0, 0],
					[1, 57, 867, 0, 0],
					[1, 57, 868, 0, 0],
					[1, 57, 870, 0, 0],
					[1, 57, 871, 0, 0],
					[1, 57, 872, 0, 0],
					[1, 57, 873, 0, 0],
					[1, 57, 874, 0, 0],
					[1, 57, 875, 0, 1],
					[1, 57, 876, 0, 1],
					[1, 57, 877, 0, 0],
					[1, 57, 878, 0, 0],
					[1, 58, 879, 0, 0],
					[1, 58, 880, 0, 0],
					[1, 58, 881, 0, 0],
					[1, 58, 883, 0, 0],
					[1, 58, 884, 0, 0],
					[1, 58, 885, 0, 0],
					[1, 58, 886, 0, 0],
					[1, 58, 887, 0, 0],
					[1, 58, 888, 0, 1],
					[1, 58, 889, 0, 1],
					[1, 58, 890, 0, 0],
					[1, 58, 891, 0, 0],
					[1, 59, 892, 0, 0],
					[1, 59, 893, 0, 0],
					[1, 59, 894, 0, 0],
					[1, 59, 895, 0, 0],
					[1, 59, 896, 0, 0],
					[1, 59, 898, 0, 0],
					[1, 59, 899, 0, 0],
					[1, 59, 900, 0, 0],
					[1, 59, 901, 0, 0],
					[1, 59, 902, 0, 1],
					[1, 59, 903, 0, 1],
					[1, 59, 904, 0, 0],
					[1, 59, 905, 0, 0],
					[1, 60, 927, 0, 0],
					[1, 60, 928, 0, 0],
					[1, 60, 929, 0, 0],
					[1, 60, 930, 0, 0],
					[1, 60, 931, 0, 0],
					[1, 60, 932, 0, 0],
					[1, 60, 933, 0, 0],
					[1, 60, 934, 0, 1],
					[1, 60, 935, 0, 1],
					[1, 60, 936, 0, 0],
					[1, 7, 938, 0, 0],
					[1, 7, 940, 0, 0],
					[1, 61, 941, 0, 0],
					[1, 61, 942, 0, 0],
					[1, 61, 943, 0, 0],
					[1, 61, 944, 0, 0],
					[1, 61, 945, 0, 0],
					[1, 61, 946, 0, 0],
					[1, 61, 947, 0, 0],
					[1, 61, 948, 0, 0],
					[1, 61, 949, 0, 0],
					[1, 61, 950, 0, 0],
					[1, 61, 951, 0, 1],
					[1, 61, 952, 0, 1],
					[1, 61, 953, 0, 0],
					[1, 61, 954, 0, 0],
					[1, 61, 955, 0, 0],
					[1, 61, 956, 0, 0],
					[1, 61, 957, 0, 0],
					[1, 61, 958, 0, 0],
					[1, 61, 959, 0, 0],
					[1, 61, 960, 0, 0],
					[1, 61, 961, 0, 0],
					[1, 61, 962, 0, 0],
					[1, 61, 963, 0, 0],
					[1, 61, 964, 0, 0],
					[1, 61, 965, 0, 0],
					[1, 61, 966, 0, 0],
					[1, 7, 967, 0, 0],
					[1, 7, 968, 0, 0],
					[1, 7, 969, 0, 0],
					[1, 6, 970, 0, 0],
					[1, 6, 971, 0, 0],
					[1, 6, 972, 0, 0],
					[1, 18, 973, 0, 0],
					[1, 18, 974, 0, 0],
					[1, 18, 975, 0, 0],
					[1, 43, 1044, 0, 0],
					[1, 29, 1045, 0, 0],
					[1, 13, 1049, 0, 0],
					[1, 7, 1065, 0, 0],
					[1, 7, 1066, 0, 0],
					[1, 7, 1067, 0, 0],
					[1, 7, 1068, 0, 0],
					[1, 7, 1069, 0, 0],
					[1, 7, 1070, 0, 0],
					[1, 7, 1071, 0, 0],
					[1, 7, 1072, 0, 0],
					[1, 4, 1073, 0, 0],
					[1, 4, 1074, 0, 0],
					[1, 4, 1075, 0, 0],
					[1, 4, 1076, 0, 0],
					[1, 4, 1077, 0, 0],
					[1, 4, 1078, 0, 0],
					[1, 4, 1079, 0, 0],
					[1, 4, 1080, 0, 0],
					[1, 4, 1081, 0, 0],
					[1, 4, 1082, 0, 0],
					[1, 4, 1083, 0, 0],
					[1, 4, 1084, 0, 0],
					[1, 4, 1085, 0, 0],
					[1, 4, 1086, 0, 0],
					[1, 4, 1087, 0, 0],
					[1, 4, 1088, 0, 0],
					[1, 6, 1089, 0, 0],
					[1, 6, 1090, 0, 0],
					[1, 6, 1091, 0, 0],
					[1, 6, 1092, 0, 0],
					[1, 6, 1093, 0, 0],
					[1, 6, 1094, 0, 0],
					[1, 6, 1095, 0, 0],
					[1, 6, 1096, 0, 0],
					[1, 6, 1097, 0, 0],
					[1, 6, 1098, 0, 0],
					[1, 6, 1099, 0, 0],
					[1, 6, 1100, 0, 0],
					[1, 6, 1101, 0, 0],
					[1, 6, 1102, 0, 0],
					[1, 6, 1103, 0, 0],
					[1, 6, 1104, 0, 0],
					[1, 18, 1105, 0, 0],
					[1, 18, 1106, 0, 0],
					[1, 18, 1107, 0, 0],
					[1, 18, 1108, 0, 0],
					[1, 18, 1109, 0, 0],
					[1, 18, 1110, 0, 0],
					[1, 18, 1111, 0, 0],
					[1, 18, 1112, 0, 0],
					[1, 18, 1113, 0, 0],
					[1, 18, 1114, 0, 0],
					[1, 18, 1115, 0, 0],
					[1, 18, 1116, 0, 0],
					[1, 18, 1117, 0, 0],
					[1, 18, 1118, 0, 0],
					[1, 18, 1119, 0, 0],
					[1, 18, 1120, 0, 0],
					[1, 18, 1121, 0, 0],
					[1, 18, 1122, 0, 0],
					[1, 18, 1123, 0, 0],
					[1, 18, 1124, 0, 0],
					[1, 18, 1125, 0, 0],
					[1, 18, 1126, 0, 0],
					[1, 18, 1127, 0, 0],
					[1, 18, 1128, 0, 0],
					[1, 6, 1160, 0, 0],
					[1, 6, 1161, 0, 0],
					[1, 6, 1162, 0, 0],
					[1, 6, 1163, 0, 0],
					[1, 6, 1164, 0, 0],
					[1, 6, 1165, 0, 0],
					[1, 6, 1166, 0, 0],
					[1, 6, 1167, 0, 0],
					[1, 7, 1276, 0, 0],
					[1, 6, 1277, 0, 0],
					[1, 4, 1278, 0, 0],
					[1, 18, 1279, 0, 0],
					[1, 6, 1285, 0, 0],
					[1, 6, 1286, 0, 0],
					[1, 6, 1287, 0, 0],
					[1, 6, 1288, 0, 0],
					[1, 6, 1289, 0, 0],
					[1, 6, 1290, 0, 0],
					[1, 7, 1291, 0, 0],
					[1, 7, 1292, 0, 0],
					[1, 4, 1293, 0, 0],
					[1, 4, 1294, 0, 0],
					[1, 4, 1295, 0, 0],
					[1, 4, 1296, 0, 0],
					[1, 37, 1314, 0, 0],
					[1, 59, 1315, 0, 0],
					[1, 57, 1316, 0, 0],
					[1, 58, 1317, 0, 0],
					[1, 42, 1318, 0, 0],
					[1, 42, 1319, 0, 0],
					[1, 42, 1320, 0, 0],
					[1, 7, 1321, 0, 0],
					[1, 29, 1322, 0, 0],
					[1, 37, 1323, 0, 0],
					[1, 37, 1324, 0, 0],
					[1, 51, 1326, 0, 0],
					[1, 51, 1327, 0, 0],
					[1, 7, 1329, 0, 0],
					[1, 6, 1330, 0, 0],
					[1, 6, 1331, 0, 0],
					[1, 4, 1332, 0, 0],
					[1, 18, 1333, 0, 0],
					[1, 43, 1340, 0, 0],
					[1, 13, 1341, 0, 0],
					[1, 58, 1344, 0, 0],
					[1, 4, 1368, 0, 0],
					[1, 6, 1369, 0, 0],
					[1, 6, 1375, 0, 0],
					[1, 26, 1378, 0, 0],
					[1, 43, 1382, 0, 0],
					[1, 13, 1384, 0, 0],
					[1, 4, 1386, 0, 0],
					[1, 4, 1391, 0, 0],
					[1, 14, 1393, 0, 0],
					[1, 35, 1395, 0, 0],
					[1, 7, 1397, 0, 0],
					[1, 8, 1399, 0, 0],
					[1, 9, 1401, 0, 0],
					[1, 15, 1403, 0, 0],
					[1, 16, 1405, 0, 0],
					[1, 18, 1407, 0, 0],
					[1, 19, 1409, 0, 0],
					[1, 34, 1419, 0, 0],
					[1, 37, 1421, 0, 0],
					[1, 41, 1423, 0, 0],
					[1, 42, 1425, 0, 0],
					[1, 51, 1431, 0, 0],
					[1, 54, 1433, 0, 0],
					[1, 57, 1435, 0, 0],
					[1, 58, 1437, 0, 0],
					[1, 59, 1439, 0, 0],
					[1, 60, 1441, 0, 0],
					[1, 61, 1443, 0, 0],
					[1, 4, 1449, 0, 0],
					[1, 6, 1450, 0, 0],
					[1, 7, 1451, 0, 0],
					[1, 8, 1452, 0, 0],
					[1, 9, 1453, 0, 0],
					[1, 13, 1454, 0, 0],
					[1, 14, 1455, 0, 0],
					[1, 15, 1456, 0, 0],
					[1, 16, 1457, 0, 0],
					[1, 18, 1458, 0, 0],
					[1, 19, 1459, 0, 0],
					[1, 26, 1464, 0, 0],
					[1, 34, 1465, 0, 0],
					[1, 35, 1466, 0, 0],
					[1, 37, 1467, 0, 0],
					[1, 41, 1468, 0, 0],
					[1, 42, 1469, 0, 0],
					[1, 43, 1470, 0, 0],
					[1, 51, 1473, 0, 0],
					[1, 54, 1474, 0, 0],
					[1, 57, 1475, 0, 0],
					[1, 58, 1476, 0, 0],
					[1, 59, 1477, 0, 0],
					[1, 60, 1478, 0, 0],
					[1, 61, 1479, 0, 0],
					[1, 13, 1482, 0, 0],
					[1, 13, 1483, 0, 0],
					[1, 37, 1484, 0, 0],
					[1, 58, 1485, 0, 0],
					[1, 7, 1486, 0, 0],
					[1, 4, 1487, 0, 0],
					[1, 6, 1488, 0, 0],
					[1, 18, 1489, 0, 0],
					[1, 4, 1494, 0, 0],
					[1, 6, 1495, 0, 0],
					[1, 18, 1496, 0, 0],
					[1, 18, 1501, 0, 0],
					[1, 6, 1502, 0, 0],
					[1, 4, 1503, 0, 0],
					[1, 74, 1504, 0, 0],
					[1, 74, 1505, 0, 0],
					[1, 74, 1506, 0, 0],
					[1, 74, 1507, 0, 0],
					[1, 74, 1508, 0, 0],
					[1, 74, 1509, 0, 0],
					[1, 74, 1511, 0, 0],
					[1, 74, 1512, 0, 0],
					[1, 74, 1513, 0, 0],
					[1, 74, 1514, 0, 0],
					[1, 74, 1515, 0, 0],
					[1, 74, 1516, 0, 0],
					[1, 74, 1517, 0, 0],
					[1, 74, 1518, 0, 0],
					[1, 74, 1519, 0, 0],
					[1, 74, 1520, 0, 0],
					[1, 74, 1521, 0, 0],
					[1, 74, 1522, 0, 0],
					[1, 4, 1524, 0, 0],
					[1, 6, 1525, 0, 0],
					[1, 7, 1526, 0, 0],
					[1, 8, 1527, 0, 0],
					[1, 9, 1528, 0, 0],
					[1, 13, 1529, 0, 0],
					[1, 14, 1530, 0, 0],
					[1, 15, 1531, 0, 0],
					[1, 18, 1532, 0, 0],
					[1, 19, 1533, 0, 0],
					[1, 26, 1538, 0, 0],
					[1, 34, 1539, 0, 0],
					[1, 35, 1540, 0, 0],
					[1, 37, 1541, 0, 0],
					[1, 41, 1542, 0, 0],
					[1, 42, 1543, 0, 0],
					[1, 43, 1544, 0, 0],
					[1, 51, 1547, 0, 0],
					[1, 54, 1548, 0, 0],
					[1, 57, 1549, 0, 0],
					[1, 58, 1550, 0, 0],
					[1, 59, 1551, 0, 0],
					[1, 60, 1552, 0, 0],
					[1, 61, 1553, 0, 0],
					[1, 74, 1556, 0, 0],
					[1, 75, 1557, 0, 0],
					[1, 75, 1558, 0, 0],
					[1, 75, 1559, 0, 0],
					[1, 75, 1560, 0, 0],
					[1, 75, 1561, 0, 0],
					[1, 75, 1562, 0, 0],
					[1, 75, 1563, 0, 0],
					[1, 75, 1564, 0, 0],
					[1, 75, 1566, 0, 0],
					[1, 75, 1567, 0, 0],
					[1, 75, 1568, 0, 0],
					[1, 75, 1569, 0, 0],
					[1, 51, 1600, 0, 0],
					[1, 9, 1603, 0, 0],
					[1, 16, 1604, 0, 0],
					[1, 78, 1605, 0, 0],
					[1, 78, 1606, 0, 0],
					[1, 78, 1607, 0, 0],
					[1, 78, 1608, 0, 0],
					[1, 78, 1609, 0, 0],
					[1, 78, 1610, 0, 0],
					[1, 78, 1611, 0, 0],
					[1, 78, 1612, 0, 0],
					[1, 78, 1613, 0, 0],
					[1, 78, 1615, 0, 0],
					[1, 79, 1616, 0, 0],
					[1, 79, 1617, 0, 0],
					[1, 79, 1618, 0, 0],
					[1, 79, 1619, 0, 0],
					[1, 79, 1620, 0, 0],
					[1, 79, 1621, 0, 0],
					[1, 79, 1622, 0, 0],
					[1, 79, 1625, 0, 0],
					[1, 79, 1626, 0, 0],
					[1, 79, 1627, 0, 0],
					[1, 79, 1628, 0, 0],
					[1, 6, 1630, 0, 0],
					[1, 80, 1632, 0, 0],
					[1, 80, 1633, 0, 0],
					[1, 80, 1634, 0, 0],
					[1, 80, 1635, 0, 0],
					[1, 80, 1636, 0, 0],
					[1, 80, 1637, 0, 0],
					[1, 80, 1638, 0, 0],
					[1, 80, 1640, 0, 0],
					[1, 80, 1641, 0, 0],
					[1, 80, 1642, 0, 0],
					[1, 80, 1643, 0, 0],
					[1, 80, 1644, 0, 0],
					[1, 81, 1645, 0, 0],
					[1, 81, 1646, 0, 0],
					[1, 81, 1647, 0, 0],
					[1, 81, 1648, 0, 0],
					[1, 81, 1649, 0, 0],
					[1, 81, 1650, 0, 0],
					[1, 81, 1651, 0, 0],
					[1, 81, 1652, 0, 0],
					[1, 81, 1653, 0, 0],
					[1, 81, 1655, 0, 0],
					[1, 81, 1656, 0, 0],
					[1, 81, 1657, 0, 0],
					[1, 81, 1658, 0, 0],
					[1, 81, 1659, 0, 0],
					[1, 81, 1660, 0, 0],
					[1, 81, 1661, 0, 0],
					[1, 81, 1666, 0, 0],
					[1, 82, 1667, 0, 0],
					[1, 82, 1668, 0, 0],
					[1, 82, 1669, 0, 0],
					[1, 82, 1670, 0, 0],
					[1, 82, 1671, 0, 0],
					[1, 82, 1672, 0, 0],
					[1, 82, 1673, 0, 0],
					[1, 82, 1674, 0, 0],
					[1, 82, 1675, 0, 0],
					[1, 82, 1677, 0, 0],
					[1, 82, 1678, 0, 0],
					[1, 82, 1679, 0, 0],
					[1, 82, 1680, 0, 0],
					[1, 82, 1681, 0, 0],
					[1, 82, 1682, 0, 0],
					[1, 82, 1683, 0, 0],
					[1, 82, 1688, 0, 0],
					[1, 18, 1689, 0, 0],
					[1, 18, 1690, 0, 0],
					[1, 18, 1691, 0, 0],
					[1, 18, 1692, 0, 0],
					[1, 18, 1693, 0, 0],
					[1, 18, 1694, 0, 0],
					[1, 83, 1708, 0, 0],
					[1, 83, 1709, 0, 0],
					[1, 83, 1710, 0, 0],
					[1, 83, 1711, 0, 0],
					[1, 83, 1712, 0, 0],
					[1, 83, 1713, 0, 0],
					[1, 16, 1714, 0, 0],
					[1, 9, 1715, 0, 0],
					[1, 84, 1716, 0, 0],
					[1, 84, 1717, 0, 0],
					[1, 84, 1718, 0, 0],
					[1, 84, 1719, 0, 0],
					[1, 84, 1720, 0, 0],
					[1, 84, 1721, 0, 0],
					[1, 84, 1722, 0, 0],
					[1, 84, 1723, 0, 0],
					[1, 84, 1724, 0, 0],
					[1, 84, 1725, 0, 0],
					[1, 84, 1726, 0, 0],
					[1, 84, 1727, 0, 0],
					[1, 84, 1728, 0, 0],
					[1, 84, 1729, 0, 0],
					[1, 84, 1730, 0, 0],
					[1, 84, 1731, 0, 0],
					[1, 84, 1732, 0, 0],
					[1, 84, 1733, 0, 0],
					[1, 84, 1735, 0, 0],
					[1, 7, 1736, 0, 0],
					[1, 7, 1737, 0, 0],
					[1, 6, 1738, 0, 0],
					[1, 29, 1740, 0, 0],
					[1, 41, 1741, 0, 0],
					[1, 42, 1742, 0, 0],
					[1, 41, 1743, 0, 0],
					[1, 4, 1744, 0, 0],
					[1, 54, 1745, 0, 0],
					[1, 4, 1746, 0, 0],
					[1, 6, 1747, 0, 0],
					[1, 6, 1756, 0, 0],
					[1, 6, 1757, 0, 0],
					[1, 6, 1758, 0, 0],
					[1, 14, 1759, 0, 0],
					[1, 16, 1761, 0, 0],
					[1, 9, 1762, 0, 0],
					[1, 15, 1763, 0, 0],
					[1, 19, 1764, 0, 0],
					[1, 85, 1765, 0, 0],
					[1, 85, 1766, 0, 0],
					[1, 85, 1767, 0, 0],
					[1, 85, 1768, 0, 0],
					[1, 85, 1769, 0, 0],
					[1, 85, 1770, 0, 0],
					[1, 85, 1771, 0, 0],
					[1, 85, 1772, 0, 0],
					[1, 85, 1773, 0, 0],
					[1, 85, 1774, 0, 0],
					[1, 85, 1775, 0, 0],
					[1, 85, 1776, 0, 0],
					[1, 85, 1777, 0, 0],
					[1, 85, 1778, 0, 0],
					[1, 86, 1779, 0, 0],
					[1, 86, 1780, 0, 0],
					[1, 86, 1781, 0, 0],
					[1, 86, 1782, 0, 0],
					[1, 86, 1783, 0, 0],
					[1, 86, 1784, 0, 0],
					[1, 86, 1785, 0, 0],
					[1, 86, 1786, 0, 0],
					[1, 86, 1787, 0, 0],
					[1, 86, 1788, 0, 0],
					[1, 87, 1789, 0, 0],
					[1, 87, 1790, 0, 0],
					[1, 87, 1791, 0, 0],
					[1, 87, 1792, 0, 0],
					[1, 87, 1793, 0, 0],
					[1, 87, 1794, 0, 0],
					[1, 87, 1795, 0, 0],
					[1, 87, 1796, 0, 0],
					[1, 87, 1797, 0, 0],
					[1, 87, 1798, 0, 0],
					[1, 87, 1799, 0, 0],
					[1, 87, 1800, 0, 0],
					[1, 87, 1801, 0, 0],
					[1, 87, 1802, 0, 0],
					[1, 87, 1803, 0, 0],
					[1, 88, 1804, 0, 0],
					[1, 88, 1805, 0, 0],
					[1, 88, 1806, 0, 0],
					[1, 88, 1807, 0, 0],
					[1, 88, 1808, 0, 0],
					[1, 88, 1809, 0, 0],
					[1, 88, 1810, 0, 0],
					[1, 88, 1811, 0, 0],
					[1, 88, 1812, 0, 0],
					[1, 88, 1813, 0, 0],
					[1, 88, 1814, 0, 0],
					[1, 88, 1815, 0, 0],
					[1, 88, 1816, 0, 0],
					[1, 88, 1817, 0, 0],
					[1, 88, 1818, 0, 0],
					[1, 89, 1819, 0, 0],
					[1, 89, 1820, 0, 0],
					[1, 89, 1821, 0, 0],
					[1, 89, 1822, 0, 0],
					[1, 89, 1823, 0, 0],
					[1, 89, 1824, 0, 0],
					[1, 89, 1825, 0, 0],
					[1, 89, 1826, 0, 0],
					[1, 89, 1827, 0, 0],
					[1, 89, 1828, 0, 0],
					[1, 89, 1829, 0, 0],
					[1, 89, 1830, 0, 0],
					[1, 89, 1831, 0, 0],
					[1, 89, 1832, 0, 0],
					[1, 89, 1833, 0, 0],
					[1, 90, 1834, 0, 0],
					[1, 90, 1835, 0, 0],
					[1, 90, 1836, 0, 0],
					[1, 90, 1837, 0, 0],
					[1, 90, 1838, 0, 0],
					[1, 90, 1839, 0, 0],
					[1, 90, 1840, 0, 0],
					[1, 90, 1841, 0, 0],
					[1, 90, 1842, 0, 0],
					[1, 90, 1843, 0, 0],
					[1, 90, 1844, 0, 0],
					[1, 90, 1845, 0, 0],
					[1, 90, 1846, 0, 0],
					[1, 90, 1847, 0, 0],
					[1, 90, 1848, 0, 0],
					[1, 90, 1849, 0, 0],
					[1, 90, 1850, 0, 0],
					[1, 90, 1851, 0, 0],
					[1, 91, 1852, 0, 0],
					[1, 91, 1853, 0, 0],
					[1, 91, 1854, 0, 0],
					[1, 91, 1855, 0, 0],
					[1, 91, 1856, 0, 0],
					[1, 91, 1857, 0, 0],
					[1, 91, 1858, 0, 0],
					[1, 91, 1859, 0, 0],
					[1, 91, 1860, 0, 0],
					[1, 91, 1861, 0, 0],
					[1, 91, 1862, 0, 0],
					[1, 91, 1863, 0, 0],
					[1, 91, 1864, 0, 0],
					[1, 91, 1865, 0, 0],
					[1, 91, 1866, 0, 0],
					[1, 91, 1867, 0, 0],
					[1, 91, 1868, 0, 0],
					[1, 91, 1869, 0, 0],
					[1, 91, 1870, 0, 0],
					[1, 91, 1871, 0, 0],
					[1, 91, 1872, 0, 0],
					[1, 91, 1873, 0, 0],
					[1, 91, 1874, 0, 0],
					[1, 91, 1875, 0, 0],
					[1, 89, 1876, 0, 0],
					[1, 89, 1877, 0, 0],
					[1, 89, 1878, 0, 0],
					[1, 89, 1879, 0, 0],
					[1, 89, 1880, 0, 0],
					[1, 89, 1881, 0, 0],
					[1, 89, 1882, 0, 0],
					[1, 89, 1883, 0, 0],
					[1, 89, 1884, 0, 0],
					[1, 89, 1885, 0, 0],
					[1, 89, 1886, 0, 0],
					[1, 89, 1887, 0, 0],
					[1, 89, 1888, 0, 0],
					[1, 89, 1889, 0, 0],
					[1, 89, 1890, 0, 0],
					[1, 89, 1891, 0, 0],
					[1, 89, 1892, 0, 0],
					[1, 89, 1893, 0, 0],
					[1, 89, 1894, 0, 0],
					[1, 89, 1895, 0, 0],
					[1, 89, 1896, 0, 0],
					[1, 89, 1897, 0, 0],
					[1, 90, 1898, 0, 0],
					[1, 90, 1899, 0, 0],
					[1, 90, 1900, 0, 0],
					[1, 90, 1901, 0, 0],
					[1, 90, 1902, 0, 0],
					[1, 90, 1903, 0, 0],
					[1, 90, 1904, 0, 0],
					[1, 90, 1905, 0, 0],
					[1, 90, 1906, 0, 0],
					[1, 90, 1907, 0, 0],
					[1, 90, 1908, 0, 0],
					[1, 90, 1909, 0, 0],
					[1, 90, 1910, 0, 0],
					[1, 90, 1911, 0, 0],
					[1, 90, 1912, 0, 0],
					[1, 90, 1913, 0, 0],
					[1, 90, 1914, 0, 0],
					[1, 90, 1915, 0, 0],
					[1, 90, 1916, 0, 0],
					[1, 90, 1917, 0, 0],
					[1, 90, 1918, 0, 0],
					[1, 90, 1919, 0, 0],
					[1, 91, 1920, 0, 0],
					[1, 91, 1921, 0, 0],
					[1, 91, 1922, 0, 0],
					[1, 91, 1923, 0, 0],
					[1, 91, 1924, 0, 0],
					[1, 91, 1925, 0, 0],
					[1, 91, 1926, 0, 0],
					[1, 91, 1927, 0, 0],
					[1, 91, 1928, 0, 0],
					[1, 91, 1929, 0, 0],
					[1, 91, 1930, 0, 0],
					[1, 91, 1931, 0, 0],
					[1, 91, 1932, 0, 0],
					[1, 91, 1933, 0, 0],
					[1, 91, 1934, 0, 0],
					[1, 91, 1935, 0, 0],
					[1, 91, 1936, 0, 0],
					[1, 91, 1937, 0, 0],
					[1, 91, 1938, 0, 0],
					[1, 91, 1939, 0, 0],
					[1, 91, 1940, 0, 0],
					[1, 91, 1941, 0, 0],
					[1, 89, 1942, 0, 0],
					[1, 90, 1943, 0, 0],
					[1, 91, 1944, 0, 0],
					[1, 6, 1945, 0, 1],
					[1, 6, 1946, 0, 1],
					[1, 37, 1947, 0, 0],
					[1, 92, 1948, 0, 0],
					[1, 92, 1949, 0, 0],
					[1, 92, 1950, 0, 0],
					[1, 92, 1951, 0, 0],
					[1, 92, 1952, 0, 0],
					[1, 92, 1953, 0, 0],
					[1, 92, 1954, 0, 0],
					[1, 92, 1955, 0, 0],
					[1, 92, 1956, 0, 0],
					[1, 92, 1957, 0, 0],
					[1, 92, 1958, 0, 0],
					[1, 92, 1959, 0, 0],
					[1, 92, 1960, 0, 0],
					[1, 92, 1961, 0, 0],
					[1, 92, 1962, 0, 0],
					[1, 92, 1963, 0, 0],
					[1, 92, 1964, 0, 0],
					[1, 92, 1965, 0, 0],
					[1, 92, 1966, 0, 0],
					[1, 92, 1967, 0, 0],
					[1, 93, 1968, 0, 0],
					[1, 93, 1969, 0, 0],
					[1, 93, 1970, 0, 0],
					[1, 93, 1971, 0, 0],
					[1, 93, 1972, 0, 0],
					[1, 93, 1973, 0, 0],
					[1, 93, 1974, 0, 0],
					[1, 93, 1975, 0, 0],
					[1, 93, 1976, 0, 0],
					[1, 93, 1977, 0, 0],
					[1, 93, 1978, 0, 0],
					[1, 93, 1979, 0, 0],
					[1, 93, 1980, 0, 0],
					[1, 93, 1981, 0, 0],
					[1, 93, 1982, 0, 0],
					[1, 93, 1983, 0, 0],
					[1, 93, 1984, 0, 0],
					[1, 93, 1985, 0, 0],
					[1, 93, 1986, 0, 0],
					[1, 93, 1987, 0, 0],
					[1, 59, 1988, 0, 0],
					[1, 37, 1989, 0, 0],
					[1, 57, 1990, 0, 0],
					[1, 58, 1991, 0, 0],
					[1, 9, 1992, 0, 0],
					[1, 16, 1993, 0, 0],
					[1, 51, 1994, 0, 0],
					[1, 51, 1995, 0, 0],
					[1, 51, 1996, 0, 0],
					[1, 4, 1997, 0, 0],
					[1, 7, 1998, 0, 0],
					[1, 18, 1999, 0, 0],
					[1, 92, 2000, 0, 0],
					[1, 93, 2001, 0, 0],
					[1, 61, 2002, 0, 0],
					[1, 86, 2003, 0, 0],
					[1, 26, 2004, 0, 0],
					[1, 41, 2005, 0, 0],
					[1, 85, 2006, 0, 0],
					[1, 87, 2007, 0, 0],
					[1, 88, 2008, 0, 0],
					[1, 89, 2009, 0, 0],
					[1, 90, 2010, 0, 0],
					[1, 91, 2011, 0, 0],
					[1, 92, 2012, 0, 0],
					[1, 93, 2013, 0, 0],
					[1, 4, 2014, 0, 0],
					[1, 6, 2015, 0, 0],
					[1, 7, 2016, 0, 0],
					[1, 13, 2017, 0, 0],
					[1, 18, 2018, 0, 0],
					[1, 26, 2019, 0, 0],
					[1, 34, 2020, 0, 0],
					[1, 43, 2021, 0, 0],
					[1, 61, 2022, 0, 0],
					[1, 85, 2023, 0, 0],
					[1, 86, 2024, 0, 0],
					[1, 87, 2025, 0, 0],
					[1, 88, 2026, 0, 0],
					[1, 89, 2027, 0, 0],
					[1, 90, 2028, 0, 0],
					[1, 91, 2029, 0, 0],
					[1, 92, 2030, 0, 0],
					[1, 93, 2031, 0, 0],
					[1, 94, 2032, 0, 0],
					[1, 94, 2033, 0, 0],
					[1, 94, 2034, 0, 0],
					[1, 94, 2035, 0, 0],
					[1, 94, 2036, 0, 0],
					[1, 94, 2037, 0, 0],
					[1, 94, 2038, 0, 0],
					[1, 94, 2039, 0, 0],
					[1, 95, 2040, 0, 0],
					[1, 95, 2041, 0, 0],
					[1, 95, 2042, 0, 0],
					[1, 95, 2043, 0, 0],
					[1, 95, 2044, 0, 0],
					[1, 95, 2046, 0, 0],
					[1, 95, 2047, 0, 0],
					[1, 95, 2048, 0, 0],
					[1, 95, 2049, 0, 0],
					[1, 95, 2050, 0, 0],
					[1, 95, 2051, 0, 0],
					[1, 95, 2052, 0, 0],
					[1, 95, 2053, 0, 0],
					[1, 95, 2054, 0, 0],
					[1, 95, 2055, 0, 0],
					[1, 95, 2056, 0, 0],
					[1, 95, 2057, 0, 0],
					[1, 95, 2058, 0, 0],
					[1, 95, 2059, 0, 0],
					[1, 95, 2060, 0, 0],
					[1, 95, 2061, 0, 0],
					[1, 95, 2062, 0, 0],
					[1, 95, 2063, 0, 0],
					[1, 95, 2064, 0, 0],
					[1, 95, 2065, 0, 0],
					[1, 95, 2066, 0, 0],
					[1, 95, 2067, 0, 0],
					[1, 95, 2068, 0, 0],
					[1, 95, 2069, 0, 0],
					[1, 95, 2070, 0, 0],
					[1, 95, 2071, 0, 0],
					[1, 95, 2072, 0, 0],
					[1, 95, 2073, 0, 0],
					[1, 95, 2074, 0, 0],
					[1, 96, 2075, 0, 0],
					[1, 96, 2076, 0, 0],
					[1, 96, 2077, 0, 0],
					[1, 96, 2078, 0, 0],
					[1, 96, 2079, 0, 0],
					[1, 96, 2080, 0, 0],
					[1, 96, 2081, 0, 0],
					[1, 96, 2082, 0, 0],
					[1, 97, 2083, 0, 0],
					[1, 97, 2084, 0, 0],
					[1, 97, 2085, 0, 0],
					[1, 97, 2086, 0, 0],
					[1, 97, 2087, 0, 0],
					[1, 97, 2088, 0, 0],
					[1, 97, 2089, 0, 0],
					[1, 97, 2090, 0, 0],
					[1, 97, 2091, 0, 0],
					[1, 97, 2092, 0, 0],
					[1, 97, 2093, 0, 0],
					[1, 86, 2094, 0, 0],
					[1, 86, 2095, 0, 0],
					[1, 86, 2096, 0, 0],
					[1, 86, 2097, 0, 0],
					[1, 86, 2098, 0, 0],
					[1, 86, 2099, 0, 0],
					[1, 86, 2100, 0, 0],
					[1, 86, 2101, 0, 0],
					[1, 86, 2102, 0, 0],
					[1, 88, 2103, 0, 0],
					[1, 88, 2104, 0, 0],
					[1, 88, 2105, 0, 0],
					[1, 89, 2106, 0, 0],
					[1, 89, 2107, 0, 0],
					[1, 89, 2108, 0, 0],
					[1, 89, 2109, 0, 0],
					[1, 89, 2110, 0, 0],
					[1, 90, 2111, 0, 0],
					[1, 90, 2112, 0, 0],
					[1, 90, 2113, 0, 0],
					[1, 90, 2114, 0, 0],
					[1, 90, 2115, 0, 0],
					[1, 98, 2116, 0, 0],
					[1, 98, 2117, 0, 0],
					[1, 98, 2118, 0, 0],
					[1, 98, 2119, 0, 0],
					[1, 98, 2120, 0, 0],
					[1, 98, 2121, 0, 0],
					[1, 99, 2122, 0, 0],
					[1, 99, 2123, 0, 0],
					[1, 99, 2124, 0, 0],
					[1, 99, 2125, 0, 0],
					[1, 99, 2126, 0, 0],
					[1, 99, 2127, 0, 0],
					[1, 99, 2128, 0, 0],
					[1, 99, 2129, 0, 0],
					[1, 99, 2130, 0, 0],
					[1, 99, 2131, 0, 0],
					[1, 99, 2132, 0, 0],
					[1, 99, 2133, 0, 0],
					[1, 99, 2134, 0, 0],
					[1, 99, 2135, 0, 0],
					[1, 99, 2136, 0, 0],
					[1, 99, 2137, 0, 0],
					[1, 99, 2138, 0, 0],
					[1, 99, 2139, 0, 0],
					[1, 99, 2140, 0, 0],
					[1, 99, 2141, 0, 0],
					[1, 99, 2142, 0, 0],
					[1, 99, 2143, 0, 0],
					[1, 99, 2144, 0, 0],
					[1, 99, 2145, 0, 0],
					[1, 99, 2146, 0, 0],
					[1, 99, 2147, 0, 0],
					[1, 99, 2148, 0, 0],
					[1, 99, 2149, 0, 0],
					[1, 99, 2150, 0, 0],
					[1, 99, 2151, 0, 0],
					[1, 99, 2152, 0, 0],
					[1, 99, 2153, 0, 0],
					[1, 99, 2154, 0, 0],
					[1, 99, 2155, 0, 0],
					[1, 100, 2156, 0, 0],
					[1, 100, 2157, 0, 0],
					[1, 100, 2158, 0, 0],
					[1, 100, 2159, 0, 0],
					[1, 100, 2160, 0, 0],
					[1, 100, 2161, 0, 0],
					[1, 100, 2162, 0, 0],
					[1, 101, 2163, 0, 0],
					[1, 101, 2164, 0, 0],
					[1, 101, 2165, 0, 0],
					[1, 101, 2166, 0, 0],
					[1, 101, 2167, 0, 0],
					[1, 101, 2168, 0, 0],
					[1, 101, 2169, 0, 0],
					[1, 98, 2170, 0, 0],
					[1, 102, 2171, 0, 0],
					[1, 102, 2172, 0, 0],
					[1, 102, 2173, 0, 0],
					[1, 102, 2174, 0, 0],
					[1, 102, 2175, 0, 0],
					[1, 102, 2176, 0, 0],
					[1, 102, 2177, 0, 0],
					[1, 98, 2178, 0, 0],
					[1, 100, 2179, 0, 0],
					[1, 101, 2180, 0, 0],
					[1, 102, 2181, 0, 0],
					[1, 103, 2182, 0, 0],
					[1, 103, 2183, 0, 0],
					[1, 103, 2184, 0, 0],
					[1, 103, 2185, 0, 0],
					[1, 103, 2186, 0, 0],
					[1, 103, 2187, 0, 0],
					[1, 103, 2188, 0, 0],
					[1, 98, 2189, 0, 0],
					[1, 98, 2190, 0, 0],
					[1, 98, 2191, 0, 0],
					[1, 98, 2192, 0, 0],
					[1, 100, 2193, 0, 0],
					[1, 100, 2194, 0, 0],
					[1, 100, 2195, 0, 0],
					[1, 100, 2196, 0, 0],
					[1, 101, 2197, 0, 0],
					[1, 101, 2198, 0, 0],
					[1, 101, 2199, 0, 0],
					[1, 102, 2200, 0, 0],
					[1, 102, 2201, 0, 0],
					[1, 102, 2202, 0, 0],
					[1, 103, 2203, 0, 0],
					[1, 103, 2204, 0, 0],
					[1, 103, 2205, 0, 0],
					[1, 103, 2206, 0, 0],
					[1, 103, 2207, 0, 0],
					[1, 14, 2208, 0, 0],
					[1, 98, 2209, 0, 0],
					[1, 96, 2210, 0, 0],
					[1, 97, 2211, 0, 0],
					[1, 97, 2212, 0, 0],
					[1, 97, 2213, 0, 0],
					[1, 97, 2214, 0, 0],
					[1, 97, 2215, 0, 0],
					[1, 97, 2216, 0, 0],
					[1, 97, 2217, 0, 0],
					[1, 97, 2218, 0, 0],
					[1, 97, 2219, 0, 0],
					[1, 97, 2220, 0, 0],
					[1, 97, 2221, 0, 0],
					[1, 104, 2222, 0, 0],
					[1, 104, 2223, 0, 0],
					[1, 104, 2224, 0, 0],
					[1, 104, 2225, 0, 0],
					[1, 104, 2226, 0, 0],
					[1, 104, 2227, 0, 0],
					[1, 104, 2228, 0, 0],
					[1, 104, 2229, 0, 0],
					[1, 104, 2230, 0, 0],
					[1, 104, 2231, 0, 0],
					[1, 104, 2232, 0, 0],
					[1, 104, 2233, 0, 0],
					[1, 104, 2234, 0, 0],
					[1, 105, 2235, 0, 0],
					[1, 105, 2236, 0, 0],
					[1, 105, 2237, 0, 0],
					[1, 105, 2238, 0, 0],
					[1, 105, 2239, 0, 0],
					[1, 105, 2240, 0, 0],
					[1, 105, 2241, 0, 0],
					[1, 105, 2242, 0, 0],
					[1, 105, 2243, 0, 0],
					[1, 105, 2244, 0, 0],
					[1, 105, 2245, 0, 0],
					[1, 106, 2246, 0, 0],
					[1, 106, 2247, 0, 0],
					[1, 106, 2248, 0, 0],
					[1, 106, 2249, 0, 0],
					[1, 106, 2250, 0, 0],
					[1, 106, 2251, 0, 0],
					[1, 106, 2252, 0, 0],
					[1, 106, 2253, 0, 0],
					[1, 106, 2254, 0, 0],
					[1, 106, 2255, 0, 0],
					[1, 106, 2256, 0, 0],
					[1, 106, 2257, 0, 0],
					[1, 106, 2258, 0, 0],
					[1, 104, 2259, 0, 0],
					[1, 106, 2260, 0, 0],
					[1, 104, 2261, 0, 0],
					[1, 106, 2262, 0, 0],
					[1, 96, 2263, 0, 0],
					[1, 96, 2264, 0, 0],
					[1, 96, 2265, 0, 0],
					[1, 96, 2266, 0, 0],
					[1, 58, 2267, 0, 0],
					[1, 37, 2268, 0, 0],
					[1, 14, 2269, 0, 0],
					[1, 107, 2270, 0, 0],
					[1, 107, 2271, 0, 0],
					[1, 107, 2272, 0, 0],
					[1, 107, 2273, 0, 0],
					[1, 107, 2274, 0, 0],
					[1, 107, 2275, 0, 0],
					[1, 107, 2276, 0, 0],
					[1, 107, 2277, 0, 0],
					[1, 107, 2278, 0, 0],
					[1, 107, 2279, 0, 0],
					[1, 107, 2280, 0, 0],
					[1, 107, 2281, 0, 0],
					[1, 107, 2282, 0, 0],
					[1, 107, 2283, 0, 0],
					[1, 107, 2284, 0, 0],
					[1, 107, 2285, 0, 0],
					[1, 107, 2286, 0, 0],
					[1, 107, 2287, 0, 0],
					[1, 107, 2288, 0, 0],
					[1, 107, 2289, 0, 0],
					[1, 107, 2290, 0, 0],
					[1, 107, 2291, 0, 0],
					[1, 107, 2292, 0, 0],
					[1, 107, 2293, 0, 0],
					[1, 107, 2294, 0, 0],
					[1, 107, 2295, 0, 0],
					[1, 107, 2296, 0, 0],
					[1, 107, 2297, 0, 0],
					[1, 107, 2298, 0, 0],
					[1, 107, 2299, 0, 0],
					[1, 107, 2300, 0, 0],
					[1, 107, 2301, 0, 0],
					[1, 107, 2302, 0, 0],
					[1, 107, 2303, 0, 0],
					[1, 107, 2304, 0, 0],
					[1, 108, 2305, 0, 0],
					[1, 108, 2306, 0, 0],
					[1, 108, 2307, 0, 0],
					[1, 108, 2308, 0, 0],
					[1, 108, 2309, 0, 0],
					[1, 108, 2310, 0, 0],
					[1, 108, 2311, 0, 0],
					[1, 108, 2312, 0, 0],
					[1, 108, 2313, 0, 0],
					[1, 108, 2314, 0, 0],
					[1, 108, 2315, 0, 0],
					[1, 108, 2316, 0, 0],
					[1, 108, 2317, 0, 0],
					[1, 108, 2318, 0, 0],
					[1, 109, 2319, 0, 0],
					[1, 109, 2320, 0, 0],
					[1, 109, 2321, 0, 0],
					[1, 109, 2322, 0, 0],
					[1, 109, 2323, 0, 0],
					[1, 109, 2324, 0, 0],
					[1, 109, 2325, 0, 0],
					[1, 109, 2326, 0, 0],
					[1, 109, 2327, 0, 0],
					[1, 109, 2328, 0, 0],
					[1, 109, 2329, 0, 0],
					[1, 109, 2330, 0, 0],
					[1, 109, 2331, 0, 0],
					[1, 13, 2332, 0, 0],
					[1, 13, 2333, 0, 0],
					[1, 14, 2334, 0, 0],
					[1, 35, 2335, 0, 0],
					[1, 37, 2336, 0, 0],
					[1, 58, 2337, 0, 0],
					[1, 90, 2344, 0, 0],
					[1, 100, 2345, 0, 0],
					[1, 6, 2346, 0, 0],
					[1, 13, 2347, 0, 0],
					[1, 13, 2348, 0, 0],
					[1, 4, 2349, 0, 0],
					[1, 6, 2350, 0, 0],
					[1, 7, 2351, 0, 0],
					[1, 92, 2352, 0, 0],
					[1, 18, 2353, 0, 0],
					[1, 93, 2354, 0, 0],
					[1, 83, 2355, 0, 0],
					[1, 83, 2356, 0, 0],
					[1, 14, 2357, 0, 0],
					[1, 89, 2358, 0, 0],
					[1, 92, 2381, 0, 0],
					[1, 61, 2382, 0, 0],
					[1, 61, 2383, 0, 0],
					[1, 61, 2384, 0, 0],
					[1, 86, 2385, 0, 0],
					[1, 43, 2386, 0, 0],
					[1, 111, 2387, 0, 0],
					[1, 111, 2388, 0, 0],
					[1, 111, 2389, 0, 0],
					[1, 111, 2390, 0, 0],
					[1, 111, 2391, 0, 0],
					[1, 111, 2392, 0, 0],
					[1, 111, 2393, 0, 0],
					[1, 111, 2395, 0, 0],
					[1, 111, 2396, 0, 0],
					[1, 111, 2397, 0, 0],
					[1, 111, 2398, 0, 0],
					[1, 111, 2399, 0, 0],
					[1, 111, 2400, 0, 0],
					[1, 6, 2401, 0, 0],
					[1, 95, 2402, 0, 0],
					[1, 95, 2403, 0, 0],
					[1, 86, 2404, 0, 0],
					[1, 6, 2405, 0, 0],
					[1, 4, 2406, 0, 0],
					[1, 7, 2407, 0, 0],
					[1, 18, 2408, 0, 0],
					[1, 14, 2409, 0, 0],
					[1, 86, 2410, 0, 0],
					[1, 97, 2411, 0, 0],
					[1, 26, 2412, 0, 0],
					[1, 16, 2413, 0, 0],
					[1, 13, 2414, 0, 0],
					[1, 105, 2415, 0, 0],
					[1, 35, 2416, 0, 0],
					[1, 106, 2417, 0, 0],
					[1, 98, 2418, 0, 0],
					[1, 54, 2419, 0, 0],
					[1, 108, 2420, 0, 0],
					[1, 100, 2421, 0, 0],
					[1, 109, 2422, 0, 0],
					[1, 9, 2423, 0, 0],
					[1, 43, 2424, 0, 0],
					[1, 101, 2425, 0, 0],
					[1, 102, 2426, 0, 0],
					[1, 37, 2427, 0, 0],
					[1, 61, 2428, 0, 0],
					[1, 103, 2429, 0, 0],
					[1, 15, 2430, 0, 0],
					[1, 74, 2431, 0, 0],
					[1, 104, 2432, 0, 0],
					[1, 34, 2433, 0, 0],
					[1, 59, 2434, 0, 0],
					[1, 51, 2435, 0, 0],
					[1, 19, 2436, 0, 0],
					[1, 41, 2437, 0, 0],
					[1, 8, 2438, 0, 0],
					[1, 58, 2439, 0, 0],
					[1, 84, 2440, 0, 0],
					[1, 60, 2441, 0, 0],
					[1, 57, 2442, 0, 0],
					[1, 83, 2443, 0, 0],
					[1, 42, 2444, 0, 0],
					[1, 75, 2445, 0, 0],
					[1, 78, 2446, 0, 0],
					[1, 79, 2447, 0, 0],
					[1, 80, 2448, 0, 0],
					[1, 81, 2449, 0, 0],
					[1, 107, 2450, 0, 0],
					[1, 82, 2451, 0, 0],
					[1, 99, 2452, 0, 0],
					[1, 94, 2453, 0, 0],
					[1, 95, 2454, 0, 0],
					[1, 96, 2455, 0, 0],
					[1, 111, 2456, 0, 0],
					[1, 85, 2458, 0, 0],
					[1, 87, 2459, 0, 0],
					[1, 89, 2460, 0, 0],
					[1, 91, 2461, 0, 0],
					[1, 93, 2462, 0, 0],
					[1, 88, 2463, 0, 0],
					[1, 90, 2464, 0, 0],
					[1, 92, 2465, 0, 0],
					[1, 33, 2466, 0, 0],
					[1, 35, 2467, 0, 0],
					[1, 29, 2468, 0, 0],
					[1, 29, 2469, 0, 0],
					[1, 29, 2470, 0, 0],
					[1, 112, 2471, 0, 0],
					[1, 112, 2472, 0, 0],
					[1, 112, 2473, 0, 0],
					[1, 112, 2474, 0, 0],
					[1, 112, 2475, 0, 0],
					[1, 112, 2476, 0, 0],
					[1, 112, 2477, 0, 0],
					[1, 112, 2478, 0, 0],
					[1, 112, 2479, 0, 0],
					[1, 112, 2480, 0, 0],
					[1, 112, 2481, 0, 0],
					[1, 112, 2482, 0, 0],
					[1, 112, 2483, 0, 0],
					[1, 86, 2484, 0, 0],
					[1, 113, 2485, 0, 0],
					[1, 113, 2486, 0, 0],
					[1, 113, 2487, 0, 0],
					[1, 113, 2488, 0, 0],
					[1, 113, 2489, 0, 0],
					[1, 113, 2490, 0, 0],
					[1, 113, 2491, 0, 0],
					[1, 113, 2492, 0, 0],
					[1, 113, 2493, 0, 0],
					[1, 113, 2494, 0, 0],
					[1, 113, 2495, 0, 0],
					[1, 113, 2496, 0, 0],
					[1, 113, 2497, 0, 0],
					[1, 113, 2498, 0, 0],
					[1, 113, 2499, 0, 0],
					[1, 113, 2500, 0, 0],
					[1, 113, 2501, 0, 0],
					[1, 113, 2502, 0, 0],
					[1, 113, 2503, 0, 0],
					[1, 113, 2504, 0, 0],
					[1, 113, 2505, 0, 0],
					[1, 113, 2506, 0, 0],
					[1, 113, 2507, 0, 0],
					[1, 113, 2508, 0, 0],
					[1, 114, 2509, 0, 0],
					[1, 114, 2510, 0, 0],
					[1, 114, 2511, 0, 0],
					[1, 114, 2512, 0, 0],
					[1, 114, 2513, 0, 0],
					[1, 114, 2514, 0, 0],
					[1, 114, 2515, 0, 0],
					[1, 114, 2516, 0, 0],
					[1, 114, 2517, 0, 0],
					[1, 114, 2518, 0, 0],
					[1, 114, 2519, 0, 0],
					[1, 115, 2520, 0, 0],
					[1, 115, 2521, 0, 0],
					[1, 115, 2522, 0, 0],
					[1, 115, 2523, 0, 0],
					[1, 115, 2524, 0, 0],
					[1, 115, 2525, 0, 0],
					[1, 115, 2526, 0, 0],
					[1, 115, 2527, 0, 0],
					[1, 115, 2528, 0, 0],
					[1, 115, 2529, 0, 0],
					[1, 115, 2530, 0, 0],
					[1, 115, 2531, 0, 0],
					[1, 115, 2532, 0, 0],
					[1, 115, 2533, 0, 0],
					[1, 115, 2534, 0, 0],
					[1, 115, 2535, 0, 0],
					[1, 115, 2536, 0, 0],
					[1, 115, 2537, 0, 0],
					[1, 115, 2538, 0, 0],
					[1, 115, 2539, 0, 0],
					[1, 115, 2540, 0, 0],
					[1, 115, 2541, 0, 0],
					[1, 115, 2542, 0, 0],
					[1, 115, 2543, 0, 0],
					[1, 115, 2544, 0, 0],
					[1, 115, 2545, 0, 0],
					[1, 115, 2546, 0, 0],
					[1, 115, 2547, 0, 0],
					[1, 115, 2548, 0, 0],
					[1, 115, 2549, 0, 0],
					[1, 115, 2550, 0, 0],
					[1, 115, 2551, 0, 0],
					[1, 115, 2552, 0, 0],
					[1, 115, 2553, 0, 0],
					[1, 115, 2554, 0, 0],
					[1, 115, 2555, 0, 0],
					[1, 115, 2556, 0, 0],
					[1, 115, 2557, 0, 0],
					[1, 115, 2558, 0, 0],
					[1, 116, 2559, 0, 0],
					[1, 116, 2560, 0, 0],
					[1, 116, 2561, 0, 0],
					[1, 116, 2562, 0, 0],
					[1, 116, 2563, 0, 0],
					[1, 116, 2564, 0, 0],
					[1, 116, 2565, 0, 0],
					[1, 116, 2566, 0, 0],
					[1, 116, 2567, 0, 0],
					[1, 116, 2568, 0, 0],
					[1, 116, 2569, 0, 0],
					[1, 95, 2570, 0, 0],
					[1, 95, 2571, 0, 0],
					[1, 95, 2572, 0, 0],
					[1, 95, 2573, 0, 0],
					[1, 113, 2574, 0, 0],
					[1, 113, 2575, 0, 0],
					[1, 113, 2576, 0, 0],
					[1, 117, 2577, 0, 0],
					[1, 117, 2578, 0, 0],
					[1, 117, 2579, 0, 0],
					[1, 117, 2580, 0, 0],
					[1, 117, 2581, 0, 0],
					[1, 117, 2582, 0, 0],
					[1, 117, 2583, 0, 0],
					[1, 117, 2584, 0, 0],
					[1, 117, 2585, 0, 0],
					[1, 117, 2586, 0, 0],
					[1, 117, 2587, 0, 0],
					[1, 117, 2588, 0, 0],
					[1, 117, 2589, 0, 0],
					[1, 117, 2590, 0, 0],
					[1, 117, 2591, 0, 0],
					[1, 117, 2592, 0, 0],
					[1, 117, 2593, 0, 0],
					[1, 117, 2594, 0, 0],
					[1, 117, 2595, 0, 0],
					[1, 117, 2596, 0, 0],
					[1, 117, 2597, 0, 0],
					[1, 117, 2598, 0, 0],
					[1, 16, 2600, 0, 0],
					[1, 16, 2601, 0, 0],
					[1, 9, 2602, 0, 0],
					[1, 9, 2603, 0, 0],
					[1, 43, 2604, 0, 0],
					[1, 83, 2605, 0, 0],
					[1, 45, 2606, 0, 0],
					[2, 6, 1, 0, 0],
					[2, 6, 2, 0, 0],
					[2, 6, 3, 0, 0],
					[2, 6, 4, 0, 0],
					[2, 6, 5, 0, 0],
					[2, 6, 7, 0, 0],
					[2, 6, 8, 0, 0],
					[2, 6, 9, 0, 0],
					[2, 6, 10, 0, 0],
					[2, 6, 11, 0, 0],
					[2, 6, 12, 0, 0],
					[2, 6, 14, 0, 0],
					[2, 6, 15, 0, 0],
					[2, 6, 16, 0, 0],
					[2, 6, 17, 0, 0],
					[2, 6, 18, 0, 0],
					[2, 6, 20, 0, 0],
					[2, 6, 21, 0, 1],
					[2, 6, 22, 0, 1],
					[2, 6, 23, 0, 0],
					[2, 6, 36, 0, 0],
					[2, 7, 39, 0, 0],
					[2, 7, 40, 0, 0],
					[2, 7, 41, 0, 0],
					[2, 7, 42, 0, 0],
					[2, 7, 43, 0, 0],
					[2, 7, 44, 0, 0],
					[2, 7, 46, 0, 0],
					[2, 7, 47, 0, 0],
					[2, 7, 48, 0, 0],
					[2, 7, 49, 0, 0],
					[2, 7, 50, 0, 0],
					[2, 7, 51, 0, 0],
					[2, 7, 53, 0, 0],
					[2, 7, 54, 0, 0],
					[2, 7, 55, 0, 0],
					[2, 7, 56, 0, 1],
					[2, 7, 57, 0, 1],
					[2, 7, 58, 0, 0],
					[2, 7, 65, 0, 0],
					[2, 4, 66, 0, 0],
					[2, 4, 67, 0, 0],
					[2, 4, 68, 0, 0],
					[2, 4, 69, 0, 0],
					[2, 4, 70, 0, 0],
					[2, 4, 71, 0, 0],
					[2, 4, 72, 0, 0],
					[2, 4, 74, 0, 0],
					[2, 4, 79, 0, 0],
					[2, 4, 80, 0, 0],
					[2, 4, 81, 0, 0],
					[2, 4, 85, 0, 0],
					[2, 4, 86, 0, 0],
					[2, 4, 87, 0, 0],
					[2, 4, 90, 0, 1],
					[2, 4, 91, 0, 1],
					[2, 4, 92, 0, 0],
					[2, 4, 93, 0, 0],
					[2, 4, 94, 0, 0],
					[2, 4, 95, 0, 0],
					[2, 4, 108, 0, 0],
					[2, 4, 109, 0, 0],
					[2, 26, 126, 0, 0],
					[2, 26, 127, 0, 0],
					[2, 26, 128, 0, 0],
					[2, 26, 129, 0, 0],
					[2, 26, 130, 0, 0],
					[2, 26, 131, 0, 0],
					[2, 26, 132, 0, 0],
					[2, 26, 133, 0, 0],
					[2, 26, 134, 0, 0],
					[2, 26, 135, 0, 0],
					[2, 26, 136, 0, 0],
					[2, 26, 137, 0, 1],
					[2, 26, 138, 0, 1],
					[2, 26, 139, 0, 0],
					[2, 26, 140, 0, 0],
					[2, 26, 141, 0, 0],
					[2, 26, 142, 0, 0],
					[2, 26, 143, 0, 0],
					[2, 26, 144, 0, 0],
					[2, 26, 145, 0, 0],
					[2, 26, 146, 0, 0],
					[2, 26, 147, 0, 0],
					[2, 26, 148, 0, 0],
					[2, 26, 149, 0, 0],
					[2, 26, 150, 0, 0],
					[2, 13, 155, 0, 0],
					[2, 13, 156, 0, 0],
					[2, 13, 157, 0, 0],
					[2, 13, 158, 0, 0],
					[2, 13, 159, 0, 0],
					[2, 13, 160, 0, 0],
					[2, 13, 161, 0, 0],
					[2, 13, 162, 0, 0],
					[2, 13, 163, 0, 0],
					[2, 13, 166, 0, 1],
					[2, 13, 167, 0, 1],
					[2, 13, 168, 0, 0],
					[2, 13, 169, 0, 0],
					[2, 13, 170, 0, 0],
					[2, 13, 171, 0, 0],
					[2, 14, 173, 0, 0],
					[2, 14, 174, 0, 0],
					[2, 14, 175, 0, 0],
					[2, 14, 176, 0, 0],
					[2, 14, 177, 0, 0],
					[2, 14, 178, 0, 0],
					[2, 14, 179, 0, 0],
					[2, 14, 180, 0, 0],
					[2, 14, 181, 0, 0],
					[2, 14, 182, 0, 0],
					[2, 14, 183, 0, 0],
					[2, 14, 184, 0, 0],
					[2, 14, 185, 0, 0],
					[2, 14, 186, 0, 0],
					[2, 14, 187, 0, 0],
					[2, 14, 188, 0, 0],
					[2, 14, 189, 0, 0],
					[2, 14, 190, 0, 1],
					[2, 14, 191, 0, 1],
					[2, 14, 192, 0, 0],
					[2, 14, 193, 0, 0],
					[2, 14, 194, 0, 0],
					[2, 14, 196, 0, 0],
					[2, 14, 197, 0, 0],
					[2, 14, 198, 0, 0],
					[2, 14, 199, 0, 0],
					[2, 14, 200, 0, 0],
					[2, 14, 201, 0, 0],
					[2, 14, 202, 0, 0],
					[2, 14, 203, 0, 0],
					[2, 8, 204, 0, 0],
					[2, 8, 205, 0, 1],
					[2, 8, 206, 0, 1],
					[2, 8, 207, 0, 0],
					[2, 8, 208, 0, 0],
					[2, 8, 209, 0, 0],
					[2, 8, 210, 0, 1],
					[2, 8, 211, 0, 1],
					[2, 8, 212, 0, 0],
					[2, 8, 213, 0, 0],
					[2, 8, 214, 0, 0],
					[2, 8, 215, 0, 1],
					[2, 8, 216, 0, 0],
					[2, 8, 217, 0, 0],
					[2, 8, 218, 0, 0],
					[2, 9, 231, 0, 0],
					[2, 9, 232, 0, 0],
					[2, 9, 233, 0, 0],
					[2, 9, 234, 0, 0],
					[2, 9, 235, 0, 0],
					[2, 9, 236, 0, 0],
					[2, 9, 237, 0, 0],
					[2, 9, 238, 0, 0],
					[2, 9, 239, 0, 0],
					[2, 9, 241, 0, 0],
					[2, 9, 242, 0, 0],
					[2, 9, 243, 0, 1],
					[2, 9, 244, 0, 1],
					[2, 9, 245, 0, 0],
					[2, 9, 246, 0, 0],
					[2, 9, 247, 0, 0],
					[2, 9, 248, 0, 0],
					[2, 9, 249, 0, 0],
					[2, 9, 250, 0, 0],
					[2, 9, 251, 0, 0],
					[2, 9, 253, 0, 0],
					[2, 9, 254, 0, 0],
					[2, 16, 255, 0, 0],
					[2, 16, 256, 0, 0],
					[2, 16, 257, 0, 0],
					[2, 16, 258, 0, 0],
					[2, 16, 259, 0, 0],
					[2, 16, 260, 0, 0],
					[2, 16, 262, 0, 0],
					[2, 16, 263, 0, 0],
					[2, 16, 264, 0, 0],
					[2, 16, 265, 0, 0],
					[2, 16, 266, 0, 0],
					[2, 16, 267, 0, 0],
					[2, 16, 268, 0, 0],
					[2, 16, 269, 0, 1],
					[2, 16, 270, 0, 1],
					[2, 16, 271, 0, 0],
					[2, 16, 272, 0, 0],
					[2, 16, 273, 0, 0],
					[2, 16, 274, 0, 0],
					[2, 16, 275, 0, 0],
					[2, 16, 276, 0, 0],
					[2, 16, 277, 0, 0],
					[2, 15, 278, 0, 0],
					[2, 15, 279, 0, 0],
					[2, 15, 280, 0, 0],
					[2, 15, 281, 0, 0],
					[2, 15, 282, 0, 0],
					[2, 15, 283, 0, 0],
					[2, 15, 284, 0, 0],
					[2, 15, 285, 0, 1],
					[2, 15, 286, 0, 1],
					[2, 15, 287, 0, 0],
					[2, 18, 288, 0, 0],
					[2, 18, 289, 0, 0],
					[2, 18, 290, 0, 0],
					[2, 18, 291, 0, 0],
					[2, 18, 292, 0, 0],
					[2, 18, 293, 0, 0],
					[2, 18, 294, 0, 0],
					[2, 18, 295, 0, 1],
					[2, 18, 296, 0, 1],
					[2, 18, 297, 0, 0],
					[2, 18, 304, 0, 0],
					[2, 19, 305, 0, 0],
					[2, 19, 306, 0, 0],
					[2, 19, 307, 0, 0],
					[2, 19, 308, 0, 1],
					[2, 19, 309, 0, 1],
					[2, 19, 310, 0, 0],
					[2, 19, 311, 0, 0],
					[2, 19, 312, 0, 0],
					[2, 29, 474, 0, 0],
					[2, 29, 475, 0, 0],
					[2, 29, 476, 0, 0],
					[2, 29, 477, 0, 0],
					[2, 29, 478, 0, 0],
					[2, 29, 479, 0, 0],
					[2, 29, 480, 0, 0],
					[2, 29, 481, 0, 0],
					[2, 29, 482, 0, 0],
					[2, 29, 483, 0, 0],
					[2, 29, 484, 0, 0],
					[2, 29, 485, 0, 0],
					[2, 29, 487, 0, 0],
					[2, 29, 492, 0, 0],
					[2, 29, 498, 0, 0],
					[2, 29, 500, 0, 0],
					[2, 29, 506, 0, 0],
					[2, 29, 507, 0, 0],
					[2, 29, 508, 0, 0],
					[2, 29, 509, 0, 0],
					[2, 29, 510, 0, 0],
					[2, 29, 511, 0, 0],
					[2, 29, 512, 0, 0],
					[2, 29, 513, 0, 0],
					[2, 29, 514, 0, 0],
					[2, 29, 515, 0, 0],
					[2, 29, 516, 0, 0],
					[2, 29, 517, 0, 0],
					[2, 33, 524, 0, 0],
					[2, 33, 525, 0, 0],
					[2, 33, 526, 0, 0],
					[2, 33, 527, 0, 0],
					[2, 33, 528, 0, 0],
					[2, 33, 529, 0, 0],
					[2, 33, 530, 0, 0],
					[2, 33, 531, 0, 0],
					[2, 33, 532, 0, 0],
					[2, 33, 533, 0, 0],
					[2, 33, 534, 0, 0],
					[2, 33, 535, 0, 0],
					[2, 33, 536, 0, 0],
					[2, 33, 537, 0, 0],
					[2, 33, 538, 0, 1],
					[2, 33, 539, 0, 1],
					[2, 29, 540, 0, 0],
					[2, 34, 541, 0, 0],
					[2, 34, 542, 0, 1],
					[2, 34, 543, 0, 1],
					[2, 34, 544, 0, 0],
					[2, 34, 545, 0, 1],
					[2, 34, 546, 0, 0],
					[2, 34, 547, 0, 0],
					[2, 34, 548, 0, 0],
					[2, 34, 549, 0, 0],
					[2, 34, 550, 0, 0],
					[2, 34, 551, 0, 0],
					[2, 34, 552, 0, 1],
					[2, 34, 553, 0, 1],
					[2, 34, 554, 0, 0],
					[2, 34, 555, 0, 0],
					[2, 34, 556, 0, 0],
					[2, 34, 557, 0, 1],
					[2, 34, 558, 0, 0],
					[2, 34, 559, 0, 0],
					[2, 35, 560, 0, 0],
					[2, 35, 561, 0, 0],
					[2, 35, 562, 0, 0],
					[2, 35, 563, 0, 0],
					[2, 35, 564, 0, 0],
					[2, 35, 565, 0, 0],
					[2, 35, 566, 0, 0],
					[2, 35, 567, 0, 0],
					[2, 35, 568, 0, 1],
					[2, 35, 569, 0, 1],
					[2, 35, 570, 0, 0],
					[2, 35, 571, 0, 0],
					[2, 35, 572, 0, 0],
					[2, 35, 573, 0, 0],
					[2, 35, 574, 0, 0],
					[2, 35, 575, 0, 0],
					[2, 35, 577, 0, 0],
					[2, 35, 578, 0, 0],
					[2, 37, 579, 0, 0],
					[2, 37, 580, 0, 0],
					[2, 37, 581, 0, 0],
					[2, 37, 582, 0, 0],
					[2, 37, 583, 0, 0],
					[2, 37, 584, 0, 0],
					[2, 37, 589, 0, 0],
					[2, 37, 590, 0, 0],
					[2, 37, 593, 0, 1],
					[2, 37, 594, 0, 1],
					[2, 37, 595, 0, 0],
					[2, 40, 597, 0, 0],
					[2, 40, 598, 0, 0],
					[2, 40, 599, 0, 1],
					[2, 40, 600, 0, 1],
					[2, 40, 601, 0, 0],
					[2, 40, 602, 0, 1],
					[2, 40, 603, 0, 0],
					[2, 41, 604, 0, 0],
					[2, 41, 605, 0, 0],
					[2, 41, 606, 0, 0],
					[2, 41, 607, 0, 0],
					[2, 41, 608, 0, 0],
					[2, 41, 609, 0, 0],
					[2, 41, 610, 0, 1],
					[2, 41, 611, 0, 1],
					[2, 41, 612, 0, 0],
					[2, 41, 613, 0, 0],
					[2, 42, 614, 0, 0],
					[2, 42, 615, 0, 0],
					[2, 42, 616, 0, 0],
					[2, 42, 617, 0, 0],
					[2, 42, 618, 0, 0],
					[2, 42, 619, 0, 0],
					[2, 42, 620, 0, 0],
					[2, 42, 621, 0, 0],
					[2, 42, 623, 0, 0],
					[2, 42, 624, 0, 0],
					[2, 42, 625, 0, 1],
					[2, 42, 626, 0, 1],
					[2, 42, 627, 0, 0],
					[2, 42, 628, 0, 0],
					[2, 43, 629, 0, 0],
					[2, 43, 630, 0, 0],
					[2, 43, 631, 0, 0],
					[2, 43, 632, 0, 0],
					[2, 43, 633, 0, 0],
					[2, 43, 634, 0, 0],
					[2, 43, 635, 0, 0],
					[2, 43, 636, 0, 0],
					[2, 43, 637, 0, 0],
					[2, 43, 638, 0, 0],
					[2, 43, 639, 0, 0],
					[2, 43, 640, 0, 0],
					[2, 43, 641, 0, 0],
					[2, 43, 642, 0, 1],
					[2, 43, 643, 0, 1],
					[2, 43, 644, 0, 0],
					[2, 43, 645, 0, 0],
					[2, 45, 646, 0, 0],
					[2, 45, 647, 0, 1],
					[2, 45, 648, 0, 1],
					[2, 45, 649, 0, 0],
					[2, 45, 650, 0, 0],
					[2, 29, 652, 0, 0],
					[2, 29, 689, 0, 0],
					[2, 42, 690, 0, 0],
					[2, 40, 691, 0, 0],
					[2, 29, 692, 0, 0],
					[2, 29, 693, 0, 0],
					[2, 29, 694, 0, 0],
					[2, 29, 699, 0, 0],
					[2, 6, 700, 0, 1],
					[2, 4, 701, 0, 1],
					[2, 29, 703, 0, 0],
					[2, 18, 708, 0, 0],
					[2, 40, 710, 0, 0],
					[2, 40, 711, 0, 0],
					[2, 29, 716, 0, 0],
					[2, 29, 719, 0, 0],
					[2, 13, 720, 0, 0],
					[2, 29, 721, 0, 0],
					[2, 29, 722, 0, 0],
					[2, 4, 725, 0, 1],
					[2, 6, 726, 0, 1],
					[2, 7, 727, 0, 1],
					[2, 9, 729, 0, 1],
					[2, 13, 730, 0, 1],
					[2, 18, 734, 0, 1],
					[2, 33, 741, 0, 1],
					[2, 34, 742, 0, 1],
					[2, 35, 743, 0, 1],
					[2, 37, 744, 0, 1],
					[2, 40, 745, 0, 1],
					[2, 41, 746, 0, 1],
					[2, 42, 747, 0, 1],
					[2, 43, 748, 0, 1],
					[2, 45, 749, 0, 1],
					[2, 51, 780, 0, 0],
					[2, 51, 781, 0, 0],
					[2, 51, 782, 0, 0],
					[2, 51, 783, 0, 0],
					[2, 51, 784, 0, 0],
					[2, 51, 785, 0, 0],
					[2, 51, 786, 0, 0],
					[2, 51, 787, 0, 0],
					[2, 51, 788, 0, 0],
					[2, 51, 791, 0, 1],
					[2, 51, 792, 0, 1],
					[2, 13, 814, 0, 0],
					[2, 42, 816, 0, 0],
					[2, 37, 818, 0, 0],
					[2, 34, 820, 0, 0],
					[2, 43, 826, 0, 0],
					[2, 54, 842, 0, 0],
					[2, 54, 843, 0, 0],
					[2, 54, 844, 0, 0],
					[2, 54, 845, 0, 0],
					[2, 54, 846, 0, 0],
					[2, 54, 847, 0, 0],
					[2, 54, 848, 0, 0],
					[2, 54, 849, 0, 0],
					[2, 54, 850, 0, 0],
					[2, 54, 851, 0, 0],
					[2, 54, 852, 0, 0],
					[2, 54, 853, 0, 0],
					[2, 54, 854, 0, 0],
					[2, 54, 855, 0, 0],
					[2, 54, 856, 0, 0],
					[2, 54, 857, 0, 0],
					[2, 54, 858, 0, 1],
					[2, 54, 859, 0, 1],
					[2, 54, 860, 0, 0],
					[2, 54, 861, 0, 1],
					[2, 54, 862, 0, 1],
					[2, 54, 863, 0, 0],
					[2, 54, 864, 0, 0],
					[2, 8, 865, 0, 0],
					[2, 57, 866, 0, 0],
					[2, 57, 867, 0, 0],
					[2, 57, 868, 0, 0],
					[2, 57, 870, 0, 0],
					[2, 57, 871, 0, 0],
					[2, 57, 872, 0, 0],
					[2, 57, 873, 0, 0],
					[2, 57, 874, 0, 0],
					[2, 57, 875, 0, 1],
					[2, 57, 876, 0, 1],
					[2, 57, 877, 0, 0],
					[2, 57, 878, 0, 0],
					[2, 58, 879, 0, 0],
					[2, 58, 880, 0, 0],
					[2, 58, 881, 0, 0],
					[2, 58, 883, 0, 0],
					[2, 58, 884, 0, 0],
					[2, 58, 885, 0, 0],
					[2, 58, 886, 0, 0],
					[2, 58, 887, 0, 0],
					[2, 58, 888, 0, 1],
					[2, 58, 889, 0, 1],
					[2, 58, 890, 0, 0],
					[2, 58, 891, 0, 0],
					[2, 59, 892, 0, 0],
					[2, 59, 893, 0, 0],
					[2, 59, 894, 0, 0],
					[2, 59, 895, 0, 0],
					[2, 59, 896, 0, 0],
					[2, 59, 898, 0, 0],
					[2, 59, 899, 0, 0],
					[2, 59, 900, 0, 0],
					[2, 59, 901, 0, 0],
					[2, 59, 902, 0, 1],
					[2, 59, 903, 0, 1],
					[2, 59, 904, 0, 0],
					[2, 59, 905, 0, 0],
					[2, 60, 927, 0, 0],
					[2, 60, 928, 0, 0],
					[2, 60, 929, 0, 0],
					[2, 60, 930, 0, 0],
					[2, 60, 931, 0, 0],
					[2, 60, 932, 0, 0],
					[2, 60, 933, 0, 0],
					[2, 60, 934, 0, 1],
					[2, 60, 935, 0, 1],
					[2, 60, 936, 0, 0],
					[2, 7, 938, 0, 0],
					[2, 7, 940, 0, 0],
					[2, 61, 941, 0, 0],
					[2, 61, 942, 0, 0],
					[2, 61, 943, 0, 0],
					[2, 61, 944, 0, 0],
					[2, 61, 945, 0, 0],
					[2, 61, 946, 0, 0],
					[2, 61, 947, 0, 0],
					[2, 61, 948, 0, 0],
					[2, 61, 949, 0, 0],
					[2, 61, 950, 0, 0],
					[2, 61, 951, 0, 1],
					[2, 61, 952, 0, 1],
					[2, 61, 953, 0, 0],
					[2, 61, 954, 0, 0],
					[2, 61, 955, 0, 0],
					[2, 61, 956, 0, 0],
					[2, 61, 957, 0, 0],
					[2, 61, 958, 0, 0],
					[2, 61, 959, 0, 0],
					[2, 61, 960, 0, 0],
					[2, 61, 961, 0, 0],
					[2, 61, 962, 0, 0],
					[2, 61, 963, 0, 0],
					[2, 61, 964, 0, 0],
					[2, 61, 965, 0, 0],
					[2, 61, 966, 0, 0],
					[2, 7, 967, 0, 0],
					[2, 7, 968, 0, 0],
					[2, 7, 969, 0, 0],
					[2, 6, 970, 0, 0],
					[2, 6, 971, 0, 0],
					[2, 6, 972, 0, 0],
					[2, 18, 973, 0, 0],
					[2, 18, 974, 0, 0],
					[2, 18, 975, 0, 0],
					[2, 43, 1044, 0, 0],
					[2, 29, 1045, 0, 0],
					[2, 13, 1049, 0, 0],
					[2, 7, 1065, 0, 0],
					[2, 7, 1066, 0, 0],
					[2, 7, 1067, 0, 0],
					[2, 7, 1068, 0, 0],
					[2, 7, 1069, 0, 0],
					[2, 7, 1070, 0, 0],
					[2, 7, 1071, 0, 0],
					[2, 7, 1072, 0, 0],
					[2, 4, 1073, 0, 0],
					[2, 4, 1074, 0, 0],
					[2, 4, 1075, 0, 0],
					[2, 4, 1076, 0, 0],
					[2, 4, 1077, 0, 0],
					[2, 4, 1078, 0, 0],
					[2, 4, 1079, 0, 0],
					[2, 4, 1080, 0, 0],
					[2, 4, 1081, 0, 0],
					[2, 4, 1082, 0, 0],
					[2, 4, 1083, 0, 0],
					[2, 4, 1084, 0, 0],
					[2, 4, 1085, 0, 0],
					[2, 4, 1086, 0, 0],
					[2, 4, 1087, 0, 0],
					[2, 4, 1088, 0, 0],
					[2, 6, 1089, 0, 0],
					[2, 6, 1090, 0, 0],
					[2, 6, 1091, 0, 0],
					[2, 6, 1092, 0, 0],
					[2, 6, 1093, 0, 0],
					[2, 6, 1094, 0, 0],
					[2, 6, 1095, 0, 0],
					[2, 6, 1096, 0, 0],
					[2, 6, 1097, 0, 0],
					[2, 6, 1098, 0, 0],
					[2, 6, 1099, 0, 0],
					[2, 6, 1100, 0, 0],
					[2, 6, 1101, 0, 0],
					[2, 6, 1102, 0, 0],
					[2, 6, 1103, 0, 0],
					[2, 6, 1104, 0, 0],
					[2, 18, 1105, 0, 0],
					[2, 18, 1106, 0, 0],
					[2, 18, 1107, 0, 0],
					[2, 18, 1108, 0, 0],
					[2, 18, 1109, 0, 0],
					[2, 18, 1110, 0, 0],
					[2, 18, 1111, 0, 0],
					[2, 18, 1112, 0, 0],
					[2, 18, 1113, 0, 0],
					[2, 18, 1114, 0, 0],
					[2, 18, 1115, 0, 0],
					[2, 18, 1116, 0, 0],
					[2, 18, 1117, 0, 0],
					[2, 18, 1118, 0, 0],
					[2, 18, 1119, 0, 0],
					[2, 18, 1120, 0, 0],
					[2, 18, 1121, 0, 0],
					[2, 18, 1122, 0, 0],
					[2, 18, 1123, 0, 0],
					[2, 18, 1124, 0, 0],
					[2, 18, 1125, 0, 0],
					[2, 18, 1126, 0, 0],
					[2, 18, 1127, 0, 0],
					[2, 18, 1128, 0, 0],
					[2, 6, 1160, 0, 0],
					[2, 6, 1161, 0, 0],
					[2, 6, 1162, 0, 0],
					[2, 6, 1163, 0, 0],
					[2, 6, 1164, 0, 0],
					[2, 6, 1165, 0, 0],
					[2, 6, 1166, 0, 0],
					[2, 6, 1167, 0, 0],
					[2, 7, 1276, 0, 0],
					[2, 6, 1277, 0, 0],
					[2, 4, 1278, 0, 0],
					[2, 18, 1279, 0, 0],
					[2, 6, 1285, 0, 0],
					[2, 6, 1286, 0, 0],
					[2, 6, 1287, 0, 0],
					[2, 6, 1288, 0, 0],
					[2, 6, 1289, 0, 0],
					[2, 6, 1290, 0, 0],
					[2, 7, 1291, 0, 0],
					[2, 7, 1292, 0, 0],
					[2, 4, 1293, 0, 0],
					[2, 4, 1294, 0, 0],
					[2, 4, 1295, 0, 0],
					[2, 4, 1296, 0, 0],
					[2, 37, 1314, 0, 0],
					[2, 59, 1315, 0, 0],
					[2, 57, 1316, 0, 0],
					[2, 58, 1317, 0, 0],
					[2, 42, 1318, 0, 0],
					[2, 42, 1319, 0, 0],
					[2, 42, 1320, 0, 0],
					[2, 7, 1321, 0, 0],
					[2, 29, 1322, 0, 0],
					[2, 37, 1323, 0, 0],
					[2, 37, 1324, 0, 0],
					[2, 51, 1326, 0, 0],
					[2, 51, 1327, 0, 0],
					[2, 7, 1329, 0, 0],
					[2, 6, 1330, 0, 0],
					[2, 6, 1331, 0, 0],
					[2, 4, 1332, 0, 0],
					[2, 18, 1333, 0, 0],
					[2, 43, 1340, 0, 0],
					[2, 13, 1341, 0, 0],
					[2, 58, 1344, 0, 0],
					[2, 4, 1368, 0, 0],
					[2, 6, 1369, 0, 0],
					[2, 6, 1375, 0, 0],
					[2, 26, 1378, 0, 0],
					[2, 43, 1382, 0, 0],
					[2, 13, 1384, 0, 0],
					[2, 4, 1386, 0, 0],
					[2, 4, 1391, 0, 0],
					[2, 14, 1393, 0, 0],
					[2, 35, 1395, 0, 0],
					[2, 7, 1397, 0, 0],
					[2, 8, 1399, 0, 0],
					[2, 9, 1401, 0, 0],
					[2, 15, 1403, 0, 0],
					[2, 16, 1405, 0, 0],
					[2, 18, 1407, 0, 0],
					[2, 19, 1409, 0, 0],
					[2, 34, 1419, 0, 0],
					[2, 37, 1421, 0, 0],
					[2, 41, 1423, 0, 0],
					[2, 42, 1425, 0, 0],
					[2, 51, 1431, 0, 0],
					[2, 54, 1433, 0, 0],
					[2, 57, 1435, 0, 0],
					[2, 58, 1437, 0, 0],
					[2, 59, 1439, 0, 0],
					[2, 60, 1441, 0, 0],
					[2, 61, 1443, 0, 0],
					[2, 4, 1449, 0, 0],
					[2, 6, 1450, 0, 0],
					[2, 7, 1451, 0, 0],
					[2, 8, 1452, 0, 0],
					[2, 9, 1453, 0, 0],
					[2, 13, 1454, 0, 0],
					[2, 14, 1455, 0, 0],
					[2, 15, 1456, 0, 0],
					[2, 16, 1457, 0, 0],
					[2, 18, 1458, 0, 0],
					[2, 19, 1459, 0, 0],
					[2, 26, 1464, 0, 0],
					[2, 34, 1465, 0, 0],
					[2, 35, 1466, 0, 0],
					[2, 37, 1467, 0, 0],
					[2, 41, 1468, 0, 0],
					[2, 42, 1469, 0, 0],
					[2, 43, 1470, 0, 0],
					[2, 51, 1473, 0, 0],
					[2, 54, 1474, 0, 0],
					[2, 57, 1475, 0, 0],
					[2, 58, 1476, 0, 0],
					[2, 59, 1477, 0, 0],
					[2, 60, 1478, 0, 0],
					[2, 61, 1479, 0, 0],
					[2, 13, 1482, 0, 0],
					[2, 13, 1483, 0, 0],
					[2, 37, 1484, 0, 0],
					[2, 58, 1485, 0, 0],
					[2, 7, 1486, 0, 0],
					[2, 4, 1487, 0, 0],
					[2, 6, 1488, 0, 0],
					[2, 18, 1489, 0, 0],
					[2, 4, 1494, 0, 0],
					[2, 6, 1495, 0, 0],
					[2, 18, 1496, 0, 0],
					[2, 18, 1501, 0, 0],
					[2, 6, 1502, 0, 0],
					[2, 4, 1503, 0, 0],
					[2, 74, 1504, 0, 0],
					[2, 74, 1505, 0, 0],
					[2, 74, 1506, 0, 0],
					[2, 74, 1507, 0, 0],
					[2, 74, 1508, 0, 0],
					[2, 74, 1509, 0, 0],
					[2, 74, 1511, 0, 0],
					[2, 74, 1512, 0, 0],
					[2, 74, 1513, 0, 0],
					[2, 74, 1514, 0, 0],
					[2, 74, 1515, 0, 0],
					[2, 74, 1516, 0, 0],
					[2, 74, 1517, 0, 0],
					[2, 74, 1518, 0, 0],
					[2, 74, 1519, 0, 0],
					[2, 74, 1520, 0, 0],
					[2, 74, 1521, 0, 0],
					[2, 74, 1522, 0, 0],
					[2, 4, 1524, 0, 0],
					[2, 6, 1525, 0, 0],
					[2, 7, 1526, 0, 0],
					[2, 8, 1527, 0, 0],
					[2, 9, 1528, 0, 0],
					[2, 13, 1529, 0, 0],
					[2, 14, 1530, 0, 0],
					[2, 15, 1531, 0, 0],
					[2, 18, 1532, 0, 0],
					[2, 19, 1533, 0, 0],
					[2, 26, 1538, 0, 0],
					[2, 34, 1539, 0, 0],
					[2, 35, 1540, 0, 0],
					[2, 37, 1541, 0, 0],
					[2, 41, 1542, 0, 0],
					[2, 42, 1543, 0, 0],
					[2, 43, 1544, 0, 0],
					[2, 51, 1547, 0, 0],
					[2, 54, 1548, 0, 0],
					[2, 57, 1549, 0, 0],
					[2, 58, 1550, 0, 0],
					[2, 59, 1551, 0, 0],
					[2, 60, 1552, 0, 0],
					[2, 61, 1553, 0, 0],
					[2, 74, 1556, 0, 0],
					[2, 75, 1557, 0, 0],
					[2, 75, 1558, 0, 0],
					[2, 75, 1559, 0, 0],
					[2, 75, 1560, 0, 0],
					[2, 75, 1561, 0, 0],
					[2, 75, 1562, 0, 0],
					[2, 75, 1563, 0, 0],
					[2, 75, 1564, 0, 0],
					[2, 75, 1566, 0, 0],
					[2, 75, 1567, 0, 0],
					[2, 75, 1568, 0, 0],
					[2, 75, 1569, 0, 0],
					[2, 51, 1600, 0, 0],
					[2, 9, 1603, 0, 0],
					[2, 16, 1604, 0, 0],
					[2, 78, 1605, 0, 0],
					[2, 78, 1606, 0, 0],
					[2, 78, 1607, 0, 0],
					[2, 78, 1608, 0, 0],
					[2, 78, 1609, 0, 0],
					[2, 78, 1610, 0, 0],
					[2, 78, 1611, 0, 0],
					[2, 78, 1612, 0, 0],
					[2, 78, 1613, 0, 0],
					[2, 78, 1615, 0, 0],
					[2, 79, 1616, 0, 0],
					[2, 79, 1617, 0, 0],
					[2, 79, 1618, 0, 0],
					[2, 79, 1619, 0, 0],
					[2, 79, 1620, 0, 0],
					[2, 79, 1621, 0, 0],
					[2, 79, 1622, 0, 0],
					[2, 79, 1625, 0, 0],
					[2, 79, 1626, 0, 0],
					[2, 79, 1627, 0, 0],
					[2, 79, 1628, 0, 0],
					[2, 6, 1630, 0, 0],
					[2, 80, 1632, 0, 0],
					[2, 80, 1633, 0, 0],
					[2, 80, 1634, 0, 0],
					[2, 80, 1635, 0, 0],
					[2, 80, 1636, 0, 0],
					[2, 80, 1637, 0, 0],
					[2, 80, 1638, 0, 0],
					[2, 80, 1640, 0, 0],
					[2, 80, 1641, 0, 0],
					[2, 80, 1642, 0, 0],
					[2, 80, 1643, 0, 0],
					[2, 80, 1644, 0, 0],
					[2, 81, 1645, 0, 0],
					[2, 81, 1646, 0, 0],
					[2, 81, 1647, 0, 0],
					[2, 81, 1648, 0, 0],
					[2, 81, 1649, 0, 0],
					[2, 81, 1650, 0, 0],
					[2, 81, 1651, 0, 0],
					[2, 81, 1652, 0, 0],
					[2, 81, 1653, 0, 0],
					[2, 81, 1655, 0, 0],
					[2, 81, 1656, 0, 0],
					[2, 81, 1657, 0, 0],
					[2, 81, 1658, 0, 0],
					[2, 81, 1659, 0, 0],
					[2, 81, 1660, 0, 0],
					[2, 81, 1661, 0, 0],
					[2, 81, 1666, 0, 0],
					[2, 82, 1667, 0, 0],
					[2, 82, 1668, 0, 0],
					[2, 82, 1669, 0, 0],
					[2, 82, 1670, 0, 0],
					[2, 82, 1671, 0, 0],
					[2, 82, 1672, 0, 0],
					[2, 82, 1673, 0, 0],
					[2, 82, 1674, 0, 0],
					[2, 82, 1675, 0, 0],
					[2, 82, 1677, 0, 0],
					[2, 82, 1678, 0, 0],
					[2, 82, 1679, 0, 0],
					[2, 82, 1680, 0, 0],
					[2, 82, 1681, 0, 0],
					[2, 82, 1682, 0, 0],
					[2, 82, 1683, 0, 0],
					[2, 82, 1688, 0, 0],
					[2, 18, 1689, 0, 0],
					[2, 18, 1690, 0, 0],
					[2, 18, 1691, 0, 0],
					[2, 18, 1692, 0, 0],
					[2, 18, 1693, 0, 0],
					[2, 18, 1694, 0, 0],
					[2, 83, 1708, 0, 0],
					[2, 83, 1709, 0, 0],
					[2, 83, 1710, 0, 0],
					[2, 83, 1711, 0, 0],
					[2, 83, 1712, 0, 0],
					[2, 83, 1713, 0, 0],
					[2, 16, 1714, 0, 0],
					[2, 9, 1715, 0, 0],
					[2, 84, 1716, 0, 0],
					[2, 84, 1717, 0, 0],
					[2, 84, 1718, 0, 0],
					[2, 84, 1719, 0, 0],
					[2, 84, 1720, 0, 0],
					[2, 84, 1721, 0, 0],
					[2, 84, 1722, 0, 0],
					[2, 84, 1723, 0, 0],
					[2, 84, 1724, 0, 0],
					[2, 84, 1725, 0, 0],
					[2, 84, 1726, 0, 0],
					[2, 84, 1727, 0, 0],
					[2, 84, 1728, 0, 0],
					[2, 84, 1729, 0, 0],
					[2, 84, 1730, 0, 0],
					[2, 84, 1731, 0, 0],
					[2, 84, 1732, 0, 0],
					[2, 84, 1733, 0, 0],
					[2, 84, 1735, 0, 0],
					[2, 7, 1736, 0, 0],
					[2, 7, 1737, 0, 0],
					[2, 6, 1738, 0, 0],
					[2, 29, 1740, 0, 0],
					[2, 41, 1741, 0, 0],
					[2, 42, 1742, 0, 0],
					[2, 41, 1743, 0, 0],
					[2, 4, 1744, 0, 0],
					[2, 54, 1745, 0, 0],
					[2, 4, 1746, 0, 0],
					[2, 6, 1747, 0, 0],
					[2, 6, 1756, 0, 0],
					[2, 6, 1757, 0, 0],
					[2, 6, 1758, 0, 0],
					[2, 14, 1759, 0, 0],
					[2, 16, 1761, 0, 0],
					[2, 9, 1762, 0, 0],
					[2, 15, 1763, 0, 0],
					[2, 19, 1764, 0, 0],
					[2, 85, 1765, 0, 0],
					[2, 85, 1766, 0, 0],
					[2, 85, 1767, 0, 0],
					[2, 85, 1768, 0, 0],
					[2, 85, 1769, 0, 0],
					[2, 85, 1770, 0, 0],
					[2, 85, 1771, 0, 0],
					[2, 85, 1772, 0, 0],
					[2, 85, 1773, 0, 0],
					[2, 85, 1774, 0, 0],
					[2, 85, 1775, 0, 0],
					[2, 85, 1776, 0, 0],
					[2, 85, 1777, 0, 0],
					[2, 85, 1778, 0, 0],
					[2, 86, 1779, 0, 0],
					[2, 86, 1780, 0, 0],
					[2, 86, 1781, 0, 0],
					[2, 86, 1782, 0, 0],
					[2, 86, 1783, 0, 0],
					[2, 86, 1784, 0, 0],
					[2, 86, 1785, 0, 0],
					[2, 86, 1786, 0, 0],
					[2, 86, 1787, 0, 0],
					[2, 86, 1788, 0, 0],
					[2, 87, 1789, 0, 0],
					[2, 87, 1790, 0, 0],
					[2, 87, 1791, 0, 0],
					[2, 87, 1792, 0, 0],
					[2, 87, 1793, 0, 0],
					[2, 87, 1794, 0, 0],
					[2, 87, 1795, 0, 0],
					[2, 87, 1796, 0, 0],
					[2, 87, 1797, 0, 0],
					[2, 87, 1798, 0, 0],
					[2, 87, 1799, 0, 0],
					[2, 87, 1800, 0, 0],
					[2, 87, 1801, 0, 0],
					[2, 87, 1802, 0, 0],
					[2, 87, 1803, 0, 0],
					[2, 88, 1804, 0, 0],
					[2, 88, 1805, 0, 0],
					[2, 88, 1806, 0, 0],
					[2, 88, 1807, 0, 0],
					[2, 88, 1808, 0, 0],
					[2, 88, 1809, 0, 0],
					[2, 88, 1810, 0, 0],
					[2, 88, 1811, 0, 0],
					[2, 88, 1812, 0, 0],
					[2, 88, 1813, 0, 0],
					[2, 88, 1814, 0, 0],
					[2, 88, 1815, 0, 0],
					[2, 88, 1816, 0, 0],
					[2, 88, 1817, 0, 0],
					[2, 88, 1818, 0, 0],
					[2, 89, 1819, 0, 0],
					[2, 89, 1820, 0, 0],
					[2, 89, 1821, 0, 0],
					[2, 89, 1822, 0, 0],
					[2, 89, 1823, 0, 0],
					[2, 89, 1824, 0, 0],
					[2, 89, 1825, 0, 0],
					[2, 89, 1826, 0, 0],
					[2, 89, 1827, 0, 0],
					[2, 89, 1828, 0, 0],
					[2, 89, 1829, 0, 0],
					[2, 89, 1830, 0, 0],
					[2, 89, 1831, 0, 0],
					[2, 89, 1832, 0, 0],
					[2, 89, 1833, 0, 0],
					[2, 90, 1834, 0, 0],
					[2, 90, 1835, 0, 0],
					[2, 90, 1836, 0, 0],
					[2, 90, 1837, 0, 0],
					[2, 90, 1838, 0, 0],
					[2, 90, 1839, 0, 0],
					[2, 90, 1840, 0, 0],
					[2, 90, 1841, 0, 0],
					[2, 90, 1842, 0, 0],
					[2, 90, 1843, 0, 0],
					[2, 90, 1844, 0, 0],
					[2, 90, 1845, 0, 0],
					[2, 90, 1846, 0, 0],
					[2, 90, 1847, 0, 0],
					[2, 90, 1848, 0, 0],
					[2, 90, 1849, 0, 0],
					[2, 90, 1850, 0, 0],
					[2, 90, 1851, 0, 0],
					[2, 91, 1852, 0, 0],
					[2, 91, 1853, 0, 0],
					[2, 91, 1854, 0, 0],
					[2, 91, 1855, 0, 0],
					[2, 91, 1856, 0, 0],
					[2, 91, 1857, 0, 0],
					[2, 91, 1858, 0, 0],
					[2, 91, 1859, 0, 0],
					[2, 91, 1860, 0, 0],
					[2, 91, 1861, 0, 0],
					[2, 91, 1862, 0, 0],
					[2, 91, 1863, 0, 0],
					[2, 91, 1864, 0, 0],
					[2, 91, 1865, 0, 0],
					[2, 91, 1866, 0, 0],
					[2, 91, 1867, 0, 0],
					[2, 91, 1868, 0, 0],
					[2, 91, 1869, 0, 0],
					[2, 91, 1870, 0, 0],
					[2, 91, 1871, 0, 0],
					[2, 91, 1872, 0, 0],
					[2, 91, 1873, 0, 0],
					[2, 91, 1874, 0, 0],
					[2, 91, 1875, 0, 0],
					[2, 89, 1876, 0, 0],
					[2, 89, 1877, 0, 0],
					[2, 89, 1878, 0, 0],
					[2, 89, 1879, 0, 0],
					[2, 89, 1880, 0, 0],
					[2, 89, 1881, 0, 0],
					[2, 89, 1882, 0, 0],
					[2, 89, 1883, 0, 0],
					[2, 89, 1884, 0, 0],
					[2, 89, 1885, 0, 0],
					[2, 89, 1886, 0, 0],
					[2, 89, 1887, 0, 0],
					[2, 89, 1888, 0, 0],
					[2, 89, 1889, 0, 0],
					[2, 89, 1890, 0, 0],
					[2, 89, 1891, 0, 0],
					[2, 89, 1892, 0, 0],
					[2, 89, 1893, 0, 0],
					[2, 89, 1894, 0, 0],
					[2, 89, 1895, 0, 0],
					[2, 89, 1896, 0, 0],
					[2, 89, 1897, 0, 0],
					[2, 90, 1898, 0, 0],
					[2, 90, 1899, 0, 0],
					[2, 90, 1900, 0, 0],
					[2, 90, 1901, 0, 0],
					[2, 90, 1902, 0, 0],
					[2, 90, 1903, 0, 0],
					[2, 90, 1904, 0, 0],
					[2, 90, 1905, 0, 0],
					[2, 90, 1906, 0, 0],
					[2, 90, 1907, 0, 0],
					[2, 90, 1908, 0, 0],
					[2, 90, 1909, 0, 0],
					[2, 90, 1910, 0, 0],
					[2, 90, 1911, 0, 0],
					[2, 90, 1912, 0, 0],
					[2, 90, 1913, 0, 0],
					[2, 90, 1914, 0, 0],
					[2, 90, 1915, 0, 0],
					[2, 90, 1916, 0, 0],
					[2, 90, 1917, 0, 0],
					[2, 90, 1918, 0, 0],
					[2, 90, 1919, 0, 0],
					[2, 91, 1920, 0, 0],
					[2, 91, 1921, 0, 0],
					[2, 91, 1922, 0, 0],
					[2, 91, 1923, 0, 0],
					[2, 91, 1924, 0, 0],
					[2, 91, 1925, 0, 0],
					[2, 91, 1926, 0, 0],
					[2, 91, 1927, 0, 0],
					[2, 91, 1928, 0, 0],
					[2, 91, 1929, 0, 0],
					[2, 91, 1930, 0, 0],
					[2, 91, 1931, 0, 0],
					[2, 91, 1932, 0, 0],
					[2, 91, 1933, 0, 0],
					[2, 91, 1934, 0, 0],
					[2, 91, 1935, 0, 0],
					[2, 91, 1936, 0, 0],
					[2, 91, 1937, 0, 0],
					[2, 91, 1938, 0, 0],
					[2, 91, 1939, 0, 0],
					[2, 91, 1940, 0, 0],
					[2, 91, 1941, 0, 0],
					[2, 89, 1942, 0, 0],
					[2, 90, 1943, 0, 0],
					[2, 91, 1944, 0, 0],
					[2, 6, 1945, 0, 1],
					[2, 6, 1946, 0, 1],
					[2, 37, 1947, 0, 0],
					[2, 92, 1948, 0, 0],
					[2, 92, 1949, 0, 0],
					[2, 92, 1950, 0, 0],
					[2, 92, 1951, 0, 0],
					[2, 92, 1952, 0, 0],
					[2, 92, 1953, 0, 0],
					[2, 92, 1954, 0, 0],
					[2, 92, 1955, 0, 0],
					[2, 92, 1956, 0, 0],
					[2, 92, 1957, 0, 0],
					[2, 92, 1958, 0, 0],
					[2, 92, 1959, 0, 0],
					[2, 92, 1960, 0, 0],
					[2, 92, 1961, 0, 0],
					[2, 92, 1962, 0, 0],
					[2, 92, 1963, 0, 0],
					[2, 92, 1964, 0, 0],
					[2, 92, 1965, 0, 0],
					[2, 92, 1966, 0, 0],
					[2, 92, 1967, 0, 0],
					[2, 93, 1968, 0, 0],
					[2, 93, 1969, 0, 0],
					[2, 93, 1970, 0, 0],
					[2, 93, 1971, 0, 0],
					[2, 93, 1972, 0, 0],
					[2, 93, 1973, 0, 0],
					[2, 93, 1974, 0, 0],
					[2, 93, 1975, 0, 0],
					[2, 93, 1976, 0, 0],
					[2, 93, 1977, 0, 0],
					[2, 93, 1978, 0, 0],
					[2, 93, 1979, 0, 0],
					[2, 93, 1980, 0, 0],
					[2, 93, 1981, 0, 0],
					[2, 93, 1982, 0, 0],
					[2, 93, 1983, 0, 0],
					[2, 93, 1984, 0, 0],
					[2, 93, 1985, 0, 0],
					[2, 93, 1986, 0, 0],
					[2, 93, 1987, 0, 0],
					[2, 59, 1988, 0, 0],
					[2, 37, 1989, 0, 0],
					[2, 57, 1990, 0, 0],
					[2, 58, 1991, 0, 0],
					[2, 9, 1992, 0, 0],
					[2, 16, 1993, 0, 0],
					[2, 51, 1994, 0, 0],
					[2, 51, 1995, 0, 0],
					[2, 51, 1996, 0, 0],
					[2, 4, 1997, 0, 0],
					[2, 7, 1998, 0, 0],
					[2, 18, 1999, 0, 0],
					[2, 92, 2000, 0, 0],
					[2, 93, 2001, 0, 0],
					[2, 61, 2002, 0, 0],
					[2, 86, 2003, 0, 0],
					[2, 26, 2004, 0, 0],
					[2, 41, 2005, 0, 0],
					[2, 85, 2006, 0, 0],
					[2, 87, 2007, 0, 0],
					[2, 88, 2008, 0, 0],
					[2, 89, 2009, 0, 0],
					[2, 90, 2010, 0, 0],
					[2, 91, 2011, 0, 0],
					[2, 92, 2012, 0, 0],
					[2, 93, 2013, 0, 0],
					[2, 4, 2014, 0, 0],
					[2, 6, 2015, 0, 0],
					[2, 7, 2016, 0, 0],
					[2, 13, 2017, 0, 0],
					[2, 18, 2018, 0, 0],
					[2, 26, 2019, 0, 0],
					[2, 34, 2020, 0, 0],
					[2, 43, 2021, 0, 0],
					[2, 61, 2022, 0, 0],
					[2, 85, 2023, 0, 0],
					[2, 86, 2024, 0, 0],
					[2, 87, 2025, 0, 0],
					[2, 88, 2026, 0, 0],
					[2, 89, 2027, 0, 0],
					[2, 90, 2028, 0, 0],
					[2, 91, 2029, 0, 0],
					[2, 92, 2030, 0, 0],
					[2, 93, 2031, 0, 0],
					[2, 94, 2032, 0, 0],
					[2, 94, 2033, 0, 0],
					[2, 94, 2034, 0, 0],
					[2, 94, 2035, 0, 0],
					[2, 94, 2036, 0, 0],
					[2, 94, 2037, 0, 0],
					[2, 94, 2038, 0, 0],
					[2, 94, 2039, 0, 0],
					[2, 95, 2040, 0, 0],
					[2, 95, 2041, 0, 0],
					[2, 95, 2042, 0, 0],
					[2, 95, 2043, 0, 0],
					[2, 95, 2044, 0, 0],
					[2, 95, 2046, 0, 0],
					[2, 95, 2047, 0, 0],
					[2, 95, 2048, 0, 0],
					[2, 95, 2049, 0, 0],
					[2, 95, 2050, 0, 0],
					[2, 95, 2051, 0, 0],
					[2, 95, 2052, 0, 0],
					[2, 95, 2053, 0, 0],
					[2, 95, 2054, 0, 0],
					[2, 95, 2055, 0, 0],
					[2, 95, 2056, 0, 0],
					[2, 95, 2057, 0, 0],
					[2, 95, 2058, 0, 0],
					[2, 95, 2059, 0, 0],
					[2, 95, 2060, 0, 0],
					[2, 95, 2061, 0, 0],
					[2, 95, 2062, 0, 0],
					[2, 95, 2063, 0, 0],
					[2, 95, 2064, 0, 0],
					[2, 95, 2065, 0, 0],
					[2, 95, 2066, 0, 0],
					[2, 95, 2067, 0, 0],
					[2, 95, 2068, 0, 0],
					[2, 95, 2069, 0, 0],
					[2, 95, 2070, 0, 0],
					[2, 95, 2071, 0, 0],
					[2, 95, 2072, 0, 0],
					[2, 95, 2073, 0, 0],
					[2, 95, 2074, 0, 0],
					[2, 96, 2075, 0, 0],
					[2, 96, 2076, 0, 0],
					[2, 96, 2077, 0, 0],
					[2, 96, 2078, 0, 0],
					[2, 96, 2079, 0, 0],
					[2, 96, 2080, 0, 0],
					[2, 96, 2081, 0, 0],
					[2, 96, 2082, 0, 0],
					[2, 97, 2083, 0, 0],
					[2, 97, 2084, 0, 0],
					[2, 97, 2085, 0, 0],
					[2, 97, 2086, 0, 0],
					[2, 97, 2087, 0, 0],
					[2, 97, 2088, 0, 0],
					[2, 97, 2089, 0, 0],
					[2, 97, 2090, 0, 0],
					[2, 97, 2091, 0, 0],
					[2, 97, 2092, 0, 0],
					[2, 97, 2093, 0, 0],
					[2, 86, 2094, 0, 0],
					[2, 86, 2095, 0, 0],
					[2, 86, 2096, 0, 0],
					[2, 86, 2097, 0, 0],
					[2, 86, 2098, 0, 0],
					[2, 86, 2099, 0, 0],
					[2, 86, 2100, 0, 0],
					[2, 86, 2101, 0, 0],
					[2, 86, 2102, 0, 0],
					[2, 88, 2103, 0, 0],
					[2, 88, 2104, 0, 0],
					[2, 88, 2105, 0, 0],
					[2, 89, 2106, 0, 0],
					[2, 89, 2107, 0, 0],
					[2, 89, 2108, 0, 0],
					[2, 89, 2109, 0, 0],
					[2, 89, 2110, 0, 0],
					[2, 90, 2111, 0, 0],
					[2, 90, 2112, 0, 0],
					[2, 90, 2113, 0, 0],
					[2, 90, 2114, 0, 0],
					[2, 90, 2115, 0, 0],
					[2, 98, 2116, 0, 0],
					[2, 98, 2117, 0, 0],
					[2, 98, 2118, 0, 0],
					[2, 98, 2119, 0, 0],
					[2, 98, 2120, 0, 0],
					[2, 98, 2121, 0, 0],
					[2, 99, 2122, 0, 0],
					[2, 99, 2123, 0, 0],
					[2, 99, 2124, 0, 0],
					[2, 99, 2125, 0, 0],
					[2, 99, 2126, 0, 0],
					[2, 99, 2127, 0, 0],
					[2, 99, 2128, 0, 0],
					[2, 99, 2129, 0, 0],
					[2, 99, 2130, 0, 0],
					[2, 99, 2131, 0, 0],
					[2, 99, 2132, 0, 0],
					[2, 99, 2133, 0, 0],
					[2, 99, 2134, 0, 0],
					[2, 99, 2135, 0, 0],
					[2, 99, 2136, 0, 0],
					[2, 99, 2137, 0, 0],
					[2, 99, 2138, 0, 0],
					[2, 99, 2139, 0, 0],
					[2, 99, 2140, 0, 0],
					[2, 99, 2141, 0, 0],
					[2, 99, 2142, 0, 0],
					[2, 99, 2143, 0, 0],
					[2, 99, 2144, 0, 0],
					[2, 99, 2145, 0, 0],
					[2, 99, 2146, 0, 0],
					[2, 99, 2147, 0, 0],
					[2, 99, 2148, 0, 0],
					[2, 99, 2149, 0, 0],
					[2, 99, 2150, 0, 0],
					[2, 99, 2151, 0, 0],
					[2, 99, 2152, 0, 0],
					[2, 99, 2153, 0, 0],
					[2, 99, 2154, 0, 0],
					[2, 99, 2155, 0, 0],
					[2, 100, 2156, 0, 0],
					[2, 100, 2157, 0, 0],
					[2, 100, 2158, 0, 0],
					[2, 100, 2159, 0, 0],
					[2, 100, 2160, 0, 0],
					[2, 100, 2161, 0, 0],
					[2, 100, 2162, 0, 0],
					[2, 101, 2163, 0, 0],
					[2, 101, 2164, 0, 0],
					[2, 101, 2165, 0, 0],
					[2, 101, 2166, 0, 0],
					[2, 101, 2167, 0, 0],
					[2, 101, 2168, 0, 0],
					[2, 101, 2169, 0, 0],
					[2, 98, 2170, 0, 0],
					[2, 102, 2171, 0, 0],
					[2, 102, 2172, 0, 0],
					[2, 102, 2173, 0, 0],
					[2, 102, 2174, 0, 0],
					[2, 102, 2175, 0, 0],
					[2, 102, 2176, 0, 0],
					[2, 102, 2177, 0, 0],
					[2, 98, 2178, 0, 0],
					[2, 100, 2179, 0, 0],
					[2, 101, 2180, 0, 0],
					[2, 102, 2181, 0, 0],
					[2, 103, 2182, 0, 0],
					[2, 103, 2183, 0, 0],
					[2, 103, 2184, 0, 0],
					[2, 103, 2185, 0, 0],
					[2, 103, 2186, 0, 0],
					[2, 103, 2187, 0, 0],
					[2, 103, 2188, 0, 0],
					[2, 98, 2189, 0, 0],
					[2, 98, 2190, 0, 0],
					[2, 98, 2191, 0, 0],
					[2, 98, 2192, 0, 0],
					[2, 100, 2193, 0, 0],
					[2, 100, 2194, 0, 0],
					[2, 100, 2195, 0, 0],
					[2, 100, 2196, 0, 0],
					[2, 101, 2197, 0, 0],
					[2, 101, 2198, 0, 0],
					[2, 101, 2199, 0, 0],
					[2, 102, 2200, 0, 0],
					[2, 102, 2201, 0, 0],
					[2, 102, 2202, 0, 0],
					[2, 103, 2203, 0, 0],
					[2, 103, 2204, 0, 0],
					[2, 103, 2205, 0, 0],
					[2, 103, 2206, 0, 0],
					[2, 103, 2207, 0, 0],
					[2, 14, 2208, 0, 0],
					[2, 98, 2209, 0, 0],
					[2, 96, 2210, 0, 0],
					[2, 97, 2211, 0, 0],
					[2, 97, 2212, 0, 0],
					[2, 97, 2213, 0, 0],
					[2, 97, 2214, 0, 0],
					[2, 97, 2215, 0, 0],
					[2, 97, 2216, 0, 0],
					[2, 97, 2217, 0, 0],
					[2, 97, 2218, 0, 0],
					[2, 97, 2219, 0, 0],
					[2, 97, 2220, 0, 0],
					[2, 97, 2221, 0, 0],
					[2, 104, 2222, 0, 0],
					[2, 104, 2223, 0, 0],
					[2, 104, 2224, 0, 0],
					[2, 104, 2225, 0, 0],
					[2, 104, 2226, 0, 0],
					[2, 104, 2227, 0, 0],
					[2, 104, 2228, 0, 0],
					[2, 104, 2229, 0, 0],
					[2, 104, 2230, 0, 0],
					[2, 104, 2231, 0, 0],
					[2, 104, 2232, 0, 0],
					[2, 104, 2233, 0, 0],
					[2, 104, 2234, 0, 0],
					[2, 105, 2235, 0, 0],
					[2, 105, 2236, 0, 0],
					[2, 105, 2237, 0, 0],
					[2, 105, 2238, 0, 0],
					[2, 105, 2239, 0, 0],
					[2, 105, 2240, 0, 0],
					[2, 105, 2241, 0, 0],
					[2, 105, 2242, 0, 0],
					[2, 105, 2243, 0, 0],
					[2, 105, 2244, 0, 0],
					[2, 105, 2245, 0, 0],
					[2, 106, 2246, 0, 0],
					[2, 106, 2247, 0, 0],
					[2, 106, 2248, 0, 0],
					[2, 106, 2249, 0, 0],
					[2, 106, 2250, 0, 0],
					[2, 106, 2251, 0, 0],
					[2, 106, 2252, 0, 0],
					[2, 106, 2253, 0, 0],
					[2, 106, 2254, 0, 0],
					[2, 106, 2255, 0, 0],
					[2, 106, 2256, 0, 0],
					[2, 106, 2257, 0, 0],
					[2, 106, 2258, 0, 0],
					[2, 104, 2259, 0, 0],
					[2, 106, 2260, 0, 0],
					[2, 104, 2261, 0, 0],
					[2, 106, 2262, 0, 0],
					[2, 96, 2263, 0, 0],
					[2, 96, 2264, 0, 0],
					[2, 96, 2265, 0, 0],
					[2, 96, 2266, 0, 0],
					[2, 58, 2267, 0, 0],
					[2, 37, 2268, 0, 0],
					[2, 14, 2269, 0, 0],
					[2, 107, 2270, 0, 0],
					[2, 107, 2271, 0, 0],
					[2, 107, 2272, 0, 0],
					[2, 107, 2273, 0, 0],
					[2, 107, 2274, 0, 0],
					[2, 107, 2275, 0, 0],
					[2, 107, 2276, 0, 0],
					[2, 107, 2277, 0, 0],
					[2, 107, 2278, 0, 0],
					[2, 107, 2279, 0, 0],
					[2, 107, 2280, 0, 0],
					[2, 107, 2281, 0, 0],
					[2, 107, 2282, 0, 0],
					[2, 107, 2283, 0, 0],
					[2, 107, 2284, 0, 0],
					[2, 107, 2285, 0, 0],
					[2, 107, 2286, 0, 0],
					[2, 107, 2287, 0, 0],
					[2, 107, 2288, 0, 0],
					[2, 107, 2289, 0, 0],
					[2, 107, 2290, 0, 0],
					[2, 107, 2291, 0, 0],
					[2, 107, 2292, 0, 0],
					[2, 107, 2293, 0, 0],
					[2, 107, 2294, 0, 0],
					[2, 107, 2295, 0, 0],
					[2, 107, 2296, 0, 0],
					[2, 107, 2297, 0, 0],
					[2, 107, 2298, 0, 0],
					[2, 107, 2299, 0, 0],
					[2, 107, 2300, 0, 0],
					[2, 107, 2301, 0, 0],
					[2, 107, 2302, 0, 0],
					[2, 107, 2303, 0, 0],
					[2, 107, 2304, 0, 0],
					[2, 108, 2305, 0, 0],
					[2, 108, 2306, 0, 0],
					[2, 108, 2307, 0, 0],
					[2, 108, 2308, 0, 0],
					[2, 108, 2309, 0, 0],
					[2, 108, 2310, 0, 0],
					[2, 108, 2311, 0, 0],
					[2, 108, 2312, 0, 0],
					[2, 108, 2313, 0, 0],
					[2, 108, 2314, 0, 0],
					[2, 108, 2315, 0, 0],
					[2, 108, 2316, 0, 0],
					[2, 108, 2317, 0, 0],
					[2, 108, 2318, 0, 0],
					[2, 109, 2319, 0, 0],
					[2, 109, 2320, 0, 0],
					[2, 109, 2321, 0, 0],
					[2, 109, 2322, 0, 0],
					[2, 109, 2323, 0, 0],
					[2, 109, 2324, 0, 0],
					[2, 109, 2325, 0, 0],
					[2, 109, 2326, 0, 0],
					[2, 109, 2327, 0, 0],
					[2, 109, 2328, 0, 0],
					[2, 109, 2329, 0, 0],
					[2, 109, 2330, 0, 0],
					[2, 109, 2331, 0, 0],
					[2, 13, 2332, 0, 0],
					[2, 13, 2333, 0, 0],
					[2, 14, 2334, 0, 0],
					[2, 35, 2335, 0, 0],
					[2, 37, 2336, 0, 0],
					[2, 58, 2337, 0, 0],
					[2, 90, 2344, 0, 0],
					[2, 100, 2345, 0, 0],
					[2, 6, 2346, 0, 0],
					[2, 13, 2347, 0, 0],
					[2, 13, 2348, 0, 0],
					[2, 4, 2349, 0, 0],
					[2, 6, 2350, 0, 0],
					[2, 7, 2351, 0, 0],
					[2, 92, 2352, 0, 0],
					[2, 18, 2353, 0, 0],
					[2, 93, 2354, 0, 0],
					[2, 83, 2355, 0, 0],
					[2, 83, 2356, 0, 0],
					[2, 14, 2357, 0, 0],
					[2, 89, 2358, 0, 0],
					[2, 92, 2381, 0, 0],
					[2, 61, 2382, 0, 0],
					[2, 61, 2383, 0, 0],
					[2, 61, 2384, 0, 0],
					[2, 86, 2385, 0, 0],
					[2, 43, 2386, 0, 0],
					[2, 111, 2387, 0, 0],
					[2, 111, 2388, 0, 0],
					[2, 111, 2389, 0, 0],
					[2, 111, 2390, 0, 0],
					[2, 111, 2391, 0, 0],
					[2, 111, 2392, 0, 0],
					[2, 111, 2393, 0, 0],
					[2, 111, 2395, 0, 0],
					[2, 111, 2396, 0, 0],
					[2, 111, 2397, 0, 0],
					[2, 111, 2398, 0, 0],
					[2, 111, 2399, 0, 0],
					[2, 111, 2400, 0, 0],
					[2, 6, 2401, 0, 0],
					[2, 95, 2402, 0, 0],
					[2, 95, 2403, 0, 0],
					[2, 86, 2404, 0, 0],
					[2, 6, 2405, 0, 0],
					[2, 4, 2406, 0, 0],
					[2, 7, 2407, 0, 0],
					[2, 18, 2408, 0, 0],
					[2, 14, 2409, 0, 0],
					[2, 86, 2410, 0, 0],
					[2, 97, 2411, 0, 0],
					[2, 26, 2412, 0, 0],
					[2, 16, 2413, 0, 0],
					[2, 13, 2414, 0, 0],
					[2, 105, 2415, 0, 0],
					[2, 35, 2416, 0, 0],
					[2, 106, 2417, 0, 0],
					[2, 98, 2418, 0, 0],
					[2, 54, 2419, 0, 0],
					[2, 108, 2420, 0, 0],
					[2, 100, 2421, 0, 0],
					[2, 109, 2422, 0, 0],
					[2, 9, 2423, 0, 0],
					[2, 43, 2424, 0, 0],
					[2, 101, 2425, 0, 0],
					[2, 102, 2426, 0, 0],
					[2, 37, 2427, 0, 0],
					[2, 61, 2428, 0, 0],
					[2, 103, 2429, 0, 0],
					[2, 15, 2430, 0, 0],
					[2, 74, 2431, 0, 0],
					[2, 104, 2432, 0, 0],
					[2, 34, 2433, 0, 0],
					[2, 59, 2434, 0, 0],
					[2, 51, 2435, 0, 0],
					[2, 19, 2436, 0, 0],
					[2, 41, 2437, 0, 0],
					[2, 8, 2438, 0, 0],
					[2, 58, 2439, 0, 0],
					[2, 84, 2440, 0, 0],
					[2, 60, 2441, 0, 0],
					[2, 57, 2442, 0, 0],
					[2, 83, 2443, 0, 0],
					[2, 42, 2444, 0, 0],
					[2, 75, 2445, 0, 0],
					[2, 78, 2446, 0, 0],
					[2, 79, 2447, 0, 0],
					[2, 80, 2448, 0, 0],
					[2, 81, 2449, 0, 0],
					[2, 107, 2450, 0, 0],
					[2, 82, 2451, 0, 0],
					[2, 99, 2452, 0, 0],
					[2, 94, 2453, 0, 0],
					[2, 95, 2454, 0, 0],
					[2, 96, 2455, 0, 0],
					[2, 111, 2456, 0, 0],
					[2, 85, 2458, 0, 0],
					[2, 87, 2459, 0, 0],
					[2, 89, 2460, 0, 0],
					[2, 91, 2461, 0, 0],
					[2, 93, 2462, 0, 0],
					[2, 88, 2463, 0, 0],
					[2, 90, 2464, 0, 0],
					[2, 92, 2465, 0, 0],
					[2, 33, 2466, 0, 0],
					[2, 35, 2467, 0, 0],
					[2, 29, 2468, 0, 0],
					[2, 29, 2469, 0, 0],
					[2, 29, 2470, 0, 0],
					[2, 112, 2471, 0, 0],
					[2, 112, 2472, 0, 0],
					[2, 112, 2473, 0, 0],
					[2, 112, 2474, 0, 0],
					[2, 112, 2475, 0, 0],
					[2, 112, 2476, 0, 0],
					[2, 112, 2477, 0, 0],
					[2, 112, 2478, 0, 0],
					[2, 112, 2479, 0, 0],
					[2, 112, 2480, 0, 0],
					[2, 112, 2481, 0, 0],
					[2, 112, 2482, 0, 0],
					[2, 112, 2483, 0, 0],
					[2, 86, 2484, 0, 0],
					[2, 113, 2485, 0, 0],
					[2, 113, 2486, 0, 0],
					[2, 113, 2487, 0, 0],
					[2, 113, 2488, 0, 0],
					[2, 113, 2489, 0, 0],
					[2, 113, 2490, 0, 0],
					[2, 113, 2491, 0, 0],
					[2, 113, 2492, 0, 0],
					[2, 113, 2493, 0, 0],
					[2, 113, 2494, 0, 0],
					[2, 113, 2495, 0, 0],
					[2, 113, 2496, 0, 0],
					[2, 113, 2497, 0, 0],
					[2, 113, 2498, 0, 0],
					[2, 113, 2499, 0, 0],
					[2, 113, 2500, 0, 0],
					[2, 113, 2501, 0, 0],
					[2, 113, 2502, 0, 0],
					[2, 113, 2503, 0, 0],
					[2, 113, 2504, 0, 0],
					[2, 113, 2505, 0, 0],
					[2, 113, 2506, 0, 0],
					[2, 113, 2507, 0, 0],
					[2, 113, 2508, 0, 0],
					[2, 114, 2509, 0, 0],
					[2, 114, 2510, 0, 0],
					[2, 114, 2511, 0, 0],
					[2, 114, 2512, 0, 0],
					[2, 114, 2513, 0, 0],
					[2, 114, 2514, 0, 0],
					[2, 114, 2515, 0, 0],
					[2, 114, 2516, 0, 0],
					[2, 114, 2517, 0, 0],
					[2, 114, 2518, 0, 0],
					[2, 114, 2519, 0, 0],
					[2, 115, 2520, 0, 0],
					[2, 115, 2521, 0, 0],
					[2, 115, 2522, 0, 0],
					[2, 115, 2523, 0, 0],
					[2, 115, 2524, 0, 0],
					[2, 115, 2525, 0, 0],
					[2, 115, 2526, 0, 0],
					[2, 115, 2527, 0, 0],
					[2, 115, 2528, 0, 0],
					[2, 115, 2529, 0, 0],
					[2, 115, 2530, 0, 0],
					[2, 115, 2531, 0, 0],
					[2, 115, 2532, 0, 0],
					[2, 115, 2533, 0, 0],
					[2, 115, 2534, 0, 0],
					[2, 115, 2535, 0, 0],
					[2, 115, 2536, 0, 0],
					[2, 115, 2537, 0, 0],
					[2, 115, 2538, 0, 0],
					[2, 115, 2539, 0, 0],
					[2, 115, 2540, 0, 0],
					[2, 115, 2541, 0, 0],
					[2, 115, 2542, 0, 0],
					[2, 115, 2543, 0, 0],
					[2, 115, 2544, 0, 0],
					[2, 115, 2545, 0, 0],
					[2, 115, 2546, 0, 0],
					[2, 115, 2547, 0, 0],
					[2, 115, 2548, 0, 0],
					[2, 115, 2549, 0, 0],
					[2, 115, 2550, 0, 0],
					[2, 115, 2551, 0, 0],
					[2, 115, 2552, 0, 0],
					[2, 115, 2553, 0, 0],
					[2, 115, 2554, 0, 0],
					[2, 115, 2555, 0, 0],
					[2, 115, 2556, 0, 0],
					[2, 115, 2557, 0, 0],
					[2, 115, 2558, 0, 0],
					[2, 116, 2559, 0, 0],
					[2, 116, 2560, 0, 0],
					[2, 116, 2561, 0, 0],
					[2, 116, 2562, 0, 0],
					[2, 116, 2563, 0, 0],
					[2, 116, 2564, 0, 0],
					[2, 116, 2565, 0, 0],
					[2, 116, 2566, 0, 0],
					[2, 116, 2567, 0, 0],
					[2, 116, 2568, 0, 0],
					[2, 116, 2569, 0, 0],
					[2, 95, 2570, 0, 0],
					[2, 95, 2571, 0, 0],
					[2, 95, 2572, 0, 0],
					[2, 95, 2573, 0, 0],
					[2, 113, 2574, 0, 0],
					[2, 113, 2575, 0, 0],
					[2, 113, 2576, 0, 0],
					[2, 117, 2577, 0, 0],
					[2, 117, 2578, 0, 0],
					[2, 117, 2579, 0, 0],
					[2, 117, 2580, 0, 0],
					[2, 117, 2581, 0, 0],
					[2, 117, 2582, 0, 0],
					[2, 117, 2583, 0, 0],
					[2, 117, 2584, 0, 0],
					[2, 117, 2585, 0, 0],
					[2, 117, 2586, 0, 0],
					[2, 117, 2587, 0, 0],
					[2, 117, 2588, 0, 0],
					[2, 117, 2589, 0, 0],
					[2, 117, 2590, 0, 0],
					[2, 117, 2591, 0, 0],
					[2, 117, 2592, 0, 0],
					[2, 117, 2593, 0, 0],
					[2, 117, 2594, 0, 0],
					[2, 117, 2595, 0, 0],
					[2, 117, 2596, 0, 0],
					[2, 117, 2597, 0, 0],
					[2, 117, 2598, 0, 0],
					[2, 16, 2600, 0, 0],
					[2, 16, 2601, 0, 0],
					[2, 9, 2602, 0, 0],
					[2, 9, 2603, 0, 0],
					[2, 43, 2604, 0, 0],
					[2, 83, 2605, 0, 0],
					[2, 45, 2606, 0, 0],
					[3, 6, 1, 0, 0],
					[3, 6, 2, 0, 0],
					[3, 6, 3, 0, 0],
					[3, 6, 4, 0, 0],
					[3, 6, 5, 0, 0],
					[3, 6, 7, 0, 0],
					[3, 6, 8, 0, 0],
					[3, 6, 9, 0, 0],
					[3, 6, 10, 0, 0],
					[3, 6, 11, 0, 0],
					[3, 6, 12, 0, 0],
					[3, 6, 14, 0, 0],
					[3, 6, 15, 0, 0],
					[3, 6, 16, 0, 0],
					[3, 6, 17, 0, 0],
					[3, 6, 18, 0, 0],
					[3, 6, 20, 0, 0],
					[3, 6, 21, 0, 1],
					[3, 6, 22, 0, 1],
					[3, 6, 23, 0, 0],
					[3, 6, 36, 0, 0],
					[3, 7, 39, 0, 0],
					[3, 7, 40, 0, 0],
					[3, 7, 41, 0, 0],
					[3, 7, 42, 0, 0],
					[3, 7, 43, 0, 0],
					[3, 7, 44, 0, 0],
					[3, 7, 46, 0, 0],
					[3, 7, 47, 0, 0],
					[3, 7, 48, 0, 0],
					[3, 7, 49, 0, 0],
					[3, 7, 50, 0, 0],
					[3, 7, 51, 0, 0],
					[3, 7, 53, 0, 0],
					[3, 7, 54, 0, 0],
					[3, 7, 55, 0, 0],
					[3, 7, 56, 0, 1],
					[3, 7, 57, 0, 1],
					[3, 7, 58, 0, 0],
					[3, 7, 65, 0, 0],
					[3, 4, 66, 0, 0],
					[3, 4, 67, 0, 0],
					[3, 4, 68, 0, 0],
					[3, 4, 69, 0, 0],
					[3, 4, 70, 0, 0],
					[3, 4, 71, 0, 0],
					[3, 4, 72, 0, 0],
					[3, 4, 74, 0, 0],
					[3, 4, 79, 0, 0],
					[3, 4, 80, 0, 0],
					[3, 4, 81, 0, 0],
					[3, 4, 85, 0, 0],
					[3, 4, 86, 0, 0],
					[3, 4, 87, 0, 0],
					[3, 4, 90, 0, 1],
					[3, 4, 91, 0, 1],
					[3, 4, 92, 0, 0],
					[3, 4, 93, 0, 0],
					[3, 4, 94, 0, 0],
					[3, 4, 95, 0, 0],
					[3, 4, 108, 0, 0],
					[3, 4, 109, 0, 0],
					[3, 26, 126, 0, 0],
					[3, 26, 127, 0, 0],
					[3, 26, 128, 0, 0],
					[3, 26, 129, 0, 0],
					[3, 26, 130, 0, 0],
					[3, 26, 131, 0, 0],
					[3, 26, 132, 0, 0],
					[3, 26, 133, 0, 0],
					[3, 26, 134, 0, 0],
					[3, 26, 135, 0, 0],
					[3, 26, 136, 0, 0],
					[3, 26, 137, 0, 1],
					[3, 26, 138, 0, 1],
					[3, 26, 139, 0, 0],
					[3, 26, 140, 0, 0],
					[3, 26, 141, 0, 0],
					[3, 26, 142, 0, 0],
					[3, 26, 143, 0, 0],
					[3, 26, 144, 0, 0],
					[3, 26, 145, 0, 0],
					[3, 26, 146, 0, 0],
					[3, 26, 147, 0, 0],
					[3, 26, 148, 0, 0],
					[3, 26, 149, 0, 0],
					[3, 26, 150, 0, 0],
					[3, 13, 155, 0, 0],
					[3, 13, 156, 0, 0],
					[3, 13, 157, 0, 0],
					[3, 13, 158, 0, 0],
					[3, 13, 159, 0, 0],
					[3, 13, 160, 0, 0],
					[3, 13, 161, 0, 0],
					[3, 13, 162, 0, 0],
					[3, 13, 163, 0, 0],
					[3, 13, 166, 0, 1],
					[3, 13, 167, 0, 1],
					[3, 13, 168, 0, 0],
					[3, 13, 169, 0, 0],
					[3, 13, 170, 0, 0],
					[3, 13, 171, 0, 0],
					[3, 14, 173, 0, 0],
					[3, 14, 174, 0, 0],
					[3, 14, 175, 0, 0],
					[3, 14, 176, 0, 0],
					[3, 14, 177, 0, 0],
					[3, 14, 178, 0, 0],
					[3, 14, 179, 0, 0],
					[3, 14, 180, 0, 0],
					[3, 14, 181, 0, 0],
					[3, 14, 182, 0, 0],
					[3, 14, 183, 0, 0],
					[3, 14, 184, 0, 0],
					[3, 14, 185, 0, 0],
					[3, 14, 186, 0, 0],
					[3, 14, 187, 0, 0],
					[3, 14, 188, 0, 0],
					[3, 14, 189, 0, 0],
					[3, 14, 190, 0, 1],
					[3, 14, 191, 0, 1],
					[3, 14, 192, 0, 0],
					[3, 14, 193, 0, 0],
					[3, 14, 194, 0, 0],
					[3, 14, 196, 0, 0],
					[3, 14, 197, 0, 0],
					[3, 14, 198, 0, 0],
					[3, 14, 199, 0, 0],
					[3, 14, 200, 0, 0],
					[3, 14, 201, 0, 0],
					[3, 14, 202, 0, 0],
					[3, 14, 203, 0, 0],
					[3, 8, 204, 0, 0],
					[3, 8, 205, 0, 1],
					[3, 8, 206, 0, 1],
					[3, 8, 207, 0, 0],
					[3, 8, 208, 0, 0],
					[3, 8, 209, 0, 0],
					[3, 8, 210, 0, 1],
					[3, 8, 211, 0, 1],
					[3, 8, 212, 0, 0],
					[3, 8, 213, 0, 0],
					[3, 8, 214, 0, 0],
					[3, 8, 215, 0, 1],
					[3, 8, 216, 0, 0],
					[3, 8, 217, 0, 0],
					[3, 8, 218, 0, 0],
					[3, 9, 231, 0, 0],
					[3, 9, 232, 0, 0],
					[3, 9, 233, 0, 0],
					[3, 9, 234, 0, 0],
					[3, 9, 235, 0, 0],
					[3, 9, 236, 0, 0],
					[3, 9, 237, 0, 0],
					[3, 9, 238, 0, 0],
					[3, 9, 239, 0, 0],
					[3, 9, 241, 0, 0],
					[3, 9, 242, 0, 0],
					[3, 9, 243, 0, 1],
					[3, 9, 244, 0, 1],
					[3, 9, 245, 0, 0],
					[3, 9, 246, 0, 0],
					[3, 9, 247, 0, 0],
					[3, 9, 248, 0, 0],
					[3, 9, 249, 0, 0],
					[3, 9, 250, 0, 0],
					[3, 9, 251, 0, 0],
					[3, 9, 253, 0, 0],
					[3, 9, 254, 0, 0],
					[3, 16, 255, 0, 0],
					[3, 16, 256, 0, 0],
					[3, 16, 257, 0, 0],
					[3, 16, 258, 0, 0],
					[3, 16, 259, 0, 0],
					[3, 16, 260, 0, 0],
					[3, 16, 262, 0, 0],
					[3, 16, 263, 0, 0],
					[3, 16, 264, 0, 0],
					[3, 16, 265, 0, 0],
					[3, 16, 266, 0, 0],
					[3, 16, 267, 0, 0],
					[3, 16, 268, 0, 0],
					[3, 16, 269, 0, 1],
					[3, 16, 270, 0, 1],
					[3, 16, 271, 0, 0],
					[3, 16, 272, 0, 0],
					[3, 16, 273, 0, 0],
					[3, 16, 274, 0, 0],
					[3, 16, 275, 0, 0],
					[3, 16, 276, 0, 0],
					[3, 16, 277, 0, 0],
					[3, 15, 278, 0, 0],
					[3, 15, 279, 0, 0],
					[3, 15, 280, 0, 0],
					[3, 15, 281, 0, 0],
					[3, 15, 282, 0, 0],
					[3, 15, 283, 0, 0],
					[3, 15, 284, 0, 0],
					[3, 15, 285, 0, 1],
					[3, 15, 286, 0, 1],
					[3, 15, 287, 0, 0],
					[3, 18, 288, 0, 0],
					[3, 18, 289, 0, 0],
					[3, 18, 290, 0, 0],
					[3, 18, 291, 0, 0],
					[3, 18, 292, 0, 0],
					[3, 18, 293, 0, 0],
					[3, 18, 294, 0, 0],
					[3, 18, 295, 0, 1],
					[3, 18, 296, 0, 1],
					[3, 18, 297, 0, 0],
					[3, 18, 304, 0, 0],
					[3, 19, 305, 0, 0],
					[3, 19, 306, 0, 0],
					[3, 19, 307, 0, 0],
					[3, 19, 308, 0, 1],
					[3, 19, 309, 0, 1],
					[3, 19, 310, 0, 0],
					[3, 19, 311, 0, 0],
					[3, 19, 312, 0, 0],
					[3, 29, 474, 0, 0],
					[3, 29, 475, 0, 0],
					[3, 29, 476, 0, 0],
					[3, 29, 477, 0, 0],
					[3, 29, 478, 0, 0],
					[3, 29, 479, 0, 0],
					[3, 29, 480, 0, 0],
					[3, 29, 481, 0, 0],
					[3, 29, 482, 0, 0],
					[3, 29, 483, 0, 0],
					[3, 29, 484, 0, 0],
					[3, 29, 485, 0, 0],
					[3, 29, 487, 0, 0],
					[3, 29, 492, 0, 0],
					[3, 29, 498, 0, 0],
					[3, 29, 500, 0, 0],
					[3, 29, 506, 0, 0],
					[3, 29, 507, 0, 0],
					[3, 29, 508, 0, 0],
					[3, 29, 509, 0, 0],
					[3, 29, 510, 0, 0],
					[3, 29, 511, 0, 0],
					[3, 29, 512, 0, 0],
					[3, 29, 513, 0, 0],
					[3, 29, 514, 0, 0],
					[3, 29, 515, 0, 0],
					[3, 29, 516, 0, 0],
					[3, 29, 517, 0, 0],
					[3, 33, 524, 0, 0],
					[3, 33, 525, 0, 0],
					[3, 33, 526, 0, 0],
					[3, 33, 527, 0, 0],
					[3, 33, 528, 0, 0],
					[3, 33, 529, 0, 0],
					[3, 33, 530, 0, 0],
					[3, 33, 531, 0, 0],
					[3, 33, 532, 0, 0],
					[3, 33, 533, 0, 0],
					[3, 33, 534, 0, 0],
					[3, 33, 535, 0, 0],
					[3, 33, 536, 0, 0],
					[3, 33, 537, 0, 0],
					[3, 33, 538, 0, 1],
					[3, 33, 539, 0, 1],
					[3, 29, 540, 0, 0],
					[3, 34, 541, 0, 0],
					[3, 34, 542, 0, 1],
					[3, 34, 543, 0, 1],
					[3, 34, 544, 0, 0],
					[3, 34, 545, 0, 1],
					[3, 34, 546, 0, 0],
					[3, 34, 547, 0, 0],
					[3, 34, 548, 0, 0],
					[3, 34, 549, 0, 0],
					[3, 34, 550, 0, 0],
					[3, 34, 551, 0, 0],
					[3, 34, 552, 0, 1],
					[3, 34, 553, 0, 1],
					[3, 34, 554, 0, 0],
					[3, 34, 555, 0, 0],
					[3, 34, 556, 0, 0],
					[3, 34, 557, 0, 1],
					[3, 34, 558, 0, 0],
					[3, 34, 559, 0, 0],
					[3, 35, 560, 0, 0],
					[3, 35, 561, 0, 0],
					[3, 35, 562, 0, 0],
					[3, 35, 563, 0, 0],
					[3, 35, 564, 0, 0],
					[3, 35, 565, 0, 0],
					[3, 35, 566, 0, 0],
					[3, 35, 567, 0, 0],
					[3, 35, 568, 0, 1],
					[3, 35, 569, 0, 1],
					[3, 35, 570, 0, 0],
					[3, 35, 571, 0, 0],
					[3, 35, 572, 0, 0],
					[3, 35, 573, 0, 0],
					[3, 35, 574, 0, 0],
					[3, 35, 575, 0, 0],
					[3, 35, 577, 0, 0],
					[3, 35, 578, 0, 0],
					[3, 37, 579, 0, 0],
					[3, 37, 580, 0, 0],
					[3, 37, 581, 0, 0],
					[3, 37, 582, 0, 0],
					[3, 37, 583, 0, 0],
					[3, 37, 584, 0, 0],
					[3, 37, 589, 0, 0],
					[3, 37, 590, 0, 0],
					[3, 37, 593, 0, 1],
					[3, 37, 594, 0, 1],
					[3, 37, 595, 0, 0],
					[3, 40, 597, 0, 0],
					[3, 40, 598, 0, 0],
					[3, 40, 599, 0, 1],
					[3, 40, 600, 0, 1],
					[3, 40, 601, 0, 0],
					[3, 40, 602, 0, 1],
					[3, 40, 603, 0, 0],
					[3, 41, 604, 0, 0],
					[3, 41, 605, 0, 0],
					[3, 41, 606, 0, 0],
					[3, 41, 607, 0, 0],
					[3, 41, 608, 0, 0],
					[3, 41, 609, 0, 0],
					[3, 41, 610, 0, 1],
					[3, 41, 611, 0, 1],
					[3, 41, 612, 0, 0],
					[3, 41, 613, 0, 0],
					[3, 42, 614, 0, 0],
					[3, 42, 615, 0, 0],
					[3, 42, 616, 0, 0],
					[3, 42, 617, 0, 0],
					[3, 42, 618, 0, 0],
					[3, 42, 619, 0, 0],
					[3, 42, 620, 0, 0],
					[3, 42, 621, 0, 0],
					[3, 42, 623, 0, 0],
					[3, 42, 624, 0, 0],
					[3, 42, 625, 0, 1],
					[3, 42, 626, 0, 1],
					[3, 42, 627, 0, 0],
					[3, 42, 628, 0, 0],
					[3, 43, 629, 0, 0],
					[3, 43, 630, 0, 0],
					[3, 43, 631, 0, 0],
					[3, 43, 632, 0, 0],
					[3, 43, 633, 0, 0],
					[3, 43, 634, 0, 0],
					[3, 43, 635, 0, 0],
					[3, 43, 636, 0, 0],
					[3, 43, 637, 0, 0],
					[3, 43, 638, 0, 0],
					[3, 43, 639, 0, 0],
					[3, 43, 640, 0, 0],
					[3, 43, 641, 0, 0],
					[3, 43, 642, 0, 1],
					[3, 43, 643, 0, 1],
					[3, 43, 644, 0, 0],
					[3, 43, 645, 0, 0],
					[3, 45, 646, 0, 0],
					[3, 45, 647, 0, 1],
					[3, 45, 648, 0, 1],
					[3, 45, 649, 0, 0],
					[3, 45, 650, 0, 0],
					[3, 29, 652, 0, 0],
					[3, 29, 689, 0, 0],
					[3, 42, 690, 0, 0],
					[3, 40, 691, 0, 0],
					[3, 29, 692, 0, 0],
					[3, 29, 693, 0, 0],
					[3, 29, 694, 0, 0],
					[3, 29, 699, 0, 0],
					[3, 6, 700, 0, 1],
					[3, 4, 701, 0, 1],
					[3, 29, 703, 0, 0],
					[3, 18, 708, 0, 0],
					[3, 40, 710, 0, 0],
					[3, 40, 711, 0, 0],
					[3, 29, 716, 0, 0],
					[3, 29, 719, 0, 0],
					[3, 13, 720, 0, 0],
					[3, 29, 721, 0, 0],
					[3, 29, 722, 0, 0],
					[3, 4, 725, 0, 1],
					[3, 6, 726, 0, 1],
					[3, 7, 727, 0, 1],
					[3, 9, 729, 0, 1],
					[3, 13, 730, 0, 1],
					[3, 18, 734, 0, 1],
					[3, 33, 741, 0, 1],
					[3, 34, 742, 0, 1],
					[3, 35, 743, 0, 1],
					[3, 37, 744, 0, 1],
					[3, 40, 745, 0, 1],
					[3, 41, 746, 0, 1],
					[3, 42, 747, 0, 1],
					[3, 43, 748, 0, 1],
					[3, 45, 749, 0, 1],
					[3, 51, 780, 0, 0],
					[3, 51, 781, 0, 0],
					[3, 51, 782, 0, 0],
					[3, 51, 783, 0, 0],
					[3, 51, 784, 0, 0],
					[3, 51, 785, 0, 0],
					[3, 51, 786, 0, 0],
					[3, 51, 787, 0, 0],
					[3, 51, 788, 0, 0],
					[3, 51, 791, 0, 1],
					[3, 51, 792, 0, 1],
					[3, 13, 814, 0, 0],
					[3, 42, 816, 0, 0],
					[3, 37, 818, 0, 0],
					[3, 34, 820, 0, 0],
					[3, 43, 826, 0, 0],
					[3, 54, 842, 0, 0],
					[3, 54, 843, 0, 0],
					[3, 54, 844, 0, 0],
					[3, 54, 845, 0, 0],
					[3, 54, 846, 0, 0],
					[3, 54, 847, 0, 0],
					[3, 54, 848, 0, 0],
					[3, 54, 849, 0, 0],
					[3, 54, 850, 0, 0],
					[3, 54, 851, 0, 0],
					[3, 54, 852, 0, 0],
					[3, 54, 853, 0, 0],
					[3, 54, 854, 0, 0],
					[3, 54, 855, 0, 0],
					[3, 54, 856, 0, 0],
					[3, 54, 857, 0, 0],
					[3, 54, 858, 0, 1],
					[3, 54, 859, 0, 1],
					[3, 54, 860, 0, 0],
					[3, 54, 861, 0, 1],
					[3, 54, 862, 0, 1],
					[3, 54, 863, 0, 0],
					[3, 54, 864, 0, 0],
					[3, 8, 865, 0, 0],
					[3, 57, 866, 0, 0],
					[3, 57, 867, 0, 0],
					[3, 57, 868, 0, 0],
					[3, 57, 870, 0, 0],
					[3, 57, 871, 0, 0],
					[3, 57, 872, 0, 0],
					[3, 57, 873, 0, 0],
					[3, 57, 874, 0, 0],
					[3, 57, 875, 0, 1],
					[3, 57, 876, 0, 1],
					[3, 57, 877, 0, 0],
					[3, 57, 878, 0, 0],
					[3, 58, 879, 0, 0],
					[3, 58, 880, 0, 0],
					[3, 58, 881, 0, 0],
					[3, 58, 883, 0, 0],
					[3, 58, 884, 0, 0],
					[3, 58, 885, 0, 0],
					[3, 58, 886, 0, 0],
					[3, 58, 887, 0, 0],
					[3, 58, 888, 0, 1],
					[3, 58, 889, 0, 1],
					[3, 58, 890, 0, 0],
					[3, 58, 891, 0, 0],
					[3, 59, 892, 0, 0],
					[3, 59, 893, 0, 0],
					[3, 59, 894, 0, 0],
					[3, 59, 895, 0, 0],
					[3, 59, 896, 0, 0],
					[3, 59, 898, 0, 0],
					[3, 59, 899, 0, 0],
					[3, 59, 900, 0, 0],
					[3, 59, 901, 0, 0],
					[3, 59, 902, 0, 1],
					[3, 59, 903, 0, 1],
					[3, 59, 904, 0, 0],
					[3, 59, 905, 0, 0],
					[3, 60, 927, 0, 0],
					[3, 60, 928, 0, 0],
					[3, 60, 929, 0, 0],
					[3, 60, 930, 0, 0],
					[3, 60, 931, 0, 0],
					[3, 60, 932, 0, 0],
					[3, 60, 933, 0, 0],
					[3, 60, 934, 0, 1],
					[3, 60, 935, 0, 1],
					[3, 60, 936, 0, 0],
					[3, 7, 938, 0, 0],
					[3, 7, 940, 0, 0],
					[3, 61, 941, 0, 0],
					[3, 61, 942, 0, 0],
					[3, 61, 943, 0, 0],
					[3, 61, 944, 0, 0],
					[3, 61, 945, 0, 0],
					[3, 61, 946, 0, 0],
					[3, 61, 947, 0, 0],
					[3, 61, 948, 0, 0],
					[3, 61, 949, 0, 0],
					[3, 61, 950, 0, 0],
					[3, 61, 951, 0, 1],
					[3, 61, 952, 0, 1],
					[3, 61, 953, 0, 0],
					[3, 61, 954, 0, 0],
					[3, 61, 955, 0, 0],
					[3, 61, 956, 0, 0],
					[3, 61, 957, 0, 0],
					[3, 61, 958, 0, 0],
					[3, 61, 959, 0, 0],
					[3, 61, 960, 0, 0],
					[3, 61, 961, 0, 0],
					[3, 61, 962, 0, 0],
					[3, 61, 963, 0, 0],
					[3, 61, 964, 0, 0],
					[3, 61, 965, 0, 0],
					[3, 61, 966, 0, 0],
					[3, 7, 967, 0, 0],
					[3, 7, 968, 0, 0],
					[3, 7, 969, 0, 0],
					[3, 6, 970, 0, 0],
					[3, 6, 971, 0, 0],
					[3, 6, 972, 0, 0],
					[3, 18, 973, 0, 0],
					[3, 18, 974, 0, 0],
					[3, 18, 975, 0, 0],
					[3, 43, 1044, 0, 0],
					[3, 29, 1045, 0, 0],
					[3, 13, 1049, 0, 0],
					[3, 7, 1065, 0, 0],
					[3, 7, 1066, 0, 0],
					[3, 7, 1067, 0, 0],
					[3, 7, 1068, 0, 0],
					[3, 7, 1069, 0, 0],
					[3, 7, 1070, 0, 0],
					[3, 7, 1071, 0, 0],
					[3, 7, 1072, 0, 0],
					[3, 4, 1073, 0, 0],
					[3, 4, 1074, 0, 0],
					[3, 4, 1075, 0, 0],
					[3, 4, 1076, 0, 0],
					[3, 4, 1077, 0, 0],
					[3, 4, 1078, 0, 0],
					[3, 4, 1079, 0, 0],
					[3, 4, 1080, 0, 0],
					[3, 4, 1081, 0, 0],
					[3, 4, 1082, 0, 0],
					[3, 4, 1083, 0, 0],
					[3, 4, 1084, 0, 0],
					[3, 4, 1085, 0, 0],
					[3, 4, 1086, 0, 0],
					[3, 4, 1087, 0, 0],
					[3, 4, 1088, 0, 0],
					[3, 6, 1089, 0, 0],
					[3, 6, 1090, 0, 0],
					[3, 6, 1091, 0, 0],
					[3, 6, 1092, 0, 0],
					[3, 6, 1093, 0, 0],
					[3, 6, 1094, 0, 0],
					[3, 6, 1095, 0, 0],
					[3, 6, 1096, 0, 0],
					[3, 6, 1097, 0, 0],
					[3, 6, 1098, 0, 0],
					[3, 6, 1099, 0, 0],
					[3, 6, 1100, 0, 0],
					[3, 6, 1101, 0, 0],
					[3, 6, 1102, 0, 0],
					[3, 6, 1103, 0, 0],
					[3, 6, 1104, 0, 0],
					[3, 18, 1105, 0, 0],
					[3, 18, 1106, 0, 0],
					[3, 18, 1107, 0, 0],
					[3, 18, 1108, 0, 0],
					[3, 18, 1109, 0, 0],
					[3, 18, 1110, 0, 0],
					[3, 18, 1111, 0, 0],
					[3, 18, 1112, 0, 0],
					[3, 18, 1113, 0, 0],
					[3, 18, 1114, 0, 0],
					[3, 18, 1115, 0, 0],
					[3, 18, 1116, 0, 0],
					[3, 18, 1117, 0, 0],
					[3, 18, 1118, 0, 0],
					[3, 18, 1119, 0, 0],
					[3, 18, 1120, 0, 0],
					[3, 18, 1121, 0, 0],
					[3, 18, 1122, 0, 0],
					[3, 18, 1123, 0, 0],
					[3, 18, 1124, 0, 0],
					[3, 18, 1125, 0, 0],
					[3, 18, 1126, 0, 0],
					[3, 18, 1127, 0, 0],
					[3, 18, 1128, 0, 0],
					[3, 6, 1160, 0, 0],
					[3, 6, 1161, 0, 0],
					[3, 6, 1162, 0, 0],
					[3, 6, 1163, 0, 0],
					[3, 6, 1164, 0, 0],
					[3, 6, 1165, 0, 0],
					[3, 6, 1166, 0, 0],
					[3, 6, 1167, 0, 0],
					[3, 7, 1276, 0, 0],
					[3, 6, 1277, 0, 0],
					[3, 4, 1278, 0, 0],
					[3, 18, 1279, 0, 0],
					[3, 6, 1285, 0, 0],
					[3, 6, 1286, 0, 0],
					[3, 6, 1287, 0, 0],
					[3, 6, 1288, 0, 0],
					[3, 6, 1289, 0, 0],
					[3, 6, 1290, 0, 0],
					[3, 7, 1291, 0, 0],
					[3, 7, 1292, 0, 0],
					[3, 4, 1293, 0, 0],
					[3, 4, 1294, 0, 0],
					[3, 4, 1295, 0, 0],
					[3, 4, 1296, 0, 0],
					[3, 37, 1314, 0, 0],
					[3, 59, 1315, 0, 0],
					[3, 57, 1316, 0, 0],
					[3, 58, 1317, 0, 0],
					[3, 42, 1318, 0, 0],
					[3, 42, 1319, 0, 0],
					[3, 42, 1320, 0, 0],
					[3, 7, 1321, 0, 0],
					[3, 29, 1322, 0, 0],
					[3, 37, 1323, 0, 0],
					[3, 37, 1324, 0, 0],
					[3, 51, 1326, 0, 0],
					[3, 51, 1327, 0, 0],
					[3, 7, 1329, 0, 0],
					[3, 6, 1330, 0, 0],
					[3, 6, 1331, 0, 0],
					[3, 4, 1332, 0, 0],
					[3, 18, 1333, 0, 0],
					[3, 43, 1340, 0, 0],
					[3, 13, 1341, 0, 0],
					[3, 58, 1344, 0, 0],
					[3, 4, 1368, 0, 0],
					[3, 6, 1369, 0, 0],
					[3, 6, 1375, 0, 0],
					[3, 26, 1378, 0, 0],
					[3, 43, 1382, 0, 0],
					[3, 13, 1384, 0, 0],
					[3, 4, 1386, 0, 0],
					[3, 4, 1391, 0, 0],
					[3, 14, 1393, 0, 0],
					[3, 35, 1395, 0, 0],
					[3, 7, 1397, 0, 0],
					[3, 8, 1399, 0, 0],
					[3, 9, 1401, 0, 0],
					[3, 15, 1403, 0, 0],
					[3, 16, 1405, 0, 0],
					[3, 18, 1407, 0, 0],
					[3, 19, 1409, 0, 0],
					[3, 34, 1419, 0, 0],
					[3, 37, 1421, 0, 0],
					[3, 41, 1423, 0, 0],
					[3, 42, 1425, 0, 0],
					[3, 51, 1431, 0, 0],
					[3, 54, 1433, 0, 0],
					[3, 57, 1435, 0, 0],
					[3, 58, 1437, 0, 0],
					[3, 59, 1439, 0, 0],
					[3, 60, 1441, 0, 0],
					[3, 61, 1443, 0, 0],
					[3, 4, 1449, 0, 0],
					[3, 6, 1450, 0, 0],
					[3, 7, 1451, 0, 0],
					[3, 8, 1452, 0, 0],
					[3, 9, 1453, 0, 0],
					[3, 13, 1454, 0, 0],
					[3, 14, 1455, 0, 0],
					[3, 15, 1456, 0, 0],
					[3, 16, 1457, 0, 0],
					[3, 18, 1458, 0, 0],
					[3, 19, 1459, 0, 0],
					[3, 26, 1464, 0, 0],
					[3, 34, 1465, 0, 0],
					[3, 35, 1466, 0, 0],
					[3, 37, 1467, 0, 0],
					[3, 41, 1468, 0, 0],
					[3, 42, 1469, 0, 0],
					[3, 43, 1470, 0, 0],
					[3, 51, 1473, 0, 0],
					[3, 54, 1474, 0, 0],
					[3, 57, 1475, 0, 0],
					[3, 58, 1476, 0, 0],
					[3, 59, 1477, 0, 0],
					[3, 60, 1478, 0, 0],
					[3, 61, 1479, 0, 0],
					[3, 13, 1482, 0, 0],
					[3, 13, 1483, 0, 0],
					[3, 37, 1484, 0, 0],
					[3, 58, 1485, 0, 0],
					[3, 7, 1486, 0, 0],
					[3, 4, 1487, 0, 0],
					[3, 6, 1488, 0, 0],
					[3, 18, 1489, 0, 0],
					[3, 4, 1494, 0, 0],
					[3, 6, 1495, 0, 0],
					[3, 18, 1496, 0, 0],
					[3, 18, 1501, 0, 0],
					[3, 6, 1502, 0, 0],
					[3, 4, 1503, 0, 0],
					[3, 74, 1504, 0, 0],
					[3, 74, 1505, 0, 0],
					[3, 74, 1506, 0, 0],
					[3, 74, 1507, 0, 0],
					[3, 74, 1508, 0, 0],
					[3, 74, 1509, 0, 0],
					[3, 74, 1511, 0, 0],
					[3, 74, 1512, 0, 0],
					[3, 74, 1513, 0, 0],
					[3, 74, 1514, 0, 0],
					[3, 74, 1515, 0, 0],
					[3, 74, 1516, 0, 0],
					[3, 74, 1517, 0, 0],
					[3, 74, 1518, 0, 0],
					[3, 74, 1519, 0, 0],
					[3, 74, 1520, 0, 0],
					[3, 74, 1521, 0, 0],
					[3, 74, 1522, 0, 0],
					[3, 4, 1524, 0, 0],
					[3, 6, 1525, 0, 0],
					[3, 7, 1526, 0, 0],
					[3, 8, 1527, 0, 0],
					[3, 9, 1528, 0, 0],
					[3, 13, 1529, 0, 0],
					[3, 14, 1530, 0, 0],
					[3, 15, 1531, 0, 0],
					[3, 18, 1532, 0, 0],
					[3, 19, 1533, 0, 0],
					[3, 26, 1538, 0, 0],
					[3, 34, 1539, 0, 0],
					[3, 35, 1540, 0, 0],
					[3, 37, 1541, 0, 0],
					[3, 41, 1542, 0, 0],
					[3, 42, 1543, 0, 0],
					[3, 43, 1544, 0, 0],
					[3, 51, 1547, 0, 0],
					[3, 54, 1548, 0, 0],
					[3, 57, 1549, 0, 0],
					[3, 58, 1550, 0, 0],
					[3, 59, 1551, 0, 0],
					[3, 60, 1552, 0, 0],
					[3, 61, 1553, 0, 0],
					[3, 74, 1556, 0, 0],
					[3, 75, 1557, 0, 0],
					[3, 75, 1558, 0, 0],
					[3, 75, 1559, 0, 0],
					[3, 75, 1560, 0, 0],
					[3, 75, 1561, 0, 0],
					[3, 75, 1562, 0, 0],
					[3, 75, 1563, 0, 0],
					[3, 75, 1564, 0, 0],
					[3, 75, 1566, 0, 0],
					[3, 75, 1567, 0, 0],
					[3, 75, 1568, 0, 0],
					[3, 75, 1569, 0, 0],
					[3, 51, 1600, 0, 0],
					[3, 9, 1603, 0, 0],
					[3, 16, 1604, 0, 0],
					[3, 78, 1605, 0, 0],
					[3, 78, 1606, 0, 0],
					[3, 78, 1607, 0, 0],
					[3, 78, 1608, 0, 0],
					[3, 78, 1609, 0, 0],
					[3, 78, 1610, 0, 0],
					[3, 78, 1611, 0, 0],
					[3, 78, 1612, 0, 0],
					[3, 78, 1613, 0, 0],
					[3, 78, 1615, 0, 0],
					[3, 79, 1616, 0, 0],
					[3, 79, 1617, 0, 0],
					[3, 79, 1618, 0, 0],
					[3, 79, 1619, 0, 0],
					[3, 79, 1620, 0, 0],
					[3, 79, 1621, 0, 0],
					[3, 79, 1622, 0, 0],
					[3, 79, 1625, 0, 0],
					[3, 79, 1626, 0, 0],
					[3, 79, 1627, 0, 0],
					[3, 79, 1628, 0, 0],
					[3, 6, 1630, 0, 0],
					[3, 80, 1632, 0, 0],
					[3, 80, 1633, 0, 0],
					[3, 80, 1634, 0, 0],
					[3, 80, 1635, 0, 0],
					[3, 80, 1636, 0, 0],
					[3, 80, 1637, 0, 0],
					[3, 80, 1638, 0, 0],
					[3, 80, 1640, 0, 0],
					[3, 80, 1641, 0, 0],
					[3, 80, 1642, 0, 0],
					[3, 80, 1643, 0, 0],
					[3, 80, 1644, 0, 0],
					[3, 81, 1645, 0, 0],
					[3, 81, 1646, 0, 0],
					[3, 81, 1647, 0, 0],
					[3, 81, 1648, 0, 0],
					[3, 81, 1649, 0, 0],
					[3, 81, 1650, 0, 0],
					[3, 81, 1651, 0, 0],
					[3, 81, 1652, 0, 0],
					[3, 81, 1653, 0, 0],
					[3, 81, 1655, 0, 0],
					[3, 81, 1656, 0, 0],
					[3, 81, 1657, 0, 0],
					[3, 81, 1658, 0, 0],
					[3, 81, 1659, 0, 0],
					[3, 81, 1660, 0, 0],
					[3, 81, 1661, 0, 0],
					[3, 81, 1666, 0, 0],
					[3, 82, 1667, 0, 0],
					[3, 82, 1668, 0, 0],
					[3, 82, 1669, 0, 0],
					[3, 82, 1670, 0, 0],
					[3, 82, 1671, 0, 0],
					[3, 82, 1672, 0, 0],
					[3, 82, 1673, 0, 0],
					[3, 82, 1674, 0, 0],
					[3, 82, 1675, 0, 0],
					[3, 82, 1677, 0, 0],
					[3, 82, 1678, 0, 0],
					[3, 82, 1679, 0, 0],
					[3, 82, 1680, 0, 0],
					[3, 82, 1681, 0, 0],
					[3, 82, 1682, 0, 0],
					[3, 82, 1683, 0, 0],
					[3, 82, 1688, 0, 0],
					[3, 18, 1689, 0, 0],
					[3, 18, 1690, 0, 0],
					[3, 18, 1691, 0, 0],
					[3, 18, 1692, 0, 0],
					[3, 18, 1693, 0, 0],
					[3, 18, 1694, 0, 0],
					[3, 83, 1708, 0, 0],
					[3, 83, 1709, 0, 0],
					[3, 83, 1710, 0, 0],
					[3, 83, 1711, 0, 0],
					[3, 83, 1712, 0, 0],
					[3, 83, 1713, 0, 0],
					[3, 16, 1714, 0, 0],
					[3, 9, 1715, 0, 0],
					[3, 84, 1716, 0, 0],
					[3, 84, 1717, 0, 0],
					[3, 84, 1718, 0, 0],
					[3, 84, 1719, 0, 0],
					[3, 84, 1720, 0, 0],
					[3, 84, 1721, 0, 0],
					[3, 84, 1722, 0, 0],
					[3, 84, 1723, 0, 0],
					[3, 84, 1724, 0, 0],
					[3, 84, 1725, 0, 0],
					[3, 84, 1726, 0, 0],
					[3, 84, 1727, 0, 0],
					[3, 84, 1728, 0, 0],
					[3, 84, 1729, 0, 0],
					[3, 84, 1730, 0, 0],
					[3, 84, 1731, 0, 0],
					[3, 84, 1732, 0, 0],
					[3, 84, 1733, 0, 0],
					[3, 84, 1735, 0, 0],
					[3, 7, 1736, 0, 0],
					[3, 7, 1737, 0, 0],
					[3, 6, 1738, 0, 0],
					[3, 29, 1740, 0, 0],
					[3, 41, 1741, 0, 0],
					[3, 42, 1742, 0, 0],
					[3, 41, 1743, 0, 0],
					[3, 4, 1744, 0, 0],
					[3, 54, 1745, 0, 0],
					[3, 4, 1746, 0, 0],
					[3, 6, 1747, 0, 0],
					[3, 6, 1756, 0, 0],
					[3, 6, 1757, 0, 0],
					[3, 6, 1758, 0, 0],
					[3, 14, 1759, 0, 0],
					[3, 16, 1761, 0, 0],
					[3, 9, 1762, 0, 0],
					[3, 15, 1763, 0, 0],
					[3, 19, 1764, 0, 0],
					[3, 85, 1765, 0, 0],
					[3, 85, 1766, 0, 0],
					[3, 85, 1767, 0, 0],
					[3, 85, 1768, 0, 0],
					[3, 85, 1769, 0, 0],
					[3, 85, 1770, 0, 0],
					[3, 85, 1771, 0, 0],
					[3, 85, 1772, 0, 0],
					[3, 85, 1773, 0, 0],
					[3, 85, 1774, 0, 0],
					[3, 85, 1775, 0, 0],
					[3, 85, 1776, 0, 0],
					[3, 85, 1777, 0, 0],
					[3, 85, 1778, 0, 0],
					[3, 86, 1779, 0, 0],
					[3, 86, 1780, 0, 0],
					[3, 86, 1781, 0, 0],
					[3, 86, 1782, 0, 0],
					[3, 86, 1783, 0, 0],
					[3, 86, 1784, 0, 0],
					[3, 86, 1785, 0, 0],
					[3, 86, 1786, 0, 0],
					[3, 86, 1787, 0, 0],
					[3, 86, 1788, 0, 0],
					[3, 87, 1789, 0, 0],
					[3, 87, 1790, 0, 0],
					[3, 87, 1791, 0, 0],
					[3, 87, 1792, 0, 0],
					[3, 87, 1793, 0, 0],
					[3, 87, 1794, 0, 0],
					[3, 87, 1795, 0, 0],
					[3, 87, 1796, 0, 0],
					[3, 87, 1797, 0, 0],
					[3, 87, 1798, 0, 0],
					[3, 87, 1799, 0, 0],
					[3, 87, 1800, 0, 0],
					[3, 87, 1801, 0, 0],
					[3, 87, 1802, 0, 0],
					[3, 87, 1803, 0, 0],
					[3, 88, 1804, 0, 0],
					[3, 88, 1805, 0, 0],
					[3, 88, 1806, 0, 0],
					[3, 88, 1807, 0, 0],
					[3, 88, 1808, 0, 0],
					[3, 88, 1809, 0, 0],
					[3, 88, 1810, 0, 0],
					[3, 88, 1811, 0, 0],
					[3, 88, 1812, 0, 0],
					[3, 88, 1813, 0, 0],
					[3, 88, 1814, 0, 0],
					[3, 88, 1815, 0, 0],
					[3, 88, 1816, 0, 0],
					[3, 88, 1817, 0, 0],
					[3, 88, 1818, 0, 0],
					[3, 89, 1819, 0, 0],
					[3, 89, 1820, 0, 0],
					[3, 89, 1821, 0, 0],
					[3, 89, 1822, 0, 0],
					[3, 89, 1823, 0, 0],
					[3, 89, 1824, 0, 0],
					[3, 89, 1825, 0, 0],
					[3, 89, 1826, 0, 0],
					[3, 89, 1827, 0, 0],
					[3, 89, 1828, 0, 0],
					[3, 89, 1829, 0, 0],
					[3, 89, 1830, 0, 0],
					[3, 89, 1831, 0, 0],
					[3, 89, 1832, 0, 0],
					[3, 89, 1833, 0, 0],
					[3, 90, 1834, 0, 0],
					[3, 90, 1835, 0, 0],
					[3, 90, 1836, 0, 0],
					[3, 90, 1837, 0, 0],
					[3, 90, 1838, 0, 0],
					[3, 90, 1839, 0, 0],
					[3, 90, 1840, 0, 0],
					[3, 90, 1841, 0, 0],
					[3, 90, 1842, 0, 0],
					[3, 90, 1843, 0, 0],
					[3, 90, 1844, 0, 0],
					[3, 90, 1845, 0, 0],
					[3, 90, 1846, 0, 0],
					[3, 90, 1847, 0, 0],
					[3, 90, 1848, 0, 0],
					[3, 90, 1849, 0, 0],
					[3, 90, 1850, 0, 0],
					[3, 90, 1851, 0, 0],
					[3, 91, 1852, 0, 0],
					[3, 91, 1853, 0, 0],
					[3, 91, 1854, 0, 0],
					[3, 91, 1855, 0, 0],
					[3, 91, 1856, 0, 0],
					[3, 91, 1857, 0, 0],
					[3, 91, 1858, 0, 0],
					[3, 91, 1859, 0, 0],
					[3, 91, 1860, 0, 0],
					[3, 91, 1861, 0, 0],
					[3, 91, 1862, 0, 0],
					[3, 91, 1863, 0, 0],
					[3, 91, 1864, 0, 0],
					[3, 91, 1865, 0, 0],
					[3, 91, 1866, 0, 0],
					[3, 91, 1867, 0, 0],
					[3, 91, 1868, 0, 0],
					[3, 91, 1869, 0, 0],
					[3, 91, 1870, 0, 0],
					[3, 91, 1871, 0, 0],
					[3, 91, 1872, 0, 0],
					[3, 91, 1873, 0, 0],
					[3, 91, 1874, 0, 0],
					[3, 91, 1875, 0, 0],
					[3, 89, 1876, 0, 0],
					[3, 89, 1877, 0, 0],
					[3, 89, 1878, 0, 0],
					[3, 89, 1879, 0, 0],
					[3, 89, 1880, 0, 0],
					[3, 89, 1881, 0, 0],
					[3, 89, 1882, 0, 0],
					[3, 89, 1883, 0, 0],
					[3, 89, 1884, 0, 0],
					[3, 89, 1885, 0, 0],
					[3, 89, 1886, 0, 0],
					[3, 89, 1887, 0, 0],
					[3, 89, 1888, 0, 0],
					[3, 89, 1889, 0, 0],
					[3, 89, 1890, 0, 0],
					[3, 89, 1891, 0, 0],
					[3, 89, 1892, 0, 0],
					[3, 89, 1893, 0, 0],
					[3, 89, 1894, 0, 0],
					[3, 89, 1895, 0, 0],
					[3, 89, 1896, 0, 0],
					[3, 89, 1897, 0, 0],
					[3, 90, 1898, 0, 0],
					[3, 90, 1899, 0, 0],
					[3, 90, 1900, 0, 0],
					[3, 90, 1901, 0, 0],
					[3, 90, 1902, 0, 0],
					[3, 90, 1903, 0, 0],
					[3, 90, 1904, 0, 0],
					[3, 90, 1905, 0, 0],
					[3, 90, 1906, 0, 0],
					[3, 90, 1907, 0, 0],
					[3, 90, 1908, 0, 0],
					[3, 90, 1909, 0, 0],
					[3, 90, 1910, 0, 0],
					[3, 90, 1911, 0, 0],
					[3, 90, 1912, 0, 0],
					[3, 90, 1913, 0, 0],
					[3, 90, 1914, 0, 0],
					[3, 90, 1915, 0, 0],
					[3, 90, 1916, 0, 0],
					[3, 90, 1917, 0, 0],
					[3, 90, 1918, 0, 0],
					[3, 90, 1919, 0, 0],
					[3, 91, 1920, 0, 0],
					[3, 91, 1921, 0, 0],
					[3, 91, 1922, 0, 0],
					[3, 91, 1923, 0, 0],
					[3, 91, 1924, 0, 0],
					[3, 91, 1925, 0, 0],
					[3, 91, 1926, 0, 0],
					[3, 91, 1927, 0, 0],
					[3, 91, 1928, 0, 0],
					[3, 91, 1929, 0, 0],
					[3, 91, 1930, 0, 0],
					[3, 91, 1931, 0, 0],
					[3, 91, 1932, 0, 0],
					[3, 91, 1933, 0, 0],
					[3, 91, 1934, 0, 0],
					[3, 91, 1935, 0, 0],
					[3, 91, 1936, 0, 0],
					[3, 91, 1937, 0, 0],
					[3, 91, 1938, 0, 0],
					[3, 91, 1939, 0, 0],
					[3, 91, 1940, 0, 0],
					[3, 91, 1941, 0, 0],
					[3, 89, 1942, 0, 0],
					[3, 90, 1943, 0, 0],
					[3, 91, 1944, 0, 0],
					[3, 6, 1945, 0, 1],
					[3, 6, 1946, 0, 1],
					[3, 37, 1947, 0, 0],
					[3, 92, 1948, 0, 0],
					[3, 92, 1949, 0, 0],
					[3, 92, 1950, 0, 0],
					[3, 92, 1951, 0, 0],
					[3, 92, 1952, 0, 0],
					[3, 92, 1953, 0, 0],
					[3, 92, 1954, 0, 0],
					[3, 92, 1955, 0, 0],
					[3, 92, 1956, 0, 0],
					[3, 92, 1957, 0, 0],
					[3, 92, 1958, 0, 0],
					[3, 92, 1959, 0, 0],
					[3, 92, 1960, 0, 0],
					[3, 92, 1961, 0, 0],
					[3, 92, 1962, 0, 0],
					[3, 92, 1963, 0, 0],
					[3, 92, 1964, 0, 0],
					[3, 92, 1965, 0, 0],
					[3, 92, 1966, 0, 0],
					[3, 92, 1967, 0, 0],
					[3, 93, 1968, 0, 0],
					[3, 93, 1969, 0, 0],
					[3, 93, 1970, 0, 0],
					[3, 93, 1971, 0, 0],
					[3, 93, 1972, 0, 0],
					[3, 93, 1973, 0, 0],
					[3, 93, 1974, 0, 0],
					[3, 93, 1975, 0, 0],
					[3, 93, 1976, 0, 0],
					[3, 93, 1977, 0, 0],
					[3, 93, 1978, 0, 0],
					[3, 93, 1979, 0, 0],
					[3, 93, 1980, 0, 0],
					[3, 93, 1981, 0, 0],
					[3, 93, 1982, 0, 0],
					[3, 93, 1983, 0, 0],
					[3, 93, 1984, 0, 0],
					[3, 93, 1985, 0, 0],
					[3, 93, 1986, 0, 0],
					[3, 93, 1987, 0, 0],
					[3, 59, 1988, 0, 0],
					[3, 37, 1989, 0, 0],
					[3, 57, 1990, 0, 0],
					[3, 58, 1991, 0, 0],
					[3, 9, 1992, 0, 0],
					[3, 16, 1993, 0, 0],
					[3, 51, 1994, 0, 0],
					[3, 51, 1995, 0, 0],
					[3, 51, 1996, 0, 0],
					[3, 4, 1997, 0, 0],
					[3, 7, 1998, 0, 0],
					[3, 18, 1999, 0, 0],
					[3, 92, 2000, 0, 0],
					[3, 93, 2001, 0, 0],
					[3, 61, 2002, 0, 0],
					[3, 86, 2003, 0, 0],
					[3, 26, 2004, 0, 0],
					[3, 41, 2005, 0, 0],
					[3, 85, 2006, 0, 0],
					[3, 87, 2007, 0, 0],
					[3, 88, 2008, 0, 0],
					[3, 89, 2009, 0, 0],
					[3, 90, 2010, 0, 0],
					[3, 91, 2011, 0, 0],
					[3, 92, 2012, 0, 0],
					[3, 93, 2013, 0, 0],
					[3, 4, 2014, 0, 0],
					[3, 6, 2015, 0, 0],
					[3, 7, 2016, 0, 0],
					[3, 13, 2017, 0, 0],
					[3, 18, 2018, 0, 0],
					[3, 26, 2019, 0, 0],
					[3, 34, 2020, 0, 0],
					[3, 43, 2021, 0, 0],
					[3, 61, 2022, 0, 0],
					[3, 85, 2023, 0, 0],
					[3, 86, 2024, 0, 0],
					[3, 87, 2025, 0, 0],
					[3, 88, 2026, 0, 0],
					[3, 89, 2027, 0, 0],
					[3, 90, 2028, 0, 0],
					[3, 91, 2029, 0, 0],
					[3, 92, 2030, 0, 0],
					[3, 93, 2031, 0, 0],
					[3, 94, 2032, 0, 0],
					[3, 94, 2033, 0, 0],
					[3, 94, 2034, 0, 0],
					[3, 94, 2035, 0, 0],
					[3, 94, 2036, 0, 0],
					[3, 94, 2037, 0, 0],
					[3, 94, 2038, 0, 0],
					[3, 94, 2039, 0, 0],
					[3, 95, 2040, 0, 0],
					[3, 95, 2041, 0, 0],
					[3, 95, 2042, 0, 0],
					[3, 95, 2043, 0, 0],
					[3, 95, 2044, 0, 0],
					[3, 95, 2046, 0, 0],
					[3, 95, 2047, 0, 0],
					[3, 95, 2048, 0, 0],
					[3, 95, 2049, 0, 0],
					[3, 95, 2050, 0, 0],
					[3, 95, 2051, 0, 0],
					[3, 95, 2052, 0, 0],
					[3, 95, 2053, 0, 0],
					[3, 95, 2054, 0, 0],
					[3, 95, 2055, 0, 0],
					[3, 95, 2056, 0, 0],
					[3, 95, 2057, 0, 0],
					[3, 95, 2058, 0, 0],
					[3, 95, 2059, 0, 0],
					[3, 95, 2060, 0, 0],
					[3, 95, 2061, 0, 0],
					[3, 95, 2062, 0, 0],
					[3, 95, 2063, 0, 0],
					[3, 95, 2064, 0, 0],
					[3, 95, 2065, 0, 0],
					[3, 95, 2066, 0, 0],
					[3, 95, 2067, 0, 0],
					[3, 95, 2068, 0, 0],
					[3, 95, 2069, 0, 0],
					[3, 95, 2070, 0, 0],
					[3, 95, 2071, 0, 0],
					[3, 95, 2072, 0, 0],
					[3, 95, 2073, 0, 0],
					[3, 95, 2074, 0, 0],
					[3, 96, 2075, 0, 0],
					[3, 96, 2076, 0, 0],
					[3, 96, 2077, 0, 0],
					[3, 96, 2078, 0, 0],
					[3, 96, 2079, 0, 0],
					[3, 96, 2080, 0, 0],
					[3, 96, 2081, 0, 0],
					[3, 96, 2082, 0, 0],
					[3, 97, 2083, 0, 0],
					[3, 97, 2084, 0, 0],
					[3, 97, 2085, 0, 0],
					[3, 97, 2086, 0, 0],
					[3, 97, 2087, 0, 0],
					[3, 97, 2088, 0, 0],
					[3, 97, 2089, 0, 0],
					[3, 97, 2090, 0, 0],
					[3, 97, 2091, 0, 0],
					[3, 97, 2092, 0, 0],
					[3, 97, 2093, 0, 0],
					[3, 86, 2094, 0, 0],
					[3, 86, 2095, 0, 0],
					[3, 86, 2096, 0, 0],
					[3, 86, 2097, 0, 0],
					[3, 86, 2098, 0, 0],
					[3, 86, 2099, 0, 0],
					[3, 86, 2100, 0, 0],
					[3, 86, 2101, 0, 0],
					[3, 86, 2102, 0, 0],
					[3, 88, 2103, 0, 0],
					[3, 88, 2104, 0, 0],
					[3, 88, 2105, 0, 0],
					[3, 89, 2106, 0, 0],
					[3, 89, 2107, 0, 0],
					[3, 89, 2108, 0, 0],
					[3, 89, 2109, 0, 0],
					[3, 89, 2110, 0, 0],
					[3, 90, 2111, 0, 0],
					[3, 90, 2112, 0, 0],
					[3, 90, 2113, 0, 0],
					[3, 90, 2114, 0, 0],
					[3, 90, 2115, 0, 0],
					[3, 98, 2116, 0, 0],
					[3, 98, 2117, 0, 0],
					[3, 98, 2118, 0, 0],
					[3, 98, 2119, 0, 0],
					[3, 98, 2120, 0, 0],
					[3, 98, 2121, 0, 0],
					[3, 99, 2122, 0, 0],
					[3, 99, 2123, 0, 0],
					[3, 99, 2124, 0, 0],
					[3, 99, 2125, 0, 0],
					[3, 99, 2126, 0, 0],
					[3, 99, 2127, 0, 0],
					[3, 99, 2128, 0, 0],
					[3, 99, 2129, 0, 0],
					[3, 99, 2130, 0, 0],
					[3, 99, 2131, 0, 0],
					[3, 99, 2132, 0, 0],
					[3, 99, 2133, 0, 0],
					[3, 99, 2134, 0, 0],
					[3, 99, 2135, 0, 0],
					[3, 99, 2136, 0, 0],
					[3, 99, 2137, 0, 0],
					[3, 99, 2138, 0, 0],
					[3, 99, 2139, 0, 0],
					[3, 99, 2140, 0, 0],
					[3, 99, 2141, 0, 0],
					[3, 99, 2142, 0, 0],
					[3, 99, 2143, 0, 0],
					[3, 99, 2144, 0, 0],
					[3, 99, 2145, 0, 0],
					[3, 99, 2146, 0, 0],
					[3, 99, 2147, 0, 0],
					[3, 99, 2148, 0, 0],
					[3, 99, 2149, 0, 0],
					[3, 99, 2150, 0, 0],
					[3, 99, 2151, 0, 0],
					[3, 99, 2152, 0, 0],
					[3, 99, 2153, 0, 0],
					[3, 99, 2154, 0, 0],
					[3, 99, 2155, 0, 0],
					[3, 100, 2156, 0, 0],
					[3, 100, 2157, 0, 0],
					[3, 100, 2158, 0, 0],
					[3, 100, 2159, 0, 0],
					[3, 100, 2160, 0, 0],
					[3, 100, 2161, 0, 0],
					[3, 100, 2162, 0, 0],
					[3, 101, 2163, 0, 0],
					[3, 101, 2164, 0, 0],
					[3, 101, 2165, 0, 0],
					[3, 101, 2166, 0, 0],
					[3, 101, 2167, 0, 0],
					[3, 101, 2168, 0, 0],
					[3, 101, 2169, 0, 0],
					[3, 98, 2170, 0, 0],
					[3, 102, 2171, 0, 0],
					[3, 102, 2172, 0, 0],
					[3, 102, 2173, 0, 0],
					[3, 102, 2174, 0, 0],
					[3, 102, 2175, 0, 0],
					[3, 102, 2176, 0, 0],
					[3, 102, 2177, 0, 0],
					[3, 98, 2178, 0, 0],
					[3, 100, 2179, 0, 0],
					[3, 101, 2180, 0, 0],
					[3, 102, 2181, 0, 0],
					[3, 103, 2182, 0, 0],
					[3, 103, 2183, 0, 0],
					[3, 103, 2184, 0, 0],
					[3, 103, 2185, 0, 0],
					[3, 103, 2186, 0, 0],
					[3, 103, 2187, 0, 0],
					[3, 103, 2188, 0, 0],
					[3, 98, 2189, 0, 0],
					[3, 98, 2190, 0, 0],
					[3, 98, 2191, 0, 0],
					[3, 98, 2192, 0, 0],
					[3, 100, 2193, 0, 0],
					[3, 100, 2194, 0, 0],
					[3, 100, 2195, 0, 0],
					[3, 100, 2196, 0, 0],
					[3, 101, 2197, 0, 0],
					[3, 101, 2198, 0, 0],
					[3, 101, 2199, 0, 0],
					[3, 102, 2200, 0, 0],
					[3, 102, 2201, 0, 0],
					[3, 102, 2202, 0, 0],
					[3, 103, 2203, 0, 0],
					[3, 103, 2204, 0, 0],
					[3, 103, 2205, 0, 0],
					[3, 103, 2206, 0, 0],
					[3, 103, 2207, 0, 0],
					[3, 14, 2208, 0, 0],
					[3, 98, 2209, 0, 0],
					[3, 96, 2210, 0, 0],
					[3, 97, 2211, 0, 0],
					[3, 97, 2212, 0, 0],
					[3, 97, 2213, 0, 0],
					[3, 97, 2214, 0, 0],
					[3, 97, 2215, 0, 0],
					[3, 97, 2216, 0, 0],
					[3, 97, 2217, 0, 0],
					[3, 97, 2218, 0, 0],
					[3, 97, 2219, 0, 0],
					[3, 97, 2220, 0, 0],
					[3, 97, 2221, 0, 0],
					[3, 104, 2222, 0, 0],
					[3, 104, 2223, 0, 0],
					[3, 104, 2224, 0, 0],
					[3, 104, 2225, 0, 0],
					[3, 104, 2226, 0, 0],
					[3, 104, 2227, 0, 0],
					[3, 104, 2228, 0, 0],
					[3, 104, 2229, 0, 0],
					[3, 104, 2230, 0, 0],
					[3, 104, 2231, 0, 0],
					[3, 104, 2232, 0, 0],
					[3, 104, 2233, 0, 0],
					[3, 104, 2234, 0, 0],
					[3, 105, 2235, 0, 0],
					[3, 105, 2236, 0, 0],
					[3, 105, 2237, 0, 0],
					[3, 105, 2238, 0, 0],
					[3, 105, 2239, 0, 0],
					[3, 105, 2240, 0, 0],
					[3, 105, 2241, 0, 0],
					[3, 105, 2242, 0, 0],
					[3, 105, 2243, 0, 0],
					[3, 105, 2244, 0, 0],
					[3, 105, 2245, 0, 0],
					[3, 106, 2246, 0, 0],
					[3, 106, 2247, 0, 0],
					[3, 106, 2248, 0, 0],
					[3, 106, 2249, 0, 0],
					[3, 106, 2250, 0, 0],
					[3, 106, 2251, 0, 0],
					[3, 106, 2252, 0, 0],
					[3, 106, 2253, 0, 0],
					[3, 106, 2254, 0, 0],
					[3, 106, 2255, 0, 0],
					[3, 106, 2256, 0, 0],
					[3, 106, 2257, 0, 0],
					[3, 106, 2258, 0, 0],
					[3, 104, 2259, 0, 0],
					[3, 106, 2260, 0, 0],
					[3, 104, 2261, 0, 0],
					[3, 106, 2262, 0, 0],
					[3, 96, 2263, 0, 0],
					[3, 96, 2264, 0, 0],
					[3, 96, 2265, 0, 0],
					[3, 96, 2266, 0, 0],
					[3, 58, 2267, 0, 0],
					[3, 37, 2268, 0, 0],
					[3, 14, 2269, 0, 0],
					[3, 107, 2270, 0, 0],
					[3, 107, 2271, 0, 0],
					[3, 107, 2272, 0, 0],
					[3, 107, 2273, 0, 0],
					[3, 107, 2274, 0, 0],
					[3, 107, 2275, 0, 0],
					[3, 107, 2276, 0, 0],
					[3, 107, 2277, 0, 0],
					[3, 107, 2278, 0, 0],
					[3, 107, 2279, 0, 0],
					[3, 107, 2280, 0, 0],
					[3, 107, 2281, 0, 0],
					[3, 107, 2282, 0, 0],
					[3, 107, 2283, 0, 0],
					[3, 107, 2284, 0, 0],
					[3, 107, 2285, 0, 0],
					[3, 107, 2286, 0, 0],
					[3, 107, 2287, 0, 0],
					[3, 107, 2288, 0, 0],
					[3, 107, 2289, 0, 0],
					[3, 107, 2290, 0, 0],
					[3, 107, 2291, 0, 0],
					[3, 107, 2292, 0, 0],
					[3, 107, 2293, 0, 0],
					[3, 107, 2294, 0, 0],
					[3, 107, 2295, 0, 0],
					[3, 107, 2296, 0, 0],
					[3, 107, 2297, 0, 0],
					[3, 107, 2298, 0, 0],
					[3, 107, 2299, 0, 0],
					[3, 107, 2300, 0, 0],
					[3, 107, 2301, 0, 0],
					[3, 107, 2302, 0, 0],
					[3, 107, 2303, 0, 0],
					[3, 107, 2304, 0, 0],
					[3, 108, 2305, 0, 0],
					[3, 108, 2306, 0, 0],
					[3, 108, 2307, 0, 0],
					[3, 108, 2308, 0, 0],
					[3, 108, 2309, 0, 0],
					[3, 108, 2310, 0, 0],
					[3, 108, 2311, 0, 0],
					[3, 108, 2312, 0, 0],
					[3, 108, 2313, 0, 0],
					[3, 108, 2314, 0, 0],
					[3, 108, 2315, 0, 0],
					[3, 108, 2316, 0, 0],
					[3, 108, 2317, 0, 0],
					[3, 108, 2318, 0, 0],
					[3, 109, 2319, 0, 0],
					[3, 109, 2320, 0, 0],
					[3, 109, 2321, 0, 0],
					[3, 109, 2322, 0, 0],
					[3, 109, 2323, 0, 0],
					[3, 109, 2324, 0, 0],
					[3, 109, 2325, 0, 0],
					[3, 109, 2326, 0, 0],
					[3, 109, 2327, 0, 0],
					[3, 109, 2328, 0, 0],
					[3, 109, 2329, 0, 0],
					[3, 109, 2330, 0, 0],
					[3, 109, 2331, 0, 0],
					[3, 13, 2332, 0, 0],
					[3, 13, 2333, 0, 0],
					[3, 14, 2334, 0, 0],
					[3, 35, 2335, 0, 0],
					[3, 37, 2336, 0, 0],
					[3, 58, 2337, 0, 0],
					[3, 90, 2344, 0, 0],
					[3, 100, 2345, 0, 0],
					[3, 6, 2346, 0, 0],
					[3, 13, 2347, 0, 0],
					[3, 13, 2348, 0, 0],
					[3, 4, 2349, 0, 0],
					[3, 6, 2350, 0, 0],
					[3, 7, 2351, 0, 0],
					[3, 92, 2352, 0, 0],
					[3, 18, 2353, 0, 0],
					[3, 93, 2354, 0, 0],
					[3, 83, 2355, 0, 0],
					[3, 83, 2356, 0, 0],
					[3, 14, 2357, 0, 0],
					[3, 89, 2358, 0, 0],
					[3, 92, 2381, 0, 0],
					[3, 61, 2382, 0, 0],
					[3, 61, 2383, 0, 0],
					[3, 61, 2384, 0, 0],
					[3, 86, 2385, 0, 0],
					[3, 43, 2386, 0, 0],
					[3, 111, 2387, 0, 0],
					[3, 111, 2388, 0, 0],
					[3, 111, 2389, 0, 0],
					[3, 111, 2390, 0, 0],
					[3, 111, 2391, 0, 0],
					[3, 111, 2392, 0, 0],
					[3, 111, 2393, 0, 0],
					[3, 111, 2395, 0, 0],
					[3, 111, 2396, 0, 0],
					[3, 111, 2397, 0, 0],
					[3, 111, 2398, 0, 0],
					[3, 111, 2399, 0, 0],
					[3, 111, 2400, 0, 0],
					[3, 6, 2401, 0, 0],
					[3, 95, 2402, 0, 0],
					[3, 95, 2403, 0, 0],
					[3, 86, 2404, 0, 0],
					[3, 6, 2405, 0, 0],
					[3, 4, 2406, 0, 0],
					[3, 7, 2407, 0, 0],
					[3, 18, 2408, 0, 0],
					[3, 14, 2409, 0, 0],
					[3, 86, 2410, 0, 0],
					[3, 97, 2411, 0, 0],
					[3, 26, 2412, 0, 0],
					[3, 16, 2413, 0, 0],
					[3, 13, 2414, 0, 0],
					[3, 105, 2415, 0, 0],
					[3, 35, 2416, 0, 0],
					[3, 106, 2417, 0, 0],
					[3, 98, 2418, 0, 0],
					[3, 54, 2419, 0, 0],
					[3, 108, 2420, 0, 0],
					[3, 100, 2421, 0, 0],
					[3, 109, 2422, 0, 0],
					[3, 9, 2423, 0, 0],
					[3, 43, 2424, 0, 0],
					[3, 101, 2425, 0, 0],
					[3, 102, 2426, 0, 0],
					[3, 37, 2427, 0, 0],
					[3, 61, 2428, 0, 0],
					[3, 103, 2429, 0, 0],
					[3, 15, 2430, 0, 0],
					[3, 74, 2431, 0, 0],
					[3, 104, 2432, 0, 0],
					[3, 34, 2433, 0, 0],
					[3, 59, 2434, 0, 0],
					[3, 51, 2435, 0, 0],
					[3, 19, 2436, 0, 0],
					[3, 41, 2437, 0, 0],
					[3, 8, 2438, 0, 0],
					[3, 58, 2439, 0, 0],
					[3, 84, 2440, 0, 0],
					[3, 60, 2441, 0, 0],
					[3, 57, 2442, 0, 0],
					[3, 83, 2443, 0, 0],
					[3, 42, 2444, 0, 0],
					[3, 75, 2445, 0, 0],
					[3, 78, 2446, 0, 0],
					[3, 79, 2447, 0, 0],
					[3, 80, 2448, 0, 0],
					[3, 81, 2449, 0, 0],
					[3, 107, 2450, 0, 0],
					[3, 82, 2451, 0, 0],
					[3, 99, 2452, 0, 0],
					[3, 94, 2453, 0, 0],
					[3, 95, 2454, 0, 0],
					[3, 96, 2455, 0, 0],
					[3, 111, 2456, 0, 0],
					[3, 85, 2458, 0, 0],
					[3, 87, 2459, 0, 0],
					[3, 89, 2460, 0, 0],
					[3, 91, 2461, 0, 0],
					[3, 93, 2462, 0, 0],
					[3, 88, 2463, 0, 0],
					[3, 90, 2464, 0, 0],
					[3, 92, 2465, 0, 0],
					[3, 33, 2466, 0, 0],
					[3, 35, 2467, 0, 0],
					[3, 29, 2468, 0, 0],
					[3, 29, 2469, 0, 0],
					[3, 29, 2470, 0, 0],
					[3, 112, 2471, 0, 0],
					[3, 112, 2472, 0, 0],
					[3, 112, 2473, 0, 0],
					[3, 112, 2474, 0, 0],
					[3, 112, 2475, 0, 0],
					[3, 112, 2476, 0, 0],
					[3, 112, 2477, 0, 0],
					[3, 112, 2478, 0, 0],
					[3, 112, 2479, 0, 0],
					[3, 112, 2480, 0, 0],
					[3, 112, 2481, 0, 0],
					[3, 112, 2482, 0, 0],
					[3, 112, 2483, 0, 0],
					[3, 86, 2484, 0, 0],
					[3, 113, 2485, 0, 0],
					[3, 113, 2486, 0, 0],
					[3, 113, 2487, 0, 0],
					[3, 113, 2488, 0, 0],
					[3, 113, 2489, 0, 0],
					[3, 113, 2490, 0, 0],
					[3, 113, 2491, 0, 0],
					[3, 113, 2492, 0, 0],
					[3, 113, 2493, 0, 0],
					[3, 113, 2494, 0, 0],
					[3, 113, 2495, 0, 0],
					[3, 113, 2496, 0, 0],
					[3, 113, 2497, 0, 0],
					[3, 113, 2498, 0, 0],
					[3, 113, 2499, 0, 0],
					[3, 113, 2500, 0, 0],
					[3, 113, 2501, 0, 0],
					[3, 113, 2502, 0, 0],
					[3, 113, 2503, 0, 0],
					[3, 113, 2504, 0, 0],
					[3, 113, 2505, 0, 0],
					[3, 113, 2506, 0, 0],
					[3, 113, 2507, 0, 0],
					[3, 113, 2508, 0, 0],
					[3, 114, 2509, 0, 0],
					[3, 114, 2510, 0, 0],
					[3, 114, 2511, 0, 0],
					[3, 114, 2512, 0, 0],
					[3, 114, 2513, 0, 0],
					[3, 114, 2514, 0, 0],
					[3, 114, 2515, 0, 0],
					[3, 114, 2516, 0, 0],
					[3, 114, 2517, 0, 0],
					[3, 114, 2518, 0, 0],
					[3, 114, 2519, 0, 0],
					[3, 115, 2520, 0, 0],
					[3, 115, 2521, 0, 0],
					[3, 115, 2522, 0, 0],
					[3, 115, 2523, 0, 0],
					[3, 115, 2524, 0, 0],
					[3, 115, 2525, 0, 0],
					[3, 115, 2526, 0, 0],
					[3, 115, 2527, 0, 0],
					[3, 115, 2528, 0, 0],
					[3, 115, 2529, 0, 0],
					[3, 115, 2530, 0, 0],
					[3, 115, 2531, 0, 0],
					[3, 115, 2532, 0, 0],
					[3, 115, 2533, 0, 0],
					[3, 115, 2534, 0, 0],
					[3, 115, 2535, 0, 0],
					[3, 115, 2536, 0, 0],
					[3, 115, 2537, 0, 0],
					[3, 115, 2538, 0, 0],
					[3, 115, 2539, 0, 0],
					[3, 115, 2540, 0, 0],
					[3, 115, 2541, 0, 0],
					[3, 115, 2542, 0, 0],
					[3, 115, 2543, 0, 0],
					[3, 115, 2544, 0, 0],
					[3, 115, 2545, 0, 0],
					[3, 115, 2546, 0, 0],
					[3, 115, 2547, 0, 0],
					[3, 115, 2548, 0, 0],
					[3, 115, 2549, 0, 0],
					[3, 115, 2550, 0, 0],
					[3, 115, 2551, 0, 0],
					[3, 115, 2552, 0, 0],
					[3, 115, 2553, 0, 0],
					[3, 115, 2554, 0, 0],
					[3, 115, 2555, 0, 0],
					[3, 115, 2556, 0, 0],
					[3, 115, 2557, 0, 0],
					[3, 115, 2558, 0, 0],
					[3, 116, 2559, 0, 0],
					[3, 116, 2560, 0, 0],
					[3, 116, 2561, 0, 0],
					[3, 116, 2562, 0, 0],
					[3, 116, 2563, 0, 0],
					[3, 116, 2564, 0, 0],
					[3, 116, 2565, 0, 0],
					[3, 116, 2566, 0, 0],
					[3, 116, 2567, 0, 0],
					[3, 116, 2568, 0, 0],
					[3, 116, 2569, 0, 0],
					[3, 95, 2570, 0, 0],
					[3, 95, 2571, 0, 0],
					[3, 95, 2572, 0, 0],
					[3, 95, 2573, 0, 0],
					[3, 113, 2574, 0, 0],
					[3, 113, 2575, 0, 0],
					[3, 113, 2576, 0, 0],
					[3, 117, 2577, 0, 0],
					[3, 117, 2578, 0, 0],
					[3, 117, 2579, 0, 0],
					[3, 117, 2580, 0, 0],
					[3, 117, 2581, 0, 0],
					[3, 117, 2582, 0, 0],
					[3, 117, 2583, 0, 0],
					[3, 117, 2584, 0, 0],
					[3, 117, 2585, 0, 0],
					[3, 117, 2586, 0, 0],
					[3, 117, 2587, 0, 0],
					[3, 117, 2588, 0, 0],
					[3, 117, 2589, 0, 0],
					[3, 117, 2590, 0, 0],
					[3, 117, 2591, 0, 0],
					[3, 117, 2592, 0, 0],
					[3, 117, 2593, 0, 0],
					[3, 117, 2594, 0, 0],
					[3, 117, 2595, 0, 0],
					[3, 117, 2596, 0, 0],
					[3, 117, 2597, 0, 0],
					[3, 117, 2598, 0, 0],
					[3, 16, 2600, 0, 0],
					[3, 16, 2601, 0, 0],
					[3, 9, 2602, 0, 0],
					[3, 9, 2603, 0, 0],
					[3, 43, 2604, 0, 0],
					[3, 83, 2605, 0, 0],
					[3, 45, 2606, 0, 0],
					[4, 6, 1, 0, 0],
					[4, 6, 2, 0, 0],
					[4, 6, 3, 0, 0],
					[4, 6, 4, 0, 0],
					[4, 6, 5, 0, 0],
					[4, 6, 7, 0, 0],
					[4, 6, 8, 0, 0],
					[4, 6, 9, 0, 0],
					[4, 6, 10, 0, 0],
					[4, 6, 11, 0, 0],
					[4, 6, 12, 0, 0],
					[4, 6, 14, 0, 0],
					[4, 6, 15, 0, 0],
					[4, 6, 16, 0, 0],
					[4, 6, 17, 0, 0],
					[4, 6, 18, 0, 0],
					[4, 6, 20, 0, 0],
					[4, 6, 21, 0, 1],
					[4, 6, 22, 0, 1],
					[4, 6, 23, 0, 0],
					[4, 6, 36, 0, 0],
					[4, 7, 39, 0, 0],
					[4, 7, 40, 0, 0],
					[4, 7, 41, 0, 0],
					[4, 7, 42, 0, 0],
					[4, 7, 43, 0, 0],
					[4, 7, 44, 0, 0],
					[4, 7, 46, 0, 0],
					[4, 7, 47, 0, 0],
					[4, 7, 48, 0, 0],
					[4, 7, 49, 0, 0],
					[4, 7, 50, 0, 0],
					[4, 7, 51, 0, 0],
					[4, 7, 53, 0, 0],
					[4, 7, 54, 0, 0],
					[4, 7, 55, 0, 0],
					[4, 7, 56, 0, 1],
					[4, 7, 57, 0, 1],
					[4, 7, 58, 0, 0],
					[4, 7, 65, 0, 0],
					[4, 4, 66, 0, 0],
					[4, 4, 67, 0, 0],
					[4, 4, 68, 0, 0],
					[4, 4, 69, 0, 0],
					[4, 4, 70, 0, 0],
					[4, 4, 71, 0, 0],
					[4, 4, 72, 0, 0],
					[4, 4, 74, 0, 0],
					[4, 4, 79, 0, 0],
					[4, 4, 80, 0, 0],
					[4, 4, 81, 0, 0],
					[4, 4, 85, 0, 0],
					[4, 4, 86, 0, 0],
					[4, 4, 87, 0, 0],
					[4, 4, 90, 0, 1],
					[4, 4, 91, 0, 1],
					[4, 4, 92, 0, 0],
					[4, 4, 93, 0, 0],
					[4, 4, 94, 0, 0],
					[4, 4, 95, 0, 0],
					[4, 4, 108, 0, 0],
					[4, 4, 109, 0, 0],
					[4, 26, 126, 0, 0],
					[4, 26, 127, 0, 0],
					[4, 26, 128, 0, 0],
					[4, 26, 129, 0, 0],
					[4, 26, 130, 0, 0],
					[4, 26, 131, 0, 0],
					[4, 26, 132, 0, 0],
					[4, 26, 133, 0, 0],
					[4, 26, 134, 0, 0],
					[4, 26, 135, 0, 0],
					[4, 26, 136, 0, 0],
					[4, 26, 137, 0, 1],
					[4, 26, 138, 0, 1],
					[4, 26, 139, 0, 0],
					[4, 26, 140, 0, 0],
					[4, 26, 141, 0, 0],
					[4, 26, 142, 0, 0],
					[4, 26, 143, 0, 0],
					[4, 26, 144, 0, 0],
					[4, 26, 145, 0, 0],
					[4, 26, 146, 0, 0],
					[4, 26, 147, 0, 0],
					[4, 26, 148, 0, 0],
					[4, 26, 149, 0, 0],
					[4, 26, 150, 0, 0],
					[4, 13, 155, 0, 0],
					[4, 13, 156, 0, 0],
					[4, 13, 157, 0, 0],
					[4, 13, 158, 0, 0],
					[4, 13, 159, 0, 0],
					[4, 13, 160, 0, 0],
					[4, 13, 161, 0, 0],
					[4, 13, 162, 0, 0],
					[4, 13, 163, 0, 0],
					[4, 13, 166, 0, 1],
					[4, 13, 167, 0, 1],
					[4, 13, 168, 0, 0],
					[4, 13, 169, 0, 0],
					[4, 13, 170, 0, 0],
					[4, 13, 171, 0, 0],
					[4, 14, 173, 0, 0],
					[4, 14, 174, 0, 0],
					[4, 14, 175, 0, 0],
					[4, 14, 176, 0, 0],
					[4, 14, 177, 0, 0],
					[4, 14, 178, 0, 0],
					[4, 14, 179, 0, 0],
					[4, 14, 180, 0, 0],
					[4, 14, 181, 0, 0],
					[4, 14, 182, 0, 0],
					[4, 14, 183, 0, 0],
					[4, 14, 184, 0, 0],
					[4, 14, 185, 0, 0],
					[4, 14, 186, 0, 0],
					[4, 14, 187, 0, 0],
					[4, 14, 188, 0, 0],
					[4, 14, 189, 0, 0],
					[4, 14, 190, 0, 1],
					[4, 14, 191, 0, 1],
					[4, 14, 192, 0, 0],
					[4, 14, 193, 0, 0],
					[4, 14, 194, 0, 0],
					[4, 14, 196, 0, 0],
					[4, 14, 197, 0, 0],
					[4, 14, 198, 0, 0],
					[4, 14, 199, 0, 0],
					[4, 14, 200, 0, 0],
					[4, 14, 201, 0, 0],
					[4, 14, 202, 0, 0],
					[4, 14, 203, 0, 0],
					[4, 8, 204, 0, 0],
					[4, 8, 205, 0, 1],
					[4, 8, 206, 0, 1],
					[4, 8, 207, 0, 0],
					[4, 8, 208, 0, 0],
					[4, 8, 209, 0, 0],
					[4, 8, 210, 0, 1],
					[4, 8, 211, 0, 1],
					[4, 8, 212, 0, 0],
					[4, 8, 213, 0, 0],
					[4, 8, 214, 0, 0],
					[4, 8, 215, 0, 1],
					[4, 8, 216, 0, 0],
					[4, 8, 217, 0, 0],
					[4, 8, 218, 0, 0],
					[4, 9, 231, 0, 0],
					[4, 9, 232, 0, 0],
					[4, 9, 233, 0, 0],
					[4, 9, 234, 0, 0],
					[4, 9, 235, 0, 0],
					[4, 9, 236, 0, 0],
					[4, 9, 237, 0, 0],
					[4, 9, 238, 0, 0],
					[4, 9, 239, 0, 0],
					[4, 9, 241, 0, 0],
					[4, 9, 242, 0, 0],
					[4, 9, 243, 0, 1],
					[4, 9, 244, 0, 1],
					[4, 9, 245, 0, 0],
					[4, 9, 246, 0, 0],
					[4, 9, 247, 0, 0],
					[4, 9, 248, 0, 0],
					[4, 9, 249, 0, 0],
					[4, 9, 250, 0, 0],
					[4, 9, 251, 0, 0],
					[4, 9, 253, 0, 0],
					[4, 9, 254, 0, 0],
					[4, 16, 255, 0, 0],
					[4, 16, 256, 0, 0],
					[4, 16, 257, 0, 0],
					[4, 16, 258, 0, 0],
					[4, 16, 259, 0, 0],
					[4, 16, 260, 0, 0],
					[4, 16, 262, 0, 0],
					[4, 16, 263, 0, 0],
					[4, 16, 264, 0, 0],
					[4, 16, 265, 0, 0],
					[4, 16, 266, 0, 0],
					[4, 16, 267, 0, 0],
					[4, 16, 268, 0, 0],
					[4, 16, 269, 0, 1],
					[4, 16, 270, 0, 1],
					[4, 16, 271, 0, 0],
					[4, 16, 272, 0, 0],
					[4, 16, 273, 0, 0],
					[4, 16, 274, 0, 0],
					[4, 16, 275, 0, 0],
					[4, 16, 276, 0, 0],
					[4, 16, 277, 0, 0],
					[4, 15, 278, 0, 0],
					[4, 15, 279, 0, 0],
					[4, 15, 280, 0, 0],
					[4, 15, 281, 0, 0],
					[4, 15, 282, 0, 0],
					[4, 15, 283, 0, 0],
					[4, 15, 284, 0, 0],
					[4, 15, 285, 0, 1],
					[4, 15, 286, 0, 1],
					[4, 15, 287, 0, 0],
					[4, 18, 288, 0, 0],
					[4, 18, 289, 0, 0],
					[4, 18, 290, 0, 0],
					[4, 18, 291, 0, 0],
					[4, 18, 292, 0, 0],
					[4, 18, 293, 0, 0],
					[4, 18, 294, 0, 0],
					[4, 18, 295, 0, 1],
					[4, 18, 296, 0, 1],
					[4, 18, 297, 0, 0],
					[4, 18, 304, 0, 0],
					[4, 19, 305, 0, 0],
					[4, 19, 306, 0, 0],
					[4, 19, 307, 0, 0],
					[4, 19, 308, 0, 1],
					[4, 19, 309, 0, 1],
					[4, 19, 310, 0, 0],
					[4, 19, 311, 0, 0],
					[4, 19, 312, 0, 0],
					[4, 29, 474, 0, 0],
					[4, 29, 475, 0, 0],
					[4, 29, 476, 0, 0],
					[4, 29, 477, 0, 0],
					[4, 29, 478, 0, 0],
					[4, 29, 479, 0, 0],
					[4, 29, 480, 0, 0],
					[4, 29, 481, 0, 0],
					[4, 29, 482, 0, 0],
					[4, 29, 483, 0, 0],
					[4, 29, 484, 0, 0],
					[4, 29, 485, 0, 0],
					[4, 29, 487, 0, 0],
					[4, 29, 492, 0, 0],
					[4, 29, 498, 0, 0],
					[4, 29, 500, 0, 0],
					[4, 29, 506, 0, 0],
					[4, 29, 507, 0, 0],
					[4, 29, 508, 0, 0],
					[4, 29, 509, 0, 0],
					[4, 29, 510, 0, 0],
					[4, 29, 511, 0, 0],
					[4, 29, 512, 0, 0],
					[4, 29, 513, 0, 0],
					[4, 29, 514, 0, 0],
					[4, 29, 515, 0, 0],
					[4, 29, 516, 0, 0],
					[4, 29, 517, 0, 0],
					[4, 33, 524, 0, 0],
					[4, 33, 525, 0, 0],
					[4, 33, 526, 0, 0],
					[4, 33, 527, 0, 0],
					[4, 33, 528, 0, 0],
					[4, 33, 529, 0, 0],
					[4, 33, 530, 0, 0],
					[4, 33, 531, 0, 0],
					[4, 33, 532, 0, 0],
					[4, 33, 533, 0, 0],
					[4, 33, 534, 0, 0],
					[4, 33, 535, 0, 0],
					[4, 33, 536, 0, 0],
					[4, 33, 537, 0, 0],
					[4, 33, 538, 0, 1],
					[4, 33, 539, 0, 1],
					[4, 29, 540, 0, 0],
					[4, 34, 541, 0, 0],
					[4, 34, 542, 0, 1],
					[4, 34, 543, 0, 1],
					[4, 34, 544, 0, 0],
					[4, 34, 545, 0, 1],
					[4, 34, 546, 0, 0],
					[4, 34, 547, 0, 0],
					[4, 34, 548, 0, 0],
					[4, 34, 549, 0, 0],
					[4, 34, 550, 0, 0],
					[4, 34, 551, 0, 0],
					[4, 34, 552, 0, 1],
					[4, 34, 553, 0, 1],
					[4, 34, 554, 0, 0],
					[4, 34, 555, 0, 0],
					[4, 34, 556, 0, 0],
					[4, 34, 557, 0, 1],
					[4, 34, 558, 0, 0],
					[4, 34, 559, 0, 0],
					[4, 35, 560, 0, 0],
					[4, 35, 561, 0, 0],
					[4, 35, 562, 0, 0],
					[4, 35, 563, 0, 0],
					[4, 35, 564, 0, 0],
					[4, 35, 565, 0, 0],
					[4, 35, 566, 0, 0],
					[4, 35, 567, 0, 0],
					[4, 35, 568, 0, 1],
					[4, 35, 569, 0, 1],
					[4, 35, 570, 0, 0],
					[4, 35, 571, 0, 0],
					[4, 35, 572, 0, 0],
					[4, 35, 573, 0, 0],
					[4, 35, 574, 0, 0],
					[4, 35, 575, 0, 0],
					[4, 35, 577, 0, 0],
					[4, 35, 578, 0, 0],
					[4, 37, 579, 0, 0],
					[4, 37, 580, 0, 0],
					[4, 37, 581, 0, 0],
					[4, 37, 582, 0, 0],
					[4, 37, 583, 0, 0],
					[4, 37, 584, 0, 0],
					[4, 37, 589, 0, 0],
					[4, 37, 590, 0, 0],
					[4, 37, 593, 0, 1],
					[4, 37, 594, 0, 1],
					[4, 37, 595, 0, 0],
					[4, 40, 597, 0, 0],
					[4, 40, 598, 0, 0],
					[4, 40, 599, 0, 1],
					[4, 40, 600, 0, 1],
					[4, 40, 601, 0, 0],
					[4, 40, 602, 0, 1],
					[4, 40, 603, 0, 0],
					[4, 41, 604, 0, 0],
					[4, 41, 605, 0, 0],
					[4, 41, 606, 0, 0],
					[4, 41, 607, 0, 0],
					[4, 41, 608, 0, 0],
					[4, 41, 609, 0, 0],
					[4, 41, 610, 0, 1],
					[4, 41, 611, 0, 1],
					[4, 41, 612, 0, 0],
					[4, 41, 613, 0, 0],
					[4, 42, 614, 0, 0],
					[4, 42, 615, 0, 0],
					[4, 42, 616, 0, 0],
					[4, 42, 617, 0, 0],
					[4, 42, 618, 0, 0],
					[4, 42, 619, 0, 0],
					[4, 42, 620, 0, 0],
					[4, 42, 621, 0, 0],
					[4, 42, 623, 0, 0],
					[4, 42, 624, 0, 0],
					[4, 42, 625, 0, 1],
					[4, 42, 626, 0, 1],
					[4, 42, 627, 0, 0],
					[4, 42, 628, 0, 0],
					[4, 43, 629, 0, 0],
					[4, 43, 630, 0, 0],
					[4, 43, 631, 0, 0],
					[4, 43, 632, 0, 0],
					[4, 43, 633, 0, 0],
					[4, 43, 634, 0, 0],
					[4, 43, 635, 0, 0],
					[4, 43, 636, 0, 0],
					[4, 43, 637, 0, 0],
					[4, 43, 638, 0, 0],
					[4, 43, 639, 0, 0],
					[4, 43, 640, 0, 0],
					[4, 43, 641, 0, 0],
					[4, 43, 642, 0, 1],
					[4, 43, 643, 0, 1],
					[4, 43, 644, 0, 0],
					[4, 43, 645, 0, 0],
					[4, 45, 646, 0, 0],
					[4, 45, 647, 0, 1],
					[4, 45, 648, 0, 1],
					[4, 45, 649, 0, 0],
					[4, 45, 650, 0, 0],
					[4, 29, 652, 0, 0],
					[4, 29, 689, 0, 0],
					[4, 42, 690, 0, 0],
					[4, 40, 691, 0, 0],
					[4, 29, 692, 0, 0],
					[4, 29, 693, 0, 0],
					[4, 29, 694, 0, 0],
					[4, 29, 699, 0, 0],
					[4, 6, 700, 0, 1],
					[4, 4, 701, 0, 1],
					[4, 29, 703, 0, 0],
					[4, 18, 708, 0, 0],
					[4, 40, 710, 0, 0],
					[4, 40, 711, 0, 0],
					[4, 29, 716, 0, 0],
					[4, 29, 719, 0, 0],
					[4, 13, 720, 0, 0],
					[4, 29, 721, 0, 0],
					[4, 29, 722, 0, 0],
					[4, 4, 725, 0, 1],
					[4, 6, 726, 0, 1],
					[4, 7, 727, 0, 1],
					[4, 9, 729, 0, 1],
					[4, 13, 730, 0, 1],
					[4, 18, 734, 0, 1],
					[4, 33, 741, 0, 1],
					[4, 34, 742, 0, 1],
					[4, 35, 743, 0, 1],
					[4, 37, 744, 0, 1],
					[4, 40, 745, 0, 1],
					[4, 41, 746, 0, 1],
					[4, 42, 747, 0, 1],
					[4, 43, 748, 0, 1],
					[4, 45, 749, 0, 1],
					[4, 51, 780, 0, 0],
					[4, 51, 781, 0, 0],
					[4, 51, 782, 0, 0],
					[4, 51, 783, 0, 0],
					[4, 51, 784, 0, 0],
					[4, 51, 785, 0, 0],
					[4, 51, 786, 0, 0],
					[4, 51, 787, 0, 0],
					[4, 51, 788, 0, 0],
					[4, 51, 791, 0, 1],
					[4, 51, 792, 0, 1],
					[4, 13, 814, 0, 0],
					[4, 42, 816, 0, 0],
					[4, 37, 818, 0, 0],
					[4, 34, 820, 0, 0],
					[4, 43, 826, 0, 0],
					[4, 54, 842, 0, 0],
					[4, 54, 843, 0, 0],
					[4, 54, 844, 0, 0],
					[4, 54, 845, 0, 0],
					[4, 54, 846, 0, 0],
					[4, 54, 847, 0, 0],
					[4, 54, 848, 0, 0],
					[4, 54, 849, 0, 0],
					[4, 54, 850, 0, 0],
					[4, 54, 851, 0, 0],
					[4, 54, 852, 0, 0],
					[4, 54, 853, 0, 0],
					[4, 54, 854, 0, 0],
					[4, 54, 855, 0, 0],
					[4, 54, 856, 0, 0],
					[4, 54, 857, 0, 0],
					[4, 54, 858, 0, 1],
					[4, 54, 859, 0, 1],
					[4, 54, 860, 0, 0],
					[4, 54, 861, 0, 1],
					[4, 54, 862, 0, 1],
					[4, 54, 863, 0, 0],
					[4, 54, 864, 0, 0],
					[4, 8, 865, 0, 0],
					[4, 57, 866, 0, 0],
					[4, 57, 867, 0, 0],
					[4, 57, 868, 0, 0],
					[4, 57, 870, 0, 0],
					[4, 57, 871, 0, 0],
					[4, 57, 872, 0, 0],
					[4, 57, 873, 0, 0],
					[4, 57, 874, 0, 0],
					[4, 57, 875, 0, 1],
					[4, 57, 876, 0, 1],
					[4, 57, 877, 0, 0],
					[4, 57, 878, 0, 0],
					[4, 58, 879, 0, 0],
					[4, 58, 880, 0, 0],
					[4, 58, 881, 0, 0],
					[4, 58, 883, 0, 0],
					[4, 58, 884, 0, 0],
					[4, 58, 885, 0, 0],
					[4, 58, 886, 0, 0],
					[4, 58, 887, 0, 0],
					[4, 58, 888, 0, 1],
					[4, 58, 889, 0, 1],
					[4, 58, 890, 0, 0],
					[4, 58, 891, 0, 0],
					[4, 59, 892, 0, 0],
					[4, 59, 893, 0, 0],
					[4, 59, 894, 0, 0],
					[4, 59, 895, 0, 0],
					[4, 59, 896, 0, 0],
					[4, 59, 898, 0, 0],
					[4, 59, 899, 0, 0],
					[4, 59, 900, 0, 0],
					[4, 59, 901, 0, 0],
					[4, 59, 902, 0, 1],
					[4, 59, 903, 0, 1],
					[4, 59, 904, 0, 0],
					[4, 59, 905, 0, 0],
					[4, 60, 927, 0, 0],
					[4, 60, 928, 0, 0],
					[4, 60, 929, 0, 0],
					[4, 60, 930, 0, 0],
					[4, 60, 931, 0, 0],
					[4, 60, 932, 0, 0],
					[4, 60, 933, 0, 0],
					[4, 60, 934, 0, 1],
					[4, 60, 935, 0, 1],
					[4, 60, 936, 0, 0],
					[4, 7, 938, 0, 0],
					[4, 7, 940, 0, 0],
					[4, 61, 941, 0, 0],
					[4, 61, 942, 0, 0],
					[4, 61, 943, 0, 0],
					[4, 61, 944, 0, 0],
					[4, 61, 945, 0, 0],
					[4, 61, 946, 0, 0],
					[4, 61, 947, 0, 0],
					[4, 61, 948, 0, 0],
					[4, 61, 949, 0, 0],
					[4, 61, 950, 0, 0],
					[4, 61, 951, 0, 1],
					[4, 61, 952, 0, 1],
					[4, 61, 953, 0, 0],
					[4, 61, 954, 0, 0],
					[4, 61, 955, 0, 0],
					[4, 61, 956, 0, 0],
					[4, 61, 957, 0, 0],
					[4, 61, 958, 0, 0],
					[4, 61, 959, 0, 0],
					[4, 61, 960, 0, 0],
					[4, 61, 961, 0, 0],
					[4, 61, 962, 0, 0],
					[4, 61, 963, 0, 0],
					[4, 61, 964, 0, 0],
					[4, 61, 965, 0, 0],
					[4, 61, 966, 0, 0],
					[4, 7, 967, 0, 0],
					[4, 7, 968, 0, 0],
					[4, 7, 969, 0, 0],
					[4, 6, 970, 0, 0],
					[4, 6, 971, 0, 0],
					[4, 6, 972, 0, 0],
					[4, 18, 973, 0, 0],
					[4, 18, 974, 0, 0],
					[4, 18, 975, 0, 0],
					[4, 43, 1044, 0, 0],
					[4, 29, 1045, 0, 0],
					[4, 13, 1049, 0, 0],
					[4, 7, 1065, 0, 0],
					[4, 7, 1066, 0, 0],
					[4, 7, 1067, 0, 0],
					[4, 7, 1068, 0, 0],
					[4, 7, 1069, 0, 0],
					[4, 7, 1070, 0, 0],
					[4, 7, 1071, 0, 0],
					[4, 7, 1072, 0, 0],
					[4, 4, 1073, 0, 0],
					[4, 4, 1074, 0, 0],
					[4, 4, 1075, 0, 0],
					[4, 4, 1076, 0, 0],
					[4, 4, 1077, 0, 0],
					[4, 4, 1078, 0, 0],
					[4, 4, 1079, 0, 0],
					[4, 4, 1080, 0, 0],
					[4, 4, 1081, 0, 0],
					[4, 4, 1082, 0, 0],
					[4, 4, 1083, 0, 0],
					[4, 4, 1084, 0, 0],
					[4, 4, 1085, 0, 0],
					[4, 4, 1086, 0, 0],
					[4, 4, 1087, 0, 0],
					[4, 4, 1088, 0, 0],
					[4, 6, 1089, 0, 0],
					[4, 6, 1090, 0, 0],
					[4, 6, 1091, 0, 0],
					[4, 6, 1092, 0, 0],
					[4, 6, 1093, 0, 0],
					[4, 6, 1094, 0, 0],
					[4, 6, 1095, 0, 0],
					[4, 6, 1096, 0, 0],
					[4, 6, 1097, 0, 0],
					[4, 6, 1098, 0, 0],
					[4, 6, 1099, 0, 0],
					[4, 6, 1100, 0, 0],
					[4, 6, 1101, 0, 0],
					[4, 6, 1102, 0, 0],
					[4, 6, 1103, 0, 0],
					[4, 6, 1104, 0, 0],
					[4, 18, 1105, 0, 0],
					[4, 18, 1106, 0, 0],
					[4, 18, 1107, 0, 0],
					[4, 18, 1108, 0, 0],
					[4, 18, 1109, 0, 0],
					[4, 18, 1110, 0, 0],
					[4, 18, 1111, 0, 0],
					[4, 18, 1112, 0, 0],
					[4, 18, 1113, 0, 0],
					[4, 18, 1114, 0, 0],
					[4, 18, 1115, 0, 0],
					[4, 18, 1116, 0, 0],
					[4, 18, 1117, 0, 0],
					[4, 18, 1118, 0, 0],
					[4, 18, 1119, 0, 0],
					[4, 18, 1120, 0, 0],
					[4, 18, 1121, 0, 0],
					[4, 18, 1122, 0, 0],
					[4, 18, 1123, 0, 0],
					[4, 18, 1124, 0, 0],
					[4, 18, 1125, 0, 0],
					[4, 18, 1126, 0, 0],
					[4, 18, 1127, 0, 0],
					[4, 18, 1128, 0, 0],
					[4, 6, 1160, 0, 0],
					[4, 6, 1161, 0, 0],
					[4, 6, 1162, 0, 0],
					[4, 6, 1163, 0, 0],
					[4, 6, 1164, 0, 0],
					[4, 6, 1165, 0, 0],
					[4, 6, 1166, 0, 0],
					[4, 6, 1167, 0, 0],
					[4, 7, 1276, 0, 0],
					[4, 6, 1277, 0, 0],
					[4, 4, 1278, 0, 0],
					[4, 18, 1279, 0, 0],
					[4, 6, 1285, 0, 0],
					[4, 6, 1286, 0, 0],
					[4, 6, 1287, 0, 0],
					[4, 6, 1288, 0, 0],
					[4, 6, 1289, 0, 0],
					[4, 6, 1290, 0, 0],
					[4, 7, 1291, 0, 0],
					[4, 7, 1292, 0, 0],
					[4, 4, 1293, 0, 0],
					[4, 4, 1294, 0, 0],
					[4, 4, 1295, 0, 0],
					[4, 4, 1296, 0, 0],
					[4, 37, 1314, 0, 0],
					[4, 59, 1315, 0, 0],
					[4, 57, 1316, 0, 0],
					[4, 58, 1317, 0, 0],
					[4, 42, 1318, 0, 0],
					[4, 42, 1319, 0, 0],
					[4, 42, 1320, 0, 0],
					[4, 7, 1321, 0, 0],
					[4, 29, 1322, 0, 0],
					[4, 37, 1323, 0, 0],
					[4, 37, 1324, 0, 0],
					[4, 51, 1326, 0, 0],
					[4, 51, 1327, 0, 0],
					[4, 7, 1329, 0, 0],
					[4, 6, 1330, 0, 0],
					[4, 6, 1331, 0, 0],
					[4, 4, 1332, 0, 0],
					[4, 18, 1333, 0, 0],
					[4, 43, 1340, 0, 0],
					[4, 13, 1341, 0, 0],
					[4, 58, 1344, 0, 0],
					[4, 4, 1368, 0, 0],
					[4, 6, 1369, 0, 0],
					[4, 6, 1375, 0, 0],
					[4, 26, 1378, 0, 0],
					[4, 43, 1382, 0, 0],
					[4, 13, 1384, 0, 0],
					[4, 4, 1386, 0, 0],
					[4, 4, 1391, 0, 0],
					[4, 14, 1393, 0, 0],
					[4, 35, 1395, 0, 0],
					[4, 7, 1397, 0, 0],
					[4, 8, 1399, 0, 0],
					[4, 9, 1401, 0, 0],
					[4, 15, 1403, 0, 0],
					[4, 16, 1405, 0, 0],
					[4, 18, 1407, 0, 0],
					[4, 19, 1409, 0, 0],
					[4, 34, 1419, 0, 0],
					[4, 37, 1421, 0, 0],
					[4, 41, 1423, 0, 0],
					[4, 42, 1425, 0, 0],
					[4, 51, 1431, 0, 0],
					[4, 54, 1433, 0, 0],
					[4, 57, 1435, 0, 0],
					[4, 58, 1437, 0, 0],
					[4, 59, 1439, 0, 0],
					[4, 60, 1441, 0, 0],
					[4, 61, 1443, 0, 0],
					[4, 4, 1449, 0, 0],
					[4, 6, 1450, 0, 0],
					[4, 7, 1451, 0, 0],
					[4, 8, 1452, 0, 0],
					[4, 9, 1453, 0, 0],
					[4, 13, 1454, 0, 0],
					[4, 14, 1455, 0, 0],
					[4, 15, 1456, 0, 0],
					[4, 16, 1457, 0, 0],
					[4, 18, 1458, 0, 0],
					[4, 19, 1459, 0, 0],
					[4, 26, 1464, 0, 0],
					[4, 34, 1465, 0, 0],
					[4, 35, 1466, 0, 0],
					[4, 37, 1467, 0, 0],
					[4, 41, 1468, 0, 0],
					[4, 42, 1469, 0, 0],
					[4, 43, 1470, 0, 0],
					[4, 51, 1473, 0, 0],
					[4, 54, 1474, 0, 0],
					[4, 57, 1475, 0, 0],
					[4, 58, 1476, 0, 0],
					[4, 59, 1477, 0, 0],
					[4, 60, 1478, 0, 0],
					[4, 61, 1479, 0, 0],
					[4, 13, 1482, 0, 0],
					[4, 13, 1483, 0, 0],
					[4, 37, 1484, 0, 0],
					[4, 58, 1485, 0, 0],
					[4, 7, 1486, 0, 0],
					[4, 4, 1487, 0, 0],
					[4, 6, 1488, 0, 0],
					[4, 18, 1489, 0, 0],
					[4, 4, 1494, 0, 0],
					[4, 6, 1495, 0, 0],
					[4, 18, 1496, 0, 0],
					[4, 18, 1501, 0, 0],
					[4, 6, 1502, 0, 0],
					[4, 4, 1503, 0, 0],
					[4, 74, 1504, 0, 0],
					[4, 74, 1505, 0, 0],
					[4, 74, 1506, 0, 0],
					[4, 74, 1507, 0, 0],
					[4, 74, 1508, 0, 0],
					[4, 74, 1509, 0, 0],
					[4, 74, 1511, 0, 0],
					[4, 74, 1512, 0, 0],
					[4, 74, 1513, 0, 0],
					[4, 74, 1514, 0, 0],
					[4, 74, 1515, 0, 0],
					[4, 74, 1516, 0, 0],
					[4, 74, 1517, 0, 0],
					[4, 74, 1518, 0, 0],
					[4, 74, 1519, 0, 0],
					[4, 74, 1520, 0, 0],
					[4, 74, 1521, 0, 0],
					[4, 74, 1522, 0, 0],
					[4, 4, 1524, 0, 0],
					[4, 6, 1525, 0, 0],
					[4, 7, 1526, 0, 0],
					[4, 8, 1527, 0, 0],
					[4, 9, 1528, 0, 0],
					[4, 13, 1529, 0, 0],
					[4, 14, 1530, 0, 0],
					[4, 15, 1531, 0, 0],
					[4, 18, 1532, 0, 0],
					[4, 19, 1533, 0, 0],
					[4, 26, 1538, 0, 0],
					[4, 34, 1539, 0, 0],
					[4, 35, 1540, 0, 0],
					[4, 37, 1541, 0, 0],
					[4, 41, 1542, 0, 0],
					[4, 42, 1543, 0, 0],
					[4, 43, 1544, 0, 0],
					[4, 51, 1547, 0, 0],
					[4, 54, 1548, 0, 0],
					[4, 57, 1549, 0, 0],
					[4, 58, 1550, 0, 0],
					[4, 59, 1551, 0, 0],
					[4, 60, 1552, 0, 0],
					[4, 61, 1553, 0, 0],
					[4, 74, 1556, 0, 0],
					[4, 75, 1557, 0, 0],
					[4, 75, 1558, 0, 0],
					[4, 75, 1559, 0, 0],
					[4, 75, 1560, 0, 0],
					[4, 75, 1561, 0, 0],
					[4, 75, 1562, 0, 0],
					[4, 75, 1563, 0, 0],
					[4, 75, 1564, 0, 0],
					[4, 75, 1566, 0, 0],
					[4, 75, 1567, 0, 0],
					[4, 75, 1568, 0, 0],
					[4, 75, 1569, 0, 0],
					[4, 51, 1600, 0, 0],
					[4, 9, 1603, 0, 0],
					[4, 16, 1604, 0, 0],
					[4, 78, 1605, 0, 0],
					[4, 78, 1606, 0, 0],
					[4, 78, 1607, 0, 0],
					[4, 78, 1608, 0, 0],
					[4, 78, 1609, 0, 0],
					[4, 78, 1610, 0, 0],
					[4, 78, 1611, 0, 0],
					[4, 78, 1612, 0, 0],
					[4, 78, 1613, 0, 0],
					[4, 78, 1615, 0, 0],
					[4, 79, 1616, 0, 0],
					[4, 79, 1617, 0, 0],
					[4, 79, 1618, 0, 0],
					[4, 79, 1619, 0, 0],
					[4, 79, 1620, 0, 0],
					[4, 79, 1621, 0, 0],
					[4, 79, 1622, 0, 0],
					[4, 79, 1625, 0, 0],
					[4, 79, 1626, 0, 0],
					[4, 79, 1627, 0, 0],
					[4, 79, 1628, 0, 0],
					[4, 6, 1630, 0, 0],
					[4, 80, 1632, 0, 0],
					[4, 80, 1633, 0, 0],
					[4, 80, 1634, 0, 0],
					[4, 80, 1635, 0, 0],
					[4, 80, 1636, 0, 0],
					[4, 80, 1637, 0, 0],
					[4, 80, 1638, 0, 0],
					[4, 80, 1640, 0, 0],
					[4, 80, 1641, 0, 0],
					[4, 80, 1642, 0, 0],
					[4, 80, 1643, 0, 0],
					[4, 80, 1644, 0, 0],
					[4, 81, 1645, 0, 0],
					[4, 81, 1646, 0, 0],
					[4, 81, 1647, 0, 0],
					[4, 81, 1648, 0, 0],
					[4, 81, 1649, 0, 0],
					[4, 81, 1650, 0, 0],
					[4, 81, 1651, 0, 0],
					[4, 81, 1652, 0, 0],
					[4, 81, 1653, 0, 0],
					[4, 81, 1655, 0, 0],
					[4, 81, 1656, 0, 0],
					[4, 81, 1657, 0, 0],
					[4, 81, 1658, 0, 0],
					[4, 81, 1659, 0, 0],
					[4, 81, 1660, 0, 0],
					[4, 81, 1661, 0, 0],
					[4, 81, 1666, 0, 0],
					[4, 82, 1667, 0, 0],
					[4, 82, 1668, 0, 0],
					[4, 82, 1669, 0, 0],
					[4, 82, 1670, 0, 0],
					[4, 82, 1671, 0, 0],
					[4, 82, 1672, 0, 0],
					[4, 82, 1673, 0, 0],
					[4, 82, 1674, 0, 0],
					[4, 82, 1675, 0, 0],
					[4, 82, 1677, 0, 0],
					[4, 82, 1678, 0, 0],
					[4, 82, 1679, 0, 0],
					[4, 82, 1680, 0, 0],
					[4, 82, 1681, 0, 0],
					[4, 82, 1682, 0, 0],
					[4, 82, 1683, 0, 0],
					[4, 82, 1688, 0, 0],
					[4, 18, 1689, 0, 0],
					[4, 18, 1690, 0, 0],
					[4, 18, 1691, 0, 0],
					[4, 18, 1692, 0, 0],
					[4, 18, 1693, 0, 0],
					[4, 18, 1694, 0, 0],
					[4, 83, 1708, 0, 0],
					[4, 83, 1709, 0, 0],
					[4, 83, 1710, 0, 0],
					[4, 83, 1711, 0, 0],
					[4, 83, 1712, 0, 0],
					[4, 83, 1713, 0, 0],
					[4, 16, 1714, 0, 0],
					[4, 9, 1715, 0, 0],
					[4, 84, 1716, 0, 0],
					[4, 84, 1717, 0, 0],
					[4, 84, 1718, 0, 0],
					[4, 84, 1719, 0, 0],
					[4, 84, 1720, 0, 0],
					[4, 84, 1721, 0, 0],
					[4, 84, 1722, 0, 0],
					[4, 84, 1723, 0, 0],
					[4, 84, 1724, 0, 0],
					[4, 84, 1725, 0, 0],
					[4, 84, 1726, 0, 0],
					[4, 84, 1727, 0, 0],
					[4, 84, 1728, 0, 0],
					[4, 84, 1729, 0, 0],
					[4, 84, 1730, 0, 0],
					[4, 84, 1731, 0, 0],
					[4, 84, 1732, 0, 0],
					[4, 84, 1733, 0, 0],
					[4, 84, 1735, 0, 0],
					[4, 7, 1736, 0, 0],
					[4, 7, 1737, 0, 0],
					[4, 6, 1738, 0, 0],
					[4, 29, 1740, 0, 0],
					[4, 41, 1741, 0, 0],
					[4, 42, 1742, 0, 0],
					[4, 41, 1743, 0, 0],
					[4, 4, 1744, 0, 0],
					[4, 54, 1745, 0, 0],
					[4, 4, 1746, 0, 0],
					[4, 6, 1747, 0, 0],
					[4, 6, 1756, 0, 0],
					[4, 6, 1757, 0, 0],
					[4, 6, 1758, 0, 0],
					[4, 14, 1759, 0, 0],
					[4, 16, 1761, 0, 0],
					[4, 9, 1762, 0, 0],
					[4, 15, 1763, 0, 0],
					[4, 19, 1764, 0, 0],
					[4, 85, 1765, 0, 0],
					[4, 85, 1766, 0, 0],
					[4, 85, 1767, 0, 0],
					[4, 85, 1768, 0, 0],
					[4, 85, 1769, 0, 0],
					[4, 85, 1770, 0, 0],
					[4, 85, 1771, 0, 0],
					[4, 85, 1772, 0, 0],
					[4, 85, 1773, 0, 0],
					[4, 85, 1774, 0, 0],
					[4, 85, 1775, 0, 0],
					[4, 85, 1776, 0, 0],
					[4, 85, 1777, 0, 0],
					[4, 85, 1778, 0, 0],
					[4, 86, 1779, 0, 0],
					[4, 86, 1780, 0, 0],
					[4, 86, 1781, 0, 0],
					[4, 86, 1782, 0, 0],
					[4, 86, 1783, 0, 0],
					[4, 86, 1784, 0, 0],
					[4, 86, 1785, 0, 0],
					[4, 86, 1786, 0, 0],
					[4, 86, 1787, 0, 0],
					[4, 86, 1788, 0, 0],
					[4, 87, 1789, 0, 0],
					[4, 87, 1790, 0, 0],
					[4, 87, 1791, 0, 0],
					[4, 87, 1792, 0, 0],
					[4, 87, 1793, 0, 0],
					[4, 87, 1794, 0, 0],
					[4, 87, 1795, 0, 0],
					[4, 87, 1796, 0, 0],
					[4, 87, 1797, 0, 0],
					[4, 87, 1798, 0, 0],
					[4, 87, 1799, 0, 0],
					[4, 87, 1800, 0, 0],
					[4, 87, 1801, 0, 0],
					[4, 87, 1802, 0, 0],
					[4, 87, 1803, 0, 0],
					[4, 88, 1804, 0, 0],
					[4, 88, 1805, 0, 0],
					[4, 88, 1806, 0, 0],
					[4, 88, 1807, 0, 0],
					[4, 88, 1808, 0, 0],
					[4, 88, 1809, 0, 0],
					[4, 88, 1810, 0, 0],
					[4, 88, 1811, 0, 0],
					[4, 88, 1812, 0, 0],
					[4, 88, 1813, 0, 0],
					[4, 88, 1814, 0, 0],
					[4, 88, 1815, 0, 0],
					[4, 88, 1816, 0, 0],
					[4, 88, 1817, 0, 0],
					[4, 88, 1818, 0, 0],
					[4, 89, 1819, 0, 0],
					[4, 89, 1820, 0, 0],
					[4, 89, 1821, 0, 0],
					[4, 89, 1822, 0, 0],
					[4, 89, 1823, 0, 0],
					[4, 89, 1824, 0, 0],
					[4, 89, 1825, 0, 0],
					[4, 89, 1826, 0, 0],
					[4, 89, 1827, 0, 0],
					[4, 89, 1828, 0, 0],
					[4, 89, 1829, 0, 0],
					[4, 89, 1830, 0, 0],
					[4, 89, 1831, 0, 0],
					[4, 89, 1832, 0, 0],
					[4, 89, 1833, 0, 0],
					[4, 90, 1834, 0, 0],
					[4, 90, 1835, 0, 0],
					[4, 90, 1836, 0, 0],
					[4, 90, 1837, 0, 0],
					[4, 90, 1838, 0, 0],
					[4, 90, 1839, 0, 0],
					[4, 90, 1840, 0, 0],
					[4, 90, 1841, 0, 0],
					[4, 90, 1842, 0, 0],
					[4, 90, 1843, 0, 0],
					[4, 90, 1844, 0, 0],
					[4, 90, 1845, 0, 0],
					[4, 90, 1846, 0, 0],
					[4, 90, 1847, 0, 0],
					[4, 90, 1848, 0, 0],
					[4, 90, 1849, 0, 0],
					[4, 90, 1850, 0, 0],
					[4, 90, 1851, 0, 0],
					[4, 91, 1852, 0, 0],
					[4, 91, 1853, 0, 0],
					[4, 91, 1854, 0, 0],
					[4, 91, 1855, 0, 0],
					[4, 91, 1856, 0, 0],
					[4, 91, 1857, 0, 0],
					[4, 91, 1858, 0, 0],
					[4, 91, 1859, 0, 0],
					[4, 91, 1860, 0, 0],
					[4, 91, 1861, 0, 0],
					[4, 91, 1862, 0, 0],
					[4, 91, 1863, 0, 0],
					[4, 91, 1864, 0, 0],
					[4, 91, 1865, 0, 0],
					[4, 91, 1866, 0, 0],
					[4, 91, 1867, 0, 0],
					[4, 91, 1868, 0, 0],
					[4, 91, 1869, 0, 0],
					[4, 91, 1870, 0, 0],
					[4, 91, 1871, 0, 0],
					[4, 91, 1872, 0, 0],
					[4, 91, 1873, 0, 0],
					[4, 91, 1874, 0, 0],
					[4, 91, 1875, 0, 0],
					[4, 89, 1876, 0, 0],
					[4, 89, 1877, 0, 0],
					[4, 89, 1878, 0, 0],
					[4, 89, 1879, 0, 0],
					[4, 89, 1880, 0, 0],
					[4, 89, 1881, 0, 0],
					[4, 89, 1882, 0, 0],
					[4, 89, 1883, 0, 0],
					[4, 89, 1884, 0, 0],
					[4, 89, 1885, 0, 0],
					[4, 89, 1886, 0, 0],
					[4, 89, 1887, 0, 0],
					[4, 89, 1888, 0, 0],
					[4, 89, 1889, 0, 0],
					[4, 89, 1890, 0, 0],
					[4, 89, 1891, 0, 0],
					[4, 89, 1892, 0, 0],
					[4, 89, 1893, 0, 0],
					[4, 89, 1894, 0, 0],
					[4, 89, 1895, 0, 0],
					[4, 89, 1896, 0, 0],
					[4, 89, 1897, 0, 0],
					[4, 90, 1898, 0, 0],
					[4, 90, 1899, 0, 0],
					[4, 90, 1900, 0, 0],
					[4, 90, 1901, 0, 0],
					[4, 90, 1902, 0, 0],
					[4, 90, 1903, 0, 0],
					[4, 90, 1904, 0, 0],
					[4, 90, 1905, 0, 0],
					[4, 90, 1906, 0, 0],
					[4, 90, 1907, 0, 0],
					[4, 90, 1908, 0, 0],
					[4, 90, 1909, 0, 0],
					[4, 90, 1910, 0, 0],
					[4, 90, 1911, 0, 0],
					[4, 90, 1912, 0, 0],
					[4, 90, 1913, 0, 0],
					[4, 90, 1914, 0, 0],
					[4, 90, 1915, 0, 0],
					[4, 90, 1916, 0, 0],
					[4, 90, 1917, 0, 0],
					[4, 90, 1918, 0, 0],
					[4, 90, 1919, 0, 0],
					[4, 91, 1920, 0, 0],
					[4, 91, 1921, 0, 0],
					[4, 91, 1922, 0, 0],
					[4, 91, 1923, 0, 0],
					[4, 91, 1924, 0, 0],
					[4, 91, 1925, 0, 0],
					[4, 91, 1926, 0, 0],
					[4, 91, 1927, 0, 0],
					[4, 91, 1928, 0, 0],
					[4, 91, 1929, 0, 0],
					[4, 91, 1930, 0, 0],
					[4, 91, 1931, 0, 0],
					[4, 91, 1932, 0, 0],
					[4, 91, 1933, 0, 0],
					[4, 91, 1934, 0, 0],
					[4, 91, 1935, 0, 0],
					[4, 91, 1936, 0, 0],
					[4, 91, 1937, 0, 0],
					[4, 91, 1938, 0, 0],
					[4, 91, 1939, 0, 0],
					[4, 91, 1940, 0, 0],
					[4, 91, 1941, 0, 0],
					[4, 89, 1942, 0, 0],
					[4, 90, 1943, 0, 0],
					[4, 91, 1944, 0, 0],
					[4, 6, 1945, 0, 1],
					[4, 6, 1946, 0, 1],
					[4, 37, 1947, 0, 0],
					[4, 92, 1948, 0, 0],
					[4, 92, 1949, 0, 0],
					[4, 92, 1950, 0, 0],
					[4, 92, 1951, 0, 0],
					[4, 92, 1952, 0, 0],
					[4, 92, 1953, 0, 0],
					[4, 92, 1954, 0, 0],
					[4, 92, 1955, 0, 0],
					[4, 92, 1956, 0, 0],
					[4, 92, 1957, 0, 0],
					[4, 92, 1958, 0, 0],
					[4, 92, 1959, 0, 0],
					[4, 92, 1960, 0, 0],
					[4, 92, 1961, 0, 0],
					[4, 92, 1962, 0, 0],
					[4, 92, 1963, 0, 0],
					[4, 92, 1964, 0, 0],
					[4, 92, 1965, 0, 0],
					[4, 92, 1966, 0, 0],
					[4, 92, 1967, 0, 0],
					[4, 93, 1968, 0, 0],
					[4, 93, 1969, 0, 0],
					[4, 93, 1970, 0, 0],
					[4, 93, 1971, 0, 0],
					[4, 93, 1972, 0, 0],
					[4, 93, 1973, 0, 0],
					[4, 93, 1974, 0, 0],
					[4, 93, 1975, 0, 0],
					[4, 93, 1976, 0, 0],
					[4, 93, 1977, 0, 0],
					[4, 93, 1978, 0, 0],
					[4, 93, 1979, 0, 0],
					[4, 93, 1980, 0, 0],
					[4, 93, 1981, 0, 0],
					[4, 93, 1982, 0, 0],
					[4, 93, 1983, 0, 0],
					[4, 93, 1984, 0, 0],
					[4, 93, 1985, 0, 0],
					[4, 93, 1986, 0, 0],
					[4, 93, 1987, 0, 0],
					[4, 59, 1988, 0, 0],
					[4, 37, 1989, 0, 0],
					[4, 57, 1990, 0, 0],
					[4, 58, 1991, 0, 0],
					[4, 9, 1992, 0, 0],
					[4, 16, 1993, 0, 0],
					[4, 51, 1994, 0, 0],
					[4, 51, 1995, 0, 0],
					[4, 51, 1996, 0, 0],
					[4, 4, 1997, 0, 0],
					[4, 7, 1998, 0, 0],
					[4, 18, 1999, 0, 0],
					[4, 92, 2000, 0, 0],
					[4, 93, 2001, 0, 0],
					[4, 61, 2002, 0, 0],
					[4, 86, 2003, 0, 0],
					[4, 26, 2004, 0, 0],
					[4, 41, 2005, 0, 0],
					[4, 85, 2006, 0, 0],
					[4, 87, 2007, 0, 0],
					[4, 88, 2008, 0, 0],
					[4, 89, 2009, 0, 0],
					[4, 90, 2010, 0, 0],
					[4, 91, 2011, 0, 0],
					[4, 92, 2012, 0, 0],
					[4, 93, 2013, 0, 0],
					[4, 4, 2014, 0, 0],
					[4, 6, 2015, 0, 0],
					[4, 7, 2016, 0, 0],
					[4, 13, 2017, 0, 0],
					[4, 18, 2018, 0, 0],
					[4, 26, 2019, 0, 0],
					[4, 34, 2020, 0, 0],
					[4, 43, 2021, 0, 0],
					[4, 61, 2022, 0, 0],
					[4, 85, 2023, 0, 0],
					[4, 86, 2024, 0, 0],
					[4, 87, 2025, 0, 0],
					[4, 88, 2026, 0, 0],
					[4, 89, 2027, 0, 0],
					[4, 90, 2028, 0, 0],
					[4, 91, 2029, 0, 0],
					[4, 92, 2030, 0, 0],
					[4, 93, 2031, 0, 0],
					[4, 94, 2032, 0, 0],
					[4, 94, 2033, 0, 0],
					[4, 94, 2034, 0, 0],
					[4, 94, 2035, 0, 0],
					[4, 94, 2036, 0, 0],
					[4, 94, 2037, 0, 0],
					[4, 94, 2038, 0, 0],
					[4, 94, 2039, 0, 0],
					[4, 95, 2040, 0, 0],
					[4, 95, 2041, 0, 0],
					[4, 95, 2042, 0, 0],
					[4, 95, 2043, 0, 0],
					[4, 95, 2044, 0, 0],
					[4, 95, 2046, 0, 0],
					[4, 95, 2047, 0, 0],
					[4, 95, 2048, 0, 0],
					[4, 95, 2049, 0, 0],
					[4, 95, 2050, 0, 0],
					[4, 95, 2051, 0, 0],
					[4, 95, 2052, 0, 0],
					[4, 95, 2053, 0, 0],
					[4, 95, 2054, 0, 0],
					[4, 95, 2055, 0, 0],
					[4, 95, 2056, 0, 0],
					[4, 95, 2057, 0, 0],
					[4, 95, 2058, 0, 0],
					[4, 95, 2059, 0, 0],
					[4, 95, 2060, 0, 0],
					[4, 95, 2061, 0, 0],
					[4, 95, 2062, 0, 0],
					[4, 95, 2063, 0, 0],
					[4, 95, 2064, 0, 0],
					[4, 95, 2065, 0, 0],
					[4, 95, 2066, 0, 0],
					[4, 95, 2067, 0, 0],
					[4, 95, 2068, 0, 0],
					[4, 95, 2069, 0, 0],
					[4, 95, 2070, 0, 0],
					[4, 95, 2071, 0, 0],
					[4, 95, 2072, 0, 0],
					[4, 95, 2073, 0, 0],
					[4, 95, 2074, 0, 0],
					[4, 96, 2075, 0, 0],
					[4, 96, 2076, 0, 0],
					[4, 96, 2077, 0, 0],
					[4, 96, 2078, 0, 0],
					[4, 96, 2079, 0, 0],
					[4, 96, 2080, 0, 0],
					[4, 96, 2081, 0, 0],
					[4, 96, 2082, 0, 0],
					[4, 97, 2083, 0, 0],
					[4, 97, 2084, 0, 0],
					[4, 97, 2085, 0, 0],
					[4, 97, 2086, 0, 0],
					[4, 97, 2087, 0, 0],
					[4, 97, 2088, 0, 0],
					[4, 97, 2089, 0, 0],
					[4, 97, 2090, 0, 0],
					[4, 97, 2091, 0, 0],
					[4, 97, 2092, 0, 0],
					[4, 97, 2093, 0, 0],
					[4, 86, 2094, 0, 0],
					[4, 86, 2095, 0, 0],
					[4, 86, 2096, 0, 0],
					[4, 86, 2097, 0, 0],
					[4, 86, 2098, 0, 0],
					[4, 86, 2099, 0, 0],
					[4, 86, 2100, 0, 0],
					[4, 86, 2101, 0, 0],
					[4, 86, 2102, 0, 0],
					[4, 88, 2103, 0, 0],
					[4, 88, 2104, 0, 0],
					[4, 88, 2105, 0, 0],
					[4, 89, 2106, 0, 0],
					[4, 89, 2107, 0, 0],
					[4, 89, 2108, 0, 0],
					[4, 89, 2109, 0, 0],
					[4, 89, 2110, 0, 0],
					[4, 90, 2111, 0, 0],
					[4, 90, 2112, 0, 0],
					[4, 90, 2113, 0, 0],
					[4, 90, 2114, 0, 0],
					[4, 90, 2115, 0, 0],
					[4, 98, 2116, 0, 0],
					[4, 98, 2117, 0, 0],
					[4, 98, 2118, 0, 0],
					[4, 98, 2119, 0, 0],
					[4, 98, 2120, 0, 0],
					[4, 98, 2121, 0, 0],
					[4, 99, 2122, 0, 0],
					[4, 99, 2123, 0, 0],
					[4, 99, 2124, 0, 0],
					[4, 99, 2125, 0, 0],
					[4, 99, 2126, 0, 0],
					[4, 99, 2127, 0, 0],
					[4, 99, 2128, 0, 0],
					[4, 99, 2129, 0, 0],
					[4, 99, 2130, 0, 0],
					[4, 99, 2131, 0, 0],
					[4, 99, 2132, 0, 0],
					[4, 99, 2133, 0, 0],
					[4, 99, 2134, 0, 0],
					[4, 99, 2135, 0, 0],
					[4, 99, 2136, 0, 0],
					[4, 99, 2137, 0, 0],
					[4, 99, 2138, 0, 0],
					[4, 99, 2139, 0, 0],
					[4, 99, 2140, 0, 0],
					[4, 99, 2141, 0, 0],
					[4, 99, 2142, 0, 0],
					[4, 99, 2143, 0, 0],
					[4, 99, 2144, 0, 0],
					[4, 99, 2145, 0, 0],
					[4, 99, 2146, 0, 0],
					[4, 99, 2147, 0, 0],
					[4, 99, 2148, 0, 0],
					[4, 99, 2149, 0, 0],
					[4, 99, 2150, 0, 0],
					[4, 99, 2151, 0, 0],
					[4, 99, 2152, 0, 0],
					[4, 99, 2153, 0, 0],
					[4, 99, 2154, 0, 0],
					[4, 99, 2155, 0, 0],
					[4, 100, 2156, 0, 0],
					[4, 100, 2157, 0, 0],
					[4, 100, 2158, 0, 0],
					[4, 100, 2159, 0, 0],
					[4, 100, 2160, 0, 0],
					[4, 100, 2161, 0, 0],
					[4, 100, 2162, 0, 0],
					[4, 101, 2163, 0, 0],
					[4, 101, 2164, 0, 0],
					[4, 101, 2165, 0, 0],
					[4, 101, 2166, 0, 0],
					[4, 101, 2167, 0, 0],
					[4, 101, 2168, 0, 0],
					[4, 101, 2169, 0, 0],
					[4, 98, 2170, 0, 0],
					[4, 102, 2171, 0, 0],
					[4, 102, 2172, 0, 0],
					[4, 102, 2173, 0, 0],
					[4, 102, 2174, 0, 0],
					[4, 102, 2175, 0, 0],
					[4, 102, 2176, 0, 0],
					[4, 102, 2177, 0, 0],
					[4, 98, 2178, 0, 0],
					[4, 100, 2179, 0, 0],
					[4, 101, 2180, 0, 0],
					[4, 102, 2181, 0, 0],
					[4, 103, 2182, 0, 0],
					[4, 103, 2183, 0, 0],
					[4, 103, 2184, 0, 0],
					[4, 103, 2185, 0, 0],
					[4, 103, 2186, 0, 0],
					[4, 103, 2187, 0, 0],
					[4, 103, 2188, 0, 0],
					[4, 98, 2189, 0, 0],
					[4, 98, 2190, 0, 0],
					[4, 98, 2191, 0, 0],
					[4, 98, 2192, 0, 0],
					[4, 100, 2193, 0, 0],
					[4, 100, 2194, 0, 0],
					[4, 100, 2195, 0, 0],
					[4, 100, 2196, 0, 0],
					[4, 101, 2197, 0, 0],
					[4, 101, 2198, 0, 0],
					[4, 101, 2199, 0, 0],
					[4, 102, 2200, 0, 0],
					[4, 102, 2201, 0, 0],
					[4, 102, 2202, 0, 0],
					[4, 103, 2203, 0, 0],
					[4, 103, 2204, 0, 0],
					[4, 103, 2205, 0, 0],
					[4, 103, 2206, 0, 0],
					[4, 103, 2207, 0, 0],
					[4, 14, 2208, 0, 0],
					[4, 98, 2209, 0, 0],
					[4, 96, 2210, 0, 0],
					[4, 97, 2211, 0, 0],
					[4, 97, 2212, 0, 0],
					[4, 97, 2213, 0, 0],
					[4, 97, 2214, 0, 0],
					[4, 97, 2215, 0, 0],
					[4, 97, 2216, 0, 0],
					[4, 97, 2217, 0, 0],
					[4, 97, 2218, 0, 0],
					[4, 97, 2219, 0, 0],
					[4, 97, 2220, 0, 0],
					[4, 97, 2221, 0, 0],
					[4, 104, 2222, 0, 0],
					[4, 104, 2223, 0, 0],
					[4, 104, 2224, 0, 0],
					[4, 104, 2225, 0, 0],
					[4, 104, 2226, 0, 0],
					[4, 104, 2227, 0, 0],
					[4, 104, 2228, 0, 0],
					[4, 104, 2229, 0, 0],
					[4, 104, 2230, 0, 0],
					[4, 104, 2231, 0, 0],
					[4, 104, 2232, 0, 0],
					[4, 104, 2233, 0, 0],
					[4, 104, 2234, 0, 0],
					[4, 105, 2235, 0, 0],
					[4, 105, 2236, 0, 0],
					[4, 105, 2237, 0, 0],
					[4, 105, 2238, 0, 0],
					[4, 105, 2239, 0, 0],
					[4, 105, 2240, 0, 0],
					[4, 105, 2241, 0, 0],
					[4, 105, 2242, 0, 0],
					[4, 105, 2243, 0, 0],
					[4, 105, 2244, 0, 0],
					[4, 105, 2245, 0, 0],
					[4, 106, 2246, 0, 0],
					[4, 106, 2247, 0, 0],
					[4, 106, 2248, 0, 0],
					[4, 106, 2249, 0, 0],
					[4, 106, 2250, 0, 0],
					[4, 106, 2251, 0, 0],
					[4, 106, 2252, 0, 0],
					[4, 106, 2253, 0, 0],
					[4, 106, 2254, 0, 0],
					[4, 106, 2255, 0, 0],
					[4, 106, 2256, 0, 0],
					[4, 106, 2257, 0, 0],
					[4, 106, 2258, 0, 0],
					[4, 104, 2259, 0, 0],
					[4, 106, 2260, 0, 0],
					[4, 104, 2261, 0, 0],
					[4, 106, 2262, 0, 0],
					[4, 96, 2263, 0, 0],
					[4, 96, 2264, 0, 0],
					[4, 96, 2265, 0, 0],
					[4, 96, 2266, 0, 0],
					[4, 58, 2267, 0, 0],
					[4, 37, 2268, 0, 0],
					[4, 14, 2269, 0, 0],
					[4, 107, 2270, 0, 0],
					[4, 107, 2271, 0, 0],
					[4, 107, 2272, 0, 0],
					[4, 107, 2273, 0, 0],
					[4, 107, 2274, 0, 0],
					[4, 107, 2275, 0, 0],
					[4, 107, 2276, 0, 0],
					[4, 107, 2277, 0, 0],
					[4, 107, 2278, 0, 0],
					[4, 107, 2279, 0, 0],
					[4, 107, 2280, 0, 0],
					[4, 107, 2281, 0, 0],
					[4, 107, 2282, 0, 0],
					[4, 107, 2283, 0, 0],
					[4, 107, 2284, 0, 0],
					[4, 107, 2285, 0, 0],
					[4, 107, 2286, 0, 0],
					[4, 107, 2287, 0, 0],
					[4, 107, 2288, 0, 0],
					[4, 107, 2289, 0, 0],
					[4, 107, 2290, 0, 0],
					[4, 107, 2291, 0, 0],
					[4, 107, 2292, 0, 0],
					[4, 107, 2293, 0, 0],
					[4, 107, 2294, 0, 0],
					[4, 107, 2295, 0, 0],
					[4, 107, 2296, 0, 0],
					[4, 107, 2297, 0, 0],
					[4, 107, 2298, 0, 0],
					[4, 107, 2299, 0, 0],
					[4, 107, 2300, 0, 0],
					[4, 107, 2301, 0, 0],
					[4, 107, 2302, 0, 0],
					[4, 107, 2303, 0, 0],
					[4, 107, 2304, 0, 0],
					[4, 108, 2305, 0, 0],
					[4, 108, 2306, 0, 0],
					[4, 108, 2307, 0, 0],
					[4, 108, 2308, 0, 0],
					[4, 108, 2309, 0, 0],
					[4, 108, 2310, 0, 0],
					[4, 108, 2311, 0, 0],
					[4, 108, 2312, 0, 0],
					[4, 108, 2313, 0, 0],
					[4, 108, 2314, 0, 0],
					[4, 108, 2315, 0, 0],
					[4, 108, 2316, 0, 0],
					[4, 108, 2317, 0, 0],
					[4, 108, 2318, 0, 0],
					[4, 109, 2319, 0, 0],
					[4, 109, 2320, 0, 0],
					[4, 109, 2321, 0, 0],
					[4, 109, 2322, 0, 0],
					[4, 109, 2323, 0, 0],
					[4, 109, 2324, 0, 0],
					[4, 109, 2325, 0, 0],
					[4, 109, 2326, 0, 0],
					[4, 109, 2327, 0, 0],
					[4, 109, 2328, 0, 0],
					[4, 109, 2329, 0, 0],
					[4, 109, 2330, 0, 0],
					[4, 109, 2331, 0, 0],
					[4, 13, 2332, 0, 0],
					[4, 13, 2333, 0, 0],
					[4, 14, 2334, 0, 0],
					[4, 35, 2335, 0, 0],
					[4, 37, 2336, 0, 0],
					[4, 58, 2337, 0, 0],
					[4, 90, 2344, 0, 0],
					[4, 100, 2345, 0, 0],
					[4, 6, 2346, 0, 0],
					[4, 13, 2347, 0, 0],
					[4, 13, 2348, 0, 0],
					[4, 4, 2349, 0, 0],
					[4, 6, 2350, 0, 0],
					[4, 7, 2351, 0, 0],
					[4, 92, 2352, 0, 0],
					[4, 18, 2353, 0, 0],
					[4, 93, 2354, 0, 0],
					[4, 83, 2355, 0, 0],
					[4, 83, 2356, 0, 0],
					[4, 14, 2357, 0, 0],
					[4, 89, 2358, 0, 0],
					[4, 92, 2381, 0, 0],
					[4, 61, 2382, 0, 0],
					[4, 61, 2383, 0, 0],
					[4, 61, 2384, 0, 0],
					[4, 86, 2385, 0, 0],
					[4, 43, 2386, 0, 0],
					[4, 111, 2387, 0, 0],
					[4, 111, 2388, 0, 0],
					[4, 111, 2389, 0, 0],
					[4, 111, 2390, 0, 0],
					[4, 111, 2391, 0, 0],
					[4, 111, 2392, 0, 0],
					[4, 111, 2393, 0, 0],
					[4, 111, 2395, 0, 0],
					[4, 111, 2396, 0, 0],
					[4, 111, 2397, 0, 0],
					[4, 111, 2398, 0, 0],
					[4, 111, 2399, 0, 0],
					[4, 111, 2400, 0, 0],
					[4, 6, 2401, 0, 0],
					[4, 95, 2402, 0, 0],
					[4, 95, 2403, 0, 0],
					[4, 86, 2404, 0, 0],
					[4, 6, 2405, 0, 0],
					[4, 4, 2406, 0, 0],
					[4, 7, 2407, 0, 0],
					[4, 18, 2408, 0, 0],
					[4, 14, 2409, 0, 0],
					[4, 86, 2410, 0, 0],
					[4, 97, 2411, 0, 0],
					[4, 26, 2412, 0, 0],
					[4, 16, 2413, 0, 0],
					[4, 13, 2414, 0, 0],
					[4, 105, 2415, 0, 0],
					[4, 35, 2416, 0, 0],
					[4, 106, 2417, 0, 0],
					[4, 98, 2418, 0, 0],
					[4, 54, 2419, 0, 0],
					[4, 108, 2420, 0, 0],
					[4, 100, 2421, 0, 0],
					[4, 109, 2422, 0, 0],
					[4, 9, 2423, 0, 0],
					[4, 43, 2424, 0, 0],
					[4, 101, 2425, 0, 0],
					[4, 102, 2426, 0, 0],
					[4, 37, 2427, 0, 0],
					[4, 61, 2428, 0, 0],
					[4, 103, 2429, 0, 0],
					[4, 15, 2430, 0, 0],
					[4, 74, 2431, 0, 0],
					[4, 104, 2432, 0, 0],
					[4, 34, 2433, 0, 0],
					[4, 59, 2434, 0, 0],
					[4, 51, 2435, 0, 0],
					[4, 19, 2436, 0, 0],
					[4, 41, 2437, 0, 0],
					[4, 8, 2438, 0, 0],
					[4, 58, 2439, 0, 0],
					[4, 84, 2440, 0, 0],
					[4, 60, 2441, 0, 0],
					[4, 57, 2442, 0, 0],
					[4, 83, 2443, 0, 0],
					[4, 42, 2444, 0, 0],
					[4, 75, 2445, 0, 0],
					[4, 78, 2446, 0, 0],
					[4, 79, 2447, 0, 0],
					[4, 80, 2448, 0, 0],
					[4, 81, 2449, 0, 0],
					[4, 107, 2450, 0, 0],
					[4, 82, 2451, 0, 0],
					[4, 99, 2452, 0, 0],
					[4, 94, 2453, 0, 0],
					[4, 95, 2454, 0, 0],
					[4, 96, 2455, 0, 0],
					[4, 111, 2456, 0, 0],
					[4, 85, 2458, 0, 0],
					[4, 87, 2459, 0, 0],
					[4, 89, 2460, 0, 0],
					[4, 91, 2461, 0, 0],
					[4, 93, 2462, 0, 0],
					[4, 88, 2463, 0, 0],
					[4, 90, 2464, 0, 0],
					[4, 92, 2465, 0, 0],
					[4, 33, 2466, 0, 0],
					[4, 35, 2467, 0, 0],
					[4, 29, 2468, 0, 0],
					[4, 29, 2469, 0, 0],
					[4, 29, 2470, 0, 0],
					[4, 112, 2471, 0, 0],
					[4, 112, 2472, 0, 0],
					[4, 112, 2473, 0, 0],
					[4, 112, 2474, 0, 0],
					[4, 112, 2475, 0, 0],
					[4, 112, 2476, 0, 0],
					[4, 112, 2477, 0, 0],
					[4, 112, 2478, 0, 0],
					[4, 112, 2479, 0, 0],
					[4, 112, 2480, 0, 0],
					[4, 112, 2481, 0, 0],
					[4, 112, 2482, 0, 0],
					[4, 112, 2483, 0, 0],
					[4, 86, 2484, 0, 0],
					[4, 113, 2485, 0, 0],
					[4, 113, 2486, 0, 0],
					[4, 113, 2487, 0, 0],
					[4, 113, 2488, 0, 0],
					[4, 113, 2489, 0, 0],
					[4, 113, 2490, 0, 0],
					[4, 113, 2491, 0, 0],
					[4, 113, 2492, 0, 0],
					[4, 113, 2493, 0, 0],
					[4, 113, 2494, 0, 0],
					[4, 113, 2495, 0, 0],
					[4, 113, 2496, 0, 0],
					[4, 113, 2497, 0, 0],
					[4, 113, 2498, 0, 0],
					[4, 113, 2499, 0, 0],
					[4, 113, 2500, 0, 0],
					[4, 113, 2501, 0, 0],
					[4, 113, 2502, 0, 0],
					[4, 113, 2503, 0, 0],
					[4, 113, 2504, 0, 0],
					[4, 113, 2505, 0, 0],
					[4, 113, 2506, 0, 0],
					[4, 113, 2507, 0, 0],
					[4, 113, 2508, 0, 0],
					[4, 114, 2509, 0, 0],
					[4, 114, 2510, 0, 0],
					[4, 114, 2511, 0, 0],
					[4, 114, 2512, 0, 0],
					[4, 114, 2513, 0, 0],
					[4, 114, 2514, 0, 0],
					[4, 114, 2515, 0, 0],
					[4, 114, 2516, 0, 0],
					[4, 114, 2517, 0, 0],
					[4, 114, 2518, 0, 0],
					[4, 114, 2519, 0, 0],
					[4, 115, 2520, 0, 0],
					[4, 115, 2521, 0, 0],
					[4, 115, 2522, 0, 0],
					[4, 115, 2523, 0, 0],
					[4, 115, 2524, 0, 0],
					[4, 115, 2525, 0, 0],
					[4, 115, 2526, 0, 0],
					[4, 115, 2527, 0, 0],
					[4, 115, 2528, 0, 0],
					[4, 115, 2529, 0, 0],
					[4, 115, 2530, 0, 0],
					[4, 115, 2531, 0, 0],
					[4, 115, 2532, 0, 0],
					[4, 115, 2533, 0, 0],
					[4, 115, 2534, 0, 0],
					[4, 115, 2535, 0, 0],
					[4, 115, 2536, 0, 0],
					[4, 115, 2537, 0, 0],
					[4, 115, 2538, 0, 0],
					[4, 115, 2539, 0, 0],
					[4, 115, 2540, 0, 0],
					[4, 115, 2541, 0, 0],
					[4, 115, 2542, 0, 0],
					[4, 115, 2543, 0, 0],
					[4, 115, 2544, 0, 0],
					[4, 115, 2545, 0, 0],
					[4, 115, 2546, 0, 0],
					[4, 115, 2547, 0, 0],
					[4, 115, 2548, 0, 0],
					[4, 115, 2549, 0, 0],
					[4, 115, 2550, 0, 0],
					[4, 115, 2551, 0, 0],
					[4, 115, 2552, 0, 0],
					[4, 115, 2553, 0, 0],
					[4, 115, 2554, 0, 0],
					[4, 115, 2555, 0, 0],
					[4, 115, 2556, 0, 0],
					[4, 115, 2557, 0, 0],
					[4, 115, 2558, 0, 0],
					[4, 116, 2559, 0, 0],
					[4, 116, 2560, 0, 0],
					[4, 116, 2561, 0, 0],
					[4, 116, 2562, 0, 0],
					[4, 116, 2563, 0, 0],
					[4, 116, 2564, 0, 0],
					[4, 116, 2565, 0, 0],
					[4, 116, 2566, 0, 0],
					[4, 116, 2567, 0, 0],
					[4, 116, 2568, 0, 0],
					[4, 116, 2569, 0, 0],
					[4, 95, 2570, 0, 0],
					[4, 95, 2571, 0, 0],
					[4, 95, 2572, 0, 0],
					[4, 95, 2573, 0, 0],
					[4, 113, 2574, 0, 0],
					[4, 113, 2575, 0, 0],
					[4, 113, 2576, 0, 0],
					[4, 117, 2577, 0, 0],
					[4, 117, 2578, 0, 0],
					[4, 117, 2579, 0, 0],
					[4, 117, 2580, 0, 0],
					[4, 117, 2581, 0, 0],
					[4, 117, 2582, 0, 0],
					[4, 117, 2583, 0, 0],
					[4, 117, 2584, 0, 0],
					[4, 117, 2585, 0, 0],
					[4, 117, 2586, 0, 0],
					[4, 117, 2587, 0, 0],
					[4, 117, 2588, 0, 0],
					[4, 117, 2589, 0, 0],
					[4, 117, 2590, 0, 0],
					[4, 117, 2591, 0, 0],
					[4, 117, 2592, 0, 0],
					[4, 117, 2593, 0, 0],
					[4, 117, 2594, 0, 0],
					[4, 117, 2595, 0, 0],
					[4, 117, 2596, 0, 0],
					[4, 117, 2597, 0, 0],
					[4, 117, 2598, 0, 0],
					[4, 16, 2600, 0, 0],
					[4, 16, 2601, 0, 0],
					[4, 9, 2602, 0, 0],
					[4, 9, 2603, 0, 0],
					[4, 43, 2604, 0, 0],
					[4, 83, 2605, 0, 0],
					[4, 45, 2606, 0, 0],
				]
			],
			'vtiger_profile2globalpermissions' => [
				'columns' => ['profileid', 'globalactionid', 'globalactionpermission'],
				'values' => [
					[1, 1, 1],
					[1, 2, 1],
					[2, 1, 1],
					[2, 2, 1],
					[3, 1, 1],
					[3, 2, 1],
					[4, 1, 1],
					[4, 2, 1],
				]
			],
			'vtiger_profile2standardpermissions' => [
				'columns' => ['profileid', 'tabid', 'operation', 'permissions'],
				'values' => [
					[1, 4, 0, 0],
					[1, 4, 1, 0],
					[1, 4, 2, 0],
					[1, 4, 3, 0],
					[1, 4, 4, 0],
					[1, 4, 7, 0],
					[1, 6, 0, 0],
					[1, 6, 1, 0],
					[1, 6, 2, 0],
					[1, 6, 3, 0],
					[1, 6, 4, 0],
					[1, 6, 7, 0],
					[1, 7, 0, 0],
					[1, 7, 1, 0],
					[1, 7, 2, 0],
					[1, 7, 3, 0],
					[1, 7, 4, 0],
					[1, 7, 7, 0],
					[1, 8, 0, 0],
					[1, 8, 1, 0],
					[1, 8, 2, 0],
					[1, 8, 3, 0],
					[1, 8, 4, 0],
					[1, 8, 7, 0],
					[1, 9, 0, 0],
					[1, 9, 1, 0],
					[1, 9, 2, 0],
					[1, 9, 3, 0],
					[1, 9, 4, 0],
					[1, 9, 7, 0],
					[1, 13, 0, 0],
					[1, 13, 1, 0],
					[1, 13, 2, 0],
					[1, 13, 3, 0],
					[1, 13, 4, 0],
					[1, 13, 7, 0],
					[1, 14, 0, 0],
					[1, 14, 1, 0],
					[1, 14, 2, 0],
					[1, 14, 3, 0],
					[1, 14, 4, 0],
					[1, 14, 7, 0],
					[1, 15, 0, 0],
					[1, 15, 1, 0],
					[1, 15, 2, 0],
					[1, 15, 3, 0],
					[1, 15, 4, 0],
					[1, 15, 7, 0],
					[1, 16, 0, 0],
					[1, 16, 1, 0],
					[1, 16, 2, 0],
					[1, 16, 3, 0],
					[1, 16, 4, 0],
					[1, 16, 7, 0],
					[1, 18, 0, 0],
					[1, 18, 1, 0],
					[1, 18, 2, 0],
					[1, 18, 3, 0],
					[1, 18, 4, 0],
					[1, 18, 7, 0],
					[1, 19, 0, 0],
					[1, 19, 1, 0],
					[1, 19, 2, 0],
					[1, 19, 3, 0],
					[1, 19, 4, 0],
					[1, 19, 7, 0],
					[1, 25, 0, 1],
					[1, 25, 1, 1],
					[1, 25, 2, 1],
					[1, 25, 4, 1],
					[1, 25, 7, 1],
					[1, 26, 0, 0],
					[1, 26, 1, 0],
					[1, 26, 2, 0],
					[1, 26, 3, 0],
					[1, 26, 4, 0],
					[1, 26, 7, 0],
					[1, 33, 0, 0],
					[1, 33, 1, 0],
					[1, 33, 2, 0],
					[1, 33, 3, 0],
					[1, 33, 4, 0],
					[1, 33, 7, 0],
					[1, 34, 0, 0],
					[1, 34, 1, 0],
					[1, 34, 2, 0],
					[1, 34, 3, 0],
					[1, 34, 4, 0],
					[1, 34, 7, 0],
					[1, 35, 0, 0],
					[1, 35, 1, 0],
					[1, 35, 2, 0],
					[1, 35, 3, 0],
					[1, 35, 4, 0],
					[1, 35, 7, 0],
					[1, 37, 0, 0],
					[1, 37, 1, 0],
					[1, 37, 2, 0],
					[1, 37, 3, 0],
					[1, 37, 4, 0],
					[1, 37, 7, 0],
					[1, 40, 0, 0],
					[1, 40, 1, 0],
					[1, 40, 2, 0],
					[1, 40, 3, 0],
					[1, 40, 4, 0],
					[1, 40, 7, 0],
					[1, 41, 0, 0],
					[1, 41, 1, 0],
					[1, 41, 2, 0],
					[1, 41, 3, 0],
					[1, 41, 4, 0],
					[1, 41, 7, 0],
					[1, 42, 0, 0],
					[1, 42, 1, 0],
					[1, 42, 2, 0],
					[1, 42, 3, 0],
					[1, 42, 4, 0],
					[1, 42, 7, 0],
					[1, 43, 0, 0],
					[1, 43, 1, 0],
					[1, 43, 2, 0],
					[1, 43, 3, 0],
					[1, 43, 4, 0],
					[1, 43, 7, 0],
					[1, 45, 0, 0],
					[1, 45, 1, 0],
					[1, 45, 2, 0],
					[1, 45, 3, 0],
					[1, 45, 4, 0],
					[1, 45, 7, 0],
					[1, 51, 0, 0],
					[1, 51, 1, 0],
					[1, 51, 2, 0],
					[1, 51, 3, 0],
					[1, 51, 4, 0],
					[1, 51, 7, 0],
					[1, 54, 0, 0],
					[1, 54, 1, 0],
					[1, 54, 2, 0],
					[1, 54, 3, 0],
					[1, 54, 4, 0],
					[1, 54, 7, 0],
					[1, 57, 0, 0],
					[1, 57, 1, 0],
					[1, 57, 2, 0],
					[1, 57, 3, 0],
					[1, 57, 4, 0],
					[1, 57, 7, 0],
					[1, 58, 0, 0],
					[1, 58, 1, 0],
					[1, 58, 2, 0],
					[1, 58, 3, 0],
					[1, 58, 4, 0],
					[1, 58, 7, 0],
					[1, 59, 0, 0],
					[1, 59, 1, 0],
					[1, 59, 2, 0],
					[1, 59, 3, 0],
					[1, 59, 4, 0],
					[1, 59, 7, 0],
					[1, 60, 0, 0],
					[1, 60, 1, 0],
					[1, 60, 2, 0],
					[1, 60, 3, 0],
					[1, 60, 4, 0],
					[1, 60, 7, 0],
					[1, 61, 0, 0],
					[1, 61, 1, 0],
					[1, 61, 2, 0],
					[1, 61, 3, 0],
					[1, 61, 4, 0],
					[1, 61, 7, 0],
					[1, 74, 0, 0],
					[1, 74, 1, 0],
					[1, 74, 2, 0],
					[1, 74, 3, 0],
					[1, 74, 4, 0],
					[1, 74, 7, 0],
					[1, 75, 0, 0],
					[1, 75, 1, 0],
					[1, 75, 2, 0],
					[1, 75, 3, 0],
					[1, 75, 4, 0],
					[1, 75, 7, 0],
					[1, 78, 0, 0],
					[1, 78, 1, 0],
					[1, 78, 2, 0],
					[1, 78, 3, 0],
					[1, 78, 4, 0],
					[1, 78, 7, 0],
					[1, 79, 0, 0],
					[1, 79, 1, 0],
					[1, 79, 2, 0],
					[1, 79, 3, 0],
					[1, 79, 4, 0],
					[1, 79, 7, 0],
					[1, 80, 0, 0],
					[1, 80, 1, 0],
					[1, 80, 2, 0],
					[1, 80, 3, 0],
					[1, 80, 4, 0],
					[1, 80, 7, 0],
					[1, 81, 0, 0],
					[1, 81, 1, 0],
					[1, 81, 2, 0],
					[1, 81, 3, 0],
					[1, 81, 4, 0],
					[1, 81, 7, 0],
					[1, 82, 0, 0],
					[1, 82, 1, 0],
					[1, 82, 2, 0],
					[1, 82, 3, 0],
					[1, 82, 4, 0],
					[1, 82, 7, 0],
					[1, 83, 0, 0],
					[1, 83, 1, 0],
					[1, 83, 2, 0],
					[1, 83, 3, 0],
					[1, 83, 4, 0],
					[1, 83, 7, 0],
					[1, 84, 0, 0],
					[1, 84, 1, 0],
					[1, 84, 2, 0],
					[1, 84, 3, 0],
					[1, 84, 4, 0],
					[1, 84, 7, 0],
					[1, 85, 0, 0],
					[1, 85, 1, 0],
					[1, 85, 2, 0],
					[1, 85, 3, 0],
					[1, 85, 4, 0],
					[1, 85, 7, 0],
					[1, 86, 0, 0],
					[1, 86, 1, 0],
					[1, 86, 2, 0],
					[1, 86, 3, 0],
					[1, 86, 4, 0],
					[1, 86, 7, 0],
					[1, 87, 0, 0],
					[1, 87, 1, 0],
					[1, 87, 2, 0],
					[1, 87, 3, 0],
					[1, 87, 4, 0],
					[1, 87, 7, 0],
					[1, 88, 0, 0],
					[1, 88, 1, 0],
					[1, 88, 2, 0],
					[1, 88, 3, 0],
					[1, 88, 4, 0],
					[1, 88, 7, 0],
					[1, 89, 0, 0],
					[1, 89, 1, 0],
					[1, 89, 2, 0],
					[1, 89, 3, 0],
					[1, 89, 4, 0],
					[1, 89, 7, 0],
					[1, 90, 0, 0],
					[1, 90, 1, 0],
					[1, 90, 2, 0],
					[1, 90, 3, 0],
					[1, 90, 4, 0],
					[1, 90, 7, 0],
					[1, 91, 0, 0],
					[1, 91, 1, 0],
					[1, 91, 2, 0],
					[1, 91, 3, 0],
					[1, 91, 4, 0],
					[1, 91, 7, 0],
					[1, 92, 0, 0],
					[1, 92, 1, 0],
					[1, 92, 2, 0],
					[1, 92, 3, 0],
					[1, 92, 4, 0],
					[1, 92, 7, 0],
					[1, 93, 0, 0],
					[1, 93, 1, 0],
					[1, 93, 2, 0],
					[1, 93, 3, 0],
					[1, 93, 4, 0],
					[1, 93, 7, 0],
					[1, 94, 0, 0],
					[1, 94, 1, 0],
					[1, 94, 2, 0],
					[1, 94, 3, 0],
					[1, 94, 4, 0],
					[1, 94, 7, 0],
					[1, 95, 0, 0],
					[1, 95, 1, 0],
					[1, 95, 2, 0],
					[1, 95, 3, 0],
					[1, 95, 4, 0],
					[1, 95, 7, 0],
					[1, 96, 0, 0],
					[1, 96, 1, 0],
					[1, 96, 2, 0],
					[1, 96, 3, 0],
					[1, 96, 4, 0],
					[1, 96, 7, 0],
					[1, 97, 0, 0],
					[1, 97, 1, 0],
					[1, 97, 2, 0],
					[1, 97, 3, 0],
					[1, 97, 4, 0],
					[1, 97, 7, 0],
					[1, 98, 0, 0],
					[1, 98, 1, 0],
					[1, 98, 2, 0],
					[1, 98, 3, 0],
					[1, 98, 4, 0],
					[1, 98, 7, 0],
					[1, 99, 0, 0],
					[1, 99, 1, 0],
					[1, 99, 2, 0],
					[1, 99, 3, 0],
					[1, 99, 4, 0],
					[1, 99, 7, 0],
					[1, 100, 0, 0],
					[1, 100, 1, 0],
					[1, 100, 2, 0],
					[1, 100, 3, 0],
					[1, 100, 4, 0],
					[1, 100, 7, 0],
					[1, 101, 0, 0],
					[1, 101, 1, 0],
					[1, 101, 2, 0],
					[1, 101, 3, 0],
					[1, 101, 4, 0],
					[1, 101, 7, 0],
					[1, 102, 0, 0],
					[1, 102, 1, 0],
					[1, 102, 2, 0],
					[1, 102, 3, 0],
					[1, 102, 4, 0],
					[1, 102, 7, 0],
					[1, 103, 0, 0],
					[1, 103, 1, 0],
					[1, 103, 2, 0],
					[1, 103, 3, 0],
					[1, 103, 4, 0],
					[1, 103, 7, 0],
					[1, 104, 0, 0],
					[1, 104, 1, 0],
					[1, 104, 2, 0],
					[1, 104, 3, 0],
					[1, 104, 4, 0],
					[1, 104, 7, 0],
					[1, 105, 0, 0],
					[1, 105, 1, 0],
					[1, 105, 2, 0],
					[1, 105, 3, 0],
					[1, 105, 4, 0],
					[1, 105, 7, 0],
					[1, 106, 0, 0],
					[1, 106, 1, 0],
					[1, 106, 2, 0],
					[1, 106, 3, 0],
					[1, 106, 4, 0],
					[1, 106, 7, 0],
					[1, 107, 0, 0],
					[1, 107, 1, 0],
					[1, 107, 2, 0],
					[1, 107, 3, 0],
					[1, 107, 4, 0],
					[1, 107, 7, 0],
					[1, 108, 0, 0],
					[1, 108, 1, 0],
					[1, 108, 2, 0],
					[1, 108, 3, 0],
					[1, 108, 4, 0],
					[1, 108, 7, 0],
					[1, 109, 0, 0],
					[1, 109, 1, 0],
					[1, 109, 2, 0],
					[1, 109, 3, 0],
					[1, 109, 4, 0],
					[1, 109, 7, 0],
					[1, 111, 0, 0],
					[1, 111, 1, 0],
					[1, 111, 2, 0],
					[1, 111, 3, 0],
					[1, 111, 4, 0],
					[1, 111, 7, 0],
					[1, 112, 0, 0],
					[1, 112, 1, 0],
					[1, 112, 2, 0],
					[1, 112, 3, 0],
					[1, 112, 4, 0],
					[1, 112, 7, 0],
					[1, 113, 0, 0],
					[1, 113, 1, 0],
					[1, 113, 2, 0],
					[1, 113, 3, 0],
					[1, 113, 4, 0],
					[1, 113, 7, 0],
					[1, 114, 0, 0],
					[1, 114, 1, 0],
					[1, 114, 2, 0],
					[1, 114, 3, 0],
					[1, 114, 4, 0],
					[1, 114, 7, 0],
					[1, 115, 0, 0],
					[1, 115, 1, 0],
					[1, 115, 2, 0],
					[1, 115, 3, 0],
					[1, 115, 4, 0],
					[1, 115, 7, 0],
					[1, 116, 0, 0],
					[1, 116, 1, 0],
					[1, 116, 2, 0],
					[1, 116, 3, 0],
					[1, 116, 4, 0],
					[1, 116, 7, 0],
					[1, 117, 0, 0],
					[1, 117, 1, 0],
					[1, 117, 2, 0],
					[1, 117, 3, 0],
					[1, 117, 4, 0],
					[1, 117, 7, 0],
					[2, 4, 0, 0],
					[2, 4, 1, 0],
					[2, 4, 2, 1],
					[2, 4, 3, 0],
					[2, 4, 4, 0],
					[2, 4, 7, 0],
					[2, 6, 0, 0],
					[2, 6, 1, 0],
					[2, 6, 2, 1],
					[2, 6, 3, 0],
					[2, 6, 4, 0],
					[2, 6, 7, 0],
					[2, 7, 0, 0],
					[2, 7, 1, 0],
					[2, 7, 2, 1],
					[2, 7, 3, 0],
					[2, 7, 4, 0],
					[2, 7, 7, 0],
					[2, 8, 0, 0],
					[2, 8, 1, 0],
					[2, 8, 2, 1],
					[2, 8, 3, 0],
					[2, 8, 4, 0],
					[2, 8, 7, 0],
					[2, 9, 0, 0],
					[2, 9, 1, 0],
					[2, 9, 2, 1],
					[2, 9, 3, 0],
					[2, 9, 4, 0],
					[2, 9, 7, 0],
					[2, 13, 0, 0],
					[2, 13, 1, 0],
					[2, 13, 2, 1],
					[2, 13, 3, 0],
					[2, 13, 4, 0],
					[2, 13, 7, 0],
					[2, 14, 0, 0],
					[2, 14, 1, 0],
					[2, 14, 2, 1],
					[2, 14, 3, 0],
					[2, 14, 4, 0],
					[2, 14, 7, 0],
					[2, 15, 0, 0],
					[2, 15, 1, 0],
					[2, 15, 2, 1],
					[2, 15, 3, 0],
					[2, 15, 4, 0],
					[2, 15, 7, 0],
					[2, 16, 0, 0],
					[2, 16, 1, 0],
					[2, 16, 2, 1],
					[2, 16, 3, 0],
					[2, 16, 4, 0],
					[2, 16, 7, 0],
					[2, 18, 0, 0],
					[2, 18, 1, 0],
					[2, 18, 2, 1],
					[2, 18, 3, 0],
					[2, 18, 4, 0],
					[2, 18, 7, 0],
					[2, 19, 0, 0],
					[2, 19, 1, 0],
					[2, 19, 2, 1],
					[2, 19, 3, 0],
					[2, 19, 4, 0],
					[2, 19, 7, 0],
					[2, 25, 0, 1],
					[2, 25, 1, 1],
					[2, 25, 2, 1],
					[2, 25, 4, 1],
					[2, 25, 7, 1],
					[2, 26, 0, 0],
					[2, 26, 1, 0],
					[2, 26, 2, 1],
					[2, 26, 3, 0],
					[2, 26, 4, 0],
					[2, 26, 7, 0],
					[2, 33, 0, 0],
					[2, 33, 1, 0],
					[2, 33, 2, 1],
					[2, 33, 3, 0],
					[2, 33, 4, 0],
					[2, 33, 7, 0],
					[2, 34, 0, 0],
					[2, 34, 1, 0],
					[2, 34, 2, 1],
					[2, 34, 3, 0],
					[2, 34, 4, 0],
					[2, 34, 7, 0],
					[2, 35, 0, 0],
					[2, 35, 1, 0],
					[2, 35, 2, 1],
					[2, 35, 3, 0],
					[2, 35, 4, 0],
					[2, 35, 7, 0],
					[2, 37, 0, 0],
					[2, 37, 1, 0],
					[2, 37, 2, 1],
					[2, 37, 3, 0],
					[2, 37, 4, 0],
					[2, 37, 7, 0],
					[2, 40, 0, 0],
					[2, 40, 1, 0],
					[2, 40, 2, 1],
					[2, 40, 3, 0],
					[2, 40, 4, 0],
					[2, 40, 7, 0],
					[2, 41, 0, 0],
					[2, 41, 1, 0],
					[2, 41, 2, 1],
					[2, 41, 3, 0],
					[2, 41, 4, 0],
					[2, 41, 7, 0],
					[2, 42, 0, 0],
					[2, 42, 1, 0],
					[2, 42, 2, 1],
					[2, 42, 3, 0],
					[2, 42, 4, 0],
					[2, 42, 7, 0],
					[2, 43, 0, 0],
					[2, 43, 1, 0],
					[2, 43, 2, 1],
					[2, 43, 3, 0],
					[2, 43, 4, 0],
					[2, 43, 7, 0],
					[2, 45, 0, 0],
					[2, 45, 1, 0],
					[2, 45, 2, 1],
					[2, 45, 3, 0],
					[2, 45, 4, 0],
					[2, 45, 7, 0],
					[2, 51, 0, 0],
					[2, 51, 1, 0],
					[2, 51, 2, 1],
					[2, 51, 3, 0],
					[2, 51, 4, 0],
					[2, 51, 7, 0],
					[2, 54, 0, 0],
					[2, 54, 1, 0],
					[2, 54, 2, 1],
					[2, 54, 3, 0],
					[2, 54, 4, 0],
					[2, 54, 7, 0],
					[2, 57, 0, 0],
					[2, 57, 1, 0],
					[2, 57, 2, 1],
					[2, 57, 3, 0],
					[2, 57, 4, 0],
					[2, 57, 7, 0],
					[2, 58, 0, 0],
					[2, 58, 1, 0],
					[2, 58, 2, 1],
					[2, 58, 3, 0],
					[2, 58, 4, 0],
					[2, 58, 7, 0],
					[2, 59, 0, 0],
					[2, 59, 1, 0],
					[2, 59, 2, 1],
					[2, 59, 3, 0],
					[2, 59, 4, 0],
					[2, 59, 7, 0],
					[2, 60, 0, 0],
					[2, 60, 1, 0],
					[2, 60, 2, 1],
					[2, 60, 3, 0],
					[2, 60, 4, 0],
					[2, 60, 7, 0],
					[2, 61, 0, 0],
					[2, 61, 1, 0],
					[2, 61, 2, 1],
					[2, 61, 3, 0],
					[2, 61, 4, 0],
					[2, 61, 7, 0],
					[2, 74, 0, 0],
					[2, 74, 1, 0],
					[2, 74, 2, 1],
					[2, 74, 3, 0],
					[2, 74, 4, 0],
					[2, 74, 7, 0],
					[2, 75, 0, 0],
					[2, 75, 1, 0],
					[2, 75, 2, 1],
					[2, 75, 3, 0],
					[2, 75, 4, 0],
					[2, 75, 7, 0],
					[2, 78, 0, 0],
					[2, 78, 1, 0],
					[2, 78, 2, 1],
					[2, 78, 3, 0],
					[2, 78, 4, 0],
					[2, 78, 7, 0],
					[2, 79, 0, 0],
					[2, 79, 1, 0],
					[2, 79, 2, 1],
					[2, 79, 3, 0],
					[2, 79, 4, 0],
					[2, 79, 7, 0],
					[2, 80, 0, 0],
					[2, 80, 1, 0],
					[2, 80, 2, 1],
					[2, 80, 3, 0],
					[2, 80, 4, 0],
					[2, 80, 7, 0],
					[2, 81, 0, 0],
					[2, 81, 1, 0],
					[2, 81, 2, 1],
					[2, 81, 3, 0],
					[2, 81, 4, 0],
					[2, 81, 7, 0],
					[2, 82, 0, 0],
					[2, 82, 1, 0],
					[2, 82, 2, 1],
					[2, 82, 3, 0],
					[2, 82, 4, 0],
					[2, 82, 7, 0],
					[2, 83, 0, 0],
					[2, 83, 1, 0],
					[2, 83, 2, 1],
					[2, 83, 3, 0],
					[2, 83, 4, 0],
					[2, 83, 7, 0],
					[2, 84, 0, 0],
					[2, 84, 1, 0],
					[2, 84, 2, 1],
					[2, 84, 3, 0],
					[2, 84, 4, 0],
					[2, 84, 7, 0],
					[2, 85, 0, 0],
					[2, 85, 1, 0],
					[2, 85, 2, 1],
					[2, 85, 3, 0],
					[2, 85, 4, 0],
					[2, 85, 7, 0],
					[2, 86, 0, 0],
					[2, 86, 1, 0],
					[2, 86, 2, 1],
					[2, 86, 3, 0],
					[2, 86, 4, 0],
					[2, 86, 7, 0],
					[2, 87, 0, 0],
					[2, 87, 1, 0],
					[2, 87, 2, 1],
					[2, 87, 3, 0],
					[2, 87, 4, 0],
					[2, 87, 7, 0],
					[2, 88, 0, 0],
					[2, 88, 1, 0],
					[2, 88, 2, 1],
					[2, 88, 3, 0],
					[2, 88, 4, 0],
					[2, 88, 7, 0],
					[2, 89, 0, 0],
					[2, 89, 1, 0],
					[2, 89, 2, 1],
					[2, 89, 3, 0],
					[2, 89, 4, 0],
					[2, 89, 7, 0],
					[2, 90, 0, 0],
					[2, 90, 1, 0],
					[2, 90, 2, 1],
					[2, 90, 3, 0],
					[2, 90, 4, 0],
					[2, 90, 7, 0],
					[2, 91, 0, 0],
					[2, 91, 1, 0],
					[2, 91, 2, 1],
					[2, 91, 3, 0],
					[2, 91, 4, 0],
					[2, 91, 7, 0],
					[2, 92, 0, 0],
					[2, 92, 1, 0],
					[2, 92, 2, 1],
					[2, 92, 3, 0],
					[2, 92, 4, 0],
					[2, 92, 7, 0],
					[2, 93, 0, 0],
					[2, 93, 1, 0],
					[2, 93, 2, 1],
					[2, 93, 3, 0],
					[2, 93, 4, 0],
					[2, 93, 7, 0],
					[2, 94, 0, 0],
					[2, 94, 1, 0],
					[2, 94, 2, 1],
					[2, 94, 3, 0],
					[2, 94, 4, 0],
					[2, 94, 7, 0],
					[2, 95, 0, 0],
					[2, 95, 1, 0],
					[2, 95, 2, 1],
					[2, 95, 3, 0],
					[2, 95, 4, 0],
					[2, 95, 7, 0],
					[2, 96, 0, 0],
					[2, 96, 1, 0],
					[2, 96, 2, 1],
					[2, 96, 3, 0],
					[2, 96, 4, 0],
					[2, 96, 7, 0],
					[2, 97, 0, 0],
					[2, 97, 1, 0],
					[2, 97, 2, 1],
					[2, 97, 3, 0],
					[2, 97, 4, 0],
					[2, 97, 7, 0],
					[2, 98, 0, 0],
					[2, 98, 1, 0],
					[2, 98, 2, 1],
					[2, 98, 3, 0],
					[2, 98, 4, 0],
					[2, 98, 7, 0],
					[2, 99, 0, 0],
					[2, 99, 1, 0],
					[2, 99, 2, 1],
					[2, 99, 3, 0],
					[2, 99, 4, 0],
					[2, 99, 7, 0],
					[2, 100, 0, 0],
					[2, 100, 1, 0],
					[2, 100, 2, 1],
					[2, 100, 3, 0],
					[2, 100, 4, 0],
					[2, 100, 7, 0],
					[2, 101, 0, 0],
					[2, 101, 1, 0],
					[2, 101, 2, 1],
					[2, 101, 3, 0],
					[2, 101, 4, 0],
					[2, 101, 7, 0],
					[2, 102, 0, 0],
					[2, 102, 1, 0],
					[2, 102, 2, 1],
					[2, 102, 3, 0],
					[2, 102, 4, 0],
					[2, 102, 7, 0],
					[2, 103, 0, 0],
					[2, 103, 1, 0],
					[2, 103, 2, 1],
					[2, 103, 3, 0],
					[2, 103, 4, 0],
					[2, 103, 7, 0],
					[2, 104, 0, 0],
					[2, 104, 1, 0],
					[2, 104, 2, 1],
					[2, 104, 3, 0],
					[2, 104, 4, 0],
					[2, 104, 7, 0],
					[2, 105, 0, 0],
					[2, 105, 1, 0],
					[2, 105, 2, 1],
					[2, 105, 3, 0],
					[2, 105, 4, 0],
					[2, 105, 7, 0],
					[2, 106, 0, 0],
					[2, 106, 1, 0],
					[2, 106, 2, 1],
					[2, 106, 3, 0],
					[2, 106, 4, 0],
					[2, 106, 7, 0],
					[2, 107, 0, 0],
					[2, 107, 1, 0],
					[2, 107, 2, 1],
					[2, 107, 3, 0],
					[2, 107, 4, 0],
					[2, 107, 7, 0],
					[2, 108, 0, 0],
					[2, 108, 1, 0],
					[2, 108, 2, 1],
					[2, 108, 3, 0],
					[2, 108, 4, 0],
					[2, 108, 7, 0],
					[2, 109, 0, 0],
					[2, 109, 1, 0],
					[2, 109, 2, 1],
					[2, 109, 3, 0],
					[2, 109, 4, 0],
					[2, 109, 7, 0],
					[2, 111, 0, 0],
					[2, 111, 1, 0],
					[2, 111, 2, 1],
					[2, 111, 3, 0],
					[2, 111, 4, 0],
					[2, 111, 7, 0],
					[2, 112, 0, 0],
					[2, 112, 1, 0],
					[2, 112, 2, 1],
					[2, 112, 3, 0],
					[2, 112, 4, 0],
					[2, 112, 7, 0],
					[2, 113, 0, 0],
					[2, 113, 1, 0],
					[2, 113, 2, 1],
					[2, 113, 3, 0],
					[2, 113, 4, 0],
					[2, 113, 7, 0],
					[2, 114, 0, 0],
					[2, 114, 1, 0],
					[2, 114, 2, 1],
					[2, 114, 3, 0],
					[2, 114, 4, 0],
					[2, 114, 7, 0],
					[2, 115, 0, 0],
					[2, 115, 1, 0],
					[2, 115, 2, 1],
					[2, 115, 3, 0],
					[2, 115, 4, 0],
					[2, 115, 7, 0],
					[2, 116, 0, 0],
					[2, 116, 1, 0],
					[2, 116, 2, 1],
					[2, 116, 3, 0],
					[2, 116, 4, 0],
					[2, 116, 7, 0],
					[2, 117, 0, 0],
					[2, 117, 1, 0],
					[2, 117, 2, 1],
					[2, 117, 3, 0],
					[2, 117, 4, 0],
					[2, 117, 7, 0],
					[3, 4, 0, 0],
					[3, 4, 1, 0],
					[3, 4, 2, 1],
					[3, 4, 3, 0],
					[3, 4, 4, 0],
					[3, 4, 7, 0],
					[3, 6, 0, 0],
					[3, 6, 1, 0],
					[3, 6, 2, 1],
					[3, 6, 3, 0],
					[3, 6, 4, 0],
					[3, 6, 7, 0],
					[3, 7, 0, 0],
					[3, 7, 1, 0],
					[3, 7, 2, 1],
					[3, 7, 3, 0],
					[3, 7, 4, 0],
					[3, 7, 7, 0],
					[3, 8, 0, 0],
					[3, 8, 1, 0],
					[3, 8, 2, 1],
					[3, 8, 3, 0],
					[3, 8, 4, 0],
					[3, 8, 7, 0],
					[3, 9, 0, 0],
					[3, 9, 1, 0],
					[3, 9, 2, 1],
					[3, 9, 3, 0],
					[3, 9, 4, 0],
					[3, 9, 7, 0],
					[3, 13, 0, 0],
					[3, 13, 1, 0],
					[3, 13, 2, 1],
					[3, 13, 3, 0],
					[3, 13, 4, 0],
					[3, 13, 7, 0],
					[3, 14, 0, 0],
					[3, 14, 1, 0],
					[3, 14, 2, 1],
					[3, 14, 3, 0],
					[3, 14, 4, 0],
					[3, 14, 7, 0],
					[3, 15, 0, 0],
					[3, 15, 1, 0],
					[3, 15, 2, 1],
					[3, 15, 3, 0],
					[3, 15, 4, 0],
					[3, 15, 7, 0],
					[3, 16, 0, 0],
					[3, 16, 1, 0],
					[3, 16, 2, 1],
					[3, 16, 3, 0],
					[3, 16, 4, 0],
					[3, 16, 7, 0],
					[3, 18, 0, 0],
					[3, 18, 1, 0],
					[3, 18, 2, 1],
					[3, 18, 3, 0],
					[3, 18, 4, 0],
					[3, 18, 7, 0],
					[3, 19, 0, 0],
					[3, 19, 1, 0],
					[3, 19, 2, 1],
					[3, 19, 3, 0],
					[3, 19, 4, 0],
					[3, 19, 7, 0],
					[3, 25, 0, 1],
					[3, 25, 1, 1],
					[3, 25, 2, 1],
					[3, 25, 4, 1],
					[3, 25, 7, 1],
					[3, 26, 0, 0],
					[3, 26, 1, 0],
					[3, 26, 2, 1],
					[3, 26, 3, 0],
					[3, 26, 4, 0],
					[3, 26, 7, 0],
					[3, 33, 0, 0],
					[3, 33, 1, 0],
					[3, 33, 2, 1],
					[3, 33, 3, 0],
					[3, 33, 4, 0],
					[3, 33, 7, 0],
					[3, 34, 0, 0],
					[3, 34, 1, 0],
					[3, 34, 2, 1],
					[3, 34, 3, 0],
					[3, 34, 4, 0],
					[3, 34, 7, 0],
					[3, 35, 0, 0],
					[3, 35, 1, 0],
					[3, 35, 2, 1],
					[3, 35, 3, 0],
					[3, 35, 4, 0],
					[3, 35, 7, 0],
					[3, 37, 0, 0],
					[3, 37, 1, 0],
					[3, 37, 2, 1],
					[3, 37, 3, 0],
					[3, 37, 4, 0],
					[3, 37, 7, 0],
					[3, 40, 0, 0],
					[3, 40, 1, 0],
					[3, 40, 2, 1],
					[3, 40, 3, 0],
					[3, 40, 4, 0],
					[3, 40, 7, 0],
					[3, 41, 0, 0],
					[3, 41, 1, 0],
					[3, 41, 2, 1],
					[3, 41, 3, 0],
					[3, 41, 4, 0],
					[3, 41, 7, 0],
					[3, 42, 0, 0],
					[3, 42, 1, 0],
					[3, 42, 2, 1],
					[3, 42, 3, 0],
					[3, 42, 4, 0],
					[3, 42, 7, 0],
					[3, 43, 0, 0],
					[3, 43, 1, 0],
					[3, 43, 2, 1],
					[3, 43, 3, 0],
					[3, 43, 4, 0],
					[3, 43, 7, 0],
					[3, 45, 0, 0],
					[3, 45, 1, 0],
					[3, 45, 2, 1],
					[3, 45, 3, 0],
					[3, 45, 4, 0],
					[3, 45, 7, 0],
					[3, 51, 0, 0],
					[3, 51, 1, 0],
					[3, 51, 2, 1],
					[3, 51, 3, 0],
					[3, 51, 4, 0],
					[3, 51, 7, 0],
					[3, 54, 0, 0],
					[3, 54, 1, 0],
					[3, 54, 2, 1],
					[3, 54, 3, 0],
					[3, 54, 4, 0],
					[3, 54, 7, 0],
					[3, 57, 0, 0],
					[3, 57, 1, 0],
					[3, 57, 2, 1],
					[3, 57, 3, 0],
					[3, 57, 4, 0],
					[3, 57, 7, 0],
					[3, 58, 0, 0],
					[3, 58, 1, 0],
					[3, 58, 2, 1],
					[3, 58, 3, 0],
					[3, 58, 4, 0],
					[3, 58, 7, 0],
					[3, 59, 0, 0],
					[3, 59, 1, 0],
					[3, 59, 2, 1],
					[3, 59, 3, 0],
					[3, 59, 4, 0],
					[3, 59, 7, 0],
					[3, 60, 0, 0],
					[3, 60, 1, 0],
					[3, 60, 2, 1],
					[3, 60, 3, 0],
					[3, 60, 4, 0],
					[3, 60, 7, 0],
					[3, 61, 0, 0],
					[3, 61, 1, 0],
					[3, 61, 2, 1],
					[3, 61, 3, 0],
					[3, 61, 4, 0],
					[3, 61, 7, 0],
					[3, 74, 0, 0],
					[3, 74, 1, 0],
					[3, 74, 2, 1],
					[3, 74, 3, 0],
					[3, 74, 4, 0],
					[3, 74, 7, 0],
					[3, 75, 0, 0],
					[3, 75, 1, 0],
					[3, 75, 2, 1],
					[3, 75, 3, 0],
					[3, 75, 4, 0],
					[3, 75, 7, 0],
					[3, 78, 0, 0],
					[3, 78, 1, 0],
					[3, 78, 2, 1],
					[3, 78, 3, 0],
					[3, 78, 4, 0],
					[3, 78, 7, 0],
					[3, 79, 0, 0],
					[3, 79, 1, 0],
					[3, 79, 2, 1],
					[3, 79, 3, 0],
					[3, 79, 4, 0],
					[3, 79, 7, 0],
					[3, 80, 0, 0],
					[3, 80, 1, 0],
					[3, 80, 2, 1],
					[3, 80, 3, 0],
					[3, 80, 4, 0],
					[3, 80, 7, 0],
					[3, 81, 0, 0],
					[3, 81, 1, 0],
					[3, 81, 2, 1],
					[3, 81, 3, 0],
					[3, 81, 4, 0],
					[3, 81, 7, 0],
					[3, 82, 0, 0],
					[3, 82, 1, 0],
					[3, 82, 2, 1],
					[3, 82, 3, 0],
					[3, 82, 4, 0],
					[3, 82, 7, 0],
					[3, 83, 0, 0],
					[3, 83, 1, 0],
					[3, 83, 2, 1],
					[3, 83, 3, 0],
					[3, 83, 4, 0],
					[3, 83, 7, 0],
					[3, 84, 0, 0],
					[3, 84, 1, 0],
					[3, 84, 2, 1],
					[3, 84, 3, 0],
					[3, 84, 4, 0],
					[3, 84, 7, 0],
					[3, 85, 0, 0],
					[3, 85, 1, 0],
					[3, 85, 2, 1],
					[3, 85, 3, 0],
					[3, 85, 4, 0],
					[3, 85, 7, 0],
					[3, 86, 0, 0],
					[3, 86, 1, 0],
					[3, 86, 2, 1],
					[3, 86, 3, 0],
					[3, 86, 4, 0],
					[3, 86, 7, 0],
					[3, 87, 0, 0],
					[3, 87, 1, 0],
					[3, 87, 2, 1],
					[3, 87, 3, 0],
					[3, 87, 4, 0],
					[3, 87, 7, 0],
					[3, 88, 0, 0],
					[3, 88, 1, 0],
					[3, 88, 2, 1],
					[3, 88, 3, 0],
					[3, 88, 4, 0],
					[3, 88, 7, 0],
					[3, 89, 0, 0],
					[3, 89, 1, 0],
					[3, 89, 2, 1],
					[3, 89, 3, 0],
					[3, 89, 4, 0],
					[3, 89, 7, 0],
					[3, 90, 0, 0],
					[3, 90, 1, 0],
					[3, 90, 2, 1],
					[3, 90, 3, 0],
					[3, 90, 4, 0],
					[3, 90, 7, 0],
					[3, 91, 0, 0],
					[3, 91, 1, 0],
					[3, 91, 2, 1],
					[3, 91, 3, 0],
					[3, 91, 4, 0],
					[3, 91, 7, 0],
					[3, 92, 0, 0],
					[3, 92, 1, 0],
					[3, 92, 2, 1],
					[3, 92, 3, 0],
					[3, 92, 4, 0],
					[3, 92, 7, 0],
					[3, 93, 0, 0],
					[3, 93, 1, 0],
					[3, 93, 2, 1],
					[3, 93, 3, 0],
					[3, 93, 4, 0],
					[3, 93, 7, 0],
					[3, 94, 0, 0],
					[3, 94, 1, 0],
					[3, 94, 2, 1],
					[3, 94, 3, 0],
					[3, 94, 4, 0],
					[3, 94, 7, 0],
					[3, 95, 0, 0],
					[3, 95, 1, 0],
					[3, 95, 2, 1],
					[3, 95, 3, 0],
					[3, 95, 4, 0],
					[3, 95, 7, 0],
					[3, 96, 0, 0],
					[3, 96, 1, 0],
					[3, 96, 2, 1],
					[3, 96, 3, 0],
					[3, 96, 4, 0],
					[3, 96, 7, 0],
					[3, 97, 0, 0],
					[3, 97, 1, 0],
					[3, 97, 2, 1],
					[3, 97, 3, 0],
					[3, 97, 4, 0],
					[3, 97, 7, 0],
					[3, 98, 0, 0],
					[3, 98, 1, 0],
					[3, 98, 2, 1],
					[3, 98, 3, 0],
					[3, 98, 4, 0],
					[3, 98, 7, 0],
					[3, 99, 0, 0],
					[3, 99, 1, 0],
					[3, 99, 2, 1],
					[3, 99, 3, 0],
					[3, 99, 4, 0],
					[3, 99, 7, 0],
					[3, 100, 0, 0],
					[3, 100, 1, 0],
					[3, 100, 2, 1],
					[3, 100, 3, 0],
					[3, 100, 4, 0],
					[3, 100, 7, 0],
					[3, 101, 0, 0],
					[3, 101, 1, 0],
					[3, 101, 2, 1],
					[3, 101, 3, 0],
					[3, 101, 4, 0],
					[3, 101, 7, 0],
					[3, 102, 0, 0],
					[3, 102, 1, 0],
					[3, 102, 2, 1],
					[3, 102, 3, 0],
					[3, 102, 4, 0],
					[3, 102, 7, 0],
					[3, 103, 0, 0],
					[3, 103, 1, 0],
					[3, 103, 2, 1],
					[3, 103, 3, 0],
					[3, 103, 4, 0],
					[3, 103, 7, 0],
					[3, 104, 0, 0],
					[3, 104, 1, 0],
					[3, 104, 2, 1],
					[3, 104, 3, 0],
					[3, 104, 4, 0],
					[3, 104, 7, 0],
					[3, 105, 0, 0],
					[3, 105, 1, 0],
					[3, 105, 2, 1],
					[3, 105, 3, 0],
					[3, 105, 4, 0],
					[3, 105, 7, 0],
					[3, 106, 0, 0],
					[3, 106, 1, 0],
					[3, 106, 2, 1],
					[3, 106, 3, 0],
					[3, 106, 4, 0],
					[3, 106, 7, 0],
					[3, 107, 0, 0],
					[3, 107, 1, 0],
					[3, 107, 2, 1],
					[3, 107, 3, 0],
					[3, 107, 4, 0],
					[3, 107, 7, 0],
					[3, 108, 0, 0],
					[3, 108, 1, 0],
					[3, 108, 2, 1],
					[3, 108, 3, 0],
					[3, 108, 4, 0],
					[3, 108, 7, 0],
					[3, 109, 0, 0],
					[3, 109, 1, 0],
					[3, 109, 2, 1],
					[3, 109, 3, 0],
					[3, 109, 4, 0],
					[3, 109, 7, 0],
					[3, 111, 0, 0],
					[3, 111, 1, 0],
					[3, 111, 2, 1],
					[3, 111, 3, 0],
					[3, 111, 4, 0],
					[3, 111, 7, 0],
					[3, 112, 0, 0],
					[3, 112, 1, 0],
					[3, 112, 2, 1],
					[3, 112, 3, 0],
					[3, 112, 4, 0],
					[3, 112, 7, 0],
					[3, 113, 0, 0],
					[3, 113, 1, 0],
					[3, 113, 2, 1],
					[3, 113, 3, 0],
					[3, 113, 4, 0],
					[3, 113, 7, 0],
					[3, 114, 0, 0],
					[3, 114, 1, 0],
					[3, 114, 2, 1],
					[3, 114, 3, 0],
					[3, 114, 4, 0],
					[3, 114, 7, 0],
					[3, 115, 0, 0],
					[3, 115, 1, 0],
					[3, 115, 2, 1],
					[3, 115, 3, 0],
					[3, 115, 4, 0],
					[3, 115, 7, 0],
					[3, 116, 0, 0],
					[3, 116, 1, 0],
					[3, 116, 2, 1],
					[3, 116, 3, 0],
					[3, 116, 4, 0],
					[3, 116, 7, 0],
					[3, 117, 0, 0],
					[3, 117, 1, 0],
					[3, 117, 2, 1],
					[3, 117, 3, 0],
					[3, 117, 4, 0],
					[3, 117, 7, 0],
					[4, 4, 0, 0],
					[4, 4, 1, 1],
					[4, 4, 2, 1],
					[4, 4, 3, 0],
					[4, 4, 4, 0],
					[4, 4, 7, 1],
					[4, 6, 0, 0],
					[4, 6, 1, 1],
					[4, 6, 2, 1],
					[4, 6, 3, 0],
					[4, 6, 4, 0],
					[4, 6, 7, 1],
					[4, 7, 0, 0],
					[4, 7, 1, 1],
					[4, 7, 2, 1],
					[4, 7, 3, 0],
					[4, 7, 4, 0],
					[4, 7, 7, 1],
					[4, 8, 0, 0],
					[4, 8, 1, 1],
					[4, 8, 2, 1],
					[4, 8, 3, 0],
					[4, 8, 4, 0],
					[4, 8, 7, 1],
					[4, 9, 0, 0],
					[4, 9, 1, 1],
					[4, 9, 2, 1],
					[4, 9, 3, 0],
					[4, 9, 4, 0],
					[4, 9, 7, 1],
					[4, 13, 0, 0],
					[4, 13, 1, 1],
					[4, 13, 2, 1],
					[4, 13, 3, 0],
					[4, 13, 4, 0],
					[4, 13, 7, 1],
					[4, 14, 0, 0],
					[4, 14, 1, 1],
					[4, 14, 2, 1],
					[4, 14, 3, 0],
					[4, 14, 4, 0],
					[4, 14, 7, 1],
					[4, 15, 0, 0],
					[4, 15, 1, 1],
					[4, 15, 2, 1],
					[4, 15, 3, 0],
					[4, 15, 4, 0],
					[4, 15, 7, 1],
					[4, 16, 0, 0],
					[4, 16, 1, 1],
					[4, 16, 2, 1],
					[4, 16, 3, 0],
					[4, 16, 4, 0],
					[4, 16, 7, 1],
					[4, 18, 0, 0],
					[4, 18, 1, 1],
					[4, 18, 2, 1],
					[4, 18, 3, 0],
					[4, 18, 4, 0],
					[4, 18, 7, 1],
					[4, 19, 0, 0],
					[4, 19, 1, 1],
					[4, 19, 2, 1],
					[4, 19, 3, 0],
					[4, 19, 4, 0],
					[4, 19, 7, 1],
					[4, 25, 0, 1],
					[4, 25, 1, 1],
					[4, 25, 2, 1],
					[4, 25, 4, 1],
					[4, 25, 7, 1],
					[4, 26, 0, 0],
					[4, 26, 1, 1],
					[4, 26, 2, 1],
					[4, 26, 3, 0],
					[4, 26, 4, 0],
					[4, 26, 7, 1],
					[4, 33, 0, 0],
					[4, 33, 1, 1],
					[4, 33, 2, 1],
					[4, 33, 3, 0],
					[4, 33, 4, 0],
					[4, 33, 7, 1],
					[4, 34, 0, 0],
					[4, 34, 1, 1],
					[4, 34, 2, 1],
					[4, 34, 3, 0],
					[4, 34, 4, 0],
					[4, 34, 7, 1],
					[4, 35, 0, 0],
					[4, 35, 1, 1],
					[4, 35, 2, 1],
					[4, 35, 3, 0],
					[4, 35, 4, 0],
					[4, 35, 7, 1],
					[4, 37, 0, 0],
					[4, 37, 1, 1],
					[4, 37, 2, 1],
					[4, 37, 3, 0],
					[4, 37, 4, 0],
					[4, 37, 7, 1],
					[4, 40, 0, 0],
					[4, 40, 1, 1],
					[4, 40, 2, 1],
					[4, 40, 3, 0],
					[4, 40, 4, 0],
					[4, 40, 7, 1],
					[4, 41, 0, 0],
					[4, 41, 1, 1],
					[4, 41, 2, 1],
					[4, 41, 3, 0],
					[4, 41, 4, 0],
					[4, 41, 7, 1],
					[4, 42, 0, 0],
					[4, 42, 1, 1],
					[4, 42, 2, 1],
					[4, 42, 3, 0],
					[4, 42, 4, 0],
					[4, 42, 7, 1],
					[4, 43, 0, 0],
					[4, 43, 1, 1],
					[4, 43, 2, 1],
					[4, 43, 3, 0],
					[4, 43, 4, 0],
					[4, 43, 7, 1],
					[4, 45, 0, 0],
					[4, 45, 1, 1],
					[4, 45, 2, 1],
					[4, 45, 3, 0],
					[4, 45, 4, 0],
					[4, 45, 7, 1],
					[4, 51, 0, 0],
					[4, 51, 1, 1],
					[4, 51, 2, 1],
					[4, 51, 3, 0],
					[4, 51, 4, 0],
					[4, 51, 7, 1],
					[4, 54, 0, 0],
					[4, 54, 1, 1],
					[4, 54, 2, 1],
					[4, 54, 3, 0],
					[4, 54, 4, 0],
					[4, 54, 7, 1],
					[4, 57, 0, 0],
					[4, 57, 1, 1],
					[4, 57, 2, 1],
					[4, 57, 3, 0],
					[4, 57, 4, 0],
					[4, 57, 7, 1],
					[4, 58, 0, 0],
					[4, 58, 1, 1],
					[4, 58, 2, 1],
					[4, 58, 3, 0],
					[4, 58, 4, 0],
					[4, 58, 7, 1],
					[4, 59, 0, 0],
					[4, 59, 1, 1],
					[4, 59, 2, 1],
					[4, 59, 3, 0],
					[4, 59, 4, 0],
					[4, 59, 7, 1],
					[4, 60, 0, 0],
					[4, 60, 1, 1],
					[4, 60, 2, 1],
					[4, 60, 3, 0],
					[4, 60, 4, 0],
					[4, 60, 7, 1],
					[4, 61, 0, 0],
					[4, 61, 1, 1],
					[4, 61, 2, 1],
					[4, 61, 3, 0],
					[4, 61, 4, 0],
					[4, 61, 7, 1],
					[4, 74, 0, 0],
					[4, 74, 1, 1],
					[4, 74, 2, 1],
					[4, 74, 3, 0],
					[4, 74, 4, 0],
					[4, 74, 7, 1],
					[4, 75, 0, 0],
					[4, 75, 1, 1],
					[4, 75, 2, 1],
					[4, 75, 3, 0],
					[4, 75, 4, 0],
					[4, 75, 7, 1],
					[4, 78, 0, 0],
					[4, 78, 1, 1],
					[4, 78, 2, 1],
					[4, 78, 3, 0],
					[4, 78, 4, 0],
					[4, 78, 7, 1],
					[4, 79, 0, 0],
					[4, 79, 1, 1],
					[4, 79, 2, 1],
					[4, 79, 3, 0],
					[4, 79, 4, 0],
					[4, 79, 7, 1],
					[4, 80, 0, 0],
					[4, 80, 1, 1],
					[4, 80, 2, 1],
					[4, 80, 3, 0],
					[4, 80, 4, 0],
					[4, 80, 7, 1],
					[4, 81, 0, 0],
					[4, 81, 1, 1],
					[4, 81, 2, 1],
					[4, 81, 3, 0],
					[4, 81, 4, 0],
					[4, 81, 7, 1],
					[4, 82, 0, 0],
					[4, 82, 1, 1],
					[4, 82, 2, 1],
					[4, 82, 3, 0],
					[4, 82, 4, 0],
					[4, 82, 7, 1],
					[4, 83, 0, 0],
					[4, 83, 1, 1],
					[4, 83, 2, 1],
					[4, 83, 3, 0],
					[4, 83, 4, 0],
					[4, 83, 7, 1],
					[4, 84, 0, 0],
					[4, 84, 1, 1],
					[4, 84, 2, 1],
					[4, 84, 3, 0],
					[4, 84, 4, 0],
					[4, 84, 7, 1],
					[4, 85, 0, 0],
					[4, 85, 1, 1],
					[4, 85, 2, 1],
					[4, 85, 3, 0],
					[4, 85, 4, 0],
					[4, 85, 7, 1],
					[4, 86, 0, 0],
					[4, 86, 1, 1],
					[4, 86, 2, 1],
					[4, 86, 3, 0],
					[4, 86, 4, 0],
					[4, 86, 7, 1],
					[4, 87, 0, 0],
					[4, 87, 1, 1],
					[4, 87, 2, 1],
					[4, 87, 3, 0],
					[4, 87, 4, 0],
					[4, 87, 7, 1],
					[4, 88, 0, 0],
					[4, 88, 1, 1],
					[4, 88, 2, 1],
					[4, 88, 3, 0],
					[4, 88, 4, 0],
					[4, 88, 7, 1],
					[4, 89, 0, 0],
					[4, 89, 1, 1],
					[4, 89, 2, 1],
					[4, 89, 3, 0],
					[4, 89, 4, 0],
					[4, 89, 7, 1],
					[4, 90, 0, 0],
					[4, 90, 1, 1],
					[4, 90, 2, 1],
					[4, 90, 3, 0],
					[4, 90, 4, 0],
					[4, 90, 7, 1],
					[4, 91, 0, 0],
					[4, 91, 1, 1],
					[4, 91, 2, 1],
					[4, 91, 3, 0],
					[4, 91, 4, 0],
					[4, 91, 7, 1],
					[4, 92, 0, 0],
					[4, 92, 1, 1],
					[4, 92, 2, 1],
					[4, 92, 3, 0],
					[4, 92, 4, 0],
					[4, 92, 7, 1],
					[4, 93, 0, 0],
					[4, 93, 1, 1],
					[4, 93, 2, 1],
					[4, 93, 3, 0],
					[4, 93, 4, 0],
					[4, 93, 7, 1],
					[4, 94, 0, 0],
					[4, 94, 1, 1],
					[4, 94, 2, 1],
					[4, 94, 3, 0],
					[4, 94, 4, 0],
					[4, 94, 7, 1],
					[4, 95, 0, 0],
					[4, 95, 1, 1],
					[4, 95, 2, 1],
					[4, 95, 3, 0],
					[4, 95, 4, 0],
					[4, 95, 7, 1],
					[4, 96, 0, 0],
					[4, 96, 1, 1],
					[4, 96, 2, 1],
					[4, 96, 3, 0],
					[4, 96, 4, 0],
					[4, 96, 7, 1],
					[4, 97, 0, 0],
					[4, 97, 1, 1],
					[4, 97, 2, 1],
					[4, 97, 3, 0],
					[4, 97, 4, 0],
					[4, 97, 7, 1],
					[4, 98, 0, 0],
					[4, 98, 1, 1],
					[4, 98, 2, 1],
					[4, 98, 3, 0],
					[4, 98, 4, 0],
					[4, 98, 7, 1],
					[4, 99, 0, 0],
					[4, 99, 1, 1],
					[4, 99, 2, 1],
					[4, 99, 3, 0],
					[4, 99, 4, 0],
					[4, 99, 7, 1],
					[4, 100, 0, 0],
					[4, 100, 1, 1],
					[4, 100, 2, 1],
					[4, 100, 3, 0],
					[4, 100, 4, 0],
					[4, 100, 7, 1],
					[4, 101, 0, 0],
					[4, 101, 1, 1],
					[4, 101, 2, 1],
					[4, 101, 3, 0],
					[4, 101, 4, 0],
					[4, 101, 7, 1],
					[4, 102, 0, 0],
					[4, 102, 1, 1],
					[4, 102, 2, 1],
					[4, 102, 3, 0],
					[4, 102, 4, 0],
					[4, 102, 7, 1],
					[4, 103, 0, 0],
					[4, 103, 1, 1],
					[4, 103, 2, 1],
					[4, 103, 3, 0],
					[4, 103, 4, 0],
					[4, 103, 7, 1],
					[4, 104, 0, 0],
					[4, 104, 1, 1],
					[4, 104, 2, 1],
					[4, 104, 3, 0],
					[4, 104, 4, 0],
					[4, 104, 7, 1],
					[4, 105, 0, 0],
					[4, 105, 1, 1],
					[4, 105, 2, 1],
					[4, 105, 3, 0],
					[4, 105, 4, 0],
					[4, 105, 7, 1],
					[4, 106, 0, 0],
					[4, 106, 1, 1],
					[4, 106, 2, 1],
					[4, 106, 3, 0],
					[4, 106, 4, 0],
					[4, 106, 7, 1],
					[4, 107, 0, 0],
					[4, 107, 1, 1],
					[4, 107, 2, 1],
					[4, 107, 3, 0],
					[4, 107, 4, 0],
					[4, 107, 7, 1],
					[4, 108, 0, 0],
					[4, 108, 1, 1],
					[4, 108, 2, 1],
					[4, 108, 3, 0],
					[4, 108, 4, 0],
					[4, 108, 7, 1],
					[4, 109, 0, 0],
					[4, 109, 1, 1],
					[4, 109, 2, 1],
					[4, 109, 3, 0],
					[4, 109, 4, 0],
					[4, 109, 7, 1],
					[4, 111, 0, 0],
					[4, 111, 1, 1],
					[4, 111, 2, 1],
					[4, 111, 3, 0],
					[4, 111, 4, 0],
					[4, 111, 7, 1],
					[4, 112, 0, 0],
					[4, 112, 1, 1],
					[4, 112, 2, 1],
					[4, 112, 3, 0],
					[4, 112, 4, 0],
					[4, 112, 7, 1],
					[4, 113, 0, 0],
					[4, 113, 1, 1],
					[4, 113, 2, 1],
					[4, 113, 3, 0],
					[4, 113, 4, 0],
					[4, 113, 7, 1],
					[4, 114, 0, 0],
					[4, 114, 1, 1],
					[4, 114, 2, 1],
					[4, 114, 3, 0],
					[4, 114, 4, 0],
					[4, 114, 7, 1],
					[4, 115, 0, 0],
					[4, 115, 1, 1],
					[4, 115, 2, 1],
					[4, 115, 3, 0],
					[4, 115, 4, 0],
					[4, 115, 7, 1],
					[4, 116, 0, 0],
					[4, 116, 1, 1],
					[4, 116, 2, 1],
					[4, 116, 3, 0],
					[4, 116, 4, 0],
					[4, 116, 7, 1],
					[4, 117, 0, 0],
					[4, 117, 1, 1],
					[4, 117, 2, 1],
					[4, 117, 3, 0],
					[4, 117, 4, 0],
					[4, 117, 7, 1],
				]
			],
			'vtiger_profile2tab' => [
				'columns' => ['profileid', 'tabid', 'permissions'],
				'values' => [
					[1, 48, 0],
					[2, 48, 0],
					[3, 48, 0],
					[4, 48, 0],
					[2, 1, 0],
					[2, 4, 0],
					[2, 6, 0],
					[2, 7, 0],
					[2, 8, 0],
					[2, 9, 0],
					[2, 13, 0],
					[2, 14, 0],
					[2, 15, 0],
					[2, 18, 0],
					[2, 19, 0],
					[2, 24, 0],
					[2, 25, 0],
					[2, 26, 0],
					[2, 27, 0],
					[2, 33, 0],
					[2, 34, 0],
					[2, 35, 0],
					[2, 37, 0],
					[2, 40, 0],
					[2, 41, 0],
					[2, 42, 0],
					[2, 43, 0],
					[2, 44, 0],
					[2, 45, 0],
					[2, 50, 0],
					[2, 51, 0],
					[2, 54, 0],
					[2, 57, 0],
					[2, 58, 0],
					[2, 59, 0],
					[2, 60, 0],
					[2, 61, 0],
					[2, 73, 0],
					[2, 74, 0],
					[2, 75, 0],
					[2, 78, 0],
					[2, 79, 0],
					[2, 80, 0],
					[2, 81, 0],
					[2, 82, 0],
					[2, 83, 0],
					[2, 84, 0],
					[2, 85, 0],
					[2, 86, 0],
					[2, 87, 0],
					[2, 88, 0],
					[2, 89, 0],
					[2, 90, 0],
					[2, 91, 0],
					[2, 92, 0],
					[2, 93, 0],
					[2, 94, 0],
					[2, 95, 0],
					[2, 96, 0],
					[2, 97, 0],
					[2, 98, 0],
					[2, 99, 0],
					[2, 100, 0],
					[2, 101, 0],
					[2, 102, 0],
					[2, 103, 0],
					[2, 104, 0],
					[2, 105, 0],
					[2, 106, 0],
					[2, 107, 0],
					[2, 108, 0],
					[2, 109, 0],
					[2, 110, 0],
					[2, 111, 0],
					[2, 112, 0],
					[2, 113, 0],
					[2, 114, 0],
					[2, 115, 0],
					[2, 116, 0],
					[2, 117, 0],
					[2, 16, 0],
					[1, 1, 0],
					[1, 4, 0],
					[1, 6, 0],
					[1, 7, 0],
					[1, 8, 0],
					[1, 9, 0],
					[1, 13, 0],
					[1, 14, 0],
					[1, 15, 0],
					[1, 18, 0],
					[1, 19, 0],
					[1, 24, 0],
					[1, 25, 0],
					[1, 26, 0],
					[1, 27, 0],
					[1, 33, 0],
					[1, 34, 0],
					[1, 35, 0],
					[1, 37, 0],
					[1, 40, 0],
					[1, 41, 0],
					[1, 42, 0],
					[1, 43, 0],
					[1, 44, 0],
					[1, 45, 0],
					[1, 50, 0],
					[1, 51, 0],
					[1, 54, 0],
					[1, 57, 0],
					[1, 58, 0],
					[1, 59, 0],
					[1, 60, 0],
					[1, 61, 0],
					[1, 73, 0],
					[1, 74, 0],
					[1, 75, 0],
					[1, 78, 0],
					[1, 79, 0],
					[1, 80, 0],
					[1, 81, 0],
					[1, 82, 0],
					[1, 83, 0],
					[1, 84, 0],
					[1, 85, 0],
					[1, 86, 0],
					[1, 87, 0],
					[1, 88, 0],
					[1, 89, 0],
					[1, 90, 0],
					[1, 91, 0],
					[1, 92, 0],
					[1, 93, 0],
					[1, 94, 0],
					[1, 95, 0],
					[1, 96, 0],
					[1, 97, 0],
					[1, 98, 0],
					[1, 99, 0],
					[1, 100, 0],
					[1, 101, 0],
					[1, 102, 0],
					[1, 103, 0],
					[1, 104, 0],
					[1, 105, 0],
					[1, 106, 0],
					[1, 107, 0],
					[1, 108, 0],
					[1, 109, 0],
					[1, 110, 0],
					[1, 111, 0],
					[1, 112, 0],
					[1, 113, 0],
					[1, 114, 0],
					[1, 115, 0],
					[1, 116, 0],
					[1, 117, 0],
					[1, 16, 0],
					[3, 1, 0],
					[3, 4, 0],
					[3, 6, 0],
					[3, 7, 0],
					[3, 8, 0],
					[3, 9, 0],
					[3, 13, 0],
					[3, 14, 0],
					[3, 15, 0],
					[3, 18, 0],
					[3, 19, 0],
					[3, 24, 0],
					[3, 25, 0],
					[3, 26, 0],
					[3, 27, 0],
					[3, 33, 0],
					[3, 34, 0],
					[3, 35, 0],
					[3, 37, 0],
					[3, 40, 0],
					[3, 41, 0],
					[3, 42, 0],
					[3, 43, 0],
					[3, 44, 0],
					[3, 45, 0],
					[3, 50, 0],
					[3, 51, 0],
					[3, 54, 0],
					[3, 57, 0],
					[3, 58, 0],
					[3, 59, 0],
					[3, 60, 0],
					[3, 61, 0],
					[3, 73, 0],
					[3, 74, 0],
					[3, 75, 0],
					[3, 78, 0],
					[3, 79, 0],
					[3, 80, 0],
					[3, 81, 0],
					[3, 82, 0],
					[3, 83, 0],
					[3, 84, 0],
					[3, 85, 0],
					[3, 86, 0],
					[3, 87, 0],
					[3, 88, 0],
					[3, 89, 0],
					[3, 90, 0],
					[3, 91, 0],
					[3, 92, 0],
					[3, 93, 0],
					[3, 94, 0],
					[3, 95, 0],
					[3, 96, 0],
					[3, 97, 0],
					[3, 98, 0],
					[3, 99, 0],
					[3, 100, 0],
					[3, 101, 0],
					[3, 102, 0],
					[3, 103, 0],
					[3, 104, 0],
					[3, 105, 0],
					[3, 106, 0],
					[3, 107, 0],
					[3, 108, 0],
					[3, 109, 0],
					[3, 110, 0],
					[3, 111, 0],
					[3, 112, 0],
					[3, 113, 0],
					[3, 114, 0],
					[3, 115, 0],
					[3, 116, 0],
					[3, 117, 0],
					[3, 16, 0],
					[4, 1, 0],
					[4, 4, 0],
					[4, 6, 0],
					[4, 7, 0],
					[4, 8, 0],
					[4, 9, 0],
					[4, 13, 0],
					[4, 14, 0],
					[4, 15, 0],
					[4, 18, 0],
					[4, 19, 0],
					[4, 24, 0],
					[4, 25, 0],
					[4, 26, 0],
					[4, 27, 0],
					[4, 33, 0],
					[4, 34, 0],
					[4, 35, 0],
					[4, 37, 0],
					[4, 40, 0],
					[4, 41, 0],
					[4, 42, 0],
					[4, 43, 0],
					[4, 44, 0],
					[4, 45, 0],
					[4, 50, 0],
					[4, 51, 0],
					[4, 54, 0],
					[4, 57, 0],
					[4, 58, 0],
					[4, 59, 0],
					[4, 60, 0],
					[4, 61, 0],
					[4, 73, 0],
					[4, 74, 0],
					[4, 75, 0],
					[4, 78, 0],
					[4, 79, 0],
					[4, 80, 0],
					[4, 81, 0],
					[4, 82, 0],
					[4, 83, 0],
					[4, 84, 0],
					[4, 85, 0],
					[4, 86, 0],
					[4, 87, 0],
					[4, 88, 0],
					[4, 89, 0],
					[4, 90, 0],
					[4, 91, 0],
					[4, 92, 0],
					[4, 93, 0],
					[4, 94, 0],
					[4, 95, 0],
					[4, 96, 0],
					[4, 97, 0],
					[4, 98, 0],
					[4, 99, 0],
					[4, 100, 0],
					[4, 101, 0],
					[4, 102, 0],
					[4, 103, 0],
					[4, 104, 0],
					[4, 105, 0],
					[4, 106, 0],
					[4, 107, 0],
					[4, 108, 0],
					[4, 109, 0],
					[4, 110, 0],
					[4, 111, 0],
					[4, 112, 0],
					[4, 113, 0],
					[4, 114, 0],
					[4, 115, 0],
					[4, 116, 0],
					[4, 117, 0],
					[4, 16, 0],
					[1, 118, 0],
					[2, 118, 0],
					[3, 118, 0],
					[4, 118, 0],
				]
			],
			'vtiger_profile2utility' => [
				'columns' => ['profileid', 'tabid', 'activityid', 'permission'],
				'values' => [
					[1, 4, 5, 0],
					[1, 4, 6, 0],
					[1, 4, 8, 1],
					[1, 4, 10, 0],
					[1, 4, 14, 0],
					[1, 4, 15, 0],
					[1, 4, 17, 0],
					[1, 4, 18, 0],
					[1, 4, 19, 0],
					[1, 4, 20, 0],
					[1, 4, 21, 0],
					[1, 4, 22, 0],
					[1, 4, 24, 0],
					[1, 4, 25, 0],
					[1, 4, 26, 0],
					[1, 4, 27, 0],
					[1, 4, 28, 0],
					[1, 4, 30, 0],
					[1, 4, 34, 0],
					[1, 4, 36, 0],
					[1, 4, 37, 0],
					[1, 4, 40, 0],
					[1, 4, 41, 0],
					[1, 4, 42, 0],
					[1, 4, 46, 0],
					[1, 4, 48, 0],
					[1, 4, 49, 0],
					[1, 4, 51, 1],
					[1, 6, 5, 0],
					[1, 6, 6, 0],
					[1, 6, 8, 1],
					[1, 6, 10, 0],
					[1, 6, 14, 0],
					[1, 6, 15, 0],
					[1, 6, 17, 0],
					[1, 6, 18, 0],
					[1, 6, 19, 0],
					[1, 6, 20, 0],
					[1, 6, 21, 0],
					[1, 6, 22, 0],
					[1, 6, 24, 0],
					[1, 6, 25, 0],
					[1, 6, 26, 0],
					[1, 6, 27, 0],
					[1, 6, 28, 0],
					[1, 6, 30, 0],
					[1, 6, 34, 0],
					[1, 6, 36, 0],
					[1, 6, 37, 0],
					[1, 6, 40, 0],
					[1, 6, 41, 0],
					[1, 6, 42, 0],
					[1, 6, 46, 0],
					[1, 6, 48, 0],
					[1, 6, 49, 0],
					[1, 6, 51, 1],
					[1, 7, 5, 0],
					[1, 7, 6, 0],
					[1, 7, 8, 1],
					[1, 7, 9, 0],
					[1, 7, 10, 0],
					[1, 7, 14, 0],
					[1, 7, 15, 0],
					[1, 7, 17, 0],
					[1, 7, 18, 0],
					[1, 7, 19, 0],
					[1, 7, 20, 0],
					[1, 7, 21, 0],
					[1, 7, 22, 0],
					[1, 7, 24, 0],
					[1, 7, 25, 0],
					[1, 7, 26, 0],
					[1, 7, 27, 0],
					[1, 7, 28, 0],
					[1, 7, 30, 0],
					[1, 7, 34, 0],
					[1, 7, 36, 0],
					[1, 7, 37, 0],
					[1, 7, 40, 0],
					[1, 7, 41, 0],
					[1, 7, 42, 0],
					[1, 7, 46, 0],
					[1, 7, 48, 0],
					[1, 7, 49, 0],
					[1, 7, 51, 1],
					[1, 8, 6, 0],
					[1, 8, 14, 0],
					[1, 8, 15, 0],
					[1, 8, 17, 0],
					[1, 8, 18, 0],
					[1, 8, 19, 0],
					[1, 8, 22, 0],
					[1, 8, 23, 0],
					[1, 8, 24, 0],
					[1, 8, 25, 0],
					[1, 8, 26, 0],
					[1, 8, 27, 0],
					[1, 8, 28, 0],
					[1, 8, 30, 0],
					[1, 8, 34, 0],
					[1, 8, 36, 0],
					[1, 8, 37, 0],
					[1, 8, 40, 0],
					[1, 8, 41, 0],
					[1, 8, 42, 0],
					[1, 8, 46, 0],
					[1, 8, 48, 0],
					[1, 8, 49, 0],
					[1, 9, 5, 0],
					[1, 9, 6, 0],
					[1, 9, 14, 0],
					[1, 9, 15, 0],
					[1, 9, 17, 0],
					[1, 9, 18, 0],
					[1, 9, 19, 0],
					[1, 9, 22, 0],
					[1, 9, 24, 0],
					[1, 9, 25, 0],
					[1, 9, 26, 0],
					[1, 9, 27, 0],
					[1, 9, 28, 0],
					[1, 9, 30, 0],
					[1, 9, 31, 0],
					[1, 9, 32, 0],
					[1, 9, 33, 0],
					[1, 9, 34, 0],
					[1, 9, 40, 0],
					[1, 9, 41, 0],
					[1, 9, 42, 0],
					[1, 9, 46, 0],
					[1, 9, 48, 0],
					[1, 9, 49, 0],
					[1, 13, 5, 0],
					[1, 13, 6, 0],
					[1, 13, 8, 1],
					[1, 13, 10, 0],
					[1, 13, 14, 0],
					[1, 13, 15, 0],
					[1, 13, 17, 0],
					[1, 13, 18, 0],
					[1, 13, 19, 0],
					[1, 13, 22, 0],
					[1, 13, 24, 0],
					[1, 13, 25, 0],
					[1, 13, 26, 0],
					[1, 13, 27, 0],
					[1, 13, 28, 0],
					[1, 13, 30, 0],
					[1, 13, 34, 0],
					[1, 13, 36, 0],
					[1, 13, 37, 0],
					[1, 13, 40, 0],
					[1, 13, 41, 0],
					[1, 13, 42, 0],
					[1, 13, 46, 0],
					[1, 13, 48, 0],
					[1, 13, 49, 0],
					[1, 13, 51, 1],
					[1, 14, 5, 0],
					[1, 14, 6, 0],
					[1, 14, 10, 0],
					[1, 14, 14, 0],
					[1, 14, 15, 0],
					[1, 14, 17, 0],
					[1, 14, 18, 0],
					[1, 14, 19, 0],
					[1, 14, 22, 0],
					[1, 14, 24, 0],
					[1, 14, 25, 0],
					[1, 14, 26, 0],
					[1, 14, 27, 0],
					[1, 14, 28, 0],
					[1, 14, 30, 0],
					[1, 14, 34, 0],
					[1, 14, 36, 0],
					[1, 14, 37, 0],
					[1, 14, 40, 0],
					[1, 14, 41, 0],
					[1, 14, 42, 0],
					[1, 14, 46, 0],
					[1, 14, 48, 0],
					[1, 14, 49, 0],
					[1, 15, 14, 0],
					[1, 15, 15, 0],
					[1, 15, 17, 0],
					[1, 15, 18, 0],
					[1, 15, 19, 0],
					[1, 15, 22, 0],
					[1, 15, 24, 0],
					[1, 15, 25, 0],
					[1, 15, 26, 0],
					[1, 15, 27, 0],
					[1, 15, 28, 0],
					[1, 15, 30, 0],
					[1, 15, 34, 0],
					[1, 15, 36, 0],
					[1, 15, 37, 0],
					[1, 15, 40, 0],
					[1, 15, 41, 0],
					[1, 15, 42, 0],
					[1, 15, 46, 0],
					[1, 15, 48, 0],
					[1, 15, 49, 0],
					[1, 16, 5, 0],
					[1, 16, 6, 0],
					[1, 16, 15, 0],
					[1, 18, 5, 0],
					[1, 18, 6, 0],
					[1, 18, 10, 0],
					[1, 18, 14, 0],
					[1, 18, 15, 0],
					[1, 18, 17, 0],
					[1, 18, 18, 0],
					[1, 18, 19, 0],
					[1, 18, 20, 0],
					[1, 18, 21, 0],
					[1, 18, 22, 0],
					[1, 18, 24, 0],
					[1, 18, 25, 0],
					[1, 18, 26, 0],
					[1, 18, 27, 0],
					[1, 18, 28, 0],
					[1, 18, 30, 0],
					[1, 18, 34, 0],
					[1, 18, 36, 0],
					[1, 18, 37, 0],
					[1, 18, 40, 0],
					[1, 18, 41, 0],
					[1, 18, 42, 0],
					[1, 18, 46, 0],
					[1, 18, 48, 0],
					[1, 18, 49, 0],
					[1, 18, 51, 1],
					[1, 19, 5, 1],
					[1, 19, 6, 1],
					[1, 19, 10, 0],
					[1, 19, 14, 0],
					[1, 19, 15, 0],
					[1, 19, 17, 0],
					[1, 19, 18, 0],
					[1, 19, 19, 0],
					[1, 19, 24, 0],
					[1, 19, 25, 0],
					[1, 19, 26, 0],
					[1, 19, 27, 0],
					[1, 19, 28, 0],
					[1, 19, 30, 0],
					[1, 19, 34, 0],
					[1, 19, 36, 0],
					[1, 19, 37, 0],
					[1, 19, 40, 0],
					[1, 19, 41, 0],
					[1, 19, 42, 0],
					[1, 19, 46, 0],
					[1, 19, 48, 0],
					[1, 19, 49, 0],
					[1, 25, 6, 0],
					[1, 25, 13, 0],
					[1, 26, 14, 0],
					[1, 26, 15, 0],
					[1, 26, 17, 0],
					[1, 26, 18, 0],
					[1, 26, 19, 0],
					[1, 26, 22, 0],
					[1, 26, 24, 0],
					[1, 26, 25, 0],
					[1, 26, 26, 0],
					[1, 26, 27, 0],
					[1, 26, 28, 0],
					[1, 26, 30, 0],
					[1, 26, 34, 0],
					[1, 26, 36, 0],
					[1, 26, 37, 0],
					[1, 26, 40, 0],
					[1, 26, 41, 0],
					[1, 26, 42, 0],
					[1, 26, 46, 0],
					[1, 26, 48, 0],
					[1, 26, 49, 0],
					[1, 26, 51, 1],
					[1, 33, 11, 0],
					[1, 33, 12, 0],
					[1, 33, 14, 0],
					[1, 34, 5, 0],
					[1, 34, 6, 0],
					[1, 34, 10, 0],
					[1, 34, 14, 0],
					[1, 34, 15, 0],
					[1, 34, 17, 0],
					[1, 34, 18, 0],
					[1, 34, 19, 0],
					[1, 34, 22, 0],
					[1, 34, 24, 0],
					[1, 34, 25, 0],
					[1, 34, 26, 0],
					[1, 34, 27, 0],
					[1, 34, 28, 0],
					[1, 34, 30, 0],
					[1, 34, 34, 0],
					[1, 34, 36, 0],
					[1, 34, 37, 0],
					[1, 34, 40, 0],
					[1, 34, 41, 0],
					[1, 34, 42, 0],
					[1, 34, 46, 0],
					[1, 34, 48, 0],
					[1, 34, 49, 0],
					[1, 34, 51, 1],
					[1, 35, 5, 0],
					[1, 35, 6, 0],
					[1, 35, 10, 0],
					[1, 35, 14, 0],
					[1, 35, 15, 0],
					[1, 35, 17, 0],
					[1, 35, 18, 0],
					[1, 35, 19, 0],
					[1, 35, 22, 0],
					[1, 35, 24, 0],
					[1, 35, 25, 0],
					[1, 35, 26, 0],
					[1, 35, 27, 0],
					[1, 35, 28, 0],
					[1, 35, 30, 0],
					[1, 35, 34, 0],
					[1, 35, 36, 0],
					[1, 35, 37, 0],
					[1, 35, 40, 0],
					[1, 35, 41, 0],
					[1, 35, 42, 0],
					[1, 35, 46, 0],
					[1, 35, 48, 0],
					[1, 35, 49, 0],
					[1, 37, 5, 0],
					[1, 37, 6, 0],
					[1, 37, 10, 0],
					[1, 37, 14, 0],
					[1, 37, 15, 0],
					[1, 37, 17, 0],
					[1, 37, 18, 0],
					[1, 37, 19, 0],
					[1, 37, 22, 0],
					[1, 37, 24, 0],
					[1, 37, 25, 0],
					[1, 37, 26, 0],
					[1, 37, 27, 0],
					[1, 37, 28, 0],
					[1, 37, 30, 0],
					[1, 37, 34, 0],
					[1, 37, 36, 0],
					[1, 37, 37, 0],
					[1, 37, 38, 0],
					[1, 37, 40, 0],
					[1, 37, 41, 0],
					[1, 37, 42, 0],
					[1, 37, 46, 0],
					[1, 37, 48, 0],
					[1, 37, 49, 0],
					[1, 40, 16, 0],
					[1, 40, 41, 0],
					[1, 40, 42, 0],
					[1, 41, 5, 0],
					[1, 41, 6, 0],
					[1, 41, 10, 0],
					[1, 41, 14, 0],
					[1, 41, 15, 0],
					[1, 41, 17, 0],
					[1, 41, 18, 0],
					[1, 41, 19, 0],
					[1, 41, 22, 0],
					[1, 41, 24, 0],
					[1, 41, 25, 0],
					[1, 41, 26, 0],
					[1, 41, 27, 0],
					[1, 41, 28, 0],
					[1, 41, 30, 0],
					[1, 41, 34, 0],
					[1, 41, 36, 0],
					[1, 41, 37, 0],
					[1, 41, 40, 0],
					[1, 41, 41, 0],
					[1, 41, 42, 0],
					[1, 41, 46, 0],
					[1, 41, 48, 0],
					[1, 41, 49, 0],
					[1, 41, 51, 1],
					[1, 42, 5, 0],
					[1, 42, 6, 0],
					[1, 42, 10, 0],
					[1, 42, 14, 0],
					[1, 42, 15, 0],
					[1, 42, 17, 0],
					[1, 42, 18, 0],
					[1, 42, 19, 0],
					[1, 42, 22, 0],
					[1, 42, 24, 0],
					[1, 42, 25, 0],
					[1, 42, 26, 0],
					[1, 42, 27, 0],
					[1, 42, 28, 0],
					[1, 42, 30, 0],
					[1, 42, 34, 0],
					[1, 42, 36, 0],
					[1, 42, 37, 0],
					[1, 42, 40, 0],
					[1, 42, 41, 0],
					[1, 42, 42, 0],
					[1, 42, 46, 0],
					[1, 42, 48, 0],
					[1, 42, 49, 0],
					[1, 42, 51, 1],
					[1, 43, 5, 0],
					[1, 43, 6, 0],
					[1, 43, 10, 0],
					[1, 43, 14, 0],
					[1, 43, 15, 0],
					[1, 43, 17, 0],
					[1, 43, 18, 0],
					[1, 43, 19, 0],
					[1, 43, 22, 0],
					[1, 43, 24, 0],
					[1, 43, 25, 0],
					[1, 43, 26, 0],
					[1, 43, 27, 0],
					[1, 43, 28, 0],
					[1, 43, 30, 0],
					[1, 43, 34, 0],
					[1, 43, 36, 0],
					[1, 43, 37, 0],
					[1, 43, 40, 0],
					[1, 43, 41, 0],
					[1, 43, 42, 0],
					[1, 43, 46, 0],
					[1, 43, 48, 0],
					[1, 43, 49, 0],
					[1, 43, 51, 1],
					[1, 45, 14, 0],
					[1, 51, 5, 0],
					[1, 51, 6, 0],
					[1, 51, 8, 1],
					[1, 51, 10, 1],
					[1, 51, 14, 0],
					[1, 51, 15, 0],
					[1, 51, 17, 0],
					[1, 51, 18, 0],
					[1, 51, 19, 0],
					[1, 51, 22, 0],
					[1, 51, 24, 0],
					[1, 51, 25, 0],
					[1, 51, 26, 0],
					[1, 51, 27, 0],
					[1, 51, 28, 0],
					[1, 51, 30, 0],
					[1, 51, 34, 0],
					[1, 51, 36, 0],
					[1, 51, 37, 0],
					[1, 51, 40, 0],
					[1, 51, 41, 0],
					[1, 51, 42, 0],
					[1, 51, 46, 0],
					[1, 51, 48, 0],
					[1, 51, 49, 0],
					[1, 54, 5, 0],
					[1, 54, 6, 0],
					[1, 54, 10, 1],
					[1, 54, 14, 0],
					[1, 54, 15, 0],
					[1, 54, 17, 0],
					[1, 54, 18, 0],
					[1, 54, 19, 0],
					[1, 54, 22, 0],
					[1, 54, 26, 0],
					[1, 54, 27, 0],
					[1, 54, 34, 0],
					[1, 54, 35, 0],
					[1, 54, 41, 0],
					[1, 54, 42, 0],
					[1, 54, 46, 0],
					[1, 54, 48, 0],
					[1, 54, 49, 0],
					[1, 54, 50, 0],
					[1, 57, 5, 0],
					[1, 57, 6, 0],
					[1, 57, 8, 1],
					[1, 57, 14, 0],
					[1, 57, 15, 0],
					[1, 57, 17, 0],
					[1, 57, 18, 0],
					[1, 57, 19, 0],
					[1, 57, 22, 0],
					[1, 57, 24, 0],
					[1, 57, 25, 0],
					[1, 57, 26, 0],
					[1, 57, 27, 0],
					[1, 57, 28, 0],
					[1, 57, 30, 0],
					[1, 57, 34, 0],
					[1, 57, 36, 0],
					[1, 57, 37, 0],
					[1, 57, 40, 0],
					[1, 57, 41, 0],
					[1, 57, 42, 0],
					[1, 57, 46, 0],
					[1, 57, 48, 0],
					[1, 57, 49, 0],
					[1, 58, 5, 0],
					[1, 58, 6, 0],
					[1, 58, 8, 1],
					[1, 58, 14, 0],
					[1, 58, 15, 0],
					[1, 58, 17, 0],
					[1, 58, 18, 0],
					[1, 58, 19, 0],
					[1, 58, 22, 0],
					[1, 58, 24, 0],
					[1, 58, 25, 0],
					[1, 58, 26, 0],
					[1, 58, 27, 0],
					[1, 58, 28, 0],
					[1, 58, 30, 0],
					[1, 58, 34, 0],
					[1, 58, 36, 0],
					[1, 58, 37, 0],
					[1, 58, 38, 0],
					[1, 58, 40, 0],
					[1, 58, 41, 0],
					[1, 58, 42, 0],
					[1, 58, 46, 0],
					[1, 58, 48, 0],
					[1, 58, 49, 0],
					[1, 59, 5, 1],
					[1, 59, 6, 1],
					[1, 59, 8, 1],
					[1, 59, 14, 0],
					[1, 59, 15, 0],
					[1, 59, 17, 0],
					[1, 59, 18, 0],
					[1, 59, 19, 0],
					[1, 59, 22, 0],
					[1, 59, 24, 0],
					[1, 59, 25, 0],
					[1, 59, 26, 0],
					[1, 59, 27, 0],
					[1, 59, 28, 0],
					[1, 59, 30, 0],
					[1, 59, 34, 0],
					[1, 59, 36, 0],
					[1, 59, 37, 0],
					[1, 59, 40, 0],
					[1, 59, 41, 0],
					[1, 59, 42, 0],
					[1, 59, 46, 0],
					[1, 59, 48, 0],
					[1, 59, 49, 0],
					[1, 60, 5, 0],
					[1, 60, 6, 0],
					[1, 60, 10, 0],
					[1, 60, 14, 0],
					[1, 60, 15, 0],
					[1, 60, 17, 0],
					[1, 60, 18, 0],
					[1, 60, 19, 0],
					[1, 60, 22, 0],
					[1, 60, 24, 0],
					[1, 60, 25, 0],
					[1, 60, 26, 0],
					[1, 60, 27, 0],
					[1, 60, 28, 0],
					[1, 60, 30, 0],
					[1, 60, 34, 0],
					[1, 60, 36, 0],
					[1, 60, 37, 0],
					[1, 60, 40, 0],
					[1, 60, 41, 0],
					[1, 60, 42, 0],
					[1, 60, 46, 0],
					[1, 60, 48, 0],
					[1, 60, 49, 0],
					[1, 61, 5, 0],
					[1, 61, 6, 0],
					[1, 61, 8, 1],
					[1, 61, 14, 0],
					[1, 61, 15, 0],
					[1, 61, 17, 0],
					[1, 61, 18, 0],
					[1, 61, 19, 0],
					[1, 61, 22, 0],
					[1, 61, 24, 0],
					[1, 61, 25, 0],
					[1, 61, 26, 0],
					[1, 61, 27, 0],
					[1, 61, 28, 0],
					[1, 61, 30, 0],
					[1, 61, 34, 0],
					[1, 61, 36, 0],
					[1, 61, 37, 0],
					[1, 61, 40, 0],
					[1, 61, 41, 0],
					[1, 61, 42, 0],
					[1, 61, 46, 0],
					[1, 61, 48, 0],
					[1, 61, 49, 0],
					[1, 61, 51, 1],
					[1, 74, 5, 0],
					[1, 74, 6, 0],
					[1, 74, 8, 1],
					[1, 74, 14, 0],
					[1, 74, 17, 0],
					[1, 74, 18, 0],
					[1, 74, 19, 0],
					[1, 74, 22, 0],
					[1, 74, 26, 0],
					[1, 74, 27, 0],
					[1, 74, 34, 0],
					[1, 74, 41, 0],
					[1, 74, 42, 0],
					[1, 74, 46, 0],
					[1, 74, 48, 0],
					[1, 74, 49, 0],
					[1, 75, 5, 0],
					[1, 75, 6, 0],
					[1, 75, 8, 1],
					[1, 75, 14, 0],
					[1, 75, 17, 0],
					[1, 75, 18, 0],
					[1, 75, 19, 0],
					[1, 75, 22, 0],
					[1, 75, 24, 0],
					[1, 75, 25, 0],
					[1, 75, 26, 0],
					[1, 75, 27, 0],
					[1, 75, 28, 0],
					[1, 75, 30, 0],
					[1, 75, 34, 0],
					[1, 75, 36, 0],
					[1, 75, 37, 0],
					[1, 75, 40, 0],
					[1, 75, 41, 0],
					[1, 75, 42, 0],
					[1, 75, 46, 0],
					[1, 75, 48, 0],
					[1, 75, 49, 0],
					[1, 78, 5, 0],
					[1, 78, 6, 0],
					[1, 78, 8, 1],
					[1, 78, 14, 0],
					[1, 78, 17, 0],
					[1, 78, 18, 0],
					[1, 78, 19, 0],
					[1, 78, 22, 0],
					[1, 78, 24, 0],
					[1, 78, 25, 0],
					[1, 78, 26, 0],
					[1, 78, 27, 0],
					[1, 78, 28, 0],
					[1, 78, 30, 0],
					[1, 78, 34, 0],
					[1, 78, 36, 0],
					[1, 78, 37, 0],
					[1, 78, 40, 0],
					[1, 78, 41, 0],
					[1, 78, 42, 0],
					[1, 78, 46, 0],
					[1, 78, 48, 0],
					[1, 78, 49, 0],
					[1, 79, 5, 0],
					[1, 79, 6, 0],
					[1, 79, 8, 1],
					[1, 79, 14, 0],
					[1, 79, 17, 0],
					[1, 79, 18, 0],
					[1, 79, 19, 0],
					[1, 79, 22, 0],
					[1, 79, 24, 0],
					[1, 79, 25, 0],
					[1, 79, 26, 0],
					[1, 79, 27, 0],
					[1, 79, 28, 0],
					[1, 79, 30, 0],
					[1, 79, 34, 0],
					[1, 79, 36, 0],
					[1, 79, 37, 0],
					[1, 79, 40, 0],
					[1, 79, 41, 0],
					[1, 79, 42, 0],
					[1, 79, 46, 0],
					[1, 79, 48, 0],
					[1, 79, 49, 0],
					[1, 80, 5, 1],
					[1, 80, 6, 1],
					[1, 80, 8, 1],
					[1, 80, 14, 0],
					[1, 80, 17, 0],
					[1, 80, 18, 0],
					[1, 80, 19, 0],
					[1, 80, 22, 0],
					[1, 80, 24, 0],
					[1, 80, 25, 0],
					[1, 80, 26, 0],
					[1, 80, 27, 0],
					[1, 80, 28, 0],
					[1, 80, 30, 0],
					[1, 80, 34, 0],
					[1, 80, 36, 0],
					[1, 80, 37, 0],
					[1, 80, 40, 0],
					[1, 80, 41, 0],
					[1, 80, 42, 0],
					[1, 80, 46, 0],
					[1, 80, 48, 0],
					[1, 80, 49, 0],
					[1, 81, 5, 0],
					[1, 81, 6, 0],
					[1, 81, 8, 1],
					[1, 81, 14, 0],
					[1, 81, 17, 0],
					[1, 81, 18, 0],
					[1, 81, 19, 0],
					[1, 81, 22, 0],
					[1, 81, 24, 0],
					[1, 81, 25, 0],
					[1, 81, 26, 0],
					[1, 81, 27, 0],
					[1, 81, 28, 0],
					[1, 81, 30, 0],
					[1, 81, 34, 0],
					[1, 81, 36, 0],
					[1, 81, 37, 0],
					[1, 81, 40, 0],
					[1, 81, 41, 0],
					[1, 81, 42, 0],
					[1, 81, 46, 0],
					[1, 81, 48, 0],
					[1, 81, 49, 0],
					[1, 82, 5, 0],
					[1, 82, 6, 0],
					[1, 82, 8, 1],
					[1, 82, 14, 0],
					[1, 82, 17, 0],
					[1, 82, 18, 0],
					[1, 82, 19, 0],
					[1, 82, 22, 0],
					[1, 82, 24, 0],
					[1, 82, 25, 0],
					[1, 82, 26, 0],
					[1, 82, 27, 0],
					[1, 82, 28, 0],
					[1, 82, 30, 0],
					[1, 82, 34, 0],
					[1, 82, 36, 0],
					[1, 82, 37, 0],
					[1, 82, 40, 0],
					[1, 82, 41, 0],
					[1, 82, 42, 0],
					[1, 82, 46, 0],
					[1, 82, 48, 0],
					[1, 82, 49, 0],
					[1, 83, 5, 0],
					[1, 83, 6, 0],
					[1, 83, 8, 1],
					[1, 83, 14, 0],
					[1, 83, 17, 0],
					[1, 83, 18, 0],
					[1, 83, 19, 0],
					[1, 83, 22, 0],
					[1, 83, 24, 0],
					[1, 83, 25, 0],
					[1, 83, 26, 0],
					[1, 83, 27, 0],
					[1, 83, 28, 0],
					[1, 83, 30, 0],
					[1, 83, 34, 0],
					[1, 83, 36, 0],
					[1, 83, 37, 0],
					[1, 83, 40, 0],
					[1, 83, 41, 0],
					[1, 83, 42, 0],
					[1, 83, 46, 0],
					[1, 83, 48, 0],
					[1, 83, 49, 0],
					[1, 84, 5, 0],
					[1, 84, 6, 0],
					[1, 84, 8, 1],
					[1, 84, 14, 0],
					[1, 84, 15, 0],
					[1, 84, 17, 0],
					[1, 84, 18, 0],
					[1, 84, 19, 0],
					[1, 84, 22, 0],
					[1, 84, 24, 0],
					[1, 84, 25, 0],
					[1, 84, 26, 0],
					[1, 84, 27, 0],
					[1, 84, 28, 0],
					[1, 84, 30, 0],
					[1, 84, 34, 0],
					[1, 84, 36, 0],
					[1, 84, 37, 0],
					[1, 84, 40, 0],
					[1, 84, 41, 0],
					[1, 84, 42, 0],
					[1, 84, 46, 0],
					[1, 84, 48, 0],
					[1, 84, 49, 0],
					[1, 85, 5, 0],
					[1, 85, 6, 0],
					[1, 85, 8, 1],
					[1, 85, 14, 0],
					[1, 85, 17, 0],
					[1, 85, 18, 0],
					[1, 85, 19, 0],
					[1, 85, 22, 0],
					[1, 85, 24, 0],
					[1, 85, 25, 0],
					[1, 85, 26, 0],
					[1, 85, 27, 0],
					[1, 85, 28, 0],
					[1, 85, 30, 0],
					[1, 85, 34, 0],
					[1, 85, 36, 0],
					[1, 85, 37, 0],
					[1, 85, 38, 0],
					[1, 85, 39, 0],
					[1, 85, 40, 0],
					[1, 85, 41, 0],
					[1, 85, 42, 0],
					[1, 85, 46, 0],
					[1, 85, 48, 0],
					[1, 85, 49, 0],
					[1, 85, 51, 1],
					[1, 86, 5, 0],
					[1, 86, 6, 0],
					[1, 86, 8, 1],
					[1, 86, 14, 0],
					[1, 86, 15, 0],
					[1, 86, 17, 0],
					[1, 86, 18, 0],
					[1, 86, 19, 0],
					[1, 86, 22, 0],
					[1, 86, 24, 0],
					[1, 86, 25, 0],
					[1, 86, 26, 0],
					[1, 86, 27, 0],
					[1, 86, 28, 0],
					[1, 86, 30, 0],
					[1, 86, 34, 0],
					[1, 86, 36, 0],
					[1, 86, 37, 0],
					[1, 86, 40, 0],
					[1, 86, 41, 0],
					[1, 86, 42, 0],
					[1, 86, 46, 0],
					[1, 86, 48, 0],
					[1, 86, 49, 0],
					[1, 86, 51, 1],
					[1, 87, 5, 0],
					[1, 87, 6, 0],
					[1, 87, 8, 1],
					[1, 87, 14, 0],
					[1, 87, 17, 0],
					[1, 87, 18, 0],
					[1, 87, 19, 0],
					[1, 87, 22, 0],
					[1, 87, 24, 0],
					[1, 87, 25, 0],
					[1, 87, 26, 0],
					[1, 87, 27, 0],
					[1, 87, 28, 0],
					[1, 87, 30, 0],
					[1, 87, 34, 0],
					[1, 87, 36, 0],
					[1, 87, 37, 0],
					[1, 87, 38, 0],
					[1, 87, 39, 0],
					[1, 87, 40, 0],
					[1, 87, 41, 0],
					[1, 87, 42, 0],
					[1, 87, 46, 0],
					[1, 87, 48, 0],
					[1, 87, 49, 0],
					[1, 87, 51, 1],
					[1, 88, 5, 0],
					[1, 88, 6, 0],
					[1, 88, 8, 1],
					[1, 88, 14, 0],
					[1, 88, 17, 0],
					[1, 88, 18, 0],
					[1, 88, 19, 0],
					[1, 88, 22, 0],
					[1, 88, 24, 0],
					[1, 88, 25, 0],
					[1, 88, 26, 0],
					[1, 88, 27, 0],
					[1, 88, 28, 0],
					[1, 88, 30, 0],
					[1, 88, 34, 0],
					[1, 88, 36, 0],
					[1, 88, 37, 0],
					[1, 88, 38, 0],
					[1, 88, 39, 0],
					[1, 88, 40, 0],
					[1, 88, 41, 0],
					[1, 88, 42, 0],
					[1, 88, 46, 0],
					[1, 88, 48, 0],
					[1, 88, 49, 0],
					[1, 88, 51, 1],
					[1, 89, 5, 0],
					[1, 89, 6, 0],
					[1, 89, 8, 1],
					[1, 89, 14, 0],
					[1, 89, 17, 0],
					[1, 89, 18, 0],
					[1, 89, 19, 0],
					[1, 89, 22, 0],
					[1, 89, 24, 0],
					[1, 89, 25, 0],
					[1, 89, 26, 0],
					[1, 89, 27, 0],
					[1, 89, 28, 0],
					[1, 89, 30, 0],
					[1, 89, 34, 0],
					[1, 89, 36, 0],
					[1, 89, 37, 0],
					[1, 89, 38, 0],
					[1, 89, 39, 0],
					[1, 89, 40, 0],
					[1, 89, 41, 0],
					[1, 89, 42, 0],
					[1, 89, 46, 0],
					[1, 89, 48, 0],
					[1, 89, 49, 0],
					[1, 89, 51, 1],
					[1, 90, 5, 0],
					[1, 90, 6, 0],
					[1, 90, 8, 1],
					[1, 90, 14, 0],
					[1, 90, 17, 0],
					[1, 90, 18, 0],
					[1, 90, 19, 0],
					[1, 90, 22, 0],
					[1, 90, 24, 0],
					[1, 90, 25, 0],
					[1, 90, 26, 0],
					[1, 90, 27, 0],
					[1, 90, 28, 0],
					[1, 90, 30, 0],
					[1, 90, 34, 0],
					[1, 90, 36, 0],
					[1, 90, 37, 0],
					[1, 90, 38, 0],
					[1, 90, 39, 0],
					[1, 90, 40, 0],
					[1, 90, 41, 0],
					[1, 90, 42, 0],
					[1, 90, 46, 0],
					[1, 90, 48, 0],
					[1, 90, 49, 0],
					[1, 90, 51, 1],
					[1, 91, 5, 0],
					[1, 91, 6, 0],
					[1, 91, 8, 1],
					[1, 91, 14, 0],
					[1, 91, 17, 0],
					[1, 91, 18, 0],
					[1, 91, 19, 0],
					[1, 91, 22, 0],
					[1, 91, 24, 0],
					[1, 91, 25, 0],
					[1, 91, 26, 0],
					[1, 91, 27, 0],
					[1, 91, 28, 0],
					[1, 91, 30, 0],
					[1, 91, 34, 0],
					[1, 91, 36, 0],
					[1, 91, 37, 0],
					[1, 91, 38, 0],
					[1, 91, 39, 0],
					[1, 91, 40, 0],
					[1, 91, 41, 0],
					[1, 91, 42, 0],
					[1, 91, 46, 0],
					[1, 91, 48, 0],
					[1, 91, 49, 0],
					[1, 91, 51, 1],
					[1, 92, 5, 0],
					[1, 92, 6, 0],
					[1, 92, 8, 1],
					[1, 92, 14, 0],
					[1, 92, 17, 0],
					[1, 92, 18, 0],
					[1, 92, 19, 0],
					[1, 92, 20, 0],
					[1, 92, 22, 0],
					[1, 92, 24, 0],
					[1, 92, 25, 0],
					[1, 92, 26, 0],
					[1, 92, 27, 0],
					[1, 92, 28, 0],
					[1, 92, 30, 0],
					[1, 92, 34, 0],
					[1, 92, 36, 0],
					[1, 92, 37, 0],
					[1, 92, 40, 0],
					[1, 92, 41, 0],
					[1, 92, 42, 0],
					[1, 92, 46, 0],
					[1, 92, 48, 0],
					[1, 92, 49, 0],
					[1, 92, 51, 1],
					[1, 93, 5, 0],
					[1, 93, 6, 0],
					[1, 93, 8, 1],
					[1, 93, 14, 0],
					[1, 93, 17, 0],
					[1, 93, 18, 0],
					[1, 93, 19, 0],
					[1, 93, 20, 0],
					[1, 93, 22, 0],
					[1, 93, 24, 0],
					[1, 93, 25, 0],
					[1, 93, 26, 0],
					[1, 93, 27, 0],
					[1, 93, 28, 0],
					[1, 93, 30, 0],
					[1, 93, 34, 0],
					[1, 93, 36, 0],
					[1, 93, 37, 0],
					[1, 93, 40, 0],
					[1, 93, 41, 0],
					[1, 93, 42, 0],
					[1, 93, 46, 0],
					[1, 93, 48, 0],
					[1, 93, 49, 0],
					[1, 93, 51, 1],
					[1, 94, 5, 0],
					[1, 94, 6, 0],
					[1, 94, 8, 1],
					[1, 94, 14, 0],
					[1, 94, 34, 0],
					[1, 94, 36, 0],
					[1, 94, 37, 0],
					[1, 94, 41, 0],
					[1, 94, 42, 0],
					[1, 94, 46, 0],
					[1, 94, 48, 0],
					[1, 94, 49, 1],
					[1, 94, 51, 1],
					[1, 95, 5, 0],
					[1, 95, 6, 0],
					[1, 95, 8, 1],
					[1, 95, 14, 0],
					[1, 95, 34, 0],
					[1, 95, 36, 0],
					[1, 95, 37, 0],
					[1, 95, 41, 0],
					[1, 95, 42, 0],
					[1, 95, 46, 0],
					[1, 95, 48, 0],
					[1, 95, 49, 1],
					[1, 95, 51, 1],
					[1, 96, 5, 0],
					[1, 96, 6, 0],
					[1, 96, 8, 1],
					[1, 96, 14, 0],
					[1, 96, 34, 0],
					[1, 96, 36, 0],
					[1, 96, 37, 0],
					[1, 96, 41, 0],
					[1, 96, 42, 0],
					[1, 96, 46, 0],
					[1, 96, 48, 0],
					[1, 96, 49, 1],
					[1, 97, 5, 0],
					[1, 97, 6, 0],
					[1, 97, 8, 1],
					[1, 97, 14, 0],
					[1, 97, 34, 0],
					[1, 97, 36, 0],
					[1, 97, 37, 0],
					[1, 97, 41, 0],
					[1, 97, 42, 0],
					[1, 97, 46, 0],
					[1, 97, 48, 0],
					[1, 97, 49, 1],
					[1, 98, 5, 0],
					[1, 98, 6, 0],
					[1, 98, 8, 1],
					[1, 98, 14, 0],
					[1, 98, 34, 0],
					[1, 98, 36, 0],
					[1, 98, 37, 0],
					[1, 98, 38, 0],
					[1, 98, 41, 0],
					[1, 98, 42, 0],
					[1, 98, 46, 0],
					[1, 98, 48, 0],
					[1, 98, 49, 1],
					[1, 99, 5, 0],
					[1, 99, 6, 0],
					[1, 99, 8, 1],
					[1, 99, 14, 0],
					[1, 99, 34, 0],
					[1, 99, 36, 0],
					[1, 99, 37, 0],
					[1, 99, 41, 0],
					[1, 99, 42, 0],
					[1, 99, 46, 0],
					[1, 99, 48, 0],
					[1, 99, 49, 1],
					[1, 100, 5, 0],
					[1, 100, 6, 0],
					[1, 100, 8, 1],
					[1, 100, 14, 0],
					[1, 100, 34, 0],
					[1, 100, 36, 0],
					[1, 100, 37, 0],
					[1, 100, 38, 0],
					[1, 100, 41, 0],
					[1, 100, 42, 0],
					[1, 100, 46, 0],
					[1, 100, 48, 0],
					[1, 100, 49, 1],
					[1, 101, 5, 0],
					[1, 101, 6, 0],
					[1, 101, 8, 1],
					[1, 101, 14, 0],
					[1, 101, 34, 0],
					[1, 101, 36, 0],
					[1, 101, 37, 0],
					[1, 101, 38, 0],
					[1, 101, 41, 0],
					[1, 101, 42, 0],
					[1, 101, 46, 0],
					[1, 101, 48, 0],
					[1, 101, 49, 1],
					[1, 102, 5, 0],
					[1, 102, 6, 0],
					[1, 102, 8, 1],
					[1, 102, 14, 0],
					[1, 102, 34, 0],
					[1, 102, 36, 0],
					[1, 102, 37, 0],
					[1, 102, 38, 0],
					[1, 102, 41, 0],
					[1, 102, 42, 0],
					[1, 102, 46, 0],
					[1, 102, 48, 0],
					[1, 102, 49, 1],
					[1, 103, 5, 0],
					[1, 103, 6, 0],
					[1, 103, 8, 1],
					[1, 103, 14, 0],
					[1, 103, 34, 0],
					[1, 103, 36, 0],
					[1, 103, 37, 0],
					[1, 103, 38, 0],
					[1, 103, 41, 0],
					[1, 103, 42, 0],
					[1, 103, 46, 0],
					[1, 103, 48, 0],
					[1, 103, 49, 1],
					[1, 104, 5, 0],
					[1, 104, 6, 0],
					[1, 104, 8, 1],
					[1, 104, 14, 0],
					[1, 104, 34, 0],
					[1, 104, 36, 0],
					[1, 104, 37, 0],
					[1, 104, 38, 0],
					[1, 104, 41, 0],
					[1, 104, 42, 0],
					[1, 104, 46, 0],
					[1, 104, 48, 0],
					[1, 104, 49, 1],
					[1, 105, 5, 0],
					[1, 105, 6, 0],
					[1, 105, 8, 1],
					[1, 105, 14, 0],
					[1, 105, 34, 0],
					[1, 105, 36, 0],
					[1, 105, 37, 0],
					[1, 105, 38, 0],
					[1, 105, 41, 0],
					[1, 105, 42, 0],
					[1, 105, 46, 0],
					[1, 105, 48, 0],
					[1, 105, 49, 1],
					[1, 106, 5, 0],
					[1, 106, 6, 0],
					[1, 106, 8, 1],
					[1, 106, 14, 0],
					[1, 106, 34, 0],
					[1, 106, 36, 0],
					[1, 106, 37, 0],
					[1, 106, 38, 0],
					[1, 106, 41, 0],
					[1, 106, 42, 0],
					[1, 106, 46, 0],
					[1, 106, 48, 0],
					[1, 106, 49, 1],
					[1, 107, 5, 0],
					[1, 107, 6, 0],
					[1, 107, 8, 1],
					[1, 107, 14, 0],
					[1, 107, 34, 0],
					[1, 107, 36, 0],
					[1, 107, 37, 0],
					[1, 107, 41, 0],
					[1, 107, 42, 0],
					[1, 107, 46, 0],
					[1, 107, 48, 0],
					[1, 107, 49, 1],
					[1, 108, 5, 0],
					[1, 108, 6, 0],
					[1, 108, 8, 1],
					[1, 108, 14, 0],
					[1, 108, 34, 0],
					[1, 108, 36, 0],
					[1, 108, 37, 0],
					[1, 108, 38, 0],
					[1, 108, 41, 0],
					[1, 108, 42, 0],
					[1, 108, 46, 0],
					[1, 108, 48, 0],
					[1, 108, 49, 1],
					[1, 109, 5, 0],
					[1, 109, 6, 0],
					[1, 109, 8, 1],
					[1, 109, 14, 0],
					[1, 109, 34, 0],
					[1, 109, 36, 0],
					[1, 109, 37, 0],
					[1, 109, 38, 0],
					[1, 109, 41, 0],
					[1, 109, 42, 0],
					[1, 109, 46, 0],
					[1, 109, 48, 0],
					[1, 109, 49, 1],
					[1, 111, 5, 0],
					[1, 111, 6, 0],
					[1, 111, 10, 0],
					[1, 111, 14, 0],
					[1, 111, 15, 0],
					[1, 111, 17, 0],
					[1, 111, 18, 0],
					[1, 111, 19, 0],
					[1, 111, 22, 0],
					[1, 111, 24, 0],
					[1, 111, 25, 0],
					[1, 111, 26, 0],
					[1, 111, 27, 0],
					[1, 111, 28, 0],
					[1, 111, 30, 0],
					[1, 111, 34, 0],
					[1, 111, 36, 0],
					[1, 111, 37, 0],
					[1, 111, 40, 0],
					[1, 111, 41, 0],
					[1, 111, 42, 0],
					[1, 111, 44, 0],
					[1, 111, 46, 0],
					[1, 111, 47, 0],
					[1, 111, 48, 0],
					[1, 112, 5, 0],
					[1, 112, 6, 0],
					[1, 112, 14, 0],
					[1, 112, 15, 0],
					[1, 112, 17, 0],
					[1, 112, 18, 0],
					[1, 112, 19, 0],
					[1, 112, 20, 0],
					[1, 112, 22, 0],
					[1, 112, 26, 0],
					[1, 112, 27, 0],
					[1, 112, 34, 0],
					[1, 112, 36, 0],
					[1, 112, 37, 0],
					[1, 112, 40, 0],
					[1, 112, 41, 0],
					[1, 112, 42, 0],
					[1, 112, 46, 0],
					[1, 112, 48, 0],
					[1, 112, 49, 0],
					[1, 113, 5, 0],
					[1, 113, 6, 0],
					[1, 113, 14, 0],
					[1, 113, 15, 0],
					[1, 113, 17, 0],
					[1, 113, 18, 0],
					[1, 113, 19, 0],
					[1, 113, 20, 0],
					[1, 113, 22, 0],
					[1, 113, 26, 0],
					[1, 113, 27, 0],
					[1, 113, 34, 0],
					[1, 113, 36, 0],
					[1, 113, 37, 0],
					[1, 113, 40, 0],
					[1, 113, 41, 0],
					[1, 113, 42, 0],
					[1, 113, 46, 0],
					[1, 113, 48, 0],
					[1, 113, 49, 0],
					[1, 114, 5, 0],
					[1, 114, 6, 0],
					[1, 114, 14, 0],
					[1, 114, 15, 0],
					[1, 114, 17, 0],
					[1, 114, 18, 0],
					[1, 114, 19, 0],
					[1, 114, 20, 0],
					[1, 114, 22, 0],
					[1, 114, 26, 0],
					[1, 114, 27, 0],
					[1, 114, 34, 0],
					[1, 114, 36, 0],
					[1, 114, 37, 0],
					[1, 114, 40, 0],
					[1, 114, 41, 0],
					[1, 114, 42, 0],
					[1, 114, 46, 0],
					[1, 114, 48, 0],
					[1, 114, 49, 0],
					[1, 115, 5, 0],
					[1, 115, 6, 0],
					[1, 115, 14, 0],
					[1, 115, 15, 0],
					[1, 115, 17, 0],
					[1, 115, 18, 0],
					[1, 115, 19, 0],
					[1, 115, 20, 0],
					[1, 115, 22, 0],
					[1, 115, 26, 0],
					[1, 115, 27, 0],
					[1, 115, 34, 0],
					[1, 115, 36, 0],
					[1, 115, 37, 0],
					[1, 115, 40, 0],
					[1, 115, 41, 0],
					[1, 115, 42, 0],
					[1, 115, 46, 0],
					[1, 115, 48, 0],
					[1, 115, 49, 0],
					[1, 116, 5, 0],
					[1, 116, 6, 0],
					[1, 116, 14, 0],
					[1, 116, 15, 0],
					[1, 116, 17, 0],
					[1, 116, 18, 0],
					[1, 116, 19, 0],
					[1, 116, 20, 0],
					[1, 116, 22, 0],
					[1, 116, 26, 0],
					[1, 116, 27, 0],
					[1, 116, 34, 0],
					[1, 116, 36, 0],
					[1, 116, 37, 0],
					[1, 116, 40, 0],
					[1, 116, 41, 0],
					[1, 116, 42, 0],
					[1, 116, 46, 0],
					[1, 116, 48, 0],
					[1, 116, 49, 0],
					[1, 117, 5, 0],
					[1, 117, 6, 0],
					[1, 117, 14, 0],
					[1, 117, 15, 0],
					[1, 117, 17, 0],
					[1, 117, 18, 0],
					[1, 117, 19, 0],
					[1, 117, 20, 0],
					[1, 117, 22, 0],
					[1, 117, 26, 0],
					[1, 117, 27, 0],
					[1, 117, 34, 0],
					[1, 117, 36, 0],
					[1, 117, 37, 0],
					[1, 117, 40, 0],
					[1, 117, 41, 0],
					[1, 117, 42, 0],
					[1, 117, 46, 0],
					[1, 117, 48, 0],
					[1, 117, 49, 0],
					[1, 117, 51, 1],
					[2, 4, 5, 1],
					[2, 4, 6, 1],
					[2, 4, 8, 1],
					[2, 4, 10, 0],
					[2, 4, 14, 0],
					[2, 4, 15, 0],
					[2, 4, 17, 0],
					[2, 4, 18, 0],
					[2, 4, 19, 0],
					[2, 4, 20, 0],
					[2, 4, 21, 0],
					[2, 4, 22, 0],
					[2, 4, 24, 0],
					[2, 4, 25, 0],
					[2, 4, 26, 0],
					[2, 4, 27, 0],
					[2, 4, 28, 0],
					[2, 4, 30, 0],
					[2, 4, 34, 0],
					[2, 4, 36, 0],
					[2, 4, 37, 0],
					[2, 4, 40, 0],
					[2, 4, 41, 0],
					[2, 4, 42, 0],
					[2, 4, 46, 0],
					[2, 4, 48, 0],
					[2, 4, 49, 0],
					[2, 4, 51, 1],
					[2, 6, 5, 1],
					[2, 6, 6, 1],
					[2, 6, 8, 1],
					[2, 6, 10, 0],
					[2, 6, 14, 0],
					[2, 6, 15, 0],
					[2, 6, 17, 0],
					[2, 6, 18, 0],
					[2, 6, 19, 0],
					[2, 6, 20, 0],
					[2, 6, 21, 0],
					[2, 6, 22, 0],
					[2, 6, 24, 0],
					[2, 6, 25, 0],
					[2, 6, 26, 0],
					[2, 6, 27, 0],
					[2, 6, 28, 0],
					[2, 6, 30, 0],
					[2, 6, 34, 0],
					[2, 6, 36, 0],
					[2, 6, 37, 0],
					[2, 6, 40, 0],
					[2, 6, 41, 0],
					[2, 6, 42, 0],
					[2, 6, 46, 0],
					[2, 6, 48, 0],
					[2, 6, 49, 0],
					[2, 6, 51, 1],
					[2, 7, 5, 1],
					[2, 7, 6, 1],
					[2, 7, 8, 1],
					[2, 7, 9, 0],
					[2, 7, 10, 0],
					[2, 7, 14, 0],
					[2, 7, 15, 0],
					[2, 7, 17, 0],
					[2, 7, 18, 0],
					[2, 7, 19, 0],
					[2, 7, 20, 0],
					[2, 7, 21, 0],
					[2, 7, 22, 0],
					[2, 7, 24, 0],
					[2, 7, 25, 0],
					[2, 7, 26, 0],
					[2, 7, 27, 0],
					[2, 7, 28, 0],
					[2, 7, 30, 0],
					[2, 7, 34, 0],
					[2, 7, 36, 0],
					[2, 7, 37, 0],
					[2, 7, 40, 0],
					[2, 7, 41, 0],
					[2, 7, 42, 0],
					[2, 7, 46, 0],
					[2, 7, 48, 0],
					[2, 7, 49, 0],
					[2, 7, 51, 1],
					[2, 8, 6, 1],
					[2, 8, 14, 0],
					[2, 8, 15, 0],
					[2, 8, 17, 0],
					[2, 8, 18, 0],
					[2, 8, 19, 0],
					[2, 8, 22, 0],
					[2, 8, 23, 0],
					[2, 8, 24, 0],
					[2, 8, 25, 0],
					[2, 8, 26, 0],
					[2, 8, 27, 0],
					[2, 8, 28, 0],
					[2, 8, 30, 0],
					[2, 8, 34, 0],
					[2, 8, 36, 0],
					[2, 8, 37, 0],
					[2, 8, 40, 0],
					[2, 8, 41, 0],
					[2, 8, 42, 0],
					[2, 8, 46, 0],
					[2, 8, 48, 0],
					[2, 8, 49, 0],
					[2, 9, 5, 0],
					[2, 9, 6, 0],
					[2, 9, 14, 0],
					[2, 9, 15, 0],
					[2, 9, 17, 0],
					[2, 9, 18, 0],
					[2, 9, 19, 0],
					[2, 9, 22, 0],
					[2, 9, 24, 0],
					[2, 9, 25, 0],
					[2, 9, 26, 0],
					[2, 9, 27, 0],
					[2, 9, 28, 0],
					[2, 9, 30, 0],
					[2, 9, 31, 0],
					[2, 9, 32, 0],
					[2, 9, 33, 0],
					[2, 9, 34, 0],
					[2, 9, 40, 0],
					[2, 9, 41, 0],
					[2, 9, 42, 0],
					[2, 9, 46, 0],
					[2, 9, 48, 0],
					[2, 9, 49, 0],
					[2, 13, 5, 1],
					[2, 13, 6, 1],
					[2, 13, 8, 1],
					[2, 13, 10, 0],
					[2, 13, 14, 0],
					[2, 13, 15, 0],
					[2, 13, 17, 0],
					[2, 13, 18, 0],
					[2, 13, 19, 0],
					[2, 13, 22, 0],
					[2, 13, 24, 0],
					[2, 13, 25, 0],
					[2, 13, 26, 0],
					[2, 13, 27, 0],
					[2, 13, 28, 0],
					[2, 13, 30, 0],
					[2, 13, 34, 0],
					[2, 13, 36, 0],
					[2, 13, 37, 0],
					[2, 13, 40, 0],
					[2, 13, 41, 0],
					[2, 13, 42, 0],
					[2, 13, 46, 0],
					[2, 13, 48, 0],
					[2, 13, 49, 0],
					[2, 13, 51, 1],
					[2, 14, 5, 1],
					[2, 14, 6, 1],
					[2, 14, 10, 0],
					[2, 14, 14, 0],
					[2, 14, 15, 0],
					[2, 14, 17, 0],
					[2, 14, 18, 0],
					[2, 14, 19, 0],
					[2, 14, 22, 0],
					[2, 14, 24, 0],
					[2, 14, 25, 0],
					[2, 14, 26, 0],
					[2, 14, 27, 0],
					[2, 14, 28, 0],
					[2, 14, 30, 0],
					[2, 14, 34, 0],
					[2, 14, 36, 0],
					[2, 14, 37, 0],
					[2, 14, 40, 0],
					[2, 14, 41, 0],
					[2, 14, 42, 0],
					[2, 14, 46, 0],
					[2, 14, 48, 0],
					[2, 14, 49, 0],
					[2, 15, 14, 0],
					[2, 15, 15, 0],
					[2, 15, 17, 0],
					[2, 15, 18, 0],
					[2, 15, 19, 0],
					[2, 15, 22, 0],
					[2, 15, 24, 0],
					[2, 15, 25, 0],
					[2, 15, 26, 0],
					[2, 15, 27, 0],
					[2, 15, 28, 0],
					[2, 15, 30, 0],
					[2, 15, 34, 0],
					[2, 15, 36, 0],
					[2, 15, 37, 0],
					[2, 15, 40, 0],
					[2, 15, 41, 0],
					[2, 15, 42, 0],
					[2, 15, 46, 0],
					[2, 15, 48, 0],
					[2, 15, 49, 0],
					[2, 16, 5, 0],
					[2, 16, 6, 0],
					[2, 16, 15, 0],
					[2, 18, 5, 1],
					[2, 18, 6, 1],
					[2, 18, 10, 0],
					[2, 18, 14, 0],
					[2, 18, 15, 0],
					[2, 18, 17, 0],
					[2, 18, 18, 0],
					[2, 18, 19, 0],
					[2, 18, 20, 0],
					[2, 18, 21, 0],
					[2, 18, 22, 0],
					[2, 18, 24, 0],
					[2, 18, 25, 0],
					[2, 18, 26, 0],
					[2, 18, 27, 0],
					[2, 18, 28, 0],
					[2, 18, 30, 0],
					[2, 18, 34, 0],
					[2, 18, 36, 0],
					[2, 18, 37, 0],
					[2, 18, 40, 0],
					[2, 18, 41, 0],
					[2, 18, 42, 0],
					[2, 18, 46, 0],
					[2, 18, 48, 0],
					[2, 18, 49, 0],
					[2, 18, 51, 1],
					[2, 19, 5, 1],
					[2, 19, 6, 1],
					[2, 19, 10, 0],
					[2, 19, 14, 0],
					[2, 19, 15, 0],
					[2, 19, 17, 0],
					[2, 19, 18, 0],
					[2, 19, 19, 0],
					[2, 19, 24, 0],
					[2, 19, 25, 0],
					[2, 19, 26, 0],
					[2, 19, 27, 0],
					[2, 19, 28, 0],
					[2, 19, 30, 0],
					[2, 19, 34, 0],
					[2, 19, 36, 0],
					[2, 19, 37, 0],
					[2, 19, 40, 0],
					[2, 19, 41, 0],
					[2, 19, 42, 0],
					[2, 19, 46, 0],
					[2, 19, 48, 0],
					[2, 19, 49, 0],
					[2, 25, 6, 0],
					[2, 25, 13, 0],
					[2, 26, 14, 0],
					[2, 26, 15, 0],
					[2, 26, 17, 0],
					[2, 26, 18, 0],
					[2, 26, 19, 0],
					[2, 26, 22, 0],
					[2, 26, 24, 0],
					[2, 26, 25, 0],
					[2, 26, 26, 0],
					[2, 26, 27, 0],
					[2, 26, 28, 0],
					[2, 26, 30, 0],
					[2, 26, 34, 0],
					[2, 26, 36, 0],
					[2, 26, 37, 0],
					[2, 26, 40, 0],
					[2, 26, 41, 0],
					[2, 26, 42, 0],
					[2, 26, 46, 0],
					[2, 26, 48, 0],
					[2, 26, 49, 0],
					[2, 26, 51, 1],
					[2, 33, 11, 0],
					[2, 33, 12, 0],
					[2, 33, 14, 0],
					[2, 34, 5, 0],
					[2, 34, 6, 0],
					[2, 34, 10, 0],
					[2, 34, 14, 0],
					[2, 34, 15, 0],
					[2, 34, 17, 0],
					[2, 34, 18, 0],
					[2, 34, 19, 0],
					[2, 34, 22, 0],
					[2, 34, 24, 0],
					[2, 34, 25, 0],
					[2, 34, 26, 0],
					[2, 34, 27, 0],
					[2, 34, 28, 0],
					[2, 34, 30, 0],
					[2, 34, 34, 0],
					[2, 34, 36, 0],
					[2, 34, 37, 0],
					[2, 34, 40, 0],
					[2, 34, 41, 0],
					[2, 34, 42, 0],
					[2, 34, 46, 0],
					[2, 34, 48, 0],
					[2, 34, 49, 0],
					[2, 34, 51, 1],
					[2, 35, 5, 0],
					[2, 35, 6, 0],
					[2, 35, 10, 0],
					[2, 35, 14, 0],
					[2, 35, 15, 0],
					[2, 35, 17, 0],
					[2, 35, 18, 0],
					[2, 35, 19, 0],
					[2, 35, 22, 0],
					[2, 35, 24, 0],
					[2, 35, 25, 0],
					[2, 35, 26, 0],
					[2, 35, 27, 0],
					[2, 35, 28, 0],
					[2, 35, 30, 0],
					[2, 35, 34, 0],
					[2, 35, 36, 0],
					[2, 35, 37, 0],
					[2, 35, 40, 0],
					[2, 35, 41, 0],
					[2, 35, 42, 0],
					[2, 35, 46, 0],
					[2, 35, 48, 0],
					[2, 35, 49, 0],
					[2, 37, 5, 0],
					[2, 37, 6, 0],
					[2, 37, 10, 0],
					[2, 37, 14, 0],
					[2, 37, 15, 0],
					[2, 37, 17, 0],
					[2, 37, 18, 0],
					[2, 37, 19, 0],
					[2, 37, 22, 0],
					[2, 37, 24, 0],
					[2, 37, 25, 0],
					[2, 37, 26, 0],
					[2, 37, 27, 0],
					[2, 37, 28, 0],
					[2, 37, 30, 0],
					[2, 37, 34, 0],
					[2, 37, 36, 0],
					[2, 37, 37, 0],
					[2, 37, 38, 0],
					[2, 37, 40, 0],
					[2, 37, 41, 0],
					[2, 37, 42, 0],
					[2, 37, 46, 0],
					[2, 37, 48, 0],
					[2, 37, 49, 0],
					[2, 40, 16, 0],
					[2, 40, 41, 0],
					[2, 40, 42, 0],
					[2, 41, 5, 0],
					[2, 41, 6, 0],
					[2, 41, 10, 0],
					[2, 41, 14, 0],
					[2, 41, 15, 0],
					[2, 41, 17, 0],
					[2, 41, 18, 0],
					[2, 41, 19, 0],
					[2, 41, 22, 0],
					[2, 41, 24, 0],
					[2, 41, 25, 0],
					[2, 41, 26, 0],
					[2, 41, 27, 0],
					[2, 41, 28, 0],
					[2, 41, 30, 0],
					[2, 41, 34, 0],
					[2, 41, 36, 0],
					[2, 41, 37, 0],
					[2, 41, 40, 0],
					[2, 41, 41, 0],
					[2, 41, 42, 0],
					[2, 41, 46, 0],
					[2, 41, 48, 0],
					[2, 41, 49, 0],
					[2, 41, 51, 1],
					[2, 42, 5, 0],
					[2, 42, 6, 0],
					[2, 42, 10, 0],
					[2, 42, 14, 0],
					[2, 42, 15, 0],
					[2, 42, 17, 0],
					[2, 42, 18, 0],
					[2, 42, 19, 0],
					[2, 42, 22, 0],
					[2, 42, 24, 0],
					[2, 42, 25, 0],
					[2, 42, 26, 0],
					[2, 42, 27, 0],
					[2, 42, 28, 0],
					[2, 42, 30, 0],
					[2, 42, 34, 0],
					[2, 42, 36, 0],
					[2, 42, 37, 0],
					[2, 42, 40, 0],
					[2, 42, 41, 0],
					[2, 42, 42, 0],
					[2, 42, 46, 0],
					[2, 42, 48, 0],
					[2, 42, 49, 0],
					[2, 42, 51, 1],
					[2, 43, 5, 0],
					[2, 43, 6, 0],
					[2, 43, 10, 0],
					[2, 43, 14, 0],
					[2, 43, 15, 0],
					[2, 43, 17, 0],
					[2, 43, 18, 0],
					[2, 43, 19, 0],
					[2, 43, 22, 0],
					[2, 43, 24, 0],
					[2, 43, 25, 0],
					[2, 43, 26, 0],
					[2, 43, 27, 0],
					[2, 43, 28, 0],
					[2, 43, 30, 0],
					[2, 43, 34, 0],
					[2, 43, 36, 0],
					[2, 43, 37, 0],
					[2, 43, 40, 0],
					[2, 43, 41, 0],
					[2, 43, 42, 0],
					[2, 43, 46, 0],
					[2, 43, 48, 0],
					[2, 43, 49, 0],
					[2, 43, 51, 1],
					[2, 45, 14, 0],
					[2, 51, 5, 0],
					[2, 51, 6, 0],
					[2, 51, 8, 1],
					[2, 51, 10, 1],
					[2, 51, 14, 0],
					[2, 51, 15, 0],
					[2, 51, 17, 0],
					[2, 51, 18, 0],
					[2, 51, 19, 0],
					[2, 51, 22, 0],
					[2, 51, 24, 0],
					[2, 51, 25, 0],
					[2, 51, 26, 0],
					[2, 51, 27, 0],
					[2, 51, 28, 0],
					[2, 51, 30, 0],
					[2, 51, 34, 0],
					[2, 51, 36, 0],
					[2, 51, 37, 0],
					[2, 51, 40, 0],
					[2, 51, 41, 0],
					[2, 51, 42, 0],
					[2, 51, 46, 0],
					[2, 51, 48, 0],
					[2, 51, 49, 0],
					[2, 54, 5, 0],
					[2, 54, 6, 0],
					[2, 54, 10, 1],
					[2, 54, 14, 0],
					[2, 54, 15, 0],
					[2, 54, 17, 0],
					[2, 54, 18, 0],
					[2, 54, 19, 0],
					[2, 54, 22, 0],
					[2, 54, 26, 0],
					[2, 54, 27, 0],
					[2, 54, 34, 0],
					[2, 54, 35, 0],
					[2, 54, 41, 0],
					[2, 54, 42, 0],
					[2, 54, 46, 0],
					[2, 54, 48, 0],
					[2, 54, 49, 0],
					[2, 54, 50, 0],
					[2, 57, 5, 0],
					[2, 57, 6, 0],
					[2, 57, 8, 1],
					[2, 57, 14, 0],
					[2, 57, 15, 0],
					[2, 57, 17, 0],
					[2, 57, 18, 0],
					[2, 57, 19, 0],
					[2, 57, 22, 0],
					[2, 57, 24, 0],
					[2, 57, 25, 0],
					[2, 57, 26, 0],
					[2, 57, 27, 0],
					[2, 57, 28, 0],
					[2, 57, 30, 0],
					[2, 57, 34, 0],
					[2, 57, 36, 0],
					[2, 57, 37, 0],
					[2, 57, 40, 0],
					[2, 57, 41, 0],
					[2, 57, 42, 0],
					[2, 57, 46, 0],
					[2, 57, 48, 0],
					[2, 57, 49, 0],
					[2, 58, 5, 0],
					[2, 58, 6, 0],
					[2, 58, 8, 1],
					[2, 58, 14, 0],
					[2, 58, 15, 0],
					[2, 58, 17, 0],
					[2, 58, 18, 0],
					[2, 58, 19, 0],
					[2, 58, 22, 0],
					[2, 58, 24, 0],
					[2, 58, 25, 0],
					[2, 58, 26, 0],
					[2, 58, 27, 0],
					[2, 58, 28, 0],
					[2, 58, 30, 0],
					[2, 58, 34, 0],
					[2, 58, 36, 0],
					[2, 58, 37, 0],
					[2, 58, 38, 0],
					[2, 58, 40, 0],
					[2, 58, 41, 0],
					[2, 58, 42, 0],
					[2, 58, 46, 0],
					[2, 58, 48, 0],
					[2, 58, 49, 0],
					[2, 59, 5, 1],
					[2, 59, 6, 1],
					[2, 59, 8, 1],
					[2, 59, 14, 0],
					[2, 59, 15, 0],
					[2, 59, 17, 0],
					[2, 59, 18, 0],
					[2, 59, 19, 0],
					[2, 59, 22, 0],
					[2, 59, 24, 0],
					[2, 59, 25, 0],
					[2, 59, 26, 0],
					[2, 59, 27, 0],
					[2, 59, 28, 0],
					[2, 59, 30, 0],
					[2, 59, 34, 0],
					[2, 59, 36, 0],
					[2, 59, 37, 0],
					[2, 59, 40, 0],
					[2, 59, 41, 0],
					[2, 59, 42, 0],
					[2, 59, 46, 0],
					[2, 59, 48, 0],
					[2, 59, 49, 0],
					[2, 60, 5, 0],
					[2, 60, 6, 0],
					[2, 60, 10, 0],
					[2, 60, 14, 0],
					[2, 60, 15, 0],
					[2, 60, 17, 0],
					[2, 60, 18, 0],
					[2, 60, 19, 0],
					[2, 60, 22, 0],
					[2, 60, 24, 0],
					[2, 60, 25, 0],
					[2, 60, 26, 0],
					[2, 60, 27, 0],
					[2, 60, 28, 0],
					[2, 60, 30, 0],
					[2, 60, 34, 0],
					[2, 60, 36, 0],
					[2, 60, 37, 0],
					[2, 60, 40, 0],
					[2, 60, 41, 0],
					[2, 60, 42, 0],
					[2, 60, 46, 0],
					[2, 60, 48, 0],
					[2, 60, 49, 0],
					[2, 61, 5, 0],
					[2, 61, 6, 0],
					[2, 61, 8, 1],
					[2, 61, 14, 0],
					[2, 61, 15, 0],
					[2, 61, 17, 0],
					[2, 61, 18, 0],
					[2, 61, 19, 0],
					[2, 61, 22, 0],
					[2, 61, 24, 0],
					[2, 61, 25, 0],
					[2, 61, 26, 0],
					[2, 61, 27, 0],
					[2, 61, 28, 0],
					[2, 61, 30, 0],
					[2, 61, 34, 0],
					[2, 61, 36, 0],
					[2, 61, 37, 0],
					[2, 61, 40, 0],
					[2, 61, 41, 0],
					[2, 61, 42, 0],
					[2, 61, 46, 0],
					[2, 61, 48, 0],
					[2, 61, 49, 0],
					[2, 61, 51, 1],
					[2, 74, 5, 0],
					[2, 74, 6, 0],
					[2, 74, 8, 1],
					[2, 74, 14, 0],
					[2, 74, 17, 0],
					[2, 74, 18, 0],
					[2, 74, 19, 0],
					[2, 74, 22, 0],
					[2, 74, 26, 0],
					[2, 74, 27, 0],
					[2, 74, 34, 0],
					[2, 74, 41, 0],
					[2, 74, 42, 0],
					[2, 74, 46, 0],
					[2, 74, 48, 0],
					[2, 74, 49, 0],
					[2, 75, 5, 0],
					[2, 75, 6, 0],
					[2, 75, 8, 1],
					[2, 75, 14, 0],
					[2, 75, 17, 0],
					[2, 75, 18, 0],
					[2, 75, 19, 0],
					[2, 75, 22, 0],
					[2, 75, 24, 0],
					[2, 75, 25, 0],
					[2, 75, 26, 0],
					[2, 75, 27, 0],
					[2, 75, 28, 0],
					[2, 75, 30, 0],
					[2, 75, 34, 0],
					[2, 75, 36, 0],
					[2, 75, 37, 0],
					[2, 75, 40, 0],
					[2, 75, 41, 0],
					[2, 75, 42, 0],
					[2, 75, 46, 0],
					[2, 75, 48, 0],
					[2, 75, 49, 0],
					[2, 78, 5, 0],
					[2, 78, 6, 0],
					[2, 78, 8, 1],
					[2, 78, 14, 0],
					[2, 78, 17, 0],
					[2, 78, 18, 0],
					[2, 78, 19, 0],
					[2, 78, 22, 0],
					[2, 78, 24, 0],
					[2, 78, 25, 0],
					[2, 78, 26, 0],
					[2, 78, 27, 0],
					[2, 78, 28, 0],
					[2, 78, 30, 0],
					[2, 78, 34, 0],
					[2, 78, 36, 0],
					[2, 78, 37, 0],
					[2, 78, 40, 0],
					[2, 78, 41, 0],
					[2, 78, 42, 0],
					[2, 78, 46, 0],
					[2, 78, 48, 0],
					[2, 78, 49, 0],
					[2, 79, 5, 0],
					[2, 79, 6, 0],
					[2, 79, 8, 1],
					[2, 79, 14, 0],
					[2, 79, 17, 0],
					[2, 79, 18, 0],
					[2, 79, 19, 0],
					[2, 79, 22, 0],
					[2, 79, 24, 0],
					[2, 79, 25, 0],
					[2, 79, 26, 0],
					[2, 79, 27, 0],
					[2, 79, 28, 0],
					[2, 79, 30, 0],
					[2, 79, 34, 0],
					[2, 79, 36, 0],
					[2, 79, 37, 0],
					[2, 79, 40, 0],
					[2, 79, 41, 0],
					[2, 79, 42, 0],
					[2, 79, 46, 0],
					[2, 79, 48, 0],
					[2, 79, 49, 0],
					[2, 80, 5, 1],
					[2, 80, 6, 1],
					[2, 80, 8, 1],
					[2, 80, 14, 0],
					[2, 80, 17, 0],
					[2, 80, 18, 0],
					[2, 80, 19, 0],
					[2, 80, 22, 0],
					[2, 80, 24, 0],
					[2, 80, 25, 0],
					[2, 80, 26, 0],
					[2, 80, 27, 0],
					[2, 80, 28, 0],
					[2, 80, 30, 0],
					[2, 80, 34, 0],
					[2, 80, 36, 0],
					[2, 80, 37, 0],
					[2, 80, 40, 0],
					[2, 80, 41, 0],
					[2, 80, 42, 0],
					[2, 80, 46, 0],
					[2, 80, 48, 0],
					[2, 80, 49, 0],
					[2, 81, 5, 0],
					[2, 81, 6, 0],
					[2, 81, 8, 1],
					[2, 81, 14, 0],
					[2, 81, 17, 0],
					[2, 81, 18, 0],
					[2, 81, 19, 0],
					[2, 81, 22, 0],
					[2, 81, 24, 0],
					[2, 81, 25, 0],
					[2, 81, 26, 0],
					[2, 81, 27, 0],
					[2, 81, 28, 0],
					[2, 81, 30, 0],
					[2, 81, 34, 0],
					[2, 81, 36, 0],
					[2, 81, 37, 0],
					[2, 81, 40, 0],
					[2, 81, 41, 0],
					[2, 81, 42, 0],
					[2, 81, 46, 0],
					[2, 81, 48, 0],
					[2, 81, 49, 0],
					[2, 82, 5, 0],
					[2, 82, 6, 0],
					[2, 82, 8, 1],
					[2, 82, 14, 0],
					[2, 82, 17, 0],
					[2, 82, 18, 0],
					[2, 82, 19, 0],
					[2, 82, 22, 0],
					[2, 82, 24, 0],
					[2, 82, 25, 0],
					[2, 82, 26, 0],
					[2, 82, 27, 0],
					[2, 82, 28, 0],
					[2, 82, 30, 0],
					[2, 82, 34, 0],
					[2, 82, 36, 0],
					[2, 82, 37, 0],
					[2, 82, 40, 0],
					[2, 82, 41, 0],
					[2, 82, 42, 0],
					[2, 82, 46, 0],
					[2, 82, 48, 0],
					[2, 82, 49, 0],
					[2, 83, 5, 0],
					[2, 83, 6, 0],
					[2, 83, 8, 1],
					[2, 83, 14, 0],
					[2, 83, 17, 0],
					[2, 83, 18, 0],
					[2, 83, 19, 0],
					[2, 83, 22, 0],
					[2, 83, 24, 0],
					[2, 83, 25, 0],
					[2, 83, 26, 0],
					[2, 83, 27, 0],
					[2, 83, 28, 0],
					[2, 83, 30, 0],
					[2, 83, 34, 0],
					[2, 83, 36, 0],
					[2, 83, 37, 0],
					[2, 83, 40, 0],
					[2, 83, 41, 0],
					[2, 83, 42, 0],
					[2, 83, 46, 0],
					[2, 83, 48, 0],
					[2, 83, 49, 0],
					[2, 84, 5, 0],
					[2, 84, 6, 0],
					[2, 84, 8, 1],
					[2, 84, 14, 0],
					[2, 84, 15, 0],
					[2, 84, 17, 0],
					[2, 84, 18, 0],
					[2, 84, 19, 0],
					[2, 84, 22, 0],
					[2, 84, 24, 0],
					[2, 84, 25, 0],
					[2, 84, 26, 0],
					[2, 84, 27, 0],
					[2, 84, 28, 0],
					[2, 84, 30, 0],
					[2, 84, 34, 0],
					[2, 84, 36, 0],
					[2, 84, 37, 0],
					[2, 84, 40, 0],
					[2, 84, 41, 0],
					[2, 84, 42, 0],
					[2, 84, 46, 0],
					[2, 84, 48, 0],
					[2, 84, 49, 0],
					[2, 85, 5, 0],
					[2, 85, 6, 0],
					[2, 85, 8, 1],
					[2, 85, 14, 0],
					[2, 85, 17, 0],
					[2, 85, 18, 0],
					[2, 85, 19, 0],
					[2, 85, 22, 0],
					[2, 85, 24, 0],
					[2, 85, 25, 0],
					[2, 85, 26, 0],
					[2, 85, 27, 0],
					[2, 85, 28, 0],
					[2, 85, 30, 0],
					[2, 85, 34, 0],
					[2, 85, 36, 0],
					[2, 85, 37, 0],
					[2, 85, 38, 0],
					[2, 85, 39, 0],
					[2, 85, 40, 0],
					[2, 85, 41, 0],
					[2, 85, 42, 0],
					[2, 85, 46, 0],
					[2, 85, 48, 0],
					[2, 85, 49, 0],
					[2, 85, 51, 1],
					[2, 86, 5, 0],
					[2, 86, 6, 0],
					[2, 86, 8, 1],
					[2, 86, 14, 0],
					[2, 86, 15, 0],
					[2, 86, 17, 0],
					[2, 86, 18, 0],
					[2, 86, 19, 0],
					[2, 86, 22, 0],
					[2, 86, 24, 0],
					[2, 86, 25, 0],
					[2, 86, 26, 0],
					[2, 86, 27, 0],
					[2, 86, 28, 0],
					[2, 86, 30, 0],
					[2, 86, 34, 0],
					[2, 86, 36, 0],
					[2, 86, 37, 0],
					[2, 86, 40, 0],
					[2, 86, 41, 0],
					[2, 86, 42, 0],
					[2, 86, 46, 0],
					[2, 86, 48, 0],
					[2, 86, 49, 0],
					[2, 86, 51, 1],
					[2, 87, 5, 0],
					[2, 87, 6, 0],
					[2, 87, 8, 1],
					[2, 87, 14, 0],
					[2, 87, 17, 0],
					[2, 87, 18, 0],
					[2, 87, 19, 0],
					[2, 87, 22, 0],
					[2, 87, 24, 0],
					[2, 87, 25, 0],
					[2, 87, 26, 0],
					[2, 87, 27, 0],
					[2, 87, 28, 0],
					[2, 87, 30, 0],
					[2, 87, 34, 0],
					[2, 87, 36, 0],
					[2, 87, 37, 0],
					[2, 87, 38, 0],
					[2, 87, 39, 0],
					[2, 87, 40, 0],
					[2, 87, 41, 0],
					[2, 87, 42, 0],
					[2, 87, 46, 0],
					[2, 87, 48, 0],
					[2, 87, 49, 0],
					[2, 87, 51, 1],
					[2, 88, 5, 0],
					[2, 88, 6, 0],
					[2, 88, 8, 1],
					[2, 88, 14, 0],
					[2, 88, 17, 0],
					[2, 88, 18, 0],
					[2, 88, 19, 0],
					[2, 88, 22, 0],
					[2, 88, 24, 0],
					[2, 88, 25, 0],
					[2, 88, 26, 0],
					[2, 88, 27, 0],
					[2, 88, 28, 0],
					[2, 88, 30, 0],
					[2, 88, 34, 0],
					[2, 88, 36, 0],
					[2, 88, 37, 0],
					[2, 88, 38, 0],
					[2, 88, 39, 0],
					[2, 88, 40, 0],
					[2, 88, 41, 0],
					[2, 88, 42, 0],
					[2, 88, 46, 0],
					[2, 88, 48, 0],
					[2, 88, 49, 0],
					[2, 88, 51, 1],
					[2, 89, 5, 0],
					[2, 89, 6, 0],
					[2, 89, 8, 1],
					[2, 89, 14, 0],
					[2, 89, 17, 0],
					[2, 89, 18, 0],
					[2, 89, 19, 0],
					[2, 89, 22, 0],
					[2, 89, 24, 0],
					[2, 89, 25, 0],
					[2, 89, 26, 0],
					[2, 89, 27, 0],
					[2, 89, 28, 0],
					[2, 89, 30, 0],
					[2, 89, 34, 0],
					[2, 89, 36, 0],
					[2, 89, 37, 0],
					[2, 89, 38, 0],
					[2, 89, 39, 0],
					[2, 89, 40, 0],
					[2, 89, 41, 0],
					[2, 89, 42, 0],
					[2, 89, 46, 0],
					[2, 89, 48, 0],
					[2, 89, 49, 0],
					[2, 89, 51, 1],
					[2, 90, 5, 0],
					[2, 90, 6, 0],
					[2, 90, 8, 1],
					[2, 90, 14, 0],
					[2, 90, 17, 0],
					[2, 90, 18, 0],
					[2, 90, 19, 0],
					[2, 90, 22, 0],
					[2, 90, 24, 0],
					[2, 90, 25, 0],
					[2, 90, 26, 0],
					[2, 90, 27, 0],
					[2, 90, 28, 0],
					[2, 90, 30, 0],
					[2, 90, 34, 0],
					[2, 90, 36, 0],
					[2, 90, 37, 0],
					[2, 90, 38, 0],
					[2, 90, 39, 0],
					[2, 90, 40, 0],
					[2, 90, 41, 0],
					[2, 90, 42, 0],
					[2, 90, 46, 0],
					[2, 90, 48, 0],
					[2, 90, 49, 0],
					[2, 90, 51, 1],
					[2, 91, 5, 0],
					[2, 91, 6, 0],
					[2, 91, 8, 1],
					[2, 91, 14, 0],
					[2, 91, 17, 0],
					[2, 91, 18, 0],
					[2, 91, 19, 0],
					[2, 91, 22, 0],
					[2, 91, 24, 0],
					[2, 91, 25, 0],
					[2, 91, 26, 0],
					[2, 91, 27, 0],
					[2, 91, 28, 0],
					[2, 91, 30, 0],
					[2, 91, 34, 0],
					[2, 91, 36, 0],
					[2, 91, 37, 0],
					[2, 91, 38, 0],
					[2, 91, 39, 0],
					[2, 91, 40, 0],
					[2, 91, 41, 0],
					[2, 91, 42, 0],
					[2, 91, 46, 0],
					[2, 91, 48, 0],
					[2, 91, 49, 0],
					[2, 91, 51, 1],
					[2, 92, 5, 0],
					[2, 92, 6, 0],
					[2, 92, 8, 1],
					[2, 92, 14, 0],
					[2, 92, 17, 0],
					[2, 92, 18, 0],
					[2, 92, 19, 0],
					[2, 92, 20, 0],
					[2, 92, 22, 0],
					[2, 92, 24, 0],
					[2, 92, 25, 0],
					[2, 92, 26, 0],
					[2, 92, 27, 0],
					[2, 92, 28, 0],
					[2, 92, 30, 0],
					[2, 92, 34, 0],
					[2, 92, 36, 0],
					[2, 92, 37, 0],
					[2, 92, 40, 0],
					[2, 92, 41, 0],
					[2, 92, 42, 0],
					[2, 92, 46, 0],
					[2, 92, 48, 0],
					[2, 92, 49, 0],
					[2, 92, 51, 1],
					[2, 93, 5, 0],
					[2, 93, 6, 0],
					[2, 93, 8, 1],
					[2, 93, 14, 0],
					[2, 93, 17, 0],
					[2, 93, 18, 0],
					[2, 93, 19, 0],
					[2, 93, 20, 0],
					[2, 93, 22, 0],
					[2, 93, 24, 0],
					[2, 93, 25, 0],
					[2, 93, 26, 0],
					[2, 93, 27, 0],
					[2, 93, 28, 0],
					[2, 93, 30, 0],
					[2, 93, 34, 0],
					[2, 93, 36, 0],
					[2, 93, 37, 0],
					[2, 93, 40, 0],
					[2, 93, 41, 0],
					[2, 93, 42, 0],
					[2, 93, 46, 0],
					[2, 93, 48, 0],
					[2, 93, 49, 0],
					[2, 93, 51, 1],
					[2, 94, 5, 0],
					[2, 94, 6, 0],
					[2, 94, 8, 1],
					[2, 94, 14, 0],
					[2, 94, 34, 0],
					[2, 94, 36, 0],
					[2, 94, 37, 0],
					[2, 94, 41, 0],
					[2, 94, 42, 0],
					[2, 94, 46, 0],
					[2, 94, 48, 0],
					[2, 94, 49, 1],
					[2, 94, 51, 1],
					[2, 95, 5, 0],
					[2, 95, 6, 0],
					[2, 95, 8, 1],
					[2, 95, 14, 0],
					[2, 95, 34, 0],
					[2, 95, 36, 0],
					[2, 95, 37, 0],
					[2, 95, 41, 0],
					[2, 95, 42, 0],
					[2, 95, 46, 0],
					[2, 95, 48, 0],
					[2, 95, 49, 1],
					[2, 95, 51, 1],
					[2, 96, 5, 0],
					[2, 96, 6, 0],
					[2, 96, 8, 1],
					[2, 96, 14, 0],
					[2, 96, 34, 0],
					[2, 96, 36, 0],
					[2, 96, 37, 0],
					[2, 96, 41, 0],
					[2, 96, 42, 0],
					[2, 96, 46, 0],
					[2, 96, 48, 0],
					[2, 96, 49, 1],
					[2, 97, 5, 0],
					[2, 97, 6, 0],
					[2, 97, 8, 1],
					[2, 97, 14, 0],
					[2, 97, 34, 0],
					[2, 97, 36, 0],
					[2, 97, 37, 0],
					[2, 97, 41, 0],
					[2, 97, 42, 0],
					[2, 97, 46, 0],
					[2, 97, 48, 0],
					[2, 97, 49, 1],
					[2, 98, 5, 0],
					[2, 98, 6, 0],
					[2, 98, 8, 1],
					[2, 98, 14, 0],
					[2, 98, 34, 0],
					[2, 98, 36, 0],
					[2, 98, 37, 0],
					[2, 98, 38, 0],
					[2, 98, 41, 0],
					[2, 98, 42, 0],
					[2, 98, 46, 0],
					[2, 98, 48, 0],
					[2, 98, 49, 1],
					[2, 99, 5, 0],
					[2, 99, 6, 0],
					[2, 99, 8, 1],
					[2, 99, 14, 0],
					[2, 99, 34, 0],
					[2, 99, 36, 0],
					[2, 99, 37, 0],
					[2, 99, 41, 0],
					[2, 99, 42, 0],
					[2, 99, 46, 0],
					[2, 99, 48, 0],
					[2, 99, 49, 1],
					[2, 100, 5, 0],
					[2, 100, 6, 0],
					[2, 100, 8, 1],
					[2, 100, 14, 0],
					[2, 100, 34, 0],
					[2, 100, 36, 0],
					[2, 100, 37, 0],
					[2, 100, 38, 0],
					[2, 100, 41, 0],
					[2, 100, 42, 0],
					[2, 100, 46, 0],
					[2, 100, 48, 0],
					[2, 100, 49, 1],
					[2, 101, 5, 0],
					[2, 101, 6, 0],
					[2, 101, 8, 1],
					[2, 101, 14, 0],
					[2, 101, 34, 0],
					[2, 101, 36, 0],
					[2, 101, 37, 0],
					[2, 101, 38, 0],
					[2, 101, 41, 0],
					[2, 101, 42, 0],
					[2, 101, 46, 0],
					[2, 101, 48, 0],
					[2, 101, 49, 1],
					[2, 102, 5, 0],
					[2, 102, 6, 0],
					[2, 102, 8, 1],
					[2, 102, 14, 0],
					[2, 102, 34, 0],
					[2, 102, 36, 0],
					[2, 102, 37, 0],
					[2, 102, 38, 0],
					[2, 102, 41, 0],
					[2, 102, 42, 0],
					[2, 102, 46, 0],
					[2, 102, 48, 0],
					[2, 102, 49, 1],
					[2, 103, 5, 0],
					[2, 103, 6, 0],
					[2, 103, 8, 1],
					[2, 103, 14, 0],
					[2, 103, 34, 0],
					[2, 103, 36, 0],
					[2, 103, 37, 0],
					[2, 103, 38, 0],
					[2, 103, 41, 0],
					[2, 103, 42, 0],
					[2, 103, 46, 0],
					[2, 103, 48, 0],
					[2, 103, 49, 1],
					[2, 104, 5, 0],
					[2, 104, 6, 0],
					[2, 104, 8, 1],
					[2, 104, 14, 0],
					[2, 104, 34, 0],
					[2, 104, 36, 0],
					[2, 104, 37, 0],
					[2, 104, 38, 0],
					[2, 104, 41, 0],
					[2, 104, 42, 0],
					[2, 104, 46, 0],
					[2, 104, 48, 0],
					[2, 104, 49, 1],
					[2, 105, 5, 0],
					[2, 105, 6, 0],
					[2, 105, 8, 1],
					[2, 105, 14, 0],
					[2, 105, 34, 0],
					[2, 105, 36, 0],
					[2, 105, 37, 0],
					[2, 105, 38, 0],
					[2, 105, 41, 0],
					[2, 105, 42, 0],
					[2, 105, 46, 0],
					[2, 105, 48, 0],
					[2, 105, 49, 1],
					[2, 106, 5, 0],
					[2, 106, 6, 0],
					[2, 106, 8, 1],
					[2, 106, 14, 0],
					[2, 106, 34, 0],
					[2, 106, 36, 0],
					[2, 106, 37, 0],
					[2, 106, 38, 0],
					[2, 106, 41, 0],
					[2, 106, 42, 0],
					[2, 106, 46, 0],
					[2, 106, 48, 0],
					[2, 106, 49, 1],
					[2, 107, 5, 0],
					[2, 107, 6, 0],
					[2, 107, 8, 1],
					[2, 107, 14, 0],
					[2, 107, 34, 0],
					[2, 107, 36, 0],
					[2, 107, 37, 0],
					[2, 107, 41, 0],
					[2, 107, 42, 0],
					[2, 107, 46, 0],
					[2, 107, 48, 0],
					[2, 107, 49, 1],
					[2, 108, 5, 0],
					[2, 108, 6, 0],
					[2, 108, 8, 1],
					[2, 108, 14, 0],
					[2, 108, 34, 0],
					[2, 108, 36, 0],
					[2, 108, 37, 0],
					[2, 108, 38, 0],
					[2, 108, 41, 0],
					[2, 108, 42, 0],
					[2, 108, 46, 0],
					[2, 108, 48, 0],
					[2, 108, 49, 1],
					[2, 109, 5, 0],
					[2, 109, 6, 0],
					[2, 109, 8, 1],
					[2, 109, 14, 0],
					[2, 109, 34, 0],
					[2, 109, 36, 0],
					[2, 109, 37, 0],
					[2, 109, 38, 0],
					[2, 109, 41, 0],
					[2, 109, 42, 0],
					[2, 109, 46, 0],
					[2, 109, 48, 0],
					[2, 109, 49, 1],
					[2, 111, 5, 0],
					[2, 111, 6, 0],
					[2, 111, 10, 0],
					[2, 111, 14, 0],
					[2, 111, 15, 0],
					[2, 111, 17, 0],
					[2, 111, 18, 0],
					[2, 111, 19, 0],
					[2, 111, 22, 0],
					[2, 111, 24, 0],
					[2, 111, 25, 0],
					[2, 111, 26, 0],
					[2, 111, 27, 0],
					[2, 111, 28, 0],
					[2, 111, 30, 0],
					[2, 111, 34, 0],
					[2, 111, 36, 0],
					[2, 111, 37, 0],
					[2, 111, 40, 0],
					[2, 111, 41, 0],
					[2, 111, 42, 0],
					[2, 111, 44, 0],
					[2, 111, 46, 0],
					[2, 111, 47, 0],
					[2, 111, 48, 0],
					[2, 112, 5, 0],
					[2, 112, 6, 0],
					[2, 112, 14, 0],
					[2, 112, 15, 0],
					[2, 112, 17, 0],
					[2, 112, 18, 0],
					[2, 112, 19, 0],
					[2, 112, 20, 0],
					[2, 112, 22, 0],
					[2, 112, 26, 0],
					[2, 112, 27, 0],
					[2, 112, 34, 0],
					[2, 112, 36, 0],
					[2, 112, 37, 0],
					[2, 112, 40, 0],
					[2, 112, 41, 0],
					[2, 112, 42, 0],
					[2, 112, 46, 0],
					[2, 112, 48, 0],
					[2, 112, 49, 0],
					[2, 113, 5, 0],
					[2, 113, 6, 0],
					[2, 113, 14, 0],
					[2, 113, 15, 0],
					[2, 113, 17, 0],
					[2, 113, 18, 0],
					[2, 113, 19, 0],
					[2, 113, 20, 0],
					[2, 113, 22, 0],
					[2, 113, 26, 0],
					[2, 113, 27, 0],
					[2, 113, 34, 0],
					[2, 113, 36, 0],
					[2, 113, 37, 0],
					[2, 113, 40, 0],
					[2, 113, 41, 0],
					[2, 113, 42, 0],
					[2, 113, 46, 0],
					[2, 113, 48, 0],
					[2, 113, 49, 0],
					[2, 114, 5, 0],
					[2, 114, 6, 0],
					[2, 114, 14, 0],
					[2, 114, 15, 0],
					[2, 114, 17, 0],
					[2, 114, 18, 0],
					[2, 114, 19, 0],
					[2, 114, 20, 0],
					[2, 114, 22, 0],
					[2, 114, 26, 0],
					[2, 114, 27, 0],
					[2, 114, 34, 0],
					[2, 114, 36, 0],
					[2, 114, 37, 0],
					[2, 114, 40, 0],
					[2, 114, 41, 0],
					[2, 114, 42, 0],
					[2, 114, 46, 0],
					[2, 114, 48, 0],
					[2, 114, 49, 0],
					[2, 115, 5, 0],
					[2, 115, 6, 0],
					[2, 115, 14, 0],
					[2, 115, 15, 0],
					[2, 115, 17, 0],
					[2, 115, 18, 0],
					[2, 115, 19, 0],
					[2, 115, 20, 0],
					[2, 115, 22, 0],
					[2, 115, 26, 0],
					[2, 115, 27, 0],
					[2, 115, 34, 0],
					[2, 115, 36, 0],
					[2, 115, 37, 0],
					[2, 115, 40, 0],
					[2, 115, 41, 0],
					[2, 115, 42, 0],
					[2, 115, 46, 0],
					[2, 115, 48, 0],
					[2, 115, 49, 0],
					[2, 116, 5, 0],
					[2, 116, 6, 0],
					[2, 116, 14, 0],
					[2, 116, 15, 0],
					[2, 116, 17, 0],
					[2, 116, 18, 0],
					[2, 116, 19, 0],
					[2, 116, 20, 0],
					[2, 116, 22, 0],
					[2, 116, 26, 0],
					[2, 116, 27, 0],
					[2, 116, 34, 0],
					[2, 116, 36, 0],
					[2, 116, 37, 0],
					[2, 116, 40, 0],
					[2, 116, 41, 0],
					[2, 116, 42, 0],
					[2, 116, 46, 0],
					[2, 116, 48, 0],
					[2, 116, 49, 0],
					[2, 117, 5, 0],
					[2, 117, 6, 0],
					[2, 117, 14, 0],
					[2, 117, 15, 0],
					[2, 117, 17, 0],
					[2, 117, 18, 0],
					[2, 117, 19, 0],
					[2, 117, 20, 0],
					[2, 117, 22, 0],
					[2, 117, 26, 0],
					[2, 117, 27, 0],
					[2, 117, 34, 0],
					[2, 117, 36, 0],
					[2, 117, 37, 0],
					[2, 117, 40, 0],
					[2, 117, 41, 0],
					[2, 117, 42, 0],
					[2, 117, 46, 0],
					[2, 117, 48, 0],
					[2, 117, 49, 0],
					[2, 117, 51, 1],
					[3, 4, 5, 1],
					[3, 4, 6, 1],
					[3, 4, 8, 1],
					[3, 4, 10, 0],
					[3, 4, 14, 0],
					[3, 4, 15, 0],
					[3, 4, 17, 0],
					[3, 4, 18, 0],
					[3, 4, 19, 0],
					[3, 4, 20, 0],
					[3, 4, 21, 0],
					[3, 4, 22, 0],
					[3, 4, 24, 0],
					[3, 4, 25, 0],
					[3, 4, 26, 0],
					[3, 4, 27, 0],
					[3, 4, 28, 0],
					[3, 4, 30, 0],
					[3, 4, 34, 0],
					[3, 4, 36, 0],
					[3, 4, 37, 0],
					[3, 4, 40, 0],
					[3, 4, 41, 0],
					[3, 4, 42, 0],
					[3, 4, 46, 0],
					[3, 4, 48, 0],
					[3, 4, 49, 0],
					[3, 4, 51, 1],
					[3, 6, 5, 1],
					[3, 6, 6, 1],
					[3, 6, 8, 1],
					[3, 6, 10, 0],
					[3, 6, 14, 0],
					[3, 6, 15, 0],
					[3, 6, 17, 0],
					[3, 6, 18, 0],
					[3, 6, 19, 0],
					[3, 6, 20, 0],
					[3, 6, 21, 0],
					[3, 6, 22, 0],
					[3, 6, 24, 0],
					[3, 6, 25, 0],
					[3, 6, 26, 0],
					[3, 6, 27, 0],
					[3, 6, 28, 0],
					[3, 6, 30, 0],
					[3, 6, 34, 0],
					[3, 6, 36, 0],
					[3, 6, 37, 0],
					[3, 6, 40, 0],
					[3, 6, 41, 0],
					[3, 6, 42, 0],
					[3, 6, 46, 0],
					[3, 6, 48, 0],
					[3, 6, 49, 0],
					[3, 6, 51, 1],
					[3, 7, 5, 1],
					[3, 7, 6, 1],
					[3, 7, 8, 1],
					[3, 7, 9, 0],
					[3, 7, 10, 0],
					[3, 7, 14, 0],
					[3, 7, 15, 0],
					[3, 7, 17, 0],
					[3, 7, 18, 0],
					[3, 7, 19, 0],
					[3, 7, 20, 0],
					[3, 7, 21, 0],
					[3, 7, 22, 0],
					[3, 7, 24, 0],
					[3, 7, 25, 0],
					[3, 7, 26, 0],
					[3, 7, 27, 0],
					[3, 7, 28, 0],
					[3, 7, 30, 0],
					[3, 7, 34, 0],
					[3, 7, 36, 0],
					[3, 7, 37, 0],
					[3, 7, 40, 0],
					[3, 7, 41, 0],
					[3, 7, 42, 0],
					[3, 7, 46, 0],
					[3, 7, 48, 0],
					[3, 7, 49, 0],
					[3, 7, 51, 1],
					[3, 8, 6, 1],
					[3, 8, 14, 0],
					[3, 8, 15, 0],
					[3, 8, 17, 0],
					[3, 8, 18, 0],
					[3, 8, 19, 0],
					[3, 8, 22, 0],
					[3, 8, 23, 0],
					[3, 8, 24, 0],
					[3, 8, 25, 0],
					[3, 8, 26, 0],
					[3, 8, 27, 0],
					[3, 8, 28, 0],
					[3, 8, 30, 0],
					[3, 8, 34, 0],
					[3, 8, 36, 0],
					[3, 8, 37, 0],
					[3, 8, 40, 0],
					[3, 8, 41, 0],
					[3, 8, 42, 0],
					[3, 8, 46, 0],
					[3, 8, 48, 0],
					[3, 8, 49, 0],
					[3, 9, 5, 0],
					[3, 9, 6, 0],
					[3, 9, 14, 0],
					[3, 9, 15, 0],
					[3, 9, 17, 0],
					[3, 9, 18, 0],
					[3, 9, 19, 0],
					[3, 9, 22, 0],
					[3, 9, 24, 0],
					[3, 9, 25, 0],
					[3, 9, 26, 0],
					[3, 9, 27, 0],
					[3, 9, 28, 0],
					[3, 9, 30, 0],
					[3, 9, 31, 0],
					[3, 9, 32, 0],
					[3, 9, 33, 0],
					[3, 9, 34, 0],
					[3, 9, 40, 0],
					[3, 9, 41, 0],
					[3, 9, 42, 0],
					[3, 9, 46, 0],
					[3, 9, 48, 0],
					[3, 9, 49, 0],
					[3, 13, 5, 1],
					[3, 13, 6, 1],
					[3, 13, 8, 1],
					[3, 13, 10, 0],
					[3, 13, 14, 0],
					[3, 13, 15, 0],
					[3, 13, 17, 0],
					[3, 13, 18, 0],
					[3, 13, 19, 0],
					[3, 13, 22, 0],
					[3, 13, 24, 0],
					[3, 13, 25, 0],
					[3, 13, 26, 0],
					[3, 13, 27, 0],
					[3, 13, 28, 0],
					[3, 13, 30, 0],
					[3, 13, 34, 0],
					[3, 13, 36, 0],
					[3, 13, 37, 0],
					[3, 13, 40, 0],
					[3, 13, 41, 0],
					[3, 13, 42, 0],
					[3, 13, 46, 0],
					[3, 13, 48, 0],
					[3, 13, 49, 0],
					[3, 13, 51, 1],
					[3, 14, 5, 1],
					[3, 14, 6, 1],
					[3, 14, 10, 0],
					[3, 14, 14, 0],
					[3, 14, 15, 0],
					[3, 14, 17, 0],
					[3, 14, 18, 0],
					[3, 14, 19, 0],
					[3, 14, 22, 0],
					[3, 14, 24, 0],
					[3, 14, 25, 0],
					[3, 14, 26, 0],
					[3, 14, 27, 0],
					[3, 14, 28, 0],
					[3, 14, 30, 0],
					[3, 14, 34, 0],
					[3, 14, 36, 0],
					[3, 14, 37, 0],
					[3, 14, 40, 0],
					[3, 14, 41, 0],
					[3, 14, 42, 0],
					[3, 14, 46, 0],
					[3, 14, 48, 0],
					[3, 14, 49, 0],
					[3, 15, 14, 0],
					[3, 15, 15, 0],
					[3, 15, 17, 0],
					[3, 15, 18, 0],
					[3, 15, 19, 0],
					[3, 15, 22, 0],
					[3, 15, 24, 0],
					[3, 15, 25, 0],
					[3, 15, 26, 0],
					[3, 15, 27, 0],
					[3, 15, 28, 0],
					[3, 15, 30, 0],
					[3, 15, 34, 0],
					[3, 15, 36, 0],
					[3, 15, 37, 0],
					[3, 15, 40, 0],
					[3, 15, 41, 0],
					[3, 15, 42, 0],
					[3, 15, 46, 0],
					[3, 15, 48, 0],
					[3, 15, 49, 0],
					[3, 16, 5, 0],
					[3, 16, 6, 0],
					[3, 16, 15, 0],
					[3, 18, 5, 1],
					[3, 18, 6, 1],
					[3, 18, 10, 0],
					[3, 18, 14, 0],
					[3, 18, 15, 0],
					[3, 18, 17, 0],
					[3, 18, 18, 0],
					[3, 18, 19, 0],
					[3, 18, 20, 0],
					[3, 18, 21, 0],
					[3, 18, 22, 0],
					[3, 18, 24, 0],
					[3, 18, 25, 0],
					[3, 18, 26, 0],
					[3, 18, 27, 0],
					[3, 18, 28, 0],
					[3, 18, 30, 0],
					[3, 18, 34, 0],
					[3, 18, 36, 0],
					[3, 18, 37, 0],
					[3, 18, 40, 0],
					[3, 18, 41, 0],
					[3, 18, 42, 0],
					[3, 18, 46, 0],
					[3, 18, 48, 0],
					[3, 18, 49, 0],
					[3, 18, 51, 1],
					[3, 19, 5, 1],
					[3, 19, 6, 1],
					[3, 19, 10, 0],
					[3, 19, 14, 0],
					[3, 19, 15, 0],
					[3, 19, 17, 0],
					[3, 19, 18, 0],
					[3, 19, 19, 0],
					[3, 19, 24, 0],
					[3, 19, 25, 0],
					[3, 19, 26, 0],
					[3, 19, 27, 0],
					[3, 19, 28, 0],
					[3, 19, 30, 0],
					[3, 19, 34, 0],
					[3, 19, 36, 0],
					[3, 19, 37, 0],
					[3, 19, 40, 0],
					[3, 19, 41, 0],
					[3, 19, 42, 0],
					[3, 19, 46, 0],
					[3, 19, 48, 0],
					[3, 19, 49, 0],
					[3, 25, 6, 0],
					[3, 25, 13, 0],
					[3, 26, 14, 0],
					[3, 26, 15, 0],
					[3, 26, 17, 0],
					[3, 26, 18, 0],
					[3, 26, 19, 0],
					[3, 26, 22, 0],
					[3, 26, 24, 0],
					[3, 26, 25, 0],
					[3, 26, 26, 0],
					[3, 26, 27, 0],
					[3, 26, 28, 0],
					[3, 26, 30, 0],
					[3, 26, 34, 0],
					[3, 26, 36, 0],
					[3, 26, 37, 0],
					[3, 26, 40, 0],
					[3, 26, 41, 0],
					[3, 26, 42, 0],
					[3, 26, 46, 0],
					[3, 26, 48, 0],
					[3, 26, 49, 0],
					[3, 26, 51, 1],
					[3, 33, 11, 0],
					[3, 33, 12, 0],
					[3, 33, 14, 0],
					[3, 34, 5, 0],
					[3, 34, 6, 0],
					[3, 34, 10, 0],
					[3, 34, 14, 0],
					[3, 34, 15, 0],
					[3, 34, 17, 0],
					[3, 34, 18, 0],
					[3, 34, 19, 0],
					[3, 34, 22, 0],
					[3, 34, 24, 0],
					[3, 34, 25, 0],
					[3, 34, 26, 0],
					[3, 34, 27, 0],
					[3, 34, 28, 0],
					[3, 34, 30, 0],
					[3, 34, 34, 0],
					[3, 34, 36, 0],
					[3, 34, 37, 0],
					[3, 34, 40, 0],
					[3, 34, 41, 0],
					[3, 34, 42, 0],
					[3, 34, 46, 0],
					[3, 34, 48, 0],
					[3, 34, 49, 0],
					[3, 34, 51, 1],
					[3, 35, 5, 0],
					[3, 35, 6, 0],
					[3, 35, 10, 0],
					[3, 35, 14, 0],
					[3, 35, 15, 0],
					[3, 35, 17, 0],
					[3, 35, 18, 0],
					[3, 35, 19, 0],
					[3, 35, 22, 0],
					[3, 35, 24, 0],
					[3, 35, 25, 0],
					[3, 35, 26, 0],
					[3, 35, 27, 0],
					[3, 35, 28, 0],
					[3, 35, 30, 0],
					[3, 35, 34, 0],
					[3, 35, 36, 0],
					[3, 35, 37, 0],
					[3, 35, 40, 0],
					[3, 35, 41, 0],
					[3, 35, 42, 0],
					[3, 35, 46, 0],
					[3, 35, 48, 0],
					[3, 35, 49, 0],
					[3, 37, 5, 0],
					[3, 37, 6, 0],
					[3, 37, 10, 0],
					[3, 37, 14, 0],
					[3, 37, 15, 0],
					[3, 37, 17, 0],
					[3, 37, 18, 0],
					[3, 37, 19, 0],
					[3, 37, 22, 0],
					[3, 37, 24, 0],
					[3, 37, 25, 0],
					[3, 37, 26, 0],
					[3, 37, 27, 0],
					[3, 37, 28, 0],
					[3, 37, 30, 0],
					[3, 37, 34, 0],
					[3, 37, 36, 0],
					[3, 37, 37, 0],
					[3, 37, 38, 0],
					[3, 37, 40, 0],
					[3, 37, 41, 0],
					[3, 37, 42, 0],
					[3, 37, 46, 0],
					[3, 37, 48, 0],
					[3, 37, 49, 0],
					[3, 40, 16, 0],
					[3, 40, 41, 0],
					[3, 40, 42, 0],
					[3, 41, 5, 0],
					[3, 41, 6, 0],
					[3, 41, 10, 0],
					[3, 41, 14, 0],
					[3, 41, 15, 0],
					[3, 41, 17, 0],
					[3, 41, 18, 0],
					[3, 41, 19, 0],
					[3, 41, 22, 0],
					[3, 41, 24, 0],
					[3, 41, 25, 0],
					[3, 41, 26, 0],
					[3, 41, 27, 0],
					[3, 41, 28, 0],
					[3, 41, 30, 0],
					[3, 41, 34, 0],
					[3, 41, 36, 0],
					[3, 41, 37, 0],
					[3, 41, 40, 0],
					[3, 41, 41, 0],
					[3, 41, 42, 0],
					[3, 41, 46, 0],
					[3, 41, 48, 0],
					[3, 41, 49, 0],
					[3, 41, 51, 1],
					[3, 42, 5, 0],
					[3, 42, 6, 0],
					[3, 42, 10, 0],
					[3, 42, 14, 0],
					[3, 42, 15, 0],
					[3, 42, 17, 0],
					[3, 42, 18, 0],
					[3, 42, 19, 0],
					[3, 42, 22, 0],
					[3, 42, 24, 0],
					[3, 42, 25, 0],
					[3, 42, 26, 0],
					[3, 42, 27, 0],
					[3, 42, 28, 0],
					[3, 42, 30, 0],
					[3, 42, 34, 0],
					[3, 42, 36, 0],
					[3, 42, 37, 0],
					[3, 42, 40, 0],
					[3, 42, 41, 0],
					[3, 42, 42, 0],
					[3, 42, 46, 0],
					[3, 42, 48, 0],
					[3, 42, 49, 0],
					[3, 42, 51, 1],
					[3, 43, 5, 0],
					[3, 43, 6, 0],
					[3, 43, 10, 0],
					[3, 43, 14, 0],
					[3, 43, 15, 0],
					[3, 43, 17, 0],
					[3, 43, 18, 0],
					[3, 43, 19, 0],
					[3, 43, 22, 0],
					[3, 43, 24, 0],
					[3, 43, 25, 0],
					[3, 43, 26, 0],
					[3, 43, 27, 0],
					[3, 43, 28, 0],
					[3, 43, 30, 0],
					[3, 43, 34, 0],
					[3, 43, 36, 0],
					[3, 43, 37, 0],
					[3, 43, 40, 0],
					[3, 43, 41, 0],
					[3, 43, 42, 0],
					[3, 43, 46, 0],
					[3, 43, 48, 0],
					[3, 43, 49, 0],
					[3, 43, 51, 1],
					[3, 45, 14, 0],
					[3, 51, 5, 0],
					[3, 51, 6, 0],
					[3, 51, 8, 1],
					[3, 51, 10, 1],
					[3, 51, 14, 0],
					[3, 51, 15, 0],
					[3, 51, 17, 0],
					[3, 51, 18, 0],
					[3, 51, 19, 0],
					[3, 51, 22, 0],
					[3, 51, 24, 0],
					[3, 51, 25, 0],
					[3, 51, 26, 0],
					[3, 51, 27, 0],
					[3, 51, 28, 0],
					[3, 51, 30, 0],
					[3, 51, 34, 0],
					[3, 51, 36, 0],
					[3, 51, 37, 0],
					[3, 51, 40, 0],
					[3, 51, 41, 0],
					[3, 51, 42, 0],
					[3, 51, 46, 0],
					[3, 51, 48, 0],
					[3, 51, 49, 0],
					[3, 54, 5, 0],
					[3, 54, 6, 0],
					[3, 54, 10, 1],
					[3, 54, 14, 0],
					[3, 54, 15, 0],
					[3, 54, 17, 0],
					[3, 54, 18, 0],
					[3, 54, 19, 0],
					[3, 54, 22, 0],
					[3, 54, 26, 0],
					[3, 54, 27, 0],
					[3, 54, 34, 0],
					[3, 54, 35, 0],
					[3, 54, 41, 0],
					[3, 54, 42, 0],
					[3, 54, 46, 0],
					[3, 54, 48, 0],
					[3, 54, 49, 0],
					[3, 54, 50, 0],
					[3, 57, 5, 0],
					[3, 57, 6, 0],
					[3, 57, 8, 1],
					[3, 57, 14, 0],
					[3, 57, 15, 0],
					[3, 57, 17, 0],
					[3, 57, 18, 0],
					[3, 57, 19, 0],
					[3, 57, 22, 0],
					[3, 57, 24, 0],
					[3, 57, 25, 0],
					[3, 57, 26, 0],
					[3, 57, 27, 0],
					[3, 57, 28, 0],
					[3, 57, 30, 0],
					[3, 57, 34, 0],
					[3, 57, 36, 0],
					[3, 57, 37, 0],
					[3, 57, 40, 0],
					[3, 57, 41, 0],
					[3, 57, 42, 0],
					[3, 57, 46, 0],
					[3, 57, 48, 0],
					[3, 57, 49, 0],
					[3, 58, 5, 0],
					[3, 58, 6, 0],
					[3, 58, 8, 1],
					[3, 58, 14, 0],
					[3, 58, 15, 0],
					[3, 58, 17, 0],
					[3, 58, 18, 0],
					[3, 58, 19, 0],
					[3, 58, 22, 0],
					[3, 58, 24, 0],
					[3, 58, 25, 0],
					[3, 58, 26, 0],
					[3, 58, 27, 0],
					[3, 58, 28, 0],
					[3, 58, 30, 0],
					[3, 58, 34, 0],
					[3, 58, 36, 0],
					[3, 58, 37, 0],
					[3, 58, 38, 0],
					[3, 58, 40, 0],
					[3, 58, 41, 0],
					[3, 58, 42, 0],
					[3, 58, 46, 0],
					[3, 58, 48, 0],
					[3, 58, 49, 0],
					[3, 59, 5, 1],
					[3, 59, 6, 1],
					[3, 59, 8, 1],
					[3, 59, 14, 0],
					[3, 59, 15, 0],
					[3, 59, 17, 0],
					[3, 59, 18, 0],
					[3, 59, 19, 0],
					[3, 59, 22, 0],
					[3, 59, 24, 0],
					[3, 59, 25, 0],
					[3, 59, 26, 0],
					[3, 59, 27, 0],
					[3, 59, 28, 0],
					[3, 59, 30, 0],
					[3, 59, 34, 0],
					[3, 59, 36, 0],
					[3, 59, 37, 0],
					[3, 59, 40, 0],
					[3, 59, 41, 0],
					[3, 59, 42, 0],
					[3, 59, 46, 0],
					[3, 59, 48, 0],
					[3, 59, 49, 0],
					[3, 60, 5, 0],
					[3, 60, 6, 0],
					[3, 60, 10, 0],
					[3, 60, 14, 0],
					[3, 60, 15, 0],
					[3, 60, 17, 0],
					[3, 60, 18, 0],
					[3, 60, 19, 0],
					[3, 60, 22, 0],
					[3, 60, 24, 0],
					[3, 60, 25, 0],
					[3, 60, 26, 0],
					[3, 60, 27, 0],
					[3, 60, 28, 0],
					[3, 60, 30, 0],
					[3, 60, 34, 0],
					[3, 60, 36, 0],
					[3, 60, 37, 0],
					[3, 60, 40, 0],
					[3, 60, 41, 0],
					[3, 60, 42, 0],
					[3, 60, 46, 0],
					[3, 60, 48, 0],
					[3, 60, 49, 0],
					[3, 61, 5, 0],
					[3, 61, 6, 0],
					[3, 61, 8, 1],
					[3, 61, 14, 0],
					[3, 61, 15, 0],
					[3, 61, 17, 0],
					[3, 61, 18, 0],
					[3, 61, 19, 0],
					[3, 61, 22, 0],
					[3, 61, 24, 0],
					[3, 61, 25, 0],
					[3, 61, 26, 0],
					[3, 61, 27, 0],
					[3, 61, 28, 0],
					[3, 61, 30, 0],
					[3, 61, 34, 0],
					[3, 61, 36, 0],
					[3, 61, 37, 0],
					[3, 61, 40, 0],
					[3, 61, 41, 0],
					[3, 61, 42, 0],
					[3, 61, 46, 0],
					[3, 61, 48, 0],
					[3, 61, 49, 0],
					[3, 61, 51, 1],
					[3, 74, 5, 0],
					[3, 74, 6, 0],
					[3, 74, 8, 1],
					[3, 74, 14, 0],
					[3, 74, 17, 0],
					[3, 74, 18, 0],
					[3, 74, 19, 0],
					[3, 74, 22, 0],
					[3, 74, 26, 0],
					[3, 74, 27, 0],
					[3, 74, 34, 0],
					[3, 74, 41, 0],
					[3, 74, 42, 0],
					[3, 74, 46, 0],
					[3, 74, 48, 0],
					[3, 74, 49, 0],
					[3, 75, 5, 0],
					[3, 75, 6, 0],
					[3, 75, 8, 1],
					[3, 75, 14, 0],
					[3, 75, 17, 0],
					[3, 75, 18, 0],
					[3, 75, 19, 0],
					[3, 75, 22, 0],
					[3, 75, 24, 0],
					[3, 75, 25, 0],
					[3, 75, 26, 0],
					[3, 75, 27, 0],
					[3, 75, 28, 0],
					[3, 75, 30, 0],
					[3, 75, 34, 0],
					[3, 75, 36, 0],
					[3, 75, 37, 0],
					[3, 75, 40, 0],
					[3, 75, 41, 0],
					[3, 75, 42, 0],
					[3, 75, 46, 0],
					[3, 75, 48, 0],
					[3, 75, 49, 0],
					[3, 78, 5, 0],
					[3, 78, 6, 0],
					[3, 78, 8, 1],
					[3, 78, 14, 0],
					[3, 78, 17, 0],
					[3, 78, 18, 0],
					[3, 78, 19, 0],
					[3, 78, 22, 0],
					[3, 78, 24, 0],
					[3, 78, 25, 0],
					[3, 78, 26, 0],
					[3, 78, 27, 0],
					[3, 78, 28, 0],
					[3, 78, 30, 0],
					[3, 78, 34, 0],
					[3, 78, 36, 0],
					[3, 78, 37, 0],
					[3, 78, 40, 0],
					[3, 78, 41, 0],
					[3, 78, 42, 0],
					[3, 78, 46, 0],
					[3, 78, 48, 0],
					[3, 78, 49, 0],
					[3, 79, 5, 0],
					[3, 79, 6, 0],
					[3, 79, 8, 1],
					[3, 79, 14, 0],
					[3, 79, 17, 0],
					[3, 79, 18, 0],
					[3, 79, 19, 0],
					[3, 79, 22, 0],
					[3, 79, 24, 0],
					[3, 79, 25, 0],
					[3, 79, 26, 0],
					[3, 79, 27, 0],
					[3, 79, 28, 0],
					[3, 79, 30, 0],
					[3, 79, 34, 0],
					[3, 79, 36, 0],
					[3, 79, 37, 0],
					[3, 79, 40, 0],
					[3, 79, 41, 0],
					[3, 79, 42, 0],
					[3, 79, 46, 0],
					[3, 79, 48, 0],
					[3, 79, 49, 0],
					[3, 80, 5, 1],
					[3, 80, 6, 1],
					[3, 80, 8, 1],
					[3, 80, 14, 0],
					[3, 80, 17, 0],
					[3, 80, 18, 0],
					[3, 80, 19, 0],
					[3, 80, 22, 0],
					[3, 80, 24, 0],
					[3, 80, 25, 0],
					[3, 80, 26, 0],
					[3, 80, 27, 0],
					[3, 80, 28, 0],
					[3, 80, 30, 0],
					[3, 80, 34, 0],
					[3, 80, 36, 0],
					[3, 80, 37, 0],
					[3, 80, 40, 0],
					[3, 80, 41, 0],
					[3, 80, 42, 0],
					[3, 80, 46, 0],
					[3, 80, 48, 0],
					[3, 80, 49, 0],
					[3, 81, 5, 0],
					[3, 81, 6, 0],
					[3, 81, 8, 1],
					[3, 81, 14, 0],
					[3, 81, 17, 0],
					[3, 81, 18, 0],
					[3, 81, 19, 0],
					[3, 81, 22, 0],
					[3, 81, 24, 0],
					[3, 81, 25, 0],
					[3, 81, 26, 0],
					[3, 81, 27, 0],
					[3, 81, 28, 0],
					[3, 81, 30, 0],
					[3, 81, 34, 0],
					[3, 81, 36, 0],
					[3, 81, 37, 0],
					[3, 81, 40, 0],
					[3, 81, 41, 0],
					[3, 81, 42, 0],
					[3, 81, 46, 0],
					[3, 81, 48, 0],
					[3, 81, 49, 0],
					[3, 82, 5, 0],
					[3, 82, 6, 0],
					[3, 82, 8, 1],
					[3, 82, 14, 0],
					[3, 82, 17, 0],
					[3, 82, 18, 0],
					[3, 82, 19, 0],
					[3, 82, 22, 0],
					[3, 82, 24, 0],
					[3, 82, 25, 0],
					[3, 82, 26, 0],
					[3, 82, 27, 0],
					[3, 82, 28, 0],
					[3, 82, 30, 0],
					[3, 82, 34, 0],
					[3, 82, 36, 0],
					[3, 82, 37, 0],
					[3, 82, 40, 0],
					[3, 82, 41, 0],
					[3, 82, 42, 0],
					[3, 82, 46, 0],
					[3, 82, 48, 0],
					[3, 82, 49, 0],
					[3, 83, 5, 0],
					[3, 83, 6, 0],
					[3, 83, 8, 1],
					[3, 83, 14, 0],
					[3, 83, 17, 0],
					[3, 83, 18, 0],
					[3, 83, 19, 0],
					[3, 83, 22, 0],
					[3, 83, 24, 0],
					[3, 83, 25, 0],
					[3, 83, 26, 0],
					[3, 83, 27, 0],
					[3, 83, 28, 0],
					[3, 83, 30, 0],
					[3, 83, 34, 0],
					[3, 83, 36, 0],
					[3, 83, 37, 0],
					[3, 83, 40, 0],
					[3, 83, 41, 0],
					[3, 83, 42, 0],
					[3, 83, 46, 0],
					[3, 83, 48, 0],
					[3, 83, 49, 0],
					[3, 84, 5, 0],
					[3, 84, 6, 0],
					[3, 84, 8, 1],
					[3, 84, 14, 0],
					[3, 84, 15, 0],
					[3, 84, 17, 0],
					[3, 84, 18, 0],
					[3, 84, 19, 0],
					[3, 84, 22, 0],
					[3, 84, 24, 0],
					[3, 84, 25, 0],
					[3, 84, 26, 0],
					[3, 84, 27, 0],
					[3, 84, 28, 0],
					[3, 84, 30, 0],
					[3, 84, 34, 0],
					[3, 84, 36, 0],
					[3, 84, 37, 0],
					[3, 84, 40, 0],
					[3, 84, 41, 0],
					[3, 84, 42, 0],
					[3, 84, 46, 0],
					[3, 84, 48, 0],
					[3, 84, 49, 0],
					[3, 85, 5, 0],
					[3, 85, 6, 0],
					[3, 85, 8, 1],
					[3, 85, 14, 0],
					[3, 85, 17, 0],
					[3, 85, 18, 0],
					[3, 85, 19, 0],
					[3, 85, 22, 0],
					[3, 85, 24, 0],
					[3, 85, 25, 0],
					[3, 85, 26, 0],
					[3, 85, 27, 0],
					[3, 85, 28, 0],
					[3, 85, 30, 0],
					[3, 85, 34, 0],
					[3, 85, 36, 0],
					[3, 85, 37, 0],
					[3, 85, 38, 0],
					[3, 85, 39, 0],
					[3, 85, 40, 0],
					[3, 85, 41, 0],
					[3, 85, 42, 0],
					[3, 85, 46, 0],
					[3, 85, 48, 0],
					[3, 85, 49, 0],
					[3, 85, 51, 1],
					[3, 86, 5, 0],
					[3, 86, 6, 0],
					[3, 86, 8, 1],
					[3, 86, 14, 0],
					[3, 86, 15, 0],
					[3, 86, 17, 0],
					[3, 86, 18, 0],
					[3, 86, 19, 0],
					[3, 86, 22, 0],
					[3, 86, 24, 0],
					[3, 86, 25, 0],
					[3, 86, 26, 0],
					[3, 86, 27, 0],
					[3, 86, 28, 0],
					[3, 86, 30, 0],
					[3, 86, 34, 0],
					[3, 86, 36, 0],
					[3, 86, 37, 0],
					[3, 86, 40, 0],
					[3, 86, 41, 0],
					[3, 86, 42, 0],
					[3, 86, 46, 0],
					[3, 86, 48, 0],
					[3, 86, 49, 0],
					[3, 86, 51, 1],
					[3, 87, 5, 0],
					[3, 87, 6, 0],
					[3, 87, 8, 1],
					[3, 87, 14, 0],
					[3, 87, 17, 0],
					[3, 87, 18, 0],
					[3, 87, 19, 0],
					[3, 87, 22, 0],
					[3, 87, 24, 0],
					[3, 87, 25, 0],
					[3, 87, 26, 0],
					[3, 87, 27, 0],
					[3, 87, 28, 0],
					[3, 87, 30, 0],
					[3, 87, 34, 0],
					[3, 87, 36, 0],
					[3, 87, 37, 0],
					[3, 87, 38, 0],
					[3, 87, 39, 0],
					[3, 87, 40, 0],
					[3, 87, 41, 0],
					[3, 87, 42, 0],
					[3, 87, 46, 0],
					[3, 87, 48, 0],
					[3, 87, 49, 0],
					[3, 87, 51, 1],
					[3, 88, 5, 0],
					[3, 88, 6, 0],
					[3, 88, 8, 1],
					[3, 88, 14, 0],
					[3, 88, 17, 0],
					[3, 88, 18, 0],
					[3, 88, 19, 0],
					[3, 88, 22, 0],
					[3, 88, 24, 0],
					[3, 88, 25, 0],
					[3, 88, 26, 0],
					[3, 88, 27, 0],
					[3, 88, 28, 0],
					[3, 88, 30, 0],
					[3, 88, 34, 0],
					[3, 88, 36, 0],
					[3, 88, 37, 0],
					[3, 88, 38, 0],
					[3, 88, 39, 0],
					[3, 88, 40, 0],
					[3, 88, 41, 0],
					[3, 88, 42, 0],
					[3, 88, 46, 0],
					[3, 88, 48, 0],
					[3, 88, 49, 0],
					[3, 88, 51, 1],
					[3, 89, 5, 0],
					[3, 89, 6, 0],
					[3, 89, 8, 1],
					[3, 89, 14, 0],
					[3, 89, 17, 0],
					[3, 89, 18, 0],
					[3, 89, 19, 0],
					[3, 89, 22, 0],
					[3, 89, 24, 0],
					[3, 89, 25, 0],
					[3, 89, 26, 0],
					[3, 89, 27, 0],
					[3, 89, 28, 0],
					[3, 89, 30, 0],
					[3, 89, 34, 0],
					[3, 89, 36, 0],
					[3, 89, 37, 0],
					[3, 89, 38, 0],
					[3, 89, 39, 0],
					[3, 89, 40, 0],
					[3, 89, 41, 0],
					[3, 89, 42, 0],
					[3, 89, 46, 0],
					[3, 89, 48, 0],
					[3, 89, 49, 0],
					[3, 89, 51, 1],
					[3, 90, 5, 0],
					[3, 90, 6, 0],
					[3, 90, 8, 1],
					[3, 90, 14, 0],
					[3, 90, 17, 0],
					[3, 90, 18, 0],
					[3, 90, 19, 0],
					[3, 90, 22, 0],
					[3, 90, 24, 0],
					[3, 90, 25, 0],
					[3, 90, 26, 0],
					[3, 90, 27, 0],
					[3, 90, 28, 0],
					[3, 90, 30, 0],
					[3, 90, 34, 0],
					[3, 90, 36, 0],
					[3, 90, 37, 0],
					[3, 90, 38, 0],
					[3, 90, 39, 0],
					[3, 90, 40, 0],
					[3, 90, 41, 0],
					[3, 90, 42, 0],
					[3, 90, 46, 0],
					[3, 90, 48, 0],
					[3, 90, 49, 0],
					[3, 90, 51, 1],
					[3, 91, 5, 0],
					[3, 91, 6, 0],
					[3, 91, 8, 1],
					[3, 91, 14, 0],
					[3, 91, 17, 0],
					[3, 91, 18, 0],
					[3, 91, 19, 0],
					[3, 91, 22, 0],
					[3, 91, 24, 0],
					[3, 91, 25, 0],
					[3, 91, 26, 0],
					[3, 91, 27, 0],
					[3, 91, 28, 0],
					[3, 91, 30, 0],
					[3, 91, 34, 0],
					[3, 91, 36, 0],
					[3, 91, 37, 0],
					[3, 91, 38, 0],
					[3, 91, 39, 0],
					[3, 91, 40, 0],
					[3, 91, 41, 0],
					[3, 91, 42, 0],
					[3, 91, 46, 0],
					[3, 91, 48, 0],
					[3, 91, 49, 0],
					[3, 91, 51, 1],
					[3, 92, 5, 0],
					[3, 92, 6, 0],
					[3, 92, 8, 1],
					[3, 92, 14, 0],
					[3, 92, 17, 0],
					[3, 92, 18, 0],
					[3, 92, 19, 0],
					[3, 92, 20, 0],
					[3, 92, 22, 0],
					[3, 92, 24, 0],
					[3, 92, 25, 0],
					[3, 92, 26, 0],
					[3, 92, 27, 0],
					[3, 92, 28, 0],
					[3, 92, 30, 0],
					[3, 92, 34, 0],
					[3, 92, 36, 0],
					[3, 92, 37, 0],
					[3, 92, 40, 0],
					[3, 92, 41, 0],
					[3, 92, 42, 0],
					[3, 92, 46, 0],
					[3, 92, 48, 0],
					[3, 92, 49, 0],
					[3, 92, 51, 1],
					[3, 93, 5, 0],
					[3, 93, 6, 0],
					[3, 93, 8, 1],
					[3, 93, 14, 0],
					[3, 93, 17, 0],
					[3, 93, 18, 0],
					[3, 93, 19, 0],
					[3, 93, 20, 0],
					[3, 93, 22, 0],
					[3, 93, 24, 0],
					[3, 93, 25, 0],
					[3, 93, 26, 0],
					[3, 93, 27, 0],
					[3, 93, 28, 0],
					[3, 93, 30, 0],
					[3, 93, 34, 0],
					[3, 93, 36, 0],
					[3, 93, 37, 0],
					[3, 93, 40, 0],
					[3, 93, 41, 0],
					[3, 93, 42, 0],
					[3, 93, 46, 0],
					[3, 93, 48, 0],
					[3, 93, 49, 0],
					[3, 93, 51, 1],
					[3, 94, 5, 0],
					[3, 94, 6, 0],
					[3, 94, 8, 1],
					[3, 94, 14, 0],
					[3, 94, 34, 0],
					[3, 94, 36, 0],
					[3, 94, 37, 0],
					[3, 94, 41, 0],
					[3, 94, 42, 0],
					[3, 94, 46, 0],
					[3, 94, 48, 0],
					[3, 94, 49, 1],
					[3, 94, 51, 1],
					[3, 95, 5, 0],
					[3, 95, 6, 0],
					[3, 95, 8, 1],
					[3, 95, 14, 0],
					[3, 95, 34, 0],
					[3, 95, 36, 0],
					[3, 95, 37, 0],
					[3, 95, 41, 0],
					[3, 95, 42, 0],
					[3, 95, 46, 0],
					[3, 95, 48, 0],
					[3, 95, 49, 1],
					[3, 95, 51, 1],
					[3, 96, 5, 0],
					[3, 96, 6, 0],
					[3, 96, 8, 1],
					[3, 96, 14, 0],
					[3, 96, 34, 0],
					[3, 96, 36, 0],
					[3, 96, 37, 0],
					[3, 96, 41, 0],
					[3, 96, 42, 0],
					[3, 96, 46, 0],
					[3, 96, 48, 0],
					[3, 96, 49, 1],
					[3, 97, 5, 0],
					[3, 97, 6, 0],
					[3, 97, 8, 1],
					[3, 97, 14, 0],
					[3, 97, 34, 0],
					[3, 97, 36, 0],
					[3, 97, 37, 0],
					[3, 97, 41, 0],
					[3, 97, 42, 0],
					[3, 97, 46, 0],
					[3, 97, 48, 0],
					[3, 97, 49, 1],
					[3, 98, 5, 0],
					[3, 98, 6, 0],
					[3, 98, 8, 1],
					[3, 98, 14, 0],
					[3, 98, 34, 0],
					[3, 98, 36, 0],
					[3, 98, 37, 0],
					[3, 98, 38, 0],
					[3, 98, 41, 0],
					[3, 98, 42, 0],
					[3, 98, 46, 0],
					[3, 98, 48, 0],
					[3, 98, 49, 1],
					[3, 99, 5, 0],
					[3, 99, 6, 0],
					[3, 99, 8, 1],
					[3, 99, 14, 0],
					[3, 99, 34, 0],
					[3, 99, 36, 0],
					[3, 99, 37, 0],
					[3, 99, 41, 0],
					[3, 99, 42, 0],
					[3, 99, 46, 0],
					[3, 99, 48, 0],
					[3, 99, 49, 1],
					[3, 100, 5, 0],
					[3, 100, 6, 0],
					[3, 100, 8, 1],
					[3, 100, 14, 0],
					[3, 100, 34, 0],
					[3, 100, 36, 0],
					[3, 100, 37, 0],
					[3, 100, 38, 0],
					[3, 100, 41, 0],
					[3, 100, 42, 0],
					[3, 100, 46, 0],
					[3, 100, 48, 0],
					[3, 100, 49, 1],
					[3, 101, 5, 0],
					[3, 101, 6, 0],
					[3, 101, 8, 1],
					[3, 101, 14, 0],
					[3, 101, 34, 0],
					[3, 101, 36, 0],
					[3, 101, 37, 0],
					[3, 101, 38, 0],
					[3, 101, 41, 0],
					[3, 101, 42, 0],
					[3, 101, 46, 0],
					[3, 101, 48, 0],
					[3, 101, 49, 1],
					[3, 102, 5, 0],
					[3, 102, 6, 0],
					[3, 102, 8, 1],
					[3, 102, 14, 0],
					[3, 102, 34, 0],
					[3, 102, 36, 0],
					[3, 102, 37, 0],
					[3, 102, 38, 0],
					[3, 102, 41, 0],
					[3, 102, 42, 0],
					[3, 102, 46, 0],
					[3, 102, 48, 0],
					[3, 102, 49, 1],
					[3, 103, 5, 0],
					[3, 103, 6, 0],
					[3, 103, 8, 1],
					[3, 103, 14, 0],
					[3, 103, 34, 0],
					[3, 103, 36, 0],
					[3, 103, 37, 0],
					[3, 103, 38, 0],
					[3, 103, 41, 0],
					[3, 103, 42, 0],
					[3, 103, 46, 0],
					[3, 103, 48, 0],
					[3, 103, 49, 1],
					[3, 104, 5, 0],
					[3, 104, 6, 0],
					[3, 104, 8, 1],
					[3, 104, 14, 0],
					[3, 104, 34, 0],
					[3, 104, 36, 0],
					[3, 104, 37, 0],
					[3, 104, 38, 0],
					[3, 104, 41, 0],
					[3, 104, 42, 0],
					[3, 104, 46, 0],
					[3, 104, 48, 0],
					[3, 104, 49, 1],
					[3, 105, 5, 0],
					[3, 105, 6, 0],
					[3, 105, 8, 1],
					[3, 105, 14, 0],
					[3, 105, 34, 0],
					[3, 105, 36, 0],
					[3, 105, 37, 0],
					[3, 105, 38, 0],
					[3, 105, 41, 0],
					[3, 105, 42, 0],
					[3, 105, 46, 0],
					[3, 105, 48, 0],
					[3, 105, 49, 1],
					[3, 106, 5, 0],
					[3, 106, 6, 0],
					[3, 106, 8, 1],
					[3, 106, 14, 0],
					[3, 106, 34, 0],
					[3, 106, 36, 0],
					[3, 106, 37, 0],
					[3, 106, 38, 0],
					[3, 106, 41, 0],
					[3, 106, 42, 0],
					[3, 106, 46, 0],
					[3, 106, 48, 0],
					[3, 106, 49, 1],
					[3, 107, 5, 0],
					[3, 107, 6, 0],
					[3, 107, 8, 1],
					[3, 107, 14, 0],
					[3, 107, 34, 0],
					[3, 107, 36, 0],
					[3, 107, 37, 0],
					[3, 107, 41, 0],
					[3, 107, 42, 0],
					[3, 107, 46, 0],
					[3, 107, 48, 0],
					[3, 107, 49, 1],
					[3, 108, 5, 0],
					[3, 108, 6, 0],
					[3, 108, 8, 1],
					[3, 108, 14, 0],
					[3, 108, 34, 0],
					[3, 108, 36, 0],
					[3, 108, 37, 0],
					[3, 108, 38, 0],
					[3, 108, 41, 0],
					[3, 108, 42, 0],
					[3, 108, 46, 0],
					[3, 108, 48, 0],
					[3, 108, 49, 1],
					[3, 109, 5, 0],
					[3, 109, 6, 0],
					[3, 109, 8, 1],
					[3, 109, 14, 0],
					[3, 109, 34, 0],
					[3, 109, 36, 0],
					[3, 109, 37, 0],
					[3, 109, 38, 0],
					[3, 109, 41, 0],
					[3, 109, 42, 0],
					[3, 109, 46, 0],
					[3, 109, 48, 0],
					[3, 109, 49, 1],
					[3, 111, 5, 0],
					[3, 111, 6, 0],
					[3, 111, 10, 0],
					[3, 111, 14, 0],
					[3, 111, 15, 0],
					[3, 111, 17, 0],
					[3, 111, 18, 0],
					[3, 111, 19, 0],
					[3, 111, 22, 0],
					[3, 111, 24, 0],
					[3, 111, 25, 0],
					[3, 111, 26, 0],
					[3, 111, 27, 0],
					[3, 111, 28, 0],
					[3, 111, 30, 0],
					[3, 111, 34, 0],
					[3, 111, 36, 0],
					[3, 111, 37, 0],
					[3, 111, 40, 0],
					[3, 111, 41, 0],
					[3, 111, 42, 0],
					[3, 111, 44, 0],
					[3, 111, 46, 0],
					[3, 111, 47, 0],
					[3, 111, 48, 0],
					[3, 112, 5, 0],
					[3, 112, 6, 0],
					[3, 112, 14, 0],
					[3, 112, 15, 0],
					[3, 112, 17, 0],
					[3, 112, 18, 0],
					[3, 112, 19, 0],
					[3, 112, 20, 0],
					[3, 112, 22, 0],
					[3, 112, 26, 0],
					[3, 112, 27, 0],
					[3, 112, 34, 0],
					[3, 112, 36, 0],
					[3, 112, 37, 0],
					[3, 112, 40, 0],
					[3, 112, 41, 0],
					[3, 112, 42, 0],
					[3, 112, 46, 0],
					[3, 112, 48, 0],
					[3, 112, 49, 0],
					[3, 113, 5, 0],
					[3, 113, 6, 0],
					[3, 113, 14, 0],
					[3, 113, 15, 0],
					[3, 113, 17, 0],
					[3, 113, 18, 0],
					[3, 113, 19, 0],
					[3, 113, 20, 0],
					[3, 113, 22, 0],
					[3, 113, 26, 0],
					[3, 113, 27, 0],
					[3, 113, 34, 0],
					[3, 113, 36, 0],
					[3, 113, 37, 0],
					[3, 113, 40, 0],
					[3, 113, 41, 0],
					[3, 113, 42, 0],
					[3, 113, 46, 0],
					[3, 113, 48, 0],
					[3, 113, 49, 0],
					[3, 114, 5, 0],
					[3, 114, 6, 0],
					[3, 114, 14, 0],
					[3, 114, 15, 0],
					[3, 114, 17, 0],
					[3, 114, 18, 0],
					[3, 114, 19, 0],
					[3, 114, 20, 0],
					[3, 114, 22, 0],
					[3, 114, 26, 0],
					[3, 114, 27, 0],
					[3, 114, 34, 0],
					[3, 114, 36, 0],
					[3, 114, 37, 0],
					[3, 114, 40, 0],
					[3, 114, 41, 0],
					[3, 114, 42, 0],
					[3, 114, 46, 0],
					[3, 114, 48, 0],
					[3, 114, 49, 0],
					[3, 115, 5, 0],
					[3, 115, 6, 0],
					[3, 115, 14, 0],
					[3, 115, 15, 0],
					[3, 115, 17, 0],
					[3, 115, 18, 0],
					[3, 115, 19, 0],
					[3, 115, 20, 0],
					[3, 115, 22, 0],
					[3, 115, 26, 0],
					[3, 115, 27, 0],
					[3, 115, 34, 0],
					[3, 115, 36, 0],
					[3, 115, 37, 0],
					[3, 115, 40, 0],
					[3, 115, 41, 0],
					[3, 115, 42, 0],
					[3, 115, 46, 0],
					[3, 115, 48, 0],
					[3, 115, 49, 0],
					[3, 116, 5, 0],
					[3, 116, 6, 0],
					[3, 116, 14, 0],
					[3, 116, 15, 0],
					[3, 116, 17, 0],
					[3, 116, 18, 0],
					[3, 116, 19, 0],
					[3, 116, 20, 0],
					[3, 116, 22, 0],
					[3, 116, 26, 0],
					[3, 116, 27, 0],
					[3, 116, 34, 0],
					[3, 116, 36, 0],
					[3, 116, 37, 0],
					[3, 116, 40, 0],
					[3, 116, 41, 0],
					[3, 116, 42, 0],
					[3, 116, 46, 0],
					[3, 116, 48, 0],
					[3, 116, 49, 0],
					[3, 117, 5, 0],
					[3, 117, 6, 0],
					[3, 117, 14, 0],
					[3, 117, 15, 0],
					[3, 117, 17, 0],
					[3, 117, 18, 0],
					[3, 117, 19, 0],
					[3, 117, 20, 0],
					[3, 117, 22, 0],
					[3, 117, 26, 0],
					[3, 117, 27, 0],
					[3, 117, 34, 0],
					[3, 117, 36, 0],
					[3, 117, 37, 0],
					[3, 117, 40, 0],
					[3, 117, 41, 0],
					[3, 117, 42, 0],
					[3, 117, 46, 0],
					[3, 117, 48, 0],
					[3, 117, 49, 0],
					[3, 117, 51, 1],
					[4, 4, 5, 1],
					[4, 4, 6, 1],
					[4, 4, 8, 1],
					[4, 4, 10, 1],
					[4, 4, 14, 1],
					[4, 4, 15, 1],
					[4, 4, 17, 1],
					[4, 4, 18, 1],
					[4, 4, 19, 1],
					[4, 4, 20, 1],
					[4, 4, 21, 1],
					[4, 4, 22, 1],
					[4, 4, 24, 1],
					[4, 4, 25, 1],
					[4, 4, 26, 1],
					[4, 4, 27, 1],
					[4, 4, 28, 1],
					[4, 4, 30, 1],
					[4, 4, 34, 1],
					[4, 4, 36, 1],
					[4, 4, 37, 1],
					[4, 4, 40, 1],
					[4, 4, 41, 1],
					[4, 4, 42, 1],
					[4, 4, 46, 1],
					[4, 4, 48, 1],
					[4, 4, 49, 1],
					[4, 4, 51, 1],
					[4, 6, 5, 1],
					[4, 6, 6, 1],
					[4, 6, 8, 1],
					[4, 6, 10, 1],
					[4, 6, 14, 1],
					[4, 6, 15, 1],
					[4, 6, 17, 1],
					[4, 6, 18, 1],
					[4, 6, 19, 1],
					[4, 6, 20, 1],
					[4, 6, 21, 1],
					[4, 6, 22, 1],
					[4, 6, 24, 1],
					[4, 6, 25, 1],
					[4, 6, 26, 1],
					[4, 6, 27, 1],
					[4, 6, 28, 1],
					[4, 6, 30, 1],
					[4, 6, 34, 1],
					[4, 6, 36, 1],
					[4, 6, 37, 1],
					[4, 6, 40, 1],
					[4, 6, 41, 1],
					[4, 6, 42, 1],
					[4, 6, 46, 1],
					[4, 6, 48, 1],
					[4, 6, 49, 1],
					[4, 6, 51, 1],
					[4, 7, 5, 1],
					[4, 7, 6, 1],
					[4, 7, 8, 1],
					[4, 7, 9, 1],
					[4, 7, 10, 1],
					[4, 7, 14, 1],
					[4, 7, 15, 1],
					[4, 7, 17, 1],
					[4, 7, 18, 1],
					[4, 7, 19, 1],
					[4, 7, 20, 1],
					[4, 7, 21, 1],
					[4, 7, 22, 1],
					[4, 7, 24, 1],
					[4, 7, 25, 1],
					[4, 7, 26, 1],
					[4, 7, 27, 1],
					[4, 7, 28, 1],
					[4, 7, 30, 1],
					[4, 7, 34, 1],
					[4, 7, 36, 1],
					[4, 7, 37, 1],
					[4, 7, 40, 1],
					[4, 7, 41, 1],
					[4, 7, 42, 1],
					[4, 7, 46, 1],
					[4, 7, 48, 1],
					[4, 7, 49, 1],
					[4, 7, 51, 1],
					[4, 8, 6, 1],
					[4, 8, 14, 1],
					[4, 8, 15, 1],
					[4, 8, 17, 1],
					[4, 8, 18, 1],
					[4, 8, 19, 1],
					[4, 8, 22, 1],
					[4, 8, 23, 1],
					[4, 8, 24, 1],
					[4, 8, 25, 1],
					[4, 8, 26, 1],
					[4, 8, 27, 1],
					[4, 8, 28, 1],
					[4, 8, 30, 1],
					[4, 8, 34, 1],
					[4, 8, 36, 1],
					[4, 8, 37, 1],
					[4, 8, 40, 1],
					[4, 8, 41, 1],
					[4, 8, 42, 1],
					[4, 8, 46, 1],
					[4, 8, 48, 1],
					[4, 8, 49, 1],
					[4, 9, 5, 1],
					[4, 9, 6, 1],
					[4, 9, 14, 1],
					[4, 9, 15, 1],
					[4, 9, 17, 1],
					[4, 9, 18, 1],
					[4, 9, 19, 1],
					[4, 9, 22, 1],
					[4, 9, 24, 1],
					[4, 9, 25, 1],
					[4, 9, 26, 1],
					[4, 9, 27, 1],
					[4, 9, 28, 1],
					[4, 9, 30, 1],
					[4, 9, 31, 1],
					[4, 9, 32, 1],
					[4, 9, 33, 1],
					[4, 9, 34, 1],
					[4, 9, 40, 1],
					[4, 9, 41, 1],
					[4, 9, 42, 1],
					[4, 9, 46, 1],
					[4, 9, 48, 1],
					[4, 9, 49, 1],
					[4, 13, 5, 1],
					[4, 13, 6, 1],
					[4, 13, 8, 1],
					[4, 13, 10, 1],
					[4, 13, 14, 1],
					[4, 13, 15, 1],
					[4, 13, 17, 1],
					[4, 13, 18, 1],
					[4, 13, 19, 1],
					[4, 13, 22, 1],
					[4, 13, 24, 1],
					[4, 13, 25, 1],
					[4, 13, 26, 1],
					[4, 13, 27, 1],
					[4, 13, 28, 1],
					[4, 13, 30, 1],
					[4, 13, 34, 1],
					[4, 13, 36, 1],
					[4, 13, 37, 1],
					[4, 13, 40, 1],
					[4, 13, 41, 1],
					[4, 13, 42, 1],
					[4, 13, 46, 1],
					[4, 13, 48, 1],
					[4, 13, 49, 1],
					[4, 13, 51, 1],
					[4, 14, 5, 1],
					[4, 14, 6, 1],
					[4, 14, 10, 1],
					[4, 14, 14, 1],
					[4, 14, 15, 1],
					[4, 14, 17, 1],
					[4, 14, 18, 1],
					[4, 14, 19, 1],
					[4, 14, 22, 1],
					[4, 14, 24, 1],
					[4, 14, 25, 1],
					[4, 14, 26, 1],
					[4, 14, 27, 1],
					[4, 14, 28, 1],
					[4, 14, 30, 1],
					[4, 14, 34, 1],
					[4, 14, 36, 1],
					[4, 14, 37, 1],
					[4, 14, 40, 1],
					[4, 14, 41, 1],
					[4, 14, 42, 1],
					[4, 14, 46, 1],
					[4, 14, 48, 1],
					[4, 14, 49, 1],
					[4, 15, 14, 1],
					[4, 15, 15, 1],
					[4, 15, 17, 1],
					[4, 15, 18, 1],
					[4, 15, 19, 1],
					[4, 15, 22, 1],
					[4, 15, 24, 1],
					[4, 15, 25, 1],
					[4, 15, 26, 1],
					[4, 15, 27, 1],
					[4, 15, 28, 1],
					[4, 15, 30, 1],
					[4, 15, 34, 1],
					[4, 15, 36, 1],
					[4, 15, 37, 1],
					[4, 15, 40, 1],
					[4, 15, 41, 1],
					[4, 15, 42, 1],
					[4, 15, 46, 1],
					[4, 15, 48, 1],
					[4, 15, 49, 1],
					[4, 16, 5, 1],
					[4, 16, 6, 1],
					[4, 16, 15, 1],
					[4, 18, 5, 1],
					[4, 18, 6, 1],
					[4, 18, 10, 1],
					[4, 18, 14, 1],
					[4, 18, 15, 1],
					[4, 18, 17, 1],
					[4, 18, 18, 1],
					[4, 18, 19, 1],
					[4, 18, 20, 1],
					[4, 18, 21, 1],
					[4, 18, 22, 1],
					[4, 18, 24, 1],
					[4, 18, 25, 1],
					[4, 18, 26, 1],
					[4, 18, 27, 1],
					[4, 18, 28, 1],
					[4, 18, 30, 1],
					[4, 18, 34, 1],
					[4, 18, 36, 1],
					[4, 18, 37, 1],
					[4, 18, 40, 1],
					[4, 18, 41, 1],
					[4, 18, 42, 1],
					[4, 18, 46, 1],
					[4, 18, 48, 1],
					[4, 18, 49, 1],
					[4, 18, 51, 1],
					[4, 19, 5, 1],
					[4, 19, 6, 1],
					[4, 19, 10, 1],
					[4, 19, 14, 1],
					[4, 19, 15, 1],
					[4, 19, 17, 1],
					[4, 19, 18, 1],
					[4, 19, 19, 1],
					[4, 19, 24, 1],
					[4, 19, 25, 1],
					[4, 19, 26, 1],
					[4, 19, 27, 1],
					[4, 19, 28, 1],
					[4, 19, 30, 1],
					[4, 19, 34, 1],
					[4, 19, 36, 1],
					[4, 19, 37, 1],
					[4, 19, 40, 1],
					[4, 19, 41, 1],
					[4, 19, 42, 1],
					[4, 19, 46, 1],
					[4, 19, 48, 1],
					[4, 19, 49, 1],
					[4, 25, 6, 1],
					[4, 25, 13, 1],
					[4, 26, 14, 1],
					[4, 26, 15, 1],
					[4, 26, 17, 1],
					[4, 26, 18, 1],
					[4, 26, 19, 1],
					[4, 26, 22, 1],
					[4, 26, 24, 1],
					[4, 26, 25, 1],
					[4, 26, 26, 1],
					[4, 26, 27, 1],
					[4, 26, 28, 1],
					[4, 26, 30, 1],
					[4, 26, 34, 1],
					[4, 26, 36, 1],
					[4, 26, 37, 1],
					[4, 26, 40, 1],
					[4, 26, 41, 1],
					[4, 26, 42, 1],
					[4, 26, 46, 1],
					[4, 26, 48, 1],
					[4, 26, 49, 1],
					[4, 26, 51, 1],
					[4, 33, 11, 1],
					[4, 33, 12, 1],
					[4, 33, 14, 1],
					[4, 34, 5, 1],
					[4, 34, 6, 1],
					[4, 34, 10, 1],
					[4, 34, 14, 1],
					[4, 34, 15, 1],
					[4, 34, 17, 1],
					[4, 34, 18, 1],
					[4, 34, 19, 1],
					[4, 34, 22, 1],
					[4, 34, 24, 1],
					[4, 34, 25, 1],
					[4, 34, 26, 1],
					[4, 34, 27, 1],
					[4, 34, 28, 1],
					[4, 34, 30, 1],
					[4, 34, 34, 1],
					[4, 34, 36, 1],
					[4, 34, 37, 1],
					[4, 34, 40, 1],
					[4, 34, 41, 1],
					[4, 34, 42, 1],
					[4, 34, 46, 1],
					[4, 34, 48, 1],
					[4, 34, 49, 1],
					[4, 34, 51, 1],
					[4, 35, 5, 1],
					[4, 35, 6, 1],
					[4, 35, 10, 1],
					[4, 35, 14, 1],
					[4, 35, 15, 1],
					[4, 35, 17, 1],
					[4, 35, 18, 1],
					[4, 35, 19, 1],
					[4, 35, 22, 1],
					[4, 35, 24, 1],
					[4, 35, 25, 1],
					[4, 35, 26, 1],
					[4, 35, 27, 1],
					[4, 35, 28, 1],
					[4, 35, 30, 1],
					[4, 35, 34, 1],
					[4, 35, 36, 1],
					[4, 35, 37, 1],
					[4, 35, 40, 1],
					[4, 35, 41, 1],
					[4, 35, 42, 1],
					[4, 35, 46, 1],
					[4, 35, 48, 1],
					[4, 35, 49, 1],
					[4, 37, 5, 1],
					[4, 37, 6, 1],
					[4, 37, 10, 1],
					[4, 37, 14, 1],
					[4, 37, 15, 1],
					[4, 37, 17, 1],
					[4, 37, 18, 1],
					[4, 37, 19, 1],
					[4, 37, 22, 1],
					[4, 37, 24, 1],
					[4, 37, 25, 1],
					[4, 37, 26, 1],
					[4, 37, 27, 1],
					[4, 37, 28, 1],
					[4, 37, 30, 1],
					[4, 37, 34, 1],
					[4, 37, 36, 1],
					[4, 37, 37, 1],
					[4, 37, 38, 1],
					[4, 37, 40, 1],
					[4, 37, 41, 1],
					[4, 37, 42, 1],
					[4, 37, 46, 1],
					[4, 37, 48, 1],
					[4, 37, 49, 1],
					[4, 40, 16, 1],
					[4, 40, 41, 1],
					[4, 40, 42, 1],
					[4, 41, 5, 1],
					[4, 41, 6, 1],
					[4, 41, 10, 1],
					[4, 41, 14, 1],
					[4, 41, 15, 1],
					[4, 41, 17, 1],
					[4, 41, 18, 1],
					[4, 41, 19, 1],
					[4, 41, 22, 1],
					[4, 41, 24, 1],
					[4, 41, 25, 1],
					[4, 41, 26, 1],
					[4, 41, 27, 1],
					[4, 41, 28, 1],
					[4, 41, 30, 1],
					[4, 41, 34, 1],
					[4, 41, 36, 1],
					[4, 41, 37, 1],
					[4, 41, 40, 1],
					[4, 41, 41, 1],
					[4, 41, 42, 1],
					[4, 41, 46, 1],
					[4, 41, 48, 1],
					[4, 41, 49, 1],
					[4, 41, 51, 1],
					[4, 42, 5, 1],
					[4, 42, 6, 1],
					[4, 42, 10, 1],
					[4, 42, 14, 1],
					[4, 42, 15, 1],
					[4, 42, 17, 1],
					[4, 42, 18, 1],
					[4, 42, 19, 1],
					[4, 42, 22, 1],
					[4, 42, 24, 1],
					[4, 42, 25, 1],
					[4, 42, 26, 1],
					[4, 42, 27, 1],
					[4, 42, 28, 1],
					[4, 42, 30, 1],
					[4, 42, 34, 1],
					[4, 42, 36, 1],
					[4, 42, 37, 1],
					[4, 42, 40, 1],
					[4, 42, 41, 1],
					[4, 42, 42, 1],
					[4, 42, 46, 1],
					[4, 42, 48, 1],
					[4, 42, 49, 1],
					[4, 42, 51, 1],
					[4, 43, 5, 1],
					[4, 43, 6, 1],
					[4, 43, 10, 1],
					[4, 43, 14, 1],
					[4, 43, 15, 1],
					[4, 43, 17, 1],
					[4, 43, 18, 1],
					[4, 43, 19, 1],
					[4, 43, 22, 1],
					[4, 43, 24, 1],
					[4, 43, 25, 1],
					[4, 43, 26, 1],
					[4, 43, 27, 1],
					[4, 43, 28, 1],
					[4, 43, 30, 1],
					[4, 43, 34, 1],
					[4, 43, 36, 1],
					[4, 43, 37, 1],
					[4, 43, 40, 1],
					[4, 43, 41, 1],
					[4, 43, 42, 1],
					[4, 43, 46, 1],
					[4, 43, 48, 1],
					[4, 43, 49, 1],
					[4, 43, 51, 1],
					[4, 45, 14, 1],
					[4, 51, 5, 1],
					[4, 51, 6, 1],
					[4, 51, 8, 1],
					[4, 51, 10, 1],
					[4, 51, 14, 1],
					[4, 51, 15, 1],
					[4, 51, 17, 1],
					[4, 51, 18, 1],
					[4, 51, 19, 1],
					[4, 51, 22, 1],
					[4, 51, 24, 1],
					[4, 51, 25, 1],
					[4, 51, 26, 1],
					[4, 51, 27, 1],
					[4, 51, 28, 1],
					[4, 51, 30, 1],
					[4, 51, 34, 1],
					[4, 51, 36, 1],
					[4, 51, 37, 1],
					[4, 51, 40, 1],
					[4, 51, 41, 1],
					[4, 51, 42, 1],
					[4, 51, 46, 1],
					[4, 51, 48, 1],
					[4, 51, 49, 1],
					[4, 54, 5, 1],
					[4, 54, 6, 1],
					[4, 54, 10, 1],
					[4, 54, 14, 1],
					[4, 54, 15, 1],
					[4, 54, 17, 1],
					[4, 54, 18, 1],
					[4, 54, 19, 1],
					[4, 54, 22, 1],
					[4, 54, 26, 1],
					[4, 54, 27, 1],
					[4, 54, 34, 1],
					[4, 54, 35, 1],
					[4, 54, 41, 1],
					[4, 54, 42, 1],
					[4, 54, 46, 1],
					[4, 54, 48, 1],
					[4, 54, 49, 1],
					[4, 54, 50, 1],
					[4, 57, 5, 1],
					[4, 57, 6, 1],
					[4, 57, 8, 1],
					[4, 57, 14, 1],
					[4, 57, 15, 1],
					[4, 57, 17, 1],
					[4, 57, 18, 1],
					[4, 57, 19, 1],
					[4, 57, 22, 1],
					[4, 57, 24, 1],
					[4, 57, 25, 1],
					[4, 57, 26, 1],
					[4, 57, 27, 1],
					[4, 57, 28, 1],
					[4, 57, 30, 1],
					[4, 57, 34, 1],
					[4, 57, 36, 1],
					[4, 57, 37, 1],
					[4, 57, 40, 1],
					[4, 57, 41, 1],
					[4, 57, 42, 1],
					[4, 57, 46, 1],
					[4, 57, 48, 1],
					[4, 57, 49, 1],
					[4, 58, 5, 1],
					[4, 58, 6, 1],
					[4, 58, 8, 1],
					[4, 58, 14, 1],
					[4, 58, 15, 1],
					[4, 58, 17, 1],
					[4, 58, 18, 1],
					[4, 58, 19, 1],
					[4, 58, 22, 1],
					[4, 58, 24, 1],
					[4, 58, 25, 1],
					[4, 58, 26, 1],
					[4, 58, 27, 1],
					[4, 58, 28, 1],
					[4, 58, 30, 1],
					[4, 58, 34, 1],
					[4, 58, 36, 1],
					[4, 58, 37, 1],
					[4, 58, 38, 1],
					[4, 58, 40, 1],
					[4, 58, 41, 1],
					[4, 58, 42, 1],
					[4, 58, 46, 1],
					[4, 58, 48, 1],
					[4, 58, 49, 1],
					[4, 59, 5, 1],
					[4, 59, 6, 1],
					[4, 59, 8, 1],
					[4, 59, 14, 1],
					[4, 59, 15, 1],
					[4, 59, 17, 1],
					[4, 59, 18, 1],
					[4, 59, 19, 1],
					[4, 59, 22, 1],
					[4, 59, 24, 1],
					[4, 59, 25, 1],
					[4, 59, 26, 1],
					[4, 59, 27, 1],
					[4, 59, 28, 1],
					[4, 59, 30, 1],
					[4, 59, 34, 1],
					[4, 59, 36, 1],
					[4, 59, 37, 1],
					[4, 59, 40, 1],
					[4, 59, 41, 1],
					[4, 59, 42, 1],
					[4, 59, 46, 1],
					[4, 59, 48, 1],
					[4, 59, 49, 1],
					[4, 60, 5, 1],
					[4, 60, 6, 1],
					[4, 60, 10, 1],
					[4, 60, 14, 1],
					[4, 60, 15, 1],
					[4, 60, 17, 1],
					[4, 60, 18, 1],
					[4, 60, 19, 1],
					[4, 60, 22, 1],
					[4, 60, 24, 1],
					[4, 60, 25, 1],
					[4, 60, 26, 1],
					[4, 60, 27, 1],
					[4, 60, 28, 1],
					[4, 60, 30, 1],
					[4, 60, 34, 1],
					[4, 60, 36, 1],
					[4, 60, 37, 1],
					[4, 60, 40, 1],
					[4, 60, 41, 1],
					[4, 60, 42, 1],
					[4, 60, 46, 1],
					[4, 60, 48, 1],
					[4, 60, 49, 1],
					[4, 61, 5, 1],
					[4, 61, 6, 1],
					[4, 61, 8, 1],
					[4, 61, 14, 1],
					[4, 61, 15, 1],
					[4, 61, 17, 1],
					[4, 61, 18, 1],
					[4, 61, 19, 1],
					[4, 61, 22, 1],
					[4, 61, 24, 1],
					[4, 61, 25, 1],
					[4, 61, 26, 1],
					[4, 61, 27, 1],
					[4, 61, 28, 1],
					[4, 61, 30, 1],
					[4, 61, 34, 1],
					[4, 61, 36, 1],
					[4, 61, 37, 1],
					[4, 61, 40, 1],
					[4, 61, 41, 1],
					[4, 61, 42, 1],
					[4, 61, 46, 1],
					[4, 61, 48, 1],
					[4, 61, 49, 1],
					[4, 61, 51, 1],
					[4, 74, 5, 1],
					[4, 74, 6, 1],
					[4, 74, 8, 1],
					[4, 74, 14, 1],
					[4, 74, 17, 1],
					[4, 74, 18, 1],
					[4, 74, 19, 1],
					[4, 74, 22, 1],
					[4, 74, 26, 1],
					[4, 74, 27, 1],
					[4, 74, 34, 1],
					[4, 74, 41, 1],
					[4, 74, 42, 1],
					[4, 74, 46, 1],
					[4, 74, 48, 1],
					[4, 74, 49, 1],
					[4, 75, 5, 1],
					[4, 75, 6, 1],
					[4, 75, 8, 1],
					[4, 75, 14, 1],
					[4, 75, 17, 1],
					[4, 75, 18, 1],
					[4, 75, 19, 1],
					[4, 75, 22, 1],
					[4, 75, 24, 1],
					[4, 75, 25, 1],
					[4, 75, 26, 1],
					[4, 75, 27, 1],
					[4, 75, 28, 1],
					[4, 75, 30, 1],
					[4, 75, 34, 1],
					[4, 75, 36, 1],
					[4, 75, 37, 1],
					[4, 75, 40, 1],
					[4, 75, 41, 1],
					[4, 75, 42, 1],
					[4, 75, 46, 1],
					[4, 75, 48, 1],
					[4, 75, 49, 1],
					[4, 78, 5, 1],
					[4, 78, 6, 1],
					[4, 78, 8, 1],
					[4, 78, 14, 1],
					[4, 78, 17, 1],
					[4, 78, 18, 1],
					[4, 78, 19, 1],
					[4, 78, 22, 1],
					[4, 78, 24, 1],
					[4, 78, 25, 1],
					[4, 78, 26, 1],
					[4, 78, 27, 1],
					[4, 78, 28, 1],
					[4, 78, 30, 1],
					[4, 78, 34, 1],
					[4, 78, 36, 1],
					[4, 78, 37, 1],
					[4, 78, 40, 1],
					[4, 78, 41, 1],
					[4, 78, 42, 1],
					[4, 78, 46, 1],
					[4, 78, 48, 1],
					[4, 78, 49, 1],
					[4, 79, 5, 1],
					[4, 79, 6, 1],
					[4, 79, 8, 1],
					[4, 79, 14, 1],
					[4, 79, 17, 1],
					[4, 79, 18, 1],
					[4, 79, 19, 1],
					[4, 79, 22, 1],
					[4, 79, 24, 1],
					[4, 79, 25, 1],
					[4, 79, 26, 1],
					[4, 79, 27, 1],
					[4, 79, 28, 1],
					[4, 79, 30, 1],
					[4, 79, 34, 1],
					[4, 79, 36, 1],
					[4, 79, 37, 1],
					[4, 79, 40, 1],
					[4, 79, 41, 1],
					[4, 79, 42, 1],
					[4, 79, 46, 1],
					[4, 79, 48, 1],
					[4, 79, 49, 1],
					[4, 80, 5, 1],
					[4, 80, 6, 1],
					[4, 80, 8, 1],
					[4, 80, 14, 1],
					[4, 80, 17, 1],
					[4, 80, 18, 1],
					[4, 80, 19, 1],
					[4, 80, 22, 1],
					[4, 80, 24, 1],
					[4, 80, 25, 1],
					[4, 80, 26, 1],
					[4, 80, 27, 1],
					[4, 80, 28, 1],
					[4, 80, 30, 1],
					[4, 80, 34, 1],
					[4, 80, 36, 1],
					[4, 80, 37, 1],
					[4, 80, 40, 1],
					[4, 80, 41, 1],
					[4, 80, 42, 1],
					[4, 80, 46, 1],
					[4, 80, 48, 1],
					[4, 80, 49, 1],
					[4, 81, 5, 1],
					[4, 81, 6, 1],
					[4, 81, 8, 1],
					[4, 81, 14, 1],
					[4, 81, 17, 1],
					[4, 81, 18, 1],
					[4, 81, 19, 1],
					[4, 81, 22, 1],
					[4, 81, 24, 1],
					[4, 81, 25, 1],
					[4, 81, 26, 1],
					[4, 81, 27, 1],
					[4, 81, 28, 1],
					[4, 81, 30, 1],
					[4, 81, 34, 1],
					[4, 81, 36, 1],
					[4, 81, 37, 1],
					[4, 81, 40, 1],
					[4, 81, 41, 1],
					[4, 81, 42, 1],
					[4, 81, 46, 1],
					[4, 81, 48, 1],
					[4, 81, 49, 1],
					[4, 82, 5, 1],
					[4, 82, 6, 1],
					[4, 82, 8, 1],
					[4, 82, 14, 1],
					[4, 82, 17, 1],
					[4, 82, 18, 1],
					[4, 82, 19, 1],
					[4, 82, 22, 1],
					[4, 82, 24, 1],
					[4, 82, 25, 1],
					[4, 82, 26, 1],
					[4, 82, 27, 1],
					[4, 82, 28, 1],
					[4, 82, 30, 1],
					[4, 82, 34, 1],
					[4, 82, 36, 1],
					[4, 82, 37, 1],
					[4, 82, 40, 1],
					[4, 82, 41, 1],
					[4, 82, 42, 1],
					[4, 82, 46, 1],
					[4, 82, 48, 1],
					[4, 82, 49, 1],
					[4, 83, 5, 1],
					[4, 83, 6, 1],
					[4, 83, 8, 1],
					[4, 83, 14, 1],
					[4, 83, 17, 1],
					[4, 83, 18, 1],
					[4, 83, 19, 1],
					[4, 83, 22, 1],
					[4, 83, 24, 1],
					[4, 83, 25, 1],
					[4, 83, 26, 1],
					[4, 83, 27, 1],
					[4, 83, 28, 1],
					[4, 83, 30, 1],
					[4, 83, 34, 1],
					[4, 83, 36, 1],
					[4, 83, 37, 1],
					[4, 83, 40, 1],
					[4, 83, 41, 1],
					[4, 83, 42, 1],
					[4, 83, 46, 1],
					[4, 83, 48, 1],
					[4, 83, 49, 1],
					[4, 84, 5, 1],
					[4, 84, 6, 1],
					[4, 84, 8, 1],
					[4, 84, 14, 1],
					[4, 84, 15, 1],
					[4, 84, 17, 1],
					[4, 84, 18, 1],
					[4, 84, 19, 1],
					[4, 84, 22, 1],
					[4, 84, 24, 1],
					[4, 84, 25, 1],
					[4, 84, 26, 1],
					[4, 84, 27, 1],
					[4, 84, 28, 1],
					[4, 84, 30, 1],
					[4, 84, 34, 1],
					[4, 84, 36, 1],
					[4, 84, 37, 1],
					[4, 84, 40, 1],
					[4, 84, 41, 1],
					[4, 84, 42, 1],
					[4, 84, 46, 1],
					[4, 84, 48, 1],
					[4, 84, 49, 1],
					[4, 85, 5, 1],
					[4, 85, 6, 1],
					[4, 85, 8, 1],
					[4, 85, 14, 1],
					[4, 85, 17, 1],
					[4, 85, 18, 1],
					[4, 85, 19, 1],
					[4, 85, 22, 1],
					[4, 85, 24, 1],
					[4, 85, 25, 1],
					[4, 85, 26, 1],
					[4, 85, 27, 1],
					[4, 85, 28, 1],
					[4, 85, 30, 1],
					[4, 85, 34, 1],
					[4, 85, 36, 1],
					[4, 85, 37, 1],
					[4, 85, 38, 1],
					[4, 85, 39, 1],
					[4, 85, 40, 1],
					[4, 85, 41, 1],
					[4, 85, 42, 1],
					[4, 85, 46, 1],
					[4, 85, 48, 1],
					[4, 85, 49, 1],
					[4, 85, 51, 1],
					[4, 86, 5, 1],
					[4, 86, 6, 1],
					[4, 86, 8, 1],
					[4, 86, 14, 1],
					[4, 86, 15, 1],
					[4, 86, 17, 1],
					[4, 86, 18, 1],
					[4, 86, 19, 1],
					[4, 86, 22, 1],
					[4, 86, 24, 1],
					[4, 86, 25, 1],
					[4, 86, 26, 1],
					[4, 86, 27, 1],
					[4, 86, 28, 1],
					[4, 86, 30, 1],
					[4, 86, 34, 1],
					[4, 86, 36, 1],
					[4, 86, 37, 1],
					[4, 86, 40, 1],
					[4, 86, 41, 1],
					[4, 86, 42, 1],
					[4, 86, 46, 1],
					[4, 86, 48, 1],
					[4, 86, 49, 1],
					[4, 86, 51, 1],
					[4, 87, 5, 1],
					[4, 87, 6, 1],
					[4, 87, 8, 1],
					[4, 87, 14, 1],
					[4, 87, 17, 1],
					[4, 87, 18, 1],
					[4, 87, 19, 1],
					[4, 87, 22, 1],
					[4, 87, 24, 1],
					[4, 87, 25, 1],
					[4, 87, 26, 1],
					[4, 87, 27, 1],
					[4, 87, 28, 1],
					[4, 87, 30, 1],
					[4, 87, 34, 1],
					[4, 87, 36, 1],
					[4, 87, 37, 1],
					[4, 87, 38, 1],
					[4, 87, 39, 1],
					[4, 87, 40, 1],
					[4, 87, 41, 1],
					[4, 87, 42, 1],
					[4, 87, 46, 1],
					[4, 87, 48, 1],
					[4, 87, 49, 1],
					[4, 87, 51, 1],
					[4, 88, 5, 1],
					[4, 88, 6, 1],
					[4, 88, 8, 1],
					[4, 88, 14, 1],
					[4, 88, 17, 1],
					[4, 88, 18, 1],
					[4, 88, 19, 1],
					[4, 88, 22, 1],
					[4, 88, 24, 1],
					[4, 88, 25, 1],
					[4, 88, 26, 1],
					[4, 88, 27, 1],
					[4, 88, 28, 1],
					[4, 88, 30, 1],
					[4, 88, 34, 1],
					[4, 88, 36, 1],
					[4, 88, 37, 1],
					[4, 88, 38, 1],
					[4, 88, 39, 1],
					[4, 88, 40, 1],
					[4, 88, 41, 1],
					[4, 88, 42, 1],
					[4, 88, 46, 1],
					[4, 88, 48, 1],
					[4, 88, 49, 1],
					[4, 88, 51, 1],
					[4, 89, 5, 1],
					[4, 89, 6, 1],
					[4, 89, 8, 1],
					[4, 89, 14, 1],
					[4, 89, 17, 1],
					[4, 89, 18, 1],
					[4, 89, 19, 1],
					[4, 89, 22, 1],
					[4, 89, 24, 1],
					[4, 89, 25, 1],
					[4, 89, 26, 1],
					[4, 89, 27, 1],
					[4, 89, 28, 1],
					[4, 89, 30, 1],
					[4, 89, 34, 1],
					[4, 89, 36, 1],
					[4, 89, 37, 1],
					[4, 89, 38, 1],
					[4, 89, 39, 1],
					[4, 89, 40, 1],
					[4, 89, 41, 1],
					[4, 89, 42, 1],
					[4, 89, 46, 1],
					[4, 89, 48, 1],
					[4, 89, 49, 1],
					[4, 89, 51, 1],
					[4, 90, 5, 1],
					[4, 90, 6, 1],
					[4, 90, 8, 1],
					[4, 90, 14, 1],
					[4, 90, 17, 1],
					[4, 90, 18, 1],
					[4, 90, 19, 1],
					[4, 90, 22, 1],
					[4, 90, 24, 1],
					[4, 90, 25, 1],
					[4, 90, 26, 1],
					[4, 90, 27, 1],
					[4, 90, 28, 1],
					[4, 90, 30, 1],
					[4, 90, 34, 1],
					[4, 90, 36, 1],
					[4, 90, 37, 1],
					[4, 90, 38, 1],
					[4, 90, 39, 1],
					[4, 90, 40, 1],
					[4, 90, 41, 1],
					[4, 90, 42, 1],
					[4, 90, 46, 1],
					[4, 90, 48, 1],
					[4, 90, 49, 1],
					[4, 90, 51, 1],
					[4, 91, 5, 1],
					[4, 91, 6, 1],
					[4, 91, 8, 1],
					[4, 91, 14, 1],
					[4, 91, 17, 1],
					[4, 91, 18, 1],
					[4, 91, 19, 1],
					[4, 91, 22, 1],
					[4, 91, 24, 1],
					[4, 91, 25, 1],
					[4, 91, 26, 1],
					[4, 91, 27, 1],
					[4, 91, 28, 1],
					[4, 91, 30, 1],
					[4, 91, 34, 1],
					[4, 91, 36, 1],
					[4, 91, 37, 1],
					[4, 91, 38, 1],
					[4, 91, 39, 1],
					[4, 91, 40, 1],
					[4, 91, 41, 1],
					[4, 91, 42, 1],
					[4, 91, 46, 1],
					[4, 91, 48, 1],
					[4, 91, 49, 1],
					[4, 91, 51, 1],
					[4, 92, 5, 1],
					[4, 92, 6, 1],
					[4, 92, 8, 1],
					[4, 92, 14, 1],
					[4, 92, 17, 1],
					[4, 92, 18, 1],
					[4, 92, 19, 1],
					[4, 92, 20, 1],
					[4, 92, 22, 1],
					[4, 92, 24, 1],
					[4, 92, 25, 1],
					[4, 92, 26, 1],
					[4, 92, 27, 1],
					[4, 92, 28, 1],
					[4, 92, 30, 1],
					[4, 92, 34, 1],
					[4, 92, 36, 1],
					[4, 92, 37, 1],
					[4, 92, 40, 1],
					[4, 92, 41, 1],
					[4, 92, 42, 1],
					[4, 92, 46, 1],
					[4, 92, 48, 1],
					[4, 92, 49, 1],
					[4, 92, 51, 1],
					[4, 93, 5, 1],
					[4, 93, 6, 1],
					[4, 93, 8, 1],
					[4, 93, 14, 1],
					[4, 93, 17, 1],
					[4, 93, 18, 1],
					[4, 93, 19, 1],
					[4, 93, 20, 1],
					[4, 93, 22, 1],
					[4, 93, 24, 1],
					[4, 93, 25, 1],
					[4, 93, 26, 1],
					[4, 93, 27, 1],
					[4, 93, 28, 1],
					[4, 93, 30, 1],
					[4, 93, 34, 1],
					[4, 93, 36, 1],
					[4, 93, 37, 1],
					[4, 93, 40, 1],
					[4, 93, 41, 1],
					[4, 93, 42, 1],
					[4, 93, 46, 1],
					[4, 93, 48, 1],
					[4, 93, 49, 1],
					[4, 93, 51, 1],
					[4, 94, 5, 1],
					[4, 94, 6, 1],
					[4, 94, 8, 1],
					[4, 94, 14, 1],
					[4, 94, 34, 1],
					[4, 94, 36, 1],
					[4, 94, 37, 1],
					[4, 94, 41, 1],
					[4, 94, 42, 1],
					[4, 94, 46, 1],
					[4, 94, 48, 1],
					[4, 94, 49, 1],
					[4, 94, 51, 1],
					[4, 95, 5, 1],
					[4, 95, 6, 1],
					[4, 95, 8, 1],
					[4, 95, 14, 1],
					[4, 95, 34, 1],
					[4, 95, 36, 1],
					[4, 95, 37, 1],
					[4, 95, 41, 1],
					[4, 95, 42, 1],
					[4, 95, 46, 1],
					[4, 95, 48, 1],
					[4, 95, 49, 1],
					[4, 95, 51, 1],
					[4, 96, 5, 1],
					[4, 96, 6, 1],
					[4, 96, 8, 1],
					[4, 96, 14, 1],
					[4, 96, 34, 1],
					[4, 96, 36, 1],
					[4, 96, 37, 1],
					[4, 96, 41, 1],
					[4, 96, 42, 1],
					[4, 96, 46, 1],
					[4, 96, 48, 1],
					[4, 96, 49, 1],
					[4, 97, 5, 1],
					[4, 97, 6, 1],
					[4, 97, 8, 1],
					[4, 97, 14, 1],
					[4, 97, 34, 1],
					[4, 97, 36, 1],
					[4, 97, 37, 1],
					[4, 97, 41, 1],
					[4, 97, 42, 1],
					[4, 97, 46, 1],
					[4, 97, 48, 1],
					[4, 97, 49, 1],
					[4, 98, 5, 1],
					[4, 98, 6, 1],
					[4, 98, 8, 1],
					[4, 98, 14, 1],
					[4, 98, 34, 1],
					[4, 98, 36, 1],
					[4, 98, 37, 1],
					[4, 98, 38, 1],
					[4, 98, 41, 1],
					[4, 98, 42, 1],
					[4, 98, 46, 1],
					[4, 98, 48, 1],
					[4, 98, 49, 1],
					[4, 99, 5, 1],
					[4, 99, 6, 1],
					[4, 99, 8, 1],
					[4, 99, 14, 1],
					[4, 99, 34, 1],
					[4, 99, 36, 1],
					[4, 99, 37, 1],
					[4, 99, 41, 1],
					[4, 99, 42, 1],
					[4, 99, 46, 1],
					[4, 99, 48, 1],
					[4, 99, 49, 1],
					[4, 100, 5, 1],
					[4, 100, 6, 1],
					[4, 100, 8, 1],
					[4, 100, 14, 1],
					[4, 100, 34, 1],
					[4, 100, 36, 1],
					[4, 100, 37, 1],
					[4, 100, 38, 1],
					[4, 100, 41, 1],
					[4, 100, 42, 1],
					[4, 100, 46, 1],
					[4, 100, 48, 1],
					[4, 100, 49, 1],
					[4, 101, 5, 1],
					[4, 101, 6, 1],
					[4, 101, 8, 1],
					[4, 101, 14, 1],
					[4, 101, 34, 1],
					[4, 101, 36, 1],
					[4, 101, 37, 1],
					[4, 101, 38, 1],
					[4, 101, 41, 1],
					[4, 101, 42, 1],
					[4, 101, 46, 1],
					[4, 101, 48, 1],
					[4, 101, 49, 1],
					[4, 102, 5, 1],
					[4, 102, 6, 1],
					[4, 102, 8, 1],
					[4, 102, 14, 1],
					[4, 102, 34, 1],
					[4, 102, 36, 1],
					[4, 102, 37, 1],
					[4, 102, 38, 1],
					[4, 102, 41, 1],
					[4, 102, 42, 1],
					[4, 102, 46, 1],
					[4, 102, 48, 1],
					[4, 102, 49, 1],
					[4, 103, 5, 1],
					[4, 103, 6, 1],
					[4, 103, 8, 1],
					[4, 103, 14, 1],
					[4, 103, 34, 1],
					[4, 103, 36, 1],
					[4, 103, 37, 1],
					[4, 103, 38, 1],
					[4, 103, 41, 1],
					[4, 103, 42, 1],
					[4, 103, 46, 1],
					[4, 103, 48, 1],
					[4, 103, 49, 1],
					[4, 104, 5, 1],
					[4, 104, 6, 1],
					[4, 104, 8, 1],
					[4, 104, 14, 1],
					[4, 104, 34, 1],
					[4, 104, 36, 1],
					[4, 104, 37, 1],
					[4, 104, 38, 1],
					[4, 104, 41, 1],
					[4, 104, 42, 1],
					[4, 104, 46, 1],
					[4, 104, 48, 1],
					[4, 104, 49, 1],
					[4, 105, 5, 1],
					[4, 105, 6, 1],
					[4, 105, 8, 1],
					[4, 105, 14, 1],
					[4, 105, 34, 1],
					[4, 105, 36, 1],
					[4, 105, 37, 1],
					[4, 105, 38, 1],
					[4, 105, 41, 1],
					[4, 105, 42, 1],
					[4, 105, 46, 1],
					[4, 105, 48, 1],
					[4, 105, 49, 1],
					[4, 106, 5, 1],
					[4, 106, 6, 1],
					[4, 106, 8, 1],
					[4, 106, 14, 1],
					[4, 106, 34, 1],
					[4, 106, 36, 1],
					[4, 106, 37, 1],
					[4, 106, 38, 1],
					[4, 106, 41, 1],
					[4, 106, 42, 1],
					[4, 106, 46, 1],
					[4, 106, 48, 1],
					[4, 106, 49, 1],
					[4, 107, 5, 1],
					[4, 107, 6, 1],
					[4, 107, 8, 1],
					[4, 107, 14, 1],
					[4, 107, 34, 1],
					[4, 107, 36, 1],
					[4, 107, 37, 1],
					[4, 107, 41, 1],
					[4, 107, 42, 1],
					[4, 107, 46, 1],
					[4, 107, 48, 1],
					[4, 107, 49, 1],
					[4, 108, 5, 1],
					[4, 108, 6, 1],
					[4, 108, 8, 1],
					[4, 108, 14, 1],
					[4, 108, 34, 1],
					[4, 108, 36, 1],
					[4, 108, 37, 1],
					[4, 108, 38, 1],
					[4, 108, 41, 1],
					[4, 108, 42, 1],
					[4, 108, 46, 1],
					[4, 108, 48, 1],
					[4, 108, 49, 1],
					[4, 109, 5, 1],
					[4, 109, 6, 1],
					[4, 109, 8, 1],
					[4, 109, 14, 1],
					[4, 109, 34, 1],
					[4, 109, 36, 1],
					[4, 109, 37, 1],
					[4, 109, 38, 1],
					[4, 109, 41, 1],
					[4, 109, 42, 1],
					[4, 109, 46, 1],
					[4, 109, 48, 1],
					[4, 109, 49, 1],
					[4, 111, 5, 1],
					[4, 111, 6, 1],
					[4, 111, 10, 1],
					[4, 111, 14, 1],
					[4, 111, 15, 1],
					[4, 111, 17, 1],
					[4, 111, 18, 1],
					[4, 111, 19, 1],
					[4, 111, 22, 1],
					[4, 111, 24, 1],
					[4, 111, 25, 1],
					[4, 111, 26, 1],
					[4, 111, 27, 1],
					[4, 111, 28, 1],
					[4, 111, 30, 1],
					[4, 111, 34, 1],
					[4, 111, 36, 1],
					[4, 111, 37, 1],
					[4, 111, 40, 1],
					[4, 111, 41, 1],
					[4, 111, 42, 1],
					[4, 111, 44, 1],
					[4, 111, 46, 1],
					[4, 111, 47, 1],
					[4, 111, 48, 1],
					[4, 112, 5, 1],
					[4, 112, 6, 1],
					[4, 112, 14, 1],
					[4, 112, 15, 1],
					[4, 112, 17, 1],
					[4, 112, 18, 1],
					[4, 112, 19, 1],
					[4, 112, 20, 1],
					[4, 112, 22, 1],
					[4, 112, 26, 1],
					[4, 112, 27, 1],
					[4, 112, 34, 1],
					[4, 112, 36, 1],
					[4, 112, 37, 1],
					[4, 112, 40, 1],
					[4, 112, 41, 1],
					[4, 112, 42, 1],
					[4, 112, 46, 1],
					[4, 112, 48, 1],
					[4, 112, 49, 1],
					[4, 113, 5, 1],
					[4, 113, 6, 1],
					[4, 113, 14, 1],
					[4, 113, 15, 1],
					[4, 113, 17, 1],
					[4, 113, 18, 1],
					[4, 113, 19, 1],
					[4, 113, 20, 1],
					[4, 113, 22, 1],
					[4, 113, 26, 1],
					[4, 113, 27, 1],
					[4, 113, 34, 1],
					[4, 113, 36, 1],
					[4, 113, 37, 1],
					[4, 113, 40, 1],
					[4, 113, 41, 1],
					[4, 113, 42, 1],
					[4, 113, 46, 1],
					[4, 113, 48, 1],
					[4, 113, 49, 1],
					[4, 114, 5, 1],
					[4, 114, 6, 1],
					[4, 114, 14, 1],
					[4, 114, 15, 1],
					[4, 114, 17, 1],
					[4, 114, 18, 1],
					[4, 114, 19, 1],
					[4, 114, 20, 1],
					[4, 114, 22, 1],
					[4, 114, 26, 1],
					[4, 114, 27, 1],
					[4, 114, 34, 1],
					[4, 114, 36, 1],
					[4, 114, 37, 1],
					[4, 114, 40, 1],
					[4, 114, 41, 1],
					[4, 114, 42, 1],
					[4, 114, 46, 1],
					[4, 114, 48, 1],
					[4, 114, 49, 1],
					[4, 115, 5, 1],
					[4, 115, 6, 1],
					[4, 115, 14, 1],
					[4, 115, 15, 1],
					[4, 115, 17, 1],
					[4, 115, 18, 1],
					[4, 115, 19, 1],
					[4, 115, 20, 1],
					[4, 115, 22, 1],
					[4, 115, 26, 1],
					[4, 115, 27, 1],
					[4, 115, 34, 1],
					[4, 115, 36, 1],
					[4, 115, 37, 1],
					[4, 115, 40, 1],
					[4, 115, 41, 1],
					[4, 115, 42, 1],
					[4, 115, 46, 1],
					[4, 115, 48, 1],
					[4, 115, 49, 1],
					[4, 116, 5, 1],
					[4, 116, 6, 1],
					[4, 116, 14, 1],
					[4, 116, 15, 1],
					[4, 116, 17, 1],
					[4, 116, 18, 1],
					[4, 116, 19, 1],
					[4, 116, 20, 1],
					[4, 116, 22, 1],
					[4, 116, 26, 1],
					[4, 116, 27, 1],
					[4, 116, 34, 1],
					[4, 116, 36, 1],
					[4, 116, 37, 1],
					[4, 116, 40, 1],
					[4, 116, 41, 1],
					[4, 116, 42, 1],
					[4, 116, 46, 1],
					[4, 116, 48, 1],
					[4, 116, 49, 1],
					[4, 117, 5, 1],
					[4, 117, 6, 1],
					[4, 117, 14, 1],
					[4, 117, 15, 1],
					[4, 117, 17, 1],
					[4, 117, 18, 1],
					[4, 117, 19, 1],
					[4, 117, 20, 1],
					[4, 117, 22, 1],
					[4, 117, 26, 1],
					[4, 117, 27, 1],
					[4, 117, 34, 1],
					[4, 117, 36, 1],
					[4, 117, 37, 1],
					[4, 117, 40, 1],
					[4, 117, 41, 1],
					[4, 117, 42, 1],
					[4, 117, 46, 1],
					[4, 117, 48, 1],
					[4, 117, 49, 1],
					[4, 117, 51, 1],
				]
			],
			'vtiger_progress' => [
				'columns' => ['progressid', 'progress', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, '10%', 1, 278, 2],
					[3, '20%', 1, 279, 3],
					[4, '30%', 1, 280, 4],
					[5, '40%', 1, 281, 5],
					[6, '50%', 1, 282, 6],
					[7, '60%', 1, 283, 7],
					[8, '70%', 1, 284, 8],
					[9, '80%', 1, 285, 9],
					[10, '90%', 1, 286, 10],
					[11, '100%', 1, 287, 11],
				]
			],
			'vtiger_progress_seq' => [
				'columns' => ['id'],
				'values' => [
					[11],
				]
			],
			'vtiger_projectmilestone_priority' => [
				'columns' => ['projectmilestone_priorityid', 'projectmilestone_priority', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_LOW', 1, 659, 1],
					[2, 'PLL_NORMAL', 1, 660, 2],
					[3, 'PLL_HIGH', 1, 661, 3],
				]
			],
			'vtiger_projectmilestone_priority_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_projectmilestonetype' => [
				'columns' => ['projectmilestonetypeid', 'projectmilestonetype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'PLL_INTERNAL', 1, 238, 2],
					[3, 'PLL_EXTERNAL', 1, 239, 3],
					[4, 'PLL_SHARED', 1, 240, 4],
				]
			],
			'vtiger_projectmilestonetype_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_projectpriority' => [
				'columns' => ['projectpriorityid', 'projectpriority', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'low', 1, 274, 2],
					[3, 'normal', 1, 275, 3],
					[4, 'high', 1, 276, 4],
				]
			],
			'vtiger_projectpriority_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_projectstatus' => [
				'columns' => ['projectstatusid', 'projectstatus', 'presence', 'picklist_valueid', 'sortorderid', 'color'],
				'values' => [
					[2, 'prospecting', 1, 261, 2, '#E6FAD8'],
					[3, 'initiated', 1, 262, 3, '#E6FAD8'],
					[4, 'in progress', 1, 263, 4, '#E6FAD8'],
					[5, 'waiting for feedback', 1, 264, 5, '#E6FAD8'],
					[6, 'on hold', 1, 265, 6, '#E6FAD8'],
					[7, 'completed', 1, 266, 7, '#E6FAD8'],
					[8, 'delivered', 1, 267, 8, '#E6FAD8'],
					[9, 'archived', 1, 268, 9, '#E6FAD8'],
				]
			],
			'vtiger_projectstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[9],
				]
			],
			'vtiger_projecttaskpriority' => [
				'columns' => ['projecttaskpriorityid', 'projecttaskpriority', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'low', 1, 246, 2],
					[3, 'normal', 1, 247, 3],
					[4, 'high', 1, 248, 4],
				]
			],
			'vtiger_projecttaskpriority_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_projecttaskprogress' => [
				'columns' => ['projecttaskprogressid', 'projecttaskprogress', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, '10%', 1, 250, 2],
					[3, '20%', 1, 251, 3],
					[4, '30%', 1, 252, 4],
					[5, '40%', 1, 253, 5],
					[6, '50%', 1, 254, 6],
					[7, '60%', 1, 255, 7],
					[8, '70%', 1, 256, 8],
					[9, '80%', 1, 257, 9],
					[10, '90%', 1, 258, 10],
					[11, '100%', 1, 259, 11],
				]
			],
			'vtiger_projecttaskprogress_seq' => [
				'columns' => ['id'],
				'values' => [
					[11],
				]
			],
			'vtiger_projecttaskstatus' => [
				'columns' => ['projecttaskstatusid', 'projecttaskstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Open', 0, 290, 2],
					[3, 'In Progress', 0, 291, 3],
					[4, 'Completed', 0, 292, 4],
					[5, 'Deferred', 0, 293, 5],
					[6, 'Cancelled', 0, 294, 6],
				]
			],
			'vtiger_projecttaskstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[6],
				]
			],
			'vtiger_projecttasktype' => [
				'columns' => ['projecttasktypeid', 'projecttasktype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'administrative', 1, 242, 2],
					[3, 'operative', 1, 243, 3],
					[4, 'other', 1, 244, 4],
				]
			],
			'vtiger_projecttasktype_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_projecttype' => [
				'columns' => ['projecttypeid', 'projecttype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'PLL_INTERNAL', 1, 270, 2],
					[3, 'PLL_EXTERNAL', 1, 271, 3],
					[4, 'PLL_COMMON', 1, 272, 4],
				]
			],
			'vtiger_projecttype_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
		];
	}
}
