<?php

namespace Importers;

/**
 * Class that imports base database.
 *
 * @copyright YetiForce Sp. z o.o.
 * @license   YetiForce Public License 3.0 (licenses/LicenseEN.txt or yetiforce.com)
 * @author    Mariusz Krzaczkowski <m.krzaczkowski@yetiforce.com>
 */
class Base4 extends \App\Db\Importers\Base
{
	public $dbType = 'base';

	public function scheme()
	{
		$this->tables = [
			'vtiger_realization_process' => [
				'columns' => [
					'module_id' => $this->integer(10)->notNull(),
					'status_indicate_closing' => $this->stringType(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_recurring_frequency' => [
				'columns' => [
					'recurring_frequency_id' => $this->integer(10),
					'recurring_frequency' => $this->stringType(200),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_recurring_frequency_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_rel_mod' => [
				'columns' => [
					'rel_modid' => $this->primaryKey(10),
					'rel_mod' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_rel_mod_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_relatedlists' => [
				'columns' => [
					'relation_id' => $this->primaryKey(5)->unsigned(),
					'tabid' => $this->smallInteger(5)->unsigned()->notNull(),
					'related_tabid' => $this->smallInteger(5)->unsigned()->notNull(),
					'name' => $this->stringType(50),
					'sequence' => $this->smallInteger(3)->unsigned()->notNull(),
					'label' => $this->stringType(50)->notNull(),
					'presence' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'actions' => $this->stringType(50)->notNull()->defaultValue(''),
					'favorites' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'creator_detail' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'relation_comment' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'view_type' => $this->stringType(100)->notNull()->defaultValue('RelatedTab'),
				],
				'columns_mysql' => [
					'sequence' => $this->tinyInteger(3)->unsigned()->notNull(),
					'presence' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'favorites' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'creator_detail' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'relation_comment' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'index' => [
					['tabid', 'tabid'],
					['related_tabid', 'related_tabid'],
					['tabid_2', ['tabid', 'related_tabid']],
					['tabid_3', ['tabid', 'related_tabid', 'label']],
					['tabid_4', ['tabid', 'related_tabid', 'presence']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_relatedlists_fields' => [
				'columns' => [
					'relation_id' => $this->integer(10),
					'fieldid' => $this->integer(10),
					'fieldname' => $this->stringType(30),
					'sequence' => $this->integer(10),
				],
				'index' => [
					['relation_id', 'relation_id'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_reminder_interval' => [
				'columns' => [
					'reminder_intervalid' => $this->primaryKey(10),
					'reminder_interval' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull(),
					'presence' => $this->integer(1)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_reminder_interval_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_reservations' => [
				'columns' => [
					'reservationsid' => $this->integer(10)->notNull()->defaultValue(0),
					'title' => $this->stringType(128),
					'reservations_no' => $this->stringType(),
					'reservations_status' => $this->stringType(128),
					'date_start' => $this->date()->notNull(),
					'time_start' => $this->stringType(50),
					'due_date' => $this->date(),
					'time_end' => $this->stringType(50),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'link' => $this->integer(10)->defaultValue(0),
					'process' => $this->integer(10)->defaultValue(0),
					'deleted' => $this->integer(1)->defaultValue(0),
					'type' => $this->stringType(128),
					'subprocess' => $this->integer(10)->defaultValue(0),
					'linkextend' => $this->integer(10),
				],
				'index' => [
					['process', 'process'],
					['link', 'link'],
					['subprocess', 'subprocess'],
					['linkextend', 'linkextend'],
					['deleted', 'deleted'],
				],
				'primaryKeys' => [
					['reservations_pk', 'reservationsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_reservations_status' => [
				'columns' => [
					'reservations_statusid' => $this->primaryKey(10),
					'reservations_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_reservations_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_reservationscf' => [
				'columns' => [
					'reservationsid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['reservationscf_pk', 'reservationsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_role' => [
				'columns' => [
					'roleid' => $this->stringType()->notNull(),
					'rolename' => $this->stringType(200),
					'parentrole' => $this->stringType(),
					'depth' => $this->smallInteger(5)->unsigned()->notNull()->defaultValue(0),
					'allowassignedrecordsto' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'changeowner' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'searchunpriv' => $this->text(),
					'clendarallorecords' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'listrelatedrecord' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'previewrelatedrecord' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'editrelatedrecord' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'permissionsrelatedfield' => $this->stringType(10)->notNull()->defaultValue(0),
					'globalsearchadv' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'assignedmultiowner' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'auto_assign' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'columns_mysql' => [
					'allowassignedrecordsto' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
					'changeowner' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
					'clendarallorecords' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
					'listrelatedrecord' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'previewrelatedrecord' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'editrelatedrecord' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'globalsearchadv' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
					'assignedmultiowner' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'auto_assign' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'index' => [
					['parentrole', 'parentrole'],
					['parentrole_2', ['parentrole', 'depth']],
				],
				'primaryKeys' => [
					['role_pk', 'roleid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_role2picklist' => [
				'columns' => [
					'roleid' => $this->stringType()->notNull(),
					'picklistvalueid' => $this->integer(10)->notNull(),
					'picklistid' => $this->integer(10)->notNull(),
					'sortid' => $this->integer(10),
				],
				'index' => [
					['role2picklist_roleid_picklistid_idx', ['roleid', 'picklistid', 'picklistvalueid']],
					['fk_2_vtiger_role2picklist', 'picklistid'],
				],
				'primaryKeys' => [
					['role2picklist_pk', ['roleid', 'picklistvalueid', 'picklistid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_role2profile' => [
				'columns' => [
					'roleid' => $this->stringType()->notNull(),
					'profileid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['role2profile_roleid_profileid_idx', ['roleid', 'profileid']],
					['roleid', 'roleid'],
					['profileid', 'profileid'],
				],
				'primaryKeys' => [
					['role2profile_pk', ['roleid', 'profileid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_role_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_rowheight' => [
				'columns' => [
					'rowheightid' => $this->primaryKey(10),
					'rowheight' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_rowheight_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_rss' => [
				'columns' => [
					'rssid' => $this->primaryKey(10),
					'rssurl' => $this->stringType(200)->notNull()->defaultValue(''),
					'rsstitle' => $this->stringType(200),
					'rsstype' => $this->integer(10)->defaultValue(0),
					'starred' => $this->integer(1)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_salesmanticketrel' => [
				'columns' => [
					'smid' => $this->integer(10)->notNull()->defaultValue(0),
					'id' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['salesmanticketrel_smid_idx', 'smid'],
					['salesmanticketrel_id_idx', 'id'],
				],
				'primaryKeys' => [
					['salesmanticketrel_pk', ['smid', 'id']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_salutationtype' => [
				'columns' => [
					'salutationtypeid' => $this->primaryKey(10),
					'salutationtype' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_salutationtype_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_scalculations_status' => [
				'columns' => [
					'scalculations_statusid' => $this->primaryKey(10),
					'scalculations_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_seattachmentsrel' => [
				'columns' => [
					'crmid' => $this->integer(10)->notNull()->defaultValue(0),
					'attachmentsid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['seattachmentsrel_attachmentsid_idx', 'attachmentsid'],
					['seattachmentsrel_crmid_idx', 'crmid'],
					['seattachmentsrel_attachmentsid_crmid_idx', ['attachmentsid', 'crmid']],
				],
				'primaryKeys' => [
					['seattachmentsrel_pk', ['crmid', 'attachmentsid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_senotesrel' => [
				'columns' => [
					'crmid' => $this->integer(10)->notNull()->defaultValue(0),
					'notesid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['senotesrel_notesid_idx', 'notesid'],
					['senotesrel_crmid_idx', 'crmid'],
				],
				'primaryKeys' => [
					['senotesrel_pk', ['crmid', 'notesid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_seproductsrel' => [
				'columns' => [
					'crmid' => $this->integer(10)->notNull()->defaultValue(0),
					'productid' => $this->integer(10)->notNull()->defaultValue(0),
					'setype' => $this->stringType(30)->notNull(),
					'rel_created_user' => $this->integer(10)->notNull(),
					'rel_created_time' => $this->dateTime()->notNull(),
					'rel_comment' => $this->stringType(),
				],
				'index' => [
					['seproductsrel_productid_idx', 'productid'],
					['seproductrel_crmid_idx', 'crmid'],
				],
				'primaryKeys' => [
					['seproductsrel_pk', ['crmid', 'productid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_service' => [
				'columns' => [
					'serviceid' => $this->integer(10)->notNull(),
					'service_no' => $this->stringType(100)->notNull(),
					'servicename' => $this->stringType()->notNull(),
					'pscategory' => $this->stringType(200),
					'qty_per_unit' => $this->decimal('11,2')->defaultValue(0),
					'unit_price' => $this->decimal('25,8'),
					'sales_start_date' => $this->date(),
					'sales_end_date' => $this->date(),
					'start_date' => $this->date(),
					'expiry_date' => $this->date(),
					'discontinued' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'service_usageunit' => $this->stringType(200),
					'website' => $this->stringType(100),
					'currency_id' => $this->integer(10)->notNull()->defaultValue(1),
					'commissionrate' => $this->decimal('7,3'),
					'renewable' => $this->smallInteger(1)->defaultValue(0),
					'taxes' => $this->stringType(50),
				],
				'columns_mysql' => [
					'discontinued' => $this->tinyInteger(1)->notNull()->defaultValue(0),
					'renewable' => $this->tinyInteger(1)->defaultValue(0),
				],
				'primaryKeys' => [
					['service_pk', 'serviceid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_service_usageunit' => [
				'columns' => [
					'service_usageunitid' => $this->primaryKey(10),
					'service_usageunit' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_service_usageunit_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_servicecf' => [
				'columns' => [
					'serviceid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['servicecf_pk', 'serviceid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_servicecontracts' => [
				'columns' => [
					'servicecontractsid' => $this->integer(10)->notNull(),
					'start_date' => $this->date(),
					'end_date' => $this->date(),
					'sc_related_to' => $this->integer(10),
					'tracking_unit' => $this->stringType(100),
					'total_units' => $this->decimal('5,2'),
					'used_units' => $this->decimal('5,2'),
					'subject' => $this->stringType(100),
					'due_date' => $this->date(),
					'planned_duration' => $this->stringType(256),
					'actual_duration' => $this->stringType(256),
					'contract_status' => $this->stringType(200),
					'priority' => $this->stringType(200),
					'contract_type' => $this->stringType(200),
					'progress' => $this->decimal('5,2'),
					'contract_no' => $this->stringType(100),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
				],
				'index' => [
					['sc_related_to', 'sc_related_to'],
				],
				'primaryKeys' => [
					['servicecontracts_pk', 'servicecontractsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_servicecontractscf' => [
				'columns' => [
					'servicecontractsid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['servicecontractscf_pk', 'servicecontractsid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_seticketsrel' => [
				'columns' => [
					'crmid' => $this->integer(10)->notNull()->defaultValue(0),
					'ticketid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['seticketsrel_crmid_idx', 'crmid'],
					['seticketsrel_ticketid_idx', 'ticketid'],
				],
				'primaryKeys' => [
					['seticketsrel_pk', ['crmid', 'ticketid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_settings_blocks' => [
				'columns' => [
					'blockid' => $this->integer(10)->notNull(),
					'label' => $this->stringType(250),
					'sequence' => $this->integer(10),
					'icon' => $this->stringType(),
					'type' => $this->smallInteger(1),
					'linkto' => $this->text(),
					'admin_access' => $this->text(),
				],
				'columns_mysql' => [
					'type' => $this->tinyInteger(1),
				],
				'primaryKeys' => [
					['settings_blocks_pk', 'blockid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_settings_blocks_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_settings_field' => [
				'columns' => [
					'fieldid' => $this->primaryKey(10),
					'blockid' => $this->integer(10),
					'name' => $this->stringType(250),
					'iconpath' => $this->stringType(300),
					'description' => $this->stringType(250),
					'linkto' => $this->text(),
					'sequence' => $this->integer(10),
					'active' => $this->integer(10)->defaultValue(0),
					'pinned' => $this->integer(1)->defaultValue(0),
					'admin_access' => $this->text(),
				],
				'index' => [
					['fk_1_vtiger_settings_field', 'blockid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_sharedcalendar' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'sharedid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['sharedcalendar_pk', ['userid', 'sharedid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_shorturls' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'uid' => $this->stringType(50),
					'handler_path' => $this->stringType(400),
					'handler_class' => $this->stringType(100),
					'handler_function' => $this->stringType(100),
					'handler_data' => $this->stringType(),
					'onetime' => $this->integer(5),
				],
				'index' => [
					['uid', 'uid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_smsnotifier' => [
				'columns' => [
					'smsnotifierid' => $this->integer(10)->notNull(),
					'message' => $this->text(),
					'smsnotifier_status' => $this->stringType(),
				],
				'primaryKeys' => [
					['smsnotifier_pk', 'smsnotifierid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_smsnotifier_status' => [
				'columns' => [
					'smsnotifier_statusid' => $this->primaryKey(10),
					'smsnotifier_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_smsnotifiercf' => [
				'columns' => [
					'smsnotifierid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['smsnotifiercf_pk', 'smsnotifierid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_soapservice' => [
				'columns' => [
					'id' => $this->integer(10),
					'type' => $this->stringType(25),
					'sessionid' => $this->stringType(100),
					'lang' => $this->stringType(10),
				],
				'index' => [
					['id', ['id', 'type']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_squoteenquiries_status' => [
				'columns' => [
					'squoteenquiries_statusid' => $this->primaryKey(10),
					'squoteenquiries_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_squotes_status' => [
				'columns' => [
					'squotes_statusid' => $this->primaryKey(10),
					'squotes_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_srecurringorders_status' => [
				'columns' => [
					'srecurringorders_statusid' => $this->primaryKey(10),
					'srecurringorders_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_srequirementscards_status' => [
				'columns' => [
					'srequirementscards_statusid' => $this->primaryKey(10),
					'srequirementscards_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ssalesprocesses_source' => [
				'columns' => [
					'ssalesprocesses_sourceid' => $this->primaryKey(10),
					'ssalesprocesses_source' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ssalesprocesses_status' => [
				'columns' => [
					'ssalesprocesses_statusid' => $this->primaryKey(10),
					'ssalesprocesses_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ssalesprocesses_type' => [
				'columns' => [
					'ssalesprocesses_typeid' => $this->primaryKey(10),
					'ssalesprocesses_type' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ssingleorders_source' => [
				'columns' => [
					'ssingleorders_sourceid' => $this->primaryKey(10),
					'ssingleorders_source' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ssingleorders_status' => [
				'columns' => [
					'ssingleorders_statusid' => $this->primaryKey(10),
					'ssingleorders_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ssservicesstatus' => [
				'columns' => [
					'ssservicesstatusid' => $this->primaryKey(10),
					'ssservicesstatus' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ssservicesstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_start_hour' => [
				'columns' => [
					'start_hourid' => $this->primaryKey(10),
					'start_hour' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_start_hour_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_state' => [
				'columns' => [
					'stateid' => $this->primaryKey(10),
					'state' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_state_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_status' => [
				'columns' => [
					'statusid' => $this->primaryKey(10),
					'status' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_status_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_storage_status' => [
				'columns' => [
					'storage_statusid' => $this->primaryKey(10),
					'storage_status' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_storage_type' => [
				'columns' => [
					'storage_typeid' => $this->primaryKey(10),
					'storage_type' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_subindustry' => [
				'columns' => [
					'subindustryid' => $this->primaryKey(10),
					'subindustry' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_subindustry_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_subunit' => [
				'columns' => [
					'subunitid' => $this->primaryKey(10),
					'subunit' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_support_processes' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
					'ticket_status_indicate_closing' => $this->stringType()->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_svendorenquiries_status' => [
				'columns' => [
					'svendorenquiries_statusid' => $this->primaryKey(10),
					'svendorenquiries_status' => $this->stringType(),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'picklist_valueid' => $this->smallInteger(5)->defaultValue(0),
					'sortorderid' => $this->smallInteger(5)->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_systems' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
					'server' => $this->stringType(100),
					'server_port' => $this->integer(10),
					'server_username' => $this->stringType(100),
					'server_password' => $this->stringType(100),
					'server_type' => $this->stringType(20),
					'smtp_auth' => $this->stringType(5),
					'server_path' => $this->stringType(256),
					'from_email_field' => $this->stringType(50),
				],
				'primaryKeys' => [
					['systems_pk', 'id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tab' => [
				'columns' => [
					'tabid' => $this->smallInteger(5)->notNull()->defaultValue(0),
					'name' => $this->stringType(25)->notNull(),
					'presence' => $this->smallInteger(3)->unsigned()->notNull()->defaultValue(1),
					'tabsequence' => $this->smallInteger(5)->notNull()->defaultValue(0),
					'tablabel' => $this->stringType(25)->notNull(),
					'modifiedby' => $this->smallInteger(5),
					'modifiedtime' => $this->integer(10),
					'customized' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'ownedby' => $this->smallInteger(1)->notNull()->defaultValue(0),
					'isentitytype' => $this->smallInteger(1)->notNull()->defaultValue(1),
					'version' => $this->stringType(10),
					'parent' => $this->stringType(30),
					'color' => $this->stringType(30),
					'coloractive' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'type' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(3)->unsigned()->notNull()->defaultValue(1),
					'customized' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'ownedby' => $this->tinyInteger(1)->notNull()->defaultValue(0),
					'isentitytype' => $this->tinyInteger(1)->notNull()->defaultValue(1),
					'coloractive' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'type' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
				],
				'index' => [
					['tab_name_idx', 'name', true],
					['tab_modifiedby_idx', 'modifiedby'],
					['tab_tabid_idx', 'tabid'],
					['name', ['name', 'presence']],
					['presence', 'presence'],
					['name_2', ['name', 'presence', 'type']],
				],
				'primaryKeys' => [
					['tab_pk', 'tabid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tab_info' => [
				'columns' => [
					'tabid' => $this->smallInteger(5),
					'prefname' => $this->stringType(256),
					'prefvalue' => $this->stringType(256),
				],
				'index' => [
					['fk_1_vtiger_tab_info', 'tabid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_taskpriority' => [
				'columns' => [
					'taskpriorityid' => $this->primaryKey(10),
					'taskpriority' => $this->stringType(200),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_taskpriority_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ticketcf' => [
				'columns' => [
					'ticketid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['ticketcf_pk', 'ticketid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ticketpriorities' => [
				'columns' => [
					'ticketpriorities_id' => $this->primaryKey(10),
					'ticketpriorities' => $this->stringType(200),
					'presence' => $this->integer(1)->notNull()->defaultValue(0),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
					'color' => $this->stringType(25)->defaultValue('	#E6FAD8'),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ticketpriorities_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ticketseverities' => [
				'columns' => [
					'ticketseverities_id' => $this->primaryKey(10),
					'ticketseverities' => $this->stringType(200),
					'presence' => $this->integer(1)->notNull()->defaultValue(0),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ticketseverities_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ticketstatus' => [
				'columns' => [
					'ticketstatus_id' => $this->primaryKey(10),
					'ticketstatus' => $this->stringType(200),
					'presence' => $this->integer(1)->notNull()->defaultValue(0),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
					'color' => $this->stringType(25)->defaultValue('#E6FAD8'),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ticketstatus_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_time_zone' => [
				'columns' => [
					'time_zoneid' => $this->primaryKey(10),
					'time_zone' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_time_zone_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_timecontrol_type' => [
				'columns' => [
					'timecontrol_typeid' => $this->primaryKey(10),
					'timecontrol_type' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10),
					'presence' => $this->integer(10)->notNull()->defaultValue(1),
					'color' => $this->stringType(25)->defaultValue('#E6FAD8'),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_timecontrol_type_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_read_group_rel_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'relatedtabid' => $this->integer(10)->notNull(),
					'sharedgroupid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_read_group_rel_sharing_per_userid_sharedgroupid_tabid', ['userid', 'sharedgroupid', 'tabid']],
				],
				'primaryKeys' => [
					['tmp_read_group_rel_sharing_per_pk', ['userid', 'tabid', 'relatedtabid', 'sharedgroupid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_read_group_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'sharedgroupid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_read_group_sharing_per_userid_sharedgroupid_idx', ['userid', 'sharedgroupid']],
				],
				'primaryKeys' => [
					['tmp_read_group_sharing_per_pk', ['userid', 'tabid', 'sharedgroupid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_read_user_rel_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'relatedtabid' => $this->integer(10)->notNull(),
					'shareduserid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_read_user_rel_sharing_per_userid_shared_reltabid_idx', ['userid', 'shareduserid', 'relatedtabid']],
				],
				'primaryKeys' => [
					['tmp_read_user_rel_sharing_per_pk', ['userid', 'tabid', 'relatedtabid', 'shareduserid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_read_user_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'shareduserid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_read_user_sharing_per_userid_shareduserid_idx', ['userid', 'shareduserid']],
				],
				'primaryKeys' => [
					['tmp_read_user_sharing_per_pk', ['userid', 'tabid', 'shareduserid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_write_group_rel_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'relatedtabid' => $this->integer(10)->notNull(),
					'sharedgroupid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_write_group_rel_sharing_per_userid_sharedgroupid_tabid_idx', ['userid', 'sharedgroupid', 'tabid']],
				],
				'primaryKeys' => [
					['tmp_write_group_rel_sharing_per_pk', ['userid', 'tabid', 'relatedtabid', 'sharedgroupid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_write_group_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'sharedgroupid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_write_group_sharing_per_UK1', ['userid', 'sharedgroupid']],
				],
				'primaryKeys' => [
					['tmp_write_group_sharing_per_pk', ['userid', 'tabid', 'sharedgroupid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_write_user_rel_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'relatedtabid' => $this->integer(10)->notNull(),
					'shareduserid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_write_user_rel_sharing_per_userid_sharduserid_tabid_idx', ['userid', 'shareduserid', 'tabid']],
				],
				'primaryKeys' => [
					['tmp_write_user_rel_sharing_per_pk', ['userid', 'tabid', 'relatedtabid', 'shareduserid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tmp_write_user_sharing_per' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'shareduserid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['tmp_write_user_sharing_per_userid_shareduserid_idx', ['userid', 'shareduserid']],
				],
				'primaryKeys' => [
					['tmp_write_user_sharing_per_pk', ['userid', 'tabid', 'shareduserid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tracking_unit' => [
				'columns' => [
					'tracking_unitid' => $this->primaryKey(10),
					'tracking_unit' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_tracking_unit_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_trees_templates' => [
				'columns' => [
					'templateid' => $this->primaryKey(10),
					'name' => $this->stringType(),
					'module' => $this->integer(10),
					'access' => $this->integer(1)->defaultValue(1),
					'share' => $this->stringType(),
				],
				'index' => [
					['module', 'module'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_trees_templates_data' => [
				'columns' => [
					'templateid' => $this->smallInteger(5)->unsigned()->notNull(),
					'name' => $this->stringType()->notNull(),
					'tree' => $this->stringType()->notNull(),
					'parentTree' => $this->stringType()->notNull(),
					'depth' => $this->smallInteger(3)->unsigned()->notNull(),
					'label' => $this->stringType()->notNull(),
					'state' => $this->stringType(100)->notNull()->defaultValue(''),
					'icon' => $this->stringType()->notNull()->defaultValue(''),
				],
				'columns_mysql' => [
					'depth' => $this->tinyInteger(3)->unsigned()->notNull(),
				],
				'index' => [
					['id', 'templateid'],
					['parentTree', ['parentTree', 'templateid']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_troubletickets' => [
				'columns' => [
					'ticketid' => $this->integer(10)->notNull(),
					'ticket_no' => $this->stringType(100)->notNull(),
					'groupname' => $this->stringType(100),
					'parent_id' => $this->integer(10),
					'product_id' => $this->integer(10),
					'priority' => $this->stringType(200),
					'severity' => $this->stringType(200),
					'status' => $this->stringType(200),
					'category' => $this->stringType(200),
					'title' => $this->stringType()->notNull(),
					'solution' => $this->text(),
					'update_log' => $this->text(),
					'version_id' => $this->integer(10),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'servicecontractsid' => $this->integer(10),
					'attention' => $this->text(),
					'pssold_id' => $this->integer(10),
					'ordertime' => $this->decimal('10,2'),
					'from_portal' => $this->smallInteger(1),
					'contract_type' => $this->stringType(),
					'contracts_end_date' => $this->date(),
					'report_time' => $this->integer(10),
					'response_time' => $this->dateTime(),
				],
				'index' => [
					['troubletickets_ticketid_idx', 'ticketid'],
					['troubletickets_status_idx', 'status'],
					['parent_id', 'parent_id'],
					['product_id', 'product_id'],
					['servicecontractsid', 'servicecontractsid'],
					['pssold_id', 'pssold_id'],
					['ticket_no', 'ticket_no'],
				],
				'primaryKeys' => [
					['troubletickets_pk', 'ticketid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_usageunit' => [
				'columns' => [
					'usageunitid' => $this->primaryKey(10),
					'usageunit' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10),
				],
				'index' => [
					['usageunit_usageunit_idx', 'usageunit', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_usageunit_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_user2mergefields' => [
				'columns' => [
					'userid' => $this->integer(10),
					'tabid' => $this->smallInteger(5),
					'fieldid' => $this->integer(10),
					'visible' => $this->integer(2),
				],
				'index' => [
					['userid_tabid_idx', ['userid', 'tabid']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_user2role' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'roleid' => $this->stringType()->notNull(),
				],
				'index' => [
					['user2role_roleid_idx', 'roleid'],
				],
				'primaryKeys' => [
					['user2role_pk', 'userid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_user_module_preferences' => [
				'columns' => [
					'userid' => $this->stringType(30)->notNull(),
					'tabid' => $this->smallInteger(5)->notNull(),
					'default_cvid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['fk_2_vtiger_user_module_preferences', 'tabid'],
				],
				'primaryKeys' => [
					['user_module_preferences_pk', ['userid', 'tabid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_users' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'user_name' => $this->stringType(32),
					'first_name' => $this->stringType(30),
					'last_name' => $this->stringType(30),
					'email1' => $this->stringType(100),
					'is_admin' => $this->stringType(3)->defaultValue(0),
					'status' => $this->stringType(25),
					'deleted' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(0),
					'language' => $this->stringType(36),
					'user_password' => $this->stringType(200),
					'internal_mailer' => $this->smallInteger(1)->unsigned()->notNull()->defaultValue(1),
					'reports_to_id' => $this->integer(10)->unsigned(),
					'modified_user_id' => $this->stringType(36),
					'currency_id' => $this->integer(10)->notNull()->defaultValue(1),
					'description' => $this->text(),
					'date_entered' => $this->timestamp()->null(),
					'date_modified' => $this->timestamp()->null(),
					'date_password_change' => $this->dateTime(),
					'force_password_change' => $this->smallInteger(1)->defaultValue(0),
					'holidays' => $this->stringType(60),
					'namedays' => $this->stringType(60),
					'workdays' => $this->stringType(30),
					'weekstart' => $this->integer(10),
					'date_format' => $this->stringType(200),
					'hour_format' => $this->stringType(30)->defaultValue('am/pm'),
					'start_hour' => $this->stringType(30)->defaultValue('10:00'),
					'end_hour' => $this->stringType(30)->defaultValue('23:00'),
					'view_date_format' => $this->stringType(50)->defaultValue('PLL_ELAPSED'),
					'activity_view' => $this->stringType(200)->defaultValue('Today'),
					'lead_view' => $this->stringType(200)->defaultValue('Today'),
					'imagename' => $this->text(),
					'reminder_interval' => $this->stringType(100),
					'reminder_next_time' => $this->stringType(100),
					'theme' => $this->stringType(100),
					'tz' => $this->stringType(30),
					'time_zone' => $this->stringType(200),
					'currency_grouping_pattern' => $this->stringType(100),
					'currency_decimal_separator' => $this->stringType(2),
					'currency_grouping_separator' => $this->stringType(2),
					'currency_symbol_placement' => $this->stringType(20),
					'no_of_currency_decimals' => $this->stringType(1),
					'truncate_trailing_zeros' => $this->smallInteger(1)->unsigned(),
					'dayoftheweek' => $this->stringType(100),
					'callduration' => $this->stringType(3),
					'othereventduration' => $this->stringType(3),
					'default_record_view' => $this->stringType(10),
					'leftpanelhide' => $this->smallInteger(3)->unsigned(),
					'rowheight' => $this->stringType(10),
					'defaulteventstatus' => $this->stringType(50),
					'defaultactivitytype' => $this->stringType(50),
					'is_owner' => $this->stringType(5),
					'emailoptout' => $this->smallInteger(3)->unsigned()->notNull()->defaultValue(1),
					'available' => $this->smallInteger(1)->defaultValue(0),
					'auto_assign' => $this->smallInteger(1)->defaultValue(0),
					'records_limit' => $this->integer(10),
					'phone_crm_extension' => $this->stringType(100),
					'phone_crm_extension_extra' => $this->stringType(100),
					'accesskey' => $this->stringType(36),
					'confirm_password' => $this->stringType(200),
					'cal_color' => $this->stringType(25),
					'user_preferences' => $this->text(),
					'authy_methods' => $this->stringType(),
					'authy_secret_totp' => $this->stringType(),
					'login_method' => $this->stringType(50)->defaultValue('PLL_PASSWORD'),
				],
				'columns_mysql' => [
					'deleted' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(0),
					'internal_mailer' => $this->tinyInteger(1)->unsigned()->notNull()->defaultValue(1),
					'force_password_change' => $this->tinyInteger(1)->defaultValue(0),
					'truncate_trailing_zeros' => $this->tinyInteger(1)->unsigned(),
					'leftpanelhide' => $this->tinyInteger(3)->unsigned(),
					'emailoptout' => $this->tinyInteger(3)->unsigned()->notNull()->defaultValue(1),
					'available' => $this->tinyInteger(1)->defaultValue(0),
					'auto_assign' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['email1', 'email1', true],
					['user_user_name_idx', 'user_name'],
					['user_user_password_idx', 'user_password'],
					['status', 'status'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_users2group' => [
				'columns' => [
					'groupid' => $this->integer(10)->notNull(),
					'userid' => $this->integer(10)->notNull(),
				],
				'index' => [
					['users2group_groupname_uerid_idx', ['groupid', 'userid']],
					['fk_2_vtiger_users2group', 'userid'],
				],
				'primaryKeys' => [
					['users2group_pk', ['groupid', 'userid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_users_last_import' => [
				'columns' => [
					'id' => $this->primaryKey(36),
					'assigned_user_id' => $this->stringType(36),
					'bean_type' => $this->stringType(36),
					'bean_id' => $this->integer(10),
					'deleted' => $this->integer(1)->notNull()->defaultValue(0),
				],
				'index' => [
					['idx_user_id', 'assigned_user_id'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_users_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_userscf' => [
				'columns' => [
					'usersid' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['userscf_pk', 'usersid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_vendor' => [
				'columns' => [
					'vendorid' => $this->integer(10)->notNull()->defaultValue(0),
					'vendor_no' => $this->stringType(100)->notNull(),
					'vendorname' => $this->stringType(),
					'phone' => $this->stringType(100),
					'email' => $this->stringType(100),
					'website' => $this->stringType(100),
					'glacct' => $this->stringType(200),
					'category' => $this->stringType(50),
					'description' => $this->text(),
					'vat_id' => $this->stringType(30),
					'registration_number_1' => $this->stringType(30),
					'registration_number_2' => $this->stringType(30),
					'verification' => $this->text(),
					'sum_time' => $this->decimal('10,2')->defaultValue(0),
					'active' => $this->smallInteger(1)->defaultValue(0),
					'phone_extra' => $this->stringType(100),
				],
				'columns_mysql' => [
					'active' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['vendorname', 'vendorname'],
				],
				'primaryKeys' => [
					['vendor_pk', 'vendorid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_vendoraddress' => [
				'columns' => [
					'vendorid' => $this->integer(10)->notNull(),
					'addresslevel1a' => $this->stringType(),
					'addresslevel1b' => $this->stringType(),
					'addresslevel1c' => $this->stringType(),
					'addresslevel2a' => $this->stringType(),
					'addresslevel2b' => $this->stringType(),
					'addresslevel2c' => $this->stringType(),
					'addresslevel3a' => $this->stringType(),
					'addresslevel3b' => $this->stringType(),
					'addresslevel3c' => $this->stringType(),
					'addresslevel4a' => $this->stringType(),
					'addresslevel4b' => $this->stringType(),
					'addresslevel4c' => $this->stringType(),
					'addresslevel5a' => $this->stringType(),
					'addresslevel5b' => $this->stringType(),
					'addresslevel5c' => $this->stringType(),
					'addresslevel6a' => $this->stringType(),
					'addresslevel6b' => $this->stringType(),
					'addresslevel6c' => $this->stringType(),
					'addresslevel7a' => $this->stringType(),
					'addresslevel7b' => $this->stringType(),
					'addresslevel7c' => $this->stringType(),
					'addresslevel8a' => $this->stringType(),
					'addresslevel8b' => $this->stringType(),
					'addresslevel8c' => $this->stringType(),
					'poboxa' => $this->stringType(50),
					'poboxb' => $this->stringType(50),
					'poboxc' => $this->stringType(50),
					'buildingnumbera' => $this->stringType(100),
					'buildingnumberb' => $this->stringType(100),
					'buildingnumberc' => $this->stringType(100),
					'localnumbera' => $this->stringType(100),
					'localnumberb' => $this->stringType(100),
					'localnumberc' => $this->stringType(100),
				],
				'primaryKeys' => [
					['vendoraddress_pk', 'vendorid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_vendorcf' => [
				'columns' => [
					'vendorid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'primaryKeys' => [
					['vendorcf_pk', 'vendorid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_vendorcontactrel' => [
				'columns' => [
					'vendorid' => $this->integer(10)->notNull()->defaultValue(0),
					'contactid' => $this->integer(10)->notNull()->defaultValue(0),
				],
				'index' => [
					['vendorcontactrel_vendorid_idx', 'vendorid'],
					['vendorcontactrel_contact_idx', 'contactid'],
				],
				'primaryKeys' => [
					['vendorcontactrel_pk', ['vendorid', 'contactid']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_verification' => [
				'columns' => [
					'verificationid' => $this->primaryKey(10),
					'verification' => $this->stringType(200)->notNull(),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
					'picklist_valueid' => $this->integer(10)->notNull()->defaultValue(0),
					'sortorderid' => $this->integer(10)->defaultValue(0),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_verification_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_version' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'old_version' => $this->stringType(30),
					'current_version' => $this->stringType(30),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_version_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_view_date_format' => [
				'columns' => [
					'view_date_formatid' => $this->primaryKey(),
					'view_date_format' => $this->stringType(50),
					'presence' => $this->smallInteger(1)->defaultValue(1),
					'sortorderid' => $this->smallInteger()->defaultValue(0),
				],
				'columns_mysql' => [
					'presence' => $this->tinyInteger(1)->defaultValue(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_visibility' => [
				'columns' => [
					'visibilityid' => $this->primaryKey(10),
					'visibility' => $this->stringType(200)->notNull(),
					'sortorderid' => $this->integer(10)->notNull()->defaultValue(0),
					'presence' => $this->integer(1)->notNull()->defaultValue(1),
				],
				'index' => [
					['visibility_visibility_idx', 'visibility', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_visibility_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_widgets' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'tabid' => $this->smallInteger(5),
					'type' => $this->stringType(30),
					'label' => $this->stringType(100),
					'wcol' => $this->smallInteger(1)->defaultValue(1),
					'sequence' => $this->smallInteger(2),
					'data' => $this->text(),
				],
				'columns_mysql' => [
					'wcol' => $this->tinyInteger(1)->defaultValue(1),
					'sequence' => $this->tinyInteger(2),
				],
				'index' => [
					['tabid', 'tabid'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_entity' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'name' => $this->stringType(25)->notNull(),
					'handler_path' => $this->stringType()->notNull(),
					'handler_class' => $this->stringType(64)->notNull(),
					'ismodule' => $this->integer(3)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_entity_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_fieldinfo' => [
				'columns' => [
					'id' => $this->stringType(64)->notNull(),
					'property_name' => $this->stringType(32),
					'property_value' => $this->stringType(64),
				],
				'primaryKeys' => [
					['ws_fieldinfo_pk', 'id']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_fieldtype' => [
				'columns' => [
					'fieldtypeid' => $this->primaryKey(10),
					'uitype' => $this->smallInteger(3)->notNull(),
					'fieldtype' => $this->stringType(200)->notNull(),
				],
				'index' => [
					['uitype_idx', 'uitype', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_operation' => [
				'columns' => [
					'operationid' => $this->primaryKey(10),
					'name' => $this->stringType(128)->notNull(),
					'handler_path' => $this->stringType()->notNull(),
					'handler_method' => $this->stringType(64)->notNull(),
					'type' => $this->stringType(8)->notNull(),
					'prelogin' => $this->integer(3)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_operation_parameters' => [
				'columns' => [
					'operationid' => $this->integer(10)->notNull(),
					'name' => $this->stringType(128)->notNull(),
					'type' => $this->stringType(64)->notNull(),
					'sequence' => $this->integer(10)->notNull(),
				],
				'primaryKeys' => [
					['ws_operation_parameters_pk', ['operationid', 'name']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_operation_seq' => [
				'columns' => [
					'id' => $this->integer(10)->notNull(),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_referencetype' => [
				'columns' => [
					'fieldtypeid' => $this->integer(10)->notNull(),
					'type' => $this->stringType(25)->notNull(),
				],
				'index' => [
					['fieldtypeid', 'fieldtypeid'],
				],
				'primaryKeys' => [
					['ws_referencetype_pk', ['fieldtypeid', 'type']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'vtiger_ws_userauthtoken' => [
				'columns' => [
					'userid' => $this->integer(10)->notNull(),
					'token' => $this->stringType(36)->notNull(),
					'expiretime' => $this->integer(10)->notNull(),
				],
				'index' => [
					['userid_idx', 'userid', true],
				],
				'primaryKeys' => [
					['ws_userauthtoken_pk', ['userid', 'expiretime']]
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_auth' => [
				'columns' => [
					'id' => $this->primaryKey(3)->unsigned(),
					'type' => $this->stringType(20),
					'param' => $this->stringType(20),
					'value' => $this->text(),
				],
				'index' => [
					['type', ['type', 'param'], true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_currencyupdate' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'currency_id' => $this->integer(10)->notNull(),
					'fetch_date' => $this->date()->notNull(),
					'exchange_date' => $this->date()->notNull(),
					'exchange' => $this->decimal('10,4')->notNull(),
					'bank_id' => $this->integer(10)->notNull(),
				],
				'index' => [
					['fetchdate_currencyid_unique', ['currency_id', 'exchange_date', 'bank_id'], true],
					['fk_1_vtiger_osscurrencies', 'currency_id'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_currencyupdate_banks' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'bank_name' => $this->stringType()->notNull(),
					'active' => $this->integer(1)->notNull(),
				],
				'index' => [
					['unique_bankname', 'bank_name', true],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_mail_config' => [
				'columns' => [
					'type' => $this->stringType(50),
					'name' => $this->stringType(50),
					'value' => $this->text(),
				],
				'index' => [
					['type', ['type', 'name'], true],
					['type_2', 'type'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_mail_quantities' => [
				'columns' => [
					'userid' => $this->integer(10)->unsigned()->notNull(),
					'num' => $this->integer(10)->unsigned()->defaultValue(0),
					'status' => $this->smallInteger(1)->defaultValue(0),
				],
				'columns_mysql' => [
					'status' => $this->tinyInteger(1)->defaultValue(0),
				],
				'primaryKeys' => [
					['yetiforce_mail_quantities_pk', 'userid']
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_menu' => [
				'columns' => [
					'id' => $this->primaryKey(10)->unsigned(),
					'role' => $this->integer(10),
					'parentid' => $this->integer(10)->defaultValue(0),
					'type' => $this->smallInteger(1),
					'sequence' => $this->integer(3),
					'module' => $this->smallInteger(5),
					'label' => $this->stringType(100),
					'newwindow' => $this->smallInteger(1)->defaultValue(0),
					'dataurl' => $this->text(),
					'showicon' => $this->smallInteger(1)->defaultValue(0),
					'icon' => $this->stringType(),
					'sizeicon' => $this->stringType(),
					'hotkey' => $this->stringType(30),
					'filters' => $this->stringType(),
				],
				'columns_mysql' => [
					'type' => $this->tinyInteger(1),
					'newwindow' => $this->tinyInteger(1)->defaultValue(0),
					'showicon' => $this->tinyInteger(1)->defaultValue(0),
				],
				'index' => [
					['parent_id', 'parentid'],
					['role', 'role'],
					['module', 'module'],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_proc_marketing' => [
				'columns' => [
					'type' => $this->stringType(30),
					'param' => $this->stringType(30),
					'value' => $this->stringType(200),
				],
				'index' => [
					['type', ['type', 'param']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_proc_sales' => [
				'columns' => [
					'type' => $this->stringType(30),
					'param' => $this->stringType(30),
					'value' => $this->stringType(200),
				],
				'index' => [
					['type', ['type', 'param']],
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_proc_tc' => [
				'columns' => [
					'type' => $this->stringType(30),
					'param' => $this->stringType(30),
					'value' => $this->stringType(200),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
			'yetiforce_updates' => [
				'columns' => [
					'id' => $this->primaryKey(10),
					'time' => $this->timestamp()->null(),
					'user' => $this->stringType(50),
					'name' => $this->stringType(100),
					'from_version' => $this->stringType(10),
					'to_version' => $this->stringType(10),
					'result' => $this->smallInteger(1),
				],
				'columns_mysql' => [
					'result' => $this->tinyInteger(1),
				],
				'engine' => 'InnoDB',
				'charset' => 'utf8'
			],
		];
		$this->foreignKey = [
			['vtiger_reservations', 'vtiger_reservations', 'reservationsid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_reservationscf', 'vtiger_reservationscf', 'reservationsid', 'vtiger_reservations', 'reservationsid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_role2picklist', 'vtiger_role2picklist', 'roleid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_role2picklist', 'vtiger_role2picklist', 'picklistid', 'vtiger_picklist', 'picklistid', 'CASCADE', 'RESTRICT'],
			['vtiger_role2profile_ibfk_1', 'vtiger_role2profile', 'roleid', 'vtiger_role', 'roleid', 'CASCADE', 'RESTRICT'],
			['vtiger_role2profile_ibfk_2', 'vtiger_role2profile', 'profileid', 'vtiger_profile', 'profileid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_salesmanticketrel', 'vtiger_salesmanticketrel', 'smid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_senotesrel', 'vtiger_senotesrel', 'notesid', 'vtiger_notes', 'notesid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_seproductsrel', 'vtiger_seproductsrel', 'productid', 'vtiger_products', 'productid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_service', 'vtiger_service', 'serviceid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_servicecf_ibfk_1', 'vtiger_servicecf', 'serviceid', 'vtiger_service', 'serviceid', 'CASCADE', 'RESTRICT'],
			['vtiger_servicecontracts_ibfk_1', 'vtiger_servicecontracts', 'servicecontractsid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_servicecontractscf_ibfk_1', 'vtiger_servicecontractscf', 'servicecontractsid', 'vtiger_servicecontracts', 'servicecontractsid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_seticketsrel', 'vtiger_seticketsrel', 'ticketid', 'vtiger_troubletickets', 'ticketid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_settings_field', 'vtiger_settings_field', 'blockid', 'vtiger_settings_blocks', 'blockid', 'CASCADE', 'RESTRICT'],
			['vtiger_smsnotifier_ibfk_1', 'vtiger_smsnotifier', 'smsnotifierid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_smsnotifiercf_ibfk_1', 'vtiger_smsnotifiercf', 'smsnotifierid', 'vtiger_smsnotifier', 'smsnotifierid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_tab_info', 'vtiger_tab_info', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'CASCADE'],
			['fk_1_vtiger_ticketcf', 'vtiger_ticketcf', 'ticketid', 'vtiger_troubletickets', 'ticketid', 'CASCADE', 'RESTRICT'],
			['fk_4_vtiger_tmp_read_group_rel_sharing_per', 'vtiger_tmp_read_group_rel_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_tmp_read_group_sharing_per', 'vtiger_tmp_read_group_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_4_vtiger_tmp_read_user_rel_sharing_per', 'vtiger_tmp_read_user_rel_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_tmp_read_user_sharing_per', 'vtiger_tmp_read_user_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_4_vtiger_tmp_write_group_rel_sharing_per', 'vtiger_tmp_write_group_rel_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_tmp_write_group_sharing_per', 'vtiger_tmp_write_group_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_4_vtiger_tmp_write_user_rel_sharing_per', 'vtiger_tmp_write_user_rel_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_3_vtiger_tmp_write_user_sharing_per', 'vtiger_tmp_write_user_sharing_per', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_troubletickets', 'vtiger_troubletickets', 'ticketid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_user2role', 'vtiger_user2role', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_user_module_preferences', 'vtiger_user_module_preferences', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'CASCADE'],
			['fk_2_vtiger_users2group', 'vtiger_users2group', 'userid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['vtiger_userscf_ibfk_1', 'vtiger_userscf', 'usersid', 'vtiger_users', 'id', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_vendor', 'vtiger_vendor', 'vendorid', 'vtiger_crmentity', 'crmid', 'CASCADE', 'RESTRICT'],
			['vtiger_vendoraddress_ibfk_1', 'vtiger_vendoraddress', 'vendorid', 'vtiger_vendor', 'vendorid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_vendorcf', 'vtiger_vendorcf', 'vendorid', 'vtiger_vendor', 'vendorid', 'CASCADE', 'RESTRICT'],
			['fk_2_vtiger_vendorcontactrel', 'vtiger_vendorcontactrel', 'vendorid', 'vtiger_vendor', 'vendorid', 'CASCADE', 'RESTRICT'],
			['vtiger_widgets_ibfk_1', 'vtiger_widgets', 'tabid', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_referencetype', 'vtiger_ws_referencetype', 'fieldtypeid', 'vtiger_ws_fieldtype', 'fieldtypeid', 'CASCADE', 'RESTRICT'],
			['fk_1_vtiger_osscurrencies', 'yetiforce_currencyupdate', 'currency_id', 'vtiger_currency_info', 'id', 'CASCADE', 'RESTRICT'],
			['yetiforce_mail_quantities_ibfk_1', 'yetiforce_mail_quantities', 'userid', 'roundcube_users', 'user_id', 'CASCADE', 'RESTRICT'],
			['yetiforce_menu_ibfk_1', 'yetiforce_menu', 'module', 'vtiger_tab', 'tabid', 'CASCADE', 'RESTRICT'],
		];
	}

	public function data()
	{
		$this->data = [
			'vtiger_realization_process' => [
				'columns' => ['module_id', 'status_indicate_closing'],
				'values' => [
					[43, ''],
				]
			],
			'vtiger_recurring_frequency' => [
				'columns' => ['recurring_frequency_id', 'recurring_frequency', 'sortorderid', 'presence'],
				'values' => [
					[2, '+1 day', 1, 1],
					[3, '+7 days', 2, 1],
					[4, '+14 days', 3, 1],
					[5, '+21 days', 4, 1],
					[6, '+30 days', 5, 1],
					[7, '+60 days', 6, 1],
					[8, '+90 days', 7, 1],
					[9, '+180 days', 8, 1],
					[10, '+360 days', 9, 1],
					[11, '+1 month', 10, 1],
					[12, '+3 months', 11, 1],
					[13, '+6 months', 12, 1],
					[14, '+1 year', 13, 1],
					[15, 'monday next week', 14, 1],
					[16, 'friday next week', 15, 1],
					[17, 'first day of next month', 16, 1],
					[18, 'last day of next month', 17, 1],
					[19, 'first day of +3 months', 18, 1],
					[20, 'last day of +3 months', 19, 1],
				]
			],
			'vtiger_recurring_frequency_seq' => [
				'columns' => ['id'],
				'values' => [
					[20],
				]
			],
			'vtiger_rel_mod' => [
				'columns' => ['rel_modid', 'rel_mod', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Accounts', 1, 393, 1],
					[2, 'Contacts', 1, 394, 2],
					[3, 'Leads', 1, 395, 3],
					[4, 'HelpDesk', 1, 396, 4],
					[5, 'Potentials', 1, 397, 5],
					[6, 'Project', 1, 398, 6],
					[7, 'ServiceContracts', 1, 399, 7],
					[8, 'Campaigns', 1, 400, 8],
				]
			],
			'vtiger_rel_mod_seq' => [
				'columns' => ['id'],
				'values' => [
					[8],
				]
			],
			'vtiger_relatedlists' => [
				'columns' => ['relation_id', 'tabid', 'related_tabid', 'name', 'sequence', 'label', 'presence', 'actions', 'favorites', 'creator_detail', 'relation_comment'],
				'values' => [
					[1, 6, 4, 'getDependentsList', 1, 'Contacts', 0, 'add', 0, 0, 0],
					[6, 6, 9, 'getActivities', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[7, 6, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[8, 86, 51, 'getDependentsList', 18, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[9, 6, 8, 'getAttachments', 4, 'Documents', 0, 'add,select', 0, 0, 0],
					[10, 6, 13, 'getDependentsList', 5, 'HelpDesk', 0, 'add', 0, 0, 0],
					[11, 6, 14, 'getProducts', 12, 'Products', 1, 'select', 0, 1, 1],
					[12, 7, 9, 'getActivities', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[13, 7, 54, 'getEmails', 5, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[15, 7, 8, 'getAttachments', 4, 'Documents', 0, 'add,select', 0, 0, 0],
					[16, 7, 14, 'getProducts', 9, 'Products', 1, 'select', 0, 1, 1],
					[17, 7, 26, 'getCampaigns', 7, 'Campaigns', 0, 'select', 0, 0, 0],
					[19, 4, 9, 'getActivities', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[20, 4, 54, 'getEmails', 9, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[27, 4, 8, 'getAttachments', 8, 'Documents', 0, 'add,select', 0, 0, 0],
					[38, 14, 13, 'getDependentsList', 1, 'HelpDesk', 0, 'add', 0, 0, 0],
					[39, 14, 8, 'getAttachments', 3, 'Documents', 0, 'add,select', 0, 0, 0],
					[44, 14, 19, 'getProductPricebooks', 8, 'PriceBooks', 0, 'ADD,SELECT', 0, 0, 0],
					[45, 14, 7, 'getLeads', 9, 'Leads', 0, 'select', 0, 0, 0],
					[46, 14, 6, 'getAccounts', 10, 'Accounts', 0, 'select', 0, 0, 0],
					[49, 14, 14, 'getProducts', 13, 'Product Bundles', 0, 'add,select', 0, 0, 0],
					[54, 13, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[55, 13, 8, 'getAttachments', 4, 'Documents', 0, 'add,select', 0, 0, 0],
					[58, 19, 14, 'getPricebookProducts', 2, 'Products', 0, 'select', 0, 0, 0],
					[59, 18, 14, 'getProducts', 2, 'Products', 0, 'add,select', 0, 0, 0],
					[61, 18, 4, 'getDependentsList', 3, 'Contacts', 0, 'ADD', 0, 0, 0],
					[62, 18, 54, 'getEmails', 4, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[82, 9, 4, 'getContacts', 2, 'Contacts', 0, '', 0, 0, 0],
					[84, 26, 7, 'getCampaignsRecords', 4, 'Leads', 0, 'ADD,SELECT', 0, 0, 0],
					[86, 26, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[87, 6, 26, 'getCampaigns', 6, 'Campaigns', 0, 'select', 0, 0, 0],
					[88, 26, 6, 'getCampaignsRecords', 5, 'Accounts', 0, 'ADD,SELECT', 0, 0, 0],
					[89, 15, 8, 'getAttachments', 1, 'Documents', 0, 'add,select', 0, 0, 0],
					[95, 6, 34, 'getDependentsList', 8, 'Service Contracts', 0, 'ADD', 0, 0, 0],
					[109, 13, 35, 'getRelatedList', 6, 'Services', 1, 'SELECT', 0, 0, 0],
					[110, 7, 35, 'getRelatedList', 17, 'Services', 1, 'SELECT', 0, 0, 0],
					[111, 6, 35, 'getRelatedList', 15, 'Services', 1, 'SELECT', 0, 1, 1],
					[114, 19, 35, 'getPricebookServices', 3, 'Services', 0, 'SELECT', 0, 0, 0],
					[117, 6, 37, 'getDependentsList', 13, 'Assets', 1, 'ADD', 0, 0, 1],
					[118, 14, 37, 'getDependentsList', 15, 'Assets', 0, 'ADD', 0, 0, 0],
					[126, 6, 43, 'getDependentsList', 9, 'Projects', 0, 'add', 0, 0, 0],
					[157, 34, 13, 'getDependentsList', 4, 'HelpDesk', 0, 'ADD', 0, 0, 0],
					[158, 34, 8, 'getAttachments', 3, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[159, 35, 13, 'getRelatedList', 1, 'HelpDesk', 0, 'ADD,SELECT', 0, 0, 0],
					[164, 35, 19, 'getServicePricebooks', 6, 'PriceBooks', 0, 'ADD,SELECT', 0, 0, 0],
					[165, 35, 7, 'getRelatedList', 7, 'Leads', 0, 'SELECT', 0, 0, 0],
					[166, 35, 6, 'getRelatedList', 8, 'Accounts', 0, 'SELECT', 0, 0, 0],
					[169, 35, 8, 'getAttachments', 11, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[170, 37, 13, 'getRelatedList', 1, 'HelpDesk', 0, 'ADD,SELECT', 0, 0, 0],
					[171, 37, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[172, 42, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[173, 43, 42, 'getDependentsList', 2, 'Project Tasks', 0, 'ADD', 0, 0, 0],
					[174, 43, 41, 'getDependentsList', 3, 'Project Milestones', 0, 'ADD', 0, 0, 0],
					[175, 43, 8, 'getAttachments', 4, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[178, 45, 6, 'getRelatedList', 1, 'Accounts', 0, ' ', 0, 0, 0],
					[179, 45, 4, 'getRelatedList', 2, 'Contacts', 0, ' ', 0, 0, 0],
					[180, 45, 7, 'getRelatedList', 3, 'Leads', 0, ' ', 0, 0, 0],
					[182, 51, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[183, 6, 51, 'getDependentsList', 10, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[185, 13, 51, 'getDependentsList', 8, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[186, 43, 51, 'getDependentsList', 6, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[187, 42, 51, 'getDependentsList', 2, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[188, 34, 51, 'getDependentsList', 5, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[193, 54, 8, 'getAttachments', 1, 'Documents', 0, '', 0, 0, 0],
					[194, 54, 6, 'getRecordToMails', 2, 'Accounts', 0, 'ADD,SELECT', 0, 0, 0],
					[195, 54, 4, 'getRecordToMails', 3, 'Contacts', 0, 'ADD,SELECT', 0, 0, 0],
					[196, 54, 7, 'getRecordToMails', 4, 'Leads', 0, 'ADD,SELECT', 0, 0, 0],
					[198, 54, 13, 'getRecordToMails', 6, 'HelpDesk', 0, 'ADD,SELECT', 0, 0, 0],
					[199, 54, 43, 'getRecordToMails', 7, 'Project', 0, 'ADD,SELECT', 0, 0, 0],
					[200, 54, 34, 'getRecordToMails', 8, 'ServiceContracts', 0, 'ADD,SELECT', 0, 0, 0],
					[201, 54, 26, 'getRecordToMails', 9, 'Campaigns', 0, 'ADD,SELECT', 0, 0, 0],
					[207, 34, 54, 'getEmails', 6, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[208, 13, 54, 'getEmails', 10, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[210, 43, 54, 'getEmails', 7, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[211, 26, 54, 'getEmails', 2, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[213, 13, 37, 'getRelatedList', 11, 'Assets', 1, 'ADD,SELECT', 0, 0, 0],
					[214, 6, 57, 'getDependentsList', 16, 'OSSOutsourcedServices', 1, 'ADD', 0, 0, 1],
					[216, 7, 57, 'getDependentsList', 18, 'OSSOutsourcedServices', 1, 'ADD', 0, 0, 1],
					[218, 6, 58, 'getDependentsList', 17, 'OSSSoldServices', 1, 'ADD', 0, 0, 1],
					[221, 6, 59, 'getDependentsList', 14, 'OutsourcedProducts', 1, 'ADD', 0, 0, 1],
					[222, 7, 59, 'getDependentsList', 16, 'OutsourcedProducts', 1, 'ADD', 0, 0, 1],
					[230, 6, 60, 'getDependentsList', 11, 'OSSPasswords', 0, 'ADD', 0, 0, 0],
					[233, 7, 60, 'getDependentsList', 16, 'OSSPasswords', 0, 'ADD', 0, 0, 0],
					[234, 92, 60, 'getDependentsList', 12, 'OSSPasswords', 0, 'ADD', 0, 0, 0],
					[235, 93, 60, 'getDependentsList', 12, 'OSSPasswords', 0, 'ADD', 0, 0, 0],
					[236, 18, 60, 'getDependentsList', 6, 'OSSPasswords', 0, 'ADD', 0, 0, 0],
					[237, 61, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[238, 61, 51, 'getOsstimecontrol', 3, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[239, 7, 4, 'getDependentsList', 1, 'Contacts', 0, 'ADD', 0, 0, 0],
					[268, 34, 43, 'getDependentsList', 7, 'Project', 0, 'ADD', 0, 0, 0],
					[283, 7, 51, 'getDependentsList', 6, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[284, 43, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[285, 34, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[287, 4, 74, 'getDependentsList', 26, 'CallHistory', 0, '', 0, 0, 0],
					[288, 6, 74, 'getDependentsList', 18, 'CallHistory', 0, '', 0, 0, 0],
					[289, 7, 74, 'getDependentsList', 19, 'CallHistory', 0, '', 0, 0, 0],
					[290, 18, 74, 'getDependentsList', 7, 'CallHistory', 0, '', 0, 0, 0],
					[291, 61, 74, 'getDependentsList', 4, 'CallHistory', 0, '', 0, 0, 0],
					[293, 13, 74, 'getDependentsList', 14, 'CallHistory', 0, '', 0, 0, 0],
					[294, 75, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[303, 79, 8, 'getAttachments', 1, 'Documents', 0, 'SELECT,ADD', 0, 0, 0],
					[304, 6, 79, 'getDependentsList', 19, 'PaymentsIn', 0, 'ADD', 0, 0, 0],
					[307, 80, 8, 'getAttachments', 1, 'Documents', 0, 'SELECT,ADD', 0, 0, 0],
					[308, 6, 80, 'getDependentsList', 20, 'PaymentsOut', 0, 'ADD', 0, 0, 0],
					[311, 81, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[313, 6, 81, 'getDependentsList', 21, 'LettersIn', 0, 'ADD', 0, 0, 0],
					[315, 7, 81, 'getDependentsList', 20, 'LettersIn', 0, 'ADD', 0, 0, 0],
					[321, 82, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[323, 6, 82, 'getDependentsList', 22, 'LettersOut', 0, 'ADD', 0, 0, 0],
					[325, 7, 82, 'getDependentsList', 21, 'LettersOut', 0, 'ADD', 0, 0, 0],
					[326, 18, 82, 'getDependentsList', 9, 'LettersOut', 0, 'ADD', 0, 0, 0],
					[327, 18, 81, 'getDependentsList', 10, 'LettersIn', 0, 'ADD', 0, 0, 0],
					[328, 61, 82, 'getDependentsList', 6, 'LettersOut', 0, 'ADD', 0, 0, 0],
					[329, 61, 81, 'getDependentsList', 7, 'LettersIn', 0, 'ADD', 0, 0, 0],
					[331, 61, 78, 'getDependentsList', 5, 'HolidaysEntitlement', 0, 'ADD', 0, 0, 0],
					[335, 83, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[336, 84, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[337, 6, 84, 'getDependentsList', 23, 'Reservations', 0, 'ADD', 0, 0, 0],
					[338, 7, 84, 'getDependentsList', 22, 'Reservations', 0, 'ADD', 0, 0, 0],
					[339, 18, 84, 'getDependentsList', 8, 'Reservations', 0, 'ADD', 0, 0, 0],
					[341, 43, 84, 'getDependentsList', 8, 'Reservations', 0, 'ADD', 0, 0, 0],
					[342, 13, 84, 'getDependentsList', 17, 'Reservations', 0, 'ADD', 0, 0, 0],
					[347, 13, 4, 'getRelatedList', 18, 'Contacts', 0, 'SELECT', 0, 0, 0],
					[349, 59, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[350, 57, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[351, 58, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[352, 41, 42, 'getDependentsList', 1, 'ProjectTask', 0, 'ADD', 0, 0, 0],
					[353, 54, 18, 'getRecordToMails', 10, 'Vendors', 0, 'ADD,SELECT', 0, 0, 0],
					[354, 43, 4, 'getRelatedList', 9, 'Contacts', 0, 'ADD,SELECT', 0, 0, 0],
					[355, 4, 43, 'getRelatedList', 27, 'Project', 0, 'ADD,SELECT', 0, 0, 0],
					[368, 81, 4, 'getRelatedList', 2, 'Contacts', 0, 'ADD,SELECT', 0, 0, 0],
					[369, 82, 4, 'getRelatedList', 2, 'Contacts', 0, 'ADD,SELECT', 0, 0, 0],
					[370, 18, 13, 'getDependentsList', 11, 'HelpDesk', 0, 'ADD', 0, 0, 0],
					[371, 18, 43, 'getDependentsList', 12, 'Project', 0, 'ADD', 0, 0, 0],
					[372, 9, 16, 'getDependentsList', 3, 'Events', 0, 'ADD', 0, 0, 0],
					[373, 85, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[374, 85, 54, 'getEmails', 2, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[375, 86, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[376, 86, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[377, 87, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[378, 87, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[379, 86, 87, 'getDependentsList', 5, 'SRequirementsCards', 0, 'ADD', 0, 0, 0],
					[380, 86, 85, 'getDependentsList', 4, 'SQuoteEnquiries', 0, 'ADD', 0, 0, 0],
					[381, 88, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[382, 88, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[383, 86, 88, 'getDependentsList', 6, 'SCalculations', 0, 'ADD', 0, 0, 0],
					[384, 87, 88, 'getDependentsList', 4, 'SCalculations', 0, 'ADD', 0, 0, 0],
					[385, 89, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[386, 89, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[387, 86, 89, 'getDependentsList', 7, 'SQuotes', 0, 'ADD', 0, 0, 0],
					[388, 88, 89, 'getDependentsList', 4, 'SQuotes', 0, 'ADD', 0, 0, 0],
					[389, 90, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[390, 90, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[391, 86, 90, 'getDependentsList', 8, 'SSingleOrders', 0, 'ADD', 0, 0, 0],
					[392, 89, 90, 'getDependentsList', 4, 'SSingleOrders', 0, 'ADD', 0, 0, 0],
					[393, 91, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[394, 91, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[395, 86, 91, 'getDependentsList', 9, 'SRecurringOrders', 0, 'ADD', 0, 0, 0],
					[396, 89, 91, 'getDependentsList', 5, 'SRecurringOrders', 0, 'ADD', 0, 0, 0],
					[397, 18, 79, 'getDependentsList', 13, 'PaymentsIn', 0, 'ADD', 0, 0, 0],
					[398, 18, 80, 'getDependentsList', 14, 'PaymentsOut', 0, 'ADD', 0, 0, 0],
					[399, 18, 8, 'getAttachments', 5, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[400, 92, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[401, 92, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[402, 93, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[403, 93, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[404, 86, 14, 'getRelatedList', 10, 'Products', 1, 'SELECT', 0, 1, 1],
					[405, 86, 59, 'getDependentsList', 11, 'OutsourcedProducts', 1, 'ADD', 0, 0, 1],
					[406, 86, 37, 'getDependentsList', 12, 'Assets', 1, 'ADD', 0, 0, 1],
					[407, 86, 35, 'getRelatedList', 13, 'Services', 1, 'SELECT', 0, 0, 0],
					[408, 86, 57, 'getDependentsList', 14, 'OSSOutsourcedServices', 1, 'ADD', 0, 0, 1],
					[409, 86, 58, 'getDependentsList', 15, 'OSSSoldServices', 1, 'ADD', 0, 0, 1],
					[410, 18, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[411, 92, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[412, 93, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[413, 61, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[414, 86, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[415, 85, 9, 'getActivities', 3, 'Activities', 0, 'ADD', 0, 0, 0],
					[416, 87, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[417, 88, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[418, 89, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[419, 90, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[420, 91, 9, 'getActivities', 1, 'Activities', 0, 'ADD', 0, 0, 0],
					[421, 18, 51, 'getDependentsList', 15, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[422, 92, 51, 'getDependentsList', 4, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[423, 93, 51, 'getDependentsList', 4, 'OSSTimeControl', 0, 'ADD', 0, 0, 0],
					[424, 86, 4, 'getRelatedList', 16, 'Contacts', 0, 'ADD,SELECT', 0, 0, 0],
					[425, 92, 4, 'getDependentsList', 5, 'Contacts', 0, 'ADD', 0, 0, 0],
					[426, 93, 4, 'getDependentsList', 5, 'Contacts', 0, 'ADD', 0, 0, 0],
					[427, 54, 92, 'getRecordToMails', 11, 'Partners', 0, 'ADD,SELECT', 0, 0, 0],
					[428, 54, 93, 'getRecordToMails', 12, 'Competition', 0, 'ADD,SELECT', 0, 0, 0],
					[429, 54, 61, 'getRecordToMails', 13, 'OSSEmployees', 0, 'ADD,SELECT', 0, 0, 0],
					[430, 61, 54, 'getEmails', 8, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[431, 54, 86, 'getRecordToMails', 14, 'SSalesProcesses', 0, 'ADD,SELECT', 0, 0, 0],
					[432, 54, 42, 'getRecordToMails', 15, 'ProjectTask', 0, 'ADD,SELECT', 0, 0, 0],
					[433, 42, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[434, 54, 41, 'getRecordToMails', 16, 'ProjectMilestone', 0, 'ADD,SELECT', 0, 0, 0],
					[435, 41, 54, 'getEmails', 2, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[436, 54, 85, 'getRecordToMails', 17, 'SQuoteEnquiries', 0, 'ADD,SELECT', 0, 0, 0],
					[437, 54, 87, 'getRecordToMails', 18, 'SRequirementsCards', 0, 'ADD,SELECT', 0, 0, 0],
					[438, 54, 88, 'getRecordToMails', 19, 'SCalculations', 0, 'ADD,SELECT', 0, 0, 0],
					[439, 54, 89, 'getRecordToMails', 20, 'SQuotes', 0, 'ADD,SELECT', 0, 0, 0],
					[440, 54, 90, 'getRecordToMails', 21, 'SSingleOrders', 0, 'ADD,SELECT', 0, 0, 0],
					[441, 54, 91, 'getRecordToMails', 22, 'SRecurringOrders', 0, 'ADD,SELECT', 0, 0, 0],
					[442, 26, 18, 'getCampaignsRecords', 7, 'Vendors', 0, 'ADD,SELECT', 0, 0, 0],
					[443, 18, 26, 'getCampaigns', 16, 'Campaigns', 0, 'SELECT', 0, 0, 0],
					[444, 26, 4, 'getCampaignsRecords', 6, 'Contacts', 0, 'ADD,SELECT', 0, 0, 0],
					[445, 4, 26, 'getCampaigns', 28, 'Campaigns', 0, 'SELECT', 0, 0, 0],
					[446, 26, 92, 'getCampaignsRecords', 8, 'Partners', 0, 'ADD,SELECT', 0, 0, 0],
					[447, 92, 26, 'getCampaigns', 6, 'Campaigns', 0, 'SELECT', 0, 0, 0],
					[448, 26, 93, 'getCampaignsRecords', 9, 'Competition', 0, 'ADD,SELECT', 0, 0, 0],
					[449, 93, 26, 'getCampaigns', 6, 'Campaigns', 0, 'SELECT', 0, 0, 0],
					[450, 6, 94, 'getDependentsList', 24, 'FBookkeeping', 0, 'ADD', 0, 0, 0],
					[451, 6, 95, 'getDependentsList', 25, 'FInvoice', 0, 'ADD', 0, 0, 0],
					[452, 6, 86, 'getDependentsList', 26, 'SSalesProcesses', 0, 'ADD', 0, 0, 0],
					[453, 6, 99, 'getDependentsList', 27, 'FInvoiceProforma', 0, 'ADD', 0, 0, 0],
					[454, 97, 100, 'getDependentsList', 4, 'IGDN', 0, 'ADD', 0, 0, 0],
					[455, 100, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[456, 101, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[457, 97, 101, 'getDependentsList', 5, 'IIDN', 0, 'ADD', 0, 0, 0],
					[458, 98, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[459, 97, 98, 'getDependentsList', 3, 'IGRN', 0, 'ADD', 0, 0, 0],
					[460, 102, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[461, 97, 102, 'getDependentsList', 6, 'IGIN', 0, 'ADD', 0, 0, 0],
					[462, 97, 14, 'getManyToMany', 1, 'Products', 0, '', 0, 0, 0],
					[463, 14, 97, 'getManyToMany', 17, 'IStorages', 0, '', 0, 0, 0],
					[464, 103, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[465, 97, 103, 'getDependentsList', 7, 'IPreOrder', 0, 'ADD', 0, 0, 0],
					[466, 97, 8, 'getAttachments', 2, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[467, 94, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[468, 104, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[469, 97, 104, 'getDependentsList', 8, 'ISTDN', 0, 'ADD', 0, 0, 0],
					[470, 105, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[471, 105, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[472, 104, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[473, 98, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[474, 100, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[475, 101, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[476, 102, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[477, 103, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[478, 106, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[479, 106, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[480, 97, 106, 'getDependentsList', 9, 'ISTRN', 0, 'ADD', 0, 0, 0],
					[481, 105, 104, 'getDependentsList', 3, 'ISTDN', 0, 'ADD', 0, 0, 0],
					[482, 105, 106, 'getDependentsList', 4, 'ISTRN', 0, 'ADD', 0, 0, 0],
					[483, 106, 104, 'getDependentsList', 3, 'ISTDN', 0, 'ADD', 0, 0, 0],
					[484, 104, 106, 'getDependentsList', 3, 'ISTRN', 0, 'ADD', 0, 0, 0],
					[485, 96, 8, 'getAttachments', 1, 'Documents', 0, 'ADD', 0, 0, 0],
					[486, 105, 98, 'getRelatedList', 5, 'IGRN', 0, 'ADD', 0, 0, 0],
					[487, 105, 100, 'getRelatedList', 6, 'IGDN', 0, 'ADD', 0, 0, 0],
					[488, 105, 102, 'getRelatedList', 7, 'IGIN', 0, 'ADD', 0, 0, 0],
					[489, 105, 101, 'getRelatedList', 8, 'IIDN', 0, 'ADD', 0, 0, 0],
					[490, 6, 107, 'getDependentsList', 28, 'FCorectingInvoice', 0, 'ADD', 0, 0, 0],
					[491, 95, 107, 'getDependentsList', 1, 'FCorectingInvoice', 0, 'ADD', 0, 0, 0],
					[492, 108, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[493, 108, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[494, 97, 108, 'getDependentsList', 10, 'IGRNC', 0, 'ADD', 0, 0, 0],
					[495, 105, 108, 'getRelatedList', 9, 'IGRNC', 0, 'ADD', 0, 0, 0],
					[496, 98, 108, 'getDependentsList', 3, 'IGRNC', 0, 'ADD', 0, 0, 0],
					[497, 109, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[498, 109, 9, 'getRelatedList', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[499, 97, 109, 'getDependentsList', 11, 'IGDNC', 0, 'ADD', 0, 0, 0],
					[500, 105, 109, 'getRelatedList', 10, 'IGDNC', 0, 'ADD', 0, 0, 0],
					[501, 100, 109, 'getDependentsList', 3, 'IGDNC', 0, 'ADD', 0, 0, 0],
					[502, 14, 15, 'getDependentsList', 18, 'Faq', 0, 'ADD', 0, 0, 0],
					[503, 97, 90, 'getDependentsList', 12, 'SSingleOrders', 0, 'ADD', 0, 0, 0],
					[504, 90, 100, 'getDependentsList', 4, 'IGDN', 0, 'ADD', 0, 0, 0],
					[505, 13, 60, 'getRelatedList', 19, 'OSSPasswords', 0, 'ADD,SELECT', 0, 0, 0],
					[506, 86, 60, 'getRelatedList', 17, 'OSSPasswords', 0, 'ADD,SELECT', 0, 0, 0],
					[507, 43, 60, 'getRelatedList', 10, 'OSSPasswords', 0, 'ADD,SELECT', 0, 0, 0],
					[508, 61, 60, 'getDependentsList', 9, 'OSSPasswords', 0, 'ADD', 0, 0, 0],
					[509, 26, 8, 'getAttachments', 3, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[510, 13, 14, 'getRelatedList', 20, 'Products', 1, 'ADD,SELECT', 0, 0, 0],
					[511, 13, 58, 'getRelatedList', 21, 'OSSSoldServices', 1, 'ADD,SELECT', 0, 0, 0],
					[512, 58, 13, 'getRelatedList', 2, 'HelpDesk', 0, 'ADD,SELECT', 0, 0, 0],
					[513, 26, 86, 'getDependentsList', 10, 'SSalesProcesses', 0, 'ADD', 0, 0, 0],
					[514, 86, 43, 'getDependentsList', 19, 'Project', 0, 'ADD', 0, 0, 0],
					[515, 92, 86, 'getRelatedList', 13, 'SSalesProcesses', 0, 'SELECT,ADD', 0, 0, 0],
					[516, 18, 86, 'getRelatedList', 17, 'SSalesProcesses', 0, 'SELECT,ADD', 0, 0, 0],
					[517, 93, 86, 'getRelatedList', 13, 'SSalesProcesses', 0, 'SELECT,ADD', 0, 0, 0],
					[518, 86, 92, 'getRelatedList', 20, 'Partners', 0, 'SELECT,ADD', 0, 0, 0],
					[519, 86, 18, 'getRelatedList', 21, 'Vendors', 0, 'SELECT,ADD', 0, 0, 0],
					[520, 86, 93, 'getRelatedList', 22, 'Competition', 0, 'SELECT,ADD', 0, 0, 0],
					[521, 92, 43, 'getRelatedList', 14, 'Project', 0, 'SELECT,ADD', 0, 0, 0],
					[523, 43, 18, 'getRelatedList', 11, 'Vendors', 0, 'SELECT,ADD', 0, 0, 0],
					[524, 43, 92, 'getRelatedList', 12, 'Partners', 0, 'SELECT,ADD', 0, 0, 0],
					[525, 112, 8, 'getManyToMany', 1, 'Documents', 0, 'SELECT', 0, 0, 0],
					[526, 113, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[527, 114, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[528, 115, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[529, 117, 8, 'getAttachments', 1, 'Documents', 0, 'ADD,SELECT', 0, 0, 0],
					[530, 117, 9, 'getActivities', 2, 'Activities', 0, 'ADD', 0, 0, 0],
					[531, 117, 54, 'getEmails', 3, 'OSSMailView', 0, 'SEND', 0, 0, 0],
					[532, 86, 117, 'getDependentsList', 23, 'SVendorEnquiries', 0, 'ADD', 0, 0, 0],
					[533, 18, 117, 'getDependentsList', 18, 'SVendorEnquiries', 0, 'ADD', 0, 0, 0],
					[534, 88, 117, 'getDependentsList', 5, 'SVendorEnquiries', 0, 'ADD', 0, 0, 0],
					[535, 54, 117, 'getRecordToMails', 23, 'SVendorEnquiries', 0, 'ADD,SELECT', 0, 0, 0],
					[536, 6, 111, 'getDependentsList', 29, 'Notification', 0, 'ADD', 0, 0, 0],
					[537, 13, 111, 'getDependentsList', 22, 'Notification', 0, 'ADD', 0, 0, 0],
				]
			],
			'vtiger_reminder_interval' => [
				'columns' => ['reminder_intervalid', 'reminder_interval', 'sortorderid', 'presence'],
				'values' => [
					[2, '1 Minute', 1, 1],
					[3, '5 Minutes', 2, 1],
					[4, '15 Minutes', 3, 1],
					[5, '30 Minutes', 4, 1],
					[6, '45 Minutes', 5, 1],
					[7, '1 Hour', 6, 1],
					[8, '1 Day', 7, 1],
				]
			],
			'vtiger_reminder_interval_seq' => [
				'columns' => ['id'],
				'values' => [
					[8],
				]
			],
			'vtiger_reportfilters' => [
				'columns' => ['filterid', 'name'],
				'values' => [
					[1, 'Private'],
					[2, 'Public'],
					[3, 'Shared'],
				]
			],
			'vtiger_reportfolder' => [
				'columns' => ['folderid', 'foldername', 'description', 'state'],
				'values' => [
					[1, 'LBL_MAIN', '', 'SAVED'],
				]
			],
			'vtiger_reservations_status' => [
				'columns' => ['reservations_statusid', 'reservations_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_ACCEPTED', 2, 1, 851],
					[2, 'PLL_TO_APPROVAL', 3, 1, 852],
					[3, 'PLL_DRAFT', 1, 1, 853],
					[4, 'PLL_CANCELLED', 4, 1, 854],
				]
			],
			'vtiger_reservations_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_role' => [
				'columns' => ['roleid', 'rolename', 'parentrole', 'depth', 'allowassignedrecordsto', 'changeowner', 'searchunpriv', 'clendarallorecords', 'listrelatedrecord', 'previewrelatedrecord', 'editrelatedrecord', 'permissionsrelatedfield', 'globalsearchadv', 'assignedmultiowner', 'auto_assign'],
				'values' => [
					['H1', 'Organization', 'H1', 0, 1, 1, null, 1, 0, 0, 0, '0', 1, 0, 0],
					['H10', 'Sales Department', 'H1::H2::H10', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H14', 'Logistics Department', 'H1::H2::H14', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H18', 'Project Management Department', 'H1::H2::H18', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H2', 'Board of Management', 'H1::H2', 1, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H22', 'Help Desk Department', 'H1::H2::H22', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H26', 'Human Resources Department', 'H1::H2::H26', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H3', 'Secretary\'s Office', 'H1::H2::H3', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H30', 'Financial Department', 'H1::H2::H30', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H34', 'Production Department', 'H1::H2::H34', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
					['H6', 'Marketing Department', 'H1::H2::H6', 2, 1, 1, null, 3, 0, 0, 0, '0', 1, 1, 0],
				]
			],
			'vtiger_role2picklist' => [
				'columns' => ['roleid', 'picklistvalueid', 'picklistid', 'sortid'],
				'values' => [
					['H1', 659, 85, 1],
					['H1', 660, 85, 2],
					['H1', 661, 85, 3],
					['H1', 666, 86, 1],
					['H1', 667, 86, 2],
					['H1', 668, 87, 1],
					['H1', 669, 88, 1],
					['H1', 670, 89, 1],
					['H1', 671, 88, 2],
					['H1', 675, 90, 1],
					['H1', 676, 90, 2],
					['H1', 677, 90, 3],
					['H1', 678, 90, 4],
					['H1', 679, 90, 5],
					['H1', 680, 90, 6],
					['H1', 681, 90, 7],
					['H1', 682, 91, 1],
					['H1', 683, 91, 2],
					['H1', 684, 91, 3],
					['H1', 685, 91, 4],
					['H1', 686, 91, 5],
					['H1', 687, 91, 6],
					['H1', 688, 91, 7],
					['H1', 691, 92, 1],
					['H1', 692, 93, 1],
					['H1', 693, 92, 2],
					['H1', 697, 94, 1],
					['H1', 698, 94, 2],
					['H1', 699, 94, 3],
					['H1', 703, 95, 1],
					['H1', 704, 95, 2],
					['H1', 705, 96, 1],
					['H1', 706, 96, 2],
					['H1', 707, 96, 3],
					['H1', 708, 96, 4],
					['H1', 709, 96, 5],
					['H1', 710, 96, 6],
					['H1', 711, 96, 7],
					['H1', 712, 96, 8],
					['H1', 714, 97, 1],
					['H1', 715, 97, 2],
					['H1', 716, 97, 3],
					['H1', 717, 98, 1],
					['H1', 718, 98, 2],
					['H1', 719, 98, 3],
					['H1', 723, 99, 1],
					['H1', 724, 99, 2],
					['H1', 725, 99, 3],
					['H1', 726, 99, 4],
					['H1', 727, 99, 5],
					['H1', 728, 100, 1],
					['H1', 729, 100, 2],
					['H1', 730, 100, 3],
					['H1', 731, 100, 4],
					['H1', 732, 100, 5],
					['H1', 733, 100, 6],
					['H1', 734, 100, 7],
					['H1', 735, 100, 8],
					['H1', 736, 100, 9],
					['H1', 738, 102, 1],
					['H1', 739, 102, 2],
					['H1', 740, 102, 3],
					['H1', 741, 102, 4],
					['H1', 742, 102, 5],
					['H1', 743, 102, 6],
					['H1', 744, 102, 7],
					['H1', 745, 102, 8],
					['H1', 746, 103, 1],
					['H1', 747, 103, 2],
					['H1', 748, 103, 3],
					['H1', 749, 104, 1],
					['H1', 750, 104, 2],
					['H1', 751, 104, 3],
					['H1', 752, 104, 4],
					['H1', 753, 104, 5],
					['H1', 754, 104, 6],
					['H1', 755, 104, 7],
					['H1', 756, 104, 8],
					['H1', 757, 105, 1],
					['H1', 758, 105, 2],
					['H1', 759, 105, 3],
					['H1', 760, 106, 1],
					['H1', 761, 106, 2],
					['H1', 762, 106, 3],
					['H1', 763, 106, 4],
					['H1', 773, 107, 1],
					['H1', 774, 107, 2],
					['H1', 775, 107, 3],
					['H1', 776, 107, 4],
					['H1', 777, 107, 5],
					['H1', 778, 108, 1],
					['H1', 779, 108, 2],
					['H1', 780, 108, 3],
					['H1', 781, 108, 4],
					['H1', 782, 108, 5],
					['H1', 783, 108, 6],
					['H1', 784, 109, 1],
					['H1', 785, 109, 2],
					['H1', 786, 109, 3],
					['H1', 787, 109, 4],
					['H1', 788, 109, 5],
					['H1', 789, 109, 6],
					['H1', 790, 110, 1],
					['H1', 791, 110, 2],
					['H1', 792, 110, 3],
					['H1', 793, 110, 4],
					['H1', 794, 110, 5],
					['H1', 795, 110, 6],
					['H1', 796, 111, 1],
					['H1', 797, 111, 2],
					['H1', 798, 111, 3],
					['H1', 799, 111, 4],
					['H1', 800, 111, 5],
					['H1', 801, 111, 6],
					['H1', 802, 112, 1],
					['H1', 803, 112, 2],
					['H1', 804, 112, 3],
					['H1', 805, 112, 4],
					['H1', 806, 112, 5],
					['H1', 807, 112, 6],
					['H1', 808, 113, 1],
					['H1', 809, 113, 2],
					['H1', 810, 113, 3],
					['H1', 811, 113, 4],
					['H1', 812, 113, 5],
					['H1', 813, 113, 6],
					['H1', 814, 114, 1],
					['H1', 815, 114, 2],
					['H1', 816, 114, 3],
					['H1', 817, 114, 4],
					['H1', 818, 114, 5],
					['H1', 819, 114, 6],
					['H1', 820, 115, 1],
					['H1', 821, 115, 2],
					['H1', 822, 115, 3],
					['H1', 823, 115, 4],
					['H1', 824, 115, 5],
					['H1', 825, 115, 6],
					['H1', 826, 116, 1],
					['H1', 827, 116, 2],
					['H1', 828, 116, 3],
					['H1', 829, 116, 4],
					['H1', 830, 116, 5],
					['H1', 831, 116, 6],
					['H1', 832, 117, 1],
					['H1', 833, 117, 2],
					['H1', 834, 117, 3],
					['H1', 835, 117, 4],
					['H1', 836, 117, 5],
					['H1', 837, 117, 6],
					['H1', 838, 118, 1],
					['H1', 839, 118, 2],
					['H1', 840, 118, 3],
					['H1', 841, 118, 4],
					['H1', 842, 118, 5],
					['H1', 843, 118, 6],
					['H1', 844, 118, 7],
					['H1', 845, 119, 1],
					['H1', 846, 119, 2],
					['H1', 847, 120, 1],
					['H1', 848, 120, 2],
					['H1', 849, 121, 1],
					['H1', 850, 121, 2],
					['H1', 851, 122, 2],
					['H1', 852, 122, 3],
					['H1', 853, 122, 1],
					['H1', 854, 122, 4],
					['H1', 855, 123, 1],
					['H1', 856, 123, 2],
					['H1', 857, 123, 3],
					['H1', 858, 123, 4],
					['H1', 859, 123, 5],
					['H1', 860, 124, 1],
					['H1', 861, 124, 2],
					['H1', 862, 124, 3],
					['H1', 863, 124, 4],
					['H1', 864, 124, 5],
					['H1', 865, 125, 1],
					['H1', 866, 125, 2],
					['H1', 867, 125, 3],
					['H1', 868, 125, 4],
					['H1', 869, 125, 5],
					['H1', 870, 125, 6],
					['H1', 871, 125, 7],
					['H1', 872, 126, 1],
					['H1', 873, 126, 2],
					['H1', 874, 126, 3],
					['H1', 875, 126, 4],
					['H1', 876, 126, 5],
					['H1', 877, 126, 6],
					['H1', 878, 126, 7],
					['H1', 879, 126, 8],
					['H1', 880, 126, 9],
					['H1', 881, 127, 1],
					['H1', 882, 127, 2],
					['H1', 883, 127, 3],
					['H1', 884, 127, 4],
					['H1', 885, 127, 5],
					['H1', 886, 128, 1],
					['H1', 887, 128, 2],
					['H1', 888, 128, 3],
					['H1', 889, 128, 4],
					['H1', 890, 128, 5],
					['H1', 891, 128, 6],
					['H1', 892, 128, 7],
					['H1', 893, 128, 8],
					['H1', 894, 128, 9],
					['H1', 895, 128, 10],
					['H1', 896, 128, 11],
					['H1', 897, 128, 12],
					['H1', 898, 128, 13],
					['H1', 899, 128, 14],
					['H1', 900, 128, 15],
					['H1', 901, 128, 16],
					['H1', 902, 129, 1],
					['H1', 903, 129, 2],
					['H1', 904, 129, 3],
					['H1', 905, 129, 4],
					['H1', 906, 129, 5],
					['H1', 907, 129, 6],
					['H1', 908, 130, 1],
					['H1', 909, 130, 2],
					['H10', 1, 1, 0],
					['H10', 2, 1, 1],
					['H10', 3, 1, 2],
					['H10', 4, 1, 3],
					['H10', 5, 1, 4],
					['H10', 6, 1, 5],
					['H10', 7, 1, 6],
					['H10', 8, 1, 7],
					['H10', 9, 1, 8],
					['H10', 10, 1, 9],
					['H10', 11, 1, 10],
					['H10', 12, 2, 0],
					['H10', 13, 2, 1],
					['H10', 14, 3, 0],
					['H10', 15, 3, 1],
					['H10', 16, 3, 2],
					['H10', 17, 3, 3],
					['H10', 18, 3, 4],
					['H10', 19, 3, 5],
					['H10', 20, 4, 0],
					['H10', 21, 4, 1],
					['H10', 22, 4, 2],
					['H10', 23, 4, 3],
					['H10', 24, 4, 4],
					['H10', 25, 4, 5],
					['H10', 26, 4, 6],
					['H10', 27, 4, 7],
					['H10', 28, 4, 8],
					['H10', 29, 4, 9],
					['H10', 30, 4, 10],
					['H10', 31, 4, 11],
					['H10', 32, 4, 12],
					['H10', 39, 6, 1],
					['H10', 40, 6, 2],
					['H10', 41, 7, 0],
					['H10', 42, 7, 1],
					['H10', 43, 7, 2],
					['H10', 44, 7, 3],
					['H10', 45, 7, 4],
					['H10', 46, 8, 0],
					['H10', 47, 9, 0],
					['H10', 48, 9, 1],
					['H10', 49, 9, 2],
					['H10', 50, 9, 3],
					['H10', 51, 10, 0],
					['H10', 52, 10, 1],
					['H10', 53, 10, 2],
					['H10', 54, 10, 3],
					['H10', 55, 10, 4],
					['H10', 56, 10, 5],
					['H10', 57, 10, 6],
					['H10', 58, 10, 7],
					['H10', 59, 10, 8],
					['H10', 60, 11, 0],
					['H10', 61, 11, 1],
					['H10', 98, 13, 0],
					['H10', 99, 13, 1],
					['H10', 101, 13, 3],
					['H10', 106, 13, 8],
					['H10', 107, 13, 9],
					['H10', 108, 13, 10],
					['H10', 109, 13, 11],
					['H10', 110, 13, 12],
					['H10', 111, 14, 0],
					['H10', 123, 15, 0],
					['H10', 124, 15, 1],
					['H10', 125, 15, 2],
					['H10', 126, 15, 3],
					['H10', 160, 22, 0],
					['H10', 161, 22, 1],
					['H10', 162, 22, 2],
					['H10', 163, 22, 3],
					['H10', 164, 22, 4],
					['H10', 165, 22, 5],
					['H10', 170, 24, 0],
					['H10', 171, 24, 1],
					['H10', 172, 24, 2],
					['H10', 173, 25, 0],
					['H10', 174, 25, 1],
					['H10', 175, 25, 2],
					['H10', 176, 25, 3],
					['H10', 177, 25, 4],
					['H10', 178, 25, 5],
					['H10', 182, 27, 0],
					['H10', 183, 27, 1],
					['H10', 184, 27, 2],
					['H10', 185, 27, 3],
					['H10', 186, 28, 0],
					['H10', 187, 28, 1],
					['H10', 188, 28, 2],
					['H10', 189, 28, 3],
					['H10', 190, 29, 0],
					['H10', 191, 29, 1],
					['H10', 192, 29, 2],
					['H10', 193, 29, 3],
					['H10', 194, 30, 0],
					['H10', 195, 30, 1],
					['H10', 196, 30, 2],
					['H10', 197, 30, 3],
					['H10', 198, 30, 4],
					['H10', 210, 31, 1],
					['H10', 211, 31, 2],
					['H10', 212, 31, 3],
					['H10', 213, 31, 4],
					['H10', 214, 32, 1],
					['H10', 215, 32, 2],
					['H10', 216, 32, 3],
					['H10', 217, 32, 4],
					['H10', 218, 32, 5],
					['H10', 219, 32, 6],
					['H10', 220, 33, 1],
					['H10', 221, 33, 2],
					['H10', 222, 33, 3],
					['H10', 223, 34, 1],
					['H10', 224, 34, 2],
					['H10', 225, 34, 3],
					['H10', 226, 35, 1],
					['H10', 227, 35, 2],
					['H10', 228, 35, 3],
					['H10', 237, 38, 1],
					['H10', 238, 38, 2],
					['H10', 239, 38, 3],
					['H10', 240, 38, 4],
					['H10', 241, 39, 1],
					['H10', 242, 39, 2],
					['H10', 243, 39, 3],
					['H10', 244, 39, 4],
					['H10', 245, 40, 1],
					['H10', 246, 40, 2],
					['H10', 247, 40, 3],
					['H10', 248, 40, 4],
					['H10', 249, 41, 1],
					['H10', 250, 41, 2],
					['H10', 251, 41, 3],
					['H10', 252, 41, 4],
					['H10', 253, 41, 5],
					['H10', 254, 41, 6],
					['H10', 255, 41, 7],
					['H10', 256, 41, 8],
					['H10', 257, 41, 9],
					['H10', 258, 41, 10],
					['H10', 259, 41, 11],
					['H10', 260, 42, 1],
					['H10', 261, 42, 2],
					['H10', 262, 42, 3],
					['H10', 263, 42, 4],
					['H10', 264, 42, 5],
					['H10', 265, 42, 6],
					['H10', 266, 42, 7],
					['H10', 267, 42, 8],
					['H10', 268, 42, 9],
					['H10', 269, 43, 1],
					['H10', 270, 43, 2],
					['H10', 271, 43, 3],
					['H10', 272, 43, 4],
					['H10', 273, 44, 1],
					['H10', 274, 44, 2],
					['H10', 275, 44, 3],
					['H10', 276, 44, 4],
					['H10', 277, 45, 1],
					['H10', 278, 45, 2],
					['H10', 279, 45, 3],
					['H10', 280, 45, 4],
					['H10', 281, 45, 5],
					['H10', 282, 45, 6],
					['H10', 283, 45, 7],
					['H10', 284, 45, 8],
					['H10', 285, 45, 9],
					['H10', 286, 45, 10],
					['H10', 287, 45, 11],
					['H10', 289, 46, 1],
					['H10', 290, 46, 2],
					['H10', 291, 46, 3],
					['H10', 292, 46, 4],
					['H10', 293, 46, 5],
					['H10', 294, 46, 6],
					['H10', 295, 47, 1],
					['H10', 296, 47, 2],
					['H10', 297, 48, 1],
					['H10', 298, 48, 2],
					['H10', 299, 48, 3],
					['H10', 300, 48, 4],
					['H10', 301, 48, 5],
					['H10', 302, 48, 6],
					['H10', 303, 48, 7],
					['H10', 304, 48, 8],
					['H10', 305, 48, 9],
					['H10', 306, 48, 10],
					['H10', 307, 48, 11],
					['H10', 308, 48, 12],
					['H10', 309, 48, 13],
					['H10', 310, 48, 14],
					['H10', 311, 48, 15],
					['H10', 312, 48, 16],
					['H10', 313, 48, 17],
					['H10', 314, 48, 18],
					['H10', 315, 48, 19],
					['H10', 316, 48, 20],
					['H10', 317, 48, 21],
					['H10', 318, 48, 22],
					['H10', 319, 48, 23],
					['H10', 320, 48, 24],
					['H10', 323, 49, 2],
					['H10', 324, 49, 3],
					['H10', 325, 50, 1],
					['H10', 326, 50, 2],
					['H10', 387, 29, null],
					['H10', 388, 57, 1],
					['H10', 389, 57, 2],
					['H10', 393, 58, 1],
					['H10', 394, 58, 2],
					['H10', 395, 58, 3],
					['H10', 396, 58, 4],
					['H10', 397, 58, 5],
					['H10', 398, 58, 6],
					['H10', 399, 58, 7],
					['H10', 400, 58, 8],
					['H10', 401, 59, 1],
					['H10', 402, 59, 2],
					['H10', 403, 60, 1],
					['H10', 404, 60, 2],
					['H10', 416, 63, 1],
					['H10', 417, 63, 2],
					['H10', 418, 63, 3],
					['H10', 419, 64, 1],
					['H10', 420, 64, 2],
					['H10', 422, 65, 1],
					['H10', 423, 65, 2],
					['H10', 424, 65, 3],
					['H10', 425, 66, 1],
					['H10', 426, 66, 2],
					['H10', 427, 66, 3],
					['H10', 440, 68, 1],
					['H10', 444, 68, 2],
					['H10', 445, 68, 3],
					['H10', 446, 14, 12],
					['H10', 451, 14, 17],
					['H10', 452, 14, 18],
					['H10', 453, 14, 19],
					['H10', 455, 11, 2],
					['H10', 456, 11, 3],
					['H10', 457, 11, 4],
					['H10', 458, 11, 5],
					['H10', 459, 11, 6],
					['H10', 460, 11, 7],
					['H10', 461, 11, 8],
					['H10', 462, 11, 9],
					['H10', 463, 11, 10],
					['H10', 464, 11, 11],
					['H10', 465, 11, 12],
					['H10', 508, 37, 3],
					['H10', 509, 37, 4],
					['H10', 510, 37, 5],
					['H10', 511, 37, 6],
					['H10', 557, 29, 4],
					['H10', 559, 70, 1],
					['H10', 560, 70, 2],
					['H10', 564, 71, 1],
					['H10', 565, 71, 2],
					['H10', 576, 74, 1],
					['H10', 577, 74, 2],
					['H10', 578, 74, 3],
					['H10', 579, 74, 4],
					['H10', 580, 74, 5],
					['H10', 581, 74, 6],
					['H10', 582, 74, 7],
					['H10', 583, 74, 8],
					['H10', 584, 74, 9],
					['H10', 585, 74, 10],
					['H10', 586, 74, 11],
					['H10', 587, 75, 1],
					['H10', 588, 75, 2],
					['H10', 589, 75, 3],
					['H10', 590, 76, 1],
					['H10', 591, 76, 2],
					['H10', 592, 76, 3],
					['H10', 593, 77, 1],
					['H10', 594, 77, 2],
					['H10', 595, 77, 3],
					['H10', 596, 77, 4],
					['H10', 597, 77, 5],
					['H10', 598, 77, 6],
					['H10', 599, 77, 7],
					['H10', 600, 77, 8],
					['H10', 601, 77, 9],
					['H10', 602, 77, 10],
					['H10', 603, 77, 11],
					['H10', 604, 77, 12],
					['H10', 605, 77, 13],
					['H10', 606, 77, 14],
					['H10', 607, 77, 15],
					['H10', 608, 78, 1],
					['H10', 609, 78, 2],
					['H10', 610, 78, 3],
					['H10', 611, 78, 4],
					['H10', 612, 78, 5],
					['H10', 613, 78, 6],
					['H10', 614, 78, 7],
					['H10', 615, 78, 8],
					['H10', 616, 78, 9],
					['H10', 617, 78, 10],
					['H10', 618, 78, 11],
					['H10', 619, 78, 12],
					['H10', 620, 78, 13],
					['H10', 621, 78, 14],
					['H10', 622, 78, 15],
					['H10', 659, 85, 1],
					['H10', 660, 85, 2],
					['H10', 661, 85, 3],
					['H10', 666, 86, 1],
					['H10', 667, 86, 2],
					['H10', 668, 87, 1],
					['H10', 669, 88, 1],
					['H10', 670, 89, 1],
					['H10', 671, 88, 2],
					['H10', 672, 37, 7],
					['H10', 675, 90, 1],
					['H10', 676, 90, 2],
					['H10', 677, 90, 3],
					['H10', 678, 90, 4],
					['H10', 679, 90, 5],
					['H10', 680, 90, 6],
					['H10', 681, 90, 7],
					['H10', 682, 91, 1],
					['H10', 683, 91, 2],
					['H10', 684, 91, 3],
					['H10', 685, 91, 4],
					['H10', 686, 91, 5],
					['H10', 687, 91, 6],
					['H10', 688, 91, 7],
					['H10', 689, 63, 4],
					['H10', 690, 63, 5],
					['H10', 691, 92, 1],
					['H10', 692, 93, 1],
					['H10', 693, 92, 2],
					['H10', 694, 29, 5],
					['H10', 695, 29, 6],
					['H10', 696, 29, 7],
					['H10', 697, 94, 1],
					['H10', 698, 94, 2],
					['H10', 699, 94, 3],
					['H10', 700, 57, 3],
					['H10', 701, 90, 8],
					['H10', 702, 91, 8],
					['H10', 703, 95, 1],
					['H10', 704, 95, 2],
					['H10', 705, 96, 1],
					['H10', 706, 96, 2],
					['H10', 707, 96, 3],
					['H10', 708, 96, 4],
					['H10', 709, 96, 5],
					['H10', 710, 96, 6],
					['H10', 711, 96, 7],
					['H10', 712, 96, 8],
					['H10', 713, 86, 3],
					['H10', 714, 97, 1],
					['H10', 715, 97, 2],
					['H10', 716, 97, 3],
					['H10', 717, 98, 1],
					['H10', 718, 98, 2],
					['H10', 719, 98, 3],
					['H10', 720, 14, 20],
					['H10', 721, 14, 21],
					['H10', 722, 14, 22],
					['H10', 723, 99, 1],
					['H10', 724, 99, 2],
					['H10', 725, 99, 3],
					['H10', 726, 99, 4],
					['H10', 727, 99, 5],
					['H10', 728, 100, 1],
					['H10', 729, 100, 2],
					['H10', 730, 100, 3],
					['H10', 731, 100, 4],
					['H10', 732, 100, 5],
					['H10', 733, 100, 6],
					['H10', 734, 100, 7],
					['H10', 735, 100, 8],
					['H10', 736, 100, 9],
					['H10', 738, 102, 1],
					['H10', 739, 102, 2],
					['H10', 740, 102, 3],
					['H10', 741, 102, 4],
					['H10', 742, 102, 5],
					['H10', 743, 102, 6],
					['H10', 744, 102, 7],
					['H10', 745, 102, 8],
					['H10', 746, 103, 1],
					['H10', 747, 103, 2],
					['H10', 748, 103, 3],
					['H10', 749, 104, 1],
					['H10', 750, 104, 2],
					['H10', 751, 104, 3],
					['H10', 752, 104, 4],
					['H10', 753, 104, 5],
					['H10', 754, 104, 6],
					['H10', 755, 104, 7],
					['H10', 756, 104, 8],
					['H10', 757, 105, 1],
					['H10', 758, 105, 2],
					['H10', 759, 105, 3],
					['H10', 760, 106, 1],
					['H10', 761, 106, 2],
					['H10', 762, 106, 3],
					['H10', 763, 106, 4],
					['H10', 766, 87, 2],
					['H10', 767, 87, 3],
					['H10', 768, 87, 4],
					['H10', 769, 87, 5],
					['H10', 770, 87, 6],
					['H10', 771, 87, 7],
					['H10', 772, 87, 8],
					['H10', 773, 107, 1],
					['H10', 774, 107, 2],
					['H10', 775, 107, 3],
					['H10', 776, 107, 4],
					['H10', 777, 107, 5],
					['H10', 778, 108, 1],
					['H10', 779, 108, 2],
					['H10', 780, 108, 3],
					['H10', 781, 108, 4],
					['H10', 782, 108, 5],
					['H10', 783, 108, 6],
					['H10', 784, 109, 1],
					['H10', 785, 109, 2],
					['H10', 786, 109, 3],
					['H10', 787, 109, 4],
					['H10', 788, 109, 5],
					['H10', 789, 109, 6],
					['H10', 790, 110, 1],
					['H10', 791, 110, 2],
					['H10', 792, 110, 3],
					['H10', 793, 110, 4],
					['H10', 794, 110, 5],
					['H10', 795, 110, 6],
					['H10', 796, 111, 1],
					['H10', 797, 111, 2],
					['H10', 798, 111, 3],
					['H10', 799, 111, 4],
					['H10', 800, 111, 5],
					['H10', 801, 111, 6],
					['H10', 802, 112, 1],
					['H10', 803, 112, 2],
					['H10', 804, 112, 3],
					['H10', 805, 112, 4],
					['H10', 806, 112, 5],
					['H10', 807, 112, 6],
					['H10', 808, 113, 1],
					['H10', 809, 113, 2],
					['H10', 810, 113, 3],
					['H10', 811, 113, 4],
					['H10', 812, 113, 5],
					['H10', 813, 113, 6],
					['H10', 814, 114, 1],
					['H10', 815, 114, 2],
					['H10', 816, 114, 3],
					['H10', 817, 114, 4],
					['H10', 818, 114, 5],
					['H10', 819, 114, 6],
					['H10', 820, 115, 1],
					['H10', 821, 115, 2],
					['H10', 822, 115, 3],
					['H10', 823, 115, 4],
					['H10', 824, 115, 5],
					['H10', 825, 115, 6],
					['H10', 826, 116, 1],
					['H10', 827, 116, 2],
					['H10', 828, 116, 3],
					['H10', 829, 116, 4],
					['H10', 830, 116, 5],
					['H10', 831, 116, 6],
					['H10', 832, 117, 1],
					['H10', 833, 117, 2],
					['H10', 834, 117, 3],
					['H10', 835, 117, 4],
					['H10', 836, 117, 5],
					['H10', 837, 117, 6],
					['H10', 838, 118, 1],
					['H10', 839, 118, 2],
					['H10', 840, 118, 3],
					['H10', 841, 118, 4],
					['H10', 842, 118, 5],
					['H10', 843, 118, 6],
					['H10', 844, 118, 7],
					['H10', 845, 119, 1],
					['H10', 846, 119, 2],
					['H10', 847, 120, 1],
					['H10', 848, 120, 2],
					['H10', 849, 121, 1],
					['H10', 850, 121, 2],
					['H10', 851, 122, 2],
					['H10', 852, 122, 3],
					['H10', 853, 122, 1],
					['H10', 854, 122, 4],
					['H10', 855, 123, 1],
					['H10', 856, 123, 2],
					['H10', 857, 123, 3],
					['H10', 858, 123, 4],
					['H10', 859, 123, 5],
					['H10', 860, 124, 1],
					['H10', 861, 124, 2],
					['H10', 862, 124, 3],
					['H10', 863, 124, 4],
					['H10', 864, 124, 5],
					['H10', 865, 125, 1],
					['H10', 866, 125, 2],
					['H10', 867, 125, 3],
					['H10', 868, 125, 4],
					['H10', 869, 125, 5],
					['H10', 870, 125, 6],
					['H10', 871, 125, 7],
					['H10', 872, 126, 1],
					['H10', 873, 126, 2],
					['H10', 874, 126, 3],
					['H10', 875, 126, 4],
					['H10', 876, 126, 5],
					['H10', 877, 126, 6],
					['H10', 878, 126, 7],
					['H10', 879, 126, 8],
					['H10', 880, 126, 9],
					['H10', 881, 127, 1],
					['H10', 882, 127, 2],
					['H10', 883, 127, 3],
					['H10', 884, 127, 4],
					['H10', 885, 127, 5],
					['H10', 886, 128, 1],
					['H10', 887, 128, 2],
					['H10', 888, 128, 3],
					['H10', 889, 128, 4],
					['H10', 890, 128, 5],
					['H10', 891, 128, 6],
					['H10', 892, 128, 7],
					['H10', 893, 128, 8],
					['H10', 894, 128, 9],
					['H10', 895, 128, 10],
					['H10', 896, 128, 11],
					['H10', 897, 128, 12],
					['H10', 898, 128, 13],
					['H10', 899, 128, 14],
					['H10', 900, 128, 15],
					['H10', 901, 128, 16],
					['H10', 902, 129, 1],
					['H10', 903, 129, 2],
					['H10', 904, 129, 3],
					['H10', 905, 129, 4],
					['H10', 906, 129, 5],
					['H10', 907, 129, 6],
					['H10', 908, 130, 1],
					['H10', 909, 130, 2],
					['H14', 1, 1, 0],
					['H14', 2, 1, 1],
					['H14', 3, 1, 2],
					['H14', 4, 1, 3],
					['H14', 5, 1, 4],
					['H14', 6, 1, 5],
					['H14', 7, 1, 6],
					['H14', 8, 1, 7],
					['H14', 9, 1, 8],
					['H14', 10, 1, 9],
					['H14', 11, 1, 10],
					['H14', 12, 2, 0],
					['H14', 13, 2, 1],
					['H14', 14, 3, 0],
					['H14', 15, 3, 1],
					['H14', 16, 3, 2],
					['H14', 17, 3, 3],
					['H14', 18, 3, 4],
					['H14', 19, 3, 5],
					['H14', 20, 4, 0],
					['H14', 21, 4, 1],
					['H14', 22, 4, 2],
					['H14', 23, 4, 3],
					['H14', 24, 4, 4],
					['H14', 25, 4, 5],
					['H14', 26, 4, 6],
					['H14', 27, 4, 7],
					['H14', 28, 4, 8],
					['H14', 29, 4, 9],
					['H14', 30, 4, 10],
					['H14', 31, 4, 11],
					['H14', 32, 4, 12],
					['H14', 39, 6, 1],
					['H14', 40, 6, 2],
					['H14', 41, 7, 0],
					['H14', 42, 7, 1],
					['H14', 43, 7, 2],
					['H14', 44, 7, 3],
					['H14', 45, 7, 4],
					['H14', 46, 8, 0],
					['H14', 47, 9, 0],
					['H14', 48, 9, 1],
					['H14', 49, 9, 2],
					['H14', 50, 9, 3],
					['H14', 51, 10, 0],
					['H14', 52, 10, 1],
					['H14', 53, 10, 2],
					['H14', 54, 10, 3],
					['H14', 55, 10, 4],
					['H14', 56, 10, 5],
					['H14', 57, 10, 6],
					['H14', 58, 10, 7],
					['H14', 59, 10, 8],
					['H14', 60, 11, 0],
					['H14', 61, 11, 1],
					['H14', 98, 13, 0],
					['H14', 99, 13, 1],
					['H14', 101, 13, 3],
					['H14', 106, 13, 8],
					['H14', 107, 13, 9],
					['H14', 108, 13, 10],
					['H14', 109, 13, 11],
					['H14', 110, 13, 12],
					['H14', 111, 14, 0],
					['H14', 123, 15, 0],
					['H14', 124, 15, 1],
					['H14', 125, 15, 2],
					['H14', 126, 15, 3],
					['H14', 160, 22, 0],
					['H14', 161, 22, 1],
					['H14', 162, 22, 2],
					['H14', 163, 22, 3],
					['H14', 164, 22, 4],
					['H14', 165, 22, 5],
					['H14', 170, 24, 0],
					['H14', 171, 24, 1],
					['H14', 172, 24, 2],
					['H14', 173, 25, 0],
					['H14', 174, 25, 1],
					['H14', 175, 25, 2],
					['H14', 176, 25, 3],
					['H14', 177, 25, 4],
					['H14', 178, 25, 5],
					['H14', 182, 27, 0],
					['H14', 183, 27, 1],
					['H14', 184, 27, 2],
					['H14', 185, 27, 3],
					['H14', 186, 28, 0],
					['H14', 187, 28, 1],
					['H14', 188, 28, 2],
					['H14', 189, 28, 3],
					['H14', 190, 29, 0],
					['H14', 191, 29, 1],
					['H14', 192, 29, 2],
					['H14', 193, 29, 3],
					['H14', 194, 30, 0],
					['H14', 195, 30, 1],
					['H14', 196, 30, 2],
					['H14', 197, 30, 3],
					['H14', 198, 30, 4],
					['H14', 210, 31, 1],
					['H14', 211, 31, 2],
					['H14', 212, 31, 3],
					['H14', 213, 31, 4],
					['H14', 214, 32, 1],
					['H14', 215, 32, 2],
					['H14', 216, 32, 3],
					['H14', 217, 32, 4],
					['H14', 218, 32, 5],
					['H14', 219, 32, 6],
					['H14', 220, 33, 1],
					['H14', 221, 33, 2],
					['H14', 222, 33, 3],
					['H14', 223, 34, 1],
					['H14', 224, 34, 2],
					['H14', 225, 34, 3],
					['H14', 226, 35, 1],
					['H14', 227, 35, 2],
					['H14', 228, 35, 3],
					['H14', 237, 38, 1],
					['H14', 238, 38, 2],
					['H14', 239, 38, 3],
					['H14', 240, 38, 4],
					['H14', 241, 39, 1],
					['H14', 242, 39, 2],
					['H14', 243, 39, 3],
					['H14', 244, 39, 4],
					['H14', 245, 40, 1],
					['H14', 246, 40, 2],
					['H14', 247, 40, 3],
					['H14', 248, 40, 4],
					['H14', 249, 41, 1],
					['H14', 250, 41, 2],
					['H14', 251, 41, 3],
					['H14', 252, 41, 4],
					['H14', 253, 41, 5],
					['H14', 254, 41, 6],
					['H14', 255, 41, 7],
					['H14', 256, 41, 8],
					['H14', 257, 41, 9],
					['H14', 258, 41, 10],
					['H14', 259, 41, 11],
					['H14', 260, 42, 1],
					['H14', 261, 42, 2],
					['H14', 262, 42, 3],
					['H14', 263, 42, 4],
					['H14', 264, 42, 5],
					['H14', 265, 42, 6],
					['H14', 266, 42, 7],
					['H14', 267, 42, 8],
					['H14', 268, 42, 9],
					['H14', 269, 43, 1],
					['H14', 270, 43, 2],
					['H14', 271, 43, 3],
					['H14', 272, 43, 4],
					['H14', 273, 44, 1],
					['H14', 274, 44, 2],
					['H14', 275, 44, 3],
					['H14', 276, 44, 4],
					['H14', 277, 45, 1],
					['H14', 278, 45, 2],
					['H14', 279, 45, 3],
					['H14', 280, 45, 4],
					['H14', 281, 45, 5],
					['H14', 282, 45, 6],
					['H14', 283, 45, 7],
					['H14', 284, 45, 8],
					['H14', 285, 45, 9],
					['H14', 286, 45, 10],
					['H14', 287, 45, 11],
					['H14', 289, 46, 1],
					['H14', 290, 46, 2],
					['H14', 291, 46, 3],
					['H14', 292, 46, 4],
					['H14', 293, 46, 5],
					['H14', 294, 46, 6],
					['H14', 295, 47, 1],
					['H14', 296, 47, 2],
					['H14', 297, 48, 1],
					['H14', 298, 48, 2],
					['H14', 299, 48, 3],
					['H14', 300, 48, 4],
					['H14', 301, 48, 5],
					['H14', 302, 48, 6],
					['H14', 303, 48, 7],
					['H14', 304, 48, 8],
					['H14', 305, 48, 9],
					['H14', 306, 48, 10],
					['H14', 307, 48, 11],
					['H14', 308, 48, 12],
					['H14', 309, 48, 13],
					['H14', 310, 48, 14],
					['H14', 311, 48, 15],
					['H14', 312, 48, 16],
					['H14', 313, 48, 17],
					['H14', 314, 48, 18],
					['H14', 315, 48, 19],
					['H14', 316, 48, 20],
					['H14', 317, 48, 21],
					['H14', 318, 48, 22],
					['H14', 319, 48, 23],
					['H14', 320, 48, 24],
					['H14', 323, 49, 2],
					['H14', 324, 49, 3],
					['H14', 325, 50, 1],
					['H14', 326, 50, 2],
					['H14', 387, 29, null],
					['H14', 388, 57, 1],
					['H14', 389, 57, 2],
					['H14', 393, 58, 1],
					['H14', 394, 58, 2],
					['H14', 395, 58, 3],
					['H14', 396, 58, 4],
					['H14', 397, 58, 5],
					['H14', 398, 58, 6],
					['H14', 399, 58, 7],
					['H14', 400, 58, 8],
					['H14', 401, 59, 1],
					['H14', 402, 59, 2],
					['H14', 403, 60, 1],
					['H14', 404, 60, 2],
					['H14', 416, 63, 1],
					['H14', 417, 63, 2],
					['H14', 418, 63, 3],
					['H14', 419, 64, 1],
					['H14', 420, 64, 2],
					['H14', 422, 65, 1],
					['H14', 423, 65, 2],
					['H14', 424, 65, 3],
					['H14', 425, 66, 1],
					['H14', 426, 66, 2],
					['H14', 427, 66, 3],
					['H14', 440, 68, 1],
					['H14', 444, 68, 2],
					['H14', 445, 68, 3],
					['H14', 446, 14, 12],
					['H14', 451, 14, 17],
					['H14', 452, 14, 18],
					['H14', 453, 14, 19],
					['H14', 455, 11, 2],
					['H14', 456, 11, 3],
					['H14', 457, 11, 4],
					['H14', 458, 11, 5],
					['H14', 459, 11, 6],
					['H14', 460, 11, 7],
					['H14', 461, 11, 8],
					['H14', 462, 11, 9],
					['H14', 463, 11, 10],
					['H14', 464, 11, 11],
					['H14', 465, 11, 12],
					['H14', 508, 37, 3],
					['H14', 509, 37, 4],
					['H14', 510, 37, 5],
					['H14', 511, 37, 6],
					['H14', 557, 29, 4],
					['H14', 559, 70, 1],
					['H14', 560, 70, 2],
					['H14', 564, 71, 1],
					['H14', 565, 71, 2],
					['H14', 576, 74, 1],
					['H14', 577, 74, 2],
					['H14', 578, 74, 3],
					['H14', 579, 74, 4],
					['H14', 580, 74, 5],
					['H14', 581, 74, 6],
					['H14', 582, 74, 7],
					['H14', 583, 74, 8],
					['H14', 584, 74, 9],
					['H14', 585, 74, 10],
					['H14', 586, 74, 11],
					['H14', 587, 75, 1],
					['H14', 588, 75, 2],
					['H14', 589, 75, 3],
					['H14', 590, 76, 1],
					['H14', 591, 76, 2],
					['H14', 592, 76, 3],
					['H14', 593, 77, 1],
					['H14', 594, 77, 2],
					['H14', 595, 77, 3],
					['H14', 596, 77, 4],
					['H14', 597, 77, 5],
					['H14', 598, 77, 6],
					['H14', 599, 77, 7],
					['H14', 600, 77, 8],
					['H14', 601, 77, 9],
					['H14', 602, 77, 10],
					['H14', 603, 77, 11],
					['H14', 604, 77, 12],
					['H14', 605, 77, 13],
					['H14', 606, 77, 14],
					['H14', 607, 77, 15],
					['H14', 608, 78, 1],
					['H14', 609, 78, 2],
					['H14', 610, 78, 3],
					['H14', 611, 78, 4],
					['H14', 612, 78, 5],
					['H14', 613, 78, 6],
					['H14', 614, 78, 7],
					['H14', 615, 78, 8],
					['H14', 616, 78, 9],
					['H14', 617, 78, 10],
					['H14', 618, 78, 11],
					['H14', 619, 78, 12],
					['H14', 620, 78, 13],
					['H14', 621, 78, 14],
					['H14', 622, 78, 15],
					['H14', 659, 85, 1],
					['H14', 660, 85, 2],
					['H14', 661, 85, 3],
					['H14', 666, 86, 1],
					['H14', 667, 86, 2],
					['H14', 668, 87, 1],
					['H14', 669, 88, 1],
					['H14', 670, 89, 1],
					['H14', 671, 88, 2],
					['H14', 672, 37, 7],
					['H14', 675, 90, 1],
					['H14', 676, 90, 2],
					['H14', 677, 90, 3],
					['H14', 678, 90, 4],
					['H14', 679, 90, 5],
					['H14', 680, 90, 6],
					['H14', 681, 90, 7],
					['H14', 682, 91, 1],
					['H14', 683, 91, 2],
					['H14', 684, 91, 3],
					['H14', 685, 91, 4],
					['H14', 686, 91, 5],
					['H14', 687, 91, 6],
					['H14', 688, 91, 7],
					['H14', 689, 63, 4],
					['H14', 690, 63, 5],
					['H14', 691, 92, 1],
					['H14', 692, 93, 1],
					['H14', 693, 92, 2],
					['H14', 694, 29, 5],
					['H14', 695, 29, 6],
					['H14', 696, 29, 7],
					['H14', 697, 94, 1],
					['H14', 698, 94, 2],
					['H14', 699, 94, 3],
					['H14', 700, 57, 3],
					['H14', 701, 90, 8],
					['H14', 702, 91, 8],
					['H14', 703, 95, 1],
					['H14', 704, 95, 2],
					['H14', 705, 96, 1],
					['H14', 706, 96, 2],
					['H14', 707, 96, 3],
					['H14', 708, 96, 4],
					['H14', 709, 96, 5],
					['H14', 710, 96, 6],
					['H14', 711, 96, 7],
					['H14', 712, 96, 8],
					['H14', 713, 86, 3],
					['H14', 714, 97, 1],
					['H14', 715, 97, 2],
					['H14', 716, 97, 3],
					['H14', 717, 98, 1],
					['H14', 718, 98, 2],
					['H14', 719, 98, 3],
					['H14', 720, 14, 20],
					['H14', 721, 14, 21],
					['H14', 722, 14, 22],
					['H14', 723, 99, 1],
					['H14', 724, 99, 2],
					['H14', 725, 99, 3],
					['H14', 726, 99, 4],
					['H14', 727, 99, 5],
					['H14', 728, 100, 1],
					['H14', 729, 100, 2],
					['H14', 730, 100, 3],
					['H14', 731, 100, 4],
					['H14', 732, 100, 5],
					['H14', 733, 100, 6],
					['H14', 734, 100, 7],
					['H14', 735, 100, 8],
					['H14', 736, 100, 9],
					['H14', 738, 102, 1],
					['H14', 739, 102, 2],
					['H14', 740, 102, 3],
					['H14', 741, 102, 4],
					['H14', 742, 102, 5],
					['H14', 743, 102, 6],
					['H14', 744, 102, 7],
					['H14', 745, 102, 8],
					['H14', 746, 103, 1],
					['H14', 747, 103, 2],
					['H14', 748, 103, 3],
					['H14', 749, 104, 1],
					['H14', 750, 104, 2],
					['H14', 751, 104, 3],
					['H14', 752, 104, 4],
					['H14', 753, 104, 5],
					['H14', 754, 104, 6],
					['H14', 755, 104, 7],
					['H14', 756, 104, 8],
					['H14', 757, 105, 1],
					['H14', 758, 105, 2],
					['H14', 759, 105, 3],
					['H14', 760, 106, 1],
					['H14', 761, 106, 2],
					['H14', 762, 106, 3],
					['H14', 763, 106, 4],
					['H14', 766, 87, 2],
					['H14', 767, 87, 3],
					['H14', 768, 87, 4],
					['H14', 769, 87, 5],
					['H14', 770, 87, 6],
					['H14', 771, 87, 7],
					['H14', 772, 87, 8],
					['H14', 773, 107, 1],
					['H14', 774, 107, 2],
					['H14', 775, 107, 3],
					['H14', 776, 107, 4],
					['H14', 777, 107, 5],
					['H14', 778, 108, 1],
					['H14', 779, 108, 2],
					['H14', 780, 108, 3],
					['H14', 781, 108, 4],
					['H14', 782, 108, 5],
					['H14', 783, 108, 6],
					['H14', 784, 109, 1],
					['H14', 785, 109, 2],
					['H14', 786, 109, 3],
					['H14', 787, 109, 4],
					['H14', 788, 109, 5],
					['H14', 789, 109, 6],
					['H14', 790, 110, 1],
					['H14', 791, 110, 2],
					['H14', 792, 110, 3],
					['H14', 793, 110, 4],
					['H14', 794, 110, 5],
					['H14', 795, 110, 6],
					['H14', 796, 111, 1],
					['H14', 797, 111, 2],
					['H14', 798, 111, 3],
					['H14', 799, 111, 4],
					['H14', 800, 111, 5],
					['H14', 801, 111, 6],
					['H14', 802, 112, 1],
					['H14', 803, 112, 2],
					['H14', 804, 112, 3],
					['H14', 805, 112, 4],
					['H14', 806, 112, 5],
					['H14', 807, 112, 6],
					['H14', 808, 113, 1],
					['H14', 809, 113, 2],
					['H14', 810, 113, 3],
					['H14', 811, 113, 4],
					['H14', 812, 113, 5],
					['H14', 813, 113, 6],
					['H14', 814, 114, 1],
					['H14', 815, 114, 2],
					['H14', 816, 114, 3],
					['H14', 817, 114, 4],
					['H14', 818, 114, 5],
					['H14', 819, 114, 6],
					['H14', 820, 115, 1],
					['H14', 821, 115, 2],
					['H14', 822, 115, 3],
					['H14', 823, 115, 4],
					['H14', 824, 115, 5],
					['H14', 825, 115, 6],
					['H14', 826, 116, 1],
					['H14', 827, 116, 2],
					['H14', 828, 116, 3],
					['H14', 829, 116, 4],
					['H14', 830, 116, 5],
					['H14', 831, 116, 6],
					['H14', 832, 117, 1],
					['H14', 833, 117, 2],
					['H14', 834, 117, 3],
					['H14', 835, 117, 4],
					['H14', 836, 117, 5],
					['H14', 837, 117, 6],
					['H14', 838, 118, 1],
					['H14', 839, 118, 2],
					['H14', 840, 118, 3],
					['H14', 841, 118, 4],
					['H14', 842, 118, 5],
					['H14', 843, 118, 6],
					['H14', 844, 118, 7],
					['H14', 845, 119, 1],
					['H14', 846, 119, 2],
					['H14', 847, 120, 1],
					['H14', 848, 120, 2],
					['H14', 849, 121, 1],
					['H14', 850, 121, 2],
					['H14', 851, 122, 2],
					['H14', 852, 122, 3],
					['H14', 853, 122, 1],
					['H14', 854, 122, 4],
					['H14', 855, 123, 1],
					['H14', 856, 123, 2],
					['H14', 857, 123, 3],
					['H14', 858, 123, 4],
					['H14', 859, 123, 5],
					['H14', 860, 124, 1],
					['H14', 861, 124, 2],
					['H14', 862, 124, 3],
					['H14', 863, 124, 4],
					['H14', 864, 124, 5],
					['H14', 865, 125, 1],
					['H14', 866, 125, 2],
					['H14', 867, 125, 3],
					['H14', 868, 125, 4],
					['H14', 869, 125, 5],
					['H14', 870, 125, 6],
					['H14', 871, 125, 7],
					['H14', 872, 126, 1],
					['H14', 873, 126, 2],
					['H14', 874, 126, 3],
					['H14', 875, 126, 4],
					['H14', 876, 126, 5],
					['H14', 877, 126, 6],
					['H14', 878, 126, 7],
					['H14', 879, 126, 8],
					['H14', 880, 126, 9],
					['H14', 881, 127, 1],
					['H14', 882, 127, 2],
					['H14', 883, 127, 3],
					['H14', 884, 127, 4],
					['H14', 885, 127, 5],
					['H14', 886, 128, 1],
					['H14', 887, 128, 2],
					['H14', 888, 128, 3],
					['H14', 889, 128, 4],
					['H14', 890, 128, 5],
					['H14', 891, 128, 6],
					['H14', 892, 128, 7],
					['H14', 893, 128, 8],
					['H14', 894, 128, 9],
					['H14', 895, 128, 10],
					['H14', 896, 128, 11],
					['H14', 897, 128, 12],
					['H14', 898, 128, 13],
					['H14', 899, 128, 14],
					['H14', 900, 128, 15],
					['H14', 901, 128, 16],
					['H14', 902, 129, 1],
					['H14', 903, 129, 2],
					['H14', 904, 129, 3],
					['H14', 905, 129, 4],
					['H14', 906, 129, 5],
					['H14', 907, 129, 6],
					['H14', 908, 130, 1],
					['H14', 909, 130, 2],
					['H18', 1, 1, 0],
					['H18', 2, 1, 1],
					['H18', 3, 1, 2],
					['H18', 4, 1, 3],
					['H18', 5, 1, 4],
					['H18', 6, 1, 5],
					['H18', 7, 1, 6],
					['H18', 8, 1, 7],
					['H18', 9, 1, 8],
					['H18', 10, 1, 9],
					['H18', 11, 1, 10],
					['H18', 12, 2, 0],
					['H18', 13, 2, 1],
					['H18', 14, 3, 0],
					['H18', 15, 3, 1],
					['H18', 16, 3, 2],
					['H18', 17, 3, 3],
					['H18', 18, 3, 4],
					['H18', 19, 3, 5],
					['H18', 20, 4, 0],
					['H18', 21, 4, 1],
					['H18', 22, 4, 2],
					['H18', 23, 4, 3],
					['H18', 24, 4, 4],
					['H18', 25, 4, 5],
					['H18', 26, 4, 6],
					['H18', 27, 4, 7],
					['H18', 28, 4, 8],
					['H18', 29, 4, 9],
					['H18', 30, 4, 10],
					['H18', 31, 4, 11],
					['H18', 32, 4, 12],
					['H18', 39, 6, 1],
					['H18', 40, 6, 2],
					['H18', 41, 7, 0],
					['H18', 42, 7, 1],
					['H18', 43, 7, 2],
					['H18', 44, 7, 3],
					['H18', 45, 7, 4],
					['H18', 46, 8, 0],
					['H18', 47, 9, 0],
					['H18', 48, 9, 1],
					['H18', 49, 9, 2],
					['H18', 50, 9, 3],
					['H18', 51, 10, 0],
					['H18', 52, 10, 1],
					['H18', 53, 10, 2],
					['H18', 54, 10, 3],
					['H18', 55, 10, 4],
					['H18', 56, 10, 5],
					['H18', 57, 10, 6],
					['H18', 58, 10, 7],
					['H18', 59, 10, 8],
					['H18', 60, 11, 0],
					['H18', 61, 11, 1],
					['H18', 98, 13, 0],
					['H18', 99, 13, 1],
					['H18', 101, 13, 3],
					['H18', 106, 13, 8],
					['H18', 107, 13, 9],
					['H18', 108, 13, 10],
					['H18', 109, 13, 11],
					['H18', 110, 13, 12],
					['H18', 111, 14, 0],
					['H18', 123, 15, 0],
					['H18', 124, 15, 1],
					['H18', 125, 15, 2],
					['H18', 126, 15, 3],
					['H18', 160, 22, 0],
					['H18', 161, 22, 1],
					['H18', 162, 22, 2],
					['H18', 163, 22, 3],
					['H18', 164, 22, 4],
					['H18', 165, 22, 5],
					['H18', 170, 24, 0],
					['H18', 171, 24, 1],
					['H18', 172, 24, 2],
					['H18', 173, 25, 0],
					['H18', 174, 25, 1],
					['H18', 175, 25, 2],
					['H18', 176, 25, 3],
					['H18', 177, 25, 4],
					['H18', 178, 25, 5],
					['H18', 182, 27, 0],
					['H18', 183, 27, 1],
					['H18', 184, 27, 2],
					['H18', 185, 27, 3],
					['H18', 186, 28, 0],
					['H18', 187, 28, 1],
					['H18', 188, 28, 2],
					['H18', 189, 28, 3],
					['H18', 190, 29, 0],
					['H18', 191, 29, 1],
					['H18', 192, 29, 2],
					['H18', 193, 29, 3],
					['H18', 194, 30, 0],
					['H18', 195, 30, 1],
					['H18', 196, 30, 2],
					['H18', 197, 30, 3],
					['H18', 198, 30, 4],
					['H18', 210, 31, 1],
					['H18', 211, 31, 2],
					['H18', 212, 31, 3],
					['H18', 213, 31, 4],
					['H18', 214, 32, 1],
					['H18', 215, 32, 2],
					['H18', 216, 32, 3],
					['H18', 217, 32, 4],
					['H18', 218, 32, 5],
					['H18', 219, 32, 6],
					['H18', 220, 33, 1],
					['H18', 221, 33, 2],
					['H18', 222, 33, 3],
					['H18', 223, 34, 1],
					['H18', 224, 34, 2],
					['H18', 225, 34, 3],
					['H18', 226, 35, 1],
					['H18', 227, 35, 2],
					['H18', 228, 35, 3],
					['H18', 237, 38, 1],
					['H18', 238, 38, 2],
					['H18', 239, 38, 3],
					['H18', 240, 38, 4],
					['H18', 241, 39, 1],
					['H18', 242, 39, 2],
					['H18', 243, 39, 3],
					['H18', 244, 39, 4],
					['H18', 245, 40, 1],
					['H18', 246, 40, 2],
					['H18', 247, 40, 3],
					['H18', 248, 40, 4],
					['H18', 249, 41, 1],
					['H18', 250, 41, 2],
					['H18', 251, 41, 3],
					['H18', 252, 41, 4],
					['H18', 253, 41, 5],
					['H18', 254, 41, 6],
					['H18', 255, 41, 7],
					['H18', 256, 41, 8],
					['H18', 257, 41, 9],
					['H18', 258, 41, 10],
					['H18', 259, 41, 11],
					['H18', 260, 42, 1],
					['H18', 261, 42, 2],
					['H18', 262, 42, 3],
					['H18', 263, 42, 4],
					['H18', 264, 42, 5],
					['H18', 265, 42, 6],
					['H18', 266, 42, 7],
					['H18', 267, 42, 8],
					['H18', 268, 42, 9],
					['H18', 269, 43, 1],
					['H18', 270, 43, 2],
					['H18', 271, 43, 3],
					['H18', 272, 43, 4],
					['H18', 273, 44, 1],
					['H18', 274, 44, 2],
					['H18', 275, 44, 3],
					['H18', 276, 44, 4],
					['H18', 277, 45, 1],
					['H18', 278, 45, 2],
					['H18', 279, 45, 3],
					['H18', 280, 45, 4],
					['H18', 281, 45, 5],
					['H18', 282, 45, 6],
					['H18', 283, 45, 7],
					['H18', 284, 45, 8],
					['H18', 285, 45, 9],
					['H18', 286, 45, 10],
					['H18', 287, 45, 11],
					['H18', 289, 46, 1],
					['H18', 290, 46, 2],
					['H18', 291, 46, 3],
					['H18', 292, 46, 4],
					['H18', 293, 46, 5],
					['H18', 294, 46, 6],
					['H18', 295, 47, 1],
					['H18', 296, 47, 2],
					['H18', 297, 48, 1],
					['H18', 298, 48, 2],
					['H18', 299, 48, 3],
					['H18', 300, 48, 4],
					['H18', 301, 48, 5],
					['H18', 302, 48, 6],
					['H18', 303, 48, 7],
					['H18', 304, 48, 8],
					['H18', 305, 48, 9],
					['H18', 306, 48, 10],
					['H18', 307, 48, 11],
					['H18', 308, 48, 12],
					['H18', 309, 48, 13],
					['H18', 310, 48, 14],
					['H18', 311, 48, 15],
					['H18', 312, 48, 16],
					['H18', 313, 48, 17],
					['H18', 314, 48, 18],
					['H18', 315, 48, 19],
					['H18', 316, 48, 20],
					['H18', 317, 48, 21],
					['H18', 318, 48, 22],
					['H18', 319, 48, 23],
					['H18', 320, 48, 24],
					['H18', 323, 49, 2],
					['H18', 324, 49, 3],
					['H18', 325, 50, 1],
					['H18', 326, 50, 2],
					['H18', 387, 29, null],
					['H18', 388, 57, 1],
					['H18', 389, 57, 2],
					['H18', 393, 58, 1],
					['H18', 394, 58, 2],
					['H18', 395, 58, 3],
					['H18', 396, 58, 4],
					['H18', 397, 58, 5],
					['H18', 398, 58, 6],
					['H18', 399, 58, 7],
					['H18', 400, 58, 8],
					['H18', 401, 59, 1],
					['H18', 402, 59, 2],
					['H18', 403, 60, 1],
					['H18', 404, 60, 2],
					['H18', 416, 63, 1],
					['H18', 417, 63, 2],
					['H18', 418, 63, 3],
					['H18', 419, 64, 1],
					['H18', 420, 64, 2],
					['H18', 422, 65, 1],
					['H18', 423, 65, 2],
					['H18', 424, 65, 3],
					['H18', 425, 66, 1],
					['H18', 426, 66, 2],
					['H18', 427, 66, 3],
					['H18', 440, 68, 1],
					['H18', 444, 68, 2],
					['H18', 445, 68, 3],
					['H18', 446, 14, 12],
					['H18', 451, 14, 17],
					['H18', 452, 14, 18],
					['H18', 453, 14, 19],
					['H18', 455, 11, 2],
					['H18', 456, 11, 3],
					['H18', 457, 11, 4],
					['H18', 458, 11, 5],
					['H18', 459, 11, 6],
					['H18', 460, 11, 7],
					['H18', 461, 11, 8],
					['H18', 462, 11, 9],
					['H18', 463, 11, 10],
					['H18', 464, 11, 11],
					['H18', 465, 11, 12],
					['H18', 508, 37, 3],
					['H18', 509, 37, 4],
					['H18', 510, 37, 5],
					['H18', 511, 37, 6],
					['H18', 557, 29, 4],
					['H18', 559, 70, 1],
					['H18', 560, 70, 2],
					['H18', 564, 71, 1],
					['H18', 565, 71, 2],
					['H18', 576, 74, 1],
					['H18', 577, 74, 2],
					['H18', 578, 74, 3],
					['H18', 579, 74, 4],
					['H18', 580, 74, 5],
					['H18', 581, 74, 6],
					['H18', 582, 74, 7],
					['H18', 583, 74, 8],
					['H18', 584, 74, 9],
					['H18', 585, 74, 10],
					['H18', 586, 74, 11],
					['H18', 587, 75, 1],
					['H18', 588, 75, 2],
					['H18', 589, 75, 3],
					['H18', 590, 76, 1],
					['H18', 591, 76, 2],
					['H18', 592, 76, 3],
					['H18', 593, 77, 1],
					['H18', 594, 77, 2],
					['H18', 595, 77, 3],
					['H18', 596, 77, 4],
					['H18', 597, 77, 5],
					['H18', 598, 77, 6],
					['H18', 599, 77, 7],
					['H18', 600, 77, 8],
					['H18', 601, 77, 9],
					['H18', 602, 77, 10],
					['H18', 603, 77, 11],
					['H18', 604, 77, 12],
					['H18', 605, 77, 13],
					['H18', 606, 77, 14],
					['H18', 607, 77, 15],
					['H18', 608, 78, 1],
					['H18', 609, 78, 2],
					['H18', 610, 78, 3],
					['H18', 611, 78, 4],
					['H18', 612, 78, 5],
					['H18', 613, 78, 6],
					['H18', 614, 78, 7],
					['H18', 615, 78, 8],
					['H18', 616, 78, 9],
					['H18', 617, 78, 10],
					['H18', 618, 78, 11],
					['H18', 619, 78, 12],
					['H18', 620, 78, 13],
					['H18', 621, 78, 14],
					['H18', 622, 78, 15],
					['H18', 659, 85, 1],
					['H18', 660, 85, 2],
					['H18', 661, 85, 3],
					['H18', 666, 86, 1],
					['H18', 667, 86, 2],
					['H18', 668, 87, 1],
					['H18', 669, 88, 1],
					['H18', 670, 89, 1],
					['H18', 671, 88, 2],
					['H18', 672, 37, 7],
					['H18', 675, 90, 1],
					['H18', 676, 90, 2],
					['H18', 677, 90, 3],
					['H18', 678, 90, 4],
					['H18', 679, 90, 5],
					['H18', 680, 90, 6],
					['H18', 681, 90, 7],
					['H18', 682, 91, 1],
					['H18', 683, 91, 2],
					['H18', 684, 91, 3],
					['H18', 685, 91, 4],
					['H18', 686, 91, 5],
					['H18', 687, 91, 6],
					['H18', 688, 91, 7],
					['H18', 689, 63, 4],
					['H18', 690, 63, 5],
					['H18', 691, 92, 1],
					['H18', 692, 93, 1],
					['H18', 693, 92, 2],
					['H18', 694, 29, 5],
					['H18', 695, 29, 6],
					['H18', 696, 29, 7],
					['H18', 697, 94, 1],
					['H18', 698, 94, 2],
					['H18', 699, 94, 3],
					['H18', 700, 57, 3],
					['H18', 701, 90, 8],
					['H18', 702, 91, 8],
					['H18', 703, 95, 1],
					['H18', 704, 95, 2],
					['H18', 705, 96, 1],
					['H18', 706, 96, 2],
					['H18', 707, 96, 3],
					['H18', 708, 96, 4],
					['H18', 709, 96, 5],
					['H18', 710, 96, 6],
					['H18', 711, 96, 7],
					['H18', 712, 96, 8],
					['H18', 713, 86, 3],
					['H18', 714, 97, 1],
					['H18', 715, 97, 2],
					['H18', 716, 97, 3],
					['H18', 717, 98, 1],
					['H18', 718, 98, 2],
					['H18', 719, 98, 3],
					['H18', 720, 14, 20],
					['H18', 721, 14, 21],
					['H18', 722, 14, 22],
					['H18', 723, 99, 1],
					['H18', 724, 99, 2],
					['H18', 725, 99, 3],
					['H18', 726, 99, 4],
					['H18', 727, 99, 5],
					['H18', 728, 100, 1],
					['H18', 729, 100, 2],
					['H18', 730, 100, 3],
					['H18', 731, 100, 4],
					['H18', 732, 100, 5],
					['H18', 733, 100, 6],
					['H18', 734, 100, 7],
					['H18', 735, 100, 8],
					['H18', 736, 100, 9],
					['H18', 738, 102, 1],
					['H18', 739, 102, 2],
					['H18', 740, 102, 3],
					['H18', 741, 102, 4],
					['H18', 742, 102, 5],
					['H18', 743, 102, 6],
					['H18', 744, 102, 7],
					['H18', 745, 102, 8],
					['H18', 746, 103, 1],
					['H18', 747, 103, 2],
					['H18', 748, 103, 3],
					['H18', 749, 104, 1],
					['H18', 750, 104, 2],
					['H18', 751, 104, 3],
					['H18', 752, 104, 4],
					['H18', 753, 104, 5],
					['H18', 754, 104, 6],
					['H18', 755, 104, 7],
					['H18', 756, 104, 8],
					['H18', 757, 105, 1],
					['H18', 758, 105, 2],
					['H18', 759, 105, 3],
					['H18', 760, 106, 1],
					['H18', 761, 106, 2],
					['H18', 762, 106, 3],
					['H18', 763, 106, 4],
					['H18', 766, 87, 2],
					['H18', 767, 87, 3],
					['H18', 768, 87, 4],
					['H18', 769, 87, 5],
					['H18', 770, 87, 6],
					['H18', 771, 87, 7],
					['H18', 772, 87, 8],
					['H18', 773, 107, 1],
					['H18', 774, 107, 2],
					['H18', 775, 107, 3],
					['H18', 776, 107, 4],
					['H18', 777, 107, 5],
					['H18', 778, 108, 1],
					['H18', 779, 108, 2],
					['H18', 780, 108, 3],
					['H18', 781, 108, 4],
					['H18', 782, 108, 5],
					['H18', 783, 108, 6],
					['H18', 784, 109, 1],
					['H18', 785, 109, 2],
					['H18', 786, 109, 3],
					['H18', 787, 109, 4],
					['H18', 788, 109, 5],
					['H18', 789, 109, 6],
					['H18', 790, 110, 1],
					['H18', 791, 110, 2],
					['H18', 792, 110, 3],
					['H18', 793, 110, 4],
					['H18', 794, 110, 5],
					['H18', 795, 110, 6],
					['H18', 796, 111, 1],
					['H18', 797, 111, 2],
					['H18', 798, 111, 3],
					['H18', 799, 111, 4],
					['H18', 800, 111, 5],
					['H18', 801, 111, 6],
					['H18', 802, 112, 1],
					['H18', 803, 112, 2],
					['H18', 804, 112, 3],
					['H18', 805, 112, 4],
					['H18', 806, 112, 5],
					['H18', 807, 112, 6],
					['H18', 808, 113, 1],
					['H18', 809, 113, 2],
					['H18', 810, 113, 3],
					['H18', 811, 113, 4],
					['H18', 812, 113, 5],
					['H18', 813, 113, 6],
					['H18', 814, 114, 1],
					['H18', 815, 114, 2],
					['H18', 816, 114, 3],
					['H18', 817, 114, 4],
					['H18', 818, 114, 5],
					['H18', 819, 114, 6],
					['H18', 820, 115, 1],
					['H18', 821, 115, 2],
					['H18', 822, 115, 3],
					['H18', 823, 115, 4],
					['H18', 824, 115, 5],
					['H18', 825, 115, 6],
					['H18', 826, 116, 1],
					['H18', 827, 116, 2],
					['H18', 828, 116, 3],
					['H18', 829, 116, 4],
					['H18', 830, 116, 5],
					['H18', 831, 116, 6],
					['H18', 832, 117, 1],
					['H18', 833, 117, 2],
					['H18', 834, 117, 3],
					['H18', 835, 117, 4],
					['H18', 836, 117, 5],
					['H18', 837, 117, 6],
					['H18', 838, 118, 1],
					['H18', 839, 118, 2],
					['H18', 840, 118, 3],
					['H18', 841, 118, 4],
					['H18', 842, 118, 5],
					['H18', 843, 118, 6],
					['H18', 844, 118, 7],
					['H18', 845, 119, 1],
					['H18', 846, 119, 2],
					['H18', 847, 120, 1],
					['H18', 848, 120, 2],
					['H18', 849, 121, 1],
					['H18', 850, 121, 2],
					['H18', 851, 122, 2],
					['H18', 852, 122, 3],
					['H18', 853, 122, 1],
					['H18', 854, 122, 4],
					['H18', 855, 123, 1],
					['H18', 856, 123, 2],
					['H18', 857, 123, 3],
					['H18', 858, 123, 4],
					['H18', 859, 123, 5],
					['H18', 860, 124, 1],
					['H18', 861, 124, 2],
					['H18', 862, 124, 3],
					['H18', 863, 124, 4],
					['H18', 864, 124, 5],
					['H18', 865, 125, 1],
					['H18', 866, 125, 2],
					['H18', 867, 125, 3],
					['H18', 868, 125, 4],
					['H18', 869, 125, 5],
					['H18', 870, 125, 6],
					['H18', 871, 125, 7],
					['H18', 872, 126, 1],
					['H18', 873, 126, 2],
					['H18', 874, 126, 3],
					['H18', 875, 126, 4],
					['H18', 876, 126, 5],
					['H18', 877, 126, 6],
					['H18', 878, 126, 7],
					['H18', 879, 126, 8],
					['H18', 880, 126, 9],
					['H18', 881, 127, 1],
					['H18', 882, 127, 2],
					['H18', 883, 127, 3],
					['H18', 884, 127, 4],
					['H18', 885, 127, 5],
					['H18', 886, 128, 1],
					['H18', 887, 128, 2],
					['H18', 888, 128, 3],
					['H18', 889, 128, 4],
					['H18', 890, 128, 5],
					['H18', 891, 128, 6],
					['H18', 892, 128, 7],
					['H18', 893, 128, 8],
					['H18', 894, 128, 9],
					['H18', 895, 128, 10],
					['H18', 896, 128, 11],
					['H18', 897, 128, 12],
					['H18', 898, 128, 13],
					['H18', 899, 128, 14],
					['H18', 900, 128, 15],
					['H18', 901, 128, 16],
					['H18', 902, 129, 1],
					['H18', 903, 129, 2],
					['H18', 904, 129, 3],
					['H18', 905, 129, 4],
					['H18', 906, 129, 5],
					['H18', 907, 129, 6],
					['H18', 908, 130, 1],
					['H18', 909, 130, 2],
					['H2', 1, 1, 0],
					['H2', 2, 1, 1],
					['H2', 3, 1, 2],
					['H2', 4, 1, 3],
					['H2', 5, 1, 4],
					['H2', 6, 1, 5],
					['H2', 7, 1, 6],
					['H2', 8, 1, 7],
					['H2', 9, 1, 8],
					['H2', 10, 1, 9],
					['H2', 11, 1, 10],
					['H2', 12, 2, 0],
					['H2', 13, 2, 1],
					['H2', 14, 3, 0],
					['H2', 15, 3, 1],
					['H2', 16, 3, 2],
					['H2', 17, 3, 3],
					['H2', 18, 3, 4],
					['H2', 19, 3, 5],
					['H2', 20, 4, 0],
					['H2', 21, 4, 1],
					['H2', 22, 4, 2],
					['H2', 23, 4, 3],
					['H2', 24, 4, 4],
					['H2', 25, 4, 5],
					['H2', 26, 4, 6],
					['H2', 27, 4, 7],
					['H2', 28, 4, 8],
					['H2', 29, 4, 9],
					['H2', 30, 4, 10],
					['H2', 31, 4, 11],
					['H2', 32, 4, 12],
					['H2', 39, 6, 1],
					['H2', 40, 6, 2],
					['H2', 41, 7, 0],
					['H2', 42, 7, 1],
					['H2', 43, 7, 2],
					['H2', 44, 7, 3],
					['H2', 45, 7, 4],
					['H2', 46, 8, 0],
					['H2', 47, 9, 0],
					['H2', 48, 9, 1],
					['H2', 49, 9, 2],
					['H2', 50, 9, 3],
					['H2', 51, 10, 0],
					['H2', 52, 10, 1],
					['H2', 53, 10, 2],
					['H2', 54, 10, 3],
					['H2', 55, 10, 4],
					['H2', 56, 10, 5],
					['H2', 57, 10, 6],
					['H2', 58, 10, 7],
					['H2', 59, 10, 8],
					['H2', 60, 11, 0],
					['H2', 61, 11, 1],
					['H2', 98, 13, 0],
					['H2', 99, 13, 1],
					['H2', 101, 13, 3],
					['H2', 106, 13, 8],
					['H2', 107, 13, 9],
					['H2', 108, 13, 10],
					['H2', 109, 13, 11],
					['H2', 110, 13, 12],
					['H2', 111, 14, 0],
					['H2', 123, 15, 0],
					['H2', 124, 15, 1],
					['H2', 125, 15, 2],
					['H2', 126, 15, 3],
					['H2', 160, 22, 0],
					['H2', 161, 22, 1],
					['H2', 162, 22, 2],
					['H2', 163, 22, 3],
					['H2', 164, 22, 4],
					['H2', 165, 22, 5],
					['H2', 170, 24, 0],
					['H2', 171, 24, 1],
					['H2', 172, 24, 2],
					['H2', 173, 25, 0],
					['H2', 174, 25, 1],
					['H2', 175, 25, 2],
					['H2', 176, 25, 3],
					['H2', 177, 25, 4],
					['H2', 178, 25, 5],
					['H2', 182, 27, 0],
					['H2', 183, 27, 1],
					['H2', 184, 27, 2],
					['H2', 185, 27, 3],
					['H2', 186, 28, 0],
					['H2', 187, 28, 1],
					['H2', 188, 28, 2],
					['H2', 189, 28, 3],
					['H2', 190, 29, 0],
					['H2', 191, 29, 1],
					['H2', 192, 29, 2],
					['H2', 193, 29, 3],
					['H2', 194, 30, 0],
					['H2', 195, 30, 1],
					['H2', 196, 30, 2],
					['H2', 197, 30, 3],
					['H2', 198, 30, 4],
					['H2', 210, 31, 1],
					['H2', 211, 31, 2],
					['H2', 212, 31, 3],
					['H2', 213, 31, 4],
					['H2', 214, 32, 1],
					['H2', 215, 32, 2],
					['H2', 216, 32, 3],
					['H2', 217, 32, 4],
					['H2', 218, 32, 5],
					['H2', 219, 32, 6],
					['H2', 220, 33, 1],
					['H2', 221, 33, 2],
					['H2', 222, 33, 3],
					['H2', 223, 34, 1],
					['H2', 224, 34, 2],
					['H2', 225, 34, 3],
					['H2', 226, 35, 1],
					['H2', 227, 35, 2],
					['H2', 228, 35, 3],
					['H2', 237, 38, 1],
					['H2', 238, 38, 2],
					['H2', 239, 38, 3],
					['H2', 240, 38, 4],
					['H2', 241, 39, 1],
					['H2', 242, 39, 2],
					['H2', 243, 39, 3],
					['H2', 244, 39, 4],
					['H2', 245, 40, 1],
					['H2', 246, 40, 2],
					['H2', 247, 40, 3],
					['H2', 248, 40, 4],
					['H2', 249, 41, 1],
					['H2', 250, 41, 2],
					['H2', 251, 41, 3],
					['H2', 252, 41, 4],
					['H2', 253, 41, 5],
					['H2', 254, 41, 6],
					['H2', 255, 41, 7],
					['H2', 256, 41, 8],
					['H2', 257, 41, 9],
					['H2', 258, 41, 10],
					['H2', 259, 41, 11],
					['H2', 260, 42, 1],
					['H2', 261, 42, 2],
					['H2', 262, 42, 3],
					['H2', 263, 42, 4],
					['H2', 264, 42, 5],
					['H2', 265, 42, 6],
					['H2', 266, 42, 7],
					['H2', 267, 42, 8],
					['H2', 268, 42, 9],
					['H2', 269, 43, 1],
					['H2', 270, 43, 2],
					['H2', 271, 43, 3],
					['H2', 272, 43, 4],
					['H2', 273, 44, 1],
					['H2', 274, 44, 2],
					['H2', 275, 44, 3],
					['H2', 276, 44, 4],
					['H2', 277, 45, 1],
					['H2', 278, 45, 2],
					['H2', 279, 45, 3],
					['H2', 280, 45, 4],
					['H2', 281, 45, 5],
					['H2', 282, 45, 6],
					['H2', 283, 45, 7],
					['H2', 284, 45, 8],
					['H2', 285, 45, 9],
					['H2', 286, 45, 10],
					['H2', 287, 45, 11],
					['H2', 289, 46, 1],
					['H2', 290, 46, 2],
					['H2', 291, 46, 3],
					['H2', 292, 46, 4],
					['H2', 293, 46, 5],
					['H2', 294, 46, 6],
					['H2', 295, 47, 1],
					['H2', 296, 47, 2],
					['H2', 297, 48, 1],
					['H2', 298, 48, 2],
					['H2', 299, 48, 3],
					['H2', 300, 48, 4],
					['H2', 301, 48, 5],
					['H2', 302, 48, 6],
					['H2', 303, 48, 7],
					['H2', 304, 48, 8],
					['H2', 305, 48, 9],
					['H2', 306, 48, 10],
					['H2', 307, 48, 11],
					['H2', 308, 48, 12],
					['H2', 309, 48, 13],
					['H2', 310, 48, 14],
					['H2', 311, 48, 15],
					['H2', 312, 48, 16],
					['H2', 313, 48, 17],
					['H2', 314, 48, 18],
					['H2', 315, 48, 19],
					['H2', 316, 48, 20],
					['H2', 317, 48, 21],
					['H2', 318, 48, 22],
					['H2', 319, 48, 23],
					['H2', 320, 48, 24],
					['H2', 323, 49, 2],
					['H2', 324, 49, 3],
					['H2', 325, 50, 1],
					['H2', 326, 50, 2],
					['H2', 387, 29, null],
					['H2', 388, 57, 1],
					['H2', 389, 57, 2],
					['H2', 393, 58, 1],
					['H2', 394, 58, 2],
					['H2', 395, 58, 3],
					['H2', 396, 58, 4],
					['H2', 397, 58, 5],
					['H2', 398, 58, 6],
					['H2', 399, 58, 7],
					['H2', 400, 58, 8],
					['H2', 401, 59, 1],
					['H2', 402, 59, 2],
					['H2', 403, 60, 1],
					['H2', 404, 60, 2],
					['H2', 416, 63, 1],
					['H2', 417, 63, 2],
					['H2', 418, 63, 3],
					['H2', 419, 64, 1],
					['H2', 420, 64, 2],
					['H2', 422, 65, 1],
					['H2', 423, 65, 2],
					['H2', 424, 65, 3],
					['H2', 425, 66, 1],
					['H2', 426, 66, 2],
					['H2', 427, 66, 3],
					['H2', 440, 68, 1],
					['H2', 444, 68, 2],
					['H2', 445, 68, 3],
					['H2', 446, 14, 12],
					['H2', 451, 14, 17],
					['H2', 452, 14, 18],
					['H2', 453, 14, 19],
					['H2', 455, 11, 2],
					['H2', 456, 11, 3],
					['H2', 457, 11, 4],
					['H2', 458, 11, 5],
					['H2', 459, 11, 6],
					['H2', 460, 11, 7],
					['H2', 461, 11, 8],
					['H2', 462, 11, 9],
					['H2', 463, 11, 10],
					['H2', 464, 11, 11],
					['H2', 465, 11, 12],
					['H2', 508, 37, 3],
					['H2', 509, 37, 4],
					['H2', 510, 37, 5],
					['H2', 511, 37, 6],
					['H2', 557, 29, 4],
					['H2', 559, 70, 1],
					['H2', 560, 70, 2],
					['H2', 564, 71, 1],
					['H2', 565, 71, 2],
					['H2', 576, 74, 1],
					['H2', 577, 74, 2],
					['H2', 578, 74, 3],
					['H2', 579, 74, 4],
					['H2', 580, 74, 5],
					['H2', 581, 74, 6],
					['H2', 582, 74, 7],
					['H2', 583, 74, 8],
					['H2', 584, 74, 9],
					['H2', 585, 74, 10],
					['H2', 586, 74, 11],
					['H2', 587, 75, 1],
					['H2', 588, 75, 2],
					['H2', 589, 75, 3],
					['H2', 590, 76, 1],
					['H2', 591, 76, 2],
					['H2', 592, 76, 3],
					['H2', 593, 77, 1],
					['H2', 594, 77, 2],
					['H2', 595, 77, 3],
					['H2', 596, 77, 4],
					['H2', 597, 77, 5],
					['H2', 598, 77, 6],
					['H2', 599, 77, 7],
					['H2', 600, 77, 8],
					['H2', 601, 77, 9],
					['H2', 602, 77, 10],
					['H2', 603, 77, 11],
					['H2', 604, 77, 12],
					['H2', 605, 77, 13],
					['H2', 606, 77, 14],
					['H2', 607, 77, 15],
					['H2', 608, 78, 1],
					['H2', 609, 78, 2],
					['H2', 610, 78, 3],
					['H2', 611, 78, 4],
					['H2', 612, 78, 5],
					['H2', 613, 78, 6],
					['H2', 614, 78, 7],
					['H2', 615, 78, 8],
					['H2', 616, 78, 9],
					['H2', 617, 78, 10],
					['H2', 618, 78, 11],
					['H2', 619, 78, 12],
					['H2', 620, 78, 13],
					['H2', 621, 78, 14],
					['H2', 622, 78, 15],
					['H2', 659, 85, 1],
					['H2', 660, 85, 2],
					['H2', 661, 85, 3],
					['H2', 666, 86, 1],
					['H2', 667, 86, 2],
					['H2', 668, 87, 1],
					['H2', 669, 88, 1],
					['H2', 670, 89, 1],
					['H2', 671, 88, 2],
					['H2', 672, 37, 7],
					['H2', 675, 90, 1],
					['H2', 676, 90, 2],
					['H2', 677, 90, 3],
					['H2', 678, 90, 4],
					['H2', 679, 90, 5],
					['H2', 680, 90, 6],
					['H2', 681, 90, 7],
					['H2', 682, 91, 1],
					['H2', 683, 91, 2],
					['H2', 684, 91, 3],
					['H2', 685, 91, 4],
					['H2', 686, 91, 5],
					['H2', 687, 91, 6],
					['H2', 688, 91, 7],
					['H2', 689, 63, 4],
					['H2', 690, 63, 5],
					['H2', 691, 92, 1],
					['H2', 692, 93, 1],
					['H2', 693, 92, 2],
					['H2', 694, 29, 5],
					['H2', 695, 29, 6],
					['H2', 696, 29, 7],
					['H2', 697, 94, 1],
					['H2', 698, 94, 2],
					['H2', 699, 94, 3],
					['H2', 700, 57, 3],
					['H2', 701, 90, 8],
					['H2', 702, 91, 8],
					['H2', 703, 95, 1],
					['H2', 704, 95, 2],
					['H2', 705, 96, 1],
					['H2', 706, 96, 2],
					['H2', 707, 96, 3],
					['H2', 708, 96, 4],
					['H2', 709, 96, 5],
					['H2', 710, 96, 6],
					['H2', 711, 96, 7],
					['H2', 712, 96, 8],
					['H2', 713, 86, 3],
					['H2', 714, 97, 1],
					['H2', 715, 97, 2],
					['H2', 716, 97, 3],
					['H2', 717, 98, 1],
					['H2', 718, 98, 2],
					['H2', 719, 98, 3],
					['H2', 720, 14, 20],
					['H2', 721, 14, 21],
					['H2', 722, 14, 22],
					['H2', 723, 99, 1],
					['H2', 724, 99, 2],
					['H2', 725, 99, 3],
					['H2', 726, 99, 4],
					['H2', 727, 99, 5],
					['H2', 728, 100, 1],
					['H2', 729, 100, 2],
					['H2', 730, 100, 3],
					['H2', 731, 100, 4],
					['H2', 732, 100, 5],
					['H2', 733, 100, 6],
					['H2', 734, 100, 7],
					['H2', 735, 100, 8],
					['H2', 736, 100, 9],
					['H2', 738, 102, 1],
					['H2', 739, 102, 2],
					['H2', 740, 102, 3],
					['H2', 741, 102, 4],
					['H2', 742, 102, 5],
					['H2', 743, 102, 6],
					['H2', 744, 102, 7],
					['H2', 745, 102, 8],
					['H2', 746, 103, 1],
					['H2', 747, 103, 2],
					['H2', 748, 103, 3],
					['H2', 749, 104, 1],
					['H2', 750, 104, 2],
					['H2', 751, 104, 3],
					['H2', 752, 104, 4],
					['H2', 753, 104, 5],
					['H2', 754, 104, 6],
					['H2', 755, 104, 7],
					['H2', 756, 104, 8],
					['H2', 757, 105, 1],
					['H2', 758, 105, 2],
					['H2', 759, 105, 3],
					['H2', 760, 106, 1],
					['H2', 761, 106, 2],
					['H2', 762, 106, 3],
					['H2', 763, 106, 4],
					['H2', 766, 87, 2],
					['H2', 767, 87, 3],
					['H2', 768, 87, 4],
					['H2', 769, 87, 5],
					['H2', 770, 87, 6],
					['H2', 771, 87, 7],
					['H2', 772, 87, 8],
					['H2', 773, 107, 1],
					['H2', 774, 107, 2],
					['H2', 775, 107, 3],
					['H2', 776, 107, 4],
					['H2', 777, 107, 5],
					['H2', 778, 108, 1],
					['H2', 779, 108, 2],
					['H2', 780, 108, 3],
					['H2', 781, 108, 4],
					['H2', 782, 108, 5],
					['H2', 783, 108, 6],
					['H2', 784, 109, 1],
					['H2', 785, 109, 2],
					['H2', 786, 109, 3],
					['H2', 787, 109, 4],
					['H2', 788, 109, 5],
					['H2', 789, 109, 6],
					['H2', 790, 110, 1],
					['H2', 791, 110, 2],
					['H2', 792, 110, 3],
					['H2', 793, 110, 4],
					['H2', 794, 110, 5],
					['H2', 795, 110, 6],
					['H2', 796, 111, 1],
					['H2', 797, 111, 2],
					['H2', 798, 111, 3],
					['H2', 799, 111, 4],
					['H2', 800, 111, 5],
					['H2', 801, 111, 6],
					['H2', 802, 112, 1],
					['H2', 803, 112, 2],
					['H2', 804, 112, 3],
					['H2', 805, 112, 4],
					['H2', 806, 112, 5],
					['H2', 807, 112, 6],
					['H2', 808, 113, 1],
					['H2', 809, 113, 2],
					['H2', 810, 113, 3],
					['H2', 811, 113, 4],
					['H2', 812, 113, 5],
					['H2', 813, 113, 6],
					['H2', 814, 114, 1],
					['H2', 815, 114, 2],
					['H2', 816, 114, 3],
					['H2', 817, 114, 4],
					['H2', 818, 114, 5],
					['H2', 819, 114, 6],
					['H2', 820, 115, 1],
					['H2', 821, 115, 2],
					['H2', 822, 115, 3],
					['H2', 823, 115, 4],
					['H2', 824, 115, 5],
					['H2', 825, 115, 6],
					['H2', 826, 116, 1],
					['H2', 827, 116, 2],
					['H2', 828, 116, 3],
					['H2', 829, 116, 4],
					['H2', 830, 116, 5],
					['H2', 831, 116, 6],
					['H2', 832, 117, 1],
					['H2', 833, 117, 2],
					['H2', 834, 117, 3],
					['H2', 835, 117, 4],
					['H2', 836, 117, 5],
					['H2', 837, 117, 6],
					['H2', 838, 118, 1],
					['H2', 839, 118, 2],
					['H2', 840, 118, 3],
					['H2', 841, 118, 4],
					['H2', 842, 118, 5],
					['H2', 843, 118, 6],
					['H2', 844, 118, 7],
					['H2', 845, 119, 1],
					['H2', 846, 119, 2],
					['H2', 847, 120, 1],
					['H2', 848, 120, 2],
					['H2', 849, 121, 1],
					['H2', 850, 121, 2],
					['H2', 851, 122, 2],
					['H2', 852, 122, 3],
					['H2', 853, 122, 1],
					['H2', 854, 122, 4],
					['H2', 855, 123, 1],
					['H2', 856, 123, 2],
					['H2', 857, 123, 3],
					['H2', 858, 123, 4],
					['H2', 859, 123, 5],
					['H2', 860, 124, 1],
					['H2', 861, 124, 2],
					['H2', 862, 124, 3],
					['H2', 863, 124, 4],
					['H2', 864, 124, 5],
					['H2', 865, 125, 1],
					['H2', 866, 125, 2],
					['H2', 867, 125, 3],
					['H2', 868, 125, 4],
					['H2', 869, 125, 5],
					['H2', 870, 125, 6],
					['H2', 871, 125, 7],
					['H2', 872, 126, 1],
					['H2', 873, 126, 2],
					['H2', 874, 126, 3],
					['H2', 875, 126, 4],
					['H2', 876, 126, 5],
					['H2', 877, 126, 6],
					['H2', 878, 126, 7],
					['H2', 879, 126, 8],
					['H2', 880, 126, 9],
					['H2', 881, 127, 1],
					['H2', 882, 127, 2],
					['H2', 883, 127, 3],
					['H2', 884, 127, 4],
					['H2', 885, 127, 5],
					['H2', 886, 128, 1],
					['H2', 887, 128, 2],
					['H2', 888, 128, 3],
					['H2', 889, 128, 4],
					['H2', 890, 128, 5],
					['H2', 891, 128, 6],
					['H2', 892, 128, 7],
					['H2', 893, 128, 8],
					['H2', 894, 128, 9],
					['H2', 895, 128, 10],
					['H2', 896, 128, 11],
					['H2', 897, 128, 12],
					['H2', 898, 128, 13],
					['H2', 899, 128, 14],
					['H2', 900, 128, 15],
					['H2', 901, 128, 16],
					['H2', 902, 129, 1],
					['H2', 903, 129, 2],
					['H2', 904, 129, 3],
					['H2', 905, 129, 4],
					['H2', 906, 129, 5],
					['H2', 907, 129, 6],
					['H2', 908, 130, 1],
					['H2', 909, 130, 2],
					['H22', 1, 1, 0],
					['H22', 2, 1, 1],
					['H22', 3, 1, 2],
					['H22', 4, 1, 3],
					['H22', 5, 1, 4],
					['H22', 6, 1, 5],
					['H22', 7, 1, 6],
					['H22', 8, 1, 7],
					['H22', 9, 1, 8],
					['H22', 10, 1, 9],
					['H22', 11, 1, 10],
					['H22', 12, 2, 0],
					['H22', 13, 2, 1],
					['H22', 14, 3, 0],
					['H22', 15, 3, 1],
					['H22', 16, 3, 2],
					['H22', 17, 3, 3],
					['H22', 18, 3, 4],
					['H22', 19, 3, 5],
					['H22', 20, 4, 0],
					['H22', 21, 4, 1],
					['H22', 22, 4, 2],
					['H22', 23, 4, 3],
					['H22', 24, 4, 4],
					['H22', 25, 4, 5],
					['H22', 26, 4, 6],
					['H22', 27, 4, 7],
					['H22', 28, 4, 8],
					['H22', 29, 4, 9],
					['H22', 30, 4, 10],
					['H22', 31, 4, 11],
					['H22', 32, 4, 12],
					['H22', 39, 6, 1],
					['H22', 40, 6, 2],
					['H22', 41, 7, 0],
					['H22', 42, 7, 1],
					['H22', 43, 7, 2],
					['H22', 44, 7, 3],
					['H22', 45, 7, 4],
					['H22', 46, 8, 0],
					['H22', 47, 9, 0],
					['H22', 48, 9, 1],
					['H22', 49, 9, 2],
					['H22', 50, 9, 3],
					['H22', 51, 10, 0],
					['H22', 52, 10, 1],
					['H22', 53, 10, 2],
					['H22', 54, 10, 3],
					['H22', 55, 10, 4],
					['H22', 56, 10, 5],
					['H22', 57, 10, 6],
					['H22', 58, 10, 7],
					['H22', 59, 10, 8],
					['H22', 60, 11, 0],
					['H22', 61, 11, 1],
					['H22', 98, 13, 0],
					['H22', 99, 13, 1],
					['H22', 101, 13, 3],
					['H22', 106, 13, 8],
					['H22', 107, 13, 9],
					['H22', 108, 13, 10],
					['H22', 109, 13, 11],
					['H22', 110, 13, 12],
					['H22', 111, 14, 0],
					['H22', 123, 15, 0],
					['H22', 124, 15, 1],
					['H22', 125, 15, 2],
					['H22', 126, 15, 3],
					['H22', 160, 22, 0],
					['H22', 161, 22, 1],
					['H22', 162, 22, 2],
					['H22', 163, 22, 3],
					['H22', 164, 22, 4],
					['H22', 165, 22, 5],
					['H22', 170, 24, 0],
					['H22', 171, 24, 1],
					['H22', 172, 24, 2],
					['H22', 173, 25, 0],
					['H22', 174, 25, 1],
					['H22', 175, 25, 2],
					['H22', 176, 25, 3],
					['H22', 177, 25, 4],
					['H22', 178, 25, 5],
					['H22', 182, 27, 0],
					['H22', 183, 27, 1],
					['H22', 184, 27, 2],
					['H22', 185, 27, 3],
					['H22', 186, 28, 0],
					['H22', 187, 28, 1],
					['H22', 188, 28, 2],
					['H22', 189, 28, 3],
					['H22', 190, 29, 0],
					['H22', 191, 29, 1],
					['H22', 192, 29, 2],
					['H22', 193, 29, 3],
					['H22', 194, 30, 0],
					['H22', 195, 30, 1],
					['H22', 196, 30, 2],
					['H22', 197, 30, 3],
					['H22', 198, 30, 4],
					['H22', 210, 31, 1],
					['H22', 211, 31, 2],
					['H22', 212, 31, 3],
					['H22', 213, 31, 4],
					['H22', 214, 32, 1],
					['H22', 215, 32, 2],
					['H22', 216, 32, 3],
					['H22', 217, 32, 4],
					['H22', 218, 32, 5],
					['H22', 219, 32, 6],
					['H22', 220, 33, 1],
					['H22', 221, 33, 2],
					['H22', 222, 33, 3],
					['H22', 223, 34, 1],
					['H22', 224, 34, 2],
					['H22', 225, 34, 3],
					['H22', 226, 35, 1],
					['H22', 227, 35, 2],
					['H22', 228, 35, 3],
					['H22', 237, 38, 1],
					['H22', 238, 38, 2],
					['H22', 239, 38, 3],
					['H22', 240, 38, 4],
					['H22', 241, 39, 1],
					['H22', 242, 39, 2],
					['H22', 243, 39, 3],
					['H22', 244, 39, 4],
					['H22', 245, 40, 1],
					['H22', 246, 40, 2],
					['H22', 247, 40, 3],
					['H22', 248, 40, 4],
					['H22', 249, 41, 1],
					['H22', 250, 41, 2],
					['H22', 251, 41, 3],
					['H22', 252, 41, 4],
					['H22', 253, 41, 5],
					['H22', 254, 41, 6],
					['H22', 255, 41, 7],
					['H22', 256, 41, 8],
					['H22', 257, 41, 9],
					['H22', 258, 41, 10],
					['H22', 259, 41, 11],
					['H22', 260, 42, 1],
					['H22', 261, 42, 2],
					['H22', 262, 42, 3],
					['H22', 263, 42, 4],
					['H22', 264, 42, 5],
					['H22', 265, 42, 6],
					['H22', 266, 42, 7],
					['H22', 267, 42, 8],
					['H22', 268, 42, 9],
					['H22', 269, 43, 1],
					['H22', 270, 43, 2],
					['H22', 271, 43, 3],
					['H22', 272, 43, 4],
					['H22', 273, 44, 1],
					['H22', 274, 44, 2],
					['H22', 275, 44, 3],
					['H22', 276, 44, 4],
					['H22', 277, 45, 1],
					['H22', 278, 45, 2],
					['H22', 279, 45, 3],
					['H22', 280, 45, 4],
					['H22', 281, 45, 5],
					['H22', 282, 45, 6],
					['H22', 283, 45, 7],
					['H22', 284, 45, 8],
					['H22', 285, 45, 9],
					['H22', 286, 45, 10],
					['H22', 287, 45, 11],
					['H22', 289, 46, 1],
					['H22', 290, 46, 2],
					['H22', 291, 46, 3],
					['H22', 292, 46, 4],
					['H22', 293, 46, 5],
					['H22', 294, 46, 6],
					['H22', 295, 47, 1],
					['H22', 296, 47, 2],
					['H22', 297, 48, 1],
					['H22', 298, 48, 2],
					['H22', 299, 48, 3],
					['H22', 300, 48, 4],
					['H22', 301, 48, 5],
					['H22', 302, 48, 6],
					['H22', 303, 48, 7],
					['H22', 304, 48, 8],
					['H22', 305, 48, 9],
					['H22', 306, 48, 10],
					['H22', 307, 48, 11],
					['H22', 308, 48, 12],
					['H22', 309, 48, 13],
					['H22', 310, 48, 14],
					['H22', 311, 48, 15],
					['H22', 312, 48, 16],
					['H22', 313, 48, 17],
					['H22', 314, 48, 18],
					['H22', 315, 48, 19],
					['H22', 316, 48, 20],
					['H22', 317, 48, 21],
					['H22', 318, 48, 22],
					['H22', 319, 48, 23],
					['H22', 320, 48, 24],
					['H22', 323, 49, 2],
					['H22', 324, 49, 3],
					['H22', 325, 50, 1],
					['H22', 326, 50, 2],
					['H22', 387, 29, null],
					['H22', 388, 57, 1],
					['H22', 389, 57, 2],
					['H22', 393, 58, 1],
					['H22', 394, 58, 2],
					['H22', 395, 58, 3],
					['H22', 396, 58, 4],
					['H22', 397, 58, 5],
					['H22', 398, 58, 6],
					['H22', 399, 58, 7],
					['H22', 400, 58, 8],
					['H22', 401, 59, 1],
					['H22', 402, 59, 2],
					['H22', 403, 60, 1],
					['H22', 404, 60, 2],
					['H22', 416, 63, 1],
					['H22', 417, 63, 2],
					['H22', 418, 63, 3],
					['H22', 419, 64, 1],
					['H22', 420, 64, 2],
					['H22', 422, 65, 1],
					['H22', 423, 65, 2],
					['H22', 424, 65, 3],
					['H22', 425, 66, 1],
					['H22', 426, 66, 2],
					['H22', 427, 66, 3],
					['H22', 440, 68, 1],
					['H22', 444, 68, 2],
					['H22', 445, 68, 3],
					['H22', 446, 14, 12],
					['H22', 451, 14, 17],
					['H22', 452, 14, 18],
					['H22', 453, 14, 19],
					['H22', 455, 11, 2],
					['H22', 456, 11, 3],
					['H22', 457, 11, 4],
					['H22', 458, 11, 5],
					['H22', 459, 11, 6],
					['H22', 460, 11, 7],
					['H22', 461, 11, 8],
					['H22', 462, 11, 9],
					['H22', 463, 11, 10],
					['H22', 464, 11, 11],
					['H22', 465, 11, 12],
					['H22', 508, 37, 3],
					['H22', 509, 37, 4],
					['H22', 510, 37, 5],
					['H22', 511, 37, 6],
					['H22', 557, 29, 4],
					['H22', 559, 70, 1],
					['H22', 560, 70, 2],
					['H22', 564, 71, 1],
					['H22', 565, 71, 2],
					['H22', 576, 74, 1],
					['H22', 577, 74, 2],
					['H22', 578, 74, 3],
					['H22', 579, 74, 4],
					['H22', 580, 74, 5],
					['H22', 581, 74, 6],
					['H22', 582, 74, 7],
					['H22', 583, 74, 8],
					['H22', 584, 74, 9],
					['H22', 585, 74, 10],
					['H22', 586, 74, 11],
					['H22', 587, 75, 1],
					['H22', 588, 75, 2],
					['H22', 589, 75, 3],
					['H22', 590, 76, 1],
					['H22', 591, 76, 2],
					['H22', 592, 76, 3],
					['H22', 593, 77, 1],
					['H22', 594, 77, 2],
					['H22', 595, 77, 3],
					['H22', 596, 77, 4],
					['H22', 597, 77, 5],
					['H22', 598, 77, 6],
					['H22', 599, 77, 7],
					['H22', 600, 77, 8],
					['H22', 601, 77, 9],
					['H22', 602, 77, 10],
					['H22', 603, 77, 11],
					['H22', 604, 77, 12],
					['H22', 605, 77, 13],
					['H22', 606, 77, 14],
					['H22', 607, 77, 15],
					['H22', 608, 78, 1],
					['H22', 609, 78, 2],
					['H22', 610, 78, 3],
					['H22', 611, 78, 4],
					['H22', 612, 78, 5],
					['H22', 613, 78, 6],
					['H22', 614, 78, 7],
					['H22', 615, 78, 8],
					['H22', 616, 78, 9],
					['H22', 617, 78, 10],
					['H22', 618, 78, 11],
					['H22', 619, 78, 12],
					['H22', 620, 78, 13],
					['H22', 621, 78, 14],
					['H22', 622, 78, 15],
					['H22', 659, 85, 1],
					['H22', 660, 85, 2],
					['H22', 661, 85, 3],
					['H22', 666, 86, 1],
					['H22', 667, 86, 2],
					['H22', 668, 87, 1],
					['H22', 669, 88, 1],
					['H22', 670, 89, 1],
					['H22', 671, 88, 2],
					['H22', 672, 37, 7],
					['H22', 675, 90, 1],
					['H22', 676, 90, 2],
					['H22', 677, 90, 3],
					['H22', 678, 90, 4],
					['H22', 679, 90, 5],
					['H22', 680, 90, 6],
					['H22', 681, 90, 7],
					['H22', 682, 91, 1],
					['H22', 683, 91, 2],
					['H22', 684, 91, 3],
					['H22', 685, 91, 4],
					['H22', 686, 91, 5],
					['H22', 687, 91, 6],
					['H22', 688, 91, 7],
					['H22', 689, 63, 4],
					['H22', 690, 63, 5],
					['H22', 691, 92, 1],
					['H22', 692, 93, 1],
					['H22', 693, 92, 2],
					['H22', 694, 29, 5],
					['H22', 695, 29, 6],
					['H22', 696, 29, 7],
					['H22', 697, 94, 1],
					['H22', 698, 94, 2],
					['H22', 699, 94, 3],
					['H22', 700, 57, 3],
					['H22', 701, 90, 8],
					['H22', 702, 91, 8],
					['H22', 703, 95, 1],
					['H22', 704, 95, 2],
					['H22', 705, 96, 1],
					['H22', 706, 96, 2],
					['H22', 707, 96, 3],
					['H22', 708, 96, 4],
					['H22', 709, 96, 5],
					['H22', 710, 96, 6],
					['H22', 711, 96, 7],
					['H22', 712, 96, 8],
					['H22', 713, 86, 3],
					['H22', 714, 97, 1],
					['H22', 715, 97, 2],
					['H22', 716, 97, 3],
					['H22', 717, 98, 1],
					['H22', 718, 98, 2],
					['H22', 719, 98, 3],
					['H22', 720, 14, 20],
					['H22', 721, 14, 21],
					['H22', 722, 14, 22],
					['H22', 723, 99, 1],
					['H22', 724, 99, 2],
					['H22', 725, 99, 3],
					['H22', 726, 99, 4],
					['H22', 727, 99, 5],
					['H22', 728, 100, 1],
					['H22', 729, 100, 2],
					['H22', 730, 100, 3],
					['H22', 731, 100, 4],
					['H22', 732, 100, 5],
					['H22', 733, 100, 6],
					['H22', 734, 100, 7],
					['H22', 735, 100, 8],
					['H22', 736, 100, 9],
					['H22', 738, 102, 1],
					['H22', 739, 102, 2],
					['H22', 740, 102, 3],
					['H22', 741, 102, 4],
					['H22', 742, 102, 5],
					['H22', 743, 102, 6],
					['H22', 744, 102, 7],
					['H22', 745, 102, 8],
					['H22', 746, 103, 1],
					['H22', 747, 103, 2],
					['H22', 748, 103, 3],
					['H22', 749, 104, 1],
					['H22', 750, 104, 2],
					['H22', 751, 104, 3],
					['H22', 752, 104, 4],
					['H22', 753, 104, 5],
					['H22', 754, 104, 6],
					['H22', 755, 104, 7],
					['H22', 756, 104, 8],
					['H22', 757, 105, 1],
					['H22', 758, 105, 2],
					['H22', 759, 105, 3],
					['H22', 760, 106, 1],
					['H22', 761, 106, 2],
					['H22', 762, 106, 3],
					['H22', 763, 106, 4],
					['H22', 766, 87, 2],
					['H22', 767, 87, 3],
					['H22', 768, 87, 4],
					['H22', 769, 87, 5],
					['H22', 770, 87, 6],
					['H22', 771, 87, 7],
					['H22', 772, 87, 8],
					['H22', 773, 107, 1],
					['H22', 774, 107, 2],
					['H22', 775, 107, 3],
					['H22', 776, 107, 4],
					['H22', 777, 107, 5],
					['H22', 778, 108, 1],
					['H22', 779, 108, 2],
					['H22', 780, 108, 3],
					['H22', 781, 108, 4],
					['H22', 782, 108, 5],
					['H22', 783, 108, 6],
					['H22', 784, 109, 1],
					['H22', 785, 109, 2],
					['H22', 786, 109, 3],
					['H22', 787, 109, 4],
					['H22', 788, 109, 5],
					['H22', 789, 109, 6],
					['H22', 790, 110, 1],
					['H22', 791, 110, 2],
					['H22', 792, 110, 3],
					['H22', 793, 110, 4],
					['H22', 794, 110, 5],
					['H22', 795, 110, 6],
					['H22', 796, 111, 1],
					['H22', 797, 111, 2],
					['H22', 798, 111, 3],
					['H22', 799, 111, 4],
					['H22', 800, 111, 5],
					['H22', 801, 111, 6],
					['H22', 802, 112, 1],
					['H22', 803, 112, 2],
					['H22', 804, 112, 3],
					['H22', 805, 112, 4],
					['H22', 806, 112, 5],
					['H22', 807, 112, 6],
					['H22', 808, 113, 1],
					['H22', 809, 113, 2],
					['H22', 810, 113, 3],
					['H22', 811, 113, 4],
					['H22', 812, 113, 5],
					['H22', 813, 113, 6],
					['H22', 814, 114, 1],
					['H22', 815, 114, 2],
					['H22', 816, 114, 3],
					['H22', 817, 114, 4],
					['H22', 818, 114, 5],
					['H22', 819, 114, 6],
					['H22', 820, 115, 1],
					['H22', 821, 115, 2],
					['H22', 822, 115, 3],
					['H22', 823, 115, 4],
					['H22', 824, 115, 5],
					['H22', 825, 115, 6],
					['H22', 826, 116, 1],
					['H22', 827, 116, 2],
					['H22', 828, 116, 3],
					['H22', 829, 116, 4],
					['H22', 830, 116, 5],
					['H22', 831, 116, 6],
					['H22', 832, 117, 1],
					['H22', 833, 117, 2],
					['H22', 834, 117, 3],
					['H22', 835, 117, 4],
					['H22', 836, 117, 5],
					['H22', 837, 117, 6],
					['H22', 838, 118, 1],
					['H22', 839, 118, 2],
					['H22', 840, 118, 3],
					['H22', 841, 118, 4],
					['H22', 842, 118, 5],
					['H22', 843, 118, 6],
					['H22', 844, 118, 7],
					['H22', 845, 119, 1],
					['H22', 846, 119, 2],
					['H22', 847, 120, 1],
					['H22', 848, 120, 2],
					['H22', 849, 121, 1],
					['H22', 850, 121, 2],
					['H22', 851, 122, 2],
					['H22', 852, 122, 3],
					['H22', 853, 122, 1],
					['H22', 854, 122, 4],
					['H22', 855, 123, 1],
					['H22', 856, 123, 2],
					['H22', 857, 123, 3],
					['H22', 858, 123, 4],
					['H22', 859, 123, 5],
					['H22', 860, 124, 1],
					['H22', 861, 124, 2],
					['H22', 862, 124, 3],
					['H22', 863, 124, 4],
					['H22', 864, 124, 5],
					['H22', 865, 125, 1],
					['H22', 866, 125, 2],
					['H22', 867, 125, 3],
					['H22', 868, 125, 4],
					['H22', 869, 125, 5],
					['H22', 870, 125, 6],
					['H22', 871, 125, 7],
					['H22', 872, 126, 1],
					['H22', 873, 126, 2],
					['H22', 874, 126, 3],
					['H22', 875, 126, 4],
					['H22', 876, 126, 5],
					['H22', 877, 126, 6],
					['H22', 878, 126, 7],
					['H22', 879, 126, 8],
					['H22', 880, 126, 9],
					['H22', 881, 127, 1],
					['H22', 882, 127, 2],
					['H22', 883, 127, 3],
					['H22', 884, 127, 4],
					['H22', 885, 127, 5],
					['H22', 886, 128, 1],
					['H22', 887, 128, 2],
					['H22', 888, 128, 3],
					['H22', 889, 128, 4],
					['H22', 890, 128, 5],
					['H22', 891, 128, 6],
					['H22', 892, 128, 7],
					['H22', 893, 128, 8],
					['H22', 894, 128, 9],
					['H22', 895, 128, 10],
					['H22', 896, 128, 11],
					['H22', 897, 128, 12],
					['H22', 898, 128, 13],
					['H22', 899, 128, 14],
					['H22', 900, 128, 15],
					['H22', 901, 128, 16],
					['H22', 902, 129, 1],
					['H22', 903, 129, 2],
					['H22', 904, 129, 3],
					['H22', 905, 129, 4],
					['H22', 906, 129, 5],
					['H22', 907, 129, 6],
					['H22', 908, 130, 1],
					['H22', 909, 130, 2],
					['H26', 1, 1, 0],
					['H26', 2, 1, 1],
					['H26', 3, 1, 2],
					['H26', 4, 1, 3],
					['H26', 5, 1, 4],
					['H26', 6, 1, 5],
					['H26', 7, 1, 6],
					['H26', 8, 1, 7],
					['H26', 9, 1, 8],
					['H26', 10, 1, 9],
					['H26', 11, 1, 10],
					['H26', 12, 2, 0],
					['H26', 13, 2, 1],
					['H26', 14, 3, 0],
					['H26', 15, 3, 1],
					['H26', 16, 3, 2],
					['H26', 17, 3, 3],
					['H26', 18, 3, 4],
					['H26', 19, 3, 5],
					['H26', 20, 4, 0],
					['H26', 21, 4, 1],
					['H26', 22, 4, 2],
					['H26', 23, 4, 3],
					['H26', 24, 4, 4],
					['H26', 25, 4, 5],
					['H26', 26, 4, 6],
					['H26', 27, 4, 7],
					['H26', 28, 4, 8],
					['H26', 29, 4, 9],
					['H26', 30, 4, 10],
					['H26', 31, 4, 11],
					['H26', 32, 4, 12],
					['H26', 39, 6, 1],
					['H26', 40, 6, 2],
					['H26', 41, 7, 0],
					['H26', 42, 7, 1],
					['H26', 43, 7, 2],
					['H26', 44, 7, 3],
					['H26', 45, 7, 4],
					['H26', 46, 8, 0],
					['H26', 47, 9, 0],
					['H26', 48, 9, 1],
					['H26', 49, 9, 2],
					['H26', 50, 9, 3],
					['H26', 51, 10, 0],
					['H26', 52, 10, 1],
					['H26', 53, 10, 2],
					['H26', 54, 10, 3],
					['H26', 55, 10, 4],
					['H26', 56, 10, 5],
					['H26', 57, 10, 6],
					['H26', 58, 10, 7],
					['H26', 59, 10, 8],
					['H26', 60, 11, 0],
					['H26', 61, 11, 1],
					['H26', 98, 13, 0],
					['H26', 99, 13, 1],
					['H26', 101, 13, 3],
					['H26', 106, 13, 8],
					['H26', 107, 13, 9],
					['H26', 108, 13, 10],
					['H26', 109, 13, 11],
					['H26', 110, 13, 12],
					['H26', 111, 14, 0],
					['H26', 123, 15, 0],
					['H26', 124, 15, 1],
					['H26', 125, 15, 2],
					['H26', 126, 15, 3],
					['H26', 160, 22, 0],
					['H26', 161, 22, 1],
					['H26', 162, 22, 2],
					['H26', 163, 22, 3],
					['H26', 164, 22, 4],
					['H26', 165, 22, 5],
					['H26', 170, 24, 0],
					['H26', 171, 24, 1],
					['H26', 172, 24, 2],
					['H26', 173, 25, 0],
					['H26', 174, 25, 1],
					['H26', 175, 25, 2],
					['H26', 176, 25, 3],
					['H26', 177, 25, 4],
					['H26', 178, 25, 5],
					['H26', 182, 27, 0],
					['H26', 183, 27, 1],
					['H26', 184, 27, 2],
					['H26', 185, 27, 3],
					['H26', 186, 28, 0],
					['H26', 187, 28, 1],
					['H26', 188, 28, 2],
					['H26', 189, 28, 3],
					['H26', 190, 29, 0],
					['H26', 191, 29, 1],
					['H26', 192, 29, 2],
					['H26', 193, 29, 3],
					['H26', 194, 30, 0],
					['H26', 195, 30, 1],
					['H26', 196, 30, 2],
					['H26', 197, 30, 3],
					['H26', 198, 30, 4],
					['H26', 210, 31, 1],
					['H26', 211, 31, 2],
					['H26', 212, 31, 3],
					['H26', 213, 31, 4],
					['H26', 214, 32, 1],
					['H26', 215, 32, 2],
					['H26', 216, 32, 3],
					['H26', 217, 32, 4],
					['H26', 218, 32, 5],
					['H26', 219, 32, 6],
					['H26', 220, 33, 1],
					['H26', 221, 33, 2],
					['H26', 222, 33, 3],
					['H26', 223, 34, 1],
					['H26', 224, 34, 2],
					['H26', 225, 34, 3],
					['H26', 226, 35, 1],
					['H26', 227, 35, 2],
					['H26', 228, 35, 3],
					['H26', 237, 38, 1],
					['H26', 238, 38, 2],
					['H26', 239, 38, 3],
					['H26', 240, 38, 4],
					['H26', 241, 39, 1],
					['H26', 242, 39, 2],
					['H26', 243, 39, 3],
					['H26', 244, 39, 4],
					['H26', 245, 40, 1],
					['H26', 246, 40, 2],
					['H26', 247, 40, 3],
					['H26', 248, 40, 4],
					['H26', 249, 41, 1],
					['H26', 250, 41, 2],
					['H26', 251, 41, 3],
					['H26', 252, 41, 4],
					['H26', 253, 41, 5],
					['H26', 254, 41, 6],
					['H26', 255, 41, 7],
					['H26', 256, 41, 8],
					['H26', 257, 41, 9],
					['H26', 258, 41, 10],
					['H26', 259, 41, 11],
					['H26', 260, 42, 1],
					['H26', 261, 42, 2],
					['H26', 262, 42, 3],
					['H26', 263, 42, 4],
					['H26', 264, 42, 5],
					['H26', 265, 42, 6],
					['H26', 266, 42, 7],
					['H26', 267, 42, 8],
					['H26', 268, 42, 9],
					['H26', 269, 43, 1],
					['H26', 270, 43, 2],
					['H26', 271, 43, 3],
					['H26', 272, 43, 4],
					['H26', 273, 44, 1],
					['H26', 274, 44, 2],
					['H26', 275, 44, 3],
					['H26', 276, 44, 4],
					['H26', 277, 45, 1],
					['H26', 278, 45, 2],
					['H26', 279, 45, 3],
					['H26', 280, 45, 4],
					['H26', 281, 45, 5],
					['H26', 282, 45, 6],
					['H26', 283, 45, 7],
					['H26', 284, 45, 8],
					['H26', 285, 45, 9],
					['H26', 286, 45, 10],
					['H26', 287, 45, 11],
					['H26', 289, 46, 1],
					['H26', 290, 46, 2],
					['H26', 291, 46, 3],
					['H26', 292, 46, 4],
					['H26', 293, 46, 5],
					['H26', 294, 46, 6],
					['H26', 295, 47, 1],
					['H26', 296, 47, 2],
					['H26', 297, 48, 1],
					['H26', 298, 48, 2],
					['H26', 299, 48, 3],
					['H26', 300, 48, 4],
					['H26', 301, 48, 5],
					['H26', 302, 48, 6],
					['H26', 303, 48, 7],
					['H26', 304, 48, 8],
					['H26', 305, 48, 9],
					['H26', 306, 48, 10],
					['H26', 307, 48, 11],
					['H26', 308, 48, 12],
					['H26', 309, 48, 13],
					['H26', 310, 48, 14],
					['H26', 311, 48, 15],
					['H26', 312, 48, 16],
					['H26', 313, 48, 17],
					['H26', 314, 48, 18],
					['H26', 315, 48, 19],
					['H26', 316, 48, 20],
					['H26', 317, 48, 21],
					['H26', 318, 48, 22],
					['H26', 319, 48, 23],
					['H26', 320, 48, 24],
					['H26', 323, 49, 2],
					['H26', 324, 49, 3],
					['H26', 325, 50, 1],
					['H26', 326, 50, 2],
					['H26', 387, 29, null],
					['H26', 388, 57, 1],
					['H26', 389, 57, 2],
					['H26', 393, 58, 1],
					['H26', 394, 58, 2],
					['H26', 395, 58, 3],
					['H26', 396, 58, 4],
					['H26', 397, 58, 5],
					['H26', 398, 58, 6],
					['H26', 399, 58, 7],
					['H26', 400, 58, 8],
					['H26', 401, 59, 1],
					['H26', 402, 59, 2],
					['H26', 403, 60, 1],
					['H26', 404, 60, 2],
					['H26', 416, 63, 1],
					['H26', 417, 63, 2],
					['H26', 418, 63, 3],
					['H26', 419, 64, 1],
					['H26', 420, 64, 2],
					['H26', 422, 65, 1],
					['H26', 423, 65, 2],
					['H26', 424, 65, 3],
					['H26', 425, 66, 1],
					['H26', 426, 66, 2],
					['H26', 427, 66, 3],
					['H26', 440, 68, 1],
					['H26', 444, 68, 2],
					['H26', 445, 68, 3],
					['H26', 446, 14, 12],
					['H26', 451, 14, 17],
					['H26', 452, 14, 18],
					['H26', 453, 14, 19],
					['H26', 455, 11, 2],
					['H26', 456, 11, 3],
					['H26', 457, 11, 4],
					['H26', 458, 11, 5],
					['H26', 459, 11, 6],
					['H26', 460, 11, 7],
					['H26', 461, 11, 8],
					['H26', 462, 11, 9],
					['H26', 463, 11, 10],
					['H26', 464, 11, 11],
					['H26', 465, 11, 12],
					['H26', 508, 37, 3],
					['H26', 509, 37, 4],
					['H26', 510, 37, 5],
					['H26', 511, 37, 6],
					['H26', 557, 29, 4],
					['H26', 559, 70, 1],
					['H26', 560, 70, 2],
					['H26', 564, 71, 1],
					['H26', 565, 71, 2],
					['H26', 576, 74, 1],
					['H26', 577, 74, 2],
					['H26', 578, 74, 3],
					['H26', 579, 74, 4],
					['H26', 580, 74, 5],
					['H26', 581, 74, 6],
					['H26', 582, 74, 7],
					['H26', 583, 74, 8],
					['H26', 584, 74, 9],
					['H26', 585, 74, 10],
					['H26', 586, 74, 11],
					['H26', 587, 75, 1],
					['H26', 588, 75, 2],
					['H26', 589, 75, 3],
					['H26', 590, 76, 1],
					['H26', 591, 76, 2],
					['H26', 592, 76, 3],
					['H26', 593, 77, 1],
					['H26', 594, 77, 2],
					['H26', 595, 77, 3],
					['H26', 596, 77, 4],
					['H26', 597, 77, 5],
					['H26', 598, 77, 6],
					['H26', 599, 77, 7],
					['H26', 600, 77, 8],
					['H26', 601, 77, 9],
					['H26', 602, 77, 10],
					['H26', 603, 77, 11],
					['H26', 604, 77, 12],
					['H26', 605, 77, 13],
					['H26', 606, 77, 14],
					['H26', 607, 77, 15],
					['H26', 608, 78, 1],
					['H26', 609, 78, 2],
					['H26', 610, 78, 3],
					['H26', 611, 78, 4],
					['H26', 612, 78, 5],
					['H26', 613, 78, 6],
					['H26', 614, 78, 7],
					['H26', 615, 78, 8],
					['H26', 616, 78, 9],
					['H26', 617, 78, 10],
					['H26', 618, 78, 11],
					['H26', 619, 78, 12],
					['H26', 620, 78, 13],
					['H26', 621, 78, 14],
					['H26', 622, 78, 15],
					['H26', 659, 85, 1],
					['H26', 660, 85, 2],
					['H26', 661, 85, 3],
					['H26', 666, 86, 1],
					['H26', 667, 86, 2],
					['H26', 668, 87, 1],
					['H26', 669, 88, 1],
					['H26', 670, 89, 1],
					['H26', 671, 88, 2],
					['H26', 672, 37, 7],
					['H26', 675, 90, 1],
					['H26', 676, 90, 2],
					['H26', 677, 90, 3],
					['H26', 678, 90, 4],
					['H26', 679, 90, 5],
					['H26', 680, 90, 6],
					['H26', 681, 90, 7],
					['H26', 682, 91, 1],
					['H26', 683, 91, 2],
					['H26', 684, 91, 3],
					['H26', 685, 91, 4],
					['H26', 686, 91, 5],
					['H26', 687, 91, 6],
					['H26', 688, 91, 7],
					['H26', 689, 63, 4],
					['H26', 690, 63, 5],
					['H26', 691, 92, 1],
					['H26', 692, 93, 1],
					['H26', 693, 92, 2],
					['H26', 694, 29, 5],
					['H26', 695, 29, 6],
					['H26', 696, 29, 7],
					['H26', 697, 94, 1],
					['H26', 698, 94, 2],
					['H26', 699, 94, 3],
					['H26', 700, 57, 3],
					['H26', 701, 90, 8],
					['H26', 702, 91, 8],
					['H26', 703, 95, 1],
					['H26', 704, 95, 2],
					['H26', 705, 96, 1],
					['H26', 706, 96, 2],
					['H26', 707, 96, 3],
					['H26', 708, 96, 4],
					['H26', 709, 96, 5],
					['H26', 710, 96, 6],
					['H26', 711, 96, 7],
					['H26', 712, 96, 8],
					['H26', 713, 86, 3],
					['H26', 714, 97, 1],
					['H26', 715, 97, 2],
					['H26', 716, 97, 3],
					['H26', 717, 98, 1],
					['H26', 718, 98, 2],
					['H26', 719, 98, 3],
					['H26', 720, 14, 20],
					['H26', 721, 14, 21],
					['H26', 722, 14, 22],
					['H26', 723, 99, 1],
					['H26', 724, 99, 2],
					['H26', 725, 99, 3],
					['H26', 726, 99, 4],
					['H26', 727, 99, 5],
					['H26', 728, 100, 1],
					['H26', 729, 100, 2],
					['H26', 730, 100, 3],
					['H26', 731, 100, 4],
					['H26', 732, 100, 5],
					['H26', 733, 100, 6],
					['H26', 734, 100, 7],
					['H26', 735, 100, 8],
					['H26', 736, 100, 9],
					['H26', 738, 102, 1],
					['H26', 739, 102, 2],
					['H26', 740, 102, 3],
					['H26', 741, 102, 4],
					['H26', 742, 102, 5],
					['H26', 743, 102, 6],
					['H26', 744, 102, 7],
					['H26', 745, 102, 8],
					['H26', 746, 103, 1],
					['H26', 747, 103, 2],
					['H26', 748, 103, 3],
					['H26', 749, 104, 1],
					['H26', 750, 104, 2],
					['H26', 751, 104, 3],
					['H26', 752, 104, 4],
					['H26', 753, 104, 5],
					['H26', 754, 104, 6],
					['H26', 755, 104, 7],
					['H26', 756, 104, 8],
					['H26', 757, 105, 1],
					['H26', 758, 105, 2],
					['H26', 759, 105, 3],
					['H26', 760, 106, 1],
					['H26', 761, 106, 2],
					['H26', 762, 106, 3],
					['H26', 763, 106, 4],
					['H26', 766, 87, 2],
					['H26', 767, 87, 3],
					['H26', 768, 87, 4],
					['H26', 769, 87, 5],
					['H26', 770, 87, 6],
					['H26', 771, 87, 7],
					['H26', 772, 87, 8],
					['H26', 773, 107, 1],
					['H26', 774, 107, 2],
					['H26', 775, 107, 3],
					['H26', 776, 107, 4],
					['H26', 777, 107, 5],
					['H26', 778, 108, 1],
					['H26', 779, 108, 2],
					['H26', 780, 108, 3],
					['H26', 781, 108, 4],
					['H26', 782, 108, 5],
					['H26', 783, 108, 6],
					['H26', 784, 109, 1],
					['H26', 785, 109, 2],
					['H26', 786, 109, 3],
					['H26', 787, 109, 4],
					['H26', 788, 109, 5],
					['H26', 789, 109, 6],
					['H26', 790, 110, 1],
					['H26', 791, 110, 2],
					['H26', 792, 110, 3],
					['H26', 793, 110, 4],
					['H26', 794, 110, 5],
					['H26', 795, 110, 6],
					['H26', 796, 111, 1],
					['H26', 797, 111, 2],
					['H26', 798, 111, 3],
					['H26', 799, 111, 4],
					['H26', 800, 111, 5],
					['H26', 801, 111, 6],
					['H26', 802, 112, 1],
					['H26', 803, 112, 2],
					['H26', 804, 112, 3],
					['H26', 805, 112, 4],
					['H26', 806, 112, 5],
					['H26', 807, 112, 6],
					['H26', 808, 113, 1],
					['H26', 809, 113, 2],
					['H26', 810, 113, 3],
					['H26', 811, 113, 4],
					['H26', 812, 113, 5],
					['H26', 813, 113, 6],
					['H26', 814, 114, 1],
					['H26', 815, 114, 2],
					['H26', 816, 114, 3],
					['H26', 817, 114, 4],
					['H26', 818, 114, 5],
					['H26', 819, 114, 6],
					['H26', 820, 115, 1],
					['H26', 821, 115, 2],
					['H26', 822, 115, 3],
					['H26', 823, 115, 4],
					['H26', 824, 115, 5],
					['H26', 825, 115, 6],
					['H26', 826, 116, 1],
					['H26', 827, 116, 2],
					['H26', 828, 116, 3],
					['H26', 829, 116, 4],
					['H26', 830, 116, 5],
					['H26', 831, 116, 6],
					['H26', 832, 117, 1],
					['H26', 833, 117, 2],
					['H26', 834, 117, 3],
					['H26', 835, 117, 4],
					['H26', 836, 117, 5],
					['H26', 837, 117, 6],
					['H26', 838, 118, 1],
					['H26', 839, 118, 2],
					['H26', 840, 118, 3],
					['H26', 841, 118, 4],
					['H26', 842, 118, 5],
					['H26', 843, 118, 6],
					['H26', 844, 118, 7],
					['H26', 845, 119, 1],
					['H26', 846, 119, 2],
					['H26', 847, 120, 1],
					['H26', 848, 120, 2],
					['H26', 849, 121, 1],
					['H26', 850, 121, 2],
					['H26', 851, 122, 2],
					['H26', 852, 122, 3],
					['H26', 853, 122, 1],
					['H26', 854, 122, 4],
					['H26', 855, 123, 1],
					['H26', 856, 123, 2],
					['H26', 857, 123, 3],
					['H26', 858, 123, 4],
					['H26', 859, 123, 5],
					['H26', 860, 124, 1],
					['H26', 861, 124, 2],
					['H26', 862, 124, 3],
					['H26', 863, 124, 4],
					['H26', 864, 124, 5],
					['H26', 865, 125, 1],
					['H26', 866, 125, 2],
					['H26', 867, 125, 3],
					['H26', 868, 125, 4],
					['H26', 869, 125, 5],
					['H26', 870, 125, 6],
					['H26', 871, 125, 7],
					['H26', 872, 126, 1],
					['H26', 873, 126, 2],
					['H26', 874, 126, 3],
					['H26', 875, 126, 4],
					['H26', 876, 126, 5],
					['H26', 877, 126, 6],
					['H26', 878, 126, 7],
					['H26', 879, 126, 8],
					['H26', 880, 126, 9],
					['H26', 881, 127, 1],
					['H26', 882, 127, 2],
					['H26', 883, 127, 3],
					['H26', 884, 127, 4],
					['H26', 885, 127, 5],
					['H26', 886, 128, 1],
					['H26', 887, 128, 2],
					['H26', 888, 128, 3],
					['H26', 889, 128, 4],
					['H26', 890, 128, 5],
					['H26', 891, 128, 6],
					['H26', 892, 128, 7],
					['H26', 893, 128, 8],
					['H26', 894, 128, 9],
					['H26', 895, 128, 10],
					['H26', 896, 128, 11],
					['H26', 897, 128, 12],
					['H26', 898, 128, 13],
					['H26', 899, 128, 14],
					['H26', 900, 128, 15],
					['H26', 901, 128, 16],
					['H26', 902, 129, 1],
					['H26', 903, 129, 2],
					['H26', 904, 129, 3],
					['H26', 905, 129, 4],
					['H26', 906, 129, 5],
					['H26', 907, 129, 6],
					['H26', 908, 130, 1],
					['H26', 909, 130, 2],
					['H3', 1, 1, 0],
					['H3', 2, 1, 1],
					['H3', 3, 1, 2],
					['H3', 4, 1, 3],
					['H3', 5, 1, 4],
					['H3', 6, 1, 5],
					['H3', 7, 1, 6],
					['H3', 8, 1, 7],
					['H3', 9, 1, 8],
					['H3', 10, 1, 9],
					['H3', 11, 1, 10],
					['H3', 12, 2, 0],
					['H3', 13, 2, 1],
					['H3', 14, 3, 0],
					['H3', 15, 3, 1],
					['H3', 16, 3, 2],
					['H3', 17, 3, 3],
					['H3', 18, 3, 4],
					['H3', 19, 3, 5],
					['H3', 20, 4, 0],
					['H3', 21, 4, 1],
					['H3', 22, 4, 2],
					['H3', 23, 4, 3],
					['H3', 24, 4, 4],
					['H3', 25, 4, 5],
					['H3', 26, 4, 6],
					['H3', 27, 4, 7],
					['H3', 28, 4, 8],
					['H3', 29, 4, 9],
					['H3', 30, 4, 10],
					['H3', 31, 4, 11],
					['H3', 32, 4, 12],
					['H3', 39, 6, 1],
					['H3', 40, 6, 2],
					['H3', 41, 7, 0],
					['H3', 42, 7, 1],
					['H3', 43, 7, 2],
					['H3', 44, 7, 3],
					['H3', 45, 7, 4],
					['H3', 46, 8, 0],
					['H3', 47, 9, 0],
					['H3', 48, 9, 1],
					['H3', 49, 9, 2],
					['H3', 50, 9, 3],
					['H3', 51, 10, 0],
					['H3', 52, 10, 1],
					['H3', 53, 10, 2],
					['H3', 54, 10, 3],
					['H3', 55, 10, 4],
					['H3', 56, 10, 5],
					['H3', 57, 10, 6],
					['H3', 58, 10, 7],
					['H3', 59, 10, 8],
					['H3', 60, 11, 0],
					['H3', 61, 11, 1],
					['H3', 98, 13, 0],
					['H3', 99, 13, 1],
					['H3', 101, 13, 3],
					['H3', 106, 13, 8],
					['H3', 107, 13, 9],
					['H3', 108, 13, 10],
					['H3', 109, 13, 11],
					['H3', 110, 13, 12],
					['H3', 111, 14, 0],
					['H3', 123, 15, 0],
					['H3', 124, 15, 1],
					['H3', 125, 15, 2],
					['H3', 126, 15, 3],
					['H3', 160, 22, 0],
					['H3', 161, 22, 1],
					['H3', 162, 22, 2],
					['H3', 163, 22, 3],
					['H3', 164, 22, 4],
					['H3', 165, 22, 5],
					['H3', 170, 24, 0],
					['H3', 171, 24, 1],
					['H3', 172, 24, 2],
					['H3', 173, 25, 0],
					['H3', 174, 25, 1],
					['H3', 175, 25, 2],
					['H3', 176, 25, 3],
					['H3', 177, 25, 4],
					['H3', 178, 25, 5],
					['H3', 182, 27, 0],
					['H3', 183, 27, 1],
					['H3', 184, 27, 2],
					['H3', 185, 27, 3],
					['H3', 186, 28, 0],
					['H3', 187, 28, 1],
					['H3', 188, 28, 2],
					['H3', 189, 28, 3],
					['H3', 190, 29, 0],
					['H3', 191, 29, 1],
					['H3', 192, 29, 2],
					['H3', 193, 29, 3],
					['H3', 194, 30, 0],
					['H3', 195, 30, 1],
					['H3', 196, 30, 2],
					['H3', 197, 30, 3],
					['H3', 198, 30, 4],
					['H3', 210, 31, 1],
					['H3', 211, 31, 2],
					['H3', 212, 31, 3],
					['H3', 213, 31, 4],
					['H3', 214, 32, 1],
					['H3', 215, 32, 2],
					['H3', 216, 32, 3],
					['H3', 217, 32, 4],
					['H3', 218, 32, 5],
					['H3', 219, 32, 6],
					['H3', 220, 33, 1],
					['H3', 221, 33, 2],
					['H3', 222, 33, 3],
					['H3', 223, 34, 1],
					['H3', 224, 34, 2],
					['H3', 225, 34, 3],
					['H3', 226, 35, 1],
					['H3', 227, 35, 2],
					['H3', 228, 35, 3],
					['H3', 237, 38, 1],
					['H3', 238, 38, 2],
					['H3', 239, 38, 3],
					['H3', 240, 38, 4],
					['H3', 241, 39, 1],
					['H3', 242, 39, 2],
					['H3', 243, 39, 3],
					['H3', 244, 39, 4],
					['H3', 245, 40, 1],
					['H3', 246, 40, 2],
					['H3', 247, 40, 3],
					['H3', 248, 40, 4],
					['H3', 249, 41, 1],
					['H3', 250, 41, 2],
					['H3', 251, 41, 3],
					['H3', 252, 41, 4],
					['H3', 253, 41, 5],
					['H3', 254, 41, 6],
					['H3', 255, 41, 7],
					['H3', 256, 41, 8],
					['H3', 257, 41, 9],
					['H3', 258, 41, 10],
					['H3', 259, 41, 11],
					['H3', 260, 42, 1],
					['H3', 261, 42, 2],
					['H3', 262, 42, 3],
					['H3', 263, 42, 4],
					['H3', 264, 42, 5],
					['H3', 265, 42, 6],
					['H3', 266, 42, 7],
					['H3', 267, 42, 8],
					['H3', 268, 42, 9],
					['H3', 269, 43, 1],
					['H3', 270, 43, 2],
					['H3', 271, 43, 3],
					['H3', 272, 43, 4],
					['H3', 273, 44, 1],
					['H3', 274, 44, 2],
					['H3', 275, 44, 3],
					['H3', 276, 44, 4],
					['H3', 277, 45, 1],
					['H3', 278, 45, 2],
					['H3', 279, 45, 3],
					['H3', 280, 45, 4],
					['H3', 281, 45, 5],
					['H3', 282, 45, 6],
					['H3', 283, 45, 7],
					['H3', 284, 45, 8],
					['H3', 285, 45, 9],
					['H3', 286, 45, 10],
					['H3', 287, 45, 11],
					['H3', 289, 46, 1],
					['H3', 290, 46, 2],
					['H3', 291, 46, 3],
					['H3', 292, 46, 4],
					['H3', 293, 46, 5],
					['H3', 294, 46, 6],
					['H3', 295, 47, 1],
					['H3', 296, 47, 2],
					['H3', 297, 48, 1],
					['H3', 298, 48, 2],
					['H3', 299, 48, 3],
					['H3', 300, 48, 4],
					['H3', 301, 48, 5],
					['H3', 302, 48, 6],
					['H3', 303, 48, 7],
					['H3', 304, 48, 8],
					['H3', 305, 48, 9],
					['H3', 306, 48, 10],
					['H3', 307, 48, 11],
					['H3', 308, 48, 12],
					['H3', 309, 48, 13],
					['H3', 310, 48, 14],
					['H3', 311, 48, 15],
					['H3', 312, 48, 16],
					['H3', 313, 48, 17],
					['H3', 314, 48, 18],
					['H3', 315, 48, 19],
					['H3', 316, 48, 20],
					['H3', 317, 48, 21],
					['H3', 318, 48, 22],
					['H3', 319, 48, 23],
					['H3', 320, 48, 24],
					['H3', 323, 49, 2],
					['H3', 324, 49, 3],
					['H3', 325, 50, 1],
					['H3', 326, 50, 2],
					['H3', 387, 29, null],
					['H3', 388, 57, 1],
					['H3', 389, 57, 2],
					['H3', 393, 58, 1],
					['H3', 394, 58, 2],
					['H3', 395, 58, 3],
					['H3', 396, 58, 4],
					['H3', 397, 58, 5],
					['H3', 398, 58, 6],
					['H3', 399, 58, 7],
					['H3', 400, 58, 8],
					['H3', 401, 59, 1],
					['H3', 402, 59, 2],
					['H3', 403, 60, 1],
					['H3', 404, 60, 2],
					['H3', 416, 63, 1],
					['H3', 417, 63, 2],
					['H3', 418, 63, 3],
					['H3', 419, 64, 1],
					['H3', 420, 64, 2],
					['H3', 422, 65, 1],
					['H3', 423, 65, 2],
					['H3', 424, 65, 3],
					['H3', 425, 66, 1],
					['H3', 426, 66, 2],
					['H3', 427, 66, 3],
					['H3', 440, 68, 1],
					['H3', 444, 68, 2],
					['H3', 445, 68, 3],
					['H3', 446, 14, 12],
					['H3', 451, 14, 17],
					['H3', 452, 14, 18],
					['H3', 453, 14, 19],
					['H3', 455, 11, 2],
					['H3', 456, 11, 3],
					['H3', 457, 11, 4],
					['H3', 458, 11, 5],
					['H3', 459, 11, 6],
					['H3', 460, 11, 7],
					['H3', 461, 11, 8],
					['H3', 462, 11, 9],
					['H3', 463, 11, 10],
					['H3', 464, 11, 11],
					['H3', 465, 11, 12],
					['H3', 508, 37, 3],
					['H3', 509, 37, 4],
					['H3', 510, 37, 5],
					['H3', 511, 37, 6],
					['H3', 557, 29, 4],
					['H3', 559, 70, 1],
					['H3', 560, 70, 2],
					['H3', 564, 71, 1],
					['H3', 565, 71, 2],
					['H3', 576, 74, 1],
					['H3', 577, 74, 2],
					['H3', 578, 74, 3],
					['H3', 579, 74, 4],
					['H3', 580, 74, 5],
					['H3', 581, 74, 6],
					['H3', 582, 74, 7],
					['H3', 583, 74, 8],
					['H3', 584, 74, 9],
					['H3', 585, 74, 10],
					['H3', 586, 74, 11],
					['H3', 587, 75, 1],
					['H3', 588, 75, 2],
					['H3', 589, 75, 3],
					['H3', 590, 76, 1],
					['H3', 591, 76, 2],
					['H3', 592, 76, 3],
					['H3', 593, 77, 1],
					['H3', 594, 77, 2],
					['H3', 595, 77, 3],
					['H3', 596, 77, 4],
					['H3', 597, 77, 5],
					['H3', 598, 77, 6],
					['H3', 599, 77, 7],
					['H3', 600, 77, 8],
					['H3', 601, 77, 9],
					['H3', 602, 77, 10],
					['H3', 603, 77, 11],
					['H3', 604, 77, 12],
					['H3', 605, 77, 13],
					['H3', 606, 77, 14],
					['H3', 607, 77, 15],
					['H3', 608, 78, 1],
					['H3', 609, 78, 2],
					['H3', 610, 78, 3],
					['H3', 611, 78, 4],
					['H3', 612, 78, 5],
					['H3', 613, 78, 6],
					['H3', 614, 78, 7],
					['H3', 615, 78, 8],
					['H3', 616, 78, 9],
					['H3', 617, 78, 10],
					['H3', 618, 78, 11],
					['H3', 619, 78, 12],
					['H3', 620, 78, 13],
					['H3', 621, 78, 14],
					['H3', 622, 78, 15],
					['H3', 659, 85, 1],
					['H3', 660, 85, 2],
					['H3', 661, 85, 3],
					['H3', 666, 86, 1],
					['H3', 667, 86, 2],
					['H3', 668, 87, 1],
					['H3', 669, 88, 1],
					['H3', 670, 89, 1],
					['H3', 671, 88, 2],
					['H3', 672, 37, 7],
					['H3', 675, 90, 1],
					['H3', 676, 90, 2],
					['H3', 677, 90, 3],
					['H3', 678, 90, 4],
					['H3', 679, 90, 5],
					['H3', 680, 90, 6],
					['H3', 681, 90, 7],
					['H3', 682, 91, 1],
					['H3', 683, 91, 2],
					['H3', 684, 91, 3],
					['H3', 685, 91, 4],
					['H3', 686, 91, 5],
					['H3', 687, 91, 6],
					['H3', 688, 91, 7],
					['H3', 689, 63, 4],
					['H3', 690, 63, 5],
					['H3', 691, 92, 1],
					['H3', 692, 93, 1],
					['H3', 693, 92, 2],
					['H3', 694, 29, 5],
					['H3', 695, 29, 6],
					['H3', 696, 29, 7],
					['H3', 697, 94, 1],
					['H3', 698, 94, 2],
					['H3', 699, 94, 3],
					['H3', 700, 57, 3],
					['H3', 701, 90, 8],
					['H3', 702, 91, 8],
					['H3', 703, 95, 1],
					['H3', 704, 95, 2],
					['H3', 705, 96, 1],
					['H3', 706, 96, 2],
					['H3', 707, 96, 3],
					['H3', 708, 96, 4],
					['H3', 709, 96, 5],
					['H3', 710, 96, 6],
					['H3', 711, 96, 7],
					['H3', 712, 96, 8],
					['H3', 713, 86, 3],
					['H3', 714, 97, 1],
					['H3', 715, 97, 2],
					['H3', 716, 97, 3],
					['H3', 717, 98, 1],
					['H3', 718, 98, 2],
					['H3', 719, 98, 3],
					['H3', 720, 14, 20],
					['H3', 721, 14, 21],
					['H3', 722, 14, 22],
					['H3', 723, 99, 1],
					['H3', 724, 99, 2],
					['H3', 725, 99, 3],
					['H3', 726, 99, 4],
					['H3', 727, 99, 5],
					['H3', 728, 100, 1],
					['H3', 729, 100, 2],
					['H3', 730, 100, 3],
					['H3', 731, 100, 4],
					['H3', 732, 100, 5],
					['H3', 733, 100, 6],
					['H3', 734, 100, 7],
					['H3', 735, 100, 8],
					['H3', 736, 100, 9],
					['H3', 738, 102, 1],
					['H3', 739, 102, 2],
					['H3', 740, 102, 3],
					['H3', 741, 102, 4],
					['H3', 742, 102, 5],
					['H3', 743, 102, 6],
					['H3', 744, 102, 7],
					['H3', 745, 102, 8],
					['H3', 746, 103, 1],
					['H3', 747, 103, 2],
					['H3', 748, 103, 3],
					['H3', 749, 104, 1],
					['H3', 750, 104, 2],
					['H3', 751, 104, 3],
					['H3', 752, 104, 4],
					['H3', 753, 104, 5],
					['H3', 754, 104, 6],
					['H3', 755, 104, 7],
					['H3', 756, 104, 8],
					['H3', 757, 105, 1],
					['H3', 758, 105, 2],
					['H3', 759, 105, 3],
					['H3', 760, 106, 1],
					['H3', 761, 106, 2],
					['H3', 762, 106, 3],
					['H3', 763, 106, 4],
					['H3', 766, 87, 2],
					['H3', 767, 87, 3],
					['H3', 768, 87, 4],
					['H3', 769, 87, 5],
					['H3', 770, 87, 6],
					['H3', 771, 87, 7],
					['H3', 772, 87, 8],
					['H3', 773, 107, 1],
					['H3', 774, 107, 2],
					['H3', 775, 107, 3],
					['H3', 776, 107, 4],
					['H3', 777, 107, 5],
					['H3', 778, 108, 1],
					['H3', 779, 108, 2],
					['H3', 780, 108, 3],
					['H3', 781, 108, 4],
					['H3', 782, 108, 5],
					['H3', 783, 108, 6],
					['H3', 784, 109, 1],
					['H3', 785, 109, 2],
					['H3', 786, 109, 3],
					['H3', 787, 109, 4],
					['H3', 788, 109, 5],
					['H3', 789, 109, 6],
					['H3', 790, 110, 1],
					['H3', 791, 110, 2],
					['H3', 792, 110, 3],
					['H3', 793, 110, 4],
					['H3', 794, 110, 5],
					['H3', 795, 110, 6],
					['H3', 796, 111, 1],
					['H3', 797, 111, 2],
					['H3', 798, 111, 3],
					['H3', 799, 111, 4],
					['H3', 800, 111, 5],
					['H3', 801, 111, 6],
					['H3', 802, 112, 1],
					['H3', 803, 112, 2],
					['H3', 804, 112, 3],
					['H3', 805, 112, 4],
					['H3', 806, 112, 5],
					['H3', 807, 112, 6],
					['H3', 808, 113, 1],
					['H3', 809, 113, 2],
					['H3', 810, 113, 3],
					['H3', 811, 113, 4],
					['H3', 812, 113, 5],
					['H3', 813, 113, 6],
					['H3', 814, 114, 1],
					['H3', 815, 114, 2],
					['H3', 816, 114, 3],
					['H3', 817, 114, 4],
					['H3', 818, 114, 5],
					['H3', 819, 114, 6],
					['H3', 820, 115, 1],
					['H3', 821, 115, 2],
					['H3', 822, 115, 3],
					['H3', 823, 115, 4],
					['H3', 824, 115, 5],
					['H3', 825, 115, 6],
					['H3', 826, 116, 1],
					['H3', 827, 116, 2],
					['H3', 828, 116, 3],
					['H3', 829, 116, 4],
					['H3', 830, 116, 5],
					['H3', 831, 116, 6],
					['H3', 832, 117, 1],
					['H3', 833, 117, 2],
					['H3', 834, 117, 3],
					['H3', 835, 117, 4],
					['H3', 836, 117, 5],
					['H3', 837, 117, 6],
					['H3', 838, 118, 1],
					['H3', 839, 118, 2],
					['H3', 840, 118, 3],
					['H3', 841, 118, 4],
					['H3', 842, 118, 5],
					['H3', 843, 118, 6],
					['H3', 844, 118, 7],
					['H3', 845, 119, 1],
					['H3', 846, 119, 2],
					['H3', 847, 120, 1],
					['H3', 848, 120, 2],
					['H3', 849, 121, 1],
					['H3', 850, 121, 2],
					['H3', 851, 122, 2],
					['H3', 852, 122, 3],
					['H3', 853, 122, 1],
					['H3', 854, 122, 4],
					['H3', 855, 123, 1],
					['H3', 856, 123, 2],
					['H3', 857, 123, 3],
					['H3', 858, 123, 4],
					['H3', 859, 123, 5],
					['H3', 860, 124, 1],
					['H3', 861, 124, 2],
					['H3', 862, 124, 3],
					['H3', 863, 124, 4],
					['H3', 864, 124, 5],
					['H3', 865, 125, 1],
					['H3', 866, 125, 2],
					['H3', 867, 125, 3],
					['H3', 868, 125, 4],
					['H3', 869, 125, 5],
					['H3', 870, 125, 6],
					['H3', 871, 125, 7],
					['H3', 872, 126, 1],
					['H3', 873, 126, 2],
					['H3', 874, 126, 3],
					['H3', 875, 126, 4],
					['H3', 876, 126, 5],
					['H3', 877, 126, 6],
					['H3', 878, 126, 7],
					['H3', 879, 126, 8],
					['H3', 880, 126, 9],
					['H3', 881, 127, 1],
					['H3', 882, 127, 2],
					['H3', 883, 127, 3],
					['H3', 884, 127, 4],
					['H3', 885, 127, 5],
					['H3', 886, 128, 1],
					['H3', 887, 128, 2],
					['H3', 888, 128, 3],
					['H3', 889, 128, 4],
					['H3', 890, 128, 5],
					['H3', 891, 128, 6],
					['H3', 892, 128, 7],
					['H3', 893, 128, 8],
					['H3', 894, 128, 9],
					['H3', 895, 128, 10],
					['H3', 896, 128, 11],
					['H3', 897, 128, 12],
					['H3', 898, 128, 13],
					['H3', 899, 128, 14],
					['H3', 900, 128, 15],
					['H3', 901, 128, 16],
					['H3', 902, 129, 1],
					['H3', 903, 129, 2],
					['H3', 904, 129, 3],
					['H3', 905, 129, 4],
					['H3', 906, 129, 5],
					['H3', 907, 129, 6],
					['H3', 908, 130, 1],
					['H3', 909, 130, 2],
					['H30', 1, 1, 0],
					['H30', 2, 1, 1],
					['H30', 3, 1, 2],
					['H30', 4, 1, 3],
					['H30', 5, 1, 4],
					['H30', 6, 1, 5],
					['H30', 7, 1, 6],
					['H30', 8, 1, 7],
					['H30', 9, 1, 8],
					['H30', 10, 1, 9],
					['H30', 11, 1, 10],
					['H30', 12, 2, 0],
					['H30', 13, 2, 1],
					['H30', 14, 3, 0],
					['H30', 15, 3, 1],
					['H30', 16, 3, 2],
					['H30', 17, 3, 3],
					['H30', 18, 3, 4],
					['H30', 19, 3, 5],
					['H30', 20, 4, 0],
					['H30', 21, 4, 1],
					['H30', 22, 4, 2],
					['H30', 23, 4, 3],
					['H30', 24, 4, 4],
					['H30', 25, 4, 5],
					['H30', 26, 4, 6],
					['H30', 27, 4, 7],
					['H30', 28, 4, 8],
					['H30', 29, 4, 9],
					['H30', 30, 4, 10],
					['H30', 31, 4, 11],
					['H30', 32, 4, 12],
					['H30', 39, 6, 1],
					['H30', 40, 6, 2],
					['H30', 41, 7, 0],
					['H30', 42, 7, 1],
					['H30', 43, 7, 2],
					['H30', 44, 7, 3],
					['H30', 45, 7, 4],
					['H30', 46, 8, 0],
					['H30', 47, 9, 0],
					['H30', 48, 9, 1],
					['H30', 49, 9, 2],
					['H30', 50, 9, 3],
					['H30', 51, 10, 0],
					['H30', 52, 10, 1],
					['H30', 53, 10, 2],
					['H30', 54, 10, 3],
					['H30', 55, 10, 4],
					['H30', 56, 10, 5],
					['H30', 57, 10, 6],
					['H30', 58, 10, 7],
					['H30', 59, 10, 8],
					['H30', 60, 11, 0],
					['H30', 61, 11, 1],
					['H30', 98, 13, 0],
					['H30', 99, 13, 1],
					['H30', 101, 13, 3],
					['H30', 106, 13, 8],
					['H30', 107, 13, 9],
					['H30', 108, 13, 10],
					['H30', 109, 13, 11],
					['H30', 110, 13, 12],
					['H30', 111, 14, 0],
					['H30', 123, 15, 0],
					['H30', 124, 15, 1],
					['H30', 125, 15, 2],
					['H30', 126, 15, 3],
					['H30', 160, 22, 0],
					['H30', 161, 22, 1],
					['H30', 162, 22, 2],
					['H30', 163, 22, 3],
					['H30', 164, 22, 4],
					['H30', 165, 22, 5],
					['H30', 170, 24, 0],
					['H30', 171, 24, 1],
					['H30', 172, 24, 2],
					['H30', 173, 25, 0],
					['H30', 174, 25, 1],
					['H30', 175, 25, 2],
					['H30', 176, 25, 3],
					['H30', 177, 25, 4],
					['H30', 178, 25, 5],
					['H30', 182, 27, 0],
					['H30', 183, 27, 1],
					['H30', 184, 27, 2],
					['H30', 185, 27, 3],
					['H30', 186, 28, 0],
					['H30', 187, 28, 1],
					['H30', 188, 28, 2],
					['H30', 189, 28, 3],
					['H30', 190, 29, 0],
					['H30', 191, 29, 1],
					['H30', 192, 29, 2],
					['H30', 193, 29, 3],
					['H30', 194, 30, 0],
					['H30', 195, 30, 1],
					['H30', 196, 30, 2],
					['H30', 197, 30, 3],
					['H30', 198, 30, 4],
					['H30', 210, 31, 1],
					['H30', 211, 31, 2],
					['H30', 212, 31, 3],
					['H30', 213, 31, 4],
					['H30', 214, 32, 1],
					['H30', 215, 32, 2],
					['H30', 216, 32, 3],
					['H30', 217, 32, 4],
					['H30', 218, 32, 5],
					['H30', 219, 32, 6],
					['H30', 220, 33, 1],
					['H30', 221, 33, 2],
					['H30', 222, 33, 3],
					['H30', 223, 34, 1],
					['H30', 224, 34, 2],
					['H30', 225, 34, 3],
					['H30', 226, 35, 1],
					['H30', 227, 35, 2],
					['H30', 228, 35, 3],
					['H30', 237, 38, 1],
					['H30', 238, 38, 2],
					['H30', 239, 38, 3],
					['H30', 240, 38, 4],
					['H30', 241, 39, 1],
					['H30', 242, 39, 2],
					['H30', 243, 39, 3],
					['H30', 244, 39, 4],
					['H30', 245, 40, 1],
					['H30', 246, 40, 2],
					['H30', 247, 40, 3],
					['H30', 248, 40, 4],
					['H30', 249, 41, 1],
					['H30', 250, 41, 2],
					['H30', 251, 41, 3],
					['H30', 252, 41, 4],
					['H30', 253, 41, 5],
					['H30', 254, 41, 6],
					['H30', 255, 41, 7],
					['H30', 256, 41, 8],
					['H30', 257, 41, 9],
					['H30', 258, 41, 10],
					['H30', 259, 41, 11],
					['H30', 260, 42, 1],
					['H30', 261, 42, 2],
					['H30', 262, 42, 3],
					['H30', 263, 42, 4],
					['H30', 264, 42, 5],
					['H30', 265, 42, 6],
					['H30', 266, 42, 7],
					['H30', 267, 42, 8],
					['H30', 268, 42, 9],
					['H30', 269, 43, 1],
					['H30', 270, 43, 2],
					['H30', 271, 43, 3],
					['H30', 272, 43, 4],
					['H30', 273, 44, 1],
					['H30', 274, 44, 2],
					['H30', 275, 44, 3],
					['H30', 276, 44, 4],
					['H30', 277, 45, 1],
					['H30', 278, 45, 2],
					['H30', 279, 45, 3],
					['H30', 280, 45, 4],
					['H30', 281, 45, 5],
					['H30', 282, 45, 6],
					['H30', 283, 45, 7],
					['H30', 284, 45, 8],
					['H30', 285, 45, 9],
					['H30', 286, 45, 10],
					['H30', 287, 45, 11],
					['H30', 289, 46, 1],
					['H30', 290, 46, 2],
					['H30', 291, 46, 3],
					['H30', 292, 46, 4],
					['H30', 293, 46, 5],
					['H30', 294, 46, 6],
					['H30', 295, 47, 1],
					['H30', 296, 47, 2],
					['H30', 297, 48, 1],
					['H30', 298, 48, 2],
					['H30', 299, 48, 3],
					['H30', 300, 48, 4],
					['H30', 301, 48, 5],
					['H30', 302, 48, 6],
					['H30', 303, 48, 7],
					['H30', 304, 48, 8],
					['H30', 305, 48, 9],
					['H30', 306, 48, 10],
					['H30', 307, 48, 11],
					['H30', 308, 48, 12],
					['H30', 309, 48, 13],
					['H30', 310, 48, 14],
					['H30', 311, 48, 15],
					['H30', 312, 48, 16],
					['H30', 313, 48, 17],
					['H30', 314, 48, 18],
					['H30', 315, 48, 19],
					['H30', 316, 48, 20],
					['H30', 317, 48, 21],
					['H30', 318, 48, 22],
					['H30', 319, 48, 23],
					['H30', 320, 48, 24],
					['H30', 323, 49, 2],
					['H30', 324, 49, 3],
					['H30', 325, 50, 1],
					['H30', 326, 50, 2],
					['H30', 387, 29, null],
					['H30', 388, 57, 1],
					['H30', 389, 57, 2],
					['H30', 393, 58, 1],
					['H30', 394, 58, 2],
					['H30', 395, 58, 3],
					['H30', 396, 58, 4],
					['H30', 397, 58, 5],
					['H30', 398, 58, 6],
					['H30', 399, 58, 7],
					['H30', 400, 58, 8],
					['H30', 401, 59, 1],
					['H30', 402, 59, 2],
					['H30', 403, 60, 1],
					['H30', 404, 60, 2],
					['H30', 416, 63, 1],
					['H30', 417, 63, 2],
					['H30', 418, 63, 3],
					['H30', 419, 64, 1],
					['H30', 420, 64, 2],
					['H30', 422, 65, 1],
					['H30', 423, 65, 2],
					['H30', 424, 65, 3],
					['H30', 425, 66, 1],
					['H30', 426, 66, 2],
					['H30', 427, 66, 3],
					['H30', 440, 68, 1],
					['H30', 444, 68, 2],
					['H30', 445, 68, 3],
					['H30', 446, 14, 12],
					['H30', 451, 14, 17],
					['H30', 452, 14, 18],
					['H30', 453, 14, 19],
					['H30', 455, 11, 2],
					['H30', 456, 11, 3],
					['H30', 457, 11, 4],
					['H30', 458, 11, 5],
					['H30', 459, 11, 6],
					['H30', 460, 11, 7],
					['H30', 461, 11, 8],
					['H30', 462, 11, 9],
					['H30', 463, 11, 10],
					['H30', 464, 11, 11],
					['H30', 465, 11, 12],
					['H30', 508, 37, 3],
					['H30', 509, 37, 4],
					['H30', 510, 37, 5],
					['H30', 511, 37, 6],
					['H30', 557, 29, 4],
					['H30', 559, 70, 1],
					['H30', 560, 70, 2],
					['H30', 564, 71, 1],
					['H30', 565, 71, 2],
					['H30', 576, 74, 1],
					['H30', 577, 74, 2],
					['H30', 578, 74, 3],
					['H30', 579, 74, 4],
					['H30', 580, 74, 5],
					['H30', 581, 74, 6],
					['H30', 582, 74, 7],
					['H30', 583, 74, 8],
					['H30', 584, 74, 9],
					['H30', 585, 74, 10],
					['H30', 586, 74, 11],
					['H30', 587, 75, 1],
					['H30', 588, 75, 2],
					['H30', 589, 75, 3],
					['H30', 590, 76, 1],
					['H30', 591, 76, 2],
					['H30', 592, 76, 3],
					['H30', 593, 77, 1],
					['H30', 594, 77, 2],
					['H30', 595, 77, 3],
					['H30', 596, 77, 4],
					['H30', 597, 77, 5],
					['H30', 598, 77, 6],
					['H30', 599, 77, 7],
					['H30', 600, 77, 8],
					['H30', 601, 77, 9],
					['H30', 602, 77, 10],
					['H30', 603, 77, 11],
					['H30', 604, 77, 12],
					['H30', 605, 77, 13],
					['H30', 606, 77, 14],
					['H30', 607, 77, 15],
					['H30', 608, 78, 1],
					['H30', 609, 78, 2],
					['H30', 610, 78, 3],
					['H30', 611, 78, 4],
					['H30', 612, 78, 5],
					['H30', 613, 78, 6],
					['H30', 614, 78, 7],
					['H30', 615, 78, 8],
					['H30', 616, 78, 9],
					['H30', 617, 78, 10],
					['H30', 618, 78, 11],
					['H30', 619, 78, 12],
					['H30', 620, 78, 13],
					['H30', 621, 78, 14],
					['H30', 622, 78, 15],
					['H30', 659, 85, 1],
					['H30', 660, 85, 2],
					['H30', 661, 85, 3],
					['H30', 666, 86, 1],
					['H30', 667, 86, 2],
					['H30', 668, 87, 1],
					['H30', 669, 88, 1],
					['H30', 670, 89, 1],
					['H30', 671, 88, 2],
					['H30', 672, 37, 7],
					['H30', 675, 90, 1],
					['H30', 676, 90, 2],
					['H30', 677, 90, 3],
					['H30', 678, 90, 4],
					['H30', 679, 90, 5],
					['H30', 680, 90, 6],
					['H30', 681, 90, 7],
					['H30', 682, 91, 1],
					['H30', 683, 91, 2],
					['H30', 684, 91, 3],
					['H30', 685, 91, 4],
					['H30', 686, 91, 5],
					['H30', 687, 91, 6],
					['H30', 688, 91, 7],
					['H30', 689, 63, 4],
					['H30', 690, 63, 5],
					['H30', 691, 92, 1],
					['H30', 692, 93, 1],
					['H30', 693, 92, 2],
					['H30', 694, 29, 5],
					['H30', 695, 29, 6],
					['H30', 696, 29, 7],
					['H30', 697, 94, 1],
					['H30', 698, 94, 2],
					['H30', 699, 94, 3],
					['H30', 700, 57, 3],
					['H30', 701, 90, 8],
					['H30', 702, 91, 8],
					['H30', 703, 95, 1],
					['H30', 704, 95, 2],
					['H30', 705, 96, 1],
					['H30', 706, 96, 2],
					['H30', 707, 96, 3],
					['H30', 708, 96, 4],
					['H30', 709, 96, 5],
					['H30', 710, 96, 6],
					['H30', 711, 96, 7],
					['H30', 712, 96, 8],
					['H30', 713, 86, 3],
					['H30', 714, 97, 1],
					['H30', 715, 97, 2],
					['H30', 716, 97, 3],
					['H30', 717, 98, 1],
					['H30', 718, 98, 2],
					['H30', 719, 98, 3],
					['H30', 720, 14, 20],
					['H30', 721, 14, 21],
					['H30', 722, 14, 22],
					['H30', 723, 99, 1],
					['H30', 724, 99, 2],
					['H30', 725, 99, 3],
					['H30', 726, 99, 4],
					['H30', 727, 99, 5],
					['H30', 728, 100, 1],
					['H30', 729, 100, 2],
					['H30', 730, 100, 3],
					['H30', 731, 100, 4],
					['H30', 732, 100, 5],
					['H30', 733, 100, 6],
					['H30', 734, 100, 7],
					['H30', 735, 100, 8],
					['H30', 736, 100, 9],
					['H30', 738, 102, 1],
					['H30', 739, 102, 2],
					['H30', 740, 102, 3],
					['H30', 741, 102, 4],
					['H30', 742, 102, 5],
					['H30', 743, 102, 6],
					['H30', 744, 102, 7],
					['H30', 745, 102, 8],
					['H30', 746, 103, 1],
					['H30', 747, 103, 2],
					['H30', 748, 103, 3],
					['H30', 749, 104, 1],
					['H30', 750, 104, 2],
					['H30', 751, 104, 3],
					['H30', 752, 104, 4],
					['H30', 753, 104, 5],
					['H30', 754, 104, 6],
					['H30', 755, 104, 7],
					['H30', 756, 104, 8],
					['H30', 757, 105, 1],
					['H30', 758, 105, 2],
					['H30', 759, 105, 3],
					['H30', 760, 106, 1],
					['H30', 761, 106, 2],
					['H30', 762, 106, 3],
					['H30', 763, 106, 4],
					['H30', 766, 87, 2],
					['H30', 767, 87, 3],
					['H30', 768, 87, 4],
					['H30', 769, 87, 5],
					['H30', 770, 87, 6],
					['H30', 771, 87, 7],
					['H30', 772, 87, 8],
					['H30', 773, 107, 1],
					['H30', 774, 107, 2],
					['H30', 775, 107, 3],
					['H30', 776, 107, 4],
					['H30', 777, 107, 5],
					['H30', 778, 108, 1],
					['H30', 779, 108, 2],
					['H30', 780, 108, 3],
					['H30', 781, 108, 4],
					['H30', 782, 108, 5],
					['H30', 783, 108, 6],
					['H30', 784, 109, 1],
					['H30', 785, 109, 2],
					['H30', 786, 109, 3],
					['H30', 787, 109, 4],
					['H30', 788, 109, 5],
					['H30', 789, 109, 6],
					['H30', 790, 110, 1],
					['H30', 791, 110, 2],
					['H30', 792, 110, 3],
					['H30', 793, 110, 4],
					['H30', 794, 110, 5],
					['H30', 795, 110, 6],
					['H30', 796, 111, 1],
					['H30', 797, 111, 2],
					['H30', 798, 111, 3],
					['H30', 799, 111, 4],
					['H30', 800, 111, 5],
					['H30', 801, 111, 6],
					['H30', 802, 112, 1],
					['H30', 803, 112, 2],
					['H30', 804, 112, 3],
					['H30', 805, 112, 4],
					['H30', 806, 112, 5],
					['H30', 807, 112, 6],
					['H30', 808, 113, 1],
					['H30', 809, 113, 2],
					['H30', 810, 113, 3],
					['H30', 811, 113, 4],
					['H30', 812, 113, 5],
					['H30', 813, 113, 6],
					['H30', 814, 114, 1],
					['H30', 815, 114, 2],
					['H30', 816, 114, 3],
					['H30', 817, 114, 4],
					['H30', 818, 114, 5],
					['H30', 819, 114, 6],
					['H30', 820, 115, 1],
					['H30', 821, 115, 2],
					['H30', 822, 115, 3],
					['H30', 823, 115, 4],
					['H30', 824, 115, 5],
					['H30', 825, 115, 6],
					['H30', 826, 116, 1],
					['H30', 827, 116, 2],
					['H30', 828, 116, 3],
					['H30', 829, 116, 4],
					['H30', 830, 116, 5],
					['H30', 831, 116, 6],
					['H30', 832, 117, 1],
					['H30', 833, 117, 2],
					['H30', 834, 117, 3],
					['H30', 835, 117, 4],
					['H30', 836, 117, 5],
					['H30', 837, 117, 6],
					['H30', 838, 118, 1],
					['H30', 839, 118, 2],
					['H30', 840, 118, 3],
					['H30', 841, 118, 4],
					['H30', 842, 118, 5],
					['H30', 843, 118, 6],
					['H30', 844, 118, 7],
					['H30', 845, 119, 1],
					['H30', 846, 119, 2],
					['H30', 847, 120, 1],
					['H30', 848, 120, 2],
					['H30', 849, 121, 1],
					['H30', 850, 121, 2],
					['H30', 851, 122, 2],
					['H30', 852, 122, 3],
					['H30', 853, 122, 1],
					['H30', 854, 122, 4],
					['H30', 855, 123, 1],
					['H30', 856, 123, 2],
					['H30', 857, 123, 3],
					['H30', 858, 123, 4],
					['H30', 859, 123, 5],
					['H30', 860, 124, 1],
					['H30', 861, 124, 2],
					['H30', 862, 124, 3],
					['H30', 863, 124, 4],
					['H30', 864, 124, 5],
					['H30', 865, 125, 1],
					['H30', 866, 125, 2],
					['H30', 867, 125, 3],
					['H30', 868, 125, 4],
					['H30', 869, 125, 5],
					['H30', 870, 125, 6],
					['H30', 871, 125, 7],
					['H30', 872, 126, 1],
					['H30', 873, 126, 2],
					['H30', 874, 126, 3],
					['H30', 875, 126, 4],
					['H30', 876, 126, 5],
					['H30', 877, 126, 6],
					['H30', 878, 126, 7],
					['H30', 879, 126, 8],
					['H30', 880, 126, 9],
					['H30', 881, 127, 1],
					['H30', 882, 127, 2],
					['H30', 883, 127, 3],
					['H30', 884, 127, 4],
					['H30', 885, 127, 5],
					['H30', 886, 128, 1],
					['H30', 887, 128, 2],
					['H30', 888, 128, 3],
					['H30', 889, 128, 4],
					['H30', 890, 128, 5],
					['H30', 891, 128, 6],
					['H30', 892, 128, 7],
					['H30', 893, 128, 8],
					['H30', 894, 128, 9],
					['H30', 895, 128, 10],
					['H30', 896, 128, 11],
					['H30', 897, 128, 12],
					['H30', 898, 128, 13],
					['H30', 899, 128, 14],
					['H30', 900, 128, 15],
					['H30', 901, 128, 16],
					['H30', 902, 129, 1],
					['H30', 903, 129, 2],
					['H30', 904, 129, 3],
					['H30', 905, 129, 4],
					['H30', 906, 129, 5],
					['H30', 907, 129, 6],
					['H30', 908, 130, 1],
					['H30', 909, 130, 2],
					['H34', 1, 1, 0],
					['H34', 2, 1, 1],
					['H34', 3, 1, 2],
					['H34', 4, 1, 3],
					['H34', 5, 1, 4],
					['H34', 6, 1, 5],
					['H34', 7, 1, 6],
					['H34', 8, 1, 7],
					['H34', 9, 1, 8],
					['H34', 10, 1, 9],
					['H34', 11, 1, 10],
					['H34', 12, 2, 0],
					['H34', 13, 2, 1],
					['H34', 14, 3, 0],
					['H34', 15, 3, 1],
					['H34', 16, 3, 2],
					['H34', 17, 3, 3],
					['H34', 18, 3, 4],
					['H34', 19, 3, 5],
					['H34', 20, 4, 0],
					['H34', 21, 4, 1],
					['H34', 22, 4, 2],
					['H34', 23, 4, 3],
					['H34', 24, 4, 4],
					['H34', 25, 4, 5],
					['H34', 26, 4, 6],
					['H34', 27, 4, 7],
					['H34', 28, 4, 8],
					['H34', 29, 4, 9],
					['H34', 30, 4, 10],
					['H34', 31, 4, 11],
					['H34', 32, 4, 12],
					['H34', 39, 6, 1],
					['H34', 40, 6, 2],
					['H34', 41, 7, 0],
					['H34', 42, 7, 1],
					['H34', 43, 7, 2],
					['H34', 44, 7, 3],
					['H34', 45, 7, 4],
					['H34', 46, 8, 0],
					['H34', 47, 9, 0],
					['H34', 48, 9, 1],
					['H34', 49, 9, 2],
					['H34', 50, 9, 3],
					['H34', 51, 10, 0],
					['H34', 52, 10, 1],
					['H34', 53, 10, 2],
					['H34', 54, 10, 3],
					['H34', 55, 10, 4],
					['H34', 56, 10, 5],
					['H34', 57, 10, 6],
					['H34', 58, 10, 7],
					['H34', 59, 10, 8],
					['H34', 60, 11, 0],
					['H34', 61, 11, 1],
					['H34', 98, 13, 0],
					['H34', 99, 13, 1],
					['H34', 101, 13, 3],
					['H34', 106, 13, 8],
					['H34', 107, 13, 9],
					['H34', 108, 13, 10],
					['H34', 109, 13, 11],
					['H34', 110, 13, 12],
					['H34', 111, 14, 0],
					['H34', 123, 15, 0],
					['H34', 124, 15, 1],
					['H34', 125, 15, 2],
					['H34', 126, 15, 3],
					['H34', 160, 22, 0],
					['H34', 161, 22, 1],
					['H34', 162, 22, 2],
					['H34', 163, 22, 3],
					['H34', 164, 22, 4],
					['H34', 165, 22, 5],
					['H34', 170, 24, 0],
					['H34', 171, 24, 1],
					['H34', 172, 24, 2],
					['H34', 173, 25, 0],
					['H34', 174, 25, 1],
					['H34', 175, 25, 2],
					['H34', 176, 25, 3],
					['H34', 177, 25, 4],
					['H34', 178, 25, 5],
					['H34', 182, 27, 0],
					['H34', 183, 27, 1],
					['H34', 184, 27, 2],
					['H34', 185, 27, 3],
					['H34', 186, 28, 0],
					['H34', 187, 28, 1],
					['H34', 188, 28, 2],
					['H34', 189, 28, 3],
					['H34', 190, 29, 0],
					['H34', 191, 29, 1],
					['H34', 192, 29, 2],
					['H34', 193, 29, 3],
					['H34', 194, 30, 0],
					['H34', 195, 30, 1],
					['H34', 196, 30, 2],
					['H34', 197, 30, 3],
					['H34', 198, 30, 4],
					['H34', 210, 31, 1],
					['H34', 211, 31, 2],
					['H34', 212, 31, 3],
					['H34', 213, 31, 4],
					['H34', 214, 32, 1],
					['H34', 215, 32, 2],
					['H34', 216, 32, 3],
					['H34', 217, 32, 4],
					['H34', 218, 32, 5],
					['H34', 219, 32, 6],
					['H34', 220, 33, 1],
					['H34', 221, 33, 2],
					['H34', 222, 33, 3],
					['H34', 223, 34, 1],
					['H34', 224, 34, 2],
					['H34', 225, 34, 3],
					['H34', 226, 35, 1],
					['H34', 227, 35, 2],
					['H34', 228, 35, 3],
					['H34', 237, 38, 1],
					['H34', 238, 38, 2],
					['H34', 239, 38, 3],
					['H34', 240, 38, 4],
					['H34', 241, 39, 1],
					['H34', 242, 39, 2],
					['H34', 243, 39, 3],
					['H34', 244, 39, 4],
					['H34', 245, 40, 1],
					['H34', 246, 40, 2],
					['H34', 247, 40, 3],
					['H34', 248, 40, 4],
					['H34', 249, 41, 1],
					['H34', 250, 41, 2],
					['H34', 251, 41, 3],
					['H34', 252, 41, 4],
					['H34', 253, 41, 5],
					['H34', 254, 41, 6],
					['H34', 255, 41, 7],
					['H34', 256, 41, 8],
					['H34', 257, 41, 9],
					['H34', 258, 41, 10],
					['H34', 259, 41, 11],
					['H34', 260, 42, 1],
					['H34', 261, 42, 2],
					['H34', 262, 42, 3],
					['H34', 263, 42, 4],
					['H34', 264, 42, 5],
					['H34', 265, 42, 6],
					['H34', 266, 42, 7],
					['H34', 267, 42, 8],
					['H34', 268, 42, 9],
					['H34', 269, 43, 1],
					['H34', 270, 43, 2],
					['H34', 271, 43, 3],
					['H34', 272, 43, 4],
					['H34', 273, 44, 1],
					['H34', 274, 44, 2],
					['H34', 275, 44, 3],
					['H34', 276, 44, 4],
					['H34', 277, 45, 1],
					['H34', 278, 45, 2],
					['H34', 279, 45, 3],
					['H34', 280, 45, 4],
					['H34', 281, 45, 5],
					['H34', 282, 45, 6],
					['H34', 283, 45, 7],
					['H34', 284, 45, 8],
					['H34', 285, 45, 9],
					['H34', 286, 45, 10],
					['H34', 287, 45, 11],
					['H34', 289, 46, 1],
					['H34', 290, 46, 2],
					['H34', 291, 46, 3],
					['H34', 292, 46, 4],
					['H34', 293, 46, 5],
					['H34', 294, 46, 6],
					['H34', 295, 47, 1],
					['H34', 296, 47, 2],
					['H34', 297, 48, 1],
					['H34', 298, 48, 2],
					['H34', 299, 48, 3],
					['H34', 300, 48, 4],
					['H34', 301, 48, 5],
					['H34', 302, 48, 6],
					['H34', 303, 48, 7],
					['H34', 304, 48, 8],
					['H34', 305, 48, 9],
					['H34', 306, 48, 10],
					['H34', 307, 48, 11],
					['H34', 308, 48, 12],
					['H34', 309, 48, 13],
					['H34', 310, 48, 14],
					['H34', 311, 48, 15],
					['H34', 312, 48, 16],
					['H34', 313, 48, 17],
					['H34', 314, 48, 18],
					['H34', 315, 48, 19],
					['H34', 316, 48, 20],
					['H34', 317, 48, 21],
					['H34', 318, 48, 22],
					['H34', 319, 48, 23],
					['H34', 320, 48, 24],
					['H34', 323, 49, 2],
					['H34', 324, 49, 3],
					['H34', 325, 50, 1],
					['H34', 326, 50, 2],
					['H34', 387, 29, null],
					['H34', 388, 57, 1],
					['H34', 389, 57, 2],
					['H34', 393, 58, 1],
					['H34', 394, 58, 2],
					['H34', 395, 58, 3],
					['H34', 396, 58, 4],
					['H34', 397, 58, 5],
					['H34', 398, 58, 6],
					['H34', 399, 58, 7],
					['H34', 400, 58, 8],
					['H34', 401, 59, 1],
					['H34', 402, 59, 2],
					['H34', 403, 60, 1],
					['H34', 404, 60, 2],
					['H34', 416, 63, 1],
					['H34', 417, 63, 2],
					['H34', 418, 63, 3],
					['H34', 419, 64, 1],
					['H34', 420, 64, 2],
					['H34', 422, 65, 1],
					['H34', 423, 65, 2],
					['H34', 424, 65, 3],
					['H34', 425, 66, 1],
					['H34', 426, 66, 2],
					['H34', 427, 66, 3],
					['H34', 440, 68, 1],
					['H34', 444, 68, 2],
					['H34', 445, 68, 3],
					['H34', 446, 14, 12],
					['H34', 451, 14, 17],
					['H34', 452, 14, 18],
					['H34', 453, 14, 19],
					['H34', 455, 11, 2],
					['H34', 456, 11, 3],
					['H34', 457, 11, 4],
					['H34', 458, 11, 5],
					['H34', 459, 11, 6],
					['H34', 460, 11, 7],
					['H34', 461, 11, 8],
					['H34', 462, 11, 9],
					['H34', 463, 11, 10],
					['H34', 464, 11, 11],
					['H34', 465, 11, 12],
					['H34', 508, 37, 3],
					['H34', 509, 37, 4],
					['H34', 510, 37, 5],
					['H34', 511, 37, 6],
					['H34', 557, 29, 4],
					['H34', 559, 70, 1],
					['H34', 560, 70, 2],
					['H34', 564, 71, 1],
					['H34', 565, 71, 2],
					['H34', 576, 74, 1],
					['H34', 577, 74, 2],
					['H34', 578, 74, 3],
					['H34', 579, 74, 4],
					['H34', 580, 74, 5],
					['H34', 581, 74, 6],
					['H34', 582, 74, 7],
					['H34', 583, 74, 8],
					['H34', 584, 74, 9],
					['H34', 585, 74, 10],
					['H34', 586, 74, 11],
					['H34', 587, 75, 1],
					['H34', 588, 75, 2],
					['H34', 589, 75, 3],
					['H34', 590, 76, 1],
					['H34', 591, 76, 2],
					['H34', 592, 76, 3],
					['H34', 593, 77, 1],
					['H34', 594, 77, 2],
					['H34', 595, 77, 3],
					['H34', 596, 77, 4],
					['H34', 597, 77, 5],
					['H34', 598, 77, 6],
					['H34', 599, 77, 7],
					['H34', 600, 77, 8],
					['H34', 601, 77, 9],
					['H34', 602, 77, 10],
					['H34', 603, 77, 11],
					['H34', 604, 77, 12],
					['H34', 605, 77, 13],
					['H34', 606, 77, 14],
					['H34', 607, 77, 15],
					['H34', 608, 78, 1],
					['H34', 609, 78, 2],
					['H34', 610, 78, 3],
					['H34', 611, 78, 4],
					['H34', 612, 78, 5],
					['H34', 613, 78, 6],
					['H34', 614, 78, 7],
					['H34', 615, 78, 8],
					['H34', 616, 78, 9],
					['H34', 617, 78, 10],
					['H34', 618, 78, 11],
					['H34', 619, 78, 12],
					['H34', 620, 78, 13],
					['H34', 621, 78, 14],
					['H34', 622, 78, 15],
					['H34', 659, 85, 1],
					['H34', 660, 85, 2],
					['H34', 661, 85, 3],
					['H34', 666, 86, 1],
					['H34', 667, 86, 2],
					['H34', 668, 87, 1],
					['H34', 669, 88, 1],
					['H34', 670, 89, 1],
					['H34', 671, 88, 2],
					['H34', 672, 37, 7],
					['H34', 675, 90, 1],
					['H34', 676, 90, 2],
					['H34', 677, 90, 3],
					['H34', 678, 90, 4],
					['H34', 679, 90, 5],
					['H34', 680, 90, 6],
					['H34', 681, 90, 7],
					['H34', 682, 91, 1],
					['H34', 683, 91, 2],
					['H34', 684, 91, 3],
					['H34', 685, 91, 4],
					['H34', 686, 91, 5],
					['H34', 687, 91, 6],
					['H34', 688, 91, 7],
					['H34', 689, 63, 4],
					['H34', 690, 63, 5],
					['H34', 691, 92, 1],
					['H34', 692, 93, 1],
					['H34', 693, 92, 2],
					['H34', 694, 29, 5],
					['H34', 695, 29, 6],
					['H34', 696, 29, 7],
					['H34', 697, 94, 1],
					['H34', 698, 94, 2],
					['H34', 699, 94, 3],
					['H34', 700, 57, 3],
					['H34', 701, 90, 8],
					['H34', 702, 91, 8],
					['H34', 703, 95, 1],
					['H34', 704, 95, 2],
					['H34', 705, 96, 1],
					['H34', 706, 96, 2],
					['H34', 707, 96, 3],
					['H34', 708, 96, 4],
					['H34', 709, 96, 5],
					['H34', 710, 96, 6],
					['H34', 711, 96, 7],
					['H34', 712, 96, 8],
					['H34', 713, 86, 3],
					['H34', 714, 97, 1],
					['H34', 715, 97, 2],
					['H34', 716, 97, 3],
					['H34', 717, 98, 1],
					['H34', 718, 98, 2],
					['H34', 719, 98, 3],
					['H34', 720, 14, 20],
					['H34', 721, 14, 21],
					['H34', 722, 14, 22],
					['H34', 723, 99, 1],
					['H34', 724, 99, 2],
					['H34', 725, 99, 3],
					['H34', 726, 99, 4],
					['H34', 727, 99, 5],
					['H34', 728, 100, 1],
					['H34', 729, 100, 2],
					['H34', 730, 100, 3],
					['H34', 731, 100, 4],
					['H34', 732, 100, 5],
					['H34', 733, 100, 6],
					['H34', 734, 100, 7],
					['H34', 735, 100, 8],
					['H34', 736, 100, 9],
					['H34', 738, 102, 1],
					['H34', 739, 102, 2],
					['H34', 740, 102, 3],
					['H34', 741, 102, 4],
					['H34', 742, 102, 5],
					['H34', 743, 102, 6],
					['H34', 744, 102, 7],
					['H34', 745, 102, 8],
					['H34', 746, 103, 1],
					['H34', 747, 103, 2],
					['H34', 748, 103, 3],
					['H34', 749, 104, 1],
					['H34', 750, 104, 2],
					['H34', 751, 104, 3],
					['H34', 752, 104, 4],
					['H34', 753, 104, 5],
					['H34', 754, 104, 6],
					['H34', 755, 104, 7],
					['H34', 756, 104, 8],
					['H34', 757, 105, 1],
					['H34', 758, 105, 2],
					['H34', 759, 105, 3],
					['H34', 760, 106, 1],
					['H34', 761, 106, 2],
					['H34', 762, 106, 3],
					['H34', 763, 106, 4],
					['H34', 766, 87, 2],
					['H34', 767, 87, 3],
					['H34', 768, 87, 4],
					['H34', 769, 87, 5],
					['H34', 770, 87, 6],
					['H34', 771, 87, 7],
					['H34', 772, 87, 8],
					['H34', 773, 107, 1],
					['H34', 774, 107, 2],
					['H34', 775, 107, 3],
					['H34', 776, 107, 4],
					['H34', 777, 107, 5],
					['H34', 778, 108, 1],
					['H34', 779, 108, 2],
					['H34', 780, 108, 3],
					['H34', 781, 108, 4],
					['H34', 782, 108, 5],
					['H34', 783, 108, 6],
					['H34', 784, 109, 1],
					['H34', 785, 109, 2],
					['H34', 786, 109, 3],
					['H34', 787, 109, 4],
					['H34', 788, 109, 5],
					['H34', 789, 109, 6],
					['H34', 790, 110, 1],
					['H34', 791, 110, 2],
					['H34', 792, 110, 3],
					['H34', 793, 110, 4],
					['H34', 794, 110, 5],
					['H34', 795, 110, 6],
					['H34', 796, 111, 1],
					['H34', 797, 111, 2],
					['H34', 798, 111, 3],
					['H34', 799, 111, 4],
					['H34', 800, 111, 5],
					['H34', 801, 111, 6],
					['H34', 802, 112, 1],
					['H34', 803, 112, 2],
					['H34', 804, 112, 3],
					['H34', 805, 112, 4],
					['H34', 806, 112, 5],
					['H34', 807, 112, 6],
					['H34', 808, 113, 1],
					['H34', 809, 113, 2],
					['H34', 810, 113, 3],
					['H34', 811, 113, 4],
					['H34', 812, 113, 5],
					['H34', 813, 113, 6],
					['H34', 814, 114, 1],
					['H34', 815, 114, 2],
					['H34', 816, 114, 3],
					['H34', 817, 114, 4],
					['H34', 818, 114, 5],
					['H34', 819, 114, 6],
					['H34', 820, 115, 1],
					['H34', 821, 115, 2],
					['H34', 822, 115, 3],
					['H34', 823, 115, 4],
					['H34', 824, 115, 5],
					['H34', 825, 115, 6],
					['H34', 826, 116, 1],
					['H34', 827, 116, 2],
					['H34', 828, 116, 3],
					['H34', 829, 116, 4],
					['H34', 830, 116, 5],
					['H34', 831, 116, 6],
					['H34', 832, 117, 1],
					['H34', 833, 117, 2],
					['H34', 834, 117, 3],
					['H34', 835, 117, 4],
					['H34', 836, 117, 5],
					['H34', 837, 117, 6],
					['H34', 838, 118, 1],
					['H34', 839, 118, 2],
					['H34', 840, 118, 3],
					['H34', 841, 118, 4],
					['H34', 842, 118, 5],
					['H34', 843, 118, 6],
					['H34', 844, 118, 7],
					['H34', 845, 119, 1],
					['H34', 846, 119, 2],
					['H34', 847, 120, 1],
					['H34', 848, 120, 2],
					['H34', 849, 121, 1],
					['H34', 850, 121, 2],
					['H34', 851, 122, 2],
					['H34', 852, 122, 3],
					['H34', 853, 122, 1],
					['H34', 854, 122, 4],
					['H34', 855, 123, 1],
					['H34', 856, 123, 2],
					['H34', 857, 123, 3],
					['H34', 858, 123, 4],
					['H34', 859, 123, 5],
					['H34', 860, 124, 1],
					['H34', 861, 124, 2],
					['H34', 862, 124, 3],
					['H34', 863, 124, 4],
					['H34', 864, 124, 5],
					['H34', 865, 125, 1],
					['H34', 866, 125, 2],
					['H34', 867, 125, 3],
					['H34', 868, 125, 4],
					['H34', 869, 125, 5],
					['H34', 870, 125, 6],
					['H34', 871, 125, 7],
					['H34', 872, 126, 1],
					['H34', 873, 126, 2],
					['H34', 874, 126, 3],
					['H34', 875, 126, 4],
					['H34', 876, 126, 5],
					['H34', 877, 126, 6],
					['H34', 878, 126, 7],
					['H34', 879, 126, 8],
					['H34', 880, 126, 9],
					['H34', 881, 127, 1],
					['H34', 882, 127, 2],
					['H34', 883, 127, 3],
					['H34', 884, 127, 4],
					['H34', 885, 127, 5],
					['H34', 886, 128, 1],
					['H34', 887, 128, 2],
					['H34', 888, 128, 3],
					['H34', 889, 128, 4],
					['H34', 890, 128, 5],
					['H34', 891, 128, 6],
					['H34', 892, 128, 7],
					['H34', 893, 128, 8],
					['H34', 894, 128, 9],
					['H34', 895, 128, 10],
					['H34', 896, 128, 11],
					['H34', 897, 128, 12],
					['H34', 898, 128, 13],
					['H34', 899, 128, 14],
					['H34', 900, 128, 15],
					['H34', 901, 128, 16],
					['H34', 902, 129, 1],
					['H34', 903, 129, 2],
					['H34', 904, 129, 3],
					['H34', 905, 129, 4],
					['H34', 906, 129, 5],
					['H34', 907, 129, 6],
					['H34', 908, 130, 1],
					['H34', 909, 130, 2],
					['H6', 1, 1, 0],
					['H6', 2, 1, 1],
					['H6', 3, 1, 2],
					['H6', 4, 1, 3],
					['H6', 5, 1, 4],
					['H6', 6, 1, 5],
					['H6', 7, 1, 6],
					['H6', 8, 1, 7],
					['H6', 9, 1, 8],
					['H6', 10, 1, 9],
					['H6', 11, 1, 10],
					['H6', 12, 2, 0],
					['H6', 13, 2, 1],
					['H6', 14, 3, 0],
					['H6', 15, 3, 1],
					['H6', 16, 3, 2],
					['H6', 17, 3, 3],
					['H6', 18, 3, 4],
					['H6', 19, 3, 5],
					['H6', 20, 4, 0],
					['H6', 21, 4, 1],
					['H6', 22, 4, 2],
					['H6', 23, 4, 3],
					['H6', 24, 4, 4],
					['H6', 25, 4, 5],
					['H6', 26, 4, 6],
					['H6', 27, 4, 7],
					['H6', 28, 4, 8],
					['H6', 29, 4, 9],
					['H6', 30, 4, 10],
					['H6', 31, 4, 11],
					['H6', 32, 4, 12],
					['H6', 39, 6, 1],
					['H6', 40, 6, 2],
					['H6', 41, 7, 0],
					['H6', 42, 7, 1],
					['H6', 43, 7, 2],
					['H6', 44, 7, 3],
					['H6', 45, 7, 4],
					['H6', 46, 8, 0],
					['H6', 47, 9, 0],
					['H6', 48, 9, 1],
					['H6', 49, 9, 2],
					['H6', 50, 9, 3],
					['H6', 51, 10, 0],
					['H6', 52, 10, 1],
					['H6', 53, 10, 2],
					['H6', 54, 10, 3],
					['H6', 55, 10, 4],
					['H6', 56, 10, 5],
					['H6', 57, 10, 6],
					['H6', 58, 10, 7],
					['H6', 59, 10, 8],
					['H6', 60, 11, 0],
					['H6', 61, 11, 1],
					['H6', 98, 13, 0],
					['H6', 99, 13, 1],
					['H6', 101, 13, 3],
					['H6', 106, 13, 8],
					['H6', 107, 13, 9],
					['H6', 108, 13, 10],
					['H6', 109, 13, 11],
					['H6', 110, 13, 12],
					['H6', 111, 14, 0],
					['H6', 123, 15, 0],
					['H6', 124, 15, 1],
					['H6', 125, 15, 2],
					['H6', 126, 15, 3],
					['H6', 160, 22, 0],
					['H6', 161, 22, 1],
					['H6', 162, 22, 2],
					['H6', 163, 22, 3],
					['H6', 164, 22, 4],
					['H6', 165, 22, 5],
					['H6', 170, 24, 0],
					['H6', 171, 24, 1],
					['H6', 172, 24, 2],
					['H6', 173, 25, 0],
					['H6', 174, 25, 1],
					['H6', 175, 25, 2],
					['H6', 176, 25, 3],
					['H6', 177, 25, 4],
					['H6', 178, 25, 5],
					['H6', 182, 27, 0],
					['H6', 183, 27, 1],
					['H6', 184, 27, 2],
					['H6', 185, 27, 3],
					['H6', 186, 28, 0],
					['H6', 187, 28, 1],
					['H6', 188, 28, 2],
					['H6', 189, 28, 3],
					['H6', 190, 29, 0],
					['H6', 191, 29, 1],
					['H6', 192, 29, 2],
					['H6', 193, 29, 3],
					['H6', 194, 30, 0],
					['H6', 195, 30, 1],
					['H6', 196, 30, 2],
					['H6', 197, 30, 3],
					['H6', 198, 30, 4],
					['H6', 210, 31, 1],
					['H6', 211, 31, 2],
					['H6', 212, 31, 3],
					['H6', 213, 31, 4],
					['H6', 214, 32, 1],
					['H6', 215, 32, 2],
					['H6', 216, 32, 3],
					['H6', 217, 32, 4],
					['H6', 218, 32, 5],
					['H6', 219, 32, 6],
					['H6', 220, 33, 1],
					['H6', 221, 33, 2],
					['H6', 222, 33, 3],
					['H6', 223, 34, 1],
					['H6', 224, 34, 2],
					['H6', 225, 34, 3],
					['H6', 226, 35, 1],
					['H6', 227, 35, 2],
					['H6', 228, 35, 3],
					['H6', 237, 38, 1],
					['H6', 238, 38, 2],
					['H6', 239, 38, 3],
					['H6', 240, 38, 4],
					['H6', 241, 39, 1],
					['H6', 242, 39, 2],
					['H6', 243, 39, 3],
					['H6', 244, 39, 4],
					['H6', 245, 40, 1],
					['H6', 246, 40, 2],
					['H6', 247, 40, 3],
					['H6', 248, 40, 4],
					['H6', 249, 41, 1],
					['H6', 250, 41, 2],
					['H6', 251, 41, 3],
					['H6', 252, 41, 4],
					['H6', 253, 41, 5],
					['H6', 254, 41, 6],
					['H6', 255, 41, 7],
					['H6', 256, 41, 8],
					['H6', 257, 41, 9],
					['H6', 258, 41, 10],
					['H6', 259, 41, 11],
					['H6', 260, 42, 1],
					['H6', 261, 42, 2],
					['H6', 262, 42, 3],
					['H6', 263, 42, 4],
					['H6', 264, 42, 5],
					['H6', 265, 42, 6],
					['H6', 266, 42, 7],
					['H6', 267, 42, 8],
					['H6', 268, 42, 9],
					['H6', 269, 43, 1],
					['H6', 270, 43, 2],
					['H6', 271, 43, 3],
					['H6', 272, 43, 4],
					['H6', 273, 44, 1],
					['H6', 274, 44, 2],
					['H6', 275, 44, 3],
					['H6', 276, 44, 4],
					['H6', 277, 45, 1],
					['H6', 278, 45, 2],
					['H6', 279, 45, 3],
					['H6', 280, 45, 4],
					['H6', 281, 45, 5],
					['H6', 282, 45, 6],
					['H6', 283, 45, 7],
					['H6', 284, 45, 8],
					['H6', 285, 45, 9],
					['H6', 286, 45, 10],
					['H6', 287, 45, 11],
					['H6', 289, 46, 1],
					['H6', 290, 46, 2],
					['H6', 291, 46, 3],
					['H6', 292, 46, 4],
					['H6', 293, 46, 5],
					['H6', 294, 46, 6],
					['H6', 295, 47, 1],
					['H6', 296, 47, 2],
					['H6', 297, 48, 1],
					['H6', 298, 48, 2],
					['H6', 299, 48, 3],
					['H6', 300, 48, 4],
					['H6', 301, 48, 5],
					['H6', 302, 48, 6],
					['H6', 303, 48, 7],
					['H6', 304, 48, 8],
					['H6', 305, 48, 9],
					['H6', 306, 48, 10],
					['H6', 307, 48, 11],
					['H6', 308, 48, 12],
					['H6', 309, 48, 13],
					['H6', 310, 48, 14],
					['H6', 311, 48, 15],
					['H6', 312, 48, 16],
					['H6', 313, 48, 17],
					['H6', 314, 48, 18],
					['H6', 315, 48, 19],
					['H6', 316, 48, 20],
					['H6', 317, 48, 21],
					['H6', 318, 48, 22],
					['H6', 319, 48, 23],
					['H6', 320, 48, 24],
					['H6', 323, 49, 2],
					['H6', 324, 49, 3],
					['H6', 325, 50, 1],
					['H6', 326, 50, 2],
					['H6', 387, 29, null],
					['H6', 388, 57, 1],
					['H6', 389, 57, 2],
					['H6', 393, 58, 1],
					['H6', 394, 58, 2],
					['H6', 395, 58, 3],
					['H6', 396, 58, 4],
					['H6', 397, 58, 5],
					['H6', 398, 58, 6],
					['H6', 399, 58, 7],
					['H6', 400, 58, 8],
					['H6', 401, 59, 1],
					['H6', 402, 59, 2],
					['H6', 403, 60, 1],
					['H6', 404, 60, 2],
					['H6', 416, 63, 1],
					['H6', 417, 63, 2],
					['H6', 418, 63, 3],
					['H6', 419, 64, 1],
					['H6', 420, 64, 2],
					['H6', 422, 65, 1],
					['H6', 423, 65, 2],
					['H6', 424, 65, 3],
					['H6', 425, 66, 1],
					['H6', 426, 66, 2],
					['H6', 427, 66, 3],
					['H6', 440, 68, 1],
					['H6', 444, 68, 2],
					['H6', 445, 68, 3],
					['H6', 446, 14, 12],
					['H6', 451, 14, 17],
					['H6', 452, 14, 18],
					['H6', 453, 14, 19],
					['H6', 455, 11, 2],
					['H6', 456, 11, 3],
					['H6', 457, 11, 4],
					['H6', 458, 11, 5],
					['H6', 459, 11, 6],
					['H6', 460, 11, 7],
					['H6', 461, 11, 8],
					['H6', 462, 11, 9],
					['H6', 463, 11, 10],
					['H6', 464, 11, 11],
					['H6', 465, 11, 12],
					['H6', 508, 37, 3],
					['H6', 509, 37, 4],
					['H6', 510, 37, 5],
					['H6', 511, 37, 6],
					['H6', 557, 29, 4],
					['H6', 559, 70, 1],
					['H6', 560, 70, 2],
					['H6', 564, 71, 1],
					['H6', 565, 71, 2],
					['H6', 576, 74, 1],
					['H6', 577, 74, 2],
					['H6', 578, 74, 3],
					['H6', 579, 74, 4],
					['H6', 580, 74, 5],
					['H6', 581, 74, 6],
					['H6', 582, 74, 7],
					['H6', 583, 74, 8],
					['H6', 584, 74, 9],
					['H6', 585, 74, 10],
					['H6', 586, 74, 11],
					['H6', 587, 75, 1],
					['H6', 588, 75, 2],
					['H6', 589, 75, 3],
					['H6', 590, 76, 1],
					['H6', 591, 76, 2],
					['H6', 592, 76, 3],
					['H6', 593, 77, 1],
					['H6', 594, 77, 2],
					['H6', 595, 77, 3],
					['H6', 596, 77, 4],
					['H6', 597, 77, 5],
					['H6', 598, 77, 6],
					['H6', 599, 77, 7],
					['H6', 600, 77, 8],
					['H6', 601, 77, 9],
					['H6', 602, 77, 10],
					['H6', 603, 77, 11],
					['H6', 604, 77, 12],
					['H6', 605, 77, 13],
					['H6', 606, 77, 14],
					['H6', 607, 77, 15],
					['H6', 608, 78, 1],
					['H6', 609, 78, 2],
					['H6', 610, 78, 3],
					['H6', 611, 78, 4],
					['H6', 612, 78, 5],
					['H6', 613, 78, 6],
					['H6', 614, 78, 7],
					['H6', 615, 78, 8],
					['H6', 616, 78, 9],
					['H6', 617, 78, 10],
					['H6', 618, 78, 11],
					['H6', 619, 78, 12],
					['H6', 620, 78, 13],
					['H6', 621, 78, 14],
					['H6', 622, 78, 15],
					['H6', 659, 85, 1],
					['H6', 660, 85, 2],
					['H6', 661, 85, 3],
					['H6', 666, 86, 1],
					['H6', 667, 86, 2],
					['H6', 668, 87, 1],
					['H6', 669, 88, 1],
					['H6', 670, 89, 1],
					['H6', 671, 88, 2],
					['H6', 672, 37, 7],
					['H6', 675, 90, 1],
					['H6', 676, 90, 2],
					['H6', 677, 90, 3],
					['H6', 678, 90, 4],
					['H6', 679, 90, 5],
					['H6', 680, 90, 6],
					['H6', 681, 90, 7],
					['H6', 682, 91, 1],
					['H6', 683, 91, 2],
					['H6', 684, 91, 3],
					['H6', 685, 91, 4],
					['H6', 686, 91, 5],
					['H6', 687, 91, 6],
					['H6', 688, 91, 7],
					['H6', 689, 63, 4],
					['H6', 690, 63, 5],
					['H6', 691, 92, 1],
					['H6', 692, 93, 1],
					['H6', 693, 92, 2],
					['H6', 694, 29, 5],
					['H6', 695, 29, 6],
					['H6', 696, 29, 7],
					['H6', 697, 94, 1],
					['H6', 698, 94, 2],
					['H6', 699, 94, 3],
					['H6', 700, 57, 3],
					['H6', 701, 90, 8],
					['H6', 702, 91, 8],
					['H6', 703, 95, 1],
					['H6', 704, 95, 2],
					['H6', 705, 96, 1],
					['H6', 706, 96, 2],
					['H6', 707, 96, 3],
					['H6', 708, 96, 4],
					['H6', 709, 96, 5],
					['H6', 710, 96, 6],
					['H6', 711, 96, 7],
					['H6', 712, 96, 8],
					['H6', 713, 86, 3],
					['H6', 714, 97, 1],
					['H6', 715, 97, 2],
					['H6', 716, 97, 3],
					['H6', 717, 98, 1],
					['H6', 718, 98, 2],
					['H6', 719, 98, 3],
					['H6', 720, 14, 20],
					['H6', 721, 14, 21],
					['H6', 722, 14, 22],
					['H6', 723, 99, 1],
					['H6', 724, 99, 2],
					['H6', 725, 99, 3],
					['H6', 726, 99, 4],
					['H6', 727, 99, 5],
					['H6', 728, 100, 1],
					['H6', 729, 100, 2],
					['H6', 730, 100, 3],
					['H6', 731, 100, 4],
					['H6', 732, 100, 5],
					['H6', 733, 100, 6],
					['H6', 734, 100, 7],
					['H6', 735, 100, 8],
					['H6', 736, 100, 9],
					['H6', 738, 102, 1],
					['H6', 739, 102, 2],
					['H6', 740, 102, 3],
					['H6', 741, 102, 4],
					['H6', 742, 102, 5],
					['H6', 743, 102, 6],
					['H6', 744, 102, 7],
					['H6', 745, 102, 8],
					['H6', 746, 103, 1],
					['H6', 747, 103, 2],
					['H6', 748, 103, 3],
					['H6', 749, 104, 1],
					['H6', 750, 104, 2],
					['H6', 751, 104, 3],
					['H6', 752, 104, 4],
					['H6', 753, 104, 5],
					['H6', 754, 104, 6],
					['H6', 755, 104, 7],
					['H6', 756, 104, 8],
					['H6', 757, 105, 1],
					['H6', 758, 105, 2],
					['H6', 759, 105, 3],
					['H6', 760, 106, 1],
					['H6', 761, 106, 2],
					['H6', 762, 106, 3],
					['H6', 763, 106, 4],
					['H6', 766, 87, 2],
					['H6', 767, 87, 3],
					['H6', 768, 87, 4],
					['H6', 769, 87, 5],
					['H6', 770, 87, 6],
					['H6', 771, 87, 7],
					['H6', 772, 87, 8],
					['H6', 773, 107, 1],
					['H6', 774, 107, 2],
					['H6', 775, 107, 3],
					['H6', 776, 107, 4],
					['H6', 777, 107, 5],
					['H6', 778, 108, 1],
					['H6', 779, 108, 2],
					['H6', 780, 108, 3],
					['H6', 781, 108, 4],
					['H6', 782, 108, 5],
					['H6', 783, 108, 6],
					['H6', 784, 109, 1],
					['H6', 785, 109, 2],
					['H6', 786, 109, 3],
					['H6', 787, 109, 4],
					['H6', 788, 109, 5],
					['H6', 789, 109, 6],
					['H6', 790, 110, 1],
					['H6', 791, 110, 2],
					['H6', 792, 110, 3],
					['H6', 793, 110, 4],
					['H6', 794, 110, 5],
					['H6', 795, 110, 6],
					['H6', 796, 111, 1],
					['H6', 797, 111, 2],
					['H6', 798, 111, 3],
					['H6', 799, 111, 4],
					['H6', 800, 111, 5],
					['H6', 801, 111, 6],
					['H6', 802, 112, 1],
					['H6', 803, 112, 2],
					['H6', 804, 112, 3],
					['H6', 805, 112, 4],
					['H6', 806, 112, 5],
					['H6', 807, 112, 6],
					['H6', 808, 113, 1],
					['H6', 809, 113, 2],
					['H6', 810, 113, 3],
					['H6', 811, 113, 4],
					['H6', 812, 113, 5],
					['H6', 813, 113, 6],
					['H6', 814, 114, 1],
					['H6', 815, 114, 2],
					['H6', 816, 114, 3],
					['H6', 817, 114, 4],
					['H6', 818, 114, 5],
					['H6', 819, 114, 6],
					['H6', 820, 115, 1],
					['H6', 821, 115, 2],
					['H6', 822, 115, 3],
					['H6', 823, 115, 4],
					['H6', 824, 115, 5],
					['H6', 825, 115, 6],
					['H6', 826, 116, 1],
					['H6', 827, 116, 2],
					['H6', 828, 116, 3],
					['H6', 829, 116, 4],
					['H6', 830, 116, 5],
					['H6', 831, 116, 6],
					['H6', 832, 117, 1],
					['H6', 833, 117, 2],
					['H6', 834, 117, 3],
					['H6', 835, 117, 4],
					['H6', 836, 117, 5],
					['H6', 837, 117, 6],
					['H6', 838, 118, 1],
					['H6', 839, 118, 2],
					['H6', 840, 118, 3],
					['H6', 841, 118, 4],
					['H6', 842, 118, 5],
					['H6', 843, 118, 6],
					['H6', 844, 118, 7],
					['H6', 845, 119, 1],
					['H6', 846, 119, 2],
					['H6', 847, 120, 1],
					['H6', 848, 120, 2],
					['H6', 849, 121, 1],
					['H6', 850, 121, 2],
					['H6', 851, 122, 2],
					['H6', 852, 122, 3],
					['H6', 853, 122, 1],
					['H6', 854, 122, 4],
					['H6', 855, 123, 1],
					['H6', 856, 123, 2],
					['H6', 857, 123, 3],
					['H6', 858, 123, 4],
					['H6', 859, 123, 5],
					['H6', 860, 124, 1],
					['H6', 861, 124, 2],
					['H6', 862, 124, 3],
					['H6', 863, 124, 4],
					['H6', 864, 124, 5],
					['H6', 865, 125, 1],
					['H6', 866, 125, 2],
					['H6', 867, 125, 3],
					['H6', 868, 125, 4],
					['H6', 869, 125, 5],
					['H6', 870, 125, 6],
					['H6', 871, 125, 7],
					['H6', 872, 126, 1],
					['H6', 873, 126, 2],
					['H6', 874, 126, 3],
					['H6', 875, 126, 4],
					['H6', 876, 126, 5],
					['H6', 877, 126, 6],
					['H6', 878, 126, 7],
					['H6', 879, 126, 8],
					['H6', 880, 126, 9],
					['H6', 881, 127, 1],
					['H6', 882, 127, 2],
					['H6', 883, 127, 3],
					['H6', 884, 127, 4],
					['H6', 885, 127, 5],
					['H6', 886, 128, 1],
					['H6', 887, 128, 2],
					['H6', 888, 128, 3],
					['H6', 889, 128, 4],
					['H6', 890, 128, 5],
					['H6', 891, 128, 6],
					['H6', 892, 128, 7],
					['H6', 893, 128, 8],
					['H6', 894, 128, 9],
					['H6', 895, 128, 10],
					['H6', 896, 128, 11],
					['H6', 897, 128, 12],
					['H6', 898, 128, 13],
					['H6', 899, 128, 14],
					['H6', 900, 128, 15],
					['H6', 901, 128, 16],
					['H6', 902, 129, 1],
					['H6', 903, 129, 2],
					['H6', 904, 129, 3],
					['H6', 905, 129, 4],
					['H6', 906, 129, 5],
					['H6', 907, 129, 6],
					['H6', 908, 130, 1],
					['H6', 909, 130, 2],
				]
			],
			'vtiger_role2profile' => [
				'columns' => ['roleid', 'profileid'],
				'values' => [
					['H10', 1],
					['H14', 1],
					['H18', 1],
					['H2', 1],
					['H22', 1],
					['H26', 1],
					['H3', 2],
					['H30', 1],
					['H34', 1],
					['H6', 1],
				]
			],
			'vtiger_role_seq' => [
				'columns' => ['id'],
				'values' => [
					[37],
				]
			],
			'vtiger_rowheight' => [
				'columns' => ['rowheightid', 'rowheight', 'sortorderid', 'presence'],
				'values' => [
					[1, 'wide', 1, 1],
					[2, 'medium', 2, 1],
					[3, 'narrow', 3, 1],
				]
			],
			'vtiger_rowheight_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_salutationtype' => [
				'columns' => ['salutationtypeid', 'salutationtype', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Mr.', 1, 161, 1],
					[3, 'Ms.', 1, 162, 2],
					[4, 'Mrs.', 1, 163, 3],
					[5, 'Dr.', 1, 164, 4],
					[6, 'Prof.', 1, 165, 5],
				]
			],
			'vtiger_salutationtype_seq' => [
				'columns' => ['id'],
				'values' => [
					[6],
				]
			],
			'vtiger_scalculations_status' => [
				'columns' => ['scalculations_statusid', 'scalculations_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 855],
					[2, 'PLL_IN_REALIZATION', 2, 1, 856],
					[3, 'PLL_FOR_VERIFICATION', 3, 1, 857],
					[4, 'PLL_CANCELLED', 4, 0, 858],
					[5, 'PLL_COMPLETED', 5, 0, 859],
				]
			],
			'vtiger_selectquery_seq' => [
				'columns' => ['id'],
				'values' => [
					[25],
				]
			],
			'vtiger_service_usageunit' => [
				'columns' => ['service_usageunitid', 'service_usageunit', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Hours', 1, 226, 1],
					[2, 'Days', 1, 227, 2],
					[3, 'Incidents', 1, 228, 3],
				]
			],
			'vtiger_service_usageunit_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_settings_blocks' => [
				'columns' => ['blockid', 'label', 'sequence', 'icon', 'type', 'linkto', 'admin_access'],
				'values' => [
					[1, 'LBL_USER_MANAGEMENT', 2, 'adminIcon-permissions', 0, null, null],
					[2, 'LBL_STUDIO', 3, 'adminIcon-standard-modules', 0, null, null],
					[3, 'LBL_COMPANY', 13, 'adminIcon-company-information', 0, null, null],
					[4, 'LBL_SYSTEM_TOOLS', 12, 'adminIcon-system-tools', 0, null, null],
					[5, 'LBL_INTEGRATION', 9, 'adminIcon-integration', 0, null, null],
					[6, 'LBL_PROCESSES', 14, 'adminIcon-processes', 0, null, null],
					[7, 'LBL_SECURITY_MANAGEMENT', 7, 'adminIcon-security', 0, null, null],
					[8, 'LBL_MAIL_TOOLS', 11, 'adminIcon-mail-tools', 0, null, null],
					[9, 'LBL_About_YetiForce', 15, 'adminIcon-about-yetiforce', 0, null, null],
					[11, 'LBL_ADVANCED_MODULES', 4, 'adminIcon-advenced-modules', 0, null, null],
					[12, 'LBL_CALENDAR_LABELS_COLORS', 5, 'adminIcon-calendar-labels-colors', 0, null, null],
					[13, 'LBL_SEARCH_AND_FILTERS', 6, 'adminIcon-search-and-filtres', 0, null, null],
					[14, 'LBL_LOGS', 8, 'adminIcon-logs', 0, null, null],
					[15, 'LBL_AUTOMATION', 10, 'adminIcon-automation', 0, null, null],
					[16, 'LBL_MENU_SUMMARRY', 0, 'userIcon-Home', 1, 'index.php?module=Vtiger&parent=Settings&view=Index', null],
				]
			],
			'vtiger_settings_blocks_seq' => [
				'columns' => ['id'],
				'values' => [
					[17],
				]
			],
			'vtiger_settings_field' => [
				'columns' => ['fieldid', 'blockid', 'name', 'iconpath', 'description', 'linkto', 'sequence', 'active', 'pinned', 'admin_access'],
				'values' => [
					[1, 1, 'LBL_USERS', 'adminIcon-user', 'LBL_USER_DESCRIPTION', 'index.php?module=Users&parent=Settings&view=List', 1, 0, 1, null],
					[2, 1, 'LBL_ROLES', 'adminIcon-roles', 'LBL_ROLE_DESCRIPTION', 'index.php?module=Roles&parent=Settings&view=Index', 2, 0, 0, null],
					[3, 1, 'LBL_PROFILES', 'adminIcon-profiles', 'LBL_PROFILE_DESCRIPTION', 'index.php?module=Profiles&parent=Settings&view=List', 3, 0, 0, null],
					[4, 1, 'USERGROUPLIST', 'adminIcon-groups', 'LBL_GROUP_DESCRIPTION', 'index.php?module=Groups&parent=Settings&view=List', 4, 0, 0, null],
					[5, 1, 'LBL_SHARING_ACCESS', 'adminIcon-module-access', 'LBL_SHARING_ACCESS_DESCRIPTION', 'index.php?module=SharingAccess&parent=Settings&view=Index', 5, 0, 0, null],
					[6, 1, 'LBL_FIELDS_ACCESS', 'adminIcon-special-access', 'LBL_SHARING_FIELDS_DESCRIPTION', 'index.php?module=FieldAccess&parent=Settings&view=Index', 6, 0, 0, null],
					[7, 14, 'LBL_LOGIN_HISTORY_DETAILS', 'adminIcon-users-login', 'LBL_LOGIN_HISTORY_DESCRIPTION', 'index.php?module=LoginHistory&parent=Settings&view=List', 3, 0, 0, null],
					[8, 2, 'VTLIB_LBL_MODULE_MANAGER', 'adminIcon-modules-installation', 'VTLIB_LBL_MODULE_MANAGER_DESCRIPTION', 'index.php?module=ModuleManager&parent=Settings&view=List', 1, 0, 1, null],
					[9, 2, 'LBL_PICKLIST_EDITOR', 'adminIcon-fields-picklists', 'LBL_PICKLIST_DESCRIPTION', 'index.php?parent=Settings&module=Picklist&view=Index', 9, 0, 1, null],
					[10, 2, 'LBL_PICKLIST_DEPENDENCY_SETUP', 'adminIcon-fields-picklists-relations', 'LBL_PICKLIST_DEPENDENCY_DESCRIPTION', 'index.php?parent=Settings&module=PickListDependency&view=List', 10, 0, 0, null],
					[11, 1, 'LBL_ADVANCED_PERMISSION', 'glyphicon glyphicon-ice-lolly', null, 'index.php?module=AdvancedPermission&parent=Settings&view=List', 10, 0, 0, null],
					[12, 3, 'NOTIFICATIONSCHEDULERS', '', 'LBL_NOTIF_SCHED_DESCRIPTION', 'index.php?module=Settings&view=listnotificationschedulers&parenttab=Settings', 4, 0, 0, null],
					[13, 4, 'INVENTORYNOTIFICATION', '', 'LBL_INV_NOTIF_DESCRIPTION', 'index.php?module=Settings&view=listinventorynotifications&parenttab=Settings', 1, 0, 0, null],
					[14, 3, 'LBL_COMPANY_DETAILS', 'adminIcon-company-detlis', 'LBL_COMPANY_DESCRIPTION', 'index.php?parent=Settings&module=Companies&view=List', 2, 0, 0, null],
					[15, 5, 'LBL_PBX', 'adminIcon-pbx-manager', 'LBL_PBX_DESCRIPTION', 'index.php?module=PBX&parent=Settings&view=List', 3, 0, 0, null],
					[16, 4, 'LBL_CURRENCY_SETTINGS', 'adminIcon-currencies', 'LBL_CURRENCY_DESCRIPTION', 'index.php?parent=Settings&module=Currency&view=List', 4, 0, 0, null],
					[17, 4, 'LBL_SWITCH_USERS', 'adminIcon-users', 'LBL_SWITCH_USERS_DESCRIPTION', 'index.php?module=Users&view=SwitchUsers&parent=Settings', 11, 0, 0, null],
					[18, 4, 'LBL_SYSTEM_INFO', 'adminIcon-server-configuration', 'LBL_SYSTEM_DESCRIPTION', 'index.php?module=Settings&submodule=Server&view=ProxyConfig', 6, 1, 0, null],
					[20, 4, 'LBL_DEFAULT_MODULE_VIEW', 'adminIcon-standard-modules', 'LBL_DEFAULT_MODULE_VIEW_DESC', 'index.php?module=Settings&action=DefModuleView&parenttab=Settings', 2, 0, 0, null],
					[22, 2, 'LBL_CUSTOMIZE_RECORD_NUMBERING', 'adminIcon-recording-control', 'LBL_CUSTOMIZE_MODENT_NUMBER_DESCRIPTION', 'index.php?module=Vtiger&parent=Settings&view=CustomRecordNumbering', 6, 0, 0, null],
					[24, 15, 'LBL_LIST_WORKFLOWS', 'adminIcon-triggers', 'LBL_LIST_WORKFLOWS_DESCRIPTION', 'index.php?module=Workflows&parent=Settings&view=List', 1, 0, 1, null],
					[25, 4, 'LBL_CONFIG_EDITOR', 'adminIcon-system-tools', 'LBL_CONFIG_EDITOR_DESCRIPTION', 'index.php?module=Vtiger&parent=Settings&view=ConfigEditorDetail', 7, 0, 0, null],
					[26, 15, 'Scheduler', 'adminIcon-cron', 'LBL_SCHEDULER_DESCRIPTION', 'index.php?module=CronTasks&parent=Settings&view=List', 3, 0, 0, null],
					[27, 15, 'LBL_WORKFLOW_LIST', 'adminIcon-workflow', 'LBL_AVAILABLE_WORKLIST_LIST', 'index.php?module=com_vtiger_workflow&action=workflowlist', 1, 0, 0, null],
					[28, 4, 'ModTracker', 'adminIcon-modules-track-chanegs', 'LBL_MODTRACKER_DESCRIPTION', 'index.php?module=ModTracker&action=BasicSettings&parenttab=Settings&formodule=ModTracker', 9, 0, 0, null],
					[33, 2, 'LBL_EDIT_FIELDS', 'adminIcon-modules-fields', 'LBL_LAYOUT_EDITOR_DESCRIPTION', 'index.php?module=LayoutEditor&parent=Settings&view=Index', 2, 0, 0, null],
					[35, 4, 'LBL_PDF', 'adminIcon-modules-pdf-templates', 'LBL_PDF_DESCRIPTION', 'index.php?module=PDF&parent=Settings&view=List', 10, 0, 0, null],
					[38, 7, 'LBL_PASSWORD_CONF', 'adminIcon-passwords-configuration', 'LBL_PASSWORD_DESCRIPTION', 'index.php?module=Password&parent=Settings&view=Index', 1, 0, 0, null],
					[40, 2, 'LBL_MENU_BUILDER', 'adminIcon-menu-configuration', 'LBL_MENU_BUILDER_DESCRIPTION', 'index.php?module=Menu&view=Index&parent=Settings', 14, 0, 1, null],
					[41, 2, 'LBL_ARRANGE_RELATED_TABS', 'adminIcon-modules-relations', 'LBL_ARRANGE_RELATED_TABS', 'index.php?module=LayoutEditor&parent=Settings&view=Index&mode=showRelatedListLayout', 4, 0, 1, null],
					[44, 8, 'Mail Scanner', 'adminIcon-mail-scanner', 'LBL_MAIL_SCANNER_DESCRIPTION', 'index.php?module=OSSMailScanner&parent=Settings&view=Index', 3, 0, 0, null],
					[45, 14, 'Mail Logs', 'adminIcon-mail-download-history', 'LBL_MAIL_LOGS_DESCRIPTION', 'index.php?module=OSSMailScanner&parent=Settings&view=Logs', 4, 0, 0, null],
					[46, 8, 'Mail View', 'adminIcon-oss_mailview', 'LBL_MAIL_VIEW_DESCRIPTION', 'index.php?module=OSSMailView&parent=Settings&view=index', 21, 0, 0, null],
					[49, 9, 'License', 'adminIcon-license', 'LBL_LICENSE_DESCRIPTION', 'index.php?module=Vtiger&parent=Settings&view=License', 4, 0, 0, null],
					[51, 7, 'OSSPassword Configuration', 'adminIcon-passwords-encryption', 'LBL_OSSPASSWORD_CONFIGURATION_DESCRIPTION', 'index.php?module=OSSPasswords&view=ConfigurePass&parent=Settings', 3, 0, 0, null],
					[53, 4, 'LangManagement', 'adminIcon-languages-and-translations', 'LBL_LANGMANAGEMENT_DESCRIPTION', 'index.php?module=LangManagement&parent=Settings&view=Index', 1, 0, 0, null],
					[54, 1, 'GlobalPermission', 'adminIcon-special-access', 'LBL_GLOBALPERMISSION_DESCRIPTION', 'index.php?module=GlobalPermission&parent=Settings&view=Index', 7, 0, 0, null],
					[56, 13, 'Search Setup', 'adminIcon-search-configuration', 'LBL_SEARCH_SETUP_DESCRIPTION', 'index.php?module=Search&parent=Settings&view=Index', 1, 0, 0, null],
					[57, 13, 'CustomView', 'adminIcon-filters-configuration', 'LBL_CUSTOMVIEW_DESCRIPTION', 'index.php?module=CustomView&parent=Settings&view=Index', 2, 0, 0, null],
					[58, 2, 'Widgets', 'adminIcon-modules-widgets', 'LBL_WIDGETS_DESCRIPTION', 'index.php?module=Widgets&parent=Settings&view=Index', 3, 0, 1, null],
					[59, 9, 'Credits', 'adminIcon-contributors', 'LBL_CREDITS_DESCRIPTION', 'index.php?module=Vtiger&view=Credits&parent=Settings', 3, 0, 0, null],
					[60, 2, 'LBL_QUICK_CREATE_EDITOR', 'adminIcon-fields-quick-create', 'LBL_QUICK_CREATE_EDITOR_DESCRIPTION', 'index.php?module=QuickCreateEditor&parent=Settings&view=Index', 8, 0, 0, null],
					[61, 5, 'LBL_API_ADDRESS', 'adminIcon-address', 'LBL_API_ADDRESS_DESCRIPTION', 'index.php?module=ApiAddress&parent=Settings&view=Configuration', 5, 0, 0, null],
					[62, 7, 'LBL_BRUTEFORCE', 'adminIcon-brute-force', 'LBL_BRUTEFORCE_DESCRIPTION', 'index.php?module=BruteForce&parent=Settings&view=Index', 2, 0, 0, null],
					[63, 14, 'LBL_UPDATES_HISTORY', 'adminIcon-server-updates', 'LBL_UPDATES_HISTORY_DESCRIPTION', 'index.php?parent=Settings&module=Updates&view=Index', 2, 0, 0, null],
					[65, 14, 'LBL_CONFREPORT', 'adminIcon-server-configuration', 'LBL_CONFREPORT_DESCRIPTION', 'index.php?parent=Settings&module=ConfReport&view=Index', 1, 0, 0, null],
					[67, 2, 'LBL_WIDGETS_MANAGEMENT', 'adminIcon-widgets-configuration', 'LBL_WIDGETS_MANAGEMENT_DESCRIPTION', 'index.php?module=WidgetsManagement&parent=Settings&view=Configuration', 15, 0, 0, null],
					[70, 2, 'LBL_TREES_MANAGER', 'adminIcon-field-folders', 'LBL_TREES_MANAGER_DESCRIPTION', 'index.php?module=TreesManager&parent=Settings&view=List', 11, 0, 0, null],
					[71, 2, 'LBL_MODTRACKER_SETTINGS', 'adminIcon-modules-track-chanegs', 'LBL_MODTRACKER_SETTINGS_DESCRIPTION', 'index.php?module=ModTracker&parent=Settings&view=List', 5, 0, 0, null],
					[72, 2, 'LBL_HIDEBLOCKS', 'adminIcon-filed-hide-bloks', 'LBL_HIDEBLOCKS_DESCRIPTION', 'index.php?module=HideBlocks&parent=Settings&view=List', 12, 0, 0, null],
					[73, 12, 'LBL_PUBLIC_HOLIDAY', 'adminIcon-calendar-holidys', 'LBL_PUBLIC_HOLIDAY_DESCRIPTION', 'index.php?module=PublicHoliday&view=Configuration&parent=Settings', 3, 0, 0, null],
					[74, 12, 'LBL_CALENDAR_CONFIG', 'adminIcon-calendar-configuration', 'LBL_CALENDAR_CONFIG_DESCRIPTION', 'index.php?parent=Settings&module=Calendar&view=UserColors', 2, 0, 0, null],
					[75, 6, 'LBL_SALES_PROCESSES', 'adminIcon-sales', 'LBL_SALES_PROCESSES_DESCRIPTION', 'index.php?module=SalesProcesses&view=Index&parent=Settings', 2, 0, 0, null],
					[77, 5, 'LBL_DAV_KEYS', 'adminIcon-dav-applications', 'LBL_DAV_KEYS_DESCRIPTION', 'index.php?parent=Settings&module=Dav&view=Keys', 7, 0, 0, null],
					[78, 8, 'LBL_AUTOLOGIN', 'adminIcon-mail-auto-login', 'LBL_AUTOLOGIN_DESCRIPTION', 'index.php?parent=Settings&module=Mail&view=Autologin', 4, 0, 0, null],
					[79, 8, 'LBL_MAIL_GENERAL_CONFIGURATION', 'adminIcon-mail-smtp-server', 'LBL_MAIL_GENERAL_CONFIGURATION_DESCRIPTION', 'index.php?parent=Settings&module=Mail&view=Config', 1, 0, 0, null],
					[80, 6, 'LBL_SUPPORT_PROCESSES', 'adminIcon-support ', 'LBL_SUPPORT_PROCESSES_DESCRIPTION', 'index.php?module=SupportProcesses&view=Index&parent=Settings', 6, 0, 0, null],
					[81, 12, 'LBL_COLORS', 'adminIcon-colors', 'LBL_COLORS_DESCRIPTION', 'index.php?module=Users&parent=Settings&view=Colors', 4, 0, 0, null],
					[82, 6, 'LBL_REALIZATION_PROCESSES', 'adminIcon-realization', 'LBL_REALIZATION_PROCESSES_DESCRIPTION', 'index.php?module=RealizationProcesses&view=Index&parent=Settings', 3, 0, 0, null],
					[83, 6, 'LBL_MARKETING_PROCESSES', 'adminIcon-marketing', 'LBL_MARKETING_PROCESSES_DESCRIPTION', 'index.php?module=MarketingProcesses&view=Index&parent=Settings', 1, 0, 0, null],
					[84, 6, 'LBL_FINANCIAL_PROCESSES', 'adminIcon-finances', 'LBL_FINANCIAL_PROCESSES_DESCRIPTION', 'index.php?module=FinancialProcesses&view=Index&parent=Settings', 5, 0, 0, null],
					[85, 5, 'LBL_AUTHORIZATION', 'adminIcon-automation', 'LBL_AUTHORIZATION_DESCRIPTION', 'index.php?module=Users&view=Auth&parent=Settings', 1, 0, 0, null],
					[86, 6, 'LBL_TIMECONTROL_PROCESSES', 'adminIcon-logistics', 'LBL_TIMECONTROL_PROCESSES_DESCRIPTION', 'index.php?module=TimeControlProcesses&parent=Settings&view=Index', 7, 0, 0, null],
					[87, 2, 'LBL_CUSTOM_FIELD_MAPPING', 'adminIcon-filed-mapping', 'LBL_CUSTOM_FIELD_MAPPING_DESCRIPTION', 'index.php?parent=Settings&module=Leads&view=MappingDetail', 13, 0, 0, null],
					[88, 5, 'LBL_CURRENCY_UPDATE', 'adminIcon-currencies', 'LBL_CURRENCY_UPDATE_DESCRIPTION', 'index.php?module=CurrencyUpdate&view=Index&parent=Settings', 2, 0, 0, null],
					[89, 11, 'LBL_CREDITLIMITS', 'adminIcon-credit-limit-base_2', 'LBL_CREDITLIMITS_DESCRIPTION', 'index.php?module=Inventory&parent=Settings&view=CreditLimits', 5, 0, 0, null],
					[90, 11, 'LBL_TAXES', 'adminIcon-taxes-rates', 'LBL_TAXES_DESCRIPTION', 'index.php?module=Inventory&parent=Settings&view=Taxes', 1, 0, 0, null],
					[91, 11, 'LBL_DISCOUNTS', 'adminIcon-discount-base', 'LBL_DISCOUNTS_DESCRIPTION', 'index.php?module=Inventory&parent=Settings&view=Discounts', 3, 0, 0, null],
					[92, 11, 'LBL_TAXCONFIGURATION', 'adminIcon-taxes-caonfiguration', 'LBL_TAXCONFIGURATION_DESCRIPTION', 'index.php?module=Inventory&parent=Settings&view=TaxConfiguration', 4, 0, 0, null],
					[93, 11, 'LBL_DISCOUNTCONFIGURATION', 'adminIcon-discount-configuration', 'LBL_DISCOUNTCONFIGURATION_DESCRIPTION', 'index.php?module=Inventory&parent=Settings&view=DiscountConfiguration', 2, 0, 0, null],
					[94, 8, 'Mail', 'adminIcon-mail-download-history', 'LBL_OSSMAIL_DESCRIPTION', 'index.php?module=OSSMail&parent=Settings&view=Index', 2, 0, 0, null],
					[95, 2, 'LBL_MAPPEDFIELDS', 'adminIcon-mapped-fields', 'LBL_MAPPEDFIELDS_DESCRIPTION', 'index.php?module=MappedFields&parent=Settings&view=List', 16, 0, 0, null],
					[96, 1, 'LBL_LOCKS', 'adminIcon-locks', 'LBL_LOCKS_DESCRIPTION', 'index.php?module=Users&view=Locks&parent=Settings', 8, 0, 0, null],
					[98, 4, 'LBL_NOTIFICATIONS_CONFIGURATION', 'adminIcon-NotificationConfiguration', 'LBL_TYPE_NOTIFICATIONS_DESCRIPTION', 'index.php?module=Notifications&view=Configuration&parent=Settings', 13, 0, 0, null],
					[100, 5, 'LBL_WEBSERVICE_APPS', 'adminIcon-webservice-apps', null, 'index.php?module=WebserviceApps&view=Index&parent=Settings', 11, 0, 0, null],
					[101, 1, 'LBL_OWNER_ALLOCATION', 'adminIcon-owner', 'LBL_OWNER_ALLOCATION_DESCRIPTION', 'index.php?module=RecordAllocation&view=Index&parent=Settings&type=owner', 9, 0, 0, null],
					[102, 1, 'LBL_MULTIOWNER_ALLOCATION', 'adminIcon-shared-owner', 'LBL_MULTIOWNER_ALLOCATION_DESCRIPTION', 'index.php?module=RecordAllocation&view=Index&parent=Settings&type=sharedOwner', 10, 0, 0, null],
					[103, 1, 'LBL_AUTOMATIC_ASSIGNMENT', 'adminIcon-automatic-assignment', 'LBL_AUTOMATICASSIGNMENT_DESCRIPTION', 'index.php?module=AutomaticAssignment&view=List&parent=Settings', 11, 0, 0, null],
					[104, 8, 'LBL_EMAILS_TO_SEND', 'adminIcon-mail-queue', 'LBL_EMAILS_TO_SEND_DESCRIPTION', 'index.php?module=Mail&parent=Settings&view=List', 22, 0, 0, null],
					[105, 8, 'LBL_MAIL_SMTP', 'adminIcon-mail-configuration', 'LBL_MAILSMTP_TO_SEND_DESCRIPTION', 'index.php?module=MailSmtp&parent=Settings&view=List', 23, 0, 0, null],
					[106, 5, 'LBL_WEBSERVICE_USERS', 'adminIcon-webservice-users', 'LBL_WEBSERVICE_USERS_DESCRIPTION', 'index.php?module=WebserviceUsers&view=List&parent=Settings', 11, 0, 0, null],
					[107, 5, 'LBL_SMSNOTIFIER', 'userIcon-SMSNotifier', 'LBL_SMSNOTIFIER_DESCRIPTION', 'index.php?module=SMSNotifier&parent=Settings&view=List', 12, 0, 0, null],
				]
			],
			'vtiger_smsnotifier_status' => [
				'columns' => ['smsnotifier_statusid', 'smsnotifier_status', 'presence', 'sortorderid'],
				'values' => [
					[1, 'PLL_UNDEFINED', 0, 1],
					[2, 'PLL_DELIVERED', 0, 2],
					[3, 'PLL_FAILED', 0, 3],
				]
			],
			'vtiger_squoteenquiries_status' => [
				'columns' => ['squoteenquiries_statusid', 'squoteenquiries_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 860],
					[2, 'PLL_IN_REALIZATION', 2, 1, 861],
					[3, 'PLL_FOR_VERIFICATION', 3, 1, 862],
					[4, 'PLL_CANCELLED', 4, 0, 863],
					[5, 'PLL_COMPLETED', 5, 0, 864],
				]
			],
			'vtiger_squotes_status' => [
				'columns' => ['squotes_statusid', 'squotes_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 865],
					[2, 'PLL_IN_REALIZATION', 2, 1, 866],
					[3, 'PLL_FOR_VERIFICATION', 3, 1, 867],
					[4, 'PLL_AWAITING_DECISION', 4, 1, 868],
					[5, 'PLL_NEGOTIATIONS', 5, 1, 869],
					[6, 'PLL_CANCELLED', 6, 0, 870],
					[7, 'PLL_ACCEPTED', 7, 0, 871],
				]
			],
			'vtiger_srecurringorders_status' => [
				'columns' => ['srecurringorders_statusid', 'srecurringorders_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 872],
					[2, 'PLL_REQUIRES_TO_BE_COMPLEMENTED', 2, 1, 873],
					[3, 'PLL_REQUIRES_CONSULTATION', 3, 1, 874],
					[4, 'PLL_WAITING_FOR_SHIPPING', 4, 1, 875],
					[5, 'PLL_WAITING_FOR_SIGNATURE', 5, 1, 876],
					[6, 'PLL_WAITING_FOR_REALIZATION', 6, 1, 877],
					[7, 'PLL_AUTOMATIC_GENERATION', 7, 0, 878],
					[8, 'PLL_UNREALIZED', 8, 0, 879],
					[9, 'PLL_REALIZED', 9, 0, 880],
				]
			],
			'vtiger_srequirementscards_status' => [
				'columns' => ['srequirementscards_statusid', 'srequirementscards_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 881],
					[2, 'PLL_IN_REALIZATION', 2, 1, 882],
					[3, 'PLL_FOR_VERIFICATION', 3, 1, 883],
					[4, 'PLL_CANCELLED', 4, 0, 884],
					[5, 'PLL_COMPLETED', 5, 0, 885],
				]
			],
			'vtiger_ssalesprocesses_source' => [
				'columns' => ['ssalesprocesses_sourceid', 'ssalesprocesses_source', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_WEBSITE', 1, 1],
					[2, 'PLL_PHONE', 2, 1],
					[3, 'PLL_MAIL_CAMPAIGN', 3, 1],
					[4, 'PLL_CONFERENCE', 4, 1],
					[5, 'PLL_RECOMMENDATION', 5, 1],
					[6, 'PLL_MEETING', 6, 1],
					[7, 'PLL_ADVERTISING', 7, 1],
					[8, 'PLL_COMMUNITY', 8, 1],
					[9, 'PLL_OTHER', 9, 1],
				]
			],
			'vtiger_ssalesprocesses_status' => [
				'columns' => ['ssalesprocesses_statusid', 'ssalesprocesses_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 886],
					[2, 'PLL_FOR_REALIZATION', 2, 1, 887],
					[3, 'PLL_QUOTE_ENQUIRY', 3, 1, 888],
					[4, 'PLL_REQUIREMENTS_CARD', 4, 1, 889],
					[5, 'PLL_CALCULATION', 5, 1, 890],
					[6, 'PLL_QUOTE', 6, 1, 891],
					[7, 'PLL_NEGOTIATIONS', 7, 1, 892],
					[8, 'PLL_AWAITING_DECISION', 8, 1, 893],
					[9, 'PLL_AWAITING_SIGNATURES', 9, 1, 894],
					[10, 'PLL_ORDER', 10, 1, 895],
					[11, 'PLL_REALIZATION', 11, 1, 896],
					[12, 'PLL_AWAITING_PAYMENT', 12, 1, 897],
					[13, 'PLL_INVOICING', 13, 1, 898],
					[14, 'PLL_SALE_COMPLETED', 14, 0, 899],
					[15, 'PLL_SALE_FAILED', 15, 0, 900],
					[16, 'PLL_SALE_CANCELLED', 16, 0, 901],
				]
			],
			'vtiger_ssalesprocesses_type' => [
				'columns' => ['ssalesprocesses_typeid', 'ssalesprocesses_type', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_NEW_SALES', 1, 1],
					[2, 'PLL_RENEWAL', 2, 1],
					[3, 'PLL_AFTERSALES', 3, 1],
					[4, 'PLL_TENDER', 4, 1],
				]
			],
			'vtiger_ssingleorders_source' => [
				'columns' => ['ssingleorders_sourceid', 'ssingleorders_source', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_MANUAL', 1, 0],
					[2, 'PLL_POS', 2, 0],
					[3, 'PLL_SHOP', 3, 0],
				]
			],
			'vtiger_ssingleorders_status' => [
				'columns' => ['ssingleorders_statusid', 'ssingleorders_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 1, 902],
					[2, 'PLL_IN_REALIZATION', 2, 1, 903],
					[3, 'PLL_FOR_VERIFICATION', 3, 1, 904],
					[4, 'PLL_AWAITING_SIGNATURES', 4, 1, 905],
					[5, 'PLL_CANCELLED', 5, 0, 906],
					[6, 'PLL_ACCEPTED', 6, 0, 907],
				]
			],
			'vtiger_ssservicesstatus' => [
				'columns' => ['ssservicesstatusid', 'ssservicesstatus', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 416, 1],
					[2, 'PLL_WAITING_FOR_VERIFICATION', 1, 417, 2],
					[3, 'PLL_WAITING_FOR_ACCEPTANCE', 1, 418, 3],
					[4, 'PLL_ACCEPTED', 0, 689, 4],
					[5, 'PLL_CANCELLED', 0, 690, 5],
				]
			],
			'vtiger_ssservicesstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_start_hour' => [
				'columns' => ['start_hourid', 'start_hour', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, '00:00', 1, 297, 1],
					[2, '01:00', 1, 298, 2],
					[3, '02:00', 1, 299, 3],
					[4, '03:00', 1, 300, 4],
					[5, '04:00', 1, 301, 5],
					[6, '05:00', 1, 302, 6],
					[7, '06:00', 1, 303, 7],
					[8, '07:00', 1, 304, 8],
					[9, '08:00', 1, 305, 9],
					[10, '09:00', 1, 306, 10],
					[11, '10:00', 1, 307, 11],
					[12, '11:00', 1, 308, 12],
					[13, '12:00', 1, 309, 13],
					[14, '13:00', 1, 310, 14],
					[15, '14:00', 1, 311, 15],
					[16, '15:00', 1, 312, 16],
					[17, '16:00', 1, 313, 17],
					[18, '17:00', 1, 314, 18],
					[19, '18:00', 1, 315, 19],
					[20, '19:00', 1, 316, 20],
					[21, '20:00', 1, 317, 21],
					[22, '21:00', 1, 318, 22],
					[23, '22:00', 1, 319, 23],
					[24, '23:00', 1, 320, 24],
				]
			],
			'vtiger_start_hour_seq' => [
				'columns' => ['id'],
				'values' => [
					[24],
				]
			],
			'vtiger_state' => [
				'columns' => ['stateid', 'state', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_OPAQUE', 1, 1],
					[2, 'PLL_TRANSPARENT', 2, 1],
				]
			],
			'vtiger_state_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_status' => [
				'columns' => ['statusid', 'status', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'Active', 0, 1],
					[2, 'Inactive', 1, 1],
				]
			],
			'vtiger_status_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_storage_status' => [
				'columns' => ['storage_statusid', 'storage_status', 'sortorderid', 'presence', 'picklist_valueid'],
				'values' => [
					[1, 'PLL_ACTIVE', 1, 1, 908],
					[2, 'PLL_INACTIVE', 2, 1, 909],
				]
			],
			'vtiger_storage_type' => [
				'columns' => ['storage_typeid', 'storage_type', 'sortorderid', 'presence'],
				'values' => [
					[1, 'PLL_INTERNAL', 1, 1],
					[2, 'PLL_EXTERNAL', 2, 1],
				]
			],
			'vtiger_subindustry' => [
				'columns' => ['subindustryid', 'subindustry', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Ministry', 1, 1],
					[2, 'Chancellery', 2, 1],
					[3, 'Voivodeship Office', 3, 1],
					[4, 'Marshal Office', 4, 1],
					[5, 'District', 5, 1],
					[6, 'City/Township/District', 6, 1],
					[7, 'Social Welfare Centre', 7, 1],
					[8, 'Water and Sewerage Company', 8, 1],
					[9, 'Voivodeship Job Centre', 9, 1],
					[10, 'District Job Center', 10, 1],
					[11, 'Court of justice', 11, 1],
					[12, 'Attorney General\'s Office', 12, 1],
					[13, 'Other', 44, 1],
					[14, 'Developers', 14, 1],
					[15, 'Real Estate', 15, 1],
					[16, 'Primary Schools', 16, 1],
					[17, 'High Schools', 17, 1],
					[18, 'Banking', 18, 1],
					[19, 'Capital Market', 19, 1],
					[20, 'Financial Services', 20, 1],
					[21, 'Investments', 21, 1],
					[22, 'Insurance', 22, 1],
					[23, 'Retail', 23, 1],
					[24, 'Wholesale', 24, 1],
					[25, 'Resale', 25, 1],
					[26, 'Automotive', 26, 1],
					[27, 'Plastics', 27, 1],
					[28, 'Chamical', 28, 1],
					[29, 'Raw material', 29, 1],
					[30, 'Fuel', 30, 1],
					[31, 'Wood and paper', 31, 1],
					[32, 'Electromechanical', 32, 1],
					[33, 'Pharmaceutical', 33, 1],
					[34, 'Building Materials', 34, 1],
					[35, 'Metal', 35, 1],
					[36, 'Light', 36, 1],
					[37, 'Food industry', 37, 1],
					[38, 'Recycling', 38, 1],
					[39, 'Army', 39, 1],
					[40, 'Police', 40, 1],
					[41, 'Information Technology', 41, 1],
					[42, 'Telecommunication', 42, 1],
					[43, 'Media', 43, 1],
				]
			],
			'vtiger_subindustry_seq' => [
				'columns' => ['id'],
				'values' => [
					[43],
				]
			],
			'vtiger_subunit' => [
				'columns' => ['subunitid', 'subunit', 'sortorderid', 'presence'],
				'values' => [
					[1, '50g', 1, 1],
					[2, '100g', 2, 1],
					[3, '300g', 3, 1],
					[4, '500g', 4, 1],
					[5, '100ml', 5, 1],
					[6, '250ml', 6, 1],
					[7, '330ml', 7, 1],
					[8, '500ml', 8, 1],
				]
			],
			'vtiger_support_processes' => [
				'columns' => ['id', 'ticket_status_indicate_closing'],
				'values' => [
					[1, ''],
				]
			],
			'vtiger_svendorenquiries_status' => [
				'columns' => ['svendorenquiries_statusid', 'svendorenquiries_status', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'PLL_DRAFT', 1, 773, 1],
					[2, 'PLL_IN_REALIZATION', 1, 774, 2],
					[3, 'PLL_FOR_VERIFICATION', 1, 775, 3],
					[4, 'PLL_CANCELLED', 1, 776, 4],
					[5, 'PLL_COMPLETED', 1, 777, 5],
				]
			],
			'vtiger_tab' => [
				'columns' => ['tabid', 'name', 'presence', 'tabsequence', 'tablabel', 'modifiedby', 'modifiedtime', 'customized', 'ownedby', 'isentitytype', 'version', 'parent', 'color', 'coloractive', 'type'],
				'values' => [
					[1, 'Dashboard', 0, 12, 'Dashboards', null, null, 0, 1, 0, null, 'Analytics', null, 0, 0],
					[3, 'Home', 0, 1, 'Home', null, null, 0, 1, 0, null, null, null, 0, 0],
					[4, 'Contacts', 0, 6, 'Contacts', null, null, 0, 0, 1, null, 'Sales', null, 0, 0],
					[6, 'Accounts', 0, 5, 'Accounts', null, null, 0, 0, 1, null, 'Sales', null, 0, 0],
					[7, 'Leads', 0, 4, 'Leads', null, null, 0, 0, 1, null, 'Sales', null, 0, 0],
					[8, 'Documents', 0, 9, 'Documents', null, null, 0, 0, 1, null, 'Tools', null, 0, 0],
					[9, 'Calendar', 0, 3, 'Calendar', null, null, 0, 0, 1, null, 'Tools', null, 0, 0],
					[13, 'HelpDesk', 0, 11, 'HelpDesk', null, null, 0, 0, 1, null, 'Support', null, 0, 0],
					[14, 'Products', 0, 8, 'Products', null, null, 0, 0, 1, null, 'Inventory', null, 0, 0],
					[15, 'Faq', 0, -1, 'Faq', null, null, 0, 1, 1, null, 'Support', null, 0, 0],
					[16, 'Events', 2, -1, 'Events', null, null, 0, 0, 1, null, null, null, 0, 0],
					[18, 'Vendors', 0, -1, 'Vendors', null, null, 0, 0, 1, null, 'Inventory', null, 0, 0],
					[19, 'PriceBooks', 0, -1, 'PriceBooks', null, null, 0, 1, 1, null, 'Inventory', null, 0, 0],
					[24, 'Rss', 0, -1, 'Rss', null, null, 0, 1, 0, null, 'Tools', null, 0, 0],
					[25, 'Reports', 0, -1, 'Reports', null, null, 0, 1, 0, null, 'Analytics', null, 0, 0],
					[26, 'Campaigns', 0, -1, 'Campaigns', null, null, 0, 0, 1, null, 'Marketing', null, 0, 0],
					[27, 'Portal', 0, -1, 'Portal', null, null, 0, 1, 0, null, 'Tools', null, 0, 0],
					[29, 'Users', 0, -1, 'Users', null, null, 0, 1, 0, null, null, null, 0, 0],
					[30, 'Import', 0, -1, 'Import', null, null, 1, 0, 0, '1.7', '', null, 0, 0],
					[32, 'ModTracker', 0, -1, 'ModTracker', null, null, 0, 0, 0, '1.2', '', null, 0, 0],
					[34, 'ServiceContracts', 0, -1, 'Service Contracts', null, null, 0, 0, 1, '2.4', 'Support', null, 0, 0],
					[35, 'Services', 0, -1, 'Services', null, null, 0, 0, 1, '2.6', 'Inventory', null, 0, 0],
					[37, 'Assets', 0, -1, 'Assets', null, null, 0, 0, 1, '2.0', 'Inventory', null, 0, 0],
					[40, 'ModComments', 0, -1, 'Comments', null, null, 0, 0, 1, '2.1', 'Tools', null, 0, 0],
					[41, 'ProjectMilestone', 0, -1, 'ProjectMilestone', null, null, 0, 0, 1, '3.0', 'Support', null, 0, 0],
					[42, 'ProjectTask', 0, -1, 'ProjectTask', null, null, 0, 0, 1, '3.1', 'Support', null, 0, 0],
					[43, 'Project', 0, -1, 'Project', null, null, 0, 0, 1, '3.3', 'Support', null, 0, 0],
					[45, 'SMSNotifier', 0, -1, 'SMSNotifier', null, null, 0, 0, 1, '1.9', 'Tools', null, 0, 0],
					[48, 'OSSMail', 1, -1, 'OSSMail', null, null, 0, 0, 0, '1.50', 'Tools', null, 0, 0],
					[50, 'Password', 0, -1, 'Password', null, null, 0, 0, 0, '1.00', '', null, 0, 0],
					[51, 'OSSTimeControl', 0, -1, 'OSSTimeControl', null, null, 0, 0, 1, '1.0.4', 'Tools', null, 0, 0],
					[53, 'OSSMailScanner', 0, -1, 'OSSMailScanner', null, null, 0, 0, 0, '1.30', '', null, 0, 0],
					[54, 'OSSMailView', 0, -1, 'OSSMailView', null, null, 0, 0, 1, '1.36', 'Tools', null, 0, 0],
					[57, 'OSSOutsourcedServices', 0, -1, 'OSSOutsourcedServices', null, null, 0, 0, 1, '1.0.2', 'Tools', null, 0, 0],
					[58, 'OSSSoldServices', 0, -1, 'OSSSoldServices', null, null, 0, 0, 1, '1.0.2', 'Tools', null, 0, 0],
					[59, 'OutsourcedProducts', 0, -1, 'OutsourcedProducts', null, null, 0, 0, 1, '1.0', 'Support', null, 0, 0],
					[60, 'OSSPasswords', 0, -1, 'OSSPasswords', null, null, 0, 0, 1, '0.96', 'Tools', null, 0, 0],
					[61, 'OSSEmployees', 0, -1, 'OSSEmployees', null, null, 0, 0, 1, '1.0.1', 'Tools', null, 0, 0],
					[73, 'ApiAddress', 0, -1, 'Api Address', null, null, 0, 0, 0, '0.1', '', null, 0, 0],
					[74, 'CallHistory', 0, -1, 'CallHistory', null, null, 0, 0, 1, '0', '0', null, 0, 0],
					[75, 'Ideas', 0, -1, 'Ideas', null, null, 0, 0, 1, '0', '0', null, 0, 0],
					[78, 'HolidaysEntitlement', 0, -1, 'HolidaysEntitlement', null, null, 0, 0, 1, '0', '0', null, 0, 0],
					[79, 'PaymentsIn', 0, -1, 'PaymentsIn', null, null, 0, 0, 1, '1.0.12', '0', null, 0, 0],
					[80, 'PaymentsOut', 0, -1, 'PaymentsOut', null, null, 0, 0, 1, '1.0.2', '0', null, 0, 0],
					[81, 'LettersIn', 0, -1, 'LettersIn', null, null, 0, 0, 1, '1.0.0', '0', null, 0, 0],
					[82, 'LettersOut', 0, -1, 'LettersOut', null, null, 0, 0, 1, '1.0.0', '0', null, 0, 0],
					[83, 'Announcements', 0, -1, 'Announcements', null, null, 0, 0, 1, '0', '0', null, 0, 0],
					[84, 'Reservations', 0, -1, 'Reservations', null, null, 0, 0, 1, '1.0.0', '0', null, 0, 0],
					[85, 'SQuoteEnquiries', 0, -1, 'SQuoteEnquiries', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[86, 'SSalesProcesses', 0, -1, 'SSalesProcesses', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[87, 'SRequirementsCards', 0, -1, 'SRequirementsCards', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[88, 'SCalculations', 0, -1, 'SCalculations', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[89, 'SQuotes', 0, -1, 'SQuotes', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[90, 'SSingleOrders', 0, -1, 'SSingleOrders', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[91, 'SRecurringOrders', 0, -1, 'SRecurringOrders', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[92, 'Partners', 0, -1, 'Partners', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[93, 'Competition', 0, -1, 'Competition', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[94, 'FBookkeeping', 0, -1, 'FBookkeeping', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[95, 'FInvoice', 0, -1, 'FInvoice', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[96, 'KnowledgeBase', 0, -1, 'KnowledgeBase', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[97, 'IStorages', 0, -1, 'IStorages', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[98, 'IGRN', 0, -1, 'IGRN', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[99, 'FInvoiceProforma', 0, -1, 'FInvoiceProforma', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[100, 'IGDN', 0, -1, 'IGDN', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[101, 'IIDN', 0, -1, 'IIDN', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[102, 'IGIN', 0, -1, 'IGIN', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[103, 'IPreOrder', 0, -1, 'IPreOrder', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[104, 'ISTDN', 0, -1, 'ISTDN', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[105, 'ISTN', 0, -1, 'ISTN', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[106, 'ISTRN', 0, -1, 'ISTRN', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[107, 'FCorectingInvoice', 0, -1, 'FCorectingInvoice', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[108, 'IGRNC', 0, -1, 'IGRNC', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[109, 'IGDNC', 0, -1, 'IGDNC', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[110, 'OpenStreetMap', 0, -1, 'OpenStreetMap', null, null, 0, 0, 0, '0', '', null, 0, 0],
					[111, 'Notification', 0, -1, 'Notification', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[112, 'EmailTemplates', 0, 13, 'EmailTemplates', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[113, 'CFixedAssets', 0, 14, 'CFixedAssets', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[114, 'CInternalTickets', 0, 15, 'CInternalTickets', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[115, 'FInvoiceCost', 0, 16, 'FInvoiceCost', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[116, 'CMileageLogbook', 0, 17, 'CMileageLogbook', null, null, 0, 0, 1, '0', '', null, 0, 0],
					[117, 'SVendorEnquiries', 0, 18, 'SVendorEnquiries', null, null, 0, 0, 1, '0', '', null, 0, 1],
					[118, 'Chat', 0, 19, 'Chat', null, null, 0, 0, 0, '0', '', null, 0, 0],
				]
			],
			'vtiger_tab_info' => [
				'columns' => ['tabid', 'prefname', 'prefvalue'],
				'values' => [
					[30, 'vtiger_min_version', '6.0.0rc'],
					[30, 'vtiger_max_version', '6.*'],
					[32, 'vtiger_min_version', '6.0.0rc'],
					[33, 'vtiger_min_version', '6.0.0'],
					[33, 'vtiger_max_version', '6.*'],
					[34, 'vtiger_min_version', '6.0.0rc'],
					[35, 'vtiger_min_version', '6.0.0rc'],
					[35, 'vtiger_max_version', '6.*'],
					[37, 'vtiger_min_version', '6.0.0rc'],
					[37, 'vtiger_max_version', '6.*'],
					[40, 'vtiger_min_version', '6.0.0rc'],
					[40, 'vtiger_max_version', '6.*'],
					[41, 'vtiger_min_version', '6.0.0rc'],
					[41, 'vtiger_max_version', '6.*'],
					[42, 'vtiger_min_version', '6.0.0rc'],
					[43, 'vtiger_min_version', '6.0.0rc'],
					[45, 'vtiger_min_version', '6.0.0rc'],
					[45, 'vtiger_max_version', '6.*'],
					[48, 'vtiger_min_version', '6.0.0'],
					[48, 'vtiger_max_version', '6.*'],
					[50, 'vtiger_min_version', '6.0.0'],
					[51, 'vtiger_min_version', '6.1.0'],
					[51, 'vtiger_max_version', '6.*'],
					[53, 'vtiger_min_version', '6.0.0'],
					[53, 'vtiger_max_version', '6.*'],
					[54, 'vtiger_min_version', '6.0.0'],
					[54, 'vtiger_max_version', '6.*'],
					[57, 'vtiger_min_version', '6.0.0'],
					[57, 'vtiger_max_version', '6.*'],
					[58, 'vtiger_min_version', '6.0.0'],
					[58, 'vtiger_max_version', '6.*'],
					[59, 'vtiger_min_version', '6.0.0'],
					[60, 'vtiger_min_version', '6.0.0rc'],
					[60, 'vtiger_max_version', '6.*'],
					[61, 'vtiger_min_version', '6.0.0'],
					[61, 'vtiger_max_version', '6.*'],
					[73, 'vtiger_min_version', '6.0.0rc'],
					[73, 'vtiger_max_version', '6.*'],
					[74, 'vtiger_min_version', '1.1.25 RC'],
					[75, 'vtiger_min_version', '1.2.100 RC'],
					[78, 'vtiger_min_version', '1.3.103 RC'],
					[79, 'vtiger_min_version', '6.0.0'],
					[80, 'vtiger_min_version', '6.0.0'],
					[81, 'vtiger_min_version', '6.0.0'],
					[81, 'vtiger_max_version', '6.*'],
					[82, 'vtiger_min_version', '6.0.0'],
					[82, 'vtiger_max_version', '6.*'],
					[83, 'vtiger_min_version', '1.3.150 RC'],
					[84, 'vtiger_min_version', '6.1.0'],
					[84, 'vtiger_max_version', '6.*'],
					[85, 'vtiger_min_version', '2.3.387'],
					[86, 'vtiger_min_version', '2.3.413'],
					[87, 'vtiger_min_version', '2.3.426'],
					[88, 'vtiger_min_version', '2.3.445'],
					[89, 'vtiger_min_version', '2.3.457'],
					[90, 'vtiger_min_version', '2.3.461'],
					[91, 'vtiger_min_version', '2.3.471'],
					[92, 'vtiger_min_version', '2.3.774'],
					[93, 'vtiger_min_version', '2.3.782'],
					[95, 'vtiger_min_version', '2.3.1075'],
					[97, 'vtiger_min_version', '2.3.1118'],
					[98, 'vtiger_min_version', '2.3.1153'],
					[99, 'vtiger_min_version', '2.3.1096'],
					[100, 'vtiger_min_version', '2.3.1158'],
					[101, 'vtiger_min_version', '2.3.1175'],
					[102, 'vtiger_min_version', '2.3.1180'],
					[103, 'vtiger_min_version', '2.3.1184'],
					[104, 'vtiger_min_version', '2.3.1315'],
					[105, 'vtiger_min_version', '2.3.1321'],
					[106, 'vtiger_min_version', '2.3.1326'],
					[107, 'vtiger_min_version', '2.3.1422'],
					[108, 'vtiger_min_version', '2.3.1514'],
					[109, 'vtiger_min_version', '2.3.1528'],
					[110, 'vtiger_min_version', '3.2.999'],
					[111, 'vtiger_min_version', '3.3.999'],
					[113, 'vtiger_min_version', '3.5'],
					[114, 'vtiger_min_version', '3.5'],
					[115, 'vtiger_min_version', '3.5'],
					[116, 'vtiger_min_version', '3.4.1517'],
					[117, 'vtiger_min_version', '3.5'],
					[118, 'vtiger_min_version', '4.0.999'],
				]
			],
			'vtiger_taskpriority' => [
				'columns' => ['taskpriorityid', 'taskpriority', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'High', 1, 170, 0],
					[2, 'Medium', 1, 171, 1],
					[3, 'Low', 1, 172, 2],
				]
			],
			'vtiger_taskpriority_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_ticketpriorities' => [
				'columns' => ['ticketpriorities_id', 'ticketpriorities', 'presence', 'picklist_valueid', 'sortorderid', 'color'],
				'values' => [
					[1, 'Low', 1, 182, 0, '	#E6FAD8'],
					[2, 'Normal', 1, 183, 1, '	#E6FAD8'],
					[3, 'High', 1, 184, 2, '	#E6FAD8'],
					[4, 'Urgent', 1, 185, 3, '	#E6FAD8'],
				]
			],
			'vtiger_ticketpriorities_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_ticketseverities' => [
				'columns' => ['ticketseverities_id', 'ticketseverities', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Minor', 1, 186, 0],
					[2, 'Major', 1, 187, 1],
					[3, 'Feature', 1, 188, 2],
					[4, 'Critical', 1, 189, 3],
				]
			],
			'vtiger_ticketseverities_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_ticketstatus' => [
				'columns' => ['ticketstatus_id', 'ticketstatus', 'presence', 'picklist_valueid', 'sortorderid', 'color'],
				'values' => [
					[1, 'Open', 0, 190, 1, '#E6FAD8'],
					[2, 'In Progress', 0, 191, 5, '#E6FAD8'],
					[3, 'Wait For Response', 0, 192, 3, '#E6FAD8'],
					[4, 'Closed', 0, 193, 8, '#E6FAD8'],
					[5, 'Answered', 0, 387, 2, '#E6FAD8'],
					[6, 'Rejected', 0, 557, 9, '#E6FAD8'],
					[7, 'PLL_SUBMITTED_COMMENTS', 1, 694, 4, '#E6FAD8'],
					[8, 'PLL_FOR_APPROVAL', 1, 695, 6, '#E6FAD8'],
					[9, 'PLL_TO_CLOSE', 1, 696, 7, '#E6FAD8'],
				]
			],
			'vtiger_ticketstatus_seq' => [
				'columns' => ['id'],
				'values' => [
					[9],
				]
			],
			'vtiger_time_zone' => [
				'columns' => ['time_zoneid', 'time_zone', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Pacific/Midway', 0, 1],
					[2, 'Pacific/Samoa', 1, 1],
					[3, 'Pacific/Honolulu', 2, 1],
					[4, 'America/Anchorage', 3, 1],
					[5, 'America/Los_Angeles', 4, 1],
					[6, 'America/Tijuana', 5, 1],
					[7, 'America/Denver', 6, 1],
					[8, 'America/Chihuahua', 7, 1],
					[9, 'America/Mazatlan', 8, 1],
					[10, 'America/Phoenix', 9, 1],
					[11, 'America/Regina', 10, 1],
					[12, 'America/Tegucigalpa', 11, 1],
					[13, 'America/Chicago', 12, 1],
					[14, 'America/Mexico_City', 13, 1],
					[15, 'America/Monterrey', 14, 1],
					[16, 'America/New_York', 15, 1],
					[17, 'America/Bogota', 16, 1],
					[18, 'America/Lima', 17, 1],
					[19, 'America/Rio_Branco', 18, 1],
					[20, 'America/Indiana/Indianapolis', 19, 1],
					[21, 'America/Caracas', 20, 1],
					[22, 'America/Halifax', 21, 1],
					[23, 'America/Manaus', 22, 1],
					[24, 'America/Santiago', 23, 1],
					[25, 'America/La_Paz', 24, 1],
					[26, 'America/Cuiaba', 25, 1],
					[27, 'America/Asuncion', 26, 1],
					[28, 'America/St_Johns', 27, 1],
					[29, 'America/Argentina/Buenos_Aires', 28, 1],
					[30, 'America/Sao_Paulo', 29, 1],
					[31, 'America/Godthab', 30, 1],
					[32, 'America/Montevideo', 31, 1],
					[33, 'Atlantic/South_Georgia', 32, 1],
					[34, 'Atlantic/Azores', 33, 1],
					[35, 'Atlantic/Cape_Verde', 34, 1],
					[36, 'Europe/London', 35, 1],
					[37, 'UTC', 36, 1],
					[38, 'Africa/Monrovia', 37, 1],
					[39, 'Africa/Casablanca', 38, 1],
					[40, 'Europe/Belgrade', 39, 1],
					[41, 'Europe/Sarajevo', 40, 1],
					[42, 'Europe/Brussels', 41, 1],
					[43, 'Africa/Algiers', 42, 1],
					[44, 'Europe/Amsterdam', 43, 1],
					[45, 'Europe/Minsk', 44, 1],
					[46, 'Africa/Cairo', 45, 1],
					[47, 'Europe/Helsinki', 46, 1],
					[48, 'Europe/Athens', 47, 1],
					[49, 'Europe/Istanbul', 48, 1],
					[50, 'Asia/Jerusalem', 49, 1],
					[51, 'Asia/Amman', 50, 1],
					[52, 'Asia/Beirut', 51, 1],
					[53, 'Africa/Windhoek', 52, 1],
					[54, 'Africa/Harare', 53, 1],
					[55, 'Asia/Kuwait', 54, 1],
					[56, 'Asia/Baghdad', 55, 1],
					[57, 'Africa/Nairobi', 56, 1],
					[58, 'Asia/Tehran', 57, 1],
					[59, 'Asia/Tbilisi', 58, 1],
					[60, 'Europe/Moscow', 59, 1],
					[61, 'Asia/Muscat', 60, 1],
					[62, 'Asia/Baku', 61, 1],
					[63, 'Asia/Yerevan', 62, 1],
					[64, 'Asia/Karachi', 63, 1],
					[65, 'Asia/Tashkent', 64, 1],
					[66, 'Asia/Kolkata', 65, 1],
					[67, 'Asia/Colombo', 66, 1],
					[68, 'Asia/Katmandu', 67, 1],
					[69, 'Asia/Dhaka', 68, 1],
					[70, 'Asia/Almaty', 69, 1],
					[71, 'Asia/Yekaterinburg', 70, 1],
					[72, 'Asia/Rangoon', 71, 1],
					[73, 'Asia/Novosibirsk', 72, 1],
					[74, 'Asia/Bangkok', 73, 1],
					[75, 'Asia/Brunei', 74, 1],
					[76, 'Asia/Krasnoyarsk', 75, 1],
					[77, 'Asia/Ulaanbaatar', 76, 1],
					[78, 'Asia/Kuala_Lumpur', 77, 1],
					[79, 'Asia/Taipei', 78, 1],
					[80, 'Australia/Perth', 79, 1],
					[81, 'Asia/Irkutsk', 80, 1],
					[82, 'Asia/Seoul', 81, 1],
					[83, 'Asia/Tokyo', 82, 1],
					[84, 'Australia/Darwin', 83, 1],
					[85, 'Australia/Adelaide', 84, 1],
					[86, 'Australia/Canberra', 85, 1],
					[87, 'Australia/Brisbane', 86, 1],
					[88, 'Australia/Hobart', 87, 1],
					[89, 'Asia/Vladivostok', 88, 1],
					[90, 'Pacific/Guam', 89, 1],
					[91, 'Asia/Yakutsk', 90, 1],
					[92, 'Pacific/Fiji', 92, 1],
					[93, 'Asia/Kamchatka', 93, 1],
					[94, 'Pacific/Auckland', 94, 1],
					[95, 'Asia/Magadan', 95, 1],
					[96, 'Pacific/Tongatapu', 96, 1],
					[97, 'Etc/GMT-11', 91, 1],
				]
			],
			'vtiger_time_zone_seq' => [
				'columns' => ['id'],
				'values' => [
					[96],
				]
			],
			'vtiger_timecontrol_type' => [
				'columns' => ['timecontrol_typeid', 'timecontrol_type', 'sortorderid', 'presence', 'color'],
				'values' => [
					[1, 'PLL_WORKING_TIME', 1, 1, '#EDC240'],
					[2, 'PLL_BREAK_TIME', 2, 1, '#AFD8F8'],
					[3, 'PLL_HOLIDAY_TIME', 3, 1, '#CB4B4B'],
				]
			],
			'vtiger_timecontrol_type_seq' => [
				'columns' => ['id'],
				'values' => [
					[3],
				]
			],
			'vtiger_tracking_unit' => [
				'columns' => ['tracking_unitid', 'tracking_unit', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[2, 'Hours', 1, 211, 2],
					[3, 'Days', 1, 212, 3],
					[4, 'Incidents', 1, 213, 4],
				]
			],
			'vtiger_tracking_unit_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_trees_templates' => [
				'columns' => ['templateid', 'name', 'module', 'access', 'share'],
				'values' => [
					[1, 'System', 8, 0, null],
					[2, 'Category', 14, 0, null],
					[3, 'Category', 35, 0, null],
					[4, 'Category', 57, 0, null],
					[5, 'Category', 58, 0, null],
					[6, 'Category', 59, 0, null],
					[7, 'Reservations', 84, 0, null],
					[8, 'Category', 85, 0, null],
					[9, 'Category', 86, 0, null],
					[10, 'Category', 87, 0, null],
					[11, 'Category', 88, 0, null],
					[12, 'Category', 89, 0, null],
					[13, 'Category', 90, 0, null],
					[14, 'Category', 91, 0, null],
					[15, 'Category', 37, 0, null],
					[16, 'Category', 96, 0, null],
					[17, 'Category', 13, 0, null],
					[18, 'Category', 92, 0, null],
					[19, 'Category', 113, 1, null],
					[20, 'Category', 115, 1, null],
					[21, 'Category', 95, 1, null],
					[22, 'Category', 117, 0, null],
				]
			],
			'vtiger_trees_templates_data' => [
				'columns' => ['templateid', 'name', 'tree', 'parentTree', 'depth', 'label', 'state', 'icon'],
				'values' => [
					[2, 'Hardware', 'T1', 'T1', 0, 'Hardware', '', ''],
					[2, 'Software', 'T2', 'T2', 0, 'Software', '', ''],
					[2, 'CRM Applications', 'T3', 'T3', 0, 'CRM Applications', '', ''],
					[2, 'Antivirus', 'T4', 'T4', 0, 'Antivirus', '', ''],
					[2, 'Backup', 'T5', 'T5', 0, 'Backup', '', ''],
					[3, 'Hardware', 'T1', 'T1', 0, 'Hardware', '', ''],
					[3, 'Software', 'T2', 'T2', 0, 'Software', '', ''],
					[3, 'CRM Applications', 'T3', 'T3', 0, 'CRM Applications', '', ''],
					[3, 'Antivirus', 'T4', 'T4', 0, 'Antivirus', '', ''],
					[3, 'Backup', 'T5', 'T5', 0, 'Backup', '', ''],
					[4, 'Hardware', 'T1', 'T1', 0, 'Hardware', '', ''],
					[4, 'Software', 'T2', 'T2', 0, 'Software', '', ''],
					[4, 'CRM Applications', 'T3', 'T3', 0, 'CRM Applications', '', ''],
					[4, 'Antivirus', 'T4', 'T4', 0, 'Antivirus', '', ''],
					[4, 'Backup', 'T5', 'T5', 0, 'Backup', '', ''],
					[5, 'Hardware', 'T1', 'T1', 0, 'Hardware', '', ''],
					[5, 'Software', 'T2', 'T2', 0, 'Software', '', ''],
					[5, 'CRM Applications', 'T3', 'T3', 0, 'CRM Applications', '', ''],
					[5, 'Antivirus', 'T4', 'T4', 0, 'Antivirus', '', ''],
					[5, 'Backup', 'T5', 'T5', 0, 'Backup', '', ''],
					[6, 'Hardware', 'T1', 'T1', 0, 'Hardware', '', ''],
					[6, 'Software', 'T2', 'T2', 0, 'Software', '', ''],
					[6, 'CRM Applications', 'T3', 'T3', 0, 'CRM Applications', '', ''],
					[6, 'Antivirus', 'T4', 'T4', 0, 'Antivirus', '', ''],
					[6, 'Backup', 'T5', 'T5', 0, 'Backup', '', ''],
					[7, 'LBL_MEETING_ROOMS', 'T3', 'T3', 0, 'LBL_MEETING_ROOMS', '', ''],
					[7, 'LBL_EQUIPMENT', 'T2', 'T2', 0, 'LBL_EQUIPMENT', '', ''],
					[7, 'LBL_CARS', 'T1', 'T1', 0, 'LBL_CARS', '', ''],
					[8, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[9, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[10, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[11, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[12, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[13, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[14, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[15, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[1, 'Default', 'T1', 'T1', 0, 'Default', '', ''],
					[1, 'Mails', 'T2', 'T2', 0, 'Mails', '', ''],
					[16, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[17, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[18, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
					[19, 'none', 'T1', 'T1', 0, 'none', '', ''],
					[20, 'none', 'T1', 'T1', 0, 'none', '', ''],
					[21, 'none', 'T1', 'T1', 0, 'none', '', ''],
					[22, 'LBL_NONE', 'T1', 'T1', 0, 'LBL_NONE', '', ''],
				]
			],
			'vtiger_usageunit' => [
				'columns' => ['usageunitid', 'usageunit', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'pcs', 1, 194, 0],
					[2, 'pack', 1, 195, 1],
					[3, 'kg', 1, 196, 2],
					[4, 'm', 1, 197, 3],
					[5, 'l', 1, 198, 4],
				]
			],
			'vtiger_usageunit_seq' => [
				'columns' => ['id'],
				'values' => [
					[5],
				]
			],
			'vtiger_user2role' => [
				'columns' => ['userid', 'roleid'],
				'values' => [
					[1, 'H2'],
				]
			],
			'vtiger_users' => [
				'columns' => ['id', 'user_name', 'user_password', 'cal_color', 'first_name', 'last_name', 'reports_to_id', 'is_admin', 'currency_id', 'description', 'date_entered', 'date_modified', 'modified_user_id', 'email1', 'status', 'user_preferences', 'tz', 'holidays', 'namedays', 'workdays', 'weekstart', 'date_format', 'hour_format', 'start_hour', 'end_hour', 'activity_view', 'lead_view', 'imagename', 'deleted', 'confirm_password', 'internal_mailer', 'reminder_interval', 'reminder_next_time', 'accesskey', 'theme', 'language', 'time_zone', 'currency_grouping_pattern', 'currency_decimal_separator', 'currency_grouping_separator', 'currency_symbol_placement', 'phone_crm_extension', 'no_of_currency_decimals', 'truncate_trailing_zeros', 'dayoftheweek', 'callduration', 'othereventduration', 'default_record_view', 'leftpanelhide', 'rowheight', 'defaulteventstatus', 'defaultactivitytype', 'is_owner', 'emailoptout', 'available', 'auto_assign', 'records_limit'],
				'values' => [
					[1, 'admin', '$1$ad000000$hzXFXvL3XVlnUE/X.1n9t/', '#E6FAD8', '', 'Administrator', 0, 'on', 1, '', 'NOW()', 'NOW()', null, 'help@yetiforce.com', 'Active', null, null, null, null, null, null, 'yyyy-mm-dd', '24', '08:00', '23:00', 'This Month', 'Today', '', 0, '$1$ad000000$nYTnfhTZRmUP.wQT9y1AE.', 1, '15 Minutes', null, 'aOFXop10GCJ1uw0P', 'twilight', 'en_us', 'Europe/Sarajevo', '123456789', ',', ' ', '1.0$', '', 2, 1, 'Monday', 60, 60, 'Summary', 0, 'medium', 'PLL_PLANNED', 'Meeting', '1', 1, 1, 0, null],
				]
			],
			'vtiger_users2group' => [
				'columns' => ['groupid', 'userid'],
				'values' => [
					[3, 1],
				]
			],
			'vtiger_users_seq' => [
				'columns' => ['id'],
				'values' => [
					[4],
				]
			],
			'vtiger_verification' => [
				'columns' => ['verificationid', 'verification', 'presence', 'picklist_valueid', 'sortorderid'],
				'values' => [
					[1, 'Address details', 1, 440, 1],
					[5, 'Contact details', 1, 444, 2],
					[6, 'Registration details', 1, 445, 3],
				]
			],
			'vtiger_verification_seq' => [
				'columns' => ['id'],
				'values' => [
					[6],
				]
			],
			'vtiger_version' => [
				'columns' => ['id', 'old_version', 'current_version'],
				'values' => [
					[1, '2.2.0', '2.2.0'],
				]
			],
			'vtiger_version_seq' => [
				'columns' => ['id'],
				'values' => [
					[1],
				]
			],
			'vtiger_visibility' => [
				'columns' => ['visibilityid', 'visibility', 'sortorderid', 'presence'],
				'values' => [
					[1, 'Private', 0, 1],
					[2, 'Public', 1, 1],
				]
			],
			'vtiger_visibility_seq' => [
				'columns' => ['id'],
				'values' => [
					[2],
				]
			],
			'vtiger_widgets' => [
				'columns' => ['id', 'tabid', 'type', 'label', 'wcol', 'sequence', 'nomargin', 'data'],
				'values' => [
					[1, 6, 'Summary', null, 1, 0, null, '[]'],
					[2, 6, 'Comments', 'ModComments', 2, 5, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[3, 6, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[4, 6, 'Activities', 'Calendar', 2, 3, null, '{"limit":"5"}'],
					[5, 6, 'RelatedModule', 'Contacts', 2, 2, null, '{"limit":"5","relatedmodule":"4","columns":"3","action":"1","filter":"-"}'],
					[7, 7, 'Summary', null, 1, 1, null, '[]'],
					[8, 7, 'Comments', 'ModComments', 1, 2, 1, '{"relatedmodule":"ModComments","limit":"5"}'],
					[9, 7, 'Updates', 'LBL_UPDATES', 2, 6, null, '[]'],
					[10, 7, 'Activities', 'Calendar', 2, 4, null, '{"limit":"5"}'],
					[11, 7, 'EmailList', 'Emails', 2, 5, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[12, 6, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[14, 4, 'Summary', null, 1, 1, null, '[]'],
					[15, 4, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[16, 4, 'Updates', 'LBL_UPDATES', 1, 3, null, '[]'],
					[17, 4, 'Activities', 'Calendar', 2, 4, null, '{"limit":"5"}'],
					[18, 4, 'EmailList', 'Emails', 2, 5, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[26, 43, 'Summary', null, 1, 1, null, '[]'],
					[27, 43, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[28, 43, 'Updates', 'LBL_UPDATES', 1, 3, null, '[]'],
					[30, 43, 'EmailList', 'Emails', 1, 5, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[31, 43, 'RelatedModule', 'ProjectTask', 2, 6, null, '{"limit":"5","relatedmodule":"42","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[32, 43, 'RelatedModule', 'ProjectMilestone', 2, 7, null, '{"limit":"5","relatedmodule":"41","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[34, 13, 'Summary', null, 1, 1, null, '[]'],
					[35, 13, 'Comments', 'ModComments', 1, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[36, 13, 'Updates', 'LBL_UPDATES', 1, 3, null, '[]'],
					[37, 13, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[38, 13, 'Activities', 'Calendar', 2, 5, null, '{"limit":"5"}'],
					[39, 13, 'RelatedModule', 'Documents', 2, 6, null, '{"limit":"5","relatedmodule":"8","columns":"3","action":"1","filter":"-"}'],
					[40, 51, 'Summary', null, 1, 1, null, '[]'],
					[41, 51, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[42, 51, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"5","relatedmodule":"8","columns":"3","filter":"-"}'],
					[43, 7, 'RelatedModule', 'Contacts', 2, 3, null, '{"limit":"5","relatedmodule":"4","columns":"3","action":"1","filter":"-"}'],
					[47, 13, 'WYSIWYG', 'WYSIWYG', 1, 7, null, '{"field_name":"description"}'],
					[48, 54, 'PreviewMail', null, 1, 1, null, '{"relatedmodule":"Emails"}'],
					[49, 42, 'Summary', null, 1, 0, null, '[]'],
					[50, 42, 'Comments', 'ModComments', 2, 1, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[51, 84, 'Summary', null, 1, 0, null, '[]'],
					[52, 84, 'Comments', '', 2, 1, null, '{"relatedmodule":"ModComments","limit":"10"}'],
					[55, 85, 'Summary', null, 1, 0, null, '[]'],
					[56, 85, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[57, 85, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[58, 85, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[59, 85, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[60, 87, 'Summary', null, 1, 0, null, '[]'],
					[61, 87, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[62, 87, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[63, 87, 'Comments', 'ModComments', 2, 5, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[64, 87, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[65, 88, 'Summary', null, 1, 0, null, '[]'],
					[66, 88, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[67, 88, 'Comments', 'ModComments', 2, 5, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[68, 88, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[69, 88, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[70, 89, 'Summary', null, 1, 0, null, '[]'],
					[71, 89, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[72, 89, 'Comments', 'ModComments', 2, 5, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[73, 89, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[74, 89, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[75, 90, 'Summary', null, 1, 0, null, '[]'],
					[76, 90, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[77, 90, 'Comments', 'ModComments', 2, 5, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[78, 90, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[79, 90, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[80, 86, 'Summary', null, 1, 0, null, '[]'],
					[81, 86, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[82, 86, 'Comments', 'ModComments', 2, 6, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[83, 86, 'RelatedModule', 'Documents', 2, 4, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"-","checkbox":"-"}'],
					[84, 86, 'EmailList', 'Emails', 2, 5, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[85, 92, 'Summary', null, 1, 0, null, '[]'],
					[86, 92, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[87, 92, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[88, 92, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[89, 92, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[90, 93, 'Summary', null, 1, 0, null, '[]'],
					[91, 93, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[92, 93, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[93, 93, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[94, 93, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[95, 18, 'Summary', null, 1, 0, null, '[]'],
					[96, 18, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[97, 18, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[98, 18, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[99, 18, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[100, 86, 'RelatedModule', 'Contacts', 1, 2, null, '{"limit":"5","relatedmodule":"4","columns":"1","action":"1","actionSelect":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[101, 92, 'RelatedModule', 'Contacts', 1, 5, null, '{"limit":"5","relatedmodule":"4","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[102, 93, 'RelatedModule', 'Contacts', 1, 5, null, '{"limit":"5","relatedmodule":"4","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[103, 6, 'RelatedModule', '', 1, 6, null, '{"relatedmodule":"86","limit":"10","columns":"3","action":"1","switchHeader":"0","filter":"-","checkbox":"-"}'],
					[104, 100, 'Summary', null, 1, 0, null, '[]'],
					[105, 100, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[106, 100, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[107, 100, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[108, 101, 'Summary', null, 1, 0, null, '[]'],
					[109, 101, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[110, 101, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[111, 101, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[112, 98, 'Summary', null, 1, 0, null, '[]'],
					[113, 98, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[114, 98, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[115, 98, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[116, 102, 'Comments', '', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[117, 102, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[118, 102, 'Summary', null, 1, 0, null, '[]'],
					[119, 102, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"-","checkbox":"-"}'],
					[120, 103, 'Summary', null, 1, 0, null, '[]'],
					[121, 103, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[122, 103, 'Comments', '', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[123, 103, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[124, 97, 'Summary', null, 1, 0, null, '[]'],
					[125, 97, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[126, 97, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[127, 97, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[128, 94, 'Summary', null, 1, 0, null, '[]'],
					[129, 94, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[130, 94, 'Comments', 'ModComments', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[131, 94, 'RelatedModule', 'Documents', 2, 3, null, '{"limit":"","relatedmodule":"8","columns":"3","action":"1","filter":"-","checkbox_selected":"","checkbox":"-"}'],
					[132, 104, 'Comments', '', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[133, 104, 'Summary', null, 1, 0, null, '[]'],
					[134, 104, 'RelatedModule', 'Documents', 2, 3, null, '{"relatedmodule":"8","limit":"","columns":"3","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[135, 104, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[136, 105, 'Comments', '', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[137, 105, 'Summary', null, 1, 0, null, '[]'],
					[138, 105, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[139, 105, 'RelatedModule', '', 2, 3, null, '{"relatedmodule":"8","limit":"","columns":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[140, 106, 'Summary', null, 1, 0, null, '[]'],
					[141, 106, 'Comments', '', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[142, 106, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[143, 106, 'RelatedModule', 'Documents', 2, 3, null, '{"relatedmodule":"8","limit":"","columns":"3","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[144, 96, 'Comments', '', 2, 2, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[145, 96, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[146, 96, 'RelatedModule', '', 2, 3, null, '{"relatedmodule":"8","limit":"5","columns":"1","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[147, 96, 'Summary', null, 1, 0, null, '[]'],
					[148, 86, 'Activities', 'Calendar', 2, 3, null, '{"limit":"5"}'],
					[149, 88, 'Activities', 'Calendar', 2, 2, null, '{"limit":"5"}'],
					[150, 85, 'Activities', 'Calendar', 1, 6, null, '{"limit":"5"}'],
					[151, 89, 'Activities', 'Calendar', 2, 2, null, '{"limit":"5"}'],
					[152, 91, 'Activities', 'Calendar', 2, 2, null, '{"limit":"5"}'],
					[153, 87, 'Activities', 'Calendar', 2, 2, null, '{"limit":"5"}'],
					[154, 90, 'Activities', 'Calendar', 2, 2, null, '{"limit":"5"}'],
					[155, 91, 'Summary', null, 1, 0, null, '[]'],
					[156, 91, 'Updates', 'LBL_UPDATES', 1, 1, null, '[]'],
					[157, 91, 'RelatedModule', 'Documents', 2, 3, null, '{"relatedmodule":"8","limit":"","columns":"3","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[158, 91, 'EmailList', 'Emails', 2, 4, null, '{"relatedmodule":"Emails","limit":"5"}'],
					[159, 91, 'Comments', 'ModComments', 2, 5, null, '{"relatedmodule":"ModComments","limit":"5"}'],
					[160, 26, 'Summary', null, 1, 0, 0, '[]'],
					[161, 26, 'Comments', 'ModComments', 2, 3, 0, '{"relatedmodule":"ModComments","limit":"5"}'],
					[162, 26, 'RelatedModule', 'Documents', 2, 2, 0, '{"relatedmodule":"8","limit":"5","columns":"3","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[163, 26, 'Updates', 'LBL_UPDATES', 1, 1, 0, '[]'],
					[164, 113, 'Summary', null, 1, 0, 0, '[]'],
					[165, 113, 'Comments', '', 2, 2, 0, '{"relatedmodule":"ModComments","limit":"5"}'],
					[166, 113, 'RelatedModule', '', 1, 1, 0, '{"relatedmodule":"8","limit":"5","columns":"1","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[167, 114, 'Summary', null, 1, 0, 0, '[]'],
					[168, 114, 'RelatedModule', '', 1, 1, 0, '{"relatedmodule":"8","limit":"5","columns":"1","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[169, 114, 'Comments', '', 2, 2, 0, '{"relatedmodule":"ModComments","limit":"5"}'],
					[170, 115, 'Summary', null, 1, 0, 0, '[]'],
					[171, 115, 'Comments', '', 2, 2, 0, '{"relatedmodule":"ModComments","limit":"5"}'],
					[172, 115, 'RelatedModule', '', 1, 1, 0, '{"relatedmodule":"8","limit":"5","columns":"1","action":"1","switchHeader":"-","filter":"-","checkbox":"-"}'],
					[173, 117, 'Activities', '', 2, 2, 0, '{"limit":"5"}'],
					[174, 117, 'EmailList', 'Emails', 2, 3, 0, '{"limit":"5"}'],
					[175, 117, 'Comments', '', 1, 1, 0, '{"relatedmodule":"ModComments","limit":"5"}'],
					[176, 117, 'Summary', null, 1, 0, 0, '[]'],
				]
			],
			'vtiger_ws_entity' => [
				'columns' => ['id', 'name', 'handler_path', 'handler_class', 'ismodule'],
				'values' => [
					[1, 'Campaigns', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[2, 'Vendors', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[3, 'Faq', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[8, 'PriceBooks', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[9, 'Calendar', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[10, 'Leads', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[11, 'Accounts', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[12, 'Contacts', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[14, 'Products', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[15, 'Documents', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[17, 'HelpDesk', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[18, 'Events', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[19, 'Users', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[20, 'Groups', 'include/Webservices/VtigerActorOperation.php', 'VtigerActorOperation', 0],
					[21, 'Currency', 'include/Webservices/VtigerActorOperation.php', 'VtigerActorOperation', 0],
					[22, 'DocumentFolders', 'include/Webservices/VtigerActorOperation.php', 'VtigerActorOperation', 0],
					[25, 'ServiceContracts', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[26, 'Services', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[27, 'Assets', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[28, 'ModComments', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[29, 'ProjectMilestone', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[30, 'ProjectTask', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[31, 'Project', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[32, 'SMSNotifier', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[38, 'OSSTimeControl', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[39, 'OSSMailView', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[41, 'OSSOutsourcedServices', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[42, 'OSSSoldServices', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[43, 'OutsourcedProducts', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[44, 'OSSPasswords', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[45, 'OSSEmployees', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[56, 'CallHistory', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[57, 'Ideas', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[60, 'HolidaysEntitlement', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[61, 'PaymentsIn', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[62, 'PaymentsOut', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[63, 'LettersIn', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[64, 'LettersOut', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[65, 'Announcements', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[66, 'Reservations', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[67, 'SQuoteEnquiries', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[68, 'SSalesProcesses', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[69, 'SRequirementsCards', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[70, 'SCalculations', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[71, 'SQuotes', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[72, 'SSingleOrders', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[73, 'SRecurringOrders', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[74, 'Partners', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[75, 'Competition', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[76, 'FBookkeeping', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[77, 'FInvoice', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[78, 'KnowledgeBase', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[79, 'IStorages', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[80, 'IGRN', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[81, 'FInvoiceProforma', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[82, 'IGDN', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[83, 'IIDN', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[84, 'IGIN', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[85, 'IPreOrder', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[86, 'ISTDN', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[87, 'ISTN', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[88, 'ISTRN', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[89, 'FCorectingInvoice', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[90, 'IGRNC', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[91, 'IGDNC', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[92, 'Notification', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[93, 'EmailTemplates', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[94, 'CFixedAssets', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[95, 'CInternalTickets', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[96, 'FInvoiceCost', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[97, 'CMileageLogbook', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
					[98, 'SVendorEnquiries', 'include/Webservices/VtigerModuleOperation.php', 'VtigerModuleOperation', 1],
				]
			],
			'vtiger_ws_entity_seq' => [
				'columns' => ['id'],
				'values' => [
					[92],
				]
			],
			'vtiger_ws_fieldtype' => [
				'columns' => ['fieldtypeid', 'uitype', 'fieldtype'],
				'values' => [
					[1, 15, 'picklist'],
					[2, 16, 'picklist'],
					[3, 19, 'text'],
					[4, 20, 'text'],
					[5, 21, 'text'],
					[6, 24, 'text'],
					[7, 3, 'autogenerated'],
					[8, 11, 'phone'],
					[9, 33, 'multipicklist'],
					[10, 17, 'url'],
					[11, 85, 'skype'],
					[12, 56, 'boolean'],
					[13, 156, 'boolean'],
					[14, 53, 'owner'],
					[15, 61, 'file'],
					[16, 28, 'file'],
					[17, 13, 'email'],
					[18, 71, 'currency'],
					[19, 72, 'currency'],
					[20, 50, 'reference'],
					[21, 51, 'reference'],
					[22, 57, 'reference'],
					[23, 58, 'reference'],
					[24, 73, 'reference'],
					[25, 75, 'reference'],
					[26, 76, 'reference'],
					[28, 80, 'reference'],
					[29, 81, 'reference'],
					[30, 101, 'reference'],
					[31, 52, 'userCreator'],
					[32, 357, 'reference'],
					[33, 59, 'reference'],
					[34, 66, 'referenceProcess'],
					[35, 67, 'referenceLink'],
					[36, 68, 'referenceSubProcess'],
					[37, 117, 'reference'],
					[38, 26, 'reference'],
					[39, 10, 'reference'],
					[40, 300, 'text'],
					[41, 120, 'sharedOwner'],
					[42, 301, 'modules'],
					[43, 302, 'tree'],
					[44, 303, 'taxes'],
					[45, 304, 'inventoryLimit'],
					[46, 305, 'multiReferenceValue'],
					[49, 309, 'categoryMultipicklist'],
					[50, 4, 'recordNumber'],
					[51, 310, 'companySelect'],
					[52, 342, 'recurrence'],
					[56, 311, 'multiImage'],
				]
			],
			'vtiger_ws_operation' => [
				'columns' => ['operationid', 'name', 'handler_path', 'handler_method', 'type', 'prelogin'],
				'values' => [
					[2, 'retrieve', 'include/Webservices/Retrieve.php', 'vtws_retrieve', 'GET', 0],
					[3, 'create', 'include/Webservices/Create.php', 'vtws_create', 'POST', 0],
					[4, 'update', 'include/Webservices/Update.php', 'vtws_update', 'POST', 0],
					[5, 'delete', 'include/Webservices/Delete.php', 'vtws_delete', 'POST', 0],
					[9, 'listtypes', 'include/Webservices/ModuleTypes.php', 'vtws_listtypes', 'GET', 0],
					[11, 'describe', 'include/Webservices/DescribeObject.php', 'vtws_describe', 'GET', 0],
					[13, 'convertlead', 'include/Webservices/ConvertLead.php', 'vtwsConvertlead', 'POST', 0],
					[14, 'revise', 'include/Webservices/Revise.php', 'vtws_revise', 'POST', 0],
					[16, 'deleteUser', 'include/Webservices/DeleteUser.php', 'vtws_deleteUser', 'POST', 0],
				]
			],
			'vtiger_ws_operation_parameters' => [
				'columns' => ['operationid', 'name', 'type', 'sequence'],
				'values' => [
					[1, 'accessKey', 'String', 2],
					[1, 'username', 'String', 1],
					[2, 'id', 'String', 1],
					[3, 'element', 'encoded', 2],
					[3, 'elementType', 'String', 1],
					[4, 'element', 'encoded', 1],
					[5, 'id', 'String', 1],
					[6, 'elementType', 'String', 2],
					[6, 'modifiedTime', 'DateTime', 1],
					[7, 'query', 'String', 1],
					[8, 'sessionName', 'String', 1],
					[9, 'fieldTypeList', 'encoded', 1],
					[10, 'username', 'String', 1],
					[11, 'elementType', 'String', 1],
					[13, 'accountName', 'String', 3],
					[13, 'assignedTo', 'String', 2],
					[13, 'avoidPotential', 'Boolean', 4],
					[13, 'leadId', 'String', 1],
					[13, 'potential', 'Encoded', 5],
					[14, 'element', 'Encoded', 1],
					[15, 'confirmPassword', 'String', 4],
					[15, 'id', 'String', 1],
					[15, 'newPassword', 'String', 3],
					[15, 'oldPassword', 'String', 2],
					[16, 'id', 'String', 1],
					[16, 'newOwnerId', 'String', 2],
					[18, 'alertid', 'string', 1],
					[19, 'module', 'string', 1],
					[20, 'record', 'string', 1],
					[21, 'record', 'string', 1],
					[22, 'filterid', 'string', 1],
					[23, 'elements', 'encoded', 1],
					[24, 'module', 'string', 1],
					[24, 'record', 'string', 2],
					[24, 'values', 'encoded', 3],
					[25, 'module', 'string', 1],
					[25, 'page', 'string', 3],
					[25, 'syncToken', 'string', 2],
					[26, 'module', 'string', 1],
					[26, 'page', 'string', 3],
					[26, 'query', 'string', 2],
					[27, 'module', 'string', 1],
					[27, 'page', 'string', 3],
					[27, 'query', 'string', 2],
					[28, 'synctype', 'string', 2],
					[28, 'type', 'string', 1],
					[29, 'key', 'string', 2],
					[29, 'type', 'string', 1],
					[30, 'key', 'string', 1],
					[30, 'module', 'string', 2],
					[30, 'token', 'string', 3],
					[31, 'element', 'encoded', 2],
					[31, 'key', 'string', 1],
					[32, 'element', 'encoded', 2],
					[32, 'key', 'string', 1],
					[33, 'id', 'String', 1],
				]
			],
			'vtiger_ws_operation_seq' => [
				'columns' => ['id'],
				'values' => [
					[33],
				]
			],
			'vtiger_ws_referencetype' => [
				'columns' => ['fieldtypeid', 'type'],
				'values' => [
					[20, 'Accounts'],
					[21, 'Accounts'],
					[22, 'Contacts'],
					[23, 'Campaigns'],
					[24, 'Accounts'],
					[25, 'Vendors'],
					[29, 'Vendors'],
					[30, 'Users'],
					[31, 'Users'],
					[32, 'Accounts'],
					[32, 'Contacts'],
					[32, 'Leads'],
					[32, 'Users'],
					[32, 'Vendors'],
					[33, 'Products'],
					[37, 'Currency'],
					[38, 'DocumentFolders'],
				]
			],
			'yetiforce_auth' => [
				'columns' => ['id', 'type', 'param', 'value'],
				'values' => [
					[1, 'ldap', 'active', 'false'],
					[2, 'ldap', 'server', 'testlab.local'],
					[3, 'ldap', 'port', '389'],
					[4, 'ldap', 'users', null],
					[5, 'ldap', 'domain', null],
				]
			],
			'yetiforce_mail_config' => [
				'columns' => ['type', 'name', 'value'],
				'values' => [
					['mailIcon', 'showMailAccounts', 'false'],
					['mailIcon', 'showNumberUnreadEmails', 'false'],
					['mailIcon', 'showMailIcon', 'true'],
					['mailIcon', 'timeCheckingMail', '30'],
					['autologin', 'autologinActive', 'false'],
					['signature', 'signature', ''],
					['signature', 'addSignature', 'false'],
				]
			],
			'yetiforce_menu' => [
				'columns' => ['id', 'role', 'parentid', 'type', 'sequence', 'module', 'label', 'newwindow', 'dataurl', 'showicon', 'icon', 'sizeicon', 'hotkey', 'filters'],
				'values' => [
					[44, 0, 0, 2, 0, null, 'MEN_VIRTUAL_DESK', 0, null, 0, 'userIcon-VirtualDesk', null, null, null],
					[45, 0, 44, 0, 0, 3, 'Home page', 0, null, 0, 'userIcon-Home', null, '', null],
					[46, 0, 44, 0, 1, 9, null, 0, null, 0, '', null, '', null],
					[47, 0, 0, 2, 1, null, 'MEN_COMPANIES_CONTACTS', 0, null, 0, 'userIcon-CompaniesAndContact', null, null, null],
					[48, 0, 47, 0, 0, 7, null, 0, null, 0, '', null, '', null],
					[49, 0, 47, 0, 5, 4, null, 0, null, 0, '', null, '', null],
					[50, 0, 47, 0, 3, 18, null, 0, null, 0, '', null, '', null],
					[51, 0, 47, 0, 1, 6, null, 0, null, 0, '', null, '', null],
					[52, 0, 0, 2, 2, null, 'MEN_MARKETING', 0, null, 0, 'userIcon-Campaigns', null, null, null],
					[54, 0, 52, 0, 0, 26, null, 0, null, 0, '', null, '', null],
					[62, 0, 118, 0, 8, 19, null, 0, null, 0, '', null, '', null],
					[63, 0, 0, 2, 5, null, 'MEN_SUPPORT', 0, null, 0, 'userIcon-Support', null, null, null],
					[64, 0, 63, 0, 0, 13, null, 0, null, 0, '', null, '', null],
					[65, 0, 63, 0, 1, 34, null, 0, null, 0, '', null, '', null],
					[66, 0, 63, 0, 2, 15, null, 0, null, 0, '', null, '', null],
					[67, 0, 0, 2, 4, null, 'MEN_PROJECTS', 0, null, 0, 'userIcon-Project', null, null, null],
					[68, 0, 67, 0, 0, 43, null, 0, null, 0, '', null, '', null],
					[69, 0, 67, 0, 1, 41, null, 0, null, 0, '', null, '', null],
					[70, 0, 67, 0, 2, 42, null, 0, null, 0, '', null, '', null],
					[71, 0, 0, 2, 6, null, 'MEN_ACCOUNTING', 0, null, 0, 'userIcon-Bookkeeping', null, null, null],
					[72, 0, 71, 0, 5, 79, null, 0, null, 0, '', null, '', null],
					[73, 0, 71, 0, 4, 80, null, 0, null, 0, '', null, '', null],
					[76, 0, 0, 2, 8, null, 'MEN_COMPANY', 0, null, 0, 'userIcon-HumanResources', null, null, null],
					[77, 0, 76, 0, 0, 61, null, 0, null, 0, '', null, '', null],
					[78, 0, 76, 0, 1, 51, null, 0, null, 0, '', null, '', null],
					[79, 0, 76, 0, 2, 78, null, 0, null, 0, '', null, '', null],
					[80, 0, 0, 2, 9, null, 'MEN_ORGANIZATION', 0, null, 0, 'userIcon-Secretary', null, null, null],
					[81, 0, 80, 0, 0, 81, null, 0, null, 0, '', null, '', null],
					[82, 0, 80, 0, 1, 82, null, 0, null, 0, '', null, '', null],
					[83, 0, 80, 0, 2, 84, null, 0, null, 0, '', null, '', null],
					[84, 0, 0, 2, 10, null, 'MEN_DATABESES', 0, null, 0, 'userIcon-Database', null, null, null],
					[85, 0, 84, 2, 0, null, 'MEN_PRODUCTBASE', 0, null, 0, null, null, null, null],
					[86, 0, 84, 0, 1, 14, null, 0, null, 0, '', null, '', null],
					[87, 0, 84, 0, 2, 59, null, 0, null, 0, '', null, '', null],
					[88, 0, 84, 0, 3, 37, null, 0, null, 0, '', null, '', null],
					[89, 0, 84, 3, 4, null, null, 0, null, 0, null, null, null, null],
					[90, 0, 84, 2, 5, null, 'MEN_SERVICESBASE', 0, null, 0, null, null, null, null],
					[91, 0, 84, 0, 6, 35, null, 0, null, 0, '', null, '', null],
					[92, 0, 84, 0, 7, 57, null, 0, null, 0, '', null, '', null],
					[93, 0, 84, 0, 8, 58, null, 0, null, 0, '', null, '', null],
					[94, 0, 84, 3, 9, null, null, 0, null, 0, null, null, null, null],
					[95, 0, 84, 2, 10, null, 'MEN_LISTS', 0, null, 0, null, null, null, null],
					[96, 0, 84, 0, 11, 54, null, 0, null, 0, '', null, '', null],
					[97, 0, 84, 0, 12, 45, null, 0, null, 0, '', null, '', null],
					[98, 0, 84, 0, 13, 33, null, 0, null, 0, '', null, '', null],
					[100, 0, 84, 0, 15, 8, null, 0, null, 0, '', null, '', null],
					[106, 0, 84, 0, 17, 74, null, 0, null, 0, '', null, '', null],
					[107, 0, 84, 3, 18, null, null, 0, null, 0, null, null, null, null],
					[108, 0, 84, 0, 23, 83, null, 0, null, 0, '', null, '', null],
					[109, 0, 84, 0, 16, 60, null, 0, null, 0, '', null, '', null],
					[111, 0, 44, 0, 3, 75, null, 0, null, 0, '', null, '', null],
					[113, 0, 44, 0, 2, 48, null, 0, null, 0, '', null, '', null],
					[114, 0, 84, 0, 22, 25, null, 0, null, 0, '', null, '', null],
					[115, 0, 84, 0, 19, 24, null, 0, null, 0, '', null, '', null],
					[116, 0, 84, 0, 20, 27, null, 0, null, 0, '', null, '', null],
					[117, 0, 84, 3, 21, null, null, 0, null, 0, null, null, null, null],
					[118, 0, 0, 2, 3, null, 'MEN_SALES', 0, null, 0, 'userIcon-Sales', null, null, null],
					[119, 0, 118, 0, 0, 86, null, 0, null, 0, '', null, '', null],
					[120, 0, 118, 0, 1, 85, null, 0, null, 0, '', null, '', null],
					[121, 0, 118, 0, 2, 87, null, 0, null, 0, '', null, '', null],
					[122, 0, 118, 0, 3, 88, null, 0, null, 0, '', null, '', null],
					[123, 0, 118, 0, 4, 89, null, 0, null, 0, '', null, '', null],
					[124, 0, 118, 0, 5, 90, null, 0, null, 0, '', null, '', null],
					[125, 0, 118, 0, 6, 91, null, 0, null, 0, '', null, '', null],
					[126, 0, 47, 0, 2, 92, null, 0, null, 0, '', null, '', null],
					[127, 0, 47, 0, 4, 93, null, 0, null, 0, '', null, '', null],
					[128, 0, 71, 0, 0, 94, null, 0, null, 0, '', null, '', null],
					[129, 0, 71, 0, 1, 95, null, 0, null, 0, '', null, '', null],
					[130, 0, 63, 0, 3, 96, null, 0, null, 0, '', null, '', null],
					[131, 0, 0, 2, 7, null, 'MEN_LOGISTICS', 0, null, 0, 'userIcon-VendorsAccounts', null, null, null],
					[132, 0, 131, 0, 10, 97, null, 0, null, 0, '', null, '', null],
					[133, 0, 131, 0, 1, 98, null, 0, null, 0, '', null, '', null],
					[134, 0, 71, 0, 2, 99, null, 0, null, 0, '', null, '', null],
					[135, 0, 131, 0, 4, 100, null, 0, null, 0, '', null, '', null],
					[136, 0, 131, 0, 5, 101, null, 0, null, 0, '', null, '', null],
					[137, 0, 131, 0, 6, 102, null, 0, null, 0, '', null, '', null],
					[138, 0, 131, 0, 7, 103, null, 0, null, 0, '', null, '', null],
					[139, 0, 131, 0, 9, 104, null, 0, null, 0, '', null, '', null],
					[140, 0, 131, 0, 0, 105, null, 0, null, 0, '', null, '', null],
					[141, 0, 131, 0, 8, 106, null, 0, null, 0, '', null, '', null],
					[142, 0, 71, 0, 3, 107, null, 0, null, 0, '', null, '', null],
					[143, 0, 131, 0, 2, 108, 'IGRNC', 0, null, 0, '', null, '', null],
					[144, 0, 131, 0, 3, 109, 'IGDNC', 0, null, 0, '', null, '', null],
					[146, 0, 84, 0, 14, 112, '', 0, null, 0, '', null, '', null],
					[147, 0, 76, 0, 3, 113, '', 0, null, 0, '', null, '', null],
					[148, 0, 76, 0, 4, 114, '', 0, null, 0, '', null, '', null],
					[149, 0, 76, 0, 5, 116, '', 0, null, 0, '', null, '', null],
					[150, 0, 71, 0, 6, 115, '', 0, null, 0, '', null, '', null],
					[151, 0, 118, 0, 7, 117, '', 0, null, 0, '', null, '', null],
				]
			],
			'yetiforce_proc_marketing' => [
				'columns' => ['type', 'param', 'value'],
				'values' => [
					['conversion', 'change_owner', 'false'],
					['lead', 'groups', ''],
					['lead', 'status', ''],
					['lead', 'convert_status', 'PLL_LEAD_ACQUIRED'],
					['lead', 'currentuser_status', 'false'],
					['conversion', 'create_always', 'false'],
					['conversion', 'mapping', '[{"company":"accountname"}]'],
				]
			],
			'yetiforce_proc_sales' => [
				'columns' => ['type', 'param', 'value'],
				'values' => [
					['popup', 'limit_product_service', 'false'],
				]
			],
			'yetiforce_proc_tc' => [
				'columns' => ['type', 'param', 'value'],
				'values' => [
					['general', 'oneDay', 'false'],
					['general', 'timeOverlap', 'false'],
					['timeControlWidget', 'holidays', 'true'],
					['timeControlWidget', 'workingDays', 'true'],
					['timeControlWidget', 'workingTime', 'true'],
				]
			],
		];
	}
}
